/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// InjectorMonDlg.cpp : implementation file
//

#include "stdafx.h"
#include "XInjectorMon.h"
#include "InjectorMonDlg.h"
#include "productdlls\cyranalyzer\analyzerdefs.h"
#include "Winsock2.h"
#include "SelectColumnDlg.h"
#include "common\CECOTAThresholds.h"
#include "common\CECRegistryTools.h"
#include "..\..\include\Solutions.h"
#include "..\..\HTMLHelp\XInjectorMon\Injector_Mon_Dlg.inl"
#include "htmlhelp.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CInjectorMonDlg dialog
#define WMU_DATA_READY (WM_APP+1)

//Keep a count of the number of times corba has been initialised
//across all instances of this ocx
static UINT uiCORBARefCount=0;

CInjectorMonDlg::CInjectorMonDlg(CWnd* pParent /*=NULL*/)
        : CDialog(CInjectorMonDlg::IDD, pParent)
{
    m_szHtmlHelpFile = getenv("CYRANO_HOME");
    m_szHtmlHelpFile += "\\Framework\\Help\\Commander.chm";

    m_pCorbaClt=new CORBAClient ( 0, NULL );
    m_pCorbaClt->Ready(true);

    uiCORBARefCount++;
    m_TestRun->_nil();

    //{{AFX_DATA_INIT(CInjectorMonDlg)
    //}}AFX_DATA_INIT

    CECRegistryTools RegTools;
    CString strRefreshRate = RegTools.GetStringValue(HKEY_CURRENT_USER,SOLUTIONS_FRAMEWORK_KEY,"Injector View Refresh");
    if(strRefreshRate.IsEmpty())
        strRefreshRate = "5";

    m_nRefreshDelay = atoi(strRefreshRate);
    m_bQuitting = false;
    m_pRefreshThread = NULL;
    m_nOTADataIndex = 0;
    m_nOTAHeaderIndex = 0;
}

CInjectorMonDlg::~CInjectorMonDlg()
{
    //Only do the destroys if this is the last occurrence of this ocx
    if((--uiCORBARefCount) == 0)
    {
        m_pCorbaClt->GetBOA()->destroy();
        m_pCorbaClt->GetORB()->NP_destroy();
    }
    delete m_pCorbaClt;

    CString strKey;
    LPVOID pVoid;
    POSITION pos = m_mapIPtoInjector.GetStartPosition();
    while(pos !=NULL)
    {
        m_mapIPtoInjector.GetNextAssoc(pos,strKey,pVoid);
        delete pVoid;
    }
}

void CInjectorMonDlg::DoDataExchange(CDataExchange* pDX)
{
    CDialog::DoDataExchange(pDX);
    //{{AFX_DATA_MAP(CInjectorMonDlg)
    DDX_Control(pDX, IDC_INJECTOR_LIST, m_ctlInjectorList);
    //}}AFX_DATA_MAP
}



BEGIN_MESSAGE_MAP(CInjectorMonDlg, CDialog)
//{{AFX_MSG_MAP(CInjectorMonDlg)
ON_WM_SIZE()
ON_COMMAND(ID_INJECTORS_SELECT_COLUMNS, OnInjectorsSelectColumns)
ON_WM_DESTROY()
ON_WM_CONTEXTMENU()
ON_COMMAND(ID_POPUP_SHOWGRAPHS, OnPopupShowgraphs)
ON_UPDATE_COMMAND_UI(ID_POPUP_SHOWGRAPHS, OnUpdatePopupShowgraphs)
ON_WM_HELPINFO()
ON_COMMAND(IDC_FULL_HELP, OnFullHelp)
//}}AFX_MSG_MAP
ON_MESSAGE(WMU_DATA_READY,OnDataReady)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CInjectorMonDlg message handlers

BOOL CInjectorMonDlg::OnInitDialog()
{
    CDialog::OnInitDialog();

    m_ctlInjectorList.InsertColumn(enNameCol,   "Injector Name",LVCFMT_LEFT,  80,enNameCol);
    m_ctlInjectorList.InsertColumn(enIPCol,     "Injector IP",  LVCFMT_CENTER,80,enIPCol);
    //  m_ctlInjectorList.InsertColumn(enTestCol,   "Test Name",    LVCFMT_CENTER,80,enTestCol);
    //  m_ctlInjectorList.InsertColumn(enStartCol,  "Start",        LVCFMT_CENTER,80,enStartCol);
    //  m_ctlInjectorList.InsertColumn(enPIDCol,    "Pid",          LVCFMT_CENTER,80,enPIDCol);
    m_ctlInjectorList.InsertColumn(enCPUCol,    "CPU",          LVCFMT_CENTER,80,enCPUCol);
    m_ctlInjectorList.InsertColumn(enSwapCol,   "Swap",         LVCFMT_CENTER,80,enSwapCol);
    m_ctlInjectorList.InsertColumn(enNetCol,    "Network",      LVCFMT_CENTER,80,enNetCol);
    //  m_ctlInjectorList.InsertColumn(enVUsCol,    "Current Users",LVCFMT_CENTER,80,enVUsCol);
    //  m_ctlInjectorList.InsertColumn(enTotalVUsCol,"Total Users", LVCFMT_CENTER,80,enTotalVUsCol);

    m_ctlInjectorList.SetExtendedStyle(LVS_EX_FULLROWSELECT|LVS_EX_GRIDLINES|LVS_EX_SUBITEMIMAGES);

    m_InjectorImages.Create(IDB_INJECTORLIST,16,0,RGB(255,255,0));
    m_ctlInjectorList.SetImageList(&m_InjectorImages,LVSIL_SMALL);

    RestoreSettings();
    return TRUE;  // return TRUE unless you set the focus to a control
    // EXCEPTION: OCX Property Pages should return FALSE
}


void CInjectorMonDlg::OnSize(UINT nType, int cx, int cy)
{
    CDialog::OnSize(nType, cx,cy);

    if(m_ctlInjectorList.GetSafeHwnd())
    {
        CRect rc;
        m_ctlInjectorList.MoveWindow(0, 0, cx, cy);
    }
}

void CInjectorMonDlg::StartMonitor(CString strRegName)
{
    try
    {
        m_strManagerName = strRegName;

        //Get the Test manager status object
        CORBA::Object_var varTestManager=m_pCorbaClt->GetObjectFromClass(TestManager::CLASS_NAME, strRegName.GetBuffer(0));
        TestManager_ptr pTestManager = narrow_object((TestManager_ptr)NULL,varTestManager);

        CORBA::Object_var varTestRun = pTestManager->GetTestRunAsObject();
        m_TestRun=narrow_object((CyrStTestRun_ptr )NULL,varTestRun);

        CORBA::release(pTestManager);

        CString strTest;
        strTest.Format("Injectors for %s",m_TestRun->GetTestName());
        ::SendNotifyMessage(GetParent()->m_hWnd,WMU_UPDATE_TITLE,(WPARAM)(LPCTSTR)strTest.GetBuffer(0), 0);

        InitOTADataIndexes();
        m_nOTADataIndex = m_TestRun->GetNoOutputEntries(CECOtaSimpleQuery::OUTPUT_TYPE);
        //      m_nOTADataIndex = m_TestRun->GetOutputOffset(CECOtaSimpleQuery::OUTPUT_TYPE);

        m_pRefreshThread = AfxBeginThread(&DataRefreshThread,this);
    }
    catch(...)
    {
        MessageBox("An error occurred while initialising the Injector monitor", "OpenSTA");
        if(GetParent() != NULL)
            GetParent()->PostMessage(WM_CLOSE);
    }
}

void CInjectorMonDlg::GetNodeList(TestLevel_ptr pTestLevel)
{
    TestLevelList ChildTestList;
    int nNumChildTestLevels = pTestLevel->GetChildTestLevelList(ChildTestList);

    TRACE("Found Node: %s\n",pTestLevel->GetNode());

    for(int nTest=0;nTest<ChildTestList.length();nTest++)
    {
        TestLevel_ptr TL=ChildTestList[nTest];

        GetNodeList(TL);
    }
}

void CInjectorMonDlg::InitOTADataIndexes()
{
    // Find out if OTA is running
    try
    {
        CString strQInfo;

        int nNumEntries = m_TestRun->GetNoOutputEntries(CECOtaHeader::OUTPUT_TYPE);
        BOOL bUpdateThresholds = (nNumEntries>m_nOTAHeaderIndex);
        for(;m_nOTAHeaderIndex<nNumEntries;m_nOTAHeaderIndex++)
        {
            strQInfo = m_TestRun->GetOutput(CECOtaHeader::OUTPUT_TYPE, m_nOTAHeaderIndex);

            CString strInjectorIP = strtok(strQInfo.GetBuffer(0),",");
            CString strIndex = strtok(NULL,",");
            CString strQueryType = strtok(NULL,",");

            AddOTAQueryType(strInjectorIP,strQueryType,atoi(strIndex));
        }
        if(bUpdateThresholds)
            LoadThresholds();
    }
    catch(...)
    {}

}

void CInjectorMonDlg::AddOTAQueryType(const CString& strInjectorIP,
                                      const CString& strQueryType,
                                      int nDataNdx)
{
    //If this is not a default query type, we can ignore it
    //otherwise remember the type
    DefQueryTypes QueryType;

    if      (strQueryType == strPercentCPUHeading)      QueryType = enCPUType;
    else if (strQueryType == strPercentNetUseHeading)   QueryType = enNetType;
    else if (strQueryType == strPageSwappingHeading)    QueryType = enSwapType;
    else return;

    //Look through our injectors to see if this injector has already been added
    CInjectorData *pInjector = NULL;
    if(!m_mapIPtoInjector.Lookup(strInjectorIP,(LPVOID&)pInjector))
    {
        //If this is a new injector, add an entry
        pInjector = new CInjectorData;
        pInjector->strIPAddress = strInjectorIP;
        pInjector->strName = GetComputerNameFromIP(strInjectorIP);
        m_mapIPtoInjector.SetAt(strInjectorIP,pInjector);
    }

    //then update this field
    CString str;
    str.Format("%s?%d",strInjectorIP,nDataNdx);
    m_mapNdxToInjector.SetAt(str,pInjector);
    if(pInjector->m_arOTAData[QueryType].nNdx == -1)
        pInjector->m_arOTAData[QueryType].nNdx = nDataNdx;
}

BOOL CInjectorMonDlg::IsOTADescriptionLine(CString strOTALine)
{
    // Strip of the trailing ','
    strOTALine.TrimRight(",");

    //If there is only one ',' this is description
    int nPos = strOTALine.Find(',',0);
    return (strOTALine.Find(',',nPos+1) == -1);
}

void CInjectorMonDlg::SetRefreshDelay(int nDelay)
{
    if(nDelay <= 0)
        nDelay = 1;

    m_nRefreshDelay = nDelay;
}

LRESULT CInjectorMonDlg::OnDataReady(WPARAM,LPARAM)
{
    UpdateInjectorList();
    return 0;
}

void CInjectorMonDlg::UpdateInjectorList()
{
    m_ctlInjectorList.SetRedraw(FALSE);
    CString strIP;
    CInjectorData *pInjector=NULL;

    POSITION pos = m_mapIPtoInjector.GetStartPosition();
    while(pos!=NULL)
    {
        m_mapIPtoInjector.GetNextAssoc(pos,strIP,(LPVOID&)pInjector);

        if(pInjector->m_nListItem == -1)
        {
            int nIndex = m_ctlInjectorList.GetItemCount();
            pInjector->m_nListItem = m_ctlInjectorList.InsertItem(nIndex,"",enInjectorBmp);

            m_ctlInjectorList.SetItemText(pInjector->m_nListItem,enNameCol,pInjector->strName);
            m_ctlInjectorList.SetItemText(pInjector->m_nListItem,enIPCol,strIP);
        }

        CString strValue;
        for(int nNdx=0;nNdx<enNumDefQueryTypes;nNdx++)
        {
            CInjectorData::OtaData &otaData = pInjector->m_arOTAData[nNdx];

            int nIcon = -1;
            if(otaData.dValue == -1)
                strValue = "-";
            else
                nIcon = (otaData.dValue >= otaData.dThreshold) ? enAboveThresholdBmp : enBelowThresholdBmp;

            UINT uiMask = LVIF_TEXT;
            uiMask |= ((nIcon == -1) ? 0 : LVIF_IMAGE);
            switch(nNdx)
            {
            case enCPUType:
                if(otaData.dValue != -1)
                    strValue.Format("%.0f%%",otaData.dValue);
                m_ctlInjectorList.SetItem(pInjector->m_nListItem,enCPUCol,uiMask,strValue,nIcon,0,0,0);
                break;
            case enSwapType:
                if(otaData.dValue != -1)
                    strValue.Format("%.0f",otaData.dValue);
                m_ctlInjectorList.SetItem(pInjector->m_nListItem,enSwapCol,uiMask,strValue,nIcon,0,0,0);
                break;
            case enNetType:
                if(otaData.dValue != -1)
                    strValue.Format("%.0f",otaData.dValue);
                m_ctlInjectorList.SetItem(pInjector->m_nListItem,enNetCol,uiMask,strValue,nIcon,0,0,0);
                break;
            default:
                ASSERT(FALSE);
                break;
            }
        }
    }
    m_ctlInjectorList.SetRedraw(TRUE);
}

UINT CInjectorMonDlg::DataRefreshThread(LPVOID lpParam)
{
    CInjectorMonDlg* pInjectorDlg = (CInjectorMonDlg*)lpParam;
    if(!pInjectorDlg)
        return 0;

    DWORD dwTickCount;
    while(!pInjectorDlg->IsShuttingDown())
    {
        //Get the start time
        dwTickCount=GetTickCount();

        //Load all the OTA data
        try
        {
            //This will throw an exception if the test is no longer running
            CORBA::Object_var varTestManager=pInjectorDlg->m_pCorbaClt->GetObjectFromClass(TestManager::CLASS_NAME, pInjectorDlg->m_strManagerName.GetBuffer(0));

            pInjectorDlg->InitOTADataIndexes();

            CString strNode,strQData,strIndex,strValue,str;
            int nNumEntries = pInjectorDlg->m_TestRun->GetNoOutputEntries(CECOtaSimpleQuery::OUTPUT_TYPE);
            for(;pInjectorDlg->m_nOTADataIndex < nNumEntries;pInjectorDlg->m_nOTADataIndex++)
            {
                strQData = pInjectorDlg->m_TestRun->GetOutput(CECOtaSimpleQuery::OUTPUT_TYPE, pInjectorDlg->m_nOTADataIndex);

                if(strQData.IsEmpty())
                    break;

                strNode = strtok(strQData.GetBuffer(0),",");
                strIndex = strtok(NULL,",");
                strtok(NULL,",");   //skip start time
                strtok(NULL,",");   //skip end time
                strValue = strtok(NULL,",");

                double dVal = atof(strValue);

                CInjectorData *pInjector=NULL;
                int nDataNdx = atoi(strIndex);

                str.Format("%s?%d",strNode,nDataNdx);

                if(pInjectorDlg->m_mapNdxToInjector.Lookup(str,(LPVOID&)pInjector))
                {
                    for(int nNdx=0;nNdx<enNumDefQueryTypes;nNdx++)
                    {
                        if(pInjector->m_arOTAData[nNdx].nNdx == nDataNdx)
                        {
                            pInjector->m_arOTAData[nNdx].dValue = dVal;
                            break;
                        }
                    }
                }
            }
        }
        catch(...)
        {
            //Close the monitor on an exception
            pInjectorDlg->GetParent()->PostMessage(WM_CLOSE);
            return 0;
        }

        //tell the dlg the data has been loaded
        pInjectorDlg->SendMessage(WMU_DATA_READY);

        //Wait for the specified delay period to pass
        while(GetTickCount()-dwTickCount <= (pInjectorDlg->m_nRefreshDelay*1000))
        {
            //Check if we've been asked to quit..
            if(pInjectorDlg->IsShuttingDown())
                return 0;

            //otherwise wait for a bit
            Sleep(100);
        }
    }

    return 0;
}

CString CInjectorMonDlg::GetComputerNameFromIP(const CString& strIP)
{
    CString strName;
    if(m_mapIPToName.Lookup(strIP,strName))
        return strName;

    CString strIPCopy(strIP);
    strIPCopy.Replace('_','.');
    DWORD dwAddr = inet_addr(strIPCopy);

    HOSTENT *pHost = gethostbyaddr((char*)&dwAddr,4,AF_INET);
    if(pHost==NULL)
    {
        TRACE("Net error %d\n",WSAGetLastError());
        strName = strIP;
    }
    else
    {
        strName = pHost->h_name;
        strName = strName.SpanExcluding(".");
        strName.MakeUpper();
    }
    m_mapIPToName.SetAt(strIP,strName);
    return strName;
}

void CInjectorMonDlg::LoadThresholds()
{
    CString strIP;
    CInjectorData *pInjector;
    POSITION pos = m_mapIPtoInjector.GetStartPosition();
    while(pos != NULL)
    {
        m_mapIPtoInjector.GetNextAssoc(pos,strIP,(LPVOID&)pInjector);

        CECOTAThresholds OT(pInjector->strName);
        pInjector->m_arOTAData[enCPUType].dThreshold = OT.GetCPUThreshold();
        pInjector->m_arOTAData[enNetType].dThreshold = OT.GetNetThreshold();
        pInjector->m_arOTAData[enSwapType].dThreshold = OT.GetSwapThreshold();
    }
}

void CInjectorMonDlg::OnInjectorsSelectColumns()
{
    FilterColumns(&m_ctlInjectorList);
}

void CInjectorMonDlg::FilterColumns(CListCtrl *pListCtrl)
{
    CSelectColumnDlg dlg;

    CHeaderCtrl *pHeader = pListCtrl->GetHeaderCtrl();
    int nNumCols = pHeader->GetItemCount();

    const int nBufferSize=50;
    char szBuffer[nBufferSize];
    LVCOLUMN lvColumn;
    lvColumn.mask = LVCF_TEXT|LVCF_WIDTH;
    lvColumn.pszText = szBuffer;
    lvColumn.cchTextMax = nBufferSize;

    for(int nCol=0;nCol<nNumCols;nCol++)
    {
        pListCtrl->GetColumn(nCol,&lvColumn);
        dlg.AddOption(lvColumn.pszText,lvColumn.cx);
    }

    if(dlg.DoModal()==IDOK)
    {
        CRect rcList;
        pListCtrl->GetClientRect(rcList);

        int nNumColumns = dlg.GetNumOptions();
        int nNumSelColumns = 0;
        for(nCol=0;nCol<nNumColumns;nCol++)
        {
            if(dlg.IsSelected(nCol))
                nNumSelColumns++;
        }

        int nColWidth = rcList.Width()/nNumSelColumns;
        for(nCol=0;nCol<nNumColumns;nCol++)
        {
            if(dlg.IsSelected(nCol))
                pListCtrl->SetColumnWidth(nCol,nColWidth);
            else
                pListCtrl->SetColumnWidth(nCol,0);
        }
    }
}

void CInjectorMonDlg::SaveSettings()
{
    CWinApp *pApp = AfxGetApp();

    //Store settings for injector list
    CString strEntry;
    int nNumColumns = m_ctlInjectorList.GetHeaderCtrl()->GetItemCount();
    pApp->WriteProfileInt("Injector List Columns","Num Columns",nNumColumns);

    int nWidth = 0;
    for(int nColumn=0;nColumn<nNumColumns;nColumn++)
    {
        nWidth = m_ctlInjectorList.GetColumnWidth(nColumn);
        strEntry.Format("%d",nColumn);
        pApp->WriteProfileInt("Injector List Columns",strEntry,nWidth);
    }
}

void CInjectorMonDlg::RestoreSettings()
{
    CWinApp *pApp = AfxGetApp();

    //Restore settings for injector list
    CHeaderCtrl* pheader = m_ctlInjectorList.GetHeaderCtrl();

    int nWidth;
    CString strEntry;
    int nNumColumns = pApp->GetProfileInt("Injector List Columns","Num Columns",0);
    for(int nColumn=0;nColumn<nNumColumns;nColumn++)
    {
        strEntry.Format("%d",nColumn);
        nWidth = pApp->GetProfileInt("Injector List Columns",strEntry,0);
        m_ctlInjectorList.SetColumnWidth(nColumn,nWidth);
    }
}

void CInjectorMonDlg::OnDestroy()
{
    //This tells the thread to quit
    //but we must wait until it has done,when m_bQuitting will be reset
    int nCount = 0;
    m_bQuitting = true;
    while(m_pRefreshThread && m_bQuitting)
    {
        Sleep(50);

        if(nCount++ == 40)
        {
            TerminateThread(m_pRefreshThread->m_hThread,-1);
            break;
        }
    }

    SaveSettings();

    CDialog::OnDestroy();
}

void CInjectorMonDlg::OnContextMenu(CWnd* pWnd, CPoint point)
{
    if (point.x == -1 && point.y == -1)
    {
        //keystroke invocation
        CRect rect;
        GetClientRect(rect);
        ClientToScreen(rect);

        point = rect.TopLeft();
        point.Offset(5, 5);
    }

    CMenu menu;

    if(pWnd->GetSafeHwnd() == m_ctlInjectorList.GetSafeHwnd())
    {
        VERIFY(menu.LoadMenu(IDR_INJECTORLIST_POPUP));
    }
    else
        return;

    CMenu* pPopup = menu.GetSubMenu(0);
    ASSERT(pPopup != NULL);

    // Show the popup menu
    pPopup->TrackPopupMenu(TPM_LEFTALIGN | TPM_RIGHTBUTTON, point.x, point.y,this);

}

BOOL CInjectorMonDlg::IsShuttingDown()
{
    if(m_bQuitting)
    {
        m_bQuitting = false;
        return true;
    }

    return false;
}

void CInjectorMonDlg::OnPopupShowgraphs()
{
    POSITION pos = m_ctlInjectorList.GetFirstSelectedItemPosition();
    if(pos != NULL)
    {
        int nItem = m_ctlInjectorList.GetNextSelectedItem(pos);
        CString strIP = m_ctlInjectorList.GetItemText(nItem,enIPCol);

        ::SendNotifyMessage(GetParent()->m_hWnd,WMU_DYN_GRAPH,(WPARAM)(LPCTSTR)strIP, (LPARAM)(LPCTSTR)m_strManagerName);
    }
}

void CInjectorMonDlg::OnUpdatePopupShowgraphs(CCmdUI* pCmdUI)
{
    UINT uiSelCount = m_ctlInjectorList.GetSelectedCount();
    pCmdUI->Enable(uiSelCount == 1);
}

BOOL CInjectorMonDlg::OnHelpInfo(HELPINFO* pHelpInfo)
{
    CString contextHelp = m_szHtmlHelpFile+"::whatisthis.txt";

    if (pHelpInfo->iCtrlId == IDC_STATIC)
        return true;

    CWnd* pWnd = GetDlgItem(pHelpInfo->iCtrlId);

    HtmlHelp(pWnd->m_hWnd, contextHelp, HH_TP_HELP_CONTEXTMENU, (DWORD)(LPVOID)g_TopicIdArray);
    return true;
}




void CInjectorMonDlg::OnFullHelp()
{
    HtmlHelp(m_hWnd, m_szHtmlHelpFile, HH_DISPLAY_TOPIC, (DWORD)"xinjecto.htm");
}
