/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// DetailedQueryPage.cpp : implementation file
//

#include "stdafx.h"
#include "xotaeditor.h"
#include "DetailedQueryPage.h"
#include "EditorSheet.h"
#include "MultipleNTQueryDlg.h"
#include "..\..\HTMLHelp\XOTAEditor\Detailed_Query.inl"
#include "htmlhelp.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CDetailedQueryPage property page

IMPLEMENT_DYNCREATE(CDetailedQueryPage, CPropertyPage)

CDetailedQueryPage::CDetailedQueryPage() : CGenericListPage(CDetailedQueryPage::IDD)
{
    //{{AFX_DATA_INIT(CDetailedQueryPage)
    //}}AFX_DATA_INIT

    m_strTokenName = _T("DetailQuery");

    m_szHtmlHelpFile = getenv("CYRANO_HOME");
    m_szHtmlHelpFile += "\\Framework\\Help\\Commander.chm";

    m_szOldEdit = "";

}

CDetailedQueryPage::~CDetailedQueryPage()
{}


void CDetailedQueryPage::DoDataExchange(CDataExchange* pDX)
{
    CGenericListPage::DoDataExchange(pDX);
    //{{AFX_DATA_MAP(CDetailedQueryPage)
    DDX_Control(pDX, IDC_GRID, m_ctlGrid);
    //}}AFX_DATA_MAP
}



BEGIN_MESSAGE_MAP(CDetailedQueryPage, CGenericListPage)
//{{AFX_MSG_MAP(CDetailedQueryPage)
ON_WM_SIZE()
ON_BN_CLICKED(IDCANCEL,OnCancel)
ON_BN_CLICKED(IDOK, OnOK)
ON_WM_HELPINFO()
//}}AFX_MSG_MAP
ON_COMMAND(ID__INSERTROW,OnInsertRow)
ON_COMMAND(ID__DELETEROW,OnDeleteRow)
ON_MESSAGE(WMU_CAN_START_EDITING,OnCanStartEditting)
ON_MESSAGE(WMU_VALIDATE_CELL,OnValidateCell)
ON_MESSAGE(WMU_CELL_BUTTON_PRESSED,OnButtonPressed)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CDetailedQueryPage message handlers

void CDetailedQueryPage::OnSize(UINT nType, int cx, int cy)
{
    CGenericListPage::OnSize(nType, cx, cy);

    if(m_ctlGrid.GetSafeHwnd()!=NULL)
    {
        const int nSeparator = 5;
        m_ctlGrid.MoveWindow(
            nSeparator,
            nSeparator,
            cx-2*nSeparator,
            cy-(2*nSeparator));
        /*
                //resize query text column to fill any gap
                int nNumColumns = m_ctlGrid.GetColumnCount();
                int nWidth = m_ctlGrid.CalcSumOfColWidths(0,nNumColumns,0);
         
                if(nWidth < cx)
                {
                    int nQueryTextWidth = m_ctlGrid.GetColWidth(enQueryTextColumn);
            
                    //-15 is how much smaller the grid control is than the page
                    int nNewWidth = cx - nWidth + nQueryTextWidth-15;
                    m_ctlGrid.SetColWidth(enQueryTextColumn,enQueryTextColumn,nNewWidth);
                }
        */
    }
}

void CDetailedQueryPage::OnInsertRow()
{

    //add a row to end of list
    int nNewRow = m_ctlGrid.GetRowCount() + 1;
    m_ctlGrid.SetRowCount(nNewRow);

    //Setup combo box editting for Query type column
    CStringArray strChoices;
    strChoices.Add(SQL_TEXT);
    strChoices.Add(EXEC_TEXT);
    strChoices.Add(NT_PERF_TEXT);
    strChoices.Add(SNMP_TEXT);

    m_ctlGrid.CreateListCell(nNewRow -1, enQueryTypeColumn, strChoices);

    // Create Targets Combo Cell
    CStringArray arstrTargets;
    GetTargets(arstrTargets);
    m_ctlGrid.CreateListCell(nNewRow-1,enTargetNameColumn, arstrTargets);

    //Setup check box editting for Enabled column
    m_ctlGrid.CreateCheckCell(nNewRow -1, enEnabledColumn);

    m_ctlGrid.CreateButtonCell(nNewRow-1,enRunQueryColumn, "->");

    //setup push button for query text
    m_ctlGrid.CreateButtonCell(nNewRow-1,enQueryTextColumn, "...", CECGridCtrl::ALIGN_RIGHT);



    //
    //set any default values
    //
    //If no targets defined select NT perf otherwise SQL for query type
    QueryTypes nType = (IsTargetListEmpty() ? enNTPerformanceQuery : enSQLQuery);
    m_ctlGrid.SetItemText(nNewRow-1,enQueryTypeColumn,strChoices[nType]);
    if(nType==enSQLQuery)
    {
        m_ctlGrid.SetItemText(nNewRow-1,enTargetNameColumn,arstrTargets[0]);
    }

    m_ctlGrid.SetItemText(nNewRow-1,enIntervalColumn,"5");
    m_ctlGrid.SetCheckBoxCellState(nNewRow-1,enEnabledColumn,TRUE);
}

LRESULT CDetailedQueryPage::OnButtonPressed(WPARAM nRow,LPARAM nColumn)
{

    switch(nColumn)
    {
    case enQueryTextColumn:
        {
            CString strQuery = m_ctlGrid.GetItemText(nRow,nColumn);
            CString strType = m_ctlGrid.GetItemText(nRow,enQueryTypeColumn);
            int nType = ConvertTargetType(strType);

            switch(nType)
            {
            case enNTPerformanceQuery:
                {
                    CMultipleNTQueryDlg dlg(this,strQuery);
                    if(dlg.DoModal()==IDOK)
                    {
                        m_ctlGrid.SetItemText(nRow,nColumn,dlg.GetQueries());
                    }
                }
                break;
            case enSQLQuery:
                if(IsTargetListEmpty())
                {
                    MessageBox("Please create a target in the target page and try again", "OpenSTA");
                    return -1;
                }
            default:
                if(EditQuery((QueryTypes)nType,strQuery))
                {
                    m_ctlGrid.SetItemText(nRow,nColumn,strQuery);
                }
                break;
            }
        }
        break;
    case enRunQueryColumn:
        RunQuery(nRow);
        break;
    }
    return 0;
}

LRESULT CDetailedQueryPage::OnCanStartEditting(WPARAM nRow,LPARAM nCol)
{

    switch(nCol)
    {
    case enTargetNameColumn:
        {
            //disable the target name box if the query type is not SQL
            if(SQL_TEXT != m_ctlGrid.GetItemText(nRow,enQueryTypeColumn))
            {
                MessageBox("The selected query type does not use targets", "OpenSTA", MB_ICONINFORMATION);
                return -1;
            }

            if(IsTargetListEmpty())
            {
                MessageBox("Please create a target in the target page and try again", "OpenSTA");
                return -1;
            }
        }
        break;
    case enQueryTextColumn:
        return -1;
        break;
    }
    m_szOldEdit = m_ctlGrid.GetItemText(nRow,nCol);
    return 1;
}

LRESULT CDetailedQueryPage::OnValidateCell(WPARAM nRow,LPARAM nCol)
{
    bool bResult = true;
    switch(nCol)
    {
    case enQueryTypeColumn:
        CString strNewValue = m_ctlGrid.GetItemText(nRow, nCol);

        //If the query type is changed delete the query text
        CString strQueryText = m_ctlGrid.GetItemText(nRow,enQueryTextColumn);
        if(strNewValue!=m_szOldEdit)
        {
            if(!strQueryText.IsEmpty())
            {
                bResult = (MessageBox("Changing the query type will result in the query text being deleted\nDo you wish to do this?", "OpenSTA",MB_YESNO)==IDYES);

                if(bResult)
                    m_ctlGrid.SetItemText(nRow,enQueryTextColumn,"");
            }
        }

        //If the query type is not SQL blank target name column
        if(strNewValue!=SQL_TEXT)
            m_ctlGrid.SetItemText(nRow,enTargetNameColumn,"");
        else
            if(m_ctlGrid.GetItemText(nRow,enTargetNameColumn).IsEmpty())
            {
                //if the target name is empty select the first one
                CStringArray arrTargets;
                GetTargets(arrTargets);

                m_ctlGrid.SetItemText(nRow,enTargetNameColumn,arrTargets[0]);
            }
        break;
    }

    return (bResult ? 1 : -1);
}

BOOL CDetailedQueryPage::OnInitDialog()
{
    CGenericListPage::OnInitDialog();

    const int nNumColumns = 7;
    CString szHeaders = "Enabled\tQuery type\tTarget name\tInterval\tHeading text\tQuery text\tTest Query";

    m_ctlGrid.SetColumnCount(nNumColumns);
    m_ctlGrid.SetRowCount(1);
    m_ctlGrid.SetFixedRowCount();

    m_ctlGrid.PasteTextFromBuffer(0,0, szHeaders);


    RestoreSettings();

    return false;//TRUE;  // return TRUE unless you set the focus to a control
    // EXCEPTION: OCX Property Pages should return FALSE
}



BOOL CDetailedQueryPage::OnSetActive()
{
    CStringArray arstrTargets;
    GetTargets(arstrTargets);

    for(int i=1;i<m_ctlGrid.GetRowCount();i++)
    {
        m_ctlGrid.CreateListCell(i,enTargetNameColumn, arstrTargets);
    }

    return CGenericListPage::OnSetActive();
}

int CDetailedQueryPage::GetNumDataEntries()
{
    return m_ctlGrid.GetRowCount();
}

//
//  GetDataEntry
//  ============
//
//  Get values from a row in the list box and store put them
//  into a list organised as {Key1,value1,Key2,value2,...}
//
bool CDetailedQueryPage::GetDataEntry(int nRow,CStringArray& arstrParams)
{
    //Check that the row required exists
    int nNumRows = m_ctlGrid.GetRowCount();
    if(nRow < 0 || nRow+1 >= nNumRows)
    {
        ASSERT(FALSE);
        return false;
    }

    bool bEmptyRow = true;

    //Go through all the columns...
    CString strValue,strToken;

    int nNumColumns = m_ctlGrid.GetColumnCount();
    for(int nColumn=0;nColumn<nNumColumns;nColumn++)
    {
        switch(nColumn)
        {
        case enEnabledColumn:
            strToken="enabled";
            break;
        case enQueryTypeColumn:
            strToken="type";
            break;
        case enTargetNameColumn:
            strToken="tgtname";
            break;
        case enIntervalColumn:
            strToken="frequency";
            break;
        case enHeadingTextColumn:
            strToken="heading";
            break;
        case enQueryTextColumn:
            strToken="text";
            break;
        default:
            ASSERT(FALSE);
            continue;
            break;
        }

        arstrParams.Add(strToken);

        //Get the value for this row and column
        //Get the value for this row and column
        switch(nColumn)
        {
        case enEnabledColumn:
            {
                BOOL bCheck = m_ctlGrid.GetCheckBoxCellState(nRow+1,nColumn);
                if (bCheck)
                    strValue = "Yes";
                else
                    strValue = "No";
                break;
            }

        default:
            strValue = m_ctlGrid.GetItemText(nRow+1,nColumn);
        }
        arstrParams.Add(strValue);

        //found at least one non empty entry in this row
        if(!strValue.IsEmpty())
            bEmptyRow = false;
    }

    return !bEmptyRow;
}

//Add a data entry to this page
void CDetailedQueryPage::AddDataEntry(const CStringArray& arstrParams)
{
    OnInsertRow();
    int nNewRow = m_ctlGrid.GetRowCount()-1;

    int nColumn=-1;
    CString strToken,strValue;
    int nNumEntries=arstrParams.GetSize()/2;
    for(int nEntry=0;nEntry<nNumEntries;nEntry++)
    {
        strToken = arstrParams[nEntry*2];
        strValue = arstrParams[nEntry*2+1];

        if(strToken=="type") nColumn = enQueryTypeColumn;
        else if(strToken == "tgtname") nColumn = enTargetNameColumn;
        else if(strToken == "frequency")nColumn = enIntervalColumn;
        else if(strToken == "heading")nColumn = enHeadingTextColumn;
        else if(strToken == "text")nColumn = enQueryTextColumn;
        else if(strToken == "enabled") nColumn = enEnabledColumn;
        else
        {
            ASSERT(FALSE);
        }

        if(nColumn != -1)
        {
            if (nColumn == enEnabledColumn)
            {
                if (strValue == "Yes")
                {
                    m_ctlGrid.SetCheckBoxCellState(nNewRow, nColumn, TRUE);
                }
                else
                {
                    m_ctlGrid.SetCheckBoxCellState(nNewRow, nColumn, FALSE);
                }
            }
            else
            {
                m_ctlGrid.SetItemText(nNewRow,nColumn,strValue);
            }
        }
    }
}

void CDetailedQueryPage::OnDeleteRow()
{
    CCellRange range = m_ctlGrid.GetSelectedCellRange();

    //remove all the selected rows from the grid
    for(int nRow=range.GetMinRow();nRow<=range.GetMaxRow();nRow++)
    {
        m_ctlGrid.DeleteRow(nRow);
    }
}

bool CDetailedQueryPage::IsTargetListEmpty()
{
    CStringArray arrTargets;
    GetTargets(arrTargets);
    if (arrTargets.GetSize()>0)
        return false;

    return true;
}

bool CDetailedQueryPage::IsTargetReferenced(const CString& strTargetName)
{
    int nNumRows = m_ctlGrid.GetRowCount();
    for(int nRow=1;nRow<=nNumRows;nRow++)
    {
        if(m_ctlGrid.GetItemText(nRow,enTargetNameColumn) == strTargetName)
        {
            return true;
        }
    }

    return false;
}

void CDetailedQueryPage::RunQuery(int nRow)
{

    CString strQuery = m_ctlGrid.GetItemText(nRow,enQueryTextColumn);
    CString strType = m_ctlGrid.GetItemText(nRow,enQueryTypeColumn);

    CString strCommandLine = getenv("CYRANO_HOME");
    strCommandLine += "\\Engines\\Mon\\texecuter_mon.exe ";
    strCommandLine += ("-q \"1/" +strType+"/"+strQuery + "\" ") ;

    int nType = ConvertTargetType(strType);
    if(nType == enSQLQuery)
    {
        CString strTarget = m_ctlGrid.GetItemText(nRow,enTargetNameColumn);

        CString strUser,strServer,strPassword,strType;
        CEditorSheet *pSheet = dynamic_cast<CEditorSheet*>(GetParent());
        if(pSheet != NULL)
        {
            //Get a pointer to the target page
            CTargetPage *pTargetPage = dynamic_cast<CTargetPage*>(pSheet->GetPage(CEditorSheet::enTargetPage));
            pTargetPage->GetTargetInfo(strTarget,strType,strServer,strUser,strPassword);
        }

        strCommandLine += ("-t \""+strType+"/"+strServer + "/" + strUser + "/" + strPassword + "\" ");
    }


    if(!SpawnProcess(strCommandLine))
    {
        MessageBox("An error occurred trying to run this query", "OpenSTA",MB_ICONEXCLAMATION);
    }

}

BOOL CDetailedQueryPage::OnHelpInfo(HELPINFO* pHelpInfo)
{
    CString contextHelp = m_szHtmlHelpFile+"::whatisthis.txt";

    if (pHelpInfo->iCtrlId == IDC_STATIC)
        return true;

    CWnd* pWnd = GetDlgItem(pHelpInfo->iCtrlId);

    HtmlHelp(pWnd->m_hWnd, contextHelp, HH_TP_HELP_CONTEXTMENU, (DWORD)(LPVOID)g_TopicIdArray);

    return true;
}

void CDetailedQueryPage::GetTargets(CStringArray& arrTargets)
{
    //Setup combo box with target names from target page
    CEditorSheet *pSheet = dynamic_cast<CEditorSheet*>(GetParent());
    if(pSheet != NULL)
    {
        //Get a pointer to the target page
        CTargetPage *pTargetPage = dynamic_cast<CTargetPage*>(pSheet->GetPage(CEditorSheet::enTargetPage));

        pTargetPage->GetTargetNameList(arrTargets);
    }
}

int CDetailedQueryPage::ConvertTargetType(CString strType)
{
    if (strType == SQL_TEXT)
    {
        return enSQLQuery;
    }
    else if (strType == EXEC_TEXT)
    {
        return enExecQuery;
    }
    else if (strType == NT_PERF_TEXT)
    {
        return enNTPerformanceQuery;
    }
    else if (strType == SNMP_TEXT)
    {
        return enSNMPQuery;
    }

    return enSQLQuery;
}
