/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// GenericPage.cpp : implementation file
//

#include "stdafx.h"
#include "xotaeditor.h"
#include "GenericPage.h"
#include "pdh.h"
#include "PdhMsg.h"
#include "..\..\HTMLHelp\XOTAEditor\Edit_Query.inl"
#include "htmlhelp.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

BEGIN_MESSAGE_MAP(CEditTextDlg, CDialog)
//{{AFX_MSG_MAP(CEditTextDlg)
ON_WM_HELPINFO()
// NOTE: the ClassWizard will add message map macros here
//}}AFX_MSG_MAP
END_MESSAGE_MAP()
/////////////////////////////////////////////////////////////////////////////
// CGenericPage property page
CGenericPage::CGenericPage(UINT uiPageID) : CPropertyPage(uiPageID)
{}


CGenericPage::~CGenericPage()
{}


BEGIN_MESSAGE_MAP(CGenericPage, CPropertyPage)
//{{AFX_MSG_MAP(CGenericPage)
ON_BN_CLICKED(IDCANCEL,OnCancel)
ON_BN_CLICKED(IDOK, OnOK)
//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CGenericPage message handlers
void CGenericPage::OnOK()
{
    //This is here so dialog is not dismissed when enter is pressed
}


void CGenericPage::OnCancel()
{
    //This is here so dialog is not dismissed when escape is pressed
}


//
//  EditQuery
//  =========
//
//  Edit a query of type strType (either SQL,Exec or NT Performance)
//

BOOL CGenericPage::EditQuery(QueryTypes nType,CString& strQuery)
{
    //Find out what type of query we're editing and use appropriate editor
    switch(nType)
    {
    case enSQLQuery:
        return EditText(strQuery,"Edit SQL Query");
        break;
    case enExecQuery:
        return EditText(strQuery,"Edit Exec Query");
        break;
    case enSNMPQuery:
        return EditText(strQuery,"Edit SNMP Query");
        break;
    case enNTPerformanceQuery:
        {
            bool bWinNT = false;
            OSVERSIONINFO OSInfo;
            OSInfo.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
            if(GetVersionEx(&OSInfo) && (OSInfo.dwPlatformId==VER_PLATFORM_WIN32_NT))
            {
                return EditNTPerformanceQuery(strQuery);
            }
            else
            {
                //can't edit with NT Perf dialog in win9x, so use text dialog
                return EditText(strQuery,"Edit Query");
            }
        }
        break;
    default:
        ASSERT(FALSE);
        break;
    }


    return false;
}

//
//  EditNTPerformanceQuery
//  =====================
//
//  Edit a query using the Performance Monitor config dialog
//  The dialog is setup using the passed in query
//
BOOL CGenericPage::EditNTPerformanceQuery(CString& strQuery)
{
    PDH_BROWSE_DLG_CONFIG PDHBrowseDlgConfig;

    DWORD dwQuerySize = 1024;

    //Select a configuration for the performance monitor config dialog
    ZeroMemory(&PDHBrowseDlgConfig, sizeof(PDHBrowseDlgConfig));

    PDHBrowseDlgConfig.bIncludeInstanceIndex = 0;
    PDHBrowseDlgConfig.bSingleCounterPerAdd = 1;
    PDHBrowseDlgConfig.bSingleCounterPerDialog = 1;
    PDHBrowseDlgConfig.bLocalCountersOnly = 0;
    PDHBrowseDlgConfig.bWildCardInstances = 0;
    PDHBrowseDlgConfig.bHideDetailBox = 1;
    PDHBrowseDlgConfig.bInitializePath = !strQuery.IsEmpty();
    PDHBrowseDlgConfig.hWndOwner = m_hWnd;
    PDHBrowseDlgConfig.pCallBack = NULL;
    PDHBrowseDlgConfig.szReturnPathBuffer = strQuery.GetBuffer(dwQuerySize);
    PDHBrowseDlgConfig.dwDefaultDetailLevel = 4;
    PDHBrowseDlgConfig.cchReturnPathLength = dwQuerySize;

    HINSTANCE hPDH = AfxLoadLibrary("PDH.dll");
    if(hPDH != NULL)
    {
        typedef PDH_STATUS  (CALLBACK* LPBROWSECOUNTERS)    (PDH_BROWSE_DLG_CONFIG*);
        LPBROWSECOUNTERS lpfnBrowseCounters =  (LPBROWSECOUNTERS)GetProcAddress(hPDH, "PdhBrowseCountersA");

        if (lpfnBrowseCounters != NULL)
        {
            //Display PerfMon config dialog

            if(lpfnBrowseCounters(&PDHBrowseDlgConfig) != ERROR_SUCCESS)
            {
                return FALSE;
            }
        }
        AfxFreeLibrary(hPDH);
        hPDH = NULL;
    }

    strQuery.ReleaseBuffer();
    return TRUE;
}

//
//  EditText
//  ========
//
//  Allow the user to edit a string.
//  strMessage is displayed on the dialog
//
BOOL CGenericPage::EditText(CString& strQuery,const CString& strMessage)
{
    //Show the edit text dialog
    CEditTextDlg dlg;
    dlg.m_strMessage = strMessage;
    dlg.m_strQuery = strQuery;
    if(dlg.DoModal()==IDOK)
    {
        strQuery = dlg.m_strQuery;
        return true;
    }
    return false;
}

BOOL CGenericPage::SpawnProcess(const CString& strCommandLine)
{
    //Create a process and redirect its stdout to a list box
    if(!m_ListOutDlg.GetSafeHwnd())
        m_ListOutDlg.Create(IDD_LIST_OUTPUT_DLG);
    m_ListOutDlg.ClearList();
    m_ListOutDlg.SetWindowText("Query Results...");
    m_ListOutDlg.ShowWindow(SW_SHOW);

    HANDLE hOutputWrite,hOutputRead,hOutputReadTmp;
    HANDLE hErrorWrite;

    // Set up the security attributes struct.
    SECURITY_ATTRIBUTES sa;
    sa.nLength= sizeof(SECURITY_ATTRIBUTES);
    sa.lpSecurityDescriptor = NULL;
    sa.bInheritHandle = TRUE;

    // Create the child output pipe.
    BOOL bResult = CreatePipe(&hOutputReadTmp,&hOutputWrite,&sa,0);
    ASSERT(bResult);

    bResult = DuplicateHandle(GetCurrentProcess(),hOutputWrite,
                              GetCurrentProcess(),&hErrorWrite,0,
                              TRUE,DUPLICATE_SAME_ACCESS);
    ASSERT(bResult);

    bResult = DuplicateHandle(GetCurrentProcess(),hOutputReadTmp,
                              GetCurrentProcess(),
                              &hOutputRead, // Address of new handle.
                              0,FALSE,      // Make it uninheritable.
                              DUPLICATE_SAME_ACCESS);
    ASSERT(bResult);

    CloseHandle(hOutputReadTmp);

    // Set up the start up info struct.
    STARTUPINFO si;
    ZeroMemory(&si,sizeof(STARTUPINFO));
    si.cb = sizeof(STARTUPINFO);
    si.dwFlags = STARTF_USESTDHANDLES|STARTF_USESHOWWINDOW;
    si.hStdOutput = hOutputWrite;
    si.hStdInput  = NULL;
    si.hStdError  = hErrorWrite;
    si.wShowWindow = SW_HIDE;

    CString strCmdLine(strCommandLine);
    PROCESS_INFORMATION pi;
    bResult = CreateProcess(NULL,strCmdLine.GetBuffer(0),NULL,NULL,TRUE,
                            CREATE_NEW_CONSOLE,NULL,NULL,&si,&pi);

    // Close any unnecessary handles.
    CloseHandle(pi.hThread);
    CloseHandle(hOutputWrite);
    CloseHandle(hErrorWrite);

    if(!bResult)
    {
        return false;
    }

    // Read the child's output.
    CHAR lpBuffer[256];
    DWORD nBytesRead;
    while(TRUE)
    {
        if (!ReadFile(hOutputRead,lpBuffer,sizeof(lpBuffer),
                      &nBytesRead,NULL) || !nBytesRead)
        {
            if (GetLastError() == ERROR_BROKEN_PIPE)
                break; // pipe done - normal exit path.

            ASSERT(FALSE);
            break;
        }
        lpBuffer[nBytesRead] = '\0';
        m_ListOutDlg.AddString(lpBuffer);
    }

    CloseHandle(hOutputRead);
    m_ListOutDlg.SetWindowText("Query Results... Finished");

    return true;
}


BOOL CEditTextDlg::OnInitDialog()
{
    CDialog::OnInitDialog();
    return TRUE;  // return TRUE unless you set the focus to a control
    // EXCEPTION: OCX Property Pages should return FALSE

    m_szHtmlHelpFile = getenv("CYRANO_HOME");
    m_szHtmlHelpFile += "\\Framework\\Help\\Commander.chm";

}

BOOL CEditTextDlg::OnHelpInfo(HELPINFO* pHelpInfo)
{
    CString contextHelp = m_szHtmlHelpFile+"::whatisthis.txt";

    if (pHelpInfo->iCtrlId == IDC_STATIC)
        return true;

    CWnd* pWnd = GetDlgItem(pHelpInfo->iCtrlId);

    HtmlHelp(pWnd->m_hWnd, contextHelp, HH_TP_HELP_CONTEXTMENU, (DWORD)(LPVOID)g_TopicIdArray);

    return true;
}

BOOL CGenericPage::PreTranslateMessage(MSG* pMsg)
{
    // TODO: Add your specialized code here and/or call the base class

    return CPropertyPage::PreTranslateMessage(pMsg);
}
