/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// TargetPage.cpp : implementation file
//

#include "stdafx.h"
#include "xotaeditor.h"
#include "TargetPage.h"
#include "common\CECTargetEnvironment.h"
#include "EditorSheet.h"
#include "DetailedQueryPage.h"
#include "SimpleQueryPage.h"
#include "..\..\HTMLHelp\XOTAEditor\Target.inl"
#include "htmlhelp.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CTargetPage property page

IMPLEMENT_DYNCREATE(CTargetPage, CPropertyPage)

CTargetPage::CTargetPage() : CGenericListPage(CTargetPage::IDD)
{
    //{{AFX_DATA_INIT(CTargetPage)
    // NOTE: the ClassWizard will add member initialization here
    //}}AFX_DATA_INIT
    m_strTokenName = "Target";
    m_szHtmlHelpFile = getenv("CYRANO_HOME");
    m_szHtmlHelpFile += "\\Framework\\Help\\Commander.chm";

    m_szOldEdit = "";
}

CTargetPage::~CTargetPage()
{}


void CTargetPage::DoDataExchange(CDataExchange* pDX)
{
    CGenericListPage::DoDataExchange(pDX);
    //{{AFX_DATA_MAP(CTargetPage)
    DDX_Control(pDX, IDC_GRID, m_ctlGrid);
    //}}AFX_DATA_MAP
}



BEGIN_MESSAGE_MAP(CTargetPage, CGenericListPage)
//{{AFX_MSG_MAP(CTargetPage)
ON_WM_SIZE()
ON_BN_CLICKED(IDCANCEL,OnCancel)
ON_BN_CLICKED(IDOK, OnOK)
ON_WM_HELPINFO()
//}}AFX_MSG_MAP
ON_COMMAND(ID__INSERTROW,OnInsertRow)
ON_COMMAND(ID__DELETEROW,OnDeleteRow)
ON_MESSAGE(WMU_VALIDATE_CELL,OnValidateCell)
ON_MESSAGE(WMU_CAN_START_EDITING,OnCanStartEditting)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CTargetPage message handlers

void CTargetPage::OnSize(UINT nType, int cx, int cy)
{
    CGenericListPage::OnSize(nType, cx, cy);

    if(m_ctlGrid.GetSafeHwnd()!=NULL)
    {
        m_ctlGrid.MoveWindow(5,5,cx-10,cy-10);
        m_ctlGrid.ExpandColumnsToFit();
    }
}

void CTargetPage::OnInsertRow()
{

    int nNewRow = m_ctlGrid.GetRowCount()+1;
    m_ctlGrid.SetRowCount(nNewRow);
    //Setup combo box editting for Target types
    CStringArray strChoices;
    strChoices.Add("Sybase");
    strChoices.Add("Oracle");
    strChoices.Add("Microsoft");

    m_ctlGrid.CreateListCell(nNewRow-1,enTargetTypeColumn, strChoices);

    //set any default values
    int nType = GetDefaultTargetType();
    m_ctlGrid.SetItemText(nNewRow-1,enTargetTypeColumn,strChoices[nType]);
    m_ctlGrid.SetItemText(nNewRow-1,enMaxRowsColumn,"100");

    //Target name defaults to TARGETx,so find next x which is not used
    int nCount=1;
    CString strTargetName;
    bool bFound=false;
    do
    {
        bFound=false;
        strTargetName.Format("TARGET%d",nCount++);
        for(int nRow=1;nRow<nNewRow && !bFound;nRow++)
        {
            bFound = (strTargetName == m_ctlGrid.GetItemText(nRow,enTargetNameColumn));
        }
    }
    while(bFound);

    m_ctlGrid.SetItemText(nNewRow-1,enTargetNameColumn,strTargetName);

}

BOOL CTargetPage::OnInitDialog()
{
    CGenericListPage::OnInitDialog();

    const int nNumColumns = 6;
    CString headers = "Target Name\tTarget type\tServer name\tUser name\tPassword\tMax rows";

    m_ctlGrid.SetColumnCount(nNumColumns);
    m_ctlGrid.SetRowCount(1);
    m_ctlGrid.SetFixedRowCount();

    m_ctlGrid.PasteTextFromBuffer(0,0, headers);

    RestoreSettings();

    return false;//TRUE;  // return TRUE unless you set the focus to a control
    // EXCEPTION: OCX Property Pages should return FALSE
}


void CTargetPage::GetTargetNameList(CStringArray& arstrTargets)
{
    arstrTargets.RemoveAll();

    if(m_ctlGrid.GetSafeHwnd() == NULL)
        return;

    int nNumRows = m_ctlGrid.GetRowCount();
    for(int nRow=1;nRow<=nNumRows;nRow++)
    {
        arstrTargets.Add(m_ctlGrid.GetItemText(nRow,enTargetNameColumn));
    }
}

int CTargetPage::GetNumDataEntries()
{
    return m_ctlGrid.GetRowCount();
}

//
//  GetDataEntry
//  ============
//
//  Get values from a row in the list box and store put them
//  into a list organised as {Key1,value1,Key2,value2,...}
//
bool CTargetPage::GetDataEntry(int nRow,CStringArray& arstrParams)
{

    //Check that the row required exists
    int nNumRows = m_ctlGrid.GetRowCount();
    if(nRow < 0 || nRow+1 >= nNumRows)
    {
        ASSERT(FALSE);
        return false;
    }

    bool bEmptyRow = true;

    //Go through all the columns...
    CString strValue,strToken;

    int nNumColumns = m_ctlGrid.GetColumnCount();
    for(int nColumn=0;nColumn<nNumColumns;nColumn++)
    {
        switch(nColumn)
        {
        case enTargetNameColumn:
            strToken="name";
            break;
        case enTargetTypeColumn:
            strToken="type";
            break;
        case enServerNameColumn:
            strToken="server";
            break;
        case enUserNameColumn:
            strToken="user";
            break;
        case enPasswordColumn:
            strToken="password";
            break;
        case enMaxRowsColumn:
            strToken="maxrows";
            break;
        default:
            ASSERT(FALSE);
            break;
        }
        arstrParams.Add(strToken);

        //Get the value for this row and column
        strValue = m_ctlGrid.GetItemText(nRow+1,nColumn);
        arstrParams.Add(strValue);

        //found at least one non empty entry in this row
        if(!strValue.IsEmpty())
            bEmptyRow = false;
    }

    return !bEmptyRow;
}

//Add a data entry to this page
void CTargetPage::AddDataEntry(const CStringArray& arstrParams)
{

    int nNewRow = m_ctlGrid.GetRowCount()+1;
    m_ctlGrid.SetRowCount(nNewRow);

    int nColumn=-1;
    CString strToken,strValue;
    int nNumEntries=arstrParams.GetSize()/2;
    for(int nEntry=0;nEntry<nNumEntries;nEntry++)
    {
        strToken = arstrParams[nEntry*2];

        if(strToken=="name") nColumn = enTargetNameColumn;
        else if(strToken == "type") nColumn = enTargetTypeColumn;
        else if(strToken == "server")nColumn = enServerNameColumn;
        else if(strToken == "user") nColumn = enUserNameColumn;
        else if(strToken == "password")nColumn = enPasswordColumn;
        else if(strToken == "maxrows")nColumn = enMaxRowsColumn;
        else
        {
            ASSERT(FALSE);
        }

        if(nColumn != -1)
        {
            strValue = arstrParams[nEntry*2+1];
            m_ctlGrid.SetItemText(nNewRow,nColumn,strValue);
        }
    }
}

int CTargetPage::GetDefaultTargetType()
{
    CECTargetEnvironment TEnv;

    CString strContext;
    TEnv.GetCurrentContext(strContext);
    TRACE(strContext +"\n");
    TARGETID TargetId = Tgt_GetTargetID (strContext.GetBuffer(0));

    switch(TargetId)
    {
    case TARGET_SYBASE:
        return enSybase;
        break;
    case TARGET_ORACLE:
        return enOracle;
        break;
    default:
        return enMicrosoft;
        break;
    }
}

void CTargetPage::OnDeleteRow()
{
    CCellRange range = m_ctlGrid.GetSelectedCellRange();

    //remove all the selected rows from the grid
    for(int nRow=range.GetMinRow();nRow<=range.GetMaxRow();nRow++)
    {
        CString strTargetName = m_ctlGrid.GetItemText(nRow,enTargetNameColumn);
        if(IsTargetReferenced(strTargetName))
        {
            CString strMessage;
            strMessage.Format("%s is referenced in a query and cannot be deleted",strTargetName);
            MessageBox(strMessage, "OpenSTA");
            continue;
        }
        m_ctlGrid.DeleteRow(nRow);
        m_ctlGrid.Invalidate();
    }

}

BOOL CTargetPage::IsTargetReferenced(const CString &strTargetName)
{

    CEditorSheet *pSheet = dynamic_cast<CEditorSheet*>(GetParent());
    if(pSheet != NULL)
    {
        //Get a pointer to the target page
        CDetailedQueryPage *pDetailedQueryPage = dynamic_cast<CDetailedQueryPage*>(pSheet->GetPage(CEditorSheet::enDetailedQueryPage));
        if(pDetailedQueryPage != NULL
                && pDetailedQueryPage->IsTargetReferenced(strTargetName))
        {
            return true;
        }

        CSimpleQueryPage *pSimpleQueryPage = dynamic_cast<CSimpleQueryPage*>(pSheet->GetPage(CEditorSheet::enSimpleQueryPage));
        if(pSimpleQueryPage != NULL &&
                pSimpleQueryPage->IsTargetReferenced(strTargetName))
        {
            return true;
        }
    }
    return false;
}

LRESULT CTargetPage::OnValidateCell(WPARAM nRow,LPARAM nCol)
{

    bool bResult = true;
    switch(nCol)
    {
    case enTargetNameColumn:
        {
            CString strNewValue = m_ctlGrid.GetItemText(nRow,nCol);
            CString strOldValue = m_szOldEdit;

            if (strNewValue.IsEmpty())
                return -1;

            if (strNewValue == strOldValue)
                return 1;

            if(!m_szOldEdit.IsEmpty() && IsTargetReferenced(strOldValue))
            {
                CString strMessage;
                strMessage.Format("%s is referenced in a query and cannot be modified",strOldValue);
                MessageBox(strMessage, "OpenSTA");
                return -1;
            }
            //search the target names to see if this one already exists
            int nRowCount = m_ctlGrid.GetRowCount();
            for(int nRow=0;nRow<nRowCount;nRow++)
            {
                if(strNewValue == m_ctlGrid.GetItemText(nRow,enTargetNameColumn))
                {
                    CString strMessage;
                    strMessage.Format("%s already exists.\n\nPlease specify another target name",strNewValue);
                    MessageBox(strMessage, "OpenSTA");
                    return -1;
                }
            }
        }
        break;
    }

    return (bResult ? 1 : -1);
}

BOOL CTargetPage::GetTargetInfo(const CString& strTarget,CString& strType,CString& strServer,CString& strUser,CString& strPassword)
{

    int nNumRows = m_ctlGrid.GetRowCount();
    for(int nRow=1;nRow<=nNumRows;nRow++)
    {
        //Look through all the targets for one named strTarget
        CString strTargetName = m_ctlGrid.GetItemText(nRow,enTargetNameColumn);
        if(strTargetName == strTarget)
        {
            //Found the target so retrieve some info about it
            strType = m_ctlGrid.GetItemText(nRow,enTargetTypeColumn);
            strServer = m_ctlGrid.GetItemText(nRow,enServerNameColumn);
            strUser = m_ctlGrid.GetItemText(nRow,enUserNameColumn);
            strPassword = m_ctlGrid.GetItemText(nRow,enPasswordColumn);
            return true;
        }
    }
    return false;
}

BOOL CTargetPage::OnHelpInfo(HELPINFO* pHelpInfo)
{
    CString contextHelp = m_szHtmlHelpFile+"::whatisthis.txt";

    if (pHelpInfo->iCtrlId == IDC_STATIC)
        return true;

    CWnd* pWnd = GetDlgItem(pHelpInfo->iCtrlId);

    HtmlHelp(pWnd->m_hWnd, contextHelp, HH_TP_HELP_CONTEXTMENU, (DWORD)(LPVOID)g_TopicIdArray);

    return true;
}

LRESULT CTargetPage::OnCanStartEditting(WPARAM nRow,LPARAM nCol)
{
    m_szOldEdit = m_ctlGrid.GetItemText(nRow, nCol);
    return 1;
}

