/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// CXProjectTree.cpp : implementation file
//

#include "stdafx.h"
#include "..\..\Studio\Common\CECCyranoHome.h"
#include "..\..\Studio\Common\CECRegistryTools.h"
#include "..\..\Studio\Common\CECTargetEnvironment.h"
#include "XReposTree.h"
#include "CXProjectTree.h"
#include <fcntl.h>
//#include "PropertiesDlg.h"
#include "StringEx.h"
#include "..\..\include\Solutions.h"
#include "..\..\Studio\Interfaces\CyrPlugIns.h"
#include "Server.h"

extern "C"
{
#include "psl_macros.h"         //+3553
#include "target_rtndef.h"      //+3553 Target/Executer routines
}


#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


//
//    The following defines should be kept in line with the loading of the icons in the LoadIcons
//    function and should also be added only in groups of three
//
//      MISSING ICON
//      LOCKED ICON
//      UNLOCKED ICON
//
//    If this order is not strictly observed problems will occur.
//
typedef enum {
    NODE_ICON, FOLDER_ICON,    TEST_ICON,
    SCL_MISSING_ICON, SCL_UNLOCKED_ICON, SCL_LOCKED_ICON,
    TSS_MISSING_ICON, TSS_UNLOCKED_ICON, TSS_LOCKED_ICON,
    THS_MISSING_ICON, THS_UNLOCKED_ICON, THS_LOCKED_ICON,
    DAT_MISSING_ICON, DAT_UNLOCKED_ICON, DAT_LOCKED_ICON,
    CMD_MISSING_ICON, CMD_UNLOCKED_ICON, CMD_LOCKED_ICON,
    ORA_MISSING_ICON, ORA_UNLOCKED_ICON, ORA_LOCKED_ICON,
    SYB_MISSING_ICON, SYB_UNLOCKED_ICON, SYB_LOCKED_ICON,
    MSS_MISSING_ICON, MSS_UNLOCKED_ICON, MSS_LOCKED_ICON,
    WEB_MISSING_ICON, WEB_UNLOCKED_ICON, WEB_LOCKED_ICON,
    OTA_MISSING_ICON, OTA_UNLOCKED_ICON, OTA_LOCKED_ICON
};

//
//    Increments from the base icon
//
#define OBJECT_UNLOCKED     1
#define OBJECT_LOCKED       2




/////////////////////////////////////////////////////////////////////////////
// CXProjectTree dialog


CXProjectTree::CXProjectTree(BOOL bUseSCC, CWnd* pParent /*=NULL*/)
        : CDialog(CXProjectTree::IDD, pParent)
{
    //{{AFX_DATA_INIT(CXProjectTree)
    // NOTE: the ClassWizard will add member initialization here
    //}}AFX_DATA_INIT

    m_bUseSCC = bUseSCC;
    m_strProject="";
    m_pServer = NULL;

}


void CXProjectTree::DoDataExchange(CDataExchange* pDX)
{
    CDialog::DoDataExchange(pDX);
    //{{AFX_DATA_MAP(CXProjectTree)
    DDX_Control(pDX, IDC_XTREE_PROJECT, m_XTreeProject);
    //}}AFX_DATA_MAP
}



BEGIN_MESSAGE_MAP(CXProjectTree, CDialog)
//{{AFX_MSG_MAP(CXProjectTree)
ON_WM_SIZE()
//}}AFX_MSG_MAP
ON_WM_ERASEBKGND()
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CXProjectTree message handlers

BEGIN_EVENTSINK_MAP(CXProjectTree, CDialog)
//{{AFX_EVENTSINK_MAP(CXProjectTree)
ON_EVENT(CXProjectTree, IDC_XTREE_PROJECT, 1 /* LBDblClick */, OnLBDblClickXtreeProject, VTS_I4 VTS_I2 VTS_BSTR)
ON_EVENT(CXProjectTree, IDC_XTREE_PROJECT, 2 /* XTreeExpanding */, OnXTreeExpandingXtreeProject, VTS_I4 VTS_I2 VTS_BSTR)
ON_EVENT(CXProjectTree, IDC_XTREE_PROJECT, 3 /* XTreeContextClick */, OnXTreeContextClickXtreeProject, VTS_BSTR VTS_I2 VTS_BSTR)
ON_EVENT(CXProjectTree, IDC_XTREE_PROJECT, 4 /* XTreeItemProperties */, OnXTreeItemProperties, VTS_BSTR VTS_I2)
ON_EVENT(CXProjectTree, IDC_XTREE_PROJECT, 5 /* XShowContext */, OnXShowContext, VTS_I2 VTS_BSTR)
//}}AFX_EVENTSINK_MAP
END_EVENTSINK_MAP()

void CXProjectTree::OnLBDblClickXtreeProject(long hTreeItem, short nNodeType, LPCTSTR szPath)
{
    HTREEITEM hti = (HTREEITEM)hTreeItem;
    CString tempString = szPath;
    int nLen = tempString.GetLength();
    int nPos = tempString.ReverseFind('|');
    tempString = tempString.Right(nLen-nPos-1);

    //
    //   Get a pointer to the main application window
    //
    CWnd* pMainWnd = AfxGetMainWnd();
    ASSERT (pMainWnd != NULL);

    //
    //    First lets get the extension of the object on which we have double clicked. This and whether
    //    the object exists will determine how we edit it (the default action for double-click)
    //
    CString Extension = GetExtension (nNodeType);

    switch (nNodeType)
    {
    case TEST:
        {
            tempString = tempString + Extension;
            LPCTSTR scriptName = tempString;
            ::SendNotifyMessage(GetParent()->m_hWnd ,WMU_OPEN_FILE ,0 ,(long)scriptName);
            break;
        }

    case TEST_FILE:
        {
            tempString = tempString + Extension;
            LPCTSTR scriptName = tempString;
            ::SendNotifyMessage(GetParent()->m_hWnd ,WMU_EDIT_FILE ,0 ,(long)scriptName);
            break;
        }

    case SCRIPT_FILE:
    case CMD_SCRIPT:
    case ORA_SCRIPT:
    case SYB_SCRIPT:
    case SQL_SCRIPT:
    case WEB_SCRIPT:
        {
            tempString = tempString + Extension;
            LPCTSTR scriptName = tempString;
            ::SendNotifyMessage(GetParent()->m_hWnd ,WMU_EDIT_FILE ,0 ,(long)scriptName);
            break;
        }

    case NODE:
        {
            LPCTSTR scriptName = tempString;
            ::SendNotifyMessage(GetParent()->m_hWnd ,WMU_XEDIT_NODE ,0 ,(long)scriptName);
            break;
        }

    case ORA_TOF:
    case SYB_TOF:
    case SQL_TOF:
    case WEB_TOF:
        {
            EditTof (nNodeType ,tempString);
            break;
        }
    case OTA_TOF:
        {
            tempString = tempString + Extension;
            EditTof(nNodeType ,tempString);
            break;
        }
    default:
        ASSERT(FALSE);
        break;
    }
}

void CXProjectTree::OnXTreeExpandingXtreeProject(long hTreeItem, short nNodeType, LPCTSTR szPath)
{
    BeginWaitCursor();

    switch (nNodeType)
    {
    case PROJECT:
        {
            m_XTreeProject.AddItem(hTreeItem,NODE_FOLDER,"Nodes", FOLDER_ICON,TRUE);
            m_XTreeProject.AddItem(hTreeItem,TEST_FOLDER,"Tests", FOLDER_ICON,TRUE);
            break;
        }

    case TEST_FOLDER:
        {
            AddTests((HTREEITEM)hTreeItem);
            break;
        }

    case NODE_FOLDER:
        {
            AddNodes((HTREEITEM)hTreeItem);
            break;
        }

    case TEST:
        {
            CString path = szPath;
            path+=".tss";
            AddTestDependencies((HTREEITEM)hTreeItem, path);
            break;
        }

    case SCRIPT_FILE:
    case CMD_SCRIPT:
    case ORA_SCRIPT:
    case SYB_SCRIPT:
    case SQL_SCRIPT:
    case WEB_SCRIPT:
        {
            CString path = szPath;
            path+=".scl";
            AddTestDependencies((HTREEITEM)hTreeItem, path);
            break;
        }

    case TEST_FILE:
        {
            CString path = szPath;
            path+=".tss";
            AddTestDependencies((HTREEITEM)hTreeItem, path);
            break;
        }

    case THREAD_FILE:
        {
            CString path = szPath;
            path+=".ths";
            AddTestDependencies((HTREEITEM)hTreeItem, path);
            break;
        }
    }

    EndWaitCursor();
}

void CXProjectTree::OnXTreeContextClickXtreeProject(LPCTSTR strPath, short nType, LPCTSTR strEntry)
{
    CString menuEntry = strEntry;
    CString tempString = strPath;
    int nLen = tempString.GetLength();
    int nPos = tempString.ReverseFind('|');
    tempString = tempString.Right(nLen-nPos-1);
    LPCTSTR itemName = tempString;

    if (menuEntry == "&New Node")
    {
        ::SendNotifyMessage(GetParent()->m_hWnd,WMU_ADD_NODE,0,0);
        return;
    }
    if (menuEntry == "&Edit Node" && nType == NODE)
    {
        ::SendNotifyMessage(GetParent()->m_hWnd,WMU_XEDIT_NODE,0,(long)itemName);
    }
    if (menuEntry == "&Delete Node" && nType == NODE)
    {
        ::SendNotifyMessage(GetParent()->m_hWnd,WMU_DELETE_NODE,(UINT)(LPCTSTR)m_pServer->GetFolderName(CYRServer::enNodesFolder),(long)itemName);
        RefreshTree();
        return;
    }

    if (menuEntry == "&New Test")
    {
        ::SendNotifyMessage(GetParent()->m_hWnd,WMU_ADD_TEST,0,0);
        return;
    }



    if (nType == TEST && menuEntry == "E&xecute")
    {
        ::SendNotifyMessage(GetParent()->m_hWnd,WMU_INIT_TEST,0,(long)itemName);
        return;
    }
    if (nType == TEST && menuEntry == "&Delete")
    {
        ::SendNotifyMessage(GetParent()->m_hWnd,WMU_DELETE_TEST,(UINT)(LPCTSTR)m_pServer->GetFolderName(CYRServer::enTestsFolder),(long)itemName);
        return;
    }

    if(nType == OTA_TOF && menuEntry == "&Edit OTA")
    {
        EditTof(OTA_TOF,tempString + ".tof");
        return;
    }

    const char *szFolderName = m_pServer->GetFolderName(CYRServer::enSourceFolder);
    switch (nType)
    {
    case TEST:
        {
            tempString = tempString+".tss";
            break;
        }
    case SCRIPT_FILE:
        {
            tempString = tempString + ".scl";
            break;
        }
    case TEST_FILE:
        {
            tempString = tempString + ".tss";
            break;
        }
    case THREAD_FILE:
        {
            tempString = tempString + ".ths";
            break;
        }
    case CMD_SCRIPT:
    case ORA_SCRIPT:
    case SYB_SCRIPT:
    case SQL_SCRIPT:
    case WEB_SCRIPT:
        {
            tempString = tempString + ".scl";
            break;
        }
    case OTA_TOF:
        {
            tempString = tempString + ".tof";
            szFolderName = m_pServer->GetFolderName(CYRServer::enObjectCodeFolder);
            break;
        }
    default:
        {
            tempString = tempString + ".scl";
            break;
        }
    }

    itemName = tempString;

    if (menuEntry == "&Edit")
    {
        ::SendNotifyMessage(GetParent()->m_hWnd,WMU_EDIT_FILE,0,(long)itemName);
    }
    if (menuEntry == "Check &In")
    {
        BOOL bCheckIn = FALSE;
        bCheckIn = ::SendNotifyMessage(GetParent()->m_hWnd,WMU_CHECK_IN,(UINT)szFolderName,(long)itemName);

        if (bCheckIn)
        {
            m_XTreeProject.EnableMenuItem(nType, "Check &In", FALSE);
            m_XTreeProject.EnableMenuItem(nType, "Check &Out", TRUE);
        }
        else
        {
            MessageBox("Check In Failed", "OpenSTA");
        }
    }
    if (menuEntry == "Check &Out")
    {
        BOOL bCheckOut = FALSE;
        bCheckOut = ::SendNotifyMessage(GetParent()->m_hWnd,(UINT)WMU_CHECK_OUT,(UINT)szFolderName,(long)itemName);

        if (bCheckOut)
        {
            m_XTreeProject.EnableMenuItem(nType, "Check &In", TRUE);
            m_XTreeProject.EnableMenuItem(nType, "Check &Out", FALSE);
        }
        else
        {
            MessageBox("Check Out Failed", "OpenSTA");
        }
    }
    if (menuEntry == "&Delete")
    {
        int status;
        status = ::SendNotifyMessage(GetParent()->m_hWnd,WMU_DELETE_FILE,(UINT)szFolderName,(long)itemName);

    }
}

BOOL CXProjectTree::OnInitDialog()
{
    CDialog::OnInitDialog();

    GetWorkDir();

    // Define node types and context menu entries for each type
    m_XTreeProject.DefineNodeType(PROJECT,  "", FALSE ,FALSE);
    m_XTreeProject.DefineNodeType(NODE_FOLDER,  "&New Node,1", FALSE ,FALSE);
    m_XTreeProject.DefineNodeType(TEST_FOLDER,  "&New Test,1", FALSE ,FALSE);
    m_XTreeProject.DefineNodeType(DATA_FILE, "", FALSE ,TRUE);
    m_XTreeProject.DefineNodeType(NODE, "&Edit Node,1|&Delete Node,1", FALSE ,FALSE);
    m_XTreeProject.DefineNodeType(CMD_SCRIPT,   "&Edit,1|&Delete,1", FALSE ,FALSE);
    m_XTreeProject.DefineNodeType(ORA_SCRIPT,   "&Edit,1|&Delete,1", FALSE ,FALSE);
    m_XTreeProject.DefineNodeType(SYB_SCRIPT,   "&Edit,1|&Delete,1", FALSE ,FALSE);
    m_XTreeProject.DefineNodeType(SQL_SCRIPT,   "&Edit,1|&Delete,1", FALSE ,FALSE);
    m_XTreeProject.DefineNodeType(WEB_SCRIPT,   "&Edit,1|&Delete,1", FALSE ,FALSE);
    m_XTreeProject.DefineNodeType(ORA_TOF,      "&Edit,1|&Delete,1", FALSE ,FALSE);
    m_XTreeProject.DefineNodeType(SYB_TOF,      "&Edit,1|&Delete,1", FALSE ,FALSE);
    m_XTreeProject.DefineNodeType(SQL_TOF,      "&Edit,1|&Delete,1", FALSE ,FALSE);
    m_XTreeProject.DefineNodeType(WEB_TOF,      "&Edit,1|&Delete,1", FALSE ,FALSE);
    m_XTreeProject.DefineNodeType(OTA_TOF,      "&Edit OTA,1|&Delete,1", FALSE ,FALSE);

    if (m_bUseSCC)
    {
        m_XTreeProject.DefineNodeType(TEST, "E&xecute,1|&Edit,1|Check &In,0|Check &Out,1|&Delete,1", FALSE ,TRUE);
        m_XTreeProject.DefineNodeType(SCRIPT_FILE,  "&Edit,1|Check &In,0|Check &Out,1|&Delete,1", FALSE ,TRUE);
        m_XTreeProject.DefineNodeType(TEST_FILE,    "&Edit,1|Check &In,0|Check &Out,1|&Delete,1", FALSE ,TRUE);
        m_XTreeProject.DefineNodeType(THREAD_FILE,  "&Edit,1|Check &In,0|Check &Out,1|&Delete,1", FALSE ,TRUE);
    }
    else
    {
        m_XTreeProject.DefineNodeType(TEST, "E&xecute,1|&Edit,1|&Delete,1", FALSE ,TRUE);
        m_XTreeProject.DefineNodeType(SCRIPT_FILE,  "&Edit,1|&Delete,1", FALSE ,TRUE);
        m_XTreeProject.DefineNodeType(TEST_FILE,    "&Edit,1|&Delete,1", FALSE ,TRUE);
        m_XTreeProject.DefineNodeType(THREAD_FILE,  "&Edit,1|&Delete,1", FALSE ,TRUE);
    }

    LoadBitmaps ();

    if (m_strProject=="")
    {
        m_XTreeProject.AddItem(-1,PROJECT,"No Project Selected",FOLDER_ICON,FALSE);
    }
    else
    {
        if (m_strProject.Left(4) == "Cyr.")
        {
            int nLen =  m_strProject.GetLength();
            CString projName = m_strProject.Right(nLen - 4);
            // Strip off the project type
            nLen = projName.GetLength();
            int nPos = projName.Find('.');
            if (nPos != -1)
            {
                projName = projName.Right(nLen - nPos-1);
            }
            m_XTreeProject.AddItem(-1,PROJECT,projName,FOLDER_ICON,TRUE);
        }
    }
    return TRUE;  // return TRUE unless you set the focus to a control
    // EXCEPTION: OCX Property Pages should return FALSE
}


void CXProjectTree::OnSize(UINT nType, int cx, int cy)
{
    CDialog::OnSize(nType, cx, cy);

    SetWindowPos( NULL,0, 0, cx, cy, SWP_NOZORDER|SWP_NOACTIVATE );
    if( m_XTreeProject.GetSafeHwnd() != NULL )
    {
        m_XTreeProject.SetWindowPos( NULL,0, 0, cx, cy, SWP_NOZORDER|SWP_NOACTIVATE );
    }
}

void CXProjectTree::AddTests(HTREEITEM parent)
{
    //
    // Make sure the parent allows children
    //
    m_XTreeProject.SetChildren((long)parent, TRUE);

    //
    //    Get a handle to the current selected project...
    //
    CStringArray arstrTests;
    m_pServer->GetFolderList(CYRServer::enTestsFolder,"",arstrTests);

    //    ...and add them to the tree beneath the project.
    for(int nNdx=0;nNdx<arstrTests.GetSize();nNdx++)
    {
        m_XTreeProject.AddItem((long)parent, TEST,arstrTests[nNdx], TEST_ICON,TRUE);
    }

    if (!arstrTests.GetSize())
    {
        m_XTreeProject.SetChildren((long)parent, FALSE);
    }
}

void CXProjectTree::Activate()
{
    m_XTreeProject.DeleteAllItems();


    if (m_strProject=="")
    {
        m_XTreeProject.AddItem(-1,PROJECT,"No Project Selected",0,FALSE);
    }
    else
    {
        if (m_strProject.Left(4) == "Cyr.")
        {
            int nLen =  m_strProject.GetLength();
            CString projName = m_strProject.Right(nLen - 4);
            // Strip off the project type
            nLen = projName.GetLength();
            int nPos = projName.Find('.');
            if (nPos != -1)
            {
                projName = projName.Right(nLen - nPos-1);
            }
            m_XTreeProject.AddItem(-1,PROJECT,projName,FOLDER_ICON,TRUE);
        }
    }
}

void CXProjectTree::SetActiveProject(CString strProject)
{
    m_strProject=strProject;
}

CString CXProjectTree::GetActiveTest()
{
    return m_strActiveTest;
}

void CXProjectTree::SetServer(CYRServer* pServer)
{
    ASSERT(pServer != NULL);

    m_pServer = pServer;
}

void CXProjectTree::AddTestDependencies(HTREEITEM parent, CString szPath)
{
    //
    // Make sure the parent allows children
    //
    m_XTreeProject.SetChildren((long)parent, TRUE);


    int nLen = szPath.GetLength();
    int nPos = szPath.ReverseFind('|');
    CString filename = szPath.Right(nLen-nPos-1);

    CString sourcePath = m_WorkDir+m_strProject+"\\Source\\"+filename;

    CECSourceFileUtils utils(m_pServer,sourcePath);

    int nDepends = utils.ReadDependencies();
    int nChildren = 0;
    for (int i = 0; i<nDepends; i++)
    {
        DEPENDENCY d = utils.GetDependency(i);
        CString strExt = d.file.Right(3);
        int nLen = d.file.GetLength();
        CString dependFile = d.file.Left(nLen-4);

        //      if (d.type == CECSourceFileUtils::DATA)
        //      {
        //          m_XTreeProject.AddItem((long)parent, DATA_FILE, dependFile, DATA_ICON,FALSE);
        //      }

        //
        //    Recover the correct icon for this dependancy
        //
        int Icon ,NodeType;
        BOOL bHasChildren;
        GetDependencyInfo (d ,dependFile ,Icon ,NodeType ,bHasChildren ,parent);

        //
        //    Increment the count of children and add this item to the tree
        //
        nChildren++;
        m_XTreeProject.AddItem((long)parent, NodeType, dependFile, Icon ,bHasChildren);
    }

    //
    //   If we have any children then show this in the tree
    //
    if (nChildren == 0)
    {
        m_XTreeProject.SetChildren((long)parent, FALSE);
    }
}

void CXProjectTree::OnXTreeItemProperties(LPCTSTR strPath, short nType)
{
    /*  CPropertiesDlg  dlg;
        dlg.SetServerInfo(m_pServer, m_strProject);
        dlg.SetItemInfo(strPath, nType);
        dlg.DoModal();  
    */
}


void CXProjectTree::OnXShowContext(short nNodeType, LPCTSTR strPath)
{
    CStringEx path = strPath;
    int nCount = path.GetFieldCount('|');
    CString item = path.GetField('|',nCount-1);

    item = item + GetExtension (nNodeType);

    if (nNodeType == TEST)
    {


        if (IsFileLocked(nNodeType, item))
        {
            m_XTreeProject.EnableMenuItem(nNodeType, "Check &In", FALSE);
            m_XTreeProject.EnableMenuItem(nNodeType, "Check &Out", TRUE);
        }
        else
        {
            m_XTreeProject.EnableMenuItem(nNodeType, "Check &In", TRUE);
            m_XTreeProject.EnableMenuItem(nNodeType, "Check &Out", FALSE);
        }
    }
}

void CXProjectTree::AddNodes(HTREEITEM parent)
{
    //
    // Make sure the parent allows children
    //
    m_XTreeProject.SetChildren((long)parent, TRUE);
    //
    //    Get a handle to the current selected project...
    //
    CStringArray arstrNodes;
    m_pServer->GetFileList(CYRServer::enNodesFolder,"",arstrNodes);

    //    ...and add them to the tree beneath the project.
    for(int nNdx=0;nNdx<arstrNodes.GetSize();nNdx++)
    {
        m_XTreeProject.AddItem((long)parent, NODE, arstrNodes[nNdx], NODE_ICON,FALSE);
    }

    if(!arstrNodes.GetSize())
    {
        m_XTreeProject.SetChildren((long)parent, FALSE);
    }
}

void CXProjectTree::RefreshTree()
{
    m_XTreeProject.RefreshTree();
}

void CXProjectTree::GetWorkDir()
{
    // Get the work directory
    CString KeyName = SOLUTIONS_SERVER_KEY;
    CString ValueName = "ProjectDir";

    CECRegistryTools reg;
    m_WorkDir =  reg.GetStringValue(HKEY_CURRENT_USER ,KeyName ,ValueName);
}


//
//    IsFileLocked
//
//    Determine if the local file for the specified test object is locked.
//
//    @parm Type of the object whose name is passed
//    @parm Name of the object to check
//
//    Note that the name of the object is its short form ie without an extension or path
//
BOOL CXProjectTree::IsFileLocked(short nNodeType, CString fileName)
{
    CString LocalFile = GetLocalFile (nNodeType ,fileName);

    try
    {
        CFile f(LocalFile, CFile::modeRead);
        CFileStatus status;
        f.GetStatus(status);
        f.Close();

        if (status.m_attribute & 0x01)
        {
            return TRUE;
        }
        else
        {
            return FALSE;
        }
    }
    catch (...)
    {
        return TRUE;
    }
}


void    CXProjectTree::GetDependencyInfo (DEPENDENCY& d ,CString DependencyName
        ,int& Icon ,int& NodeType ,BOOL& bHasChildren
        ,HTREEITEM ParentItem)
{

    //
    //    return the correct dependency icon based on whether the dependency exist, is checked out
    //    and its type.
    //
    bHasChildren = FALSE;
    switch (d.type)
    {
    case CECSourceFileUtils::SCL:
        {

            //
            //   This type of dependancy is set from a CALL SCRIPT/TOF. This can only be done within a thread
            //   type script. We therefore inherit our node type from our parent. Our icon will be the generic
            //   script icon if this is a Call Script and the specific TOF icon for a Call TOF.
            //
            NodeType = m_XTreeProject.GetItemType((long)ParentItem);

            if (d.SrcType == 0)
            {
                Icon = SCL_MISSING_ICON;
                bHasChildren = TRUE;
            }

            else
            {

                //
                //    Seeing as this is a CALL TOF then we must update the target to be a TOF and not a script
                //
                NodeType += NODE_EDITABLE;

                switch(NodeType)
                {
                    //                  case CMD_TOF:
                    //                  {
                    //                      Icon = SCL_MISSING_ICON;
                    //                      bHasChildren = TRUE;
                    //                      break;
                    //                  }
                case ORA_TOF:
                    {
                        Icon = ORA_MISSING_ICON;
                        break;
                    }

                case SYB_TOF:
                    {
                        Icon = SYB_MISSING_ICON;
                        break;
                    }

                case SQL_TOF:
                    {
                        Icon = MSS_MISSING_ICON;
                        break;
                    }

                case WEB_TOF:
                    {
                        Icon = WEB_MISSING_ICON;
                        break;
                    }

                case OTA_TOF:
                    {
                        Icon = OTA_MISSING_ICON;
                        break;
                    }
                }
            }

            break;
        }

    case CECSourceFileUtils::TSS:
        {
            Icon = TSS_MISSING_ICON;
            NodeType = TEST_FILE;
            bHasChildren = TRUE;
            break;
        }

    case CECSourceFileUtils::EXE_SCL:
        {
            if (d.SrcType == 0)
            {
                Icon = SCL_MISSING_ICON;
                bHasChildren = TRUE;
            }

            switch(d.subType)
            {
            case TARGET_COMMAND:
                {
                    NodeType = CMD_SCRIPT;
                    break;
                }

            case TARGET_ORACLE:
                {
                    if (d.SrcType != 0) Icon = ORA_MISSING_ICON;
                    NodeType = ORA_SCRIPT;
                    break;
                }

            case TARGET_SYBASE:
                {
                    if (d.SrcType != 0) Icon = SYB_MISSING_ICON;
                    NodeType = SYB_SCRIPT;
                    break;
                }

            case TARGET_SQLSERVER:
                {
                    if (d.SrcType != 0) Icon = MSS_MISSING_ICON;
                    NodeType = SQL_SCRIPT;
                    break;
                }

            case TARGET_HTTP:
                {
                    if (d.SrcType != 0) Icon = WEB_MISSING_ICON;
                    NodeType = WEB_SCRIPT;
                    break;
                }

            case TARGET_OTA:
                {
                    if(d.SrcType != 0) Icon = OTA_MISSING_ICON;
                    NodeType = OTA_TOF;
                    break;
                }
            }
        }
    }

    //
    //    Create the name of the local file
    //
    CString LocalFile = GetLocalFile (NodeType ,DependencyName);

    //
    //    We have got the base icon - now check for the item either being missing or locked
    //
    CFileStatus FileStatus;
    if (CFile::GetStatus(LocalFile, FileStatus))
    {
        Icon += OBJECT_UNLOCKED;
    }

    else if (m_bUseSCC && IsFileLocked (SCRIPT_FILE, DependencyName))
    {
        Icon += OBJECT_LOCKED;
    }

    //
    //    If the item is in fact an editable script rather than a TOF then move to the appropriate
    //    icon
    //

    return;
}


void    CXProjectTree::LoadBitmaps()
{
    //
    //   Add bitmaps to the image list in groups of three.
    //
    //   NOTE:
    //      These images MUST be added in the order defined by the #defines at the top of this routine
    //      as we use the bitmap indecis later
    //
    CBitmap* bmp = new CBitmap;

    // Misc Bitmaps
    bmp->LoadBitmap(IDB_NODE);
    m_XTreeProject.AddImageToList((long)bmp->m_hObject);
    bmp->DeleteObject();

    bmp->LoadBitmap(IDB_FOLDER);
    m_XTreeProject.AddImageToList((long)bmp->m_hObject);
    bmp->DeleteObject();

    bmp->LoadBitmap(IDB_TEST);
    m_XTreeProject.AddImageToList((long)bmp->m_hObject);
    bmp->DeleteObject();

    // SCL Icoms
    bmp->LoadBitmap(IDB_SCL_MISSING);
    m_XTreeProject.AddImageToList((long)bmp->m_hObject);
    bmp->DeleteObject();

    bmp->LoadBitmap(IDB_SCL_UNLOCKED);
    m_XTreeProject.AddImageToList((long)bmp->m_hObject);
    bmp->DeleteObject();

    bmp->LoadBitmap(IDB_SCL_LOCKED);
    m_XTreeProject.AddImageToList((long)bmp->m_hObject);
    bmp->DeleteObject();

    // TSS Icons
    bmp->LoadBitmap(IDB_TSS_MISSING);
    m_XTreeProject.AddImageToList((long)bmp->m_hObject);
    bmp->DeleteObject();

    bmp->LoadBitmap(IDB_TSS_UNLOCKED);
    m_XTreeProject.AddImageToList((long)bmp->m_hObject);
    bmp->DeleteObject();

    bmp->LoadBitmap(IDB_TSS_LOCKED);
    m_XTreeProject.AddImageToList((long)bmp->m_hObject);
    bmp->DeleteObject();

    // THS Icons
    bmp->LoadBitmap(IDB_THS_MISSING);
    m_XTreeProject.AddImageToList((long)bmp->m_hObject);
    bmp->DeleteObject();

    bmp->LoadBitmap(IDB_THS_UNLOCKED);
    m_XTreeProject.AddImageToList((long)bmp->m_hObject);
    bmp->DeleteObject();

    bmp->LoadBitmap(IDB_THS_LOCKED);
    m_XTreeProject.AddImageToList((long)bmp->m_hObject);
    bmp->DeleteObject();

    // DAT Icons
    bmp->LoadBitmap(IDB_DAT_MISSING);
    m_XTreeProject.AddImageToList((long)bmp->m_hObject);
    bmp->DeleteObject();

    bmp->LoadBitmap(IDB_DAT_UNLOCKED);
    m_XTreeProject.AddImageToList((long)bmp->m_hObject);
    bmp->DeleteObject();

    bmp->LoadBitmap(IDB_DAT_LOCKED);
    m_XTreeProject.AddImageToList((long)bmp->m_hObject);
    bmp->DeleteObject();

    // CMD Icons
    bmp->LoadBitmap(IDB_CMD_MISSING);
    m_XTreeProject.AddImageToList((long)bmp->m_hObject);
    bmp->DeleteObject();

    bmp->LoadBitmap(IDB_CMD_UNLOCKED);
    m_XTreeProject.AddImageToList((long)bmp->m_hObject);
    bmp->DeleteObject();

    bmp->LoadBitmap(IDB_CMD_LOCKED);
    m_XTreeProject.AddImageToList((long)bmp->m_hObject);
    bmp->DeleteObject();

    // ORA Icons
    bmp->LoadBitmap(IDB_ORA_MISSING);
    m_XTreeProject.AddImageToList((long)bmp->m_hObject);
    bmp->DeleteObject();

    bmp->LoadBitmap(IDB_ORA_UNLOCKED);
    m_XTreeProject.AddImageToList((long)bmp->m_hObject);
    bmp->DeleteObject();

    bmp->LoadBitmap(IDB_ORA_LOCKED);
    m_XTreeProject.AddImageToList((long)bmp->m_hObject);
    bmp->DeleteObject();

    // SYB Icons
    bmp->LoadBitmap(IDB_SYB_MISSING);
    m_XTreeProject.AddImageToList((long)bmp->m_hObject);
    bmp->DeleteObject();

    bmp->LoadBitmap(IDB_SYB_UNLOCKED);
    m_XTreeProject.AddImageToList((long)bmp->m_hObject);
    bmp->DeleteObject();

    bmp->LoadBitmap(IDB_SYB_LOCKED);
    m_XTreeProject.AddImageToList((long)bmp->m_hObject);
    bmp->DeleteObject();

    // MSS Icons
    bmp->LoadBitmap(IDB_MSS_MISSING);
    m_XTreeProject.AddImageToList((long)bmp->m_hObject);
    bmp->DeleteObject();

    bmp->LoadBitmap(IDB_MSS_UNLOCKED);
    m_XTreeProject.AddImageToList((long)bmp->m_hObject);
    bmp->DeleteObject();

    bmp->LoadBitmap(IDB_MSS_LOCKED);
    m_XTreeProject.AddImageToList((long)bmp->m_hObject);
    bmp->DeleteObject();

    // WEB Icons
    bmp->LoadBitmap(IDB_WEB_MISSING);
    m_XTreeProject.AddImageToList((long)bmp->m_hObject);
    bmp->DeleteObject();

    bmp->LoadBitmap(IDB_WEB_UNLOCKED);
    m_XTreeProject.AddImageToList((long)bmp->m_hObject);
    bmp->DeleteObject();

    bmp->LoadBitmap(IDB_WEB_LOCKED);
    m_XTreeProject.AddImageToList((long)bmp->m_hObject);
    bmp->DeleteObject();

    // OTA Icons
    bmp->LoadBitmap(IDB_OTA_MISSING);
    m_XTreeProject.AddImageToList((long)bmp->m_hObject);
    bmp->DeleteObject();

    bmp->LoadBitmap(IDB_OTA_UNLOCKED);
    m_XTreeProject.AddImageToList((long)bmp->m_hObject);
    bmp->DeleteObject();

    bmp->LoadBitmap(IDB_OTA_LOCKED);
    m_XTreeProject.AddImageToList((long)bmp->m_hObject);
    //  bmp->DeleteObject();




    //
    //    Last bitmap so simply delete it
    //
    delete bmp;
    return;
}

CString CXProjectTree::GetExtension (int NodeType) const
{
    CString Extension;
    switch (NodeType)
    {
    case TEST:
    case TEST_FILE:
        {
            Extension = ".tss";
            break;
        }

    case SCRIPT_FILE:
    case CMD_SCRIPT:
    case ORA_SCRIPT:
    case SYB_SCRIPT:
    case SQL_SCRIPT:
    case WEB_SCRIPT:
        {
            Extension = ".scl";
            break;
        }

    default:
        {
            Extension = ".tof";
            break;
        }

    }
    return Extension;
}

CString CXProjectTree::GetLocalFile     (int NodeType ,const CString ObjectName) const
{
    CString KeyName = SOLUTIONS_SERVER_KEY;
    CString ValueName = "ProjectDir";

    CECRegistryTools reg;
    CString workDir = reg.GetStringValue(HKEY_CURRENT_USER ,KeyName ,ValueName);

    //
    //    the source file may be held in either the Compiled Objects or in the source directory
    //    depending on whether the object has a source version or not. If the extension of the
    //    file name is identified as being TOF then it is in the compiled objects folder
    //
    CString localFile;

    CString Extension = GetExtension (NodeType);
    if (Extension == ".tof")
    {
        localFile.Format ("%s%s\\%s\\%s%s" ,workDir ,m_strProject ,m_pServer->GetFolderName(CYRServer::enObjectCodeFolder) ,ObjectName ,Extension);
    }
    else
    {
        localFile.Format ("%s%s\\%s\\%s%s" ,workDir ,m_strProject ,m_pServer->GetFolderName(CYRServer::enSourceFolder) ,ObjectName ,Extension);
    }

    return localFile;
}

//
//    EditTof
//
//    This is very much a bodge routine as we have no facility for invoking either the recorder or
//    the modeller. This routine should be replaced later by the registration of object types by the
//    Impact DLL allowing the use of a document to determine how to open/edit sources. In the
//    meantime we shall invoke the images ourselves here to keep the mess to a minimum
//
void    CXProjectTree::EditTof  (int NodeType ,const CString ObjectName)
{
    CString LocalFile = GetLocalFile (NodeType ,ObjectName);

    //
    //    ...and get the default extension which we will add onto the image name
    //
    enum TARGETID TargetId;
    if (NodeType == SYB_SCRIPT)
    {
        TargetId = TARGET_SYBASE;
    }
    else if (NodeType == ORA_SCRIPT)
    {
        TargetId = TARGET_ORACLE;
    }
    else if (NodeType == SQL_SCRIPT)
    {
        TargetId = TARGET_SQLSERVER;
    }
    else if (NodeType == WEB_SCRIPT)
    {
        TargetId = TARGET_HTTP;
    }
    else if(NodeType == OTA_TOF)
    {
        CWnd* pMainWnd = AfxGetMainWnd();
        CString strParams;
        strParams.Format("%s|%s" ,SOLUTIONS_GENERIC_DLL,ObjectName);
        pMainWnd->SendMessage(WMU_FIRE_SPECIFIC_EVENT,(WPARAM)WMU_EDIT_OTA_TOF,(LPARAM)&strParams);
        return;
    }
    else
    {
        TargetId = TARGET_COMMAND;
    }

    //
    //    If the file exists then we should use the test Modeller to edit it, otherwise we use
    //    the recorder to record it.
    //
    CFileStatus FileStatus;

    if (CFile::GetStatus(LocalFile, FileStatus))
    {
        CECTargetEnvironment te;
        te.InvokeModeller (TargetId ,ObjectName);
    }
    else
    {
        CECTargetEnvironment te;
        te.InvokeRecorder (TargetId ,ObjectName);
    }

    //
    //    ...and use a common routine to invoke the appropriate image
    //

    return;
}


void CXProjectTree::EnableMenus(BOOL bEnabaled)
{
    m_XTreeProject.EnableMenus(bEnabaled);
}

BOOL CXProjectTree::OnEraseBkgnd(CDC *pDC)
{
    return true;
}
