/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// CXReposTree1.cpp : implementation file
//

#include "stdafx.h"
#include "XReposTree.h"
#include "CXReposTree1.h"
#include "Server.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CXReposTree dialog


CXReposTree::CXReposTree(CWnd* pParent /*=NULL*/)
        : CDialog(CXReposTree::IDD, pParent)
{
    //{{AFX_DATA_INIT(CXReposTree)
    // NOTE: the ClassWizard will add member initialization here
    //}}AFX_DATA_INIT
    m_CurrentItem=NULL;
    m_pServer = NULL;
}


void CXReposTree::DoDataExchange(CDataExchange* pDX)
{
    CDialog::DoDataExchange(pDX);
    //{{AFX_DATA_MAP(CXReposTree)
    DDX_Control(pDX, IDC_XTREE_REPOS, m_XTreeRepos);
    //}}AFX_DATA_MAP
}



BEGIN_MESSAGE_MAP(CXReposTree, CDialog)
//{{AFX_MSG_MAP(CXReposTree)
ON_WM_SIZE()
//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CXReposTree message handlers

BOOL CXReposTree::OnInitDialog()
{
    CDialog::OnInitDialog();
    AfxEnableControlContainer(NULL);

    // Define node types and context menu entries for each type
    m_XTreeRepos.DefineNodeType(PROJECT,    "p1|p2", FALSE ,FALSE);
    m_XTreeRepos.DefineNodeType(ACTIVE_PROJECT, "p1|p2", FALSE ,FALSE);

    // Add bitmaps to the image list
    CBitmap* bmp = new CBitmap;
    bmp->LoadBitmap(IDB_FOLDER);
    m_XTreeRepos.AddImageToList((long)bmp->m_hObject);
    delete bmp;
    bmp = new CBitmap;
    bmp->LoadBitmap(IDB_ACTIVE);
    m_XTreeRepos.AddImageToList((long)bmp->m_hObject);
    delete bmp;

    AddProjects();

    return TRUE;  // return TRUE unless you set the focus to a control
    // EXCEPTION: OCX Property Pages should return FALSE
}


void CXReposTree::AddProjects()
{
    //
    //    Get a list of the projects from the server
    //
    CStringArray arstrProjects;
    m_pServer->GetProjectList(arstrProjects);

    //
    //    ...and add them to the tree.
    //
    for(int nNdx=0;nNdx<arstrProjects.GetSize();nNdx++)
    {
        m_XTreeRepos.AddItem(-1, PROJECT ,arstrProjects[nNdx], 0, TRUE);
    }
}

void CXReposTree::OnSize(UINT nType, int cx, int cy)
{
    CDialog::OnSize(nType, cx, cy);

    SetWindowPos( NULL,0, 0, cx, cy, SWP_NOZORDER|SWP_NOACTIVATE );
    if( m_XTreeRepos.GetSafeHwnd() != NULL )
    {
        m_XTreeRepos.SetWindowPos( NULL,0, 0, cx, cy, SWP_NOZORDER|SWP_NOACTIVATE );
    }
}

BEGIN_EVENTSINK_MAP(CXReposTree, CDialog)
//{{AFX_EVENTSINK_MAP(CXReposTree)
ON_EVENT(CXReposTree, IDC_XTREE_REPOS, 1 /* LBDblClick */, OnLBDblClickXtreeRepos, VTS_I4 VTS_I2 VTS_BSTR)
//}}AFX_EVENTSINK_MAP
END_EVENTSINK_MAP()

void CXReposTree::OnLBDblClickXtreeRepos(long hTreeItem, short nNodeType, LPCTSTR szPath)
{
    if (m_strActiveProject == szPath)
    {
        return;
    }
    if (m_CurrentItem!=NULL)
    {
        m_XTreeRepos.SetItemImage((long)m_CurrentItem,0);
    }
    m_CurrentItem = (HTREEITEM)hTreeItem;
    m_XTreeRepos.SetItemImage((long)m_CurrentItem,1);

    m_strActiveProject = szPath;
    LPCTSTR projName = m_strActiveProject;
    ::SendNotifyMessage(GetParent()->m_hWnd,WMU_CHANGE_PROJECT,0,(long)projName);

}

void CXReposTree::Activate()
{
    m_XTreeRepos.DeleteAllItems();
    AddProjects();
}

void CXReposTree::SetServer(CYRServer* pServer)
{
    ASSERT(pServer != NULL);

    m_pServer = pServer;
}

void CXReposTree::RefreshTree()
{
    m_XTreeRepos.RefreshTree();
}

void CXReposTree::EnableMenus(BOOL bEnabaled)
{
    m_XTreeRepos.EnableMenus(bEnabaled);

}
