/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// PropertiesDlg.cpp : implementation file
//

#include "stdafx.h"
#include "PropertiesDlg.h"
#include "CXUserMsg.h"
#include "CyrStApi\CECComSCLFile.h"
#include "CyrStApi\CECComNodeFile.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CPropertiesDlg dialog


CPropertiesDlg::CPropertiesDlg(CWnd* pParent /*=NULL*/)
        : CDialog(CPropertiesDlg::IDD, pParent)
{
    //{{AFX_DATA_INIT(CPropertiesDlg)
    m_strType = _T("");
    m_strName = _T("");
    m_strCreateTime = _T("");
    m_strDesc = _T("");
    m_strModifyTime = _T("");
    m_strLocalPath = _T("");
    m_strModifiedBy = _T("");
    //}}AFX_DATA_INIT

    m_pServer = NULL;
    m_pComProject = NULL;
    m_pComRootFolder = NULL;
}


void CPropertiesDlg::DoDataExchange(CDataExchange* pDX)
{
    CDialog::DoDataExchange(pDX);
    //{{AFX_DATA_MAP(CPropertiesDlg)
    DDX_Text(pDX, IDC_ITEM_TYPE, m_strType);
    DDX_Text(pDX, IDC_ITEM_NAME, m_strName);
    DDX_Text(pDX, IDC_ITEM_CREATE_TIME, m_strCreateTime);
    DDX_Text(pDX, IDC_ITEM_DESC, m_strDesc);
    DDX_Text(pDX, IDC_ITEM_MODIFY_TIME, m_strModifyTime);
    DDX_Text(pDX, IDC_ITEM_PATH, m_strLocalPath);
    DDX_Text(pDX, IDC_ITEM_MODIFIED_BY, m_strModifiedBy);
    //}}AFX_DATA_MAP
}



BEGIN_MESSAGE_MAP(CPropertiesDlg, CDialog)
//{{AFX_MSG_MAP(CPropertiesDlg)
//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CPropertiesDlg message handlers
void CPropertiesDlg::SetServerInfo(CECComServer* pServer, CString strActiveProject)
{
    m_pServer = pServer;
    m_strActiveProject = strActiveProject;

    m_pComProject = new CECComProject(m_pServer, m_strActiveProject.GetBuffer(0));
    m_pComRootFolder = new CECComFolder(m_pComProject);
    ;

}

void CPropertiesDlg::SetItemInfo(CString strPath, int nType)
{
    m_strName = strPath;

    switch (nType)
    {
    case PROJECT:
        {
            SetProjectInfo();
            break;
        }
    case TESTRUN:
        {
            m_strName = strPath;
            m_strType = "Test Run Time :";
            break;
        }
    case TEST:
        {
            SetFileInfo("tss");
            m_strType = "Test Name :";
            int nLen = m_strName.GetLength();
            m_strName = m_strName.Left(nLen-4);
            break;
        }
    case SCRIPT_FILE:
        {
            SetFileInfo("scl");
            break;
        }
    case TEST_FILE:
        {
            SetFileInfo("tss");
            break;
        }
    case THREAD_FILE:
        {
            SetFileInfo("ths");
            break;
        }
    case DATA_FILE:
        {
            int nLen = strPath.GetLength();
            int nPos = strPath.ReverseFind('|');
            m_strName = strPath.Right(nLen-nPos-1);
            m_strType = "Data File :";
            break;
        }
    case NODE:
        {
            SetNodeInfo();
            break;
        }
    case OTA_TOF:
        {
            SetTofInfo();
        }
        break;
    }

    //  UpdateData(FALSE);
}


void CPropertiesDlg::SetProjectInfo()
{
    RWTime rwTime;
    RWCString rwString;

    m_strType = "Project :";
    m_strModifiedBy = "N/A";

    /*
        rwTime = m_pComRootFolder->GetCreatedTime();
        rwString = rwTime.asString("%H:%M:%S %d/%m/%Y" );
        m_strCreateTime = (CString)rwString.data();
    */
    rwString = m_pComProject->GetDescription();
    m_strDesc = (CString)rwString.data();
    /*
        rwTime = m_pComRootFolder->GetModifiedTime();
        rwString = rwTime.asString("%H:%M:%S %d/%m/%Y" );
        m_strModifyTime = (CString)rwString.data();
    */
    rwString = m_pComRootFolder->GetFullPath();
    m_strLocalPath = (CString)rwString.data();

}

void CPropertiesDlg::SetFileInfo(CString strExt)
{
    RWTime rwTime;
    RWCString rwString;

    // Find the File name
    int nLen = m_strName.GetLength();
    int nPos = m_strName.ReverseFind('|');
    m_strName = m_strName.Right(nLen-nPos-1) + "." + strExt;
    ;

    // Get the file from the server
    CECComSourceFolder Sources(m_pComRootFolder);

    RWCString strFile(m_strName.GetBuffer(0));
    CECComSCLFile *psclFile;
    try
    {
        psclFile = new CECComSCLFile( &Sources, strFile);
    }
    catch(CyrRepositoryException e)
    {
        ASSERT(FALSE);
        return;
    }

    m_strType = "Script :";
    RWCString modBy = psclFile->GetModifiedBy();
    m_strModifiedBy = (CString)modBy.data();

    rwTime = psclFile->GetCreatedTime();
    rwString = rwTime.asString("%H:%M:%S %d/%m/%Y" );
    m_strCreateTime = (CString)rwString.data();

    rwTime = psclFile->GetModifiedTime();
    rwString = rwTime.asString("%H:%M:%S %d/%m/%Y" );
    m_strModifyTime = (CString)rwString.data();

    rwString = psclFile->GetDescription();
    m_strDesc = (CString)rwString.data();

    rwString = psclFile->GetFullPath();
    m_strLocalPath = (CString)rwString.data();
}

void CPropertiesDlg::SetNodeInfo()
{
    RWTime rwTime;
    RWCString rwString;

    // Find the Node name
    int nLen = m_strName.GetLength();
    int nPos = m_strName.ReverseFind('|');
    m_strName = m_strName.Right(nLen-nPos-1);

    // Get the file from the server
    CECComNodeFolder Nodes(m_pComRootFolder);

    RWCString strFile(m_strName.GetBuffer(0));
    CECComNodeFile nodeFile( &Nodes, strFile);

    m_strType = "Node :";
    RWCString modBy = nodeFile.GetModifiedBy();
    m_strModifiedBy = (CString)modBy.data();

    rwTime = nodeFile.GetCreatedTime();
    rwString = rwTime.asString("%H:%M:%S %d/%m/%Y" );
    m_strCreateTime = (CString)rwString.data();

    rwTime = nodeFile.GetModifiedTime();
    rwString = rwTime.asString("%H:%M:%S %d/%m/%Y" );
    m_strModifyTime = (CString)rwString.data();

    rwString = nodeFile.GetDescription();
    m_strDesc = (CString)rwString.data();

    rwString = nodeFile.GetFullPath();
    m_strLocalPath = (CString)rwString.data();
}

void CPropertiesDlg::SetTofInfo()
{
    RWTime rwTime;
    RWCString rwString;

    // Find the Node name
    int nLen = m_strName.GetLength();
    int nPos = m_strName.ReverseFind('|');
    m_strName = m_strName.Right(nLen-nPos-1) + ".tof";

    // Get the file from the server
    CECComObjectCodeFolder Tofs(m_pComRootFolder);

    RWCString strFile(m_strName.GetBuffer(0));
    CECComSCLFile tofFile( &Tofs, strFile);

    m_strDesc = "Tof File";
    m_strType = "Tof :";
    RWCString modBy = tofFile.GetModifiedBy();
    m_strModifiedBy = (CString)modBy.data();

    rwTime = tofFile.GetCreatedTime();
    rwString = rwTime.asString("%H:%M:%S %d/%m/%Y" );
    m_strCreateTime = (CString)rwString.data();

    rwTime = tofFile.GetModifiedTime();
    rwString = rwTime.asString("%H:%M:%S %d/%m/%Y" );
    m_strModifyTime = (CString)rwString.data();

    rwString = tofFile.GetDescription();
    m_strDesc = (CString)rwString.data();

    rwString = tofFile.GetFullPath();
    m_strLocalPath = (CString)rwString.data();
}
