/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

///////////////////////////////////////////////////////////////////////////////
// @doc     CyrImpactLib
// @module  Impact Project Wrapper | This module contains the implementation of
//          the CECImpactProject wrapper class.  This class extends the
//          CECProject wrapper class and creates all the appropriate child
//          folders.
//
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
// Cyrano includes
#include <Factory/ParamList_i.h>
#include <CyrImpactLib/CECImpactProject.h>
#include <CyrImpactLib/CECImpactFolders.h>
#include <Repository/CyrStByteIt_i.h>
#include <Repository/CyrStRepository.h>

///////////////////////////////////////////////////////////////////////////////
// Constants
const char* BUILD_FILE = "BuildSettings";

///////////////////////////////////////////////////////////////////////////////
// @mfunc   This constructor retrieves an existing Impact project from the
//          OpenSTARep database.
// @parm    Pointer to an instance of the parent Server wrapper class.
// @parm    The name of the Project to retrieve.
CECImpactProject::CECImpactProject( CECOrbServer* server, const char* name )
        : CECOrbProject( server, string( name ).c_str())
{
    m_debug = false;
    m_list = false;
    m_checkSteps = false;
    m_steps = 1;

    ReadBuildSettings();
}

///////////////////////////////////////////////////////////////////////////////
// @mfunc   The destructor of this class writes out the current buidl settings.
CECImpactProject::~CECImpactProject()
{}


bool CECImpactProject::GetBuildDebug()
{
    return m_debug;
}

bool CECImpactProject::GetBuildList()
{
    return m_list;
}

bool CECImpactProject::GetBuildCheckSteps()
{
    return m_checkSteps;
}

long CECImpactProject::GetBuildMaxSteps()
{
    return m_steps;
}

///////////////////////////////////////////////////////////////////////////////
// @mfunc   This method reads the build settings from a remote file.
void CECImpactProject::ReadBuildSettings()
{
    // Get root folder
    CECOrbFolder* rootFolder = new CECOrbFolder( this );
    CECOrbFile* settings = NULL;

    try
    {
        // Get build settings file from root folder
        settings = new CECOrbFile( rootFolder, BUILD_FILE );

        // Get underlying CORBA interface
        CyrStFile_var iFile = settings->GetServant();

        // Transfer bytes of file as appropriate
        CyrStByteIt_var iFileIt = iFile->GetFile( false );
        m_debug = iFileIt->ReadBool();
        m_list = iFileIt->ReadBool();
        m_checkSteps = iFileIt->ReadBool();
        m_steps = iFileIt->ReadLong();
    }
    catch( ... )
    {}

    // Tidy memory

    if ( settings != NULL )
        delete settings;
    if ( rootFolder != NULL )
        delete rootFolder;
}

