/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

#ifndef __LOGFILE__
#define __LOGFILE__

//==================================================================< Pragmas >
#ifdef  WIN32
#  pragma warning(disable : 4710 4275)
#endif  // WIN32


//=================================================================< Includes >
#ifndef __DEFINITION__
#  include "definition.h"
#endif  // __DEFINITION__

#ifndef __EXCEPTION__
#  include <exception.hpp>
#endif  // __EXCEPTION__

#ifndef __MTSAFE__
#  include <mtsafe.hpp>
#endif  // __MTSAFE__

#ifndef __MUTEX__
#  include <mutex.hpp>
#endif  // __MUTEX__
#ifdef  WIN32
#    pragma warning(disable : 4100 4512 4516)
#endif  // WIN32
#ifdef  WIN32
#    pragma warning(default : 4100)
#endif  // WIN32
#include <vector>
#include <string>
using std::vector;
using std::string;

#include <fstream>
#include <iostream>

#ifdef  WIN32
#  include <strstream>
#endif  // WIN32
#ifdef  UNIX
#  include <strstream.h>
#endif  // UNIX

/*#ifndef __EVENTLOG__
#include <eventlog.hpp>
#endif  // __EVENTLOG__*/
using std::ofstream;
using std::fstream;
using std::cout;
using std::cerr;
using std::endl;
using std::strstream;
using std::ostrstream;
using std::ios;

//===================================================================< Macros >
#define DLLSPEC_LOGFILE DLLSPEC (CYRTOOLS)


//====================================================================< Class >

///////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////// Class CTLogFileException //
///////////////////////////////////////////////////////////////////////////////
// Class used to keep log file exception.
class DLLSPEC_LOGFILE CTLogFileException : public CTException
{
    //-----------------------------------------------------------------------------
public:                     // Constructor(s) and destructor.
    CTLogFileException (const string& clMsg, const string& clFilename, short sLineno, long lNumber = -1L)
: CTException (clMsg, clFilename, sLineno, lNumber)
    { }


public:                     // Operators.
    void                    operator= (const CTLogFileException& exception)
    {
        CTException::operator= (exception) ;
    }

    //-----------------------------------------------------------------------------
}

; // CTLogFileException

///////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////// End of Class CTLogFileException //
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////// Class CTOLogFileIterator //
///////////////////////////////////////////////////////////////////////////////

class DLLSPEC_LOGFILE CTOLogFileIterator : public CTMTSafe
{
    //-----------------------------------------------------------------------------
public:                 // Types.
    enum enumOLogFileType                   // Log file type.
    {
        LOGFILE_OFSTREAM,
        LOGFILE_OSTREAM,
        LOGFILE_OSTRSTREAM
        //      LOGFILE_OEVENTLOGSTREAM
    }
    ;

public:                 // Constructor(s) and destructor.
    CTOLogFileIterator  (ofstream& ofs)
: os_ ( (ostream&) ofs), eOLogFileType_ (LOGFILE_OFSTREAM)
    { }
    ;
    CTOLogFileIterator  (ostream& osa)
: os_ ( (ostream&) osa), eOLogFileType_ (LOGFILE_OSTREAM)
    { }
    ;
    CTOLogFileIterator  (ostrstream& oss)
: os_ ( (ostream&) oss), eOLogFileType_ (LOGFILE_OSTRSTREAM)
    { }
    ;
    /*  CTOLogFileIterator  (CTOEventLogStream& oel)
                            : os_ ( (ostream&) oel), eOLogFileType_ (LOGFILE_OEVENTLOGSTREAM)
                            { } ;*/

public:             // Operators.
    CTOLogFileIterator& operator= (const CTOLogFileIterator& ctol)
    {
        os_ = ctol.os_;
        eOLogFileType_ = ctol.eOLogFileType_ ;
        return *this ;
    }

public:                 // Get methodes.
    ostream&            getLogFile (void) const
    {
        return os_ ;
    } ;
    enumOLogFileType    getType (void) const
    {
        return eOLogFileType_ ;
    } ;

    //-----------------------------------------------------------------------------
private:                // Members.
    ostream&            os_ ;
    enumOLogFileType    eOLogFileType_ ;

    //-----------------------------------------------------------------------------
}

; // CTOLogFileIterator

///////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////// End of Class CTOLogFileIterator //
///////////////////////////////////////////////////////////////////////////////


///////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////// Class CTOLogFileList //
///////////////////////////////////////////////////////////////////////////////
class DLLSPEC_LOGFILE CTOLogFileList : public vector<CTOLogFileIterator>, virtual public CTMTSafe
{
    //-----------------------------------------------------------------------------
public:                     // Operators.
    inline CTOLogFileList&  operator<< (CTOLogFileList& (*f) (CTOLogFileList&) ) ;
    CTOLogFileList&         operator<< (const char*) ;
    CTOLogFileList&         operator<< (char) ;
    CTOLogFileList&         operator<< (unsigned char) ;
    CTOLogFileList&         operator<< (int) ;
    CTOLogFileList&         operator<< (unsigned int) ;
    CTOLogFileList&         operator<< (long) ;
    CTOLogFileList&         operator<< (unsigned long) ;
    CTOLogFileList&         operator<< (short) ;
    CTOLogFileList&         operator<< (unsigned short) ;
    CTOLogFileList&         operator<< (float) ;
    CTOLogFileList&         operator<< (double) ;
    CTOLogFileList&         operator<< (const string&) ;

public:                     // Get methodes.
    CTMutex&                getMutex (void)
    {
        return mutexForAccess_ ;
    } ;

    //-----------------------------------------------------------------------------
private:                    // Members.
    CTMutex                 mutexForAccess_ ;

    //-----------------------------------------------------------------------------
}

; // CTOLogFileList

inline CTOLogFileList&  CTOLogFileList::operator<< (CTOLogFileList& (*f) (CTOLogFileList&) )
{
    (*f) (*this) ;
    return *this ;
}

DLLSPEC_LOGFILE CTOLogFileList& dec (CTOLogFileList& logFileList) ;
DLLSPEC_LOGFILE CTOLogFileList& hex (CTOLogFileList& logFileList) ;
DLLSPEC_LOGFILE CTOLogFileList& oct (CTOLogFileList& logFileList) ;

DLLSPEC_LOGFILE CTOLogFileList& flush (CTOLogFileList& logFileList) ;
DLLSPEC_LOGFILE CTOLogFileList& endl (CTOLogFileList& logFileList) ;
DLLSPEC_LOGFILE CTOLogFileList& ends (CTOLogFileList& logFileList) ;

DLLSPEC_LOGFILE CTOLogFileList& error (CTOLogFileList& logFileList) ;
DLLSPEC_LOGFILE CTOLogFileList& warning (CTOLogFileList& logFileList) ;
DLLSPEC_LOGFILE CTOLogFileList& information (CTOLogFileList& logFileList) ;
DLLSPEC_LOGFILE CTOLogFileList& auditSuccess (CTOLogFileList& logFileList) ;
DLLSPEC_LOGFILE CTOLogFileList& auditFailure (CTOLogFileList& logFileList) ;

class DLLSPEC_LOGFILE CTOLogFileListManipulatorArg
{
    //-----------------------------------------------------------------------------
public:                     // Constructor(s) and destructor.
    CTOLogFileListManipulatorArg (CTOLogFileList& (*pf) (CTOLogFileList&, void*, int), void* pBuf, int iBufLen)
: pf_ (pf), pBuf_ (pBuf), iBufLen_ (iBufLen)
    { }
    ;

public:                     // Operators.
    friend CTOLogFileList&  operator<< (CTOLogFileList& logFileList, const CTOLogFileListManipulatorArg& m)
    {
        (*(m.pf_) ) (logFileList, m.pBuf_, m.iBufLen_) ;
        return logFileList ;
    } ;

public:                     // Members.
    CTOLogFileList&         (*pf_) (CTOLogFileList&, void*, int) ;
    void*                   pBuf_ ;
    int                     iBufLen_ ;

    //-----------------------------------------------------------------------------
}

; // CTOLogFileListManipulatorArg

DLLSPEC_LOGFILE CTOLogFileList& __listBinaryData (CTOLogFileList& logFileList, void* pBuf, int iBufLen) ;

inline CTOLogFileListManipulatorArg listBinaryData (void* pBuf, int iBufLen)
{
    return CTOLogFileListManipulatorArg (__listBinaryData, pBuf, iBufLen) ;
}

///////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////// End of Class CTOLogFileList //
///////////////////////////////////////////////////////////////////////////////

#endif // __LOGFILE__
