/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

/* +--------------------------------------------------------------------
   | NOM FICHIER     :   process.hpp
   | DESCRIPTION     :   
   | DATE CREATION   :   Mon Jan  4 12:01:36 1999
   +--------------------------------------------------------------------
*/
#include <definition.h>

#ifdef WIN32
#include <process.h>
#define NULL 0
#endif

//#include <CyrDefs.h>
#include <string>
using std::string;

#ifdef WIN32
#include <windows.h>
typedef HANDLE processType ;
#define NULLPID     NULL
#else
#include <omnithread.h>
#ifdef SOLARIS
#include <sys/wait.h>
#endif

#include <sys/types.h>
#include <unistd.h>
#include <signal.h>
#include <errno.h>
#include <syslog.h>
#include <sys/wait.h>
#include <fcntl.h>
//#include <rw/ctoken.h>
typedef int processType ;
extern int errno;
extern char *sys_errlist[ ];
extern int sys_nerr;
typedef int processType ;
#define NULLPID     0
#endif

//===================================================================< Macros >
#define DLLSPEC_PROCESS DLLSPEC(CYRTOOLS)


class DLLSPEC_PROCESS CTProcess
{
public:
    CTProcess(string processName,
              bool autodelete=FALSE, /* only used by UNIX.... */
              processType pid = NULLPID )
: autodelete_(autodelete),processName_(processName),pid_(pid), iErrorCode_(0), errorMessage_("")
    {}

    ~CTProcess() ;

    processType         getPid()
    {
        return pid_;
    }
#ifdef _WIN32
    long                getActualPid()
    {
        return actual_pid_;
    };
#endif
    const string    getErrorMessage(void) const
    {
        return errorMessage_ ;
    }
    int                 getErrorCode(void) const
    {
        return iErrorCode_ ;
    }
    const string    getProcessName() const
    {
        return processName_ ;
    }

    void                setPid(processType pid)
    {
        pid_ = pid ;
    }
    void                setErrorMessage(string errorMessage)
    {
        errorMessage_ = errorMessage ;
    }
    void                setErrorCode(int iErrorCode)
    {
        iErrorCode_ = iErrorCode ;
    }

    int                 start() ;
    int                 wait() ;
#ifdef UNIX
    void                waitAndExit() ;
#endif
    bool            isRunning() ;
    bool            closePid() ;

    static int          setPidInSharedMemory(string memoryName) ;
    static int          getPidFromSharedMemory(string memoryName, int & iPid) ;
    static int          stop(int iPid) ;
    static int          removeSharedMemory(string memoryName);
    virtual bool    operator==(const CTProcess &other)
    {
        return (other.processName_ == processName_);
    }

protected:
    string      processName_ ;
    processType     pid_ ;
    int             iErrorCode_ ;
    string      errorMessage_ ;
#ifdef UNIX
    omni_thread *threadWait_;
#endif
    bool autodelete_;
#ifdef _WIN32
    long            actual_pid_;
#endif
};

