/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

///////////////////////////////////////////////////////////////////////////////
// @doc     VDK_MODULE
// @module  Cyrano Provider VDK | This module is the implementation of a VDK
//          module.  These modules are responsible for creating and
//          initialising Providers.
//
//          Typically VDKs will be loaded by the Cyrano Daemon applciation
//          and utilise the Factory within that application.
//
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
// Cyrano includes
#include <Factory/CyrThreadProv.h>
#include <Daemon/CyrVdk.h>

///////////////////////////////////////////////////////////////////////////////
// Global variables
// @globalv CyrThreadProvider | pProvThrd | Pointer to Provider of SpawnProcess
CyrThreadProvider* pProvThrd;

///////////////////////////////////////////////////////////////////////////////
// @func   This function initialises the Provider objects within the DLL.  It
//         is also repsoinsible for any other setup stuff.
// @rdesc  A standard Cyrano error code.
// @parm   The name of the parent factory that all Providers within this DLL
//         will register themselves with.
// @parm   A reference to some CORBA helper functionality.
VDK_EXP CyrVdkInitialise( const char* factory, CORBAServer* pServer )
{
    string name;

    // Create a new collection of Providers
    name = factory;
    name += "_CyrThreadProvider_001";
    pProvThrd = new CyrThreadProvider( pServer, name.c_str(), factory );

    return CYR_SUCCESS;
}

///////////////////////////////////////////////////////////////////////////////
// @func  This function is responisble for destroying any Provider objects
//        created in the <f CyrVdkInitialise> function.  It also performs any
//        other required tidying up.
// @rdesc A standard Cyrano error code.
VDK_EXP CyrVdkUninitialise()
{
    // Dispose of any providers
    if ( !CORBA::is_nil( pProvThrd ))
    {
        pProvThrd->Shutdown();
        pProvThrd->_dispose();
    }

    return CYR_SUCCESS;
}


