/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

///////////////////////////////////////////////////////////////////////////////
// @doc Logging
///////////////////////////////////////////////////////////////////////////////

#ifndef _CECDiagnostic_h
#define _CECDiagnostic_h

///////////////////////////////////////////////////////////////////////////////
// Cyrano includes
#include <CORBA/CORBAServer.h>
#include <Logging/CyrLog.h>
#include <CyrWrapLib/CECOrbFile.h>
#include <ctime.hpp>

#include <fstream>
using std::fstream;
///////////////////////////////////////////////////////////////////////////////
// @class   CECTrace | This class represents a single Cyrano trace entry.  It
//          parses up a returned string entry into something a little easier to
//          handle.
class CECTrace
{
    // @access Public Members
public:
    CECTrace( const char* trace );

    CTTime    GetTime() const
    {
        return m_time;
    }
    string GetNode() const
    {
        return m_node;
    }
    string GetUser() const
    {
        return m_user;
    }
    string GetCategory() const
    {
        return m_category;
    }
    string GetExecutor() const
    {
        return m_executor;
    }
    long      GetLevel() const
    {
        return m_level;
    }
    string GetMessage() const
    {
        return m_msg;
    }

    // @access Private Members
private:
    CTTime    m_time;       // @cmember Date and time of trace generation.
    string m_node;       // @cmember Name of node where trace was generated.
    string m_user;       // @cmember Name of user running test.
    string m_category;   // @cmember Category of trace output.
    string m_executor;   // @cmember Name of executor generating output.
    long      m_level;      // @cmember Level of logging output.
    string m_msg;        // @cmember Output message.
}
;

///////////////////////////////////////////////////////////////////////////////
// @class   CECDiagnostic | This class implements a client-side wrapper for the
//          Cyrano CORBA logging objects.  The class has no inherent
//          functionality of its own, and simply represents a mechanism to hide
//          CORBA.  It is intended for use as an on-line viewer.  Offline
//          viewing of log information is handled by the class
//          <c CECDiagnosticFile>.
class CECDiagnostic
{
    // @access Public Members
public:
    CECDiagnostic( CORBAServer* pServer, const char* name );

    CECTrace* GetFirstTrace();  // @cmember Gets first trace object
    CECTrace* GetNextTrace();   // @cmember Gets next trace object

    // @access Private Members
private:
    CORBAServer* m_server;  // @cmember Pointer to CORBA helper functions
    CyrLog_var   m_log;     // @cmember Reference to CORBA object.
}
;

///////////////////////////////////////////////////////////////////////////////
// @class   CECDiagnosticFile | This class allows offline reading of diagnostic
//          information from a file.
class CECDiagnosticFile
{
    // @access Public Members
public:
    CECDiagnosticFile( CECOrbFile* pFile );
    CECDiagnosticFile( const char* filename );
    virtual ~CECDiagnosticFile();

    CECTrace* GetFirstTrace();
    CECTrace* GetNextTrace();

    // @access Private Members
private:
    fstream* m_file;
    string m_filename;
    bool m_repofile;
};

#endif
