/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

///////////////////////////////////////////////////////////////////////////////
// @doc CyrStRepository
///////////////////////////////////////////////////////////////////////////////

#ifndef _CyrStByteIt_h
#define _CyrStByteIt_h

///////////////////////////////////////////////////////////////////////////////
// Cyrano includes
#include <Repository/CyrStRepository.h>

///////////////////////////////////////////////////////////////////////////////
#include <deque>
using std::deque;
///////////////////////////////////////////////////////////////////////////////
// Constants
const long BYTESEQ_SIZE = 1024;

typedef deque<CORBA::Octet> ByteArray;

///////////////////////////////////////////////////////////////////////////////
// @class   CyrStByteItImpl | This class represents the implementation of a
//          CyrStByteIt interface.  This implementation stores the bytes
//          internally before transmission.
// @base    public | _sk_CyrStByteIt
class CyrStByteItImpl : public virtual _sk_CyrStByteIt
{
    // @access Public Members
public:
    CyrStByteItImpl()
    {}

    ;           // @cmember Constructor
    virtual ~CyrStByteItImpl()
    {}

    ;  // @cmember Destructor

    // @cmember Retrieves the next sequence of bytes from the File
    ByteSeq* ReadBytes();
    void WriteBytes( const ByteSeq& bytes );
    long GetSize();  // @cmember Returns total size of File

    long ReadLong();
    void WriteLong( long data );
    short ReadShort();
    void WriteShort( short data );
    CORBA::Boolean ReadBool();
    void WriteBool( CORBA::Boolean data );
    CORBA::Octet ReadByte();
    void WriteByte( CORBA::Octet data );

    // @access Protected Members
protected:
    ByteArray m_byteArray;

    void ReadData( CORBA::Octet* bytes, size_t size );
    void WriteData( CORBA::Octet* bytes, size_t size );
};

#endif
