/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

///////////////////////////////////////////////////////////////////////////////
// @doc CyrStRepository
///////////////////////////////////////////////////////////////////////////////

#ifndef _CyrStFile_i_h
#define _CyrStFile_i_h

///////////////////////////////////////////////////////////////////////////////
// Cyrano includes
#include <Repository/CyrStLink_i.h>
#include <Repository/CyrStFileIt_i.h>

///////////////////////////////////////////////////////////////////////////////
// @class   CyrStFileImpl | This class represents the implementation of the
//          CORBA interface CyrStFile.  This interface allows access to all
//          methods and properties available via the COM IFileEx interface.
// @base    public | _sk_CyrStFile
class CyrStFileImpl : public virtual _sk_CyrStFile, public CyrStLinkImpl
{
    // @access Public Members
public:
    // @cmember Constructor
    CyrStFileImpl( CyrStProvider* pProvider, ParamList_ptr params );
    virtual ~CyrStFileImpl();   // @cmember Destructor

    char* GetName();            // @cmember Returns name of File
    long  GetCurrentStatus()
    {
        return 0;
    }
    ;   // @cmember Returns current status of File

    bool Lock();     // @cmember Locks file for specified user only.
    void Unlock();   // @cmember Releases lock on file.

    CyrStByteIt_ptr GetFile( bool lock );   // @cmember Initializes File iterator
    void PutFile( CyrStByteIt_ptr itFile, bool lock ); // @cmember Transmits File iterator

    // @access Protected Members
protected:
    // @cmember Retrieves an existing File
    bool RetrieveFile( CECFFFolder* iFolder, const char* name,
                       const char* user );
    // @cmember Transfers a file to a temporrary location
    string TransferFile( CyrStByteIt_ptr itFile, const char* name );

    CECFFFile* m_com;          // @cmember Pointer to COM wrapper
    CyrStFileItImpl* m_itFile;  // @cmember Pointer to member iterator
}
;

#endif
