/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

///////////////////////////////////////////////////////////////////////////////
// @doc CyrStRepository
///////////////////////////////////////////////////////////////////////////////

#ifndef _CyrStLink_i_h
#define _CyrStLink_i_h

///////////////////////////////////////////////////////////////////////////////
// Cyrano includes
#include <Repository/CyrStRepository.h>
#include <Repository/CyrStProv.h>

///////////////////////////////////////////////////////////////////////////////
// @class   CyrStLinkImpl | This class represents the implementation of the
//          CyrStLink interface.  This class is intended for use as a base
//          class for items that can be linked.  As these objects will be
//          CORBA objects, this will mean multiple inheritance.
// @base    public | _sk_CyrStLink
class CyrStLinkImpl : public virtual _sk_CyrStLink
{
    // @access Public Members
public:
    CyrStLinkImpl( CyrStProvider* pProvider );  // @cmember Constructor.
    virtual ~CyrStLinkImpl();                   // @cmember Destructor.

    char* GetRegisteredName();  // @cmember Returns registered name of item

    // @access Link methods
    char* GetLinkPath();       // @cmember Gets Link path for this object
    char* GetFirstLinkPath();  // @cmember Gets path for first linked object
    char* GetNextLinkPath();   // @cmember Gets path for next linked object

    // @access Protected Members
protected:
    CyrStProvider* m_provider;      // @cmember Reference to parent Provider
    string         m_regName;       // @cmember Registered name of object
    long           m_linkIndex;     // @cmember Enumeration of COM links

    // @cmember Builds complete path to specified item
    string         BuildItemPath( CECFFLink* link );
};

#endif
