/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// Project.cpp: implementation of the CYRProject class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Project.h"
#include "server.h"
#include "CECRegistryTools.h"
#include "solutions.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CYRProject::CYRProject(const CString& strName, CYRServer* pServer)
{
    m_strName = strName;
    m_pServer = pServer;

    CECRegistryTools reg;
    CString strWorkingDirectory = reg.GetStringValue(HKEY_CURRENT_USER ,SOLUTIONS_SERVER_KEY ,"ProjectDir");
    if(strWorkingDirectory.Right(1) != "\\")
        strWorkingDirectory += "\\";

    m_strPath = strWorkingDirectory + strName + "\\";
}

CYRProject::~CYRProject()
{
    CString strFolder;
    LPVOID pFolder;
    POSITION pos = m_mapFolders.GetStartPosition();
    while(pos != NULL)
    {
        m_mapFolders.GetNextAssoc(pos,strFolder,pFolder);
        delete pFolder;
    }
    m_mapFolders.RemoveAll();
}

CString CYRProject::GetName()
{
    return m_strName;
}

BOOL CYRProject::Delete()
{
    //Delete the directory on disk
    CYRServer::DeleteDirectory(m_strPath);

    return true;
}

BOOL CYRProject::IsValid()
{
    //Check that project directory exists
    CFileStatus fs;
    BOOL bExists = CFile::GetStatus(m_strPath+".",fs);
    return bExists;

}

BOOL CYRProject::Refresh()
{
    TRACE("Done Refresh\n");
    return true;
}

