/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// IE5Information.cpp: implementation of the CIE5Information class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "IE5Information.h"

#include <CSerialization.h>
#include "concertlib.h"
#include "ConstantesLib.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////
//-------------------------------------------------------------------

CIE5Information::CIE5Information(CWnd* _pWnd)
        :   CBrowserInformation(_pWnd)
{}

//-------------------------------------------------------------------


CIE5Information::~CIE5Information()
{
}

//-------------------------------------------------------------------


BOOL CIE5Information::GetDefaultParameters( _TPreferencesProxyData *pData )
{
    pData->m_bByPassProxyLocal      = HasByPassProxyLocal() ;

    pData->m_strProxyAddress = GetProxyInfo( pData->m_lAddressPort, _T("http") ) ;
    pData->m_strProxySecure  = GetProxyInfo( pData->m_lSecurePort , _T("https") ) ;

    pData->m_strAddressesNoProxy = GetAddressesWithoutproxy();

    return true;

}

//--------------------------------------------------------------------

CString CIE5Information::GetAddressesWithoutproxy()
{
    return GetExtendedProfileString(HKEY_CURRENT_USER , _T("Software\\Microsoft\\Windows\\CurrentVersion\\Internet Settings"),
                                    _T("ProxyOverride") , _T("") ) ;
}

//--------------------------------------------------------------------

BOOL CIE5Information::HasByPassProxyLocal()
{
    CString strValue ;
    strValue = GetExtendedProfileString(HKEY_CURRENT_USER , _T("Software\\Microsoft\\Windows\\CurrentVersion\\Internet Settings"),
                                        _T("ProxyOverride") , _T("") ) ;

    if( strValue.Find( _T("<local>") ) >= 0 )
        return TRUE ;
    else
        return FALSE ;

}

//--------------------------------------------------------------------

CString CIE5Information::GetProxyInfo(long &lPort , const CString& _strType )
{
    CString strValue ;
    strValue = GetExtendedProfileString(HKEY_CURRENT_USER , _T("Software\\Microsoft\\Windows\\CurrentVersion\\Internet Settings"),
                                        _T("ProxyServer") , _T("") ) ;


    int nPos = strValue.Find( '=' ) ;
    if( nPos >= 0 ) //All the prxy types do not have the same addresses
    {
        nPos = strValue.Find( _strType ) ;
        if( nPos >= 0 )
        {
            strValue = strValue.Right( strValue.GetLength() - nPos - strlen( _strType ) - 1 ) ;
            nPos = strValue.Find( ';' ) ;
            if( nPos >= 0 )
                strValue = strValue.Left( nPos ) ;
        }
        else
        {
            return _T("");
        }
    }

    CString strAddress ;
    //---
    nPos = strValue.Find(':') ;
    CString strPort ;
    //----
    strPort = strValue.Right( strValue.GetLength() - nPos - 1 ) ;
    lPort = atol( strPort ) ;

    strAddress = strValue.Left( nPos ) ;
    return strAddress;

}

//--------------------------------------------------------------------

void CIE5Information::SaveDefaultRegistryInfo()
{
    TCHAR *pczTempDir[_MAX_PATH] ;
    CString strFileName ;
    CString strData ;
    //----
    ::GetTempPath( _MAX_PATH - 1 , (char *)pczTempDir ) ;
    strFileName = (char*)pczTempDir ;
    strFileName += STR_MODELLER_IE5 ;

    CStdioFile file( strFileName , CFile::modeCreate|CFile::modeWrite| CFile::typeText ) ;

    // 1- Start Page
    strData = GetExtendedProfileString( HKEY_CURRENT_USER , _T("Software\\Microsoft\\Internet Explorer\\Main"),
                                        _T("Start Page") , _T("") ) ;
    //2 -  Autoconfig
    strData += STR_REGISTRY_SEPARATOR ;
    strData += GetExtendedProfileString(HKEY_CURRENT_USER , _T("Software\\Microsoft\\Windows\\CurrentVersion\\Internet Settings"),
                                        _T("AutoConfigURL") , _T("") ) ;

    //3 -  Proxy server Secure + Address + ...
    strData += STR_REGISTRY_SEPARATOR ;
    strData += GetExtendedProfileString(HKEY_CURRENT_USER , _T("Software\\Microsoft\\Windows\\CurrentVersion\\Internet Settings"),
                                        _T("ProxyServer") , _T("") ) ;

    //4 - machines who do not use proxy : local + other machines
    strData += STR_REGISTRY_SEPARATOR ;
    strData += GetExtendedProfileString(HKEY_CURRENT_USER , _T("Software\\Microsoft\\Windows\\CurrentVersion\\Internet Settings"),
                                        _T("ProxyOverride") , _T("") ) ;

    //5 - Connect to internet via proxy
    strData += STR_REGISTRY_SEPARATOR ;
    CString strTemp ;
    DWORD dwValue = 0 ;
    //----
    dwValue = GetExtendedProfileDword(HKEY_CURRENT_USER , _T("Software\\Microsoft\\Windows\\CurrentVersion\\Internet Settings"),
                                      _T("ProxyEnable") , 0 ) ;

    strTemp.Format( "%d" , dwValue ) ;
    strData += strTemp ;
    strData += STR_REGISTRY_SEPARATOR ;

    file.WriteString( strData ) ;

    file.Close() ;
}

//--------------------------------------------------------------------

void CIE5Information::ModifyRegistryForCapture( long lPort )
{
    // 1- Start Page
    WriteExtendedProfileString( HKEY_CURRENT_USER , _T("Software\\Microsoft\\Internet Explorer\\Main"),
                                _T("Start Page") , _T("") ) ;
    // 2- AutoConfig
    WriteExtendedProfileString( HKEY_CURRENT_USER , _T("Software\\Microsoft\\Windows\\CurrentVersion\\Internet Settings"),
                                _T("AutoConfigURL") , _T("") ) ;
    //3 - Connect to internet via proxy
    WriteExtendedProfileDword(HKEY_CURRENT_USER , _T("Software\\Microsoft\\Windows\\CurrentVersion\\Internet Settings"),
                              _T("ProxyEnable") , 1 ) ;
    //4 -  Proxy server Secure + Address + ...
    CString strData = GetExtendedProfileString(HKEY_CURRENT_USER , _T("Software\\Microsoft\\Windows\\CurrentVersion\\Internet Settings"),
                      _T("ProxyServer") , _T("") ) ;
    ModifyProxyForCapture( strData , lPort ) ;
    WriteExtendedProfileString(HKEY_CURRENT_USER , _T("Software\\Microsoft\\Windows\\CurrentVersion\\Internet Settings"),
                               _T("ProxyServer") , strData ) ;

    //5 - machines who do not use proxy : local + other machines
    WriteExtendedProfileString(HKEY_CURRENT_USER , _T("Software\\Microsoft\\Windows\\CurrentVersion\\Internet Settings"),
                               _T("ProxyOverride") , _T("") ) ;


}

//--------------------------------------------------------------------

void CIE5Information::SetDefaultParametersInRegistry()
{
    TCHAR *pczTempDir[_MAX_PATH] ;
    CString strFileName ;
    CString strData ;
    CString strTemp ;
    //----
    ::GetTempPath( _MAX_PATH - 1 , (char *)pczTempDir ) ;
    strFileName = (char*)pczTempDir ;
    strFileName += STR_MODELLER_IE5 ;

    CStdioFile file ;

    if( file.Open(strFileName , CFile::modeRead| CFile::typeText )  )
    {
        file.ReadString( strData ) ;
        int nPos = strData.Find(STR_REGISTRY_SEPARATOR);

        if( nPos < 0 )
            return ;

        // 1- Start Page
        strTemp = strData.Left( nPos ) ;
        WriteExtendedProfileString( HKEY_CURRENT_USER , _T("Software\\Microsoft\\Internet Explorer\\Main"),
                                    _T("Start Page") , strTemp ) ;

        strData = strData.Right( strData.GetLength() - nPos - 1 ) ;

        //2 -  AutoConfig
        nPos = strData.Find(STR_REGISTRY_SEPARATOR);
        if( nPos < 0 )
            return ;
        strTemp = strData.Left( nPos ) ;
        WriteExtendedProfileString(HKEY_CURRENT_USER , _T("Software\\Microsoft\\Windows\\CurrentVersion\\Internet Settings"),
                                   _T("AutoConfigURL") , strTemp ) ;
        strData = strData.Right( strData.GetLength() - nPos - 1 ) ;

        //3 -  Proxy server Secure + Address + ...
        nPos = strData.Find(STR_REGISTRY_SEPARATOR);
        if( nPos < 0 )
            return ;
        strTemp = strData.Left( nPos ) ;
        WriteExtendedProfileString(HKEY_CURRENT_USER , _T("Software\\Microsoft\\Windows\\CurrentVersion\\Internet Settings"),
                                   _T("ProxyServer") , strTemp ) ;
        strData = strData.Right( strData.GetLength() - nPos - 1 ) ;

        //4 - machines who do not use proxy : local + other machines
        nPos = strData.Find(STR_REGISTRY_SEPARATOR);
        if( nPos < 0 )
            return ;
        strTemp = strData.Left( nPos ) ;
        WriteExtendedProfileString(HKEY_CURRENT_USER , _T("Software\\Microsoft\\Windows\\CurrentVersion\\Internet Settings"),
                                   _T("ProxyOverride") , strTemp ) ;
        strData = strData.Right( strData.GetLength() - nPos - 1 ) ;

        //5 - Connect to internet via proxy
        nPos = strData.Find(STR_REGISTRY_SEPARATOR);
        if( nPos < 0 )
            return ;
        strTemp = strData.Left( nPos ) ;
        DWORD dwValue = atol( strTemp ) ;

        WriteExtendedProfileDword(HKEY_CURRENT_USER , _T("Software\\Microsoft\\Windows\\CurrentVersion\\Internet Settings"),
                                  _T("ProxyEnable") , dwValue ) ;
        file.Close() ;
        CFile::Remove( strFileName ) ;
    }
}

//--------------------------------------------------------------------

BOOL CIE5Information::GetBrowserFullPath( CString &_strPath )
{
    CString strLongPath ;
    CString strTitle ;
    CString strMsg ;
    TCHAR pczShortPath[_MAX_PATH] ;
    //---

    //1 - Test the version
    if( !IsIE5Version() )
    {
        strTitle.LoadString( IDS_WARNING ) ;
        strMsg.LoadString( IDS_BROWSER_NOT_FOUND ) ;
        m_pWnd->MessageBox(strMsg, strTitle, MB_OK ) ;
    }

    strLongPath = GetExtendedProfileString( HKEY_LOCAL_MACHINE , _T("SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\App Paths\\IEXPLORE.EXE"),
                                            _T("") , _T("") ) ;

    //2 - Test the path
    if( strLongPath.IsEmpty() )
    {
        return FALSE ;
    }

    GetShortPathName( (LPCSTR)strLongPath, pczShortPath, _MAX_PATH );
    _strPath = pczShortPath ;

    return TRUE ;
}

//--------------------------------------------------------------------

BOOL CIE5Information::IsInternetSettingOK()
{
    CString strAutomaticProxy ;
    strAutomaticProxy = GetExtendedProfileString(HKEY_CURRENT_USER , _T("Software\\Microsoft\\Windows\\CurrentVersion\\Internet Settings"),
                        _T("AutoConfigURL") , _T("") ) ;

    if( !strAutomaticProxy.IsEmpty() )
    {
        CString strWarning ;
        CString strMessage ;
        //---
        strWarning.LoadString( IDS_WARNING ) ;
        strMessage.LoadString( IDS_NOT_USE_AUTO_PROXY ) ;
        AfxGetMainWnd()->MessageBox( strMessage , strWarning , MB_OK ) ;
        return FALSE ;
    }

    return TRUE ;
}

//--------------------------------------------------------------------

BOOL CIE5Information::IsIE5Version()
{
    CString strVersion ;
    float   fVersionNumber;

    strVersion  = GetExtendedProfileString( HKEY_LOCAL_MACHINE , _T("SOFTWARE\\Microsoft\\Internet Explorer"),
                                            _T("Version") , _T("") ) ;
    fVersionNumber = atof( strVersion.Left(4) ) ;

    if( (fVersionNumber < 5.0) || (fVersionNumber >= 6.0) )
        return FALSE ;

    return TRUE ;
}

