/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/


#include "stdafx.h"
#include "datastore.h"

#define DEFAULT_GROW_STRINGBUFFER 512

//////////////////////////////////////////////////////////////////////

LPCTSTR StringSepTokenize( LPCTSTR & _pczCursor , const TCHAR cSep )
{
    LPCTSTR pczToken  = NULL ;
    LPCTSTR pczCursor = _pczCursor ;
    // ----
    if ( pczCursor != NULL )
    {
        TCHAR cc ;
        // ----
        pczToken = pczCursor ;

        for ( ; ( cc = *pczCursor ) != cSep && cc != (TCHAR)0 ; pczCursor++ )
        {
            ;
        }

        if ( cc == (TCHAR)0 )
        {
            _pczCursor = NULL ; // This is the last token.
        }

        else
        {
            _pczCursor = pczCursor + 1 ;
        }
    }

    return pczToken ;
}

//////////////////////////////////////////////////////////////////////  _T
//////////////////////////////////////////////////////////////////////
CStringStore::CStringStore( int nGrowBy /*=-1*/ ) :
        m_pBuffer(NULL) ,
        m_pCurrent(NULL) ,
        m_nGrowBy( nGrowBy > 0 ? nGrowBy : DEFAULT_GROW_STRINGBUFFER )
{}

//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////

CStringStore::~CStringStore()
{
#if 0
    Purge() ;
#else
    TBuffer * pBuffer = m_pBuffer ;
    // ----
    while ( pBuffer != NULL ) // Delete the linked list of buffers :
    {
        TBuffer * pCurrent = pBuffer ;
        // ----
        pBuffer = pCurrent->m_pNextBuffer ;

        delete pCurrent ;
    }
#endif
}

//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
void CStringStore::Purge()
{
    TBuffer * pBuffer = m_pBuffer ;
    // ----
    while ( pBuffer != NULL ) // Delete the linked list of buffers :
    {
        TBuffer * pCurrent = pBuffer ;
        // ----
        pBuffer = pCurrent->m_pNextBuffer ;

        delete pCurrent ;
    }

    m_pBuffer  = NULL ;
    m_pCurrent = NULL ;
}

//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
CStringStore::TBuffer::TBuffer( int nGrowBy ) :
m_pzBuffer(NULL) ,
m_pNextBuffer(NULL) ,
m_nSize(0) ,
m_nRemains(nGrowBy)
{
    m_pzBuffer = new TCHAR [ nGrowBy ] ;
}

//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
CStringStore::TBuffer::~TBuffer()
{
    delete [] m_pzBuffer ;
}

//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
CStringStore::TBuffer * CStringStore::NewBuffer( int _nGrowBy /*=-1*/ )
{
    TBuffer * pNewBuffer = NULL ;
    int       nGrowBy    = _nGrowBy > 0 ? _nGrowBy : m_nGrowBy ;
    // ----
    TRY
    {
        ASSERT( nGrowBy > 0 ) ;

        pNewBuffer = new TBuffer( nGrowBy ) ;

        if ( m_pCurrent == NULL )
    {
        m_pBuffer = m_pCurrent = pNewBuffer ;
    }
    else
    {
        m_pCurrent->m_pNextBuffer = pNewBuffer ;
        m_pCurrent                    = pNewBuffer ;
    }

    m_pCurrent->m_nSize = m_pCurrent->m_nRemains = nGrowBy ;
}
CATCH(CMemoryException,e)
    {
        pNewBuffer = NULL ;
    }
    END_CATCH

    return pNewBuffer ;
}

//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
#if 1

//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
static UINT _StrLen( const TCHAR * pczString , TCHAR cSep )
{
    ASSERT( pczString != NULL ) ;
    UINT nLen = 0 ;
    // ----
    if ( cSep != (TCHAR)0 )
    {
        for ( TCHAR cc ; ( cc = *( pczString + nLen ) ) != cSep && cc != (TCHAR)0 ; nLen++ )
        {
            ;
        }
    }
    else
    {
        for ( ; *( pczString + nLen ) != 0 ; nLen++ )
        {
            ;
        }
    }

    return nLen ;
}

//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
const TCHAR * CStringStore::Store( const TCHAR * pczString , TCHAR cSep /*=(TCHAR)0*/ )
{
    TCHAR   * pczStored  = NULL ;
    TBuffer * pNewBuffer = NULL ;
    UINT      nLen       = ( pczString != NULL ? _StrLen( pczString , cSep ) : 0 ) ;

    if ( nLen > 0 )
    {
        UINT nAllocSize = 0 ;

        if ( m_pCurrent == NULL ) // No buffer yet
        {
            if ( CStringStore::NewBuffer( (nLen+1) > (UINT)m_nGrowBy ? (nLen+1) : (UINT)m_nGrowBy ) != NULL )
                pczStored = m_pCurrent->m_pzBuffer ;
        }
        else
        {
            if ( (nLen+1) > (UINT)m_pCurrent->m_nRemains ) // Buffer to small
            {
                if ( CStringStore::NewBuffer( (nLen+1) > (UINT)m_nGrowBy ? (nLen+1) : (UINT)m_nGrowBy ) != NULL )
                    pczStored = m_pCurrent->m_pzBuffer ;
            }
            else
            {
                pczStored = ( m_pCurrent->m_pzBuffer + ( m_pCurrent->m_nSize - m_pCurrent->m_nRemains ) ) ;
            }
        }

        if ( pczStored != NULL )
        {
            for ( UINT i = 0 ; i < nLen ; i++ )
            {
                *( pczStored + i ) = *( pczString + i ) ;
            }
            *( pczStored + nLen ) = (TCHAR)0 ;

            m_pCurrent->m_nRemains -= (nLen+1) ;
        }
    }
    else
    {
        pczStored = "" ;
    }

    return pczStored ;
}

#else

//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
const TCHAR * CStringStore::Store( const TCHAR * pczString )
{
    TCHAR   * pczStored = NULL ;
    TBuffer * pCurrent  = m_pCurrent ; // Save current buffer on function entry.
    // ----
    if ( m_pCurrent != NULL || CStringStore::NewBuffer() != NULL )
    {
        ASSERT( m_pCurrent != NULL && m_pCurrent->m_nRemains > 0 ) ;

        int nRemains       = m_pCurrent->m_nRemains ;
        int nTryAllocTimes = 0 ;
        // ----
        pczStored = ( m_pCurrent->m_pzBuffer + ( /* m_nGrowBy */ m_pCurrent->m_nSize - nRemains ) ) ;

        while ( TRUE )
        {
            int i = 0 ;
            // ----
            for ( ; i < nRemains - 1 ; i++ ) // Try and copy...
            {
                if ( ( *( pczStored + i ) = *( pczString + i ) ) == (TCHAR)0 )
                {
                    break ;
                }
            }

            if ( *( pczStored + i ) ==  (TCHAR)0 ) // Successful copy
            {
                m_pCurrent->m_nRemains -= ( i + 1 ) ;
                break ;
            }
            else // Buffer wasn't big enough
            {
                if ( m_pCurrent != pCurrent )
                {
                    delete m_pCurrent ;
                    // Pretend no alloc has taken place :
                    if ( ( m_pCurrent = pCurrent ) != NULL )
                    {
                        m_pCurrent->m_pNextBuffer = NULL ;
                    }
                }

                if ( CStringStore::NewBuffer( ++nTryAllocTimes * m_nGrowBy ) != NULL )
                {
                    pczStored = m_pCurrent->m_pzBuffer ;
                    nRemains  = m_pCurrent->m_nRemains ;
                }
                else // Alloc Failed !
                {
                    ASSERT(FALSE) ;
                    pczStored  = NULL ;
                    m_pCurrent = pCurrent ; // Whatever might have happened in between.
                    m_pCurrent->m_pNextBuffer = NULL ;
                    break ; // Give up !
                }

            }
        }
    }

    return pczStored ;
}

#endif

//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////

const TCHAR * CStringStore::AddString( const TCHAR * pczString , TCHAR cSep /*=(TCHAR)0*/ )
{
    if ( pczString != NULL )
    {
        return Store( pczString , cSep ) ;
    }

    return NULL ;
}

//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////

BOOL CStringStore::AddStringAssign( const TCHAR * & pczAssign , const TCHAR * pczString , TCHAR cSep /*=(TCHAR)0*/ )
{
    return ( ( pczAssign = AddString( pczString , cSep ) ) != NULL ) ;
}

//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
