/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// ExtendedDataTreeCtrl.cpp: implementation of the CExtendedDataTreeCtrl class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "ExtendedDataTreeCtrl.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

//The purpose of this class is to manage tree data after its construction.
// That is to say : the map list is not destructed after the tree building.


IMPLEMENT_DYNAMIC( CExtendedDataTreeCtrl, CExtendedTreeCtrl )

//-------------------------------------------------------------------

CExtendedDataTreeCtrl::CExtendedDataTreeCtrl() :
        CExtendedTreeCtrl( FALSE )  //do not delete data inseted in the tree.
{
}

//-------------------------------------------------------------------


CExtendedDataTreeCtrl::~CExtendedDataTreeCtrl()
{
}

//-------------------------------------------------------------------
// Delete all the items even in the map list of items

BOOL CExtendedDataTreeCtrl::DeleteAllItems()
{
    HTREEITEM hRoot = GetRootItem() ;
    BOOL bRet = TRUE ;
    //---
    while( hRoot )
    {
        DeleteMapItem( hRoot ) ;
        HTREEITEM hOldRoot = hRoot ;
        hRoot = GetNextItem( hOldRoot , TVGN_NEXT ) ;
        bRet = bRet && CTreeCtrl::DeleteItem( hOldRoot ) ;
    }
    return bRet ;
}

//-------------------------------------------------------------------
// Delete the item hItem in the tree
BOOL CExtendedDataTreeCtrl::DeleteItem( HTREEITEM hItem )
{
    if( DeleteMapItem( hItem ) )
        return CTreeCtrl::DeleteItem( hItem ) ;
    else
        return FALSE ;
}

//-------------------------------------------------------------------
//private
BOOL CExtendedDataTreeCtrl::DeleteMapItem( HTREEITEM hParent )
{
    HTREEITEM hInserted = GetNextItem( hParent , TVGN_CHILD ) ;
    while( hInserted != (HTREEITEM) NULL )
    {
        DeleteMapItem( hInserted ) ;
        hInserted = GetNextItem( hInserted , TVGN_NEXT ) ;
    }
    long lItemID = GetItemData( hParent ) ;
    m_pInsertInfo->m_MapIDToItem.RemoveKey((DWORD&)lItemID ) ;
    return TRUE ;
}

//-------------------------------------------------------------------
//Modify the parent of the item hItem (probably by using drag'n drop).
BOOL CExtendedDataTreeCtrl::ModifyItem( HTREEITEM hItem , long lParent )
{
    m_ModifyList.RemoveAll() ;
    if( ModifyMapItem( hItem , lParent ) )
    {
        DeleteMapItem( hItem ) ;
        BOOL bRet = InsertOldItem()  ;
        return bRet ;
    }
    else
        return FALSE ;
}

//-------------------------------------------------------------------
//private
BOOL CExtendedDataTreeCtrl::InsertOldItem()
{
    POSITION nPos = m_ModifyList.GetHeadPosition() ;
    while( nPos )
    {
        _TreeData Data = m_ModifyList.GetAt( nPos ) ;
        HTREEITEM hItem ;
        //---
        InsertItem(hItem ,
                   Data.m_strText,
                   Data.m_nImage ,
                   Data.m_nSelectedImage ,
                   Data.m_lElementID ,
                   Data.m_lParentElementID ) ;


        m_ModifyList.GetNext( nPos ) ;
    }
    m_ModifyList.RemoveAll() ;
    return TRUE ;
}

//-------------------------------------------------------------------

BOOL CExtendedDataTreeCtrl::ModifyMapItem( HTREEITEM hParent , long lOldParent)
{
    HTREEITEM hInserted = GetNextItem( hParent , TVGN_CHILD ) ;
    while( hInserted != (HTREEITEM) NULL )
    {
        long lParent    = GetItemData( hParent ) ;
        ModifyMapItem( hInserted , lParent ) ;
        hInserted       = GetNextItem( hInserted , TVGN_NEXT ) ;
    }

    _TreeData Data ;
    //---
    int         nImage ;
    int         nSelectedImage ;
    CString     strText ;
    //---

    GetItemImage( hParent , nImage, nSelectedImage ) ;
    strText         = GetItemText( hParent ) ;
    //---
    Data.m_lElementID       = GetItemData( hParent ) ;
    Data.m_lParentElementID = lOldParent ;
    Data.m_nImage           = nImage ;
    Data.m_nSelectedImage   = nSelectedImage ;
    Data.m_strText          = strText ;

    m_ModifyList.AddTail( Data ) ;

    return TRUE ;
}
