#ifndef _IDYNMENU32_H_
#define _IDYNMENU32_H_
/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/


// --------------------------------------------------------------------------
// Interfaces for crating and managing dynamic popup menus of up to 32 items
// (more than 32 seems unnecessary, 32 is already illegible).

interface IDynMenu32Client
{
    // You may want the menu commands to be sent to a different window
    // than the one that invokes the menu (e.g : a CView-derived window
    // may want the command to reach its parent frame). Otherwise just
    // make both methods return 'this' :
    virtual HRESULT GetClientWindow( /*[out]*/ CWnd * * ppWnd ) = 0 ;
    virtual HRESULT GetCmdMsgTargetWindow( /*[out]*/ CWnd * * ppTargetWnd ) = 0 ;

    // Called by the 'IDynMenu32' right before openig the menu.
    // Gives the client a chance of modifying the menu.
    virtual HRESULT OnPreMenuPopup( /*[in]*/ CMenu * pMenu ) = 0 ;
} ;

// --------------------------------------------------------------------------

interface IDynMenu32
{
    // To be visible, items must be present in the 'dwMask' mask :
    // 32 bits <==> 32 items in sequential order. Separators are not taken into
    // account (but they will be handled all the same.
    virtual HRESULT Create( /*[in]*/ IDynMenu32Client * pClient ,
                                     /*[in]*/ UINT               nFullMenuID ,
                                     /*[in]*/ int                nSubMenuIndex ,
                                     /*[in]*/ DWORD              dwMask ) = 0 ;

    virtual HRESULT DoPopup( /*[in]*/ const CPoint * pScreenPoint ) = 0 ;
} ;

// --------------------------------------------------------------------------

class CDynMenu32DefaultImpl : public IDynMenu32
{
    friend class CCyrClassFactory ;

    CMenu              m_menu ;
    IDynMenu32Client * m_pClient ;

protected:
    CDynMenu32DefaultImpl() ;
    ~CDynMenu32DefaultImpl() ;

public:

    // To build a mask corresponding to the lib ressource-contained menu
    // pick your choice from the values below :
    enum DefaultMaskItem
    {
        mi_sort         = 0x00000001 ,
        mi_filter       = 0x00000002 ,
        mi_search       = 0x00000004 ,
        //
        mi_print        = 0x00000008 ,
        mi_preview      = 0x00000010 ,
        mi_save_row_as  = 0x00000020 ,
        //
        mi_expand_all   = 0x00000040 ,
        mi_collapse_all = 0x00000080 ,
        mi_expand       = 0x00000100 ,
        mi_collapse     = 0x00000200 ,
        //
        mi_edit         = 0x00000400 ,
        mi_rename       = 0x00000800 ,
        mi_insert       = 0x00001000 ,
        mi_remove       = 0x00002000 ,
        //
        mi_data_group   = mi_sort|mi_filter|mi_search ,
        mi_print_group  = mi_print|mi_preview|mi_save_row_as ,
        mi_expand_group = mi_expand_all|mi_collapse_all|mi_expand|mi_collapse ,
        mi_edit_group   = mi_edit|mi_rename|mi_insert|mi_remove ,
    } ;

public:
    static  HRESULT CreateObject(  CDynMenu32DefaultImpl * * ppDynMenu32 ) ;
    static  HRESULT DestroyObject( CDynMenu32DefaultImpl * * ppDynMenu32 ) ;

    virtual HRESULT Create( IDynMenu32Client * pClient ,
                            UINT               nFullMenuID ,
                            int                nSubMenuIndex ,
                            DWORD              dwMask ) ;

    virtual HRESULT DoPopup( const CPoint * pScreenPoint ) ;
} ;

// --------------------------------------------------------------------------
// Global helpers :

// Generic :
HRESULT CyrLibDoPopupMenu( IDynMenu32Client * pClient ,
                           UINT               nFullMenuID ,
                           int                nSubMenuIndex ,
                           const CPoint     * pScreenPoint ,
                           DWORD              dwMask ) ;

// Use the ressource contained menu corresponding to
// CDynMenu32DefaultImpl::DefaultMaskItem :
HRESULT CyrLibDoDefaultPopupMenu( IDynMenu32Client * pClient ,
                                  const CPoint     * pScreenPoint ,
                                  DWORD              dwMask = CDynMenu32DefaultImpl::mi_data_group ) ;

// --------------------------------------------------------------------------
// Non-Frame windows that want to process ON_UPDATE_COMMAND_UI have to :
// 1) declare :
// afx_msg void OnInitMenuPopup( CMenu* pPopupMenu, UINT nIndex, BOOL bSysMenu );
// 2) Add ON_WM_INITMENUPOPUP() to their message map.
// 3) From their implementation of OnInitMenuPopup(), call the following :

void CyrLibOnInitMenuPopup( CWnd * pThis , CMenu * pMenu, BOOL bSysMenu ) ;

// --------------------------------------------------------------------------

#endif // _IDYNMENU32_H_
