/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// DOMStructure.h: interface for the CDOMStructure class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_DOMSTRUCTURE_H__ACB94019_FFCC_11D3_AC85_00105A483E3C__INCLUDED_)
#define AFX_DOMSTRUCTURE_H__ACB94019_FFCC_11D3_AC85_00105A483E3C__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include <stdio.h>
#include <string.h>

//-------------------------------------------------------------------------------

//Type of the node : TAG or ATTRIBUTE or TEXT
enum HTMLType
{
    NodeRoot        = 0,          //The first node in the architecture
    NodeTag         = 1,          //The node is a tag
    NodeAttribute   = 2,          //The node is an attribute
    NodeText        = 3,          //The node is a text
};

//-------------------------------------------------------------------------------

//Node
class CNode
{
public :
    CNode*          m_pParent ;        //node parent
    HTMLType        m_nType ;           //element Type : tag, attributes, text
    char*           m_pczAddress ;     //The address of the element

    CNode();
    ~CNode();
    void SetAddress( char* pczAddress );

};

//-------------------------------------------------------------------------------

//Element Text
class CText : public CNode
{
public :
    char*          m_pczText ;

    CText();
    ~CText();
    void SetText( char* pczText );

};

//-------------------------------------------------------------------------------

//Element Attribute
class CAttribute : public CNode
{
public :
    char*          m_pczName ;
    char*          m_pczValue ;

    CAttribute();
    ~CAttribute();
    void SetName( char* pczName );
    void SetValue( char* pczValue );
};


//-------------------------------------------------------------------------------

class CAttributeList
{
public :
    CAttribute*     m_pAttribute ;
    CAttributeList* m_pNextAttributeList ;

    CAttributeList();
    ~CAttributeList();
};

//-------------------------------------------------------------------------------

class CTextList
{
public :
    CText*          m_pText ;
    CTextList*      m_pNextTextList ;

    CTextList();
    ~CTextList();
};

//-------------------------------------------------------------------------------

//Element Tag
class CTagList;

class CTag : public CNode
{
public :
    char*           m_pczName ;
    CTagList*       m_pTagList ;

    CAttributeList* m_pAttributesList ;
    CTextList*      m_pTextList ;
    int             m_nLastChildIndex ;


    CTag(HTMLType eType);
    CTag(HTMLType eType , char* pczName);
    ~CTag();

    Init(HTMLType eType);
    void SetName( char* pczName );

};

//-------------------------------------------------------------------------------

class CTagList
{
public :
    CTag*           m_pTag ;
    CTagList*       m_pNextTagList ;

    CTagList()  ;
    ~CTagList() ;

};



//-------------------------------------------------------------------------------

class CDOMStructure
{
    CTag          *m_pCurrentParent ;

public:
    CDOMStructure();
    virtual ~CDOMStructure();

    bool InsertTagChild( char* pczName ) ;
    bool InsertAttributeChild( char* pczName , char* pczValue ) ;
    bool InsertTextChild( char* pczValue ) ;
    bool EndTag( char* pczName ) ;

    CTag*   GetRootItem() ;

    //----
    void Reset();
    void Reset( CTagList* pTagList );
    void Reset( CTextList* pTextList ) ;
    void Reset( CAttributeList* pAttributeList ) ;
    void Init();

    //Set Addresses for tag, attribute and tect
    void SetTagAddress( int nIndex , CTag* pTag );
    void SetAttributeAddress( int nIndex , CAttribute* pAttribute ) ;
    void SetTextAddress( int nIndex , CText* pText ) ;

    //Init Parent Item list (tag + attribute + text)
    void InitParentTagList( CTag* pTag ) ;
    void InitParentAttributeList( CAttribute* pAttribute ) ;
    void InitParentTextList( CText* pText );

    bool GetValue( const char* pczAddress , char*& pczValue );

protected :
    //look for the value corresponding to the address pczAddress in the text list
    bool GetTextValue( CTextList* pTextList , const char* pczAddress , char*& pczValue ) ;
    //look for the value corresponding to the address pczAddress in the Attribute list
    bool GetAttributeValue( CAttributeList* pAttributeList , const char* pczAddress , char*& pczValue ) ;
    //look for the value corresponding to the address pczAddress in the tag list
    bool GetTagValue( CTagList* pTagList , const char* pczAddress , char*& pczValue );
};

#endif // !defined(AFX_DOMSTRUCTURE_H__ACB94019_FFCC_11D3_AC85_00105A483E3C__INCLUDED_)
