/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

#include "stdafx.h"

#include "HTMLTree.hpp"
#include <iostream>

using namespace std;


CTag::~CTag()
{
    DeleteTag();
}

void CTag::Dump(ostream& os) const
{
    DumpTag(os, 0);
}

void CTag::DeleteTag()
{
    for(TagList::iterator it = sons_.begin(); it != sons_.end(); it++)
    {
        (*it)->DeleteTag();
        delete (*it);
    }
}

void CTag::DumpTag(ostream& os, int iLevel) const
{
    // Tag name.
    Margin(os, iLevel);
    os << "<" << strLabel_ << "> ";

    // Attributes.
    BOOL bFirst = TRUE;
    for(AttributeList::const_iterator it1 = attributes_.begin(); it1 != attributes_.end(); it1++)
    {
        if(bFirst)
            bFirst = FALSE;
        else
            os << " ; ";
        DumpAttribute(os, *it1);
    }
    os << endl;

    // Texts.
    for(NodeList::const_iterator it2 = texts_.begin(); it2 != texts_.end(); it2++)
    {
        Margin(os, iLevel + 2);
        os << "TEXT: ";
        DumpNode(os, *it2);
        os << endl;
    }

    // Scripts.
    for(NodeList::const_iterator it3 = scripts_.begin(); it3 != scripts_.end(); it3++)
    {
        Margin(os, iLevel + 2);
        os << "SCRIPT: ";
        DumpNode(os, *it3);
        os << endl;
    }

    // Comments.
    for(NodeList::const_iterator it4 = comments_.begin(); it4 != comments_.end(); it4++)
    {
        Margin(os, iLevel + 2);
        os << "COMMENT: ";
        DumpNode(os, *it4);
        os << endl;
    }

    // Sons.
    for(TagList::const_iterator it5 = sons_.begin(); it5 != sons_.end(); it5++)
        (*it5)->DumpTag(os, iLevel + 2);
}

inline
void CTag::Margin(ostream& os, int iLevel) const
{
    for(int i = 0; i< iLevel; i++)
        os << " ";
}

void CTag::DumpAttribute(ostream& os, const CAttribute& att) const
{
    if(!att.strLabel_.empty())
        os << att.strLabel_ << " = ";
    DumpNode(os, att);
}

void CTag::DumpNode(ostream& os, const CNode& node) const
{
    os << node.strText_ << " (@ = " << node.strAddress_ << ")";
}
