/* HTML Parser in Phoenix format */

{
}

digit       [0-9]
integer     {digit}+
letter      [a-zA-Z]
letter2     [a-zA-Z_$#\-\.:/%\+,\(\)\*]
id          ({letter2}|{digit})+
tag_open    \<{letter}({letter}|{digit})*
tag_close   \<\/{letter}({letter}|{digit})*
string      ([\"][^\"]*[\"])|([\'][^\']*[\'])
space       [ \t\r\n]+

%%

%I                      // Turn off case sensitivity

"<!"doctype             TK_DOCTYPE

%S                      // Turn on case sensitivity

"<!--"                  {collectComment(); return TK_COMMENT;}

">"                     {
                            bInTag_ = FALSE;

                            // If we are in a script tag, then advance to </script
                            if(bInScript_)
                                collectScript();
                            else
                                collectText();

                            return TK_TAGEXIT;
                        }

"="                     {
                            if(bInTag_)
                                return TK_EQUAL;
                            else
                            {
                                // Text.
                                if(pcText_ == NULL)
                                {
                                    // Set the beginning of the text.
                                    pcText_ = pcCurChar_;
                                }
                            }
                        }

{tag_open}              {
                            bInTag_ = TRUE;

                            // We are on the beginning of a tag.
                            // If a text has been seen so far, then insert it
                            // into the DOM.
                            if(pcText_ != NULL)
                            {
                                InsertTextIntoDom(pcText_, pcFirstChar_);
                                pcText_ = NULL;
                            }

                            // Check if we are in a script tag.
                            {
                                if(InScript())
                                    bInScript_ = TRUE;
                            }

                            return TK_START_TAG;
                        }

{tag_close}             {
                            bInTag_ = TRUE;
                            bInScript_ = FALSE;

                            // We are on the beginning of a tag closure.
                            // If a text has been seen so far, then insert it
                            // into the DOM.
                            if(pcText_ != NULL)
                            {
                                InsertTextIntoDom(pcText_, pcFirstChar_);
                                pcText_ = NULL;
                            }

                            return TK_END_TAG;
                        }

{id}                    {
                            if(bInTag_)
                                return TK_ID;
                            else
                            {
                                // Text.
                                if(pcText_ == NULL)
                                {
                                    // Set the beginning of the text.
                                    pcText_ = pcCurChar_;
                                }
                            }
                        }

{string}                {
                            if(bInTag_)
                                return TK_LITERAL;
                            else
                            {
                                // Text.
                                if(pcText_ == NULL)
                                {
                                    // Set the beginning of the text.
                                    pcText_ = pcCurChar_;
                                }
                            }
                        }

{space}                 {
                            // Text.
                            if(!bInTag_)
                            {
                                if(pcText_ == NULL)
                                {
                                    // Set the beginning of the text.
                                    pcText_ = pcCurChar_;
                                }
                            }
                        }

.|\n                    {
                            // Text.
                            if(!bInTag_)
                            {
                                collectText();
                            }
                            else
                            {
                                // By default, skip any unrecognized character.
                            }
                        }

%%
