/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// Modeller.cpp : Defines the class behaviors for the application.
//

#include "stdafx.h"
#include <cmaxafx.h>
#include <ConstantesLib.h>
#include <RecoverManager.h>
#include <wininet.h>

#include "Constantes.h"
#include "MainFrm.h"
#include "ChildFrm.h"
#include "ModellerDoc.h"
#include "SubroutineDocument.h"
#include "SubroutineChildFrame.h"
#include "SubroutineView.h"
#include "BasedDocument.h"
#include "EnumModellerData.h"
#include "ModellerView.h"
#include "Modeller.h"
#include "AboutDlg.h"
#include "ProjectOpenDlg.h"
#include "CaptureOpenDlg.h"
#include "SelectBrowserDlg.h"
#include "OpenDocumentDlg.h"
#include "GatewayOptionsDlg.h"
#include "VariableOptionDlg.h"
#include <IE4Information.h>
#include <IE5Information.h>
#include <NS47Information.h>

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

//--------------------------------------------------------------------------

CModellerApp    theApp;

//--------------------------------------------------------------------------

BEGIN_MESSAGE_MAP(CModellerApp, CWinApp)
//{{AFX_MSG_MAP(CModellerApp)
ON_COMMAND(ID_APP_ABOUT, OnAppAbout)
ON_COMMAND(ID_OPEN_PROJECT, OnOpenProject)
ON_UPDATE_COMMAND_UI(ID_FILE_NEW, OnUpdateFileNew)
ON_UPDATE_COMMAND_UI(ID_FILE_OPEN, OnUpdateFileOpen)
ON_COMMAND(ID_OPTIONS_BROWSER, OnOptionsBrowser)
ON_COMMAND(ID_OPTIONS_GATEWAY, OnOptionsGateway)
ON_COMMAND(ID_OPTIONS_VARIABLES, OnOptionsVariables)
ON_COMMAND(ID_TOOLS_EDIT, OnToolsEdit)
ON_UPDATE_COMMAND_UI(ID_TOOLS_EDIT, OnUpdateToolsEdit)
//}}AFX_MSG_MAP
// Standard file based document commands
ON_COMMAND(ID_FILE_NEW, OnFileNew)
ON_COMMAND(ID_FILE_OPEN, OnFileOpen)
// Standard print setup command
ON_COMMAND(ID_FILE_PRINT_SETUP, CWinApp::OnFilePrintSetup)
END_MESSAGE_MAP()

//--------------------------------------------------------------------------

CModellerApp::CModellerApp() :
        m_bRecording( FALSE ),
        m_bReplay( FALSE ),
        m_bRegisterClass( FALSE) ,
        m_bOffLine( FALSE ),
        m_bTrace(FALSE),
        m_bConsole(FALSE),
		m_bAutoCookieGeneration(TRUE)
{
    memset(&m_ProcessInformation, 0, sizeof(m_ProcessInformation));
}

CModellerApp::~CModellerApp()
{
    m_CaptureGateway.stopGateway() ;
    TerminateProcess(m_ProcessInformation.hProcess, 0xFFFF);
}

//--------------------------------------------------------------------------

CWnd *CModellerApp::FirstInstance()
{
    CWnd *pWndPrev, *pWndChild;
    //----

    // Determine if another window with our class name exists...
    if (pWndPrev = CWnd::FindWindow( m_strClassName, NULL ))
    {
        // if so, does it have any popups?
        pWndChild = pWndPrev->GetLastActivePopup();

        // If iconic, restore the main window
        if (pWndPrev->IsIconic())
            pWndPrev->ShowWindow(SW_RESTORE);

        // Bring the main window or its popup to
        // the foreground
        pWndChild->SetForegroundWindow();

        // and we are done activating the previous one.
        return pWndChild;
    }
    // First instance. Proceed as normal.
    else
        return NULL;
}

//--------------------------------------------------------------------------

BOOL CModellerApp::InitInstance()
{
    // 1- Set App Class Name
    m_strClassName.LoadString( IDS_APPCLASS_NAME ) ;

    // 2 - Check if the App already exists
    CWnd *pWndOtherInstance = NULL ;
    // not exists : create App
    if((pWndOtherInstance = FirstInstance()) == NULL)
    {
		//Check for Unrestored Browser parameters file
		CRecoverManager RecoverManager ;

		RecoverManager.Show() ;

        // 3- Init the OffLine state of the browser
        m_bOffLine = IsGlobalOffLine() ;

        // 4- Init App
        if( !InitializeApp() )
            return FALSE ;
        //5 - Apply App project Info
        return ApplyProjectInfo( m_lpCmdLine ) ;
    }
    //exists : send message to the mainframe of the existing app
    else
    {
        COPYDATASTRUCT DataStruct ;
        //----
        DataStruct.dwData = 0 ;
        DataStruct.cbData = sizeof( TCHAR) * (strlen( m_lpCmdLine ) + 1 ) ;
        DataStruct.lpData = m_lpCmdLine ;

        pWndOtherInstance->SendMessage( WM_COPYDATA , 0, (LPARAM) &DataStruct );
        return FALSE ;
    }
}

//--------------------------------------------------------------------------

BOOL CModellerApp::ApplyProjectInfo( const CString& strArgList )
{
    // 1- Init App Arguments
    CString strUser;
    CString strPwd;
    CString strProjectName;
    CString strFileName;
    //----
    InitializeArgs( strArgList , strUser, strPwd, strProjectName, strFileName) ;

    //2 - Connection
    DoWaitCursor(1) ;

    if( !Connect( strUser , strPwd ) )
        return FALSE ;

	bool bOpenNewFile = (strFileName.IsEmpty() && !strProjectName.IsEmpty()) ? true : false;

    //If project is not specified on command line
    //look for last project opened in commander
    if(strProjectName.IsEmpty())
        strProjectName = GetLastOpenProject();

    //open the project
    OpenProject(strProjectName);

    DoWaitCursor(-1) ;
    //3 - Launch Project
    if( m_RepositoryAPI.IsProjectOpen() && !strFileName.IsEmpty() )
        OpenDocumentFromArgs(strFileName) ;
	else
	if(bOpenNewFile)
		OnFileNew();

    return TRUE ;
}

//--------------------------------------------------------------------------

BOOL CModellerApp::InitializeApp()
{
    // 1 - Register our unique class name that we wish to use : it is associated with the CMainFrame
    // See BOOL CMainFrame::PreCreateWindow(CREATESTRUCT& cs)
    WNDCLASS wndcls;
    //----
    memset(&wndcls, 0, sizeof(WNDCLASS));   // start with NULL defaults

    wndcls.style = CS_DBLCLKS | CS_HREDRAW | CS_VREDRAW;
    wndcls.lpfnWndProc = ::DefWindowProc;
    wndcls.hInstance = AfxGetInstanceHandle();
    wndcls.hIcon = LoadIcon(IDR_MAINFRAME); // or load a different icon
    wndcls.hCursor = LoadCursor( IDC_ARROW );
    wndcls.hbrBackground = (HBRUSH) (COLOR_WINDOW + 1);
    wndcls.lpszMenuName = NULL;

    // Specify our own class name for using FindWindow later

    wndcls.lpszClassName = m_strClassName ;


    // Register new class and exit if it fails
    if(!AfxRegisterClass(&wndcls))
    {
        TRACE("Class Registration Failed\n");
        return FALSE;
    }

    m_bRegisterClass = TRUE ;

    //2 - Init App
    CWinApp::InitInstance() ;
    AfxEnableControlContainer();

    // Standard initialization
    // If you are not using these features and wish to reduce the size
    //  of your final executable, you should remove from the following
    //  the specific initialization routines you do not need.

#ifdef _AFXDLL
    Enable3dControls();         // Call this when using MFC in a shared DLL
#else
    Enable3dControlsStatic();   // Call this when linking to MFC statically
#endif

    // Change the registry key under which our settings are stored.
    // TODO: You should modify this string to be something appropriate
    // such as the name of your company or organization.
    SetRegistryKey(_T("CYRANO Solutions"));

    ReadPreferences();
    //Init Serialization object
    m_Serialization.SetLocation(    CString("Software\\CYRANO Solutions\\")  +
                                    CString(m_pszAppName)                    +
                                    CString("\\Serialization")
                               );
    m_Serialization.Read() ;

    LoadStdProfileSettings();  // Load standard INI file options (including MRU)

    //3 - Add New Document model
    CMultiDocTemplate* pDocTemplate = new CMultiDocTemplate(
                                          IDR_MODELLTYPE,
                                          RUNTIME_CLASS(CModellerDoc),
                                          RUNTIME_CLASS(CChildFrame), // custom MDI child frame
                                          RUNTIME_CLASS(CModellerView));
    AddDocTemplate(pDocTemplate);
    m_pCaptureTemplate = pDocTemplate;

    pDocTemplate = new CMultiDocTemplate(
                       IDR_SUBROUTYPE,
                       RUNTIME_CLASS(CSubroutineDocument),
                       RUNTIME_CLASS(CSubroutineChildFrame), // custom MDI child frame
                       RUNTIME_CLASS(CSubroutineView));
    AddDocTemplate(pDocTemplate);
    m_pSubroutineTemplate = pDocTemplate;

    //4 - create main MDI Frame window
    CMainFrame* pMainFrame = new CMainFrame;
    if (!pMainFrame->LoadFrame(IDR_MAINFRAME))
        return FALSE;
    m_pMainWnd = pMainFrame;

    // register CodeMax
    CMRegisterControl();

    m_RepositoryAPI.SetWindow(AfxGetMainWnd());

    // The main window has been initialized, so show and update it.
    pMainFrame->ShowWindow(m_nCmdShow);
    pMainFrame->UpdateWindow();

    return TRUE ;
}

//--------------------------------------------------------------------------

void CModellerApp::InitializeArgs( const CString& strArgList, CString& strUser, CString& strPwd, CString& strProjectName, CString& strFileName)
{
    strUser.Empty() ;
    strPwd.Empty() ;
    strProjectName.Empty() ;
    strFileName.Empty() ;
    int nPos = 0 ;
    int nPos2 = 0 ;
    //----
    //find user
    nPos = strArgList.Find("--user" ) ;
    if( nPos >= 0 )
    {
        strUser = strArgList.Right(strArgList.GetLength() - nPos - strlen( "--user") - 1 ) ;
        nPos2 = strUser.Find(" ") ;
        if( nPos2 >= 0 )
            strUser = strUser.Left( nPos2 ) ;
    }

    //find pwd
    nPos = strArgList.Find("--pwd" ) ;
    if( nPos >= 0 )
    {
        strPwd = strArgList.Right(strArgList.GetLength() - nPos - strlen( "--pwd") - 1 ) ;
        nPos2 = strPwd.Find(" ") ;
        if( nPos2 >= 0 )
            strPwd = strPwd.Left( nPos2 ) ;
    }

    //find project
    nPos = strArgList.Find("--project" ) ;
    if( nPos >= 0 )
    {
        strProjectName = strArgList.Right(strArgList.GetLength() - nPos - strlen( "--project") - 1 ) ;
        strProjectName.TrimLeft(" \"");
        nPos2 = strProjectName.Find("\"") ;
        if( nPos2 >= 0 )
            strProjectName = strProjectName.Left( nPos2 ) ;
    }

    //find filename
    nPos = strArgList.Find("--file" ) ;
    if( nPos >= 0 )
    {
        strFileName = strArgList.Right(strArgList.GetLength() - nPos - strlen( "--file") - 1 ) ;
        nPos2 = strFileName.Find(" ") ;
        if( nPos2 >= 0 )
            strFileName = strFileName.Left( nPos2 ) ;
    }
}

//--------------------------------------------------------------------------

void CModellerApp::OpenDocumentFromArgs(const CString& strFileName)
{
    //    if (DoesDocumentExist(strFileName) )
    {
        OpenDocument(m_pCaptureTemplate,strFileName);
    }
    ///  else
    {
        //  OpenDocument(m_pCaptureTemplate,"");
    }

}

//--------------------------------------------------------------------------

BOOL CModellerApp::Connect( CString& strUser , CString& strPwd )
{
    if( !m_RepositoryAPI.IsConnected() )
    {
        return ConnectToRepository(strUser, strPwd) ;
    }
    else
        return TRUE ;
}

//--------------------------------------------------------------------------

BOOL CModellerApp::DoesDocumentExist(CString _strFileName)
{
    BOOL bRet = FALSE;

    CStringArray strFileArray ;
    long nCount = 0 ;
    //----
    nCount = m_RepositoryAPI.GetProjectCaptureFiles(strFileArray ) ;

    for( int nIndex = 0 ; nIndex < nCount ; nIndex ++ )
    {
        CString strFileName ;
        strFileName = strFileArray.GetAt( nIndex ) ;

        if ( strFileName.CompareNoCase(_strFileName) == 0 )
            return TRUE;
    }

    return FALSE;
}


//--------------------------------------------------------------------------

void CModellerApp::OnFileNew()
{
    OpenDocument(m_pCaptureTemplate,"");
}

//---------------------------------------------------------------------------

void CModellerApp::OnFileOpen()
{
    CCaptureOpenDlg dlg;
    if(dlg.DoModal() != IDOK)
        return;

    OpenDocument(m_pCaptureTemplate,dlg.m_strCapture);
}

//---------------------------------------------------------------------------

void CModellerApp::OpenDocument(CDocTemplate *pTemplate ,CString _strFileName)
{
    CWaitCursor wc;

    if(!_strFileName.IsEmpty() && IsDocumentOpen(m_RepositoryAPI.GetOpenProjectName(),_strFileName, TRUE))
        return;

    CBasedDocument* pDoc = static_cast<CBasedDocument*>(pTemplate->OpenDocumentFile(""));
    ASSERT_KINDOF(CBasedDocument, pDoc);

    //Create a temporary working directory
    CString strTmpPath = pDoc->GetSCLDocInfo()->GetTmpProjectPath();
    CString strWorkingFolder;
    GetTempFileName(strTmpPath,"CYR",0,strWorkingFolder.GetBuffer(MAX_PATH));
    strWorkingFolder.ReleaseBuffer();
    DeleteFile(strWorkingFolder);
    CreateDirectory(strWorkingFolder,NULL);

    if(_strFileName.IsEmpty())
        pDoc->GetSCLDocInfo()->SetFileInfo(UNTITLED_SCL_DOCUMENT);
    else
        pDoc->GetSCLDocInfo()->SetFileInfo(_strFileName);

    pDoc->GetSCLDocInfo()->SetProjectName(m_RepositoryAPI.GetOpenProjectName());
    pDoc->GetSCLDocInfo()->SetWorkingFolder(strWorkingFolder + "\\");

    pDoc->RetrieveFromRepository();
    if(!_strFileName.IsEmpty())
    {
        pDoc->SetTitle(_strFileName);
    }
}

//---------------------------------------------------------------------------

CString CModellerApp::GetRegistryPath()
{
    CString strPath ;
    //---
    strPath = _T("Software\\" ) ;
    strPath +=  m_pszRegistryKey ;
    strPath +=  _T("\\" ) ;
    strPath += m_pszAppName ;
    return strPath ;
}

//--------------------------------------------------------------------------

BOOL CModellerApp::ConnectToRepository(CString _strUser, CString _strPwd)
{
    BOOL bRet = FALSE ;

    //---
    try
    {
        bRet = m_RepositoryAPI.Connect( _strUser , _strPwd ) ;
    }
    catch( CException e )
    {
        e.ReportError() ;
        return FALSE ;
    }

    return bRet ;
}

//--------------------------------------------------------------------------

int CModellerApp::ExitInstance()
{
    if( m_RepositoryAPI.IsConnected() )
        m_RepositoryAPI.DisConnect() ;

    if(m_bRegisterClass)
        ::UnregisterClass(m_strClassName, AfxGetInstanceHandle());

    SetGlobalOffLine( m_bOffLine ) ;
    m_Serialization.Save() ;

    SavePreferences();

    return CWinApp::ExitInstance();
}

//--------------------------------------------------------------------

void CModellerApp::SetGlobalOffLine( BOOL bOffLine )
{
    INTERNET_CONNECTED_INFO ci;
    memset(&ci, 0, sizeof(ci));
    if(bOffLine)
    {
        ci.dwConnectedState = INTERNET_STATE_DISCONNECTED_BY_USER;
        ci.dwFlags = ISO_FORCE_DISCONNECTED;
    }
    else
        ci.dwConnectedState = INTERNET_STATE_CONNECTED;

    InternetSetOption(NULL,
                      INTERNET_OPTION_CONNECTED_STATE, &ci, sizeof(ci));
}

//--------------------------------------------------------------------

BOOL CModellerApp::IsGlobalOffLine()
{
    DWORD dwState = 0;
    DWORD dwSize = sizeof(DWORD);
    BOOL bRet = FALSE;

    if(InternetQueryOption(NULL,
                           INTERNET_STATE_DISCONNECTED_BY_USER, &dwState, &dwSize))
        bRet = TRUE;


    return bRet;
}

//--------------------------------------------------------------------

// App command to run the dialog
void CModellerApp::OnAppAbout()
{
    CAboutDlg aboutDlg;
    aboutDlg.DoModal();
}
//--------------------------------------------------------------------

BOOL CModellerApp::IsDocumentOpen(CString _strProjectName, CString _strFileName, BOOL bActivate)
{
    BOOL bFound = FALSE;
    CBasedDocument* pBasedDoc;
    //---
    POSITION pos = m_pCaptureTemplate->GetFirstDocPosition();
    while (pos && !bFound)
    {
        CDocument * pDoc;
        //---
        if ((pDoc=m_pCaptureTemplate->GetNextDoc(pos)) != NULL)
        {
            pBasedDoc = static_cast<CBasedDocument*>(pDoc);
            ASSERT_KINDOF(CBasedDocument, pDoc);

            bFound = ( pBasedDoc->GetSCLDocInfo()->GetProjectName() == _strProjectName );
            bFound = bFound && ( pBasedDoc->GetSCLDocInfo()->GetSCLFileName() == _strFileName );
        }
    }

    if( bFound )
    {
        POSITION pos2 = pBasedDoc->GetFirstViewPosition();
        if(pos2 != NULL )
        {
            CView* pView = pBasedDoc->GetNextView(pos2);
            pView->GetParentFrame()->ActivateFrame(SW_MAXIMIZE);
        }
    }

    return bFound;
}

CString CModellerApp::GetLastOpenProject()
{
    CString strProject;
    HKEY hKey;
    if(ERROR_SUCCESS == RegOpenKeyEx(HKEY_CURRENT_USER,"Software\\Cyrano Solutions\\Framework\\Opensta\\Environment",0,KEY_READ,&hKey))
    {
        DWORD dwType,dwLength=100;
        RegQueryValueEx(hKey,"Current Project",NULL,&dwType,(LPBYTE)strProject.GetBuffer(dwLength),&dwLength);
        RegCloseKey(hKey);
        strProject.ReleaseBuffer();
    }
    return strProject;
}

void CModellerApp::OnOpenProject()
{
    if(!m_RepositoryAPI.GetProjectCount())
    {
        AfxMessageBox("There are no projects available.\nPlease specify a project using Test Commander.");
        return;
    }

    CProjectOpenDlg dlg;
    if(dlg.DoModal() != IDOK)
        return;

    OpenProject(dlg.m_strProject);
}

void CModellerApp::OnUpdateFileNew(CCmdUI* pCmdUI)
{
    pCmdUI->Enable(m_RepositoryAPI.IsProjectOpen());
}

void CModellerApp::OnUpdateFileOpen(CCmdUI* pCmdUI)
{
    pCmdUI->Enable(m_RepositoryAPI.IsProjectOpen());
}

void CModellerApp::OnOptionsBrowser()
{
    CSelectBrowserDlg dlg;
    dlg.SetCurrentBrowser(m_nBrowserID);
    if(dlg.DoModal() != IDOK)
        return;
    m_nBrowserID = dlg.GetCurrentBrowser();
}

void CModellerApp::ReadPreferences()
{
    m_strVariablePrefix = GetProfileString("Editor","Variable Prefix","");

    m_bConsole = GetProfileInt("Gateway","Console",false);
    m_bTrace = GetProfileInt("Gateway","Trace",false);
	m_bAutoCookieGeneration = GetProfileInt("Gateway","Auto Cookie Gen",true);

    m_GatewayData.m_bLocal = GetProfileInt("Gateway","Local",TRUE);
    m_GatewayData.m_lAdministrationPort  = GetProfileInt("Gateway","Admin Port",3000);
    m_GatewayData.m_lListenPort = GetProfileInt("Gateway","Listen Port",81);

    //Address
    m_GatewayData.m_ProxyInfo.m_RemoteProxy.m_strProxyAddress = GetProfileString("Gateway","Proxy Address"," ");
    m_GatewayData.m_ProxyInfo.m_RemoteProxy.m_lAddressPort = GetProfileInt("Gateway","Proxy Port",0);

    //Secure
    m_GatewayData.m_ProxyInfo.m_RemoteProxy.m_strProxySecure = GetProfileString("Gateway","Proxy Secure"," ");
    m_GatewayData.m_ProxyInfo.m_RemoteProxy.m_lSecurePort         = GetProfileInt("Gateway","Secure Port",0);
    ;
    m_GatewayData.m_ProxyInfo.m_RemoteProxy.m_bByPassProxyLocal   = GetProfileInt("Gateway","Bypass Proxy",FALSE);

    //No proxy
    m_GatewayData.m_ProxyInfo.m_RemoteProxy.m_strAddressesNoProxy = GetProfileString("Gateway","No Proxy"," ");

    m_nBrowserID = GetProfileInt("Gateway","Browser",ID_BROWSER_IE5);
}

void CModellerApp::SavePreferences()
{
    WriteProfileString("Editor","Variable Prefix",m_strVariablePrefix);

    WriteProfileInt("Gateway","Console",m_bConsole);
    WriteProfileInt("Gateway","Trace",m_bTrace);
    WriteProfileInt("Gateway","Auto Cookie Gen",m_bAutoCookieGeneration);

    WriteProfileInt("Gateway","Local",m_GatewayData.m_bLocal);
    WriteProfileInt("Gateway","Admin Port",m_GatewayData.m_lAdministrationPort);
    WriteProfileInt("Gateway","Listen Port",m_GatewayData.m_lListenPort);

    //Address
    WriteProfileString("Gateway","Proxy Address",m_GatewayData.m_ProxyInfo.m_RemoteProxy.m_strProxyAddress);
    WriteProfileInt("Gateway","Proxy Port",m_GatewayData.m_ProxyInfo.m_RemoteProxy.m_lAddressPort);

    //Secure
    WriteProfileString("Gateway","Proxy Secure",m_GatewayData.m_ProxyInfo.m_RemoteProxy.m_strProxySecure);
    WriteProfileInt("Gateway","Secure Port",m_GatewayData.m_ProxyInfo.m_RemoteProxy.m_lSecurePort);
    WriteProfileInt("Gateway","Bypass Proxy",m_GatewayData.m_ProxyInfo.m_RemoteProxy.m_bByPassProxyLocal);

    //No proxy
    WriteProfileString("Gateway","No Proxy",m_GatewayData.m_ProxyInfo.m_RemoteProxy.m_strAddressesNoProxy);

    WriteProfileInt("Gateway","Browser",m_nBrowserID);
}

//-----------------------------------------------------------------------

void CModellerApp::OnOptionsGateway()
{
    CBrowserInformation *pBrowserInfo;
    switch(m_nBrowserID)
    {
    case ID_BROWSER_IE4:
        pBrowserInfo = static_cast<CBrowserInformation*> (new CIE4Information(AfxGetMainWnd())) ;
        break ;
    case ID_BROWSER_IE5 :
        pBrowserInfo = static_cast<CBrowserInformation*> (new CIE5Information(AfxGetMainWnd())) ;
        break ;
    case ID_BROWSER_NS47 :
        pBrowserInfo = static_cast<CBrowserInformation*> (new CNS47Information(AfxGetMainWnd())) ;
        break ;
    default:
        ASSERT(FALSE);
        pBrowserInfo = static_cast<CBrowserInformation*> (new CIE5Information(AfxGetMainWnd())) ;
        break;
    }

    BOOL bResult = pBrowserInfo->GetDefaultParameters(&m_GatewayData.m_ProxyInfo.m_LocalProxy);

    delete pBrowserInfo;
    if(!bResult)
        return;

    CGatewayOptionsDlg GatewayOptionsDlg(&m_GatewayData);
    GatewayOptionsDlg.m_bTrace = m_bTrace;
    GatewayOptionsDlg.m_bConsole = m_bConsole;
	GatewayOptionsDlg.m_bAutoCookieCheck = m_bAutoCookieGeneration;
    if(GatewayOptionsDlg.DoModal() != IDOK)
        return;

    m_bTrace = GatewayOptionsDlg.m_bTrace;
    m_bConsole = GatewayOptionsDlg.m_bConsole;
	m_bAutoCookieGeneration = GatewayOptionsDlg.m_bAutoCookieCheck;
}

//-----------------------------------------------------------------------

BOOL CModellerApp::IsCaptureLocal()
{
    return m_GatewayData.m_bLocal;
}

//-----------------------------------------------------------------------

void CModellerApp::OnOptionsVariables()
{
    CVariableOptionDlg dlg;
    dlg.m_strPrefix = m_strVariablePrefix;
    if(dlg.DoModal() == IDOK)
        m_strVariablePrefix = dlg.m_strPrefix;
}

//-----------------------------------------------------------------------

void CModellerApp::OnToolsEdit()
{
    COpenDocumentDlg dlg;
    if(dlg.DoModal() != IDOK)
        return;

    OpenDocument(m_pSubroutineTemplate,dlg.m_strFileName);
}

//-----------------------------------------------------------------------

void CModellerApp::OnUpdateToolsEdit(CCmdUI* pCmdUI)
{
    pCmdUI->Enable(m_RepositoryAPI.IsProjectOpen());
}

//-----------------------------------------------------------------------

void CModellerApp::GetGatewayInfomation( _TPreferencesGatewayData *pData )
{
	memcpy(pData,&m_GatewayData,sizeof(_TPreferencesGatewayData));
}

//-----------------------------------------------------------------------
void CModellerApp::OpenProject(const CString& strProject)
{
    m_RepositoryAPI.OpenProject(strProject);

    UpdateWindowText();
}

//-----------------------------------------------------------------------
void CModellerApp::UpdateWindowText()
{
    CString strProject = m_RepositoryAPI.GetOpenProjectName();

    CString strTitle;
    strTitle.Format("Script Modeler - %s",strProject);
    ((CFrameWnd*)m_pMainWnd)->SetTitle(strTitle);
    m_pMainWnd->SetWindowText(strTitle);
}
