/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// VarFilePPage.cpp : implementation file
//

#include "stdafx.h"
#include "modeller.h"
#include "VarFilePPage.h"
#include <ExtendedPropertyPage.h>
#include "htmlhelp.h"
#include "Help\VarFilePPage.inl"
#include <ConstantesLib.h>

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CVarFilePPage property page
//---------------------------------------------------------------------------

IMPLEMENT_DYNCREATE(CVarFilePPage, CExtendedPropertyPage)

//---------------------------------------------------------------------------

CVarFilePPage::CVarFilePPage(CVarFile * _pVarFile, BOOL _bCreateMode) : CExtendedPropertyPage(CVarFilePPage::IDD)
{
    //{{AFX_DATA_INIT(CVarFilePPage)
    // NOTE: the ClassWizard will add member initialization here
    //}}AFX_DATA_INIT

    m_pVarFile      = _pVarFile ;
    m_bCreateMode   = _bCreateMode ;
}

//---------------------------------------------------------------------------

CVarFilePPage::~CVarFilePPage()
{}

//---------------------------------------------------------------------------


BEGIN_MESSAGE_MAP(CVarFilePPage, CExtendedPropertyPage)
//{{AFX_MSG_MAP(CVarFilePPage)
ON_BN_CLICKED(ID_EDITFILE_BUTTON, OnEditFileButton)
ON_BN_CLICKED(ID_VALFILE_BUTTON, OnValFileButton)
ON_WM_CTLCOLOR()
ON_WM_HELPINFO()
//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CVarFilePPage message handlers
//---------------------------------------------------------------------------

BOOL CVarFilePPage::OnInitDialog()
{
    CExtendedPropertyPage::OnInitDialog() ;

    Values2Ctrl();
    return TRUE;  // return TRUE unless you set the focus to a control
    // EXCEPTION: OCX Property Pages should return FALSE
}

//---------------------------------------------------------------------------


void CVarFilePPage::OnEditFileButton()
{
    int iRes ;
    int nPos ;
    CString strFilePath ;

    GetDlgItemText(ID_VAR_FILEPATH_EDIT, strFilePath) ;

    if (strFilePath.IsEmpty())
        return;

    if(strFilePath.Right(strlen(STR_FVR_EXTENSION)).CompareNoCase(STR_FVR_EXTENSION)!=0)
        strFilePath += STR_FVR_EXTENSION;

	SetDlgItemText(ID_VAR_FILEPATH_EDIT, strFilePath) ;

    if( !m_bCreateMode )
    {
        nPos = strFilePath.Find( '\\' ) ;
        if( nPos < 0 )
        {
            strFilePath = m_pVarFile->m_strFVRPath + strFilePath;
        }
    }

    iRes = (int) ShellExecute(
               m_hWnd,
               TEXT("open"),
               TEXT("notepad"),
               strFilePath,
               NULL,
               SW_SHOWNORMAL) ;
}

//---------------------------------------------------------------------------

void CVarFilePPage::OnValFileButton()
{
    DWORD dwFlags = 0 ;

    dwFlags = OFN_EXPLORER | OFN_FILEMUSTEXIST | OFN_PATHMUSTEXIST ; //OFN_OVERWRITEPROMPT ;

    CFileDialog FileDlg(TRUE) ; //, NULL, NULL, dwFlags, NULL, this) ;

    if (FileDlg.DoModal() == IDOK)
    {
        CWnd* pEdit = static_cast<CWnd*>(GetDlgItem(ID_VAR_FILEPATH_EDIT)) ;
        pEdit->SetWindowText( FileDlg.GetPathName() ) ;
    }

}

//---------------------------------------------------------------------------

void CVarFilePPage::Values2Ctrl()
{
    CWnd* pEdit = static_cast<CWnd*>(GetDlgItem(ID_VAR_FILEPATH_EDIT)) ;
    pEdit->SetWindowText( m_pVarFile->m_strFilePath) ;
}

//---------------------------------------------------------------------------

void CVarFilePPage::Ctrl2Values()
{
    GetDlgItemText(ID_VAR_FILEPATH_EDIT, m_pVarFile->m_strFilePath) ;
}


//---------------------------------------------------------------------------

LRESULT CVarFilePPage::OnWizardBack()
{
    return IDD_VAR_GENERAL_PPAGE ;
}

//---------------------------------------------------------------------------

LRESULT CVarFilePPage::OnWizardNext()
{
    return CExtendedPropertyPage::OnWizardNext() ;
}

//---------------------------------------------------------------------------

HBRUSH CVarFilePPage::OnCtlColor(CDC* pDC, CWnd* pWnd, UINT nCtlColor)
{
    HBRUSH hbr = CExtendedPropertyPage::OnCtlColor(pDC, pWnd, nCtlColor);

    if( nCtlColor == CTLCOLOR_STATIC && pWnd->m_hWnd == GetDlgItem( IDC_TITLE_STATIC )->m_hWnd )
    {
        pDC->SetTextColor( RGB( 0 , 0 , 255 ) ) ;
    }

    return hbr;
}

BOOL CVarFilePPage::OnHelpInfo(HELPINFO* pHelpInfo)
{
    CString contextHelp;
    contextHelp.LoadString(IDS_MODELLER_HELP_FILE);
    contextHelp= getenv("CYRANO_HOME")+contextHelp+"::whatisthis.txt";

    if (pHelpInfo->iCtrlId == IDC_STATIC)
        return true;

    CWnd* pWnd = GetDlgItem(pHelpInfo->iCtrlId);

    HtmlHelp(pWnd->m_hWnd, contextHelp, HH_TP_HELP_CONTEXTMENU, (DWORD)(LPVOID)g_TopicIdArray);

    return true;
}

BOOL CVarFilePPage::OnWizardFinish() 
{
	CString str;
	GetDlgItemText(ID_VAR_FILEPATH_EDIT,str);
	CFileStatus fs;
	if(!CFile::GetStatus(str,fs) && !CFile::GetStatus(str + ".FVR",fs))
	{
		AfxMessageBox(str + "\nCould not be found. Please enter a valid filename");
		return FALSE;
	}
	
	return CExtendedPropertyPage::OnWizardFinish();
}
