/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

/*
 * scl_main.c:
 * ===========:
 *
 * This is SCL's main controlling module.
 */

/*
 *  PSL Macros.
 */
#include <psl_macros.h>
#include <vtest_macros.h>

/*
 *  Standard ANSI header files.
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>

/*
 *  Function Prototypes.
 */
#include <psl_rtndef.h>             /* PSL Routines     */
#include <trc_rtndef.h>             /* Trace Routines   */
#include <vtest_rtndef.h>           /* V-TEST Routines  */
#include "scl_rtndef.h"             /* SCL Routines     */

/*
 *  Structure Definitions.
 */
#include <shrmsg.h>             /* PSL Messages  */
#include <psl_trc_prmdef.h>         /* Tracing definitions    */
#include <pslerr_prmdef.h>          /* PSL Error Parameters   */
#include <psldiag_prmdef.h>         /* Diagnostic definitions */
#include "scl_cmd.h"                /* Command Line Data Structure */
#include "scl_comdef.h"             /* Common Data        */

int main (long argc, char *argv[])
{
    /*
     *   Local Data,
     */
    unsigned long    iret;          /* Return Status */
    static   struct  scl_com scl_com;   /* Common Data */
    static   struct  scl_cmd scl_cmd;   /* Command Line Struct */

    /*
    *  Include the change revision level header file.
    */
#if defined UNIX
#include <psl_chgident.h>
#endif

    /*
     *  Store the utility name for use by PSL_GET_MESSAGE.
     */
    PSL_SET_UTILNAME ("SCL");

    /*
     *  Set up pointer to Command Line Data in Common Data.
     */
    scl_com.cmd_ptrps = &scl_cmd;

    /*
     
     *  Allocate the Diagnostics Stack
     */
    scl_com.diag_ptrps = PSL_ALLOCATE_DIAGNOSTICS();
    if (scl_com.diag_ptrps == NULL)
    {
        iret = PSL_INSVIRMEM;
        PSL_SIGDIAG (NULL, 2, PSL_SYSERVERR,
                     "PSL_ALLOCATE_DIAGNOSTICS", iret);
        goto end;
    }

    /*
     *  Flag that, if any errors are signalled, we want to report them
     *  and exit.
     */
    (void) PSL_TRAP_ERRORS (        /* Say we want to trap errors */
        scl_com.diag_ptrps, /* via this diagnostics pointer */
        0,          /* all classes */
        0,          /* all numbers */
        PSLERRACT_EXIT,     /* And report them */
        NULL,           /* with no setjmp environment */
        NULL);          /* No error handling routine */

    /*
     *  Get and Parse the Command Line.
     */
    iret = SCL_GETCMD (&scl_com, argc, argv);
    if  (!(psl_test_status (iret))) goto end;

    /*
     *    If we have requested confirmation of the compilation of files then we
     *    will need to open a stream to the keyboard (stderr)
     */
    if (scl_cmd.confirmn)
    {
        PSL_CLEARERR();
#ifdef  UNIX
        scl_com.ttyfilpf = fopen("/dev/tty", "w");
#elif   WIN32
scl_com.ttyfilpf = NULL;
#else
#error  Platform unsupported at line __LINE__ (Cannot open channel to terminal)
#endif

#ifndef WIN32
        if (scl_com.ttyfilpf == NULL)
        {
            iret = PSL_OPENOUT;
            PSL_LOG_CIOERR (scl_com.diag_ptrps, iret, "/dev/tty");
            PSL_OUTPUT_DIAG (scl_com.diag_ptrps, 0, DIOUFL_LSTERR);
            goto end;
        }
#endif  /* WIN32 */
    }
    else
    {
        scl_com.ttyfilpf = NULL;
    }

    /*
     *    ...and call the main compilation routine.
     */
    SCL_COMPILE (&scl_com);

    /*
     *  Deallocate Diagnostics stack.
     */
    PSL_DEALLOCATE_DIAGNOSTICS (scl_com.diag_ptrps);

    /*
     *  Exit with 'Worst Error' status
     */
end:
    PSL_EXIT ();

    return 0;
}
