#ifndef PSL__SCL_TPACOM_LOADED
#define PSL__SCL_TPACOM_LOADED 1
/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

/*
 *  scl_tpacom.h:
 *  =============
 *
 *  TPA Common Data Structure.
 */

/* */
/*    This structure is used when parsing an array to maintain information */
/*    relating to each level of the array. */
/* */
struct array_blk
{
    /* */
    /*    This field holds the pointer to the variable symbol table entry for the */
    /*    ARRAY variable. */
    /* */
    struct symtbl           *var_ptrps;
    /*
     *    This field contains the current nesting depth of the parenthesis on entry
     *    the the current array index.
     */
    PSL_U_LONGWORD_T        nesting_levell;
    /* */
    /*    This field contains the current number of array indecis specified for */
    /*    this array. */
    /* */
    PSL_U_LONGWORD_T        array_indicesl;
    /* */
    /*    This is a filler to a quadword boundary. */
    /* */
    char                    fillerb4[4];
} ;
/* */
/*   This structure contains all the data used by the TPARSE */
/*   routines. */
/* */
struct scl_tpa
{
    /* */
    /*    Data for the source lines. */
    /*    ========================== */
    /* */
    /* */
    /*    Line as input */
    /* */
    char                    source_linec136[136];
    /* */
    /*    De-commented upper case line */
    /* */
    char                    parsable_linec136[136];
    /* */
    /*    Copy of line for underlining errors */
    /* */
    char                    error_linec136[136];
    /* */
    /*    Length of Source_line */
    /* */
    PSL_U_LONGWORD_T        line_sizel;
    /* */
    /*    Length of Parsable_line */
    /* */
    PSL_U_LONGWORD_T        parsable_sizel;

    /* */
    /*    Address of encoded branch instruction */
    /* */
    PSL_U_LONGWORD_T        branch_addrl;
    /* */
    /*    Source line read */
    /* */
    PSL_U_LONGWORD_T        line_read_flagl;
    /* */
    /*    Errors in source line */
    /* */
    PSL_U_LONGWORD_T        errors_in_linel;
    /* */
    /*    Underline required */
    /* */
    PSL_U_LONGWORD_T        underline_flagl;
    /* */
    /*    Total count of errors */
    /* */
    PSL_U_LONGWORD_T        error_countl;
    /* */
    /*    These two items are used in generating branch and jump */
    /*    instructions. The first is used to store the offset of the */
    /*    displacement field of the destination operand of a JMP */
    /*    instruction. The second holds the branch type (symbols defined */
    /*    in the macro SCLDEF). */
    /* */
    /* */
    /*    Branch displacement field offset */
    /* */
    PSL_U_LONGWORD_T        branch_offl;
    /* */
    /*    Branch type */
    /* */
    PSL_U_LONGWORD_T        branch_typel;
    /* */
    /*    Integer value (signed) */
    /* */
    PSL_LONGWORD_T          int_valuel;
    /* */
    /*    Save TPARSE options here when parsing a comment */
    /* */
    PSL_U_LONGWORD_T        comment_optsl;
    /* */
    /*    This longword is used to check for duplicate and */
    /*    conflicting convert command options. It contains a bitmap, in */
    /*    which each bit represents an option. If a bit is set, it */
    /*    indicates that the option has already been specified. If it is */
    /*    clear, it has not. */
    /* */
    PSL_U_LONGWORD_T        cnv_opt_specifiedl;
    /* */
    /*    Conversion radix */
    /* */
    PSL_U_LONGWORD_T        cnv_radixl;
    /* */
    /*    Convert options mask */
    /* */
    PSL_U_LONGWORD_T        cnv_optl;
    /* */
    /*    This longword is used to check for duplicate and */
    /*    conflicting 'string' options (such as CASE_BLIND and */
    /*    NOTRAILING). It contains a bitmap, in which */
    /*    each bit represents an option. If a bit is set, it indicates */
    /*    that the option has already been specified. If it is clear, */
    /*    it has not. */
    /* */
    PSL_U_LONGWORD_T        opt_opt_specifiedl;
    /* */
    /*    This longword is used to flag 'string' options. It */
    /*    contains a bitmap, in which each bit represents an input */
    /*    option. */
    /* */
    PSL_U_LONGWORD_T        opt_optl;
    /*    OPT_TRAILING_M    Clear = NO TRAILING SPACES */
    /*                      Set   = TRAILING SPACES */
    /* */

    /* */
    /*    This item is used in parsing commands that can contain */
    /*    input rate specifiers, i.e. the Environment INPUT RATE command */
    /*    and the DEFINE INPUT RATE command. It holds a value, which may */
    /*     be a 'Default Input Rate' Flag, a 'Maximum Input Rate' flag or */
    /*    the input delay in 10ms units. */
    /* */
    PSL_U_LONGWORD_T        input_delayl;
    /* */
    /*    This longword is used to save the address of the */
    /*    displacement operand of a branch instruction. */
    /* */
    PSL_U_LONGWORD_T        branch_disp_addrl;
    /* */
    /*    Dispacement for ONCOND calls */
    /* */
    PSL_U_LONGWORD_T        oncond_disp_addrl;
    /* */
    /*    This field holds the name of the label specified on an ordinary */
    /*    GOTO or an AT END GOTO clause/statement. */
    /* */
    char                    label_namec16[16];
    /* */
    /*    This field contains the length of the name in the previous field. */
    /* */
    PSL_U_LONGWORD_T        label_lenl;
    /* */
    /*    This field holds the name of the label specified on an ON ERROR */
    /*    GOTO clause/statement. */
    /* */
    char                    onerror_labelc16[16];
    /* */
    /*    This field contains the length of the name in the previous field. */
    /* */
    PSL_U_LONGWORD_T        onerror_lenl;
    /* */
    /*    This field contains a pointer to the SCP offset where any ON ERROR */
    /*    label offset should be stored. */
    /* */
    char                    *onerror_addrpc;
    /* */
    /* */
    /* */
    /*    This field holds the name of the label specified on an ON TIMEOUT */
    /*    GOTO clause/statement. */
    /* */
    char                    ontimeout_labelc20[20];
    /* */

    /*
     *    This field is used to hold the null-terminated subroutines name.
     */
    char                    subroutine_namec20[20];
    /* */
    /*    And a byte mask for various flags to show subroutine usage */
    /* */
    PSL_U_LONGWORD_T        subroutine_flagsl;

    /* */
    /*   This longword is used in timer name parsing to hold the */
    /*   address of a Timer's Symbol Table entry. */
    /* */
    struct timtbl           *tim_ptrps;

    /* */
    /*    The NONLOCAL_INPUT_FLAG contains zero if there are no INPUT */
    /*    commands that specify a script or global variable and one */
    /*    otherwise. This is used to determine whether or not to */
    /*    allocate a work buffer in the Local Data Area. */
    /* */
    PSL_U_LONGWORD_T        nonlocal_input_flagl;
    /* */
    /*    Parameter count */
    /* */
    PSL_U_LONGWORD_T        prm_cntl;
    /* */
    /*    VARIABLE PARSING DATA */
    /*    ===================== */
    /* */
    /*    These data items are used when parsing Variable and Timer */
    /*    definitions. Data to be entered into the Variables or Timers */
    /*    Symbol Tables is stored in these items temporarily and transferred */
    /*    to the appropriate Symbol Table once a definition is complete. */
    /*    This is done to so that only complete entries are inserted into */
    /*    the Symbol Tables. */
    /* */
    /* */
    /*    Definition Type */
    /* */
    PSL_U_LONGWORD_T        def_typl;
    /* */
    /*    Name of definition */
    /* */
    char                    def_namec16[16];
    /* */
    /*    Length of name */
    /* */
    PSL_U_LONGWORD_T        def_nlengthl;
    /* */
    /*    Addressing mode */
    /* */
    PSL_U_LONGWORD_T        def_addrl;
    /* */
    /*    Definition size in bytes */
    /* */
    PSL_U_LONGWORD_T        def_sizel;
    /* */
    /*    definition data type */
    /* */
    PSL_U_LONGWORD_T        def_dtypl;
#define            DEF_VARIABLE 1   /* Variable definition */
#define            DEF_TIMER 2  /* Timer Definition */
#define            DEF_FILE 3   /* File Definition */
#define            DEF_PIPE 4   /* Pipe Definition */
#define            DEF_TEST 5   /* External Test Declaration */
#define            DEF_TEST_THREAD 6
    /* Test Thread Declaration */
#define            DEF_THREAD 7     /* Thread Definition */
#define            DEF_SUBTEST 8    /* Test Definition */
    /* */
    /*    Definition value list offset */
    /* */
    PSL_U_LONGWORD_T        def_values_offl;
    /* */
    /*    Variable definition value */
    /* */
    PSL_U_LONGWORD_T        def_values_cntl;
    /* */
    /*    Low range value */
    /* */
    PSL_U_LONGWORD_T        def_low_rangel;
    /* */
    /*    High range value */
    /* */
    PSL_U_LONGWORD_T        def_high_rangel;
    /* */
    /*    Record Variable index */
    /* */
    PSL_U_LONGWORD_T        def_recidxl;
    /* */
    /*    Prompt string */
    /* */
    char                    def_promptc50[50];
    /* */
    /*    Filler/alignment */
    /* */
    char                    filler1b2[2];
    /* */
    /*    Length of prompt */
    /* */
    PSL_U_LONGWORD_T        def_plengthl;
    /*
     *    Filename string for file variables. This string is NULL terminated.
     */
    char                    def_filnamc256[256];
    /* */
    /*     Array Bounds */
    /* */
    PSL_LONGWORD_T          def_array_bounds1l10[10];
    /* */
    /*    Array Bounds */
    /* */
    PSL_LONGWORD_T          def_array_bounds2l10[10];
    /* */
    /*    Current no of dimensions */
    /* */
    PSL_U_LONGWORD_T        array_dimcntl;
    /* */
    /*    This longword is used during parsing of script options to determine which */
    /*    script options have been specified. */
    /* */
    PSL_U_LONGWORD_T        scl_optl;
    /* */
    /*    This longword is used to flag variable options. It */
    /*    contains a bitmap, in which each bit represents a variable */
    /*    option. */
    /* */
    PSL_U_LONGWORD_T        def_optl;
    /* */
    /*    The bits are interpreted as follows: */
    /* */
    /*    VOPT_RANDOM_M */
    /*              Clear = RANDOM character values (Default) */
    /*              Set   = REPEATABLE RANDOM character values */
    /*    VOPT_SCOPE_M */
    /*              Clear = Scope defaulted (Default) */
    /*              Set   = Scope specified */
    /*    VOPT_LIST_M */
    /*              Clear = No value list defined (Default) */
    /*              Set   = Value list defined */
    /*    VOPT_RANGE_M */
    /*              Clear = No value range defined (Default) */
    /*              Set   = Value range defined */
    /*    VOPT_SEED_M */
    /*              Clear = No seed specified */
    /*              Set   = seed specified */
    /* */
    /* */
    /*    This longword is used to maintain the value of any seed specified */
    /*    for a repeatable random variable whilst parsing the variable definition. */
    /* */
    PSL_U_LONGWORD_T        def_seedl;
    /* */
    /*    This longword is used to check for duplicate and */
    /*    conflicting variable options. It contains a bitmap, in which */
    /*    each bit represents a variable option. If a bit is set, it */
    /*    indicates that the option has already been specified within the */
    /*    current variable definition. If it is clear, it has not. */
    /* */
    PSL_U_LONGWORD_T        def_opt_specifiedl;
    /* */
    /*    Variables Symbol Table */
    /* */
    struct symtbl           *variable_begps;
    /* */
    /*    Pointer to symbol table entry of currently parsed variable */
    /* */
    struct symtbl           *var_ptrps;
    /* */
    /*    Count of variables in variable symbol table. */
    /* */
    PSL_U_LONGWORD_T        var_cntl;
    /* */
    /*   LOCAL_DATA_OFF holds the offset of the next free byte in the */
    /*   Local Data Area. This is initially set to the byte after the service */
    /*   routines parameter table by the initialization routine. */
    /* */
    PSL_U_LONGWORD_T        local_data_offl;
    /* */
    /*    LDA_WRKBUF_OFF contains the offset of the work buffer, if there */
    /*    is one, in the Local Data Area. This is required when using the */
    /*    ~INPUT command with script or global variables. The contents of */
    /*    the variable are copied into the work buffer and the input taken */
    /*    from there. This ensures that the contents of the variable are */
    /*    not updated during an input. */
    /* */
    PSL_U_LONGWORD_T        lda_wrkbuf_offl;
    /* */
    /*    SCRIPT_DATA_OFF holds the offset of the next free byte in the */
    /*    Script Data Area. */
    /* */
    PSL_U_LONGWORD_T        script_data_offl;
    /* */
    /*    Count of Script Variables */
    /* */
    PSL_U_LONGWORD_T        script_val_cntl;
    /* */
    /*    Count of number of variables with value lists/ranges */
    /* */
    PSL_U_LONGWORD_T        val_cntl;
    /* */
    /*    Count of number of variables with random value lists/ranges */
    /* */
    PSL_U_LONGWORD_T        random_val_cntl;
    /* */
    /*    Define a pointer to the labels symbol table. */
    /* */
    struct labtbl           *label_begps;
    /* */
    /*    Label count */
    /* */
    PSL_U_LONGWORD_T        lab_cntl;
    /* */
    /*    Define a pointer to the Subroutine Symbol Table */
    /* */
    struct subtbl           *subroutine_begps;
    /* */
    /*    Number of entries in the Subroutines symbol table */
    /* */
    PSL_U_LONGWORD_T        subrtn_cntl;
    /* */
    /*    CONSTANT_DATA_OFF holds the offset of the next free byte in the */
    /*    Constants Data Area. */
    /* */
    PSL_U_LONGWORD_T        constant_data_offl;
    /* */
    /*    Max size of constants area */
    /* */
#define            MAX_CONSTANT_SIZE 65535
    /* */
    /*    Pointer to constants table */
    /* */
    char                    *constant_begpc;
    /* */
    /*   Define a count of the number of script parameters in a parameter list */
    /* */
    PSL_U_LONGWORD_T        call_prm_cntl;
    /* */
    /*    DO Parsing Stack */
    /*    ================ */
    /* */
    /*    The DO Parsing Stack is used to hold data during the parsing of a */
    /*    DO expression. It is used to maintain the offset of the start of the */
    /*    DO expression and the CONTROL, START, LIMIT and STEP operands. */
    /* */
    /* */
    /*    Max nesting level for DO */
    /* */
#define            MAX_DO 10
    /* */
    /*    The DO Stack */
    /* */
    struct dostack          *dostack_begps;
    /* */
    /*    Count of current level */
    /* */
    PSL_U_LONGWORD_T        do_cntl;
    /* */
    /*    IF Parsing data */
    /*    =============== */
    /* */
    /*    Because the IF command and its associated commands may span several */
    /*    commands and may be nested, it is necessary to maintain TWO stacks. */
    /* */
    /*    The first stack is required ONLY while parsing an individual IF and */
    /*    is called the IFCMND_STACK. It maintains the operator at each level */
    /*    of the IF command when parentheses are specified. IFCMND_LEVEL is */
    /*    a count of the current level of parentheses in the IF command. */
    /* */
    /*    The second stack is required over the entire script and maintains */
    /*    information vital to the parsing of multiple NESTED IF commands and */
    /*    is called the IFSTACK. A count of the current nesting level of the IF */
    /*    commands is maintained in IFSTACK_LEVEL. */
    /* */
    /* */
    /*    Max nesting level for IF Commands and parentheses. */
    /* */
#define            MAX_IF 10
    /* */
    /*    The IF Stack */
    /* */
    struct ifstack          *ifstack_begps;
    /* */
    /*    Count of current level */
    /* */
    PSL_U_LONGWORD_T        ifstack_levell;
    /* */
    /*    Generic Conditional Expression Parsing Stack */
    /*    ============================================ */
    /* */
    /*    This stack is used when parsing test, test thread and generic */
    /*    conditional expressions in WAIT FOR TEST, WAIT FOR THREAD and */
    /*    IF/ELSEIF commands. */
    /* */
    /*    Conditional expression parsing stack. */
    /* */
    PSL_U_LONGWORD_T        cond_stackl10[10];
    /* */
    /*    Conditional expression parsing level. */
    /* */
    PSL_U_LONGWORD_T        cond_levell;
    /* */
    /*    Timer count */
    /* */
    PSL_U_LONGWORD_T        tim_cntl;
    /* */
    /*    Script Header Record Buffer */
    /*    =========================== */
    /* */
    /* */
    /* */
    /*    Length of Header Record */
    /* */
#define            OBJ_HEADER_LEN 512
    /* */
    /*    Pointer top header record */
    /* */
    struct scph             *header_begps;
    /* */
    /*    End of object code buffer */
    /* */
    PSL_U_LONGWORD_T        header_endl;
    /* */
    /*    Script Object Code Buffer */
    /*    ========================= */
    /* */
    /* */
    /* */
    /*    Max SCP size = 500K */
    /* */
#define            MAX_SCP_SIZE 512000
    /* */
    /*    Pointer to next free position in the code buffer */
    /* */
    char                    *code_ptrpc;
    /* */
    /*    Start of code buffer */
    /* */
    char                    *code_begpc;
    /* */
    /*    End of code buffer */
    /* */
    char                    *code_endpc;
    /* */
    /*    Script Value List Buffer */
    /*    ======================== */
    /* */
    /*    An integer value list consists of a list of signed long */
    /*    integer values. A character value list consists of a sequence */
    /*    of counted strings, in which the count is held as a longword. */
    /* */
    /* */
    /*    Size of value list buffer */
    /* */
#define            OBJ_VALUES_SIZE 131068
    /* */
    /*    Pointer to value list */
    /* */
    char                    *values_begpc;
    /* */
    /*    End of Symbol Table buffer */
    /* */
    char                    *values_endpc;
    /* */
    /*    Address next free byte in the OBJ_VALUES buffer */
    /* */
    char                    *values_nxtpc;
    /* */
    /*    Size of value lists */
    /* */
    PSL_U_LONGWORD_T        values_sizel;
    /* */
    /*    Script Timers Symbol Table */
    /*    ========================== */
    /* */
    /*    The Timers Symbol Table consists of entries consisting of */
    /*    a single field - a 16 byte timer name. */
    /* */
    /* */
    /*    Max number entries allowed in the Timers Symbol Table */
    /* */
#define            MAX_TIMERS 1020
    /* */
    /*    Pointer to timers table. */
    /* */
    struct timtbl           *timer_begps;
    /* */
    /*    End of Symbol Table buffer */
    /* */
    char                    *timer_endpc;
    /* */
    /*    Size of timer lists */
    /* */
    PSL_U_LONGWORD_T        timer_sizel;
    /*
     *    Shareable Variables Symbol Table
     *    ================================
     *
     *
     *    Max no. of entries allowed in Shareable Variables Symbol Table
     */
#define            MAX_SHRVAR 8000
    /* */
    /*    Pointer to start of table. */
    /* */
    struct shrvartbl        *shrvar_begps;
    /* */
    /*    End of Symbol Table buffer */
    /* */
    char                    *shrvar_endpc;
    /* */
    /*    Pointer free entry in table. */
    /* */
    struct shrvartbl        *shrvar_nxtps;
    /* */
    /*    Shareable variable count */
    /* */
    PSL_U_LONGWORD_T        shrvar_cntl;
    /* */
    /*    Size of shareable variable lists */
    /* */
    PSL_U_LONGWORD_T        shrvar_sizel;
    /* */
    /*    Script Parameters Symbol Table */
    /*    ============================== */
    /* */
    /* */
    /*    Max number entries allowed in Script Params Symbol Table */
    /* */
#define            MAX_PARAMETERS 8
    /* */
    /*    Pointer next free in table */
    /* */
    struct prmtbl           *parameter_nxtps;
    /* */
    /*    Start of table */
    /* */
    struct prmtbl           *parameter_begps;
    /* */
    /*    End of Symbol Table buffer */
    /* */
    char                    *parameter_endpc;
    /* */
    /*    Script Parameters count */
    /* */
    PSL_U_LONGWORD_T        scpprm_cntl;
    /* */
    /*    Size of parameter lists */
    /* */
    PSL_U_LONGWORD_T        parameter_sizel;
    /* */
    /*    File Access Symbol Table */
    /*    ======================== */
    /* */
    /* */
    /*    File access table pointer */
    /* */
    struct fattbl           *fat_begps;
    /* */
    /*    Files count */
    /* */
    PSL_U_LONGWORD_T        fat_cntl;
    /* */
    /*    Index of current FID (1-10) (also used for pipes) */
    /* */
    PSL_U_LONGWORD_T        open_fididxl;
    /* */
    /*    Mask of open access */
    /* */
    PSL_U_LONGWORD_T        open_maskl;

    /* */
    /*   Variables required by the VP$SCREEN descriptor */
    /*   ============================================== */
    /* */
    /* */
    /*    Count of VP$SCREEN parameters */
    /* */
    PSL_U_LONGWORD_T        vps_prm_cntl;
    /* */
    /*   Define a flag whose sole purpose is to maintain track of what instruction */
    /*   we are processing */
    /* */
    PSL_U_LONGWORD_T        ins_flagl;
    /*
     *   Mask and bit values for the DISCONNECT command options.
     *   If the connection ID flag is not set, "ALL" can be assumed to have been
     *   specified.
     */
    PSL_U_LONGWORD_T        disconnect_maskl;
#define            DISCONNECT_CONID 0X1     /* Connection ID specified */
#define            DISCONNECT_CANCEL    0X2     /* "WITH CANCEL" specified */

    /*
     *   Define a mask and bit values for the EXIT options.
     */
    PSL_U_LONGWORD_T        exit_maskl;
    /* */
    /*   Define buffers to hold the Character Attributes parsing information */
    /*   =================================================================== */
    /* */
    /* */
    /*    Attributes option mask */
    /* */
    PSL_U_LONGWORD_T        attr_optl;
    /* */
    /*    attributes specified mask */
    /* */
    PSL_U_LONGWORD_T        attr_opt_specifiedl;
    /*
     *    Expression nesting level - there is no limit.
     */
    PSL_U_LONGWORD_T        expr_levell;
    /* */
    /*    Mask and mask values for the SNAP command. */
    /* */
    PSL_U_LONGWORD_T        snap_maskl;
    /* */
    /*    Mask and mask values for the WHEN command. */
    /* */
    PSL_U_LONGWORD_T        when_maskl;
    /* */
    /*    Mask and values for the Environment Options Specified */
    /* */
    PSL_U_LONGWORD_T        env_opt_specl;
    /* */
    /*    MODE information */
    /* */
    PSL_U_LONGWORD_T        mode_flagl;
    /* */
    /*    Delimiter Character */
    /* */
    char                    delimiter_chrb;
    /* */
    /*    Filler/alignment */
    /* */
    char                    filler2b3[3];
    /* */
    /*    Input Rate */
    /* */
    PSL_U_LONGWORD_T        input_ratel;
    /* */
    /*    Integer numeric parsing data */
    /* */
    PSL_LONGWORD_T          dec_integerl;
    /* */
    /*    fractional part. */
    /* */
    PSL_U_LONGWORD_T        dec_fractionl;
    /*
     *    Character storage. This field is used to store the representation of a
     *    single character value during parsing. Once upon a time this was a byte
     *    field containing the 8-bit character, with the advent of TOF file output
     *    it has changed to hold the representation of the character, i.e. the
     *    literal character (for printable 7-bit characters only) or a character
     *    command or control command, e.g. ~~, ~^, ~<CR>, ^M, ~<0D>. The field
     *    must be long enough to hold the longest character command - currently
     *    6 characters (~<BEL>, ~<DEL> and ~<ESC>).
     *    
     */
    unsigned char           string_chrc8[8];

    /*
     *    Filler for alignment.
     */
    char                    filler2b1[1];
    /* */
    /*    Sign character */
    /* */
    unsigned char           sign_chrb;
    /* */
    /*    Command character */
    /* */
    unsigned char           cmnd_chrb;
    /* */
    /*    Control character */
    /* */
    unsigned char           ctrl_chrb;
    /* */
    /*    Character string storage */
    /* */
#define        CHRBUFSIZ 65536
    char                    chr_stringc65536[CHRBUFSIZ];
    /* */
    /*    Length of character string */
    /* */
    PSL_U_LONGWORD_T        chr_strlenl;

    /* */
    /*    Pointer to operand in code */
    /* */
    char                    *opd_addrpc;
    /* */
    /*    When a source line contains several errors, all underlining is */
    /*    merged together and all errors are output after the underline. */
    /* */
#define            MAX_ERRORS_IN_LINE 5
    /* */
    /*    Array or Errors in Source Line */
    /* */
    PSL_U_LONGWORD_T        error_arrayl5[5];
    /* */
    /*    Data for parsing datanames */
    /*    ========================== */
    /* */
    /* */
    /*    Length of dataname */
    /* */
    PSL_U_LONGWORD_T        dataname_lengthl;
    /* */
    /*    Data name */
    /* */
    char                    *dataname_ptrpc;
    /* */
    /*    DEBUG DATA */
    /*    ========== */
    /* */
    /* */
    /*    source file symbol table size */
    /* */
    PSL_U_LONGWORD_T        fil_symtbl_sizel;
    /* */
    /*    modules symbol table size */
    /* */
    PSL_U_LONGWORD_T        mod_symtbl_sizel;
    /* */
    /*    labels symbol table size */
    /* */
    PSL_U_LONGWORD_T        lab_symtbl_sizel;
    /* */
    /*    Start of variables symbol table */
    /* */
    struct var_symtbl       *var_symtbl_begps;
    /* */
    /*    Start of labels symbol table */
    /* */
    struct lab_symtbl       *lab_symtbl_begps;
    /* */
    /*    Pointer to END of WHEN */
    /* */
    char                    *when_endpc;
    /* */
    /*    This field holds the current nesting level of the IFDEF condtional */
    /*    compilation commands. */
    /* */
#define            MAX_IFDEF 10
    PSL_U_LONGWORD_T        ifdef_levell;
    PSL_U_LONGWORD_T        ifdef_discard_levell;
    char                    variant_namec20[20];
    /* */
    /*    Saved Pointer to symbol table entry of currently parsed variable */
    /* */
    struct symtbl           *save_var_ptrps;
    /* */
    /*    This field holds the linked list pointer for the list of operand */
    /*    pointers. */
    /* */
    struct psl_vm_llb       *parsed_operand_llps;
    /* */
    /*    This field holds the linked list pointer for the list of parsed arrays */
    /* */
    struct psl_vm_llb       *parsed_array_llps;
    /* */
    /*      Define a pointer to the Diagnostic stack structure. */
    /* */
    struct diagstk          *diag_ptrps;
    /* */
    /*    This field is a bitmask which maintains a record of the options which */
    /*    have been specified for a particular command. */
    /* */
    /*    Each command option (ie returning status, filename, screenname) will have */
    /*    a separate value so that when encoding the commands we can determine what */
    /*    exactly has been specified. */
    /* */
    PSL_U_LONGWORD_T        cmnd_opt_specifiedl;
    PSL_U_LONGWORD_T        cmnd_optl;
    /* */
    /*    Given that the values for the above bitmask are internal to SCL we shall */
    /*    define the values for it here. */
    /* */
#define            CMND_OPT_RETURNING             0X1
#define            CMND_OPT_RESPONSE              0X2
#define            CMND_OPT_SEED              0X4
#define            CMND_OPT_SNAP_SCR              0X8
#define            CMND_OPT_SNAP_DSC             0X10
#define            CMND_OPT_STEP             0X20
#define            CMND_OPT_WAIT_PRD             0X40
#define            CMND_OPT_WAIT_INOPD           0X80
#define            CMND_OPT_VPS_LINE1           0X100
#define            CMND_OPT_VPS_LINE2           0X200
#define            CMND_OPT_VPS_COL1            0X400
#define            CMND_OPT_VPS_COL2            0X800
#define            CMND_OPT_SCREEN_NAME        0X1000
#define            CMND_OPT_TIMEOUT        0X2000
#define            CMND_OPT_ONERROR        0X4000
#define            CMND_OPT_READ_INDEX         0X8000
#define            CMND_OPT_MAP_HEADER        0X10000
#define            CMND_OPT_MATCH_CURRENCY    0X20000
#define            CMND_OPT_MATCH_RETURNVALUE1    0X40000
#define            CMND_OPT_MATCH_RETURNVALUE2    0X80000
#define            CMND_OPT_ONNODE       0X100000
#define            CMND_OPT_NUMBER       0X200000
#define            CMND_OPT_INTERVAL         0X400000
#define            CMND_OPT_DELAY        0X800000
#define        CMND_OPT_SCRIPT      0X1000000
#define        CMND_OPT_TOF         0X2000000
#define            CMND_OPT_WITH        0X4000000
#define            CMND_OPT_WITHOUT     0X8000000
#define            CMND_OPT_BODY           0X10000000
#define            CMND_OPT_RETCOD         0X20000000
#define        CMND_OPT_HEADER         0X40000000

    /* */
    /*    This field signifies whether or not there has been an error in the */
    /*    current command. */
    /* */
    Boolean                 error_in_commandn;

    /* */
    /*    This field is used during the parsing of an array to hold a pointer to */
    /*    the current array block, this block maintains information relating to the */
    /*    current array and the progress in parsing it. */
    /* */
    struct array_blk        *array_blk_ptrps;
    /* */
    /* */
    /* */
    /*    This field holds the LL position of the item before the last parsed */
    /*    PRIMARY operand. A PRIMARY operand is the base variable for the */
    /*    command being processed that is before any array indecis have been */
    /*    parsed. */
    /* */
    char                    *primary_operandpc;
    /* */
    /*    This field maintains a count of the number of array introducers ('[') */
    /*    parsed. By incrementing this after parsing a '[' and decrementing when we */
    /*    parse a ']' and zeroing it for safety at the end of a command, we can */
    /*    identify when we are inside an array reference and when not. This allows */
    /*    identification of a PRIMARY operand as distinct from array operands. */
    /* */
    PSL_U_LONGWORD_T        array_levell;
    /* */

    /* */
    /*    This field defines the next free offset for shareable variables. */
    /* */
    PSL_U_LONGWORD_T        shrvar_data_offl;
    /* */
    /*    Address mode of record */
    /* */
    PSL_U_LONGWORD_T        rec_addrl;
    /* */
    /*    Shareable record offset */
    /* */
    PSL_U_LONGWORD_T        shrvar_rec_offl;
    /* */
    /*    Set at setting of record scope */
    /* */
    PSL_U_LONGWORD_T        in_rec_headl;
    /* */
    /*    Start of switch statement */
    /* */
    PSL_U_LONGWORD_T        sw_startl;
    /* */
    /*    SWITCH Parsing data */
    /*    =================== */
    /* */
    /* */
    /*    The SWITCH STACK */
    /* */
    struct swstack          *swstack_begps;
    /* */
    /*    Count of current switch depth */
    /* */
    PSL_U_LONGWORD_T        sw_depthl;
    /* */
    /*     CASE Parsing Data */
    /*     ================= */
    /* */
    /* */
    /*    Count of case number */
    /* */
    PSL_U_LONGWORD_T        case_countl;
    /* */
    /*    Offset of next case */
    /* */
    PSL_U_LONGWORD_T        next_casel;
    /* */
    /*    Operand LL temp holder */
    /* */
    struct psl_vm_llb       *parsed_op_templlps;
    /* */
    /*    Temp break address */
    /* */
    char                    *temp_addrpc;
    /* */
    /* */
    /* */
    /*    Pipe Access Symbol Table */
    /*    ======================== */
    /* */
    /* */
    /*    Pipe access table pointer */
    /* */
    struct fattbl           *pipe_begps;
    /* */
    /*    Pipes count */
    /* */
    PSL_U_LONGWORD_T        pipe_cntl;
    /* */
    /*    This longword is used to store options for various pipe (or mutex */
    /*    or semaphore) commands. */
    /* */
    PSL_U_LONGWORD_T        pipe_flagsl;
    /* */
    /*    The bits are interpreted as follows: */
    /* */
    /*    PIPFLG_SYNCHRONOUS */
    /*              Clear = Asynchronous pipe (Default) */
    /*              Set   = Synchronous pipe */
    /*    PIPFLG_TESTWIDE */
    /*              Clear = Local pipe (Default) */
    /*              Set   = Test-wide pipe */
    /*    PIPFLG_SYSNAME */
    /*              Clear = Pipe name was specified (Default) */
    /*              Set   = Pipe name to be determined by system */
    /* */
    /* */

    /* */
    /* */
    /*    Flag to indicate that we have just parsed a break statement. */
    /* */
    PSL_U_LONGWORD_T        break_fgl;
    /* */
    /*    Flag to indicate error in switch command */
    /* */
    PSL_U_LONGWORD_T        sw_error_fgl;
    /* */
    /*    Flag to indicate error in do command */
    /* */
    PSL_U_LONGWORD_T        do_error_fgl;
    /* */
    /*    Flag to indicate 'not' in if command */
    /* */
    PSL_U_LONGWORD_T        not_flagl;
    /* */
    /*    Flag to indicate in FORMAT command */
    /* */
    PSL_U_LONGWORD_T        format_flagl;
    /* */
    /*    Count of arguments in FORMAT command */
    /* */
    PSL_U_LONGWORD_T        format_arguml;
    /* */
    /*    Flag to indicate previous character in Character value */
    /*    was a "%" the start of a format identifier. */
    /* */
    PSL_U_LONGWORD_T        format_char_setl;
    /* */
    /* */
    /*    Flags to track state of test-case commands. */
    /* */
    PSL_U_LONGWORD_T        testcase_flagsl;
#define            TCAS_ACTIVE_TC 0X1
#define            TCAS_PASS_TC 0X2
#define            TCAS_FAIL_TC 0X4
#define            TCAS_GOTO_TC 0X8
    /* */
    /* */
    /*   Record count */
    /* */
    PSL_U_LONGWORD_T        rec_cntl;
    /* */
    /*    The DO Stack */
    /* */
    struct rectbl           *rectbl_begps;
    /* */
    /*    The source file index */
    /* */
    PSL_U_LONGWORD_T        src_fil_indxl;
    /* */
    /*   Define a count of the number of subroutine parameters */
    /*   in a parameter list. */
    /* */
    PSL_U_LONGWORD_T        subprm_cntl;
    /* */
    /*    The enable flag */
    /* */
    PSL_U_LONGWORD_T        enabledl;
    /* */
    /*    The unknown string flag */
    /* */
    PSL_U_LONGWORD_T        unknown_strl;
    /* */
    /*    File offsets array */
    /* */
    PSL_U_LONGWORD_T        dbg_file_offsetl12[12];
    /* */
    /* */
    /* */
    /* */
    /*    Pointer to file data type buffer */
    /* */
    PSL_LONGWORD_T          *file_dtyppl;
    /* */
    /*   Flag to indicate error in record definition */
    /* */
    PSL_U_LONGWORD_T        rec_error_flgl;
    /* */
    /*   Protection Mask */
    /* */
    PSL_LONGWORD_T          prot_catl;
    /* */
    /* */
    /* SCL records and alignment variables */
    /* */
    PSL_U_LONGWORD_T        shrvar_align_offl;
    PSL_U_LONGWORD_T        script_align_offl;
    PSL_U_LONGWORD_T        local_align_offl;
    /*
     *    The Shareable Variable table.
     */
    struct shrvartbl        *last_shrvar_recps;
    /* */
    /* */
    /* */
    /* */
    /*  Line reference number for listing file */
    /* */
    PSL_U_LONGWORD_T        line_ref_numberl;
    /* */
    /*    Flag to indicate that we have just parsed a PASS/FAIL test-case. */
    /* */
    PSL_U_LONGWORD_T        test_fgl;
    /* */
    /*    Flag to indicate that we may have unreachable command. */
    /* */
    PSL_U_LONGWORD_T        test_case_warnl;
    /* */
    /*    Flag to indicate that we are parsing a call script command. */
    /* */
    PSL_U_LONGWORD_T        script_flagl;
    /* */
    /*    This field defines whether or not we are allowing duplicate definitions */
    /*    of timers/files/pipes/variables to be ignored or not. It is usually only */
    /*    set when we open a ".inc_sql' include file. */
    /* */
    Boolean                 ignore_duplicatesn;
    /* */
    /*    This field holds a copy of the PRIMARY_OPERANDPC field. It is used in */
    /*    situations where we need to reset the primary operand after parsing a */
    /*    clause. For example after parsing an EXTRACT function clause we reset */
    /*    the primary operand to be the EXTRACT operand itself. */
    /* */
    char                    *save_primary_operandpc;
    /* */
    /*    This field is used to track the status of SQL mode scripts. */
    /* */
    Boolean                 in_sql_blockn;
    /* */
    /*    This field is used to warn of SQL commands outside of an SQL block. */
    /* */
    Boolean                 sql_warnn;
    /* */
    /*    SQL DAtabase type. */
    /* */
    PSL_LONGWORD_T          sql_database_typel;
    /* */
    /*    This flag indicates which section of the script is currently */
    /*    being parsed. */
    /* */
    PSL_U_LONGWORD_T        sclsec_flagl;
#define            SCLSEC_NONE 0    /* No script section */
#define            SCLSEC_ENV 1 /* Environment section */
#define            SCLSEC_DEF 2 /* Definition section */
#define            SCLSEC_COD 3 /* Code section */

    /*    Pointer to thread definition work structure. */
    /* */
    struct thrtbl           *def_thr_wrkps;
    /* */
    /*  Specified Options Flags. */
    /* */
    /*  This is used when parsing thread and test definitions to remember */
    /*  which options have been specified. */
    /* */
    Boolean                 opt_flagn64[64];
    /*
     *  Array containing the source line number on which the last occurance of
     *  each option appeared. Used in error reporting.
     */
    PSL_U_LONGWORD_T    opt_linel64[64];

#define            SCL_OPT_MAX 64
    /* */
    /*  Subscripts for Test definition options. */
    /* */
#define            OPT_TST_ABTONF 0
#define            OPT_TST_ABTTST 1
#define            OPT_TST_CMPTST 2
#define            OPT_TST_CNCTRM 3
#define            OPT_TST_DSC 4
#define            OPT_TST_DMPEXC 5
#define            OPT_TST_FAILTC 6
#define            OPT_TST_HCDEV 7
#define            OPT_TST_IGNINF 8
#define            OPT_TST_INPRAT 9
#define            OPT_TST_PASSTC 10
#define            OPT_TST_WKSQ 11
    /*
     *  Subscripts for Test Thread definition options.
     */
    /* No options */

    /* */
    /*  Subscripts for Thread definition options. */
    /* */
#define            OPT_THR_AUTSYN 0
#define            OPT_THR_SCRIPT 1
#define            OPT_THR_TRMTYP 2
#define            OPT_THR_DEVTYP 3
#define            OPT_THR_CHRSIZ 4
#define            OPT_THR_LOGIN 5
#define            OPT_THR_USRNAM 6
#define            OPT_THR_NUMBER 7
#define            OPT_THR_START 8
#define            OPT_THR_INT 9
#define            OPT_THR_DETEXT 10
#define            OPT_THR_HNGDET 11
#define            OPT_THR_PHYDEV 12
#define            OPT_THR_SRV 13
#define            OPT_THR_NODE 14
#define            OPT_THR_TRMID 15
#define            OPT_THR_WIDTH 16
#define            OPT_THR_PAGE 17
#define            OPT_THR_BAUD 18
#define            OPT_THR_TYPAHD 19
#define            OPT_THR_TRC 20
#define            OPT_THR_TRCFIL 21
#define            OPT_THR_RECORD 22
#define            OPT_THR_RECFIL 23
#define            OPT_THR_RECOPT 24
#define            OPT_THR_RECSTP 25
#define            OPT_THR_INPTIM 26
#define            OPT_THR_ANSWER 27
#define            OPT_THR_FKEY 28
#define            OPT_THR_INET 29
#define            OPT_THR_IOM 30
#define            OPT_THR_TRMNAM 31
#define            OPT_THR_PORT 32
#define            OPT_THR_SOCKET 33
#define            OPT_THR_EMULTYP 34
#define            OPT_THR_EMULMOD 35
#define            OPT_THR_TRMINF 36

    /* */
    /*    Pointer to thread parameter definition work structure. */
    /* */
    struct tdp              *def_tdp_wrkps;
    /* */
    /*    Pointer to test definition work structure. */
    /* */
    struct tsttbl           *def_tst_wrkps;
    /*
     *    Pointer to data for this test file.
     */
    struct tsttbl       *tst_thisps;
    /*
     *    Identifies the command option being processed - if any.
     *    This is set at the start of option clause parsing and
     *    cleared whenever an operand is saved to the parsed operand
     *    linked list. The purpose of this field is to identify the
     *    option clause (if any) to which each operand in the
     *    parsed operand linked list relates.
     *    Uses the same mask values as CMND_OPT.
     */
    PSL_U_LONGWORD_T        this_cmnd_optl;
    /*
     *    Temporary storage for command option being processed - if any.
     *    This field is used in conjunction with the above field. It
     *    is used in parsing option clauses that may have multiple
     *    operands, e.g. the HTTP-specific with, without and header
     *    fields, which may take a list of values.
     *
     *    You may ask why don't we just use the above field - it would
     *    be simpler to reset this field before parsing a command
     *    and 
     *    Uses the same mask values as CMND_OPT.
     */
    PSL_U_LONGWORD_T        tmp_cmnd_optl;
    /* */
    /*    TEST Symbol Table */
    /*    ================= */
    /* */
    /*    TEST Symbol Table pointer. */
    /* */
    struct tsttbl           *tst_begps;
    /* */
    /*    TEST count. */
    /* */
    PSL_U_LONGWORD_T        tst_cntl;
    /* */
    /*   The following item is used in test name parsing to hold the address of */
    /*   a test's Symbol Table entry. */
    /* */
    struct tsttbl           *tst_ptrps;
    /* */
    /*    TEST THREAD Symbol Table */
    /*    ======================== */
    /* */
    /*    TEST THREAD Symbol Table pointer. */
    /* */
    struct tstthrtbl        *tstthr_begps;
    /* */
    /*    TEST THREAD count. */
    /* */
    PSL_U_LONGWORD_T        tstthr_cntl;
    /* */
    /*   The following item is used in test thread name parsing to hold the */
    /*   address of a test thread's Symbol Table entry. */
    /* */
    struct tstthrtbl        *tstthr_ptrps;
    /* */
    /*    THREAD Symbol Table */
    /*    =================== */
    /* */
    /*    THREAD Symbol Table pointer. */
    /* */
    struct thrtbl           *thr_begps;
    /* */
    /*    THREAD count. */
    /* */
    PSL_U_LONGWORD_T        thr_cntl;
    /* */
    /*   The following item is used in thread name parsing to hold the address of */
    /*   a thread's Symbol Table entry. */
    /* */
    struct thrtbl           *thr_ptrps;
    /* */
    /*    Pointer to test thread symbol table entry definition work structure. */
    /* */
    struct tstthrtbl        *def_tstthr_wrkps;
    /* */
    /*    Sub-test usage/parsing status mask. */
    /* */
    PSL_U_LONGWORD_T        subtst_flagsl;
    /*
     *    Pointer to structure holding data required in the script object
     *    ('tof') header record.
     */
    struct scl_tofh         *tofh_begps;

    /*
     *    Pointer to first entry in code section 'tof' line list.
     */
    struct psl_vm_llb   *tofl_llps;

    /*
     *    This field is used to hold a null-terminated sub-test name.
     */
    char                    subtst_namec20[20];
    /*
     *    Sub-test count.
     */
    PSL_U_LONGWORD_T        subtst_cntl;
    /*
     *    Number of subroutines DEFINED. This is NOT necessarily the same as the
     *    number of entries in the Subroutines symbol table as the latter may
     *    contain entries for subroutines that have been referenced (i.e. called)
     *    but not yet defined.
     */
    PSL_U_LONGWORD_T        subrtn_def_cntl;
    /*
     *    Pointer to test thread definition data work structure.
     */
    struct tthr             *def_tthr_wrkps;

    /*
     *    For commands generating HTTP requests, indicates the HTTP request type.
     *    (TOF_REQTYP_* symbols).
     */
    PSL_U_LONGWORD_T        http_reqtypl;

    /*
     *    For commands generating HTTP requests, contains HTTP request flags.
     *    This field is a bitmask.
     */
    PSL_U_LONGWORD_T        http_flagsl;
#define        HTTPFLG_PRIMARY  1

    /*
     *    For LOAD commands, indicates the load data (TOF_LODDAT_* or TOF_LLDDAT_*).
     */
    PSL_U_LONGWORD_T        loddatl;

    /*
     *    Script wait units - set from Environment section WAIT UNIT statement.
     */
    PSL_U_LONGWORD_T        wait_unitl;
#define        WAIT_SECONDS      0
#define        WAIT_MILLISECONDS 1

    /*
     *    For SCL logging commands (i.e. LOG, NOTE, HISTORY, REPORT and TRACE),
     *    indicates the log destination (LOG_*).
     */
    PSL_U_LONGWORD_T        logdstl;
#define            LOG_AUDIT    0
#define            LOG_REPORT   1
#define            LOG_HISTORY  2
#define            LOG_TRACE    3
#define            LOG_CENTRAL  4
#define            LOG_NOTE 5

    /*
     *    Count of values in the character list being parsed.
     */
    PSL_U_LONGWORD_T        char_list_val_cntl;
    /*
     *    For HTTP request commands, contains the number of values associated with
     *    the HEADER clause.
     */
    PSL_U_LONGWORD_T        http_hdrval_cntl;
    /*
     *    For HTTP request commands, contains the number of values associated with
     *    the WITH clause.
     */
    PSL_U_LONGWORD_T        http_withval_cntl;
    /*
     *    For HTTP request commands, contains the number of values associated with
     *    the WITHOUT clause.
     */
    PSL_U_LONGWORD_T        http_woutval_cntl;
} ;

/*
 *  Script Attributes
 *  =================
 *
 *  Define symbolic names for the maximum and minimum values of
 *  various script attributes and string lengths so that we
 *  don't have to hard-code them into action routines.
 */
#define            MIN_SLINE 24     /* SCREEN IMAGE DEPTH */
#define            MAX_SLINE 255
#define            MIN_SCOLS 1      /* SCREEN IMAGE WIDTH */
#define            MAX_SCOLS 255

/* */
/*  Test Attributes */
/*  =============== */
/* */
/*  Define symbolic names for the maximum and minimum values of */
/*  various test attributes and string lengths so that we */
/*  don't have to hard-code them into action routines. */
/* */
#define            MIN_TST_CNCTRM 1 /* CONCURRENT TERMINALS */
#define            MAX_TST_CNCTRM 32767
#define            LEN_TST_DSC 50   /* DESCRIPTION */
#define            LEN_TST_HCDEV 50 /* HARDCOPY DEVICE */
#define            LEN_TST_FAILTC 80    /* FAIL TEST-CASE */
#define            LEN_TST_PASSTC 80    /* PASS TEST-CASE */
#define            LEN_TST_ABTTST 80    /* ABORT TEST */
#define            LEN_TST_CMPTST 80    /* COMPLETE TEST */

#define        LEN_EXE_NODE   16    /* NODE NAME ON EXECUTE TEST/THREAD */

/* */
/*  Thread Attributes */
/*  ================= */
/* */
/*  Define symbolic names for the maximum and minimum values of */
/*  various thread attributes and string lengths so that we */
/*  don't have to hard-code them into action routines. */
/* */
#define            MIN_THR_INTERVAL 0   /* INTERVAL */
#define            MAX_THR_INTERVAL 32767
#define            MIN_THR_NUMBER 1 /* NUMBER */
#define            MAX_THR_NUMBER 65535
#define            MIN_THR_PAGE 1   /* PAGE */
#define            MAX_THR_PAGE 255
#define            MIN_THR_PORT 1   /* PORT */
#define            MAX_THR_PORT 65535
#define            MIN_THR_DELAY 0  /* STARTUP DELAY */
#define            MAX_THR_DELAY 32767
#define            MIN_THR_WIDTH 1  /* WIDTH */
#define            MAX_THR_WIDTH 255
#define            LEN_THR_INET 127 /* INTERNET NAME/ADDRESS */
#define            LEN_THR_NODE 6   /* NODE */
#define            LEN_THR_PARAM 50 /* SCRIPT PARAMETER */
#define            LEN_THR_PROMPT 50    /* SCRIPT PARAMETER PROMPT */
#define            LEN_THR_PHYDEV 32    /* PHYSICAL DEVICE */
#define            LEN_THR_RECFIL 50    /* RECORDING FILE */
#define            LEN_THR_SRV 16   /* SERVICE */
#define            LEN_THR_TRMINF 32    /* TERMINFO NAME */
#define            LEN_THR_TRCFIL 50    /* TRACING FILE */
#define            LEN_THR_USRNAM 12    /* USERNAME */
#define            MAX_THR_PARAM 8  /* Maximum number of script parameter */
/* */
/*    END OF DATA DECLARATIONS */
/* */
/**/
/*  End of C Include File */
/**/
#endif  /* PSL__SCL_TPACOM_LOADED */
