/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// CECAboutDlg.cpp : implementation file
//

#include "stdafx.h"
#include "CECCommander.h"
#include "CECAboutDlg.h"
#include "CECAboutPlusDlg.h"
#include "..\Common\CECRegistryTools.h"
#include "Solutions.h"
#include "LicenceDlg.h"
#include "htmlhelp.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#include "..\..\HTMLHelp\Commander\About.inl"

/////////////////////////////////////////////////////////////////////////////
// CECAboutDlg dialog


CECAboutDlg::CECAboutDlg(CWnd* pParent /*=NULL*/)
        : CDialog(CECAboutDlg::IDD, pParent)
{
    //{{AFX_DATA_INIT(CECAboutDlg)
    //}}AFX_DATA_INIT

    m_szHtmlHelpFile = getenv("CYRANO_HOME");
    m_szHtmlHelpFile += "\\Framework\\Help\\Commander.chm";

}


void CECAboutDlg::DoDataExchange(CDataExchange* pDX)
{
    CDialog::DoDataExchange(pDX);
    //{{AFX_DATA_MAP(CECAboutDlg)
    //}}AFX_DATA_MAP
}



BEGIN_MESSAGE_MAP(CECAboutDlg, CDialog)
//{{AFX_MSG_MAP(CECAboutDlg)
ON_BN_CLICKED(IDC_ADVANCED, OnAdvanced)
ON_WM_HELPINFO()
ON_BN_CLICKED(IDC_LICENCE, OnLicence)
ON_BN_CLICKED(IDC_FULL_HELP, OnFullHelp)
//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CECAboutDlg message handlers

BOOL CECAboutDlg::OnInitDialog()
{
    CDialog::OnInitDialog();

    CWaitCursor WaitCursor;

    HBITMAP     hBitMap;
    HINSTANCE   hInstance = AfxGetInstanceHandle ();
    hBitMap = LoadBitmap (hInstance ,MAKEINTRESOURCE (IDB_HELP));
    ASSERT (hBitMap != NULL);
    CButton* pBP = (CButton*)GetDlgItem(IDC_FULL_HELP);
    pBP->SetBitmap(hBitMap);

    //
    //    Find the build information
    //
    LoadBuildInfo();
    //
    //    And finally we centre the window
    //
    CenterWindow();

    return TRUE;  // return TRUE unless you set the focus to a control
    // EXCEPTION: OCX Property Pages should return FALSE
}


void CECAboutDlg::OnOK()
{
    // TODO: Add extra validation here

    CDialog::OnOK();
}

void CECAboutDlg::OnAdvanced()
{
    CECAboutPlusDlg AdvancedDlg;
    AdvancedDlg.DoModal();
}

//
//    LoadBuildInfo
//    =============
//
//    Pick up and display the build information for this executable.
//
void    CECAboutDlg::LoadBuildInfo ()
{
    //
    //     Retrieve the product version info to do this we first get the name of the file being executed
    //
    char strFileName[512];
    CString strProdVersionString;
    CString strFileVersionString;
    BOOL bVerSuccess = FALSE;
    ::GetModuleFileName(NULL, strFileName, sizeof(strFileName));

    //
    //    Get size of version info and allocate it
    //
    DWORD handle;
    DWORD size = ::GetFileVersionInfoSize(strFileName, &handle);
    void* info = malloc(size);              // Allocate buffer for it

    //
    // Retrieve file version info
    //
    if (::GetFileVersionInfo(strFileName, handle, size, info))
    {

        //
        //    Query for the base version information.
        //
        VS_FIXEDFILEINFO *fixedfileinfo;
        UINT versionlen;
        if (::VerQueryValue(info, "\\", (void**) &fixedfileinfo, &versionlen))
        {
            strFileVersionString.Format ("Build (%d.%d.%d.%d)   %s %s"
                                         ,fixedfileinfo->dwFileVersionMS/0x10000
                                         ,fixedfileinfo->dwFileVersionMS%0x10000
                                         ,fixedfileinfo->dwFileVersionLS/0x10000
                                         ,fixedfileinfo->dwFileVersionLS%0x10000
                                         ,__DATE__
                                         ,__TIME__);

            bVerSuccess = TRUE;
        } // errur during querying of the base version info
    } // error during retrieval of file version info


    if ( !bVerSuccess )
    {
        strFileVersionString.Format("Build date/time %s %s", __DATE__, __TIME__);
    }

    //
    //    Now display it
    //
    SetDlgItemText( IDC_BUILD, strFileVersionString);
}

BOOL CECAboutDlg::OnHelpInfo(HELPINFO* pHelpInfo)
{
    CString contextHelp = m_szHtmlHelpFile+"::whatisthis.txt";

    if (pHelpInfo->iCtrlId == IDC_STATIC)
        return true;

    CWnd* pWnd = GetDlgItem(pHelpInfo->iCtrlId);

    HtmlHelp(pWnd->m_hWnd, contextHelp, HH_TP_HELP_CONTEXTMENU, (DWORD)(LPVOID)g_TopicIdArray);

    return true;
}

void CECAboutDlg::OnLicence()
{
    CLicenceDlg dlg;
    dlg.DoModal();
}

void CECAboutDlg::OnFullHelp()
{
    HtmlHelp(m_hWnd, m_szHtmlHelpFile, HH_DISPLAY_TOPIC, (DWORD)"commande.htm");
}
