/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

/////////////////////////////////////////////////////////////////////////////
// @doc
//
// @module CECOutputWnd.h
//
// Definition file for the <c CECOutputWnd> Class
// Base class for the Output Window
//
/////////////////////////////////////////////////////////////////////////////

#if !defined CECOUTPUTWND_H
#define CECOUTPUTWND_H

class CECOutputWnd : public CWnd
{
    // Construction
public:
    CECOutputWnd();
    CECOutputWnd(const CString& szName, long hWnd);

    // Attributes
public:
    void Output(const CString& szBuffer);


    // Readers + Writers
    void    SetName(const CString szName)
    {
        m_szName = szName;
    }
    CString GetName()
    {
        return m_szName;
    }

    void    SetWindowHandle(long hWnd)
    {
        m_hWindow = hWnd;
    }
    long    GetWindowHandle()
    {
        return m_hWindow;
    }

    CRichEditCtrl*  m_pEditBox;

    // Operations
public:

    // Overrides
    // ClassWizard generated virtual function overrides
    //{{AFX_VIRTUAL(CECOutputWnd)
protected:
    virtual BOOL OnNotify(WPARAM wParam, LPARAM lParam, LRESULT* pResult);
    //}}AFX_VIRTUAL

    // Implementation
public:
    int GetIndex();
    void SetIndex(int nIndex);
    void Clear();
    void Activate();
    virtual ~CECOutputWnd();

    // Generated message map functions
protected:
    //{{AFX_MSG(CECOutputWnd)
    afx_msg BOOL OnEraseBkgnd(CDC* pDC);
    afx_msg int OnCreate(LPCREATESTRUCT lpCreateStruct);
    afx_msg void OnSize(UINT nType, int cx, int cy);
    afx_msg void OnHScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar);
    //}}AFX_MSG
    DECLARE_MESSAGE_MAP()

private:
    int m_nIndex;
    void OnEditBoxLButtonDblClk();
    void    FormatOutput    (CString& Buffer);

    // Member Variables
    CString m_szName;
    long    m_hWindow;
    CFont       m_Font;

    int m_nFontSize;
    int m_nNumVisibleLines;
};

#endif

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

// @doc
//
// @module TODO_FILENAME Cyrano Test TODO_VER |
//
// Implementation file for the <c TODO_CLASSNAME> Class
// TODO_DESCRIPTION
//

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

#if !defined CECOUTPUTVIEW_H
#define CECOUTPUTVIEW_H

class CECOutputView : public CBCGSizingControlBar
{
    // Construction
public:
    DECLARE_DYNAMIC(CECOutputView);

    CECOutputView();

    CECOutputWnd    m_wndOutput;

    // Attributes
protected:
    CBCGTabWnd  m_wndTab;
    // Operations
public:

    // Overrides
    // ClassWizard generated virtual function overrides


    //{{AFX_VIRTUAL(CECOutputView)
    //}}AFX_VIRTUAL

    // Implementation
public:
    void ActivateTab(CWnd* pTabWnd);
    virtual ~CECOutputView();
    virtual void OnUpdateCmdUI  (CFrameWnd* pTarget, BOOL bDisableIfNoHndler);
    void    InsertTab(CECOutputWnd* pOutWnd);

    // Generated message map functions
protected:
    //{{AFX_MSG(CECOutputView)
    afx_msg int OnCreate(LPCREATESTRUCT lpCreateStruct);
    afx_msg void OnSize(UINT nType, int cx, int cy);
    afx_msg BOOL OnEraseBkgnd(CDC* pDC);
    //}}AFX_MSG
    DECLARE_MESSAGE_MAP()
private:
};

#endif
/////////////////////////////////////////////////////////////////////////////
