/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// CECPluginFrame.cpp : implementation of the CECPluginFrame class
//

#include "stdafx.h"
#include "CECCommander.h"

#include "CECPluginFrame.h"
#include "../Common/CECStyleFinder.h"
#include "CECProjectNewDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CECPluginFrame

IMPLEMENT_DYNAMIC(CECPluginFrame, CBCGMDIFrameWnd)

const int  iMaxUserToolbars     = 10;
const UINT uiFirstUserToolBarId = AFX_IDW_CONTROLBAR_FIRST + 40;
const UINT uiLastUserToolBarId  = uiFirstUserToolBarId + iMaxUserToolbars - 1;

BEGIN_MESSAGE_MAP(CECPluginFrame, CBCGMDIFrameWnd)
//{{AFX_MSG_MAP(CECPluginFrame)
ON_WM_CREATE()
ON_COMMAND(ID_PROJECT_NEW, OnProjectNew)
ON_WM_CLOSE()
ON_WM_SYSCOLORCHANGE()
//}}AFX_MSG_MAP
ON_COMMAND(ID_VIEW_CUSTOMIZE, OnViewCustomize)
ON_REGISTERED_MESSAGE(BCGM_RESETTOOLBAR, OnToolbarReset)
ON_REGISTERED_MESSAGE(BCGM_TOOLBARMENU, OnToolbarContextMenu)
ON_COMMAND_EX_RANGE(ID_VIEW_USER_TOOLBAR1, ID_VIEW_USER_TOOLBAR10, OnToolsViewUserToolbar)
ON_UPDATE_COMMAND_UI_RANGE(ID_VIEW_USER_TOOLBAR1, ID_VIEW_USER_TOOLBAR10, OnUpdateToolsViewUserToolbar)
ON_UPDATE_COMMAND_UI_RANGE(ID_BUTTON_START, ID_BUTTON_END, OnUpdatePlugInUI)
ON_COMMAND_RANGE(ID_BUTTON_START, ID_BUTTON_END, OnPlugIn)
END_MESSAGE_MAP()

static UINT indicators[] =
    {
        ID_SEPARATOR,           // status line indicator
        ID_INDICATOR_CAPS,
        ID_INDICATOR_NUM,
        ID_INDICATOR_SCRL,
    };

/////////////////////////////////////////////////////////////////////////////
// CECPluginFrame construction/destruction

CECPluginFrame::CECPluginFrame()
{
    register i = 0;

    // Initialize our resoure arrays which are going to be plugged in
    for (i=0; i<MAX_PLUGINS;        i++)    m_pResourceInfo[i]      = NULL;
    for (i=0; i<MAX_DOCKINGWINDOWS; i++)    m_pWindowInfo[i]        = NULL;
    for (i=0; i<MAX_OUTPUTVIEWS;    i++)    m_pOutputInfo[i]        = NULL;
    for (i=0; i<MAX_DOCUMENTS;      i++)    m_pDocInfo[i]           = NULL;
    for (i=0; i<MAX_WIZARDS;        i++)    m_pWizardInfo[i]        = NULL;
    m_pWindow = NULL;

    //  m_pControlBarManager = new SECToolBarManager(this);

    // Enable bitmap to be displayed in the standard menu
    //  EnableBmpMenus();

    CECCommanderApp *pApp = (CECCommanderApp*)AfxGetApp();
    ASSERT (pApp != NULL);

    m_pTrace = pApp->GetTrace();
    m_pUserImages = NULL;
}

CECPluginFrame::~CECPluginFrame()
{
    for(int n=0;n<m_arpMenuItemBitmaps.GetSize();n++)
        delete m_arpMenuItemBitmaps[n];
    m_arpMenuItemBitmaps.RemoveAll();

    delete m_pUserImages;

    int nNdx=0;
    while(m_pResourceInfo[nNdx])
    {
        delete m_pResourceInfo[nNdx];
        m_pResourceInfo[nNdx++] = NULL;
    }

    nNdx=0;
    while(m_pWindowInfo[nNdx])
    {
        delete m_pWindowInfo[nNdx];
        m_pWindowInfo[nNdx++] = NULL;
    }

    nNdx=0;
    while(m_pWizardInfo[nNdx])
    {
        delete m_pWizardInfo[nNdx];
        m_pWizardInfo[nNdx++] = NULL;
    }

    nNdx=0;
    while(m_pOutputInfo[nNdx])
    {
        delete m_pOutputInfo[nNdx];
        m_pOutputInfo[nNdx++] = NULL;
    }
}

int CECPluginFrame::OnCreate(LPCREATESTRUCT lpCreateStruct)
{
    if (CBCGMDIFrameWnd::OnCreate(lpCreateStruct) == -1)
        return -1;

    CBCGToolBar::EnableQuickCustomization ();

    //  CBCGToolBar::SetLook2000 ();

    // TODO: Define your own basic commands. Be sure, that each pulldown
    // menu have at least one basic command.

    m_lstBasicCommands.AddTail (ID_VIEW_TOOLBARS);
    m_lstBasicCommands.AddTail (ID_PROJECT_NEW);
    m_lstBasicCommands.AddTail (ID_FILE_OPENPROJECT);
    m_lstBasicCommands.AddTail (ID_FILE_CLOSEPROJECT);
    m_lstBasicCommands.AddTail (ID_PROJECT_DELETE);
    m_lstBasicCommands.AddTail (ID_FILE_PRINT);
    m_lstBasicCommands.AddTail (ID_APP_EXIT);
    m_lstBasicCommands.AddTail (ID_EDIT_CUT);
    m_lstBasicCommands.AddTail (ID_EDIT_PASTE);
    m_lstBasicCommands.AddTail (ID_EDIT_UNDO);
    m_lstBasicCommands.AddTail (ID_APP_ABOUT);
    m_lstBasicCommands.AddTail (ID_VIEW_TOOLBAR);
    m_lstBasicCommands.AddTail (ID_VIEW_CUSTOMIZE);
    m_lstBasicCommands.AddTail (ID_WINDOW_CASCADE);
    m_lstBasicCommands.AddTail (ID_WINDOW_TILE_VERT);
    m_lstBasicCommands.AddTail (ID_WINDOW_TILE_HORZ);
    m_lstBasicCommands.AddTail (ID_HELP_CONTENTS);
    m_lstBasicCommands.AddTail (ID_HELP_INDEX);
    m_lstBasicCommands.AddTail (ID_HELP_SEARCH);
    m_lstBasicCommands.AddTail (ID_HELP_CYRANOONTHEWEB);

    if (!m_wndMenuBar.Create (this))
    {
        TRACE0("Failed to create menubar\n");
        return -1;      // fail to create
    }


    m_wndMenuBar.SetBarStyle(m_wndMenuBar.GetBarStyle() | CBRS_SIZE_DYNAMIC);

    if (!m_wndStatusBar.Create(this) ||
            !m_wndStatusBar.SetIndicators(indicators,
                                          sizeof(indicators)/sizeof(UINT)))
    {
        TRACE0("Failed to create status bar\n");
        return -1;      // fail to create
    }

    UINT nID, nStyle;
    int cxWidth;

    m_wndStatusBar.GetPaneInfo(1, nID, nStyle, cxWidth);
    m_wndStatusBar.SetPaneInfo(1, nID, nStyle | SBT_OWNERDRAW, cxWidth/2);


    // TODO: Delete these three lines if you don't want the toolbar to
    //  be dockable
    m_wndMenuBar.EnableDocking(CBRS_ALIGN_ANY);
    EnableDocking(CBRS_ALIGN_ANY);
    DockControlBar(&m_wndMenuBar);


    // Allow user-defined toolbars operations:
    InitUserToobars (NULL,
                     uiFirstUserToolBarId,
                     uiLastUserToolBarId);
    return 0;
}

BOOL CECPluginFrame::PreCreateWindow(CREATESTRUCT& cs)
{
    if( !CBCGMDIFrameWnd::PreCreateWindow(cs) )
        return FALSE;
    // TODO: Modify the Window class or styles here by modifying
    //  the CREATESTRUCT cs

    return TRUE;
}

/////////////////////////////////////////////////////////////////////////////
// CECPluginFrame diagnostics

#ifdef _DEBUG
void CECPluginFrame::AssertValid() const
{
    CBCGMDIFrameWnd::AssertValid();
}

void CECPluginFrame::Dump(CDumpContext& dc) const
{
    CBCGMDIFrameWnd::Dump(dc);
}

#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CECPluginFrame message handlers


void CECPluginFrame::OnViewCustomize()
{
    //------------------------------------
    // Create a customize toolbars dialog:
    //------------------------------------
    CBCGToolbarCustomize* pDlgCust = new CBCGToolbarCustomize (this,
                                     TRUE /* Automatic menus scaning */,
                                     BCGCUSTOMIZE_MENU_SHADOWS | BCGCUSTOMIZE_TEXT_LABELS |
                                     BCGCUSTOMIZE_LOOK_2000 | BCGCUSTOMIZE_MENU_ANIMATIONS);

    pDlgCust->EnableUserDefinedToolbars ();
    pDlgCust->Create ();
}

LRESULT CECPluginFrame::OnToolbarContextMenu(WPARAM,LPARAM lp)
{
    CPoint point (LOWORD (lp), HIWORD(lp));

    CMenu menu;
    VERIFY(menu.LoadMenu (IDR_POPUP_TOOLBAR));

    CMenu* pPopup = menu.GetSubMenu(0);
    ASSERT(pPopup != NULL);

    SetupToolbarMenu (*pPopup, ID_VIEW_USER_TOOLBAR1, ID_VIEW_USER_TOOLBAR10);

    CBCGPopupMenu* pPopupMenu = new CBCGPopupMenu;
    pPopupMenu->Create (this, point.x, point.y, pPopup->Detach ());

    return 0;
}

void CECPluginFrame::OnToolsViewUserToolbar (UINT uiId)
{
    CBCGToolBar* pUserToolBar = GetUserBarByIndex (uiId - ID_VIEW_USER_TOOLBAR1);
    if (pUserToolBar == NULL)
    {
        ASSERT (FALSE);
        return;
    }

    ShowControlBar (pUserToolBar, !(pUserToolBar->GetStyle () & WS_VISIBLE), FALSE);
    RecalcLayout ();
}

void CECPluginFrame::OnUpdateToolsViewUserToolbar (CCmdUI* pCmdUI)
{
    CBCGToolBar* pUserToolBar = GetUserBarByIndex (pCmdUI->m_nID - ID_VIEW_USER_TOOLBAR1);
    if (pUserToolBar == NULL)
    {
        pCmdUI->Enable (FALSE);
        return;
    }

    pCmdUI->Enable ();
    pCmdUI->SetCheck (pUserToolBar->GetStyle () & WS_VISIBLE);
}

afx_msg LRESULT CECPluginFrame::OnToolbarReset(WPARAM wp, LPARAM)
{
    UINT uiToolBarId = (UINT) wp;
    if (uiToolBarId == IDR_MAINFRAME)
    {
        AddMenus();
    }

    return 0;
}

BOOL CECPluginFrame::OnShowPopupMenu (CBCGPopupMenu* pMenuPopup)
{
    //---------------------------------------------------------
    // Replace ID_VIEW_TOOLBARS menu item to the toolbars list:
    //---------------------------------------------------------
    CBCGMDIFrameWnd::OnShowPopupMenu (pMenuPopup);

    if (pMenuPopup != NULL &&
            pMenuPopup->GetMenuBar ()->CommandToIndex (ID_VIEW_TOOLBARS) >= 0)
    {
        if (CBCGToolBar::IsCustomizeMode ())
        {
            //----------------------------------------------------
            // Don't show toolbars list in the cuztomization mode!
            //----------------------------------------------------
            return FALSE;
        }

        pMenuPopup->RemoveAllItems ();

        CMenu menu;
        VERIFY(menu.LoadMenu (IDR_POPUP_TOOLBAR));

        CMenu* pPopup = menu.GetSubMenu(0);
        ASSERT(pPopup != NULL);

        SetupToolbarMenu (*pPopup, ID_VIEW_USER_TOOLBAR1, ID_VIEW_USER_TOOLBAR10);
        pMenuPopup->GetMenuBar ()->ImportFromMenu (*pPopup, TRUE);
    }

    return TRUE;
}

//
// LoadResources
// =============
//
// This function loads in the resources from all plug-ins
//
void CECPluginFrame::LoadResources()
{
    if (m_pTrace != NULL) m_pTrace->Log(10, "CECPlugInFrame", "LoadResources", "Entering");

    HINSTANCE       hDLL;
    LPRESFUNC       lpfnGetResourceInfo;
    LPWINDOWFUNC    lpfnGetWindowInfo;
    LPOUTPUTFUNC    lpfnGetOutputInfo;
    LPDOCFUNC       lpfnGetDocumentInfo;
    LPWIZARDFUNC    lpfnGetWizardInfo;

    BOOL            bReturn;
    register nResourceCount = 0;        // Count of resources in the resource array

    //
    // Declare some local structs which recieve the info from each dll
    //
    struct RESOURCEINFO*    pResourceInfo[MAX_PLUGINS];
    struct WINDOWINFO*      pWindowInfo[MAX_DOCKINGWINDOWS];
    struct OUTPUTINFO*      pOutputInfo[MAX_OUTPUTVIEWS];
    struct DOCUMENTINFO*    pDocInfo[MAX_DOCUMENTS];
    struct WIZARDINFO*      pWizardInfo[MAX_WIZARDS];

    CECCommanderApp *pApp = (CECCommanderApp*)AfxGetApp();
    ASSERT (pApp != NULL);

    CECDLLFunctions* pDLLLoader = pApp->GetDLLLoader();
    ASSERT (pDLLLoader != NULL);

    CStringArray& arExtensionDLLs = pDLLLoader->GetExtensionDLLs();

    //
    // Before we add in the toolbars for the extension DLL's we will add in those for
    // the framework. This allows us to handle all toolbars in the same way.
    //
    // Search every component which has applied to add some resources
    //
    int n = arExtensionDLLs.GetSize();

    CString szTotalNumber;
    szTotalNumber.Format("%d",n);

    for (int i = 0; i < arExtensionDLLs.GetSize(); i++)
    {
        register j = 0;

        for (j=0; j<MAX_PLUGINS;        j++) pResourceInfo[j]   = NULL;
        for (j=0; j<MAX_DOCKINGWINDOWS; j++) pWindowInfo[j]     = NULL;
        for (j=0; j<MAX_OUTPUTVIEWS;    j++) pOutputInfo[j]     = NULL;
        for (j=0; j<MAX_DOCUMENTS;      j++) pDocInfo[j]        = NULL;
        for (j=0; j<MAX_WIZARDS;        j++) pWizardInfo[j]     = NULL;

        //
        // Get the name of the dll we are collecting information from
        //
        CString szDll = arExtensionDLLs.GetAt(i);

        CString szNumber;
        szNumber.Format("%d",i+1);
        if (m_pTrace != NULL) m_pTrace->Log(10, "CECPlugInFrame", "LoadResources", "Attempting to load <" + szDll + "> Plug-In, <" + szNumber + "> of <" + szTotalNumber + ">");
        //
        // Recover the resource handle of the plug-in DLL
        //
        hDLL = pApp->GetResourceHandle(i);

        lpfnGetResourceInfo = (LPRESFUNC) pDLLLoader->GetFunctionPtr("GetResourceInfo", szDll);
        if (lpfnGetResourceInfo)
        {

            //
            // And collect the resource info from this component
            //
            bReturn = lpfnGetResourceInfo(pResourceInfo);
            if (bReturn)
            {
                // Add the returned array to our current array
                register index = 0;
                while (pResourceInfo[index] != NULL)
                {
                    BOOL bRequired = TRUE;

                    //
                    //   First we check if these buttons are valid for this product type.
                    //   (The problem being that a product may not require all the
                    //    services of the Generic DLL as it may not be using a Server
                    //    or a project. To identify if a button/command is valid for this
                    //    environment we can check the button styles mask to see what it needs).
                    //
                    CECStyleFinder styles;
                    if (styles.NeedsProject(pResourceInfo[index]->dwStyles))
                    {
                        if (!pApp->AreWeUsingProjects())
                        {
                            bRequired = FALSE;
                        }
                    }

                    if (styles.NeedsServer(pResourceInfo[index]->dwStyles))
                    {
                        if (!pApp->AreWeUsingAServer())
                        {
                            bRequired = FALSE;
                        }
                    }

                    //
                    //    If we have decided that this resource is required then add it into our list of
                    //    resources.
                    if (bRequired)
                    {
                        m_pResourceInfo[nResourceCount + index] = pResourceInfo[index];

                        //
                        //    If the resource includes a toolbar bitmap then we shall now add it to the toolbar
                        //    resources from which the toolbars are created.
                        //
                        if (pResourceInfo[index]->strPreferredToolbar[0] != '\0')
                            AddBitmapResources (hDLL ,pResourceInfo[index]->nCommandID);

                        index++;
                    }
                    else
                    {
                        delete pResourceInfo[index];
                        pResourceInfo[index]=NULL;
                    }
                }

                if (nResourceCount)
                {
                    CString szIndex;
                    szIndex.Format("%d", nResourceCount);
                    if (m_pTrace != NULL) m_pTrace->Log(10, "CECPlugInFrame", "LoadResources", "ResourceInfo Supported: The <" + szDll + "> Plug-In will customize the Framework by the addition of up to <" + szIndex + "> buttons");
                }


                // And set up all the resources just added with the component index so
                // that we know which resource handle the button belongs
                SetCommandHandle(nResourceCount, i);
                nResourceCount += index;
            }
        }

        // Now get hold of the floor info
        lpfnGetWindowInfo   = (LPWINDOWFUNC) pDLLLoader->GetFunctionPtr("GetWindowInfo", szDll);
        if (lpfnGetWindowInfo)
        {
            // Count the number of floors added in so far
            register nWindowCount=0;
            while (m_pWindowInfo[nWindowCount] != NULL) nWindowCount++;

            // And collect the resource info from this component

            bReturn = lpfnGetWindowInfo(pWindowInfo);
            if (bReturn)
            {
                // Add the returned array to our current array
                register index = 0;
                while (pWindowInfo[index] != NULL)
                {
                    m_pWindowInfo[nWindowCount + index] = pWindowInfo[index];
                    index++;
                }

                if (index)
                {
                    CString szIndex;
                    szIndex.Format("%d", index);
                    if (m_pTrace != NULL) m_pTrace->Log(10, "CECPlugInFrame", "LoadResources", "WindowInfo Supported: The <" + szDll + "> Plug-In will customize the Framework by the addition of up to <" + szIndex + "> independent Docking Windows");
                }
            }
        }

        // And the output info
        lpfnGetOutputInfo   = (LPOUTPUTFUNC) pDLLLoader->GetFunctionPtr("GetOutputInfo", szDll);
        if (lpfnGetOutputInfo)
        {
            // Count the number of outputs added in so far
            register nOutputCount=0;
            while (m_pOutputInfo[nOutputCount] != NULL) nOutputCount++;

            // And collect the resource info from this component

            bReturn = lpfnGetOutputInfo(pOutputInfo);
            if (bReturn)
            {
                // Add the returned array to our current array
                register index = 0;
                while (pOutputInfo[index] != NULL)
                {
                    m_pOutputInfo[nOutputCount + index] = pOutputInfo[index];
                    index++;
                }

                if (index)
                {
                    CString szIndex;
                    szIndex.Format("%d", index);
                    if (m_pTrace != NULL) m_pTrace->Log(10, "CECPlugInFrame", "LoadResources", "OutputInfo Supported: The <" + szDll + "> Plug-In will customize the Framework by the addition of up to <" + szIndex + "> independent Output Windows");
                }
            }
        }

        // And the document info
        lpfnGetDocumentInfo = (LPDOCFUNC) pDLLLoader->GetFunctionPtr("GetDocumentInfo", szDll);
        if (lpfnGetDocumentInfo)
        {
            // Count the number of documents added in so far
            register nDocCount=0;
            while (m_pDocInfo[nDocCount] != NULL) nDocCount++;

            // And collect the resource info from this component

            bReturn = lpfnGetDocumentInfo(pDocInfo);
            if (bReturn)
            {
                // Add the returned array to our current array
                register index = 0;
                while (pDocInfo[index] != NULL)
                {
                    m_pDocInfo[nDocCount + index] = pDocInfo[index];
                    index++;
                }
                if (index)
                {
                    CString szIndex;
                    szIndex.Format("%d", index);
                    if (m_pTrace != NULL) m_pTrace->Log(10, "CECPlugInFrame", "LoadResources", "DocumentInfo Supported: The <" + szDll + "> Plug-In will customize the Framework by the addition of up to <" + szIndex + "> independent Document Templates");
                }
            }
        }

        // And the wizard info
        lpfnGetWizardInfo = (LPWIZARDFUNC) pDLLLoader->GetFunctionPtr("GetWizardInfo", szDll);
        if (lpfnGetWizardInfo)
        {
            // Count the number of wizards added in so far
            register nWizardCount=0;
            while (m_pWizardInfo[nWizardCount] != NULL) nWizardCount++;

            // And collect the resource info from this component

            bReturn = lpfnGetWizardInfo(pWizardInfo);
            if (bReturn)
            {
                // Add the returned array to our current array
                register index = 0;
                while (pWizardInfo[index] != NULL)
                {
                    strcpy(pWizardInfo[index]->strPlugin,szDll);
                    m_pWizardInfo[nWizardCount + index] = pWizardInfo[index];
                    index++;
                }
                if (index)
                {
                    CString szIndex;
                    szIndex.Format("%d", index);
                    if (m_pTrace != NULL) m_pTrace->Log(10, "CECPlugInFrame", "LoadResources", "WizardInfo Supported: The <" + szDll + "> Plug-In will customize the Framework by the addition of up to <" + szIndex + "> independent Wizards");
                }
            }
        }
    }
    if (m_pTrace != NULL) m_pTrace->Log(10, "CECPlugInFrame", "LoadResources", "Leaving");
}

//
//    AddBitmapResources
//
void CECPluginFrame::AddBitmapResources (HINSTANCE  hDLL ,UINT nButtonID)
{
    //    If no button to add then return
    if (nButtonID == 0) return;

    HBITMAP hbmp = (HBITMAP) ::LoadImage (
                       hDLL,
                       MAKEINTRESOURCE (nButtonID),
                       IMAGE_BITMAP,
                       0, 0,
                       LR_CREATEDIBSECTION | LR_LOADMAP3DCOLORS);

    if (hbmp == NULL)
        return ;

    BITMAP bmp;
    if (::GetObject (hbmp, sizeof (BITMAP), &bmp) == 0)
    {
        ::DeleteObject (hbmp);
        return;
    }

    if (m_pUserImages == NULL)
        m_pUserImages = new CBCGToolBarImages();

    _USER_BUTTON_MAP bm;
    bm.hDLL = hDLL;
    bm.buttonID = nButtonID;
    bm.imageIndex = m_pUserImages->AddImage (hbmp);
    ::DeleteObject (hbmp);

    CString strButton;
    strButton.LoadString(nButtonID);
    int nPos = strButton.Find('\n');
    int nLen = strButton.GetLength();
    bm.tooltip = strButton.Right(nPos - nLen -1);
    m_arButtonMap.Add(bm);

    return;
}

//
// SetButtonHandle
// ===============
//
// This function counts the number of buttons added by a call to GetResourceInfo and
// sets the correct handle to them. It also sets up all the Command Arrays we are likely
// to use.
//
//  m_arCommandDLLLinks = Link each Command ID to it's resource Handle
//  m_arCommandIDs      = Sequentially order the Command ID's
//  m_arDisplayState    = Set the Display state
//
void CECPluginFrame::SetCommandHandle(int nCommandCount, int nDLL)
{
    while (m_pResourceInfo[nCommandCount] != NULL)
    {
        m_arCommandDLLLinks.SetAtGrow(nCommandCount,nDLL);
        m_arCommandIDs.SetAtGrow(nCommandCount,m_pResourceInfo[nCommandCount]->nCommandID);
        m_arDisplayState.SetAtGrow(nCommandCount,1);

        nCommandCount++;
    }
}

//
// AddFloorWnds
// ============
//
// Function to add in all the Floor Wnds based on what is in our resource array
//
void  CECPluginFrame::AddFloorWnds()
{
    if (m_pTrace != NULL) m_pTrace->Log(10, "CECPlugInFrame", "AddFloorWnds", "Attempting to add Floor Windows");

    CECCommanderApp *pApp = (CECCommanderApp*)AfxGetApp();
    ASSERT(pApp != NULL);

    // Get the FloorWnd Array
    CPtrArray& FloorWndArray = pApp->GetFloorWndArray ();

    int ResourceIdx = 0;
    while (m_pWindowInfo[ResourceIdx] != NULL)
    {
        // And add in a new FloorWnd based for each entry
        BOOL bAddWindow = TRUE;

        // But ensure that we only add the Project Viewer if we are using them
        if (m_pWindowInfo[ResourceIdx]->nType == PROJECT_WINDOW)
        {
            if (!pApp->AreWeUsingProjects())
            {
                if (m_pTrace != NULL) m_pTrace->Log(10, "CECPlugInFrame", "AddFloorWnds", "Ignoring Floor Window for non Project Based Products");

                bAddWindow = FALSE;
            }
        }

        if (bAddWindow)
        {
            CECFloorWnd* pNewWindowWnd = new CECFloorWnd(m_pWindowInfo[ResourceIdx]->strName,
                                         m_pWindowInfo[ResourceIdx]->nType);
            FloorWndArray.Add(pNewWindowWnd);

            CString szName = m_pWindowInfo[ResourceIdx]->strName;
            if (m_pTrace != NULL) m_pTrace->Log(10, "CECPlugInFrame", "AddFloorWnds", "Added new Floor Window <" + szName + ">");
        }

        ResourceIdx++;
    }
}

//
// AddMenus
// ========
//
// Function to add in all the menu items requested
//
void  CECPluginFrame::AddMenus()
{

    if (m_pTrace != NULL) m_pTrace->Log(10, "CECPlugInFrame", "AddMenus", "Attempting to Add in the Plug-In Menus");
    CECCommanderApp *pApp = (CECCommanderApp*)AfxGetApp();
    ASSERT(pApp != NULL);

    CMenu MainMenu;
    VERIFY(MainMenu.LoadMenu (IDR_MAINFRAME));

    CMenu* pPreferredMenu = NULL;
    CMenu* pFileMenu = MainMenu.GetSubMenu(0);

    // Declare a resource handle and set it to the resources from the
    // snapin
    int ResourceIdx = 0;
    while (m_pResourceInfo[ResourceIdx] != NULL)
    {
        // Variable for the Main menu label, load into a CString
        CString szMainMenuItem  = m_pResourceInfo[ResourceIdx]->strPreferredMenu;
        UINT    nCommandID      = m_pResourceInfo[ResourceIdx]->nCommandID;

        // And if one was not even specified we should add to the studio's default name menu
        if (szMainMenuItem.IsEmpty())
            szMainMenuItem = pApp->GetCoreProductTitle();

        // Check if a new top level menu option is required, if so retrieve the
        // necessary information from the Resource Info
        pPreferredMenu = FindMenu(&MainMenu, szMainMenuItem, TRUE);
        if (pPreferredMenu == NULL)
        {
            return;
        }

        CString szNumber;
        szNumber.Format("%d", nCommandID);
        if (m_pTrace != NULL) m_pTrace->Log(10, "CECPlugInFrame", "AddMenus", "Attempting to Add in the menu Item <" + szNumber + "> to the menu <" + szMainMenuItem + ">");

        // And create the actual menu items themselves
        CreateMenuItem(pPreferredMenu, nCommandID);

        // If the item is defined to be a basic command add it to the list.
        if (m_pResourceInfo[ResourceIdx]->dwStyles & CYR_BASIC_CMD)
            m_lstBasicCommands.AddTail(nCommandID);

        ResourceIdx++;
    }

    if(!pApp->CanCreateProjects())
        MainMenu.DeleteMenu(ID_PROJECT_NEW,MF_BYCOMMAND);

    // Make Sure the "Exit" item is the last one on the File Menu
    pFileMenu->DeleteMenu(ID_APP_EXIT, MF_BYCOMMAND);
    pFileMenu->AppendMenu(MF_SEPARATOR);
    pFileMenu->AppendMenu(MF_STRING, ID_APP_EXIT, "E&xit");


    DrawMenuBar();

    CBCGMenuBar* pMainMenu = (CBCGMenuBar*)GetMenuBar();
    pMainMenu->CreateFromMenu(MainMenu.m_hMenu);

    // And deactivate any optional items if not present

    // This currently covers menu items applying to the viewing of
    // any optional windows i.e. the output and floor window
    DeactivateOptionalFeatures();

    return;
}

//
// FindMenu
// ========
//
// This function determines whether a menu item already exists. If it doesn't it will create it.
//
CMenu* CECPluginFrame::FindMenu(CMenu* pMainMenu, const CString& szPreferredMenu, BOOL bAndCreate)
{
    // Get a pointer to the main menu
    CMenu* pPreferredMenu = NULL;

    if (pMainMenu != NULL)
    {
        // Scan the menu for the preferred menu entry.

        // From this we can get the index of the menu item
        CString szMenuText;
        for (int MenuId = 0; MenuId < pMainMenu->GetMenuItemCount(); MenuId++)
        {
            pMainMenu->GetMenuString (MenuId, szMenuText, MF_BYPOSITION);
            if (szMenuText.CompareNoCase(szPreferredMenu) == 0)
            {
                pPreferredMenu = pMainMenu->GetSubMenu(MenuId);
                return pPreferredMenu;
            }
        }
    }

    if ((pPreferredMenu == NULL) && (bAndCreate))
    {
        if (m_pTrace != NULL) m_pTrace->Log(10, "CECPlugInFrame", "FindMenu", "Creating a new <" + szPreferredMenu + "> menu item");

        // OKay we need to create a new top level menu entry, so
        HMENU hNewMenu = ::CreateMenu();

        // And place it directly after the File Menu - ALWAYS AT 2
        pMainMenu->InsertMenu(2, MF_STRING | MF_POPUP | MF_ENABLED | MF_BYPOSITION, (UINT) hNewMenu, szPreferredMenu);

        return pMainMenu->GetSubMenu(2);
    }
    return NULL;
}

//
// CreateMenuItem
// ==============
//
// Function to add in a menu item (with or without a separator)
//
void  CECPluginFrame::CreateMenuItem(CMenu* pPreferredMenu, const UINT& nID)
{
    BOOL bExists = FALSE;
    ASSERT (pPreferredMenu != NULL);

    // Obtain the Associated Label and Separator for this nID
    CString szLabel = GetMenuLabel(nID);

    int nCount = pPreferredMenu->GetMenuItemCount();
    if (nCount != -1)
    {
        for (int i=0; i<nCount; i++)
        {
            if (nID == pPreferredMenu->GetMenuItemID(i))
            {
                if (m_pTrace != NULL) m_pTrace->Log(10, "CECPlugInFrame", "CreateMenuItem", "Ignoring attempt to add a duplicate menu label <" + szLabel + ">");
                bExists = TRUE;
                break;
            }
        }
    }

    if (!bExists)
    {
        // Add a Separater if asked for
        if (GetMenuSeparator(nID)) pPreferredMenu->AppendMenu(MF_SEPARATOR);

        // Then add the item to the selected menu
        pPreferredMenu->AppendMenu(MF_STRING, nID, szLabel);

        AddMenuBitmap(pPreferredMenu, nID);

        CString szNumber;
        szNumber.Format("%d", nID);
        if (m_pTrace != NULL) m_pTrace->Log(10, "CECPlugInFrame", "CreateMenuItem", "Adding the menu Item <" + szLabel + "> and associating it with the bitmap resource <" + szNumber + ">");
    }
    return;
}
//
// AddMenuBitmap
// =============
//
// Function to add a bitmap to the menu
//
void CECPluginFrame::AddMenuBitmap(CMenu* pMenu, const UINT& nCommandID)
{
    // Save the current resource ID
    HINSTANCE hOriginalID = AfxGetResourceHandle();

    UINT nBitmap = nCommandID;
    if (nBitmap != 0)
    {
        CString szBitmapID;
        szBitmapID.Format("#%d",nBitmap);
        HINSTANCE ResourceID = NULL;

        // Get a the handle to the resources contained in the plug-in
        ResourceID = GetOwner(nBitmap);
        AfxSetResourceHandle(ResourceID);

        // Check to see if the bitmap exists. If it does not there are
        // two possible causes,

        // a). This resource does not have a button associated with it and
        //     is only requiring a menu item or,
        // b). The button is incorrectly defined in the Plug In. We cannot
        //     hope to rectify this so we simply skip it as it is not our
        //     responsibility to do this.
        if (FindResource(ResourceID, szBitmapID, RT_BITMAP) != NULL)
        {
            CBitmap* pBitmap = new CBitmap();
            pBitmap->LoadBitmap(nCommandID);
            m_arpMenuItemBitmaps.Add(pBitmap);  //store pointer so the bitmap can be deleted later

            pMenu->SetMenuItemBitmaps(nCommandID, MF_BYCOMMAND, pBitmap, pBitmap);
        }
        else
        {
            if (m_pTrace != NULL) m_pTrace->Log(10, "CECPlugInFrame", "AddMenuBitmap", "Failed to locate the Bitmap Resource <" + szBitmapID + ">");
        }
        //
        // Reset to the original resource handle.
        AfxSetResourceHandle(hOriginalID);
    }
}

//
// GetWizardCount
// ==============
//
// Get the number of wizards added in by the plug ins
//
int CECPluginFrame::GetWizardCount()
{
    int nCount = 0;

    // Simply count the number of entries added to the wizard structure
    while (m_pWizardInfo[nCount] != NULL) nCount++;

    return nCount;
}


//
// DeactivateOptionalFeatures
// ==========================
//
// Although the Framework may allow up to three windows based on MSDEV - a product is
// not obliged to use them if not necessary. If this is the case we perform a check on
// the optional features to deactivate them
void CECPluginFrame::DeactivateOptionalFeatures()
{
    CECCommanderApp* pApp = (CECCommanderApp*) AfxGetApp();
    ASSERT (pApp != NULL);

    CPtrArray& FloorWndArray = pApp->GetFloorWndArray ();
    //  CPtrArray& OutputWndArray = pApp->GetOutputWndArray ();

    // And remove the Menu items from the View Menu

    // First the Floor Viewer
    if (FloorWndArray.GetSize() == 0)
    {
        DeleteMenuItem("&View", ID_VIEW_FLOORVIEWER);
        if (m_pTrace != NULL) m_pTrace->Log(10, "CECPlugInFrame", "DeactivateOptionalFeatures", "Deactivating the Floor Window as their is no requirement for it");
    }
    // Now the Output Viewer
    //  if (OutputWndArray.GetSize() == 0)
    //  {
    //      DeleteMenuItem("&View", ID_VIEW_OUTPUTWINDOW);
    //      if (m_pTrace != NULL) m_pTrace->Log(10, "CECPlugInFrame", "DeactivateOptionalFeatures", "Deactivating the Output Window as their is no requirement for it");
    //  }
}

//
// GetMenuLabel
// ============
//
// This function retrieves the associated menu label for a particular Command ID
//

CString CECPluginFrame::GetMenuLabel(UINT nID)
{
    CString szLabel;

    // Get the index of the ResourceInfo Struct to which this Command ID belongs
    int ResourceIdx = GetResourceIndex(nID);

    // If valid we obtain the relevant Resource Handle
    if ((ResourceIdx != -1) && (m_pResourceInfo[ResourceIdx] != NULL))
    {
        szLabel = m_pResourceInfo[ResourceIdx]->strLabel;
    }

    return szLabel;
}

//
// GetMenuSeparator
// ================
//
// This function finds whether a separator is required for a particular Command ID
//
BOOL CECPluginFrame::GetMenuSeparator(UINT nID)
{
    CECCommanderApp *pApp = (CECCommanderApp*)AfxGetApp();
    ASSERT (pApp != NULL);

    BOOL bSeparator = FALSE;

    // Get the index of the ResourceInfo Struct to which this Command ID belongs
    int ResourceIdx = GetResourceIndex(nID);

    // If valid we obtain the relevant Resource Handle
    if ((ResourceIdx != -1) && (m_pResourceInfo[ResourceIdx] != NULL))
    {
        CECStyleFinder styles;
        bSeparator = styles.NeedsSeparator(m_pResourceInfo[ResourceIdx]->dwStyles);
    }

    return bSeparator;
}

//
// GetOwner
// ========
//
// This function retrieves the resource handle of the DLL who
// owns the resource passed in
//
HINSTANCE CECPluginFrame::GetOwner(UINT nID)
{
    CECCommanderApp *pApp = (CECCommanderApp*)AfxGetApp();
    ASSERT (pApp != NULL);

    HINSTANCE hDLL = NULL;

    // Get the index of the ResourceInfo Struct to which this Command ID belongs
    int ResourceIdx = GetResourceIndex(nID);

    // If valid we obtain the relevant Resource Handle
    if ((ResourceIdx != -1) && (ResourceIdx < m_arCommandDLLLinks.GetSize()))
    {
        // And get the Resource handle associated with this entry
        int index = m_arCommandDLLLinks.GetAt(ResourceIdx);
        hDLL = pApp->GetResourceHandle(index);
    }
    return hDLL;
}

//
// DeleteMenuItem
// ==============
//
// Function to delete a menu item
//
BOOL CECPluginFrame::DeleteMenuItem(const CString& szLabel, const UINT& nCommandID)
{
    CMenu* pMainMenu = AfxGetMainWnd()->GetMenu();
    CMenu* pMenu = NULL;

    // Find the menu we wish to delete from
    pMenu = FindMenu(pMainMenu, szLabel, FALSE);
    if (pMenu == NULL)
    {
        if (m_pTrace != NULL) m_pTrace->Log(10, "CECPlugInFrame", "DeleteMenuItem", "Tried to delete the missing menu item <" + szLabel + ">");
        return FALSE;
    }

    // Simply delete the item to the selected menu
    BOOL bState = pMenu->DeleteMenu(nCommandID, MF_BYCOMMAND);

    // And Verify that the menu has not been screwed up by,
    // a). leaving two separators next to each other or
    // b). leaving a separator at the end of the menu itself
    ValidateMenu(pMenu);

    DrawMenuBar();

    VERIFY( SetMenu( pMainMenu ));

    return TRUE;
}

//
// GetResourceIndex
// ================
//
// This function retrieves the index of the ResourceInfo struct which
// added this command ID
//
int CECPluginFrame::GetResourceIndex(UINT nID)
{
    int index = -1;

    int ResourceIdx = 0;
    while (m_pResourceInfo[ResourceIdx] != NULL)
    {
        // Locate the index of the Resource Info which added this button
        if (m_pResourceInfo[ResourceIdx]->nCommandID == nID)
        {
            // And save the index if found
            index = ResourceIdx;
            break;
        }
        ResourceIdx++;
    }
    return index;
}

//
// ValidateMenu
// ============
//
// Validate a menu to ensure that we have not left two separators
// next to each other or a separator at the bottom of the menu
void CECPluginFrame::ValidateMenu(CMenu* pMenu)
{
    int nCount = pMenu->GetMenuItemCount();
    if (nCount != -1)
    {
        BOOL bSeparator = FALSE;

        for (int i=0; i<nCount; i++)
        {
            if (pMenu->GetMenuItemID(i) == 0)
            {
                if (bSeparator)
                {
                    // Whoops - this means that we have two separators next to each other
                    pMenu->DeleteMenu(i, MF_BYPOSITION);
                    i++;
                }
                bSeparator = TRUE;
            }
            else
            {
                bSeparator = FALSE;
            }
        }

        // Whoops - this means the last entry could be a separator
        if (pMenu->GetMenuItemID(pMenu->GetMenuItemCount()-1) == 0)
        {
            pMenu->DeleteMenu(pMenu->GetMenuItemCount()-1, MF_BYPOSITION);
        }
    }

    CString szNumber;
    szNumber.Format("%d", nCount);
    if (m_pTrace != NULL) m_pTrace->Log(10, "CECPlugInFrame", "ValidateMenu", "Menu <" + szNumber + "> has been validated");
}

//
// AddToolbars
// ===========
//
// This function adds in all the requested toolbars
//
void  CECPluginFrame::AddToolbars()
{
    CECCommanderApp *pApp = (CECCommanderApp*)AfxGetApp();
    ASSERT(pApp != NULL);
    CPtrArray&  ToolBarArray = pApp->GetToolBarArray ();

    if (m_pTrace != NULL) m_pTrace->Log(10, "CECPlugInFrame", "AddToolbars", "Adding Toolbars");


    // Standard Toolbar
    if (!m_wndToolBar.CreateEx(this, TBSTYLE_FLAT, WS_CHILD | WS_VISIBLE | CBRS_TOP
                               | CBRS_GRIPPER | CBRS_TOOLTIPS | CBRS_FLYBY | CBRS_SIZE_DYNAMIC) ||
            !m_wndToolBar.LoadToolBar(IDR_MAINFRAME))
    {
        TRACE0("Failed to create toolbar\n");
        return ;      // fail to create
    }
	m_wndToolBar.SetWindowText("Standard");

    EnableDocking(CBRS_ALIGN_ANY);
    m_wndToolBar.EnableDocking(CBRS_ALIGN_ANY);
    DockControlBar(&m_wndToolBar);

    // Window Toolbar

    if (!m_windowToolbar.CreateEx(this, TBSTYLE_FLAT, WS_CHILD | WS_VISIBLE | CBRS_TOP
                                  | CBRS_GRIPPER | CBRS_TOOLTIPS | CBRS_FLYBY | CBRS_SIZE_DYNAMIC) ||
            !m_windowToolbar.LoadToolBar(IDR_WINDOW_BAR))
    {
        TRACE0("Failed to create toolbar\n");
        AfxMessageBox("Failed to create toolbar\n");
        return ;      // fail to create
    }
	m_windowToolbar.SetWindowText("Windows");


    EnableDocking(CBRS_ALIGN_ANY);
    m_windowToolbar.EnableDocking(CBRS_ALIGN_ANY);
    DockControlBarLeftOf(&m_windowToolbar, &m_wndToolBar);

    //
    //    We have now added all of the toolbar resources that we require so now define the
    //    grouping of the resources on the toolbars themselves
    //
    //     Define the 'standard' toolbars
    //
    /*
        int LastToolBar = -1;
        CBCGToolBar* pNewToolbar = 
            (CBCGToolBar*)SendMessage (BCGM_CREATETOOLBAR, 0,
                (LPARAM) (LPCTSTR) "Standard");
        if (pNewToolbar == NULL)
        {
            return;
        }
        CBCGToolBarImages images;
        images.Load("mainfram.bmp");
     
        pNewToolbar->SetUserImages(&images);
        pNewToolbar->InsertButton (CBCGToolbarButton (ID_PROJECT_NEW, 1, "User Tool 1", TRUE));
        pNewToolbar->InsertButton (CBCGToolbarButton (ID_FILE_OPENPROJECT, 2, "User Tool 2", TRUE));
        pNewToolbar->InsertButton (CBCGToolbarButton (ID_TOOLS_TESTWIZARD, 3, "User Tool 3", TRUE));
        pNewToolbar->InsertButton (CBCGToolbarButton (ID_FILE_PRINT, 4, "User Tool 3", TRUE));
        pNewToolbar->InsertButton (CBCGToolbarButton (ID_APP_ABOUT, 5, "User Tool 3", TRUE));
        pNewToolbar->InsertButton (CBCGToolbarButton (ID_HELP_CYRANOONTHEWEB, 6, "User Tool 3", TRUE));
        pNewToolbar->InsertButton (CBCGToolbarButton (ID_VIEW_FULLSCREEN, 7, "User Tool 3", TRUE));
     
    */

    //
    //    The creation of the toolbars is done in TWO stages
    //
    //      1> Scan the resources and save the names of the toolbars and the bitmaps on them
    //      2> Define the default toolbars from these lists.
    //
    //
    //    Loop through the resource structures filled in by the DLL and add any as required.
    //
    int ResourceIdx = 0;
    while (m_pResourceInfo[ResourceIdx] != NULL)
    {

        //
        //    Check if a new toolbar is requested and if so retrieve the name of the toolbar.
        //
        CString szToolbar = m_pResourceInfo[ResourceIdx]->strPreferredToolbar;
        UINT nButtonID = m_pResourceInfo[ResourceIdx]->nCommandID;

        //
        // If we do not specify a toolbar then we don't add a button at all
        //
        if ((!szToolbar.IsEmpty()) && (nButtonID != 0))
        {

            //
            //    We have a toolbar button. Scan the list of toolbars defined to see if we already have
            //    an entry for it otherwise create an entry.
            //
            CString szNumber;
            szNumber.Format("%d", nButtonID);
            if (m_pTrace != NULL) m_pTrace->Log(10, "CECPlugInFrame", "AddToolbars", "Attempting to add <" + szNumber + "> to the toolbar <" + szToolbar + ">");

            AddToolBarButton (szToolbar ,nButtonID);
        }

        ResourceIdx++;
    }

    if (m_pTrace != NULL) m_pTrace->Log(10, "CECPlugInFrame", "AddToolbars", "Finished Adding Toolbars");

}

void CECPluginFrame::AddToolBarButton(CString szToolbar, UINT nButtonID)
{
    int index = 0;
    CBCGToolBar* pBar = GetUserBarByIndex (index);
    BOOL bBarFound = FALSE;
    while ( pBar != NULL && !bBarFound)
    {
        CString strName;
        pBar->GetWindowText (strName);

        if (strName == szToolbar)
        {
            bBarFound = TRUE;
            break;
        }

        index++;
        pBar = GetUserBarByIndex (index);
    }

    if (!bBarFound)
    {
        // Create the toolbar
        CBCGToolBar* pNewToolbar =  (CBCGToolBar*)SendMessage (BCGM_CREATETOOLBAR, 0,
                                    (LPARAM) (LPCTSTR) szToolbar.GetBuffer(0));
        if (pNewToolbar == NULL)
        {
            return;
        }

        // Add the button to the new toolbar
        int nIndex = -1;
        CString tooltip = "";
        for (int i = 0; i<m_arButtonMap.GetSize(); i++)
        {
            if (m_arButtonMap[i].buttonID == nButtonID)
            {
                nIndex = m_arButtonMap[i].imageIndex;
                tooltip = m_arButtonMap[i].tooltip;
            }
        }

        pNewToolbar->SetUserImages(m_pUserImages);
        pNewToolbar->InsertButton (CBCGToolbarButton (nButtonID, nIndex, tooltip, TRUE));

        // Dock the toolbar
        pNewToolbar->EnableDocking(CBRS_ALIGN_ANY);
        DockControlBarLeftOf(pNewToolbar, &m_wndToolBar);
    }
    else
    {
        // See if the button already exists...
        int nCmdIndex = pBar->CommandToIndex(nButtonID);

        // if exists  return...
        if (nCmdIndex != -1)
            return;

        // Otherwise add the button to the toolbar
        int nIndex = -1;
        CString tooltip = "";
        for (int i = 0; i<m_arButtonMap.GetSize(); i++)
        {
            if (m_arButtonMap[i].buttonID == nButtonID)
            {
                nIndex = m_arButtonMap[i].imageIndex;
                tooltip = m_arButtonMap[i].tooltip;
            }
        }
        pBar->SetUserImages(m_pUserImages);
        pBar->InsertButton (CBCGToolbarButton (nButtonID, nIndex, tooltip, TRUE));
    }
}

void CECPluginFrame::OnProjectNew()
{
    CECProjectNewDlg    NewDlg;
    NewDlg.DoModal();
}

//
// OnPlugIn
// ========
//
// Send the event down to the DLL which added the resource
//
void CECPluginFrame::OnPlugIn(UINT nID)
{
    // To do this we have to search the Resource info to see who owns the button,
    // and route the message direct to the DLL in question
    HINSTANCE hDLL = GetOwner(nID);

    LPONEVENTFUNC   lpfnOnEvent;

    // And see if the OnEvent function is supported
    lpfnOnEvent = (LPONEVENTFUNC)::GetProcAddress(hDLL,"OnEvent");
    if (lpfnOnEvent)
    {
        // If it is then call it
        UINT msg = WM_COMMAND;
        long id = nID;
        lpfnOnEvent(&msg, &id);
    }
}

//
// OnUpdatePlugInUI
// ================
//
// Command Handler for the Plug in buttons
//
void CECPluginFrame::OnUpdatePlugInUI(CCmdUI* pCmdUI)
{
    BOOL bState = FALSE;

    // To do this we have to search the Resource info to see who owns the button,
    // and route the message direct to the DLL in question
    HINSTANCE hDLL = GetOwner(pCmdUI->m_nID);

    if(HasExtendUIUpdate(pCmdUI->m_nID))
    {
        LPCOMMANDUIFUNCEX   lpfnOnUpdateCommandUIEx;
        lpfnOnUpdateCommandUIEx = (LPCOMMANDUIFUNCEX)::GetProcAddress(hDLL,"OnUpdateCommandUIEx");
        if (lpfnOnUpdateCommandUIEx)
        {
            bState = lpfnOnUpdateCommandUIEx(&pCmdUI->m_nID,pCmdUI);
        }
    }
    else
    {
        LPCOMMANDUIFUNC lpfnOnUpdateCommandUI;

        lpfnOnUpdateCommandUI = (LPCOMMANDUIFUNC)::GetProcAddress(hDLL,"OnUpdateCommandUI");
        if (lpfnOnUpdateCommandUI)
        {
            // Find out if this button is Enabled/Disabled
            bState = lpfnOnUpdateCommandUI(&pCmdUI->m_nID);
        }
    }

    // And if Removeable we either Hide/Show the button depending
    // On it's Enabled state
    if (IsRemoveable(pCmdUI->m_nID))
    {
        // Only do this if something has changed to warrant Toggling a plug-in. It
        // is pointless doing it all the while as it does cause a slight flickering
        // on the toolbar.

        // First we find the index of the CommandID to which this resource refers
        int nTotal = m_arCommandIDs.GetSize();
        if (nTotal > 0)
        {
            for (int i=0; i<nTotal; i++)
            {
                if (m_arCommandIDs.GetAt(i) == pCmdUI->m_nID)
                {
                    // OK compare the current Display state of this Command ID
                    if (bState != m_arDisplayState.GetAt(i))
                    {
                        // And switch it ONLY if it has changed
                        if (TogglePlugIn (bState, pCmdUI->m_nID))
                        {
                            // Not forgetting to update the array of course
                            m_arDisplayState.SetAt(i, bState);
                        }
                    }
                }
            }
        }
    }

    pCmdUI->Enable(bState);
}

//
//HasExtendedUIUpdate
//===================
//
// This function retrieves the resource handle of the DLL who
// owns the resource passed in and sees if the extended update flag is set
//
BOOL CECPluginFrame::HasExtendUIUpdate(UINT nID)
{
    CECCommanderApp *pApp = (CECCommanderApp*)AfxGetApp();
    ASSERT (pApp != NULL);

    BOOL bExtended = FALSE;

    // Get the index of the ResourceInfo Struct to which this Command ID belongs
    int ResourceIdx = GetResourceIndex(nID);

    // If valid we obtain the relevant Resource Handle
    if ((ResourceIdx != -1) && (m_pResourceInfo[ResourceIdx] != NULL))
    {
        CECStyleFinder styles;
        bExtended = styles.HasExtendedUIUpdate(m_pResourceInfo[ResourceIdx]->dwStyles);
    }

    return bExtended;
}

//
// IsRemoveable
// ============
//
// This function retrieves the resource handle of the DLL who
// owns the resource passed in and sees if the removeable flag is set
//
BOOL CECPluginFrame::IsRemoveable(UINT nID)
{
    CECCommanderApp *pApp = (CECCommanderApp*)AfxGetApp();
    ASSERT (pApp != NULL);

    BOOL bRemoveable = FALSE;

    // Get the index of the ResourceInfo Struct to which this Command ID belongs
    int ResourceIdx = GetResourceIndex(nID);

    // If valid we obtain the relevant Resource Handle
    if ((ResourceIdx != -1) && (m_pResourceInfo[ResourceIdx] != NULL))
    {
        CECStyleFinder styles;
        bRemoveable = styles.IsRemoveable(m_pResourceInfo[ResourceIdx]->dwStyles);
    }

    return bRemoveable;
}

//
// TogglePlugIn
// ============
//
// This function Hides/Shows a removeable Plug-in
//
BOOL CECPluginFrame::TogglePlugIn(BOOL bShow, UINT nID)
{
    /*
        // And before we do anything we should confirm whether or not we are in Fullscreen
        // mode - this is simply to avoid the following processing as there is no menubar
        // displayed while in FullScreen mode
        CECWorkspace* pWorkspace = CECCommanderApp::GetWorkspace();
        ASSERT (pWorkspace != NULL);
     
        if (pWorkspace->IsFullScreen())
            return FALSE;
     
        // First we do the Toolbar button which may not actually exist
        SECCustomToolBar* pToolBar = NULL;
     
        pToolBar = FindToolBar(GetPreferredToolbar(nID));
        if (pToolBar != NULL)
        {
            int ButtonIndex = pToolBar->CommandToIndex(nID);
                                            // Get index from command id
     
    //
    //    Get the existing button style and either hide it or unhide it.
    //
            UINT nStyle = pToolBar->GetButtonStyle(ButtonIndex);
            if (bShow)
            {
                nStyle &= ~TBBS_HIDDEN;
            }
            else
            {
                nStyle |= TBBS_HIDDEN;
            }
     
            pToolBar->SetButtonStyle (ButtonIndex, nStyle);
        }
     
        // And now the menu item which should always exist
        CString szMenu = GetPreferredMenu(nID);
        
        if (bShow)
        {
            CMenu* pMainMenu = AfxGetMainWnd()->GetMenu();
            ASSERT(pMainMenu != NULL);
     
            CMenu* pPreferredMenu = FindMenu(pMainMenu, szMenu);
            if (pPreferredMenu != NULL)
            {
                CreateMenuItem(pPreferredMenu, nID);
                return TRUE;
            }
            else 
            {
                return FALSE;
            }
        }
        else
        {
            return DeleteMenuItem(szMenu, nID);
        }
    */
    return FALSE;
}

//
// AddOutputViews
// ==============
//
// Function to add in all the Output Wnds based on what is in our resource array
//
void  CECPluginFrame::AddOutputViews()
{
    if (m_pTrace != NULL) m_pTrace->Log(10, "CECPlugInFrame", "AddOutputViews", "Attempting to add in Output Windows");

    CECCommanderApp *pApp = (CECCommanderApp*)AfxGetApp();
    ASSERT(pApp != NULL);

    // Get the FloorWnd Array
    CPtrArray& OutputWndArray = pApp->GetOutputWndArray ();

    int ResourceIdx = 0;
    while (m_pOutputInfo[ResourceIdx] != NULL)
    {
        // And add in a new FloorWnd based for each entry
        CECOutputWnd* pNewOutputWnd = new CECOutputWnd(m_pOutputInfo[ResourceIdx]->strName,
                                      m_pOutputInfo[ResourceIdx]->hWnd);
        OutputWndArray.Add(pNewOutputWnd);

        ResourceIdx++;

        CString szName = m_pOutputInfo[ResourceIdx]->strName;
        if (m_pTrace != NULL) m_pTrace->Log(10, "CECPlugInFrame", "AddOutputViews", "Added Output Window <" + szName + "> Tab");
    }
}

//
// AddDocumentTemplates
// ====================
//
// Function to add in all the requested Document Templates
//
void  CECPluginFrame::AddDocumentTemplates()
{

    if (m_pTrace != NULL) m_pTrace->Log(10, "CECPlugInFrame", "AddDocumentTemplates", "Attempting to add in Supported Document Templates");

    CECCommanderApp *pApp = (CECCommanderApp*)AfxGetApp();
    ASSERT(pApp != NULL);

    int ResourceIdx = 0;
    while (m_pDocInfo[ResourceIdx] != NULL)
    {
        // Simply register the Template by adding it to the App
        CMultiDocTemplate *pTemplate = NULL;
        pTemplate = m_pDocInfo[ResourceIdx]->pDocTemplate;

        if (pTemplate != NULL)
        {
            // Make sure that this Template has a set of strings associated with it ...

            // In actual fact we are only bothered about the final string which distinguishes
            // the name of the template as we will use this string to determine if a particular
            // Document template actually exists or not so,
            CString szSearchString;
            if (pTemplate->GetDocString(szSearchString, CDocTemplate::regFileTypeName))
            {
                // And check to see if it already exists.
                if (pApp->FindDocumentTemplate(szSearchString) == NULL)
                {
                    pTemplate->m_bAutoDelete = false;
                    pApp->AddDocTemplate(pTemplate);

                    CECStyleFinder styles;
                    if (styles.HasTemplate(m_pDocInfo[ResourceIdx]->dwStyles))
                    {
                        // Get the template directory
                        CString templateDir;
                        templateDir.Format("%s\\Templates\\", pApp->GetInstallationDirectory());

                        // Make sure the template directory exists
                        /*
                                                SECFileSystem fs;
                                                BOOL bDirExists = fs.DirectoryExists(templateDir);
                                                if (!bDirExists)
                                                {
                                                    fs.MakePath(templateDir);
                                                }
                         
                                                CString templateFile = templateDir+"Blank."+ m_pDocInfo[ResourceIdx]->strFileExt;
                                                // Make sure a blank template exists for each document type
                                                BOOL bFileExists = fs.FileExists(templateFile);
                                                if (!bFileExists)
                                                {
                                                    CFile f(templateFile, CFile::modeCreate);
                                                    f.Close();
                                                }
                        */
                    }
                    if (m_pTrace != NULL) m_pTrace->Log(10, "CECPlugInFrame", "AddDocumentTemplates", "Added in Support for the <" + szSearchString + "> Document Template");
                }
            }
        }

        // And register the file type
        CString szExtension = m_pDocInfo[ResourceIdx]->strFileExt;

        if (!szExtension.IsEmpty())
        {
            pApp->AddKnownFileExtension(szExtension);

            if (m_pTrace != NULL) m_pTrace->Log(10, "CECPlugInFrame", "AddDocumentTemplates", "Added in Support for the <" + szExtension + "> File Extension");
        }

        // And proceed to the next type
        ResourceIdx++;
    }

}

void CECPluginFrame::OnSysColorChange()
{
    CBCGMDIFrameWnd::OnSysColorChange();

    if (m_pUserImages == NULL)
        return;

    m_pUserImages->Clear();

    for (int i = 0; i<m_arButtonMap.GetSize(); i++)
    {
        HBITMAP hbmp = (HBITMAP) ::LoadImage (
                           m_arButtonMap[i].hDLL,
                           MAKEINTRESOURCE (m_arButtonMap[i].buttonID),
                           IMAGE_BITMAP,
                           0, 0,
                           LR_CREATEDIBSECTION | LR_LOADMAP3DCOLORS);

        if (hbmp == NULL)
            continue;

        BITMAP bmp;
        if (::GetObject (hbmp, sizeof (BITMAP), &bmp) == 0)
        {
            ::DeleteObject (hbmp);
            continue;
        }

        m_arButtonMap[i].imageIndex = m_pUserImages->AddImage(hbmp);
        ::DeleteObject (hbmp);
    }
}
