/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// CECPluginFrame.h : interface of the CECPluginFrame class
//
/////////////////////////////////////////////////////////////////////////////

#if !defined(AFX_CECPLUGINFRAME_H__FF67E80A_67AF_11D4_A6DA_00500447FC27__INCLUDED_)
#define AFX_CECPLUGINFRAME_H__FF67E80A_67AF_11D4_A6DA_00500447FC27__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000
#include "../Common/CECTrace.h"
#include "../Interfaces/CyrPlugins.h"
#include "CECStatusBar.h"

struct _USER_BUTTON_MAP
{
    HINSTANCE hDLL;
    int buttonID;
    int imageIndex;
    CString tooltip;
}
typedef _USER_BUTTON_MAP;


class CECPluginFrame : public CBCGMDIFrameWnd
{
    DECLARE_DYNAMIC(CECPluginFrame)
public:
    CECPluginFrame();

    void LoadResources();
    void  AddMenus();
    CMenu* FindMenu(CMenu* pMainMenu, const CString& szPreferredMenu, BOOL bAndCreate);
    void  CreateMenuItem(CMenu* pPreferredMenu, const UINT& nID);
    void AddMenuBitmap(CMenu* pMenu, const UINT& nCommandID);
    int GetWizardCount();
    void DeactivateOptionalFeatures();
    CString GetMenuLabel(UINT nID);
    BOOL GetMenuSeparator(UINT nID);
    HINSTANCE GetOwner(UINT nID);
    BOOL DeleteMenuItem(const CString& szLabel, const UINT& nCommandID);
    int GetResourceIndex(UINT nID);
    void ValidateMenu(CMenu* pMenu);
    void  AddToolbars();
    BOOL HasExtendUIUpdate(UINT nID);
    BOOL IsRemoveable(UINT nID);
    BOOL TogglePlugIn(BOOL bShow, UINT nID);
    void Play()
    {
        m_wndStatusBar.Play();
    }
    void Stop()
    {
        m_wndStatusBar.Stop();
    }
    void SetText(LPCTSTR pText)
    {
        m_wndStatusBar.SetPaneText(0, pText, TRUE);
    }
    void  AddOutputViews();
    void  AddDocumentTemplates();
    CList<UINT, UINT>   m_lstBasicCommands;


    // Attributes
public:

    // Operations
public:

    // Overrides
    // ClassWizard generated virtual function overrides
    //{{AFX_VIRTUAL(CECPluginFrame)
public:
    virtual BOOL PreCreateWindow(CREATESTRUCT& cs);
    //}}AFX_VIRTUAL

    // Implementation
public:
    virtual ~CECPluginFrame();
#ifdef _DEBUG
    virtual void AssertValid() const;
    virtual void Dump(CDumpContext& dc) const;
#endif

protected:  // control bar embedded members
    CBCGMenuBar     m_wndMenuBar;
    CBCGToolBar     m_wndToolBar;
    CECStatusBar    m_wndStatusBar;
    CBCGToolBar     m_windowToolbar;

    // Generated message map functions
protected:
    //{{AFX_MSG(CECPluginFrame)
    afx_msg int OnCreate(LPCREATESTRUCT lpCreateStruct);
    afx_msg void OnProjectNew();
    afx_msg void OnSysColorChange();
    //}}AFX_MSG
    afx_msg void OnViewCustomize();
    afx_msg LRESULT OnToolbarReset(WPARAM,LPARAM);
    void OnToolsViewUserToolbar (UINT id);
    void OnUpdateToolsViewUserToolbar (CCmdUI* pCmdUI);
    afx_msg LRESULT OnToolbarContextMenu(WPARAM,LPARAM);
    afx_msg void OnUpdatePlugInUI(CCmdUI* pCmdUI);
    afx_msg void OnPlugIn(UINT nID);
    DECLARE_MESSAGE_MAP()

    virtual BOOL OnShowPopupMenu (CBCGPopupMenu* pMenuPopup);


    // Member Variables
    CWnd* m_pWindow;

    CUIntArray m_arCommandDLLLinks;
    CUIntArray m_arCommandIDs;
    CUIntArray m_arDisplayState;

    CArray <_USER_BUTTON_MAP, _USER_BUTTON_MAP> m_arButtonMap;

    // struct which contains all the buttons and menu items which have been added
    struct RESOURCEINFO*    m_pResourceInfo[MAX_PLUGINS];

    // Struct which contains all the Output windows to be added
    struct WINDOWINFO*      m_pWindowInfo[MAX_DOCKINGWINDOWS];

    // Struct which contains all the Output windows to be added
    struct OUTPUTINFO*      m_pOutputInfo[MAX_OUTPUTVIEWS];

    // Struct which contains all the document to be added
    struct DOCUMENTINFO*    m_pDocInfo[MAX_DOCUMENTS];

    // Struct which contains all the document to be added
    struct WIZARDINFO*      m_pWizardInfo[MAX_WIZARDS];

    // And the Trace Object
    CECTrace* m_pTrace;

    //Array to hold menu item bitmap pointers(so they can be deleted later)
    CArray <CBitmap*,CBitmap*> m_arpMenuItemBitmaps;

    void AddBitmapResources (HINSTANCE  hDLL ,UINT nButtonID);
    void SetCommandHandle(int nCommandCount, int nDLL);
    void  AddFloorWnds();

private:
    CBCGToolBarImages* m_pUserImages;
    void AddToolBarButton (CString szToolbar ,UINT nButtonID);
};

/////////////////////////////////////////////////////////////////////////////

//{{AFX_INSERT_LOCATION}}
// Microsoft Visual C++ will insert additional declarations immediately before the previous line.

#endif // !defined(AFX_CECPLUGINFRAME_H__FF67E80A_67AF_11D4_A6DA_00500447FC27__INCLUDED_)
