/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// CECUserToolManager.cpp: implementation of the CECUserToolManager class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "CECCommander.h"
#include "Solutions.h"
#include "CECUserToolManager.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

#define TOOLS_FILE  "Tools.dat"
#define SEP '|'
//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CECUserToolManager::CECUserToolManager()
{
}


CECUserToolManager::~CECUserToolManager()
{
}


void CECUserToolManager::LoadUserTools()
{
    CECCommanderApp* pApp = (CECCommanderApp*)AfxGetApp();
    CString ToolsFile = pApp->GetInstallationDirectory();
    ToolsFile+= "\\"+(CString)TOOLS_FILE;

    CBCGUserToolsManager* pUserToolsManager = pApp->GetUserToolsManager ();

    try
    {
        CStdioFile f(ToolsFile, CFile::modeRead | CFile::typeText);

        CString strTemp;

        while (f.ReadString(strTemp))
        {

            CString name, args, dir, cmd;
            GetNextField(SEP, strTemp, name);
            GetNextField(SEP, strTemp, args);
            GetNextField(SEP, strTemp, dir);
            GetNextField(SEP, strTemp, cmd);

            CBCGUserTool* pTool = pUserToolsManager->CreateNewTool ();
            pTool->m_strLabel = name;
            pTool->m_strArguments = args;
            pTool->m_strInitialDirectory = dir;
            pTool->SetCommand (cmd);
        }
    }
    catch(...)
    {}

}

void CECUserToolManager::SaveUserTools()
{
    CECCommanderApp* pApp = (CECCommanderApp*)AfxGetApp();
    CString ToolsFile = pApp->GetInstallationDirectory();
    ToolsFile+= "\\"+(CString)TOOLS_FILE;
    CBCGUserToolsManager* pUserToolsManager = pApp->GetUserToolsManager ();
	
	CFileException ex;

	CStdioFile f;
	if(!f.Open(ToolsFile, CFile::modeCreate |CFile::modeWrite| CFile::typeText,&ex))
	{
		CString strErr;
		ex.GetErrorMessage(strErr.GetBuffer(1024),1024);
		strErr.ReleaseBuffer();
		
		strErr = "Failed to write toolbar settings file.\n" + strErr;

		AfxMessageBox(strErr,MB_ICONEXCLAMATION);
		return;
	}

    POSITION pos;
    int index = 0;
    for( pos = pUserToolsManager->GetUserTools ().GetHeadPosition(); pos != NULL; )
    {
        CBCGUserTool* t = (CBCGUserTool*)pUserToolsManager->GetUserTools ().GetNext( pos );

        CString strTemp = "";

        strTemp+=t->m_strLabel + SEP;
        strTemp+=t->m_strArguments + SEP;
        strTemp+=t->m_strInitialDirectory + SEP;

        CString cmd  = t->GetCommand();
        strTemp+=cmd + "\n";

        f.WriteString(strTemp);
    }

    f.Close();
}

int CECUserToolManager::GetNextField(char sep, CString &inStr, CString &field)
{
    if (inStr=="")
    {
        return -1;
    }

    int len = inStr.GetLength();
    int nPos = inStr.Find(sep,0);

    if (nPos == -1)
    {
        field = inStr;
        inStr="";
        return 0;
    }
    else
    {
        field = inStr.Left(nPos);
        inStr=inStr.Right(len-nPos-1);
        return 0;
    }
}
