/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

/////////////////////////////////////////////////////////////////////////
//
// CECTrace.cpp
// ------------
//
//    This is the implementation file for the CECTrace class
//
//=========================================================================


#include "stdafx.h"
#include "CECTrace.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CECTrace

CECTrace::CECTrace(CString szInstallDir)
{
    m_pTraceFile = NULL;
    m_szInstallDir = szInstallDir + "\\trace.log";
}

CECTrace::~CECTrace()
{
    // And delete the file object
    if (m_pTraceFile != NULL)
    {
        delete m_pTraceFile;
        m_pTraceFile = NULL;
    }
}

/////////////////////////////////////////////////////////////////////////////
// CECTrace message handlers

//
// Create
// ======
//
// Create the Trace Log - This Must work or something is seriously
// Wrong with the System
//
BOOL CECTrace::Create()
{
    // And away we go ...
    try
    {
        m_pTraceFile = new CStdioFile (m_szInstallDir, CFile::modeCreate | CFile::modeWrite | CFile::shareDenyNone | CFile::typeText);
    }
    catch (CFileException* e)
    {
        e->ReportError();
        e->Delete();

        return FALSE;
    }

    return TRUE;
}

//
// Close
// =====
//
// Finished Initialisation so close the Trace Log
//
void CECTrace::Close()
{
    try
    {
        m_pTraceFile->Close();
    }
    catch (CFileException* e)
    {
        e->ReportError();
        e->Delete();
    }
}

//
// Log
// ===
//
// And log a message - Again this must always be successful
//
BOOL CECTrace::Log(UINT nLevel, CString szClass, CString szFunction, CString szMessage)
{
    try
    {
        CString szEntry = ComposeEntry(nLevel, szClass, szFunction, szMessage);

        m_pTraceFile->WriteString(szEntry);
        m_pTraceFile->Flush();
    }
    catch (CFileException* e)
    {
        e->ReportError();
        e->Delete();

        return FALSE;
    }

    return TRUE;
}

//
// ComposeEntry
// ============
//
// And compose the text that we are about to add to the Trace File
//
CString CECTrace::ComposeEntry(UINT nLevel, CString szClass, CString szFunction, CString szMessage)
{
    static int number = 1;

    CTime t = CTime::GetCurrentTime();

    CString szTime;
    szTime = t.Format("%H:%M:%S - %B %d, %Y");

    CString szEntry;
    szEntry.Format("%d, %s LEVEL-%d, %s, %s, \"%s\"\n", number++, szTime, nLevel, szClass, szFunction, szMessage);

    return szEntry;
}
