/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// CECAuditLogger.cpp: implementation of the CECAuditLogger class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "CECAuditLogger.h"
#include "CORBAObjects\TestManager.h"
#include "CYRWrapLib\CECTestOutput.h"
#include "..\..\interfaces\CyrPlugIns.h"
#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif



//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CECAuditLogger::CECAuditLogger()
{
    m_pCorbaSrv=new CORBAServer ( 0, NULL );
    m_pCorbaSrv->Ready(true);

}

CECAuditLogger::~CECAuditLogger()
{
    for (int i = 0; i<m_ThreadArray.GetSize(); i++)
    {
        DWORD dwExitCode=0;
        TerminateThread(m_ThreadArray[i].m_pThread->m_hThread,dwExitCode);
    }
    delete m_pCorbaSrv;
}

void CECAuditLogger::StartLogging(CString strRegName)
{
    try
    {
        //Get the Test manager status object
        CORBA::Object_var managerObj;
        managerObj=m_pCorbaSrv->GetObjectFromClass(TestManager::CLASS_NAME, strRegName.GetBuffer(0));
        TestManager_var TM = narrow_object((TestManager_ptr)NULL,managerObj);

        CORBA::Object_var testrunObj;
        testrunObj=TM->GetTestRunAsObject();
        m_TestRun=narrow_object((CyrStTestRun_ptr )NULL,testrunObj);

        // Start Thread and save add to the map
        m_szTestName = m_TestRun->GetTestName();
        CWinThread* pThread = AfxBeginThread(LoggerThread, this);

        _THREAD_MAP tItem;
        tItem.strRegName = strRegName;
        tItem.m_pThread = pThread;
        m_ThreadArray.Add(tItem);

    }
    catch(...)
    {
        AfxMessageBox("Exception");
    }
}


UINT CECAuditLogger::LoggerThread(LPVOID pParam)
{
    CECAuditLogger *pAudit = (CECAuditLogger*)pParam;

    try
    {
        CWnd* pMainWnd = AfxGetMainWnd();
        ASSERT (pMainWnd != NULL);
        char* pCompileWnd = pAudit->m_szTestName.GetBuffer(0);
        pMainWnd->SendMessage (STUDIO_CUSTOM_LINK_2, (WPARAM)pCompileWnd, NULL);

        int nLastIndex = pAudit->m_TestRun->GetOutputOffset(CECTestLog::OUTPUT_TYPE);
        for (;;)
        {
            long nEntries = pAudit->m_TestRun->GetNoOutputEntries(CECTestLog::OUTPUT_TYPE);

            for (nLastIndex; nLastIndex<nEntries; nLastIndex++)
            {
                string auditInfo = pAudit->m_TestRun->GetOutput(CECTestLog::OUTPUT_TYPE, nLastIndex);

                // Tokenize string
                CString szAudit( auditInfo.c_str() );
                // Ignore Time field and get the text message
                int nPos = szAudit.Find(',');
                int nLen = szAudit.GetLength();
                szAudit = szAudit.Right(nLen-nPos+1);
                nPos = szAudit.Find(',');
                nLen = szAudit.GetLength();
                szAudit = szAudit.Right(nLen-nPos+1);


                // Send message to mainframe to add text to outwnd
                pMainWnd->SendMessage (STUDIO_CUSTOM_LINK_2, (WPARAM)pCompileWnd, (LPARAM)(LPCTSTR)szAudit);
            }

            ::Sleep(1000);
        }
    }
    catch(...)
    {}

    return 0;
}

void CECAuditLogger::StopLogging(CString strRegName)
{
    for (int i = 0; i<m_ThreadArray.GetSize(); i++)
    {
        if (m_ThreadArray[i].strRegName == strRegName)
        {
            TerminateThread(m_ThreadArray[i].m_pThread->m_hThread, 0);
            m_ThreadArray.RemoveAt(i);
            return;
        }
    }
}
