/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// CECInjectorView.cpp : implementation file
//

#include "stdafx.h"
#include "CECInjectorView.h"
#include "..\..\common\CECClipboard.h"
#include "..\..\Interfaces\CyrPlugins.h"
#include "Solutions.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CECInjectorView

IMPLEMENT_DYNCREATE(CECInjectorView, CFormView)

CECInjectorView::CECInjectorView()
        : CFormView(CECInjectorView::IDD)
{
    //{{AFX_DATA_INIT(CECInjectorView)
    // NOTE: the ClassWizard will add member initialization here
    //}}AFX_DATA_INIT

    m_szTitle = "Injectors";
}

CECInjectorView::~CECInjectorView()
{}


void CECInjectorView::DoDataExchange(CDataExchange* pDX)
{
    CFormView::DoDataExchange(pDX);
    //{{AFX_DATA_MAP(CECInjectorView)
    DDX_Control(pDX, IDC_XINJECTORMONCTRL1, m_ctlInjectorMon);
    //}}AFX_DATA_MAP
}



BEGIN_MESSAGE_MAP(CECInjectorView, CFormView)
//{{AFX_MSG_MAP(CECInjectorView)
ON_WM_SIZE()
ON_WM_SETFOCUS()
ON_WM_ERASEBKGND()
//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CECInjectorView diagnostics

#ifdef _DEBUG
void CECInjectorView::AssertValid() const
{
    CFormView::AssertValid();
}

void CECInjectorView::Dump(CDumpContext& dc) const
{
    CFormView::Dump(dc);
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CECInjectorView message handlers

void CECInjectorView::OnSize(UINT nType, int cx, int cy)
{
    CFormView::OnSize(nType, cx, cy);

    if (m_ctlInjectorMon.m_hWnd != NULL)
    {
        m_ctlInjectorMon.SetWindowPos(NULL, 0,0,cx,cy,SWP_NOZORDER);
    }
}

void CECInjectorView::OnInitialUpdate()
{
    CFormView::OnInitialUpdate();

    try
    {
        CECClipboard cb;
        CString testRegName = cb.GetClipboardText();

        m_ctlInjectorMon.XInitialise(testRegName);
        GetDocument()->SetTitle(m_szTitle);

        CRect r;
        GetClientRect(r);
        m_ctlInjectorMon.SetWindowPos(NULL, 0,0,r.Width(),r.Height(),SWP_NOZORDER);

        SIZE sz = {0};
        SetScrollSizes(MM_TEXT, sz);
    }
    catch (...)
    {
        // If for some reason we cannot get the document - just close the window
        CFrameWnd* pParent = (CFrameWnd*) GetParentFrame();
        if(pParent != NULL)
            pParent->PostMessage(WM_CLOSE);
    }
}

void CECInjectorView::SetRefreshRate(int nRate)
{
    m_ctlInjectorMon.SetRefreshDelay(nRate);
}

void CECInjectorView::OnSetFocus(CWnd* pOldWnd)
{
    CFormView::OnSetFocus(pOldWnd);
}

BOOL CECInjectorView::OnEraseBkgnd(CDC *pDC)
{
    return true;
}

BEGIN_EVENTSINK_MAP(CECInjectorView, CFormView)
//{{AFX_EVENTSINK_MAP(CECInjectorView)
ON_EVENT(CECInjectorView, IDC_XINJECTORMONCTRL1, 1 /* ShowDynGraph */, OnShowDynGraphXinjectormonctrl1, VTS_BSTR VTS_BSTR)
ON_EVENT(CECInjectorView, IDC_XINJECTORMONCTRL1, 2 /* ControlClosing */, OnControlClosingXinjectormonctrl1, VTS_NONE)
ON_EVENT(CECInjectorView, IDC_XINJECTORMONCTRL1, 3 /* UpdateTitle */, OnUpdateTitle, VTS_BSTR)
//}}AFX_EVENTSINK_MAP
END_EVENTSINK_MAP()

void CECInjectorView::OnShowDynGraphXinjectormonctrl1(LPCTSTR szRegName,LPCTSTR szInjector)
{
    CWnd* pMainWnd = AfxGetMainWnd();

    CString monitorInfo;
    monitorInfo.Format("%s|%s|%s" ,SOLUTIONS_GENERIC_DLL ,szRegName,szInjector);

    pMainWnd->SendMessage (WMU_FIRE_SPECIFIC_EVENT
                           ,(WPARAM )WMU_DYN_TEST_GRAPH
                           ,(LPARAM) &monitorInfo);
}

void CECInjectorView::OnControlClosingXinjectormonctrl1()
{
    CFrameWnd* pParent = (CFrameWnd*) GetParentFrame();
    if(pParent)
        pParent->DestroyWindow();
}

void CECInjectorView::OnUpdateTitle(LPCTSTR szTitle)
{
    m_szTitle = szTitle;
    GetParentFrame()->SetTitle(m_szTitle);
}
