/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// CECGlobalSettings.cpp: implementation of the CECGlobalSettings class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "CECGlobalSettings.h"
#include "CECWorkspace.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////
CECGlobalSettings::CECGlobalSettings()
{
    m_pWorkspace = new CECWorkspace ();
    LoadGraphSettings();
}

CECGlobalSettings::~CECGlobalSettings()
{
    if(m_pWorkspace != NULL)
    {
        delete m_pWorkspace;
        m_pWorkspace = NULL;
    }
    SaveGraphSettings();
}

void CECGlobalSettings::LoadGraphSettings()
{
    CWinApp *pApp = AfxGetApp();
    m_TitleFont = pApp->GetProfileString("GraphTitle","TitleFont", "Times New Roman");
    m_TitleFontSize = pApp->GetProfileInt("GraphTitle","TitleFontSize",20);
    m_TitleTextCol = pApp->GetProfileInt("GraphTitle","TextClr",(int)COLORREF RGB(210,40,50));
    m_TitleBGCol = pApp->GetProfileInt("GraphTitle","BG Color",(int)COLORREF RGB(0,128,192));
    m_bShowTitle =  pApp->GetProfileInt("GraphStyles","ShowTitle",FALSE);
    CString test = pApp->GetProfileString("GraphStyles","LabelFont");
    if (test == "")
    {
        pApp->WriteProfileInt("GraphStyles","LineCol0",(int)CLR_RED);
        pApp->WriteProfileInt("GraphStyles","LineCol1",(int)CLR_BLUE);
        pApp->WriteProfileInt("GraphStyles","LineCol2",(int)CLR_OLIVE);
        pApp->WriteProfileInt("GraphStyles","LineCol3",(int)CLR_BROWN);
        pApp->WriteProfileInt("GraphStyles","LineCol4",(int)CLR_PINK);
        pApp->WriteProfileInt("GraphStyles","LineCol5",(int)CLR_ORANGE);
        pApp->WriteProfileInt("GraphStyles","LineCol6",(int)CLR_MAUVE);
        pApp->WriteProfileInt("GraphStyles","LineCol7",(int)CLR_MAGENTA);
        pApp->WriteProfileInt("GraphStyles","LineCol8",(int)CLR_GREY);
        pApp->WriteProfileInt("GraphStyles","LineCol9",(int)CLR_PURPLE);
    }

    for (int isub = 0; isub<MAX_DEF_COLOURS; isub++)
    {
        CString key;
        key.Format("LineCol%d",isub);
        m_nDefLineColours[isub] = pApp->GetProfileInt("GraphStyles",key,(int)COLORREF RGB(192,192,192));
    }

    m_LabelCol = pApp->GetProfileInt("GraphStyles","LabelCol",(int)RGB(255,0,0));
    m_BGCol = pApp->GetProfileInt("GraphStyles","BGCol",(int)RGB(0xc0,0xc0,0xc0));
    m_ATCol = pApp->GetProfileInt("GraphStyles","ATCol",(int)RGB(0,0,0));
    m_GridCol = pApp->GetProfileInt("GraphStyles","GridCol",(int)RGB(30,220,100));
    m_LabelFont = pApp->GetProfileString("GraphStyles","LabelFont", "Times New Roman");
}

void CECGlobalSettings::SaveGraphSettings()
{
    CWinApp *pApp = AfxGetApp();

    pApp->WriteProfileString("GraphTitle","TitleFont",m_TitleFont);
    pApp->WriteProfileInt("GraphTitle","TitleFontSize",m_TitleFontSize);
    pApp->WriteProfileInt("GraphTitle","TextClr",(int)m_TitleTextCol);
    pApp->WriteProfileInt("GraphTitle","BG Color",(int)m_TitleBGCol);

    for (int isub = 0; isub<MAX_DEF_COLOURS; isub++)
    {
        CString key;
        key.Format("LineCol%d",isub);
        pApp->WriteProfileInt("GraphStyles",key,(int)m_nDefLineColours[isub]);
    }

    pApp->WriteProfileInt("GraphStyles","LabelCol",(int)m_LabelCol);
    pApp->WriteProfileInt("GraphStyles","BGCol",(int)m_BGCol);
    pApp->WriteProfileInt("GraphStyles","ATCol",(int)m_ATCol);
    pApp->WriteProfileInt("GraphStyles","GridCol",(int)m_GridCol);
    pApp->WriteProfileString("GraphStyles","LabelFont",m_LabelFont);
    pApp->WriteProfileInt("GraphStyles","ShowTitle",m_bShowTitle);
}

void CECGlobalSettings::RemoveAllRunInfo()
{
    m_RunsLoaded.RemoveAll();
}

/////////////////////////////////////////////////////////////////////////////
// @mfunc   Store information about data set added to the hidden grid
void CECGlobalSettings::AddRunInfo(const CString& strTestRun)
{
    m_RunsLoaded.Add(strTestRun);
}

/////////////////////////////////////////////////////////////////////////////
// @mfunc   Retrieve the test run information for a data set in the hidden grid
// @rdesc   Success/Failure
// @parm    int | index | Array Index
BOOL CECGlobalSettings::GetRunInfo(int index,CString& strTestRun)
{
    //
    // Check that the array index is in range
    //
    int nLoadedCount = m_RunsLoaded.GetSize();
    if (nLoadedCount == 0 || index >= nLoadedCount)
    {
        return FALSE;
    }

    //
    // Return the information about the data set
    //
    strTestRun = m_RunsLoaded[index];
    return TRUE;
}

/////////////////////////////////////////////////////////////////////////////
// @mfunc   Gets the number of data sets loaded in the hidden grid.
// @rdesc   Run count
int CECGlobalSettings::GetLoadedCount()
{
    return m_RunsLoaded.GetSize();
}


