/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// CECHttpListView.cpp : implementation file
//

#include "stdafx.h"
#include "resource.h"
#include "CECGlobalSettings.h"
#include "CECHttpListView.h"
#include "AnalyzerDefs.h"
#include "Solutions.h"
#include "..\studio\Common\CECRegistryTools.h"
#include "Server.h"
#include "..\studio\interfaces\CYRPlugins.h"
#include "CyrEventsink.h"
#include "CECVirtGridView.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

extern CECGlobalSettings* g_pGlobalSettings;
extern CYREventSink* pEventSink;

/////////////////////////////////////////////////////////////////////////////
// CECHttpListView

IMPLEMENT_DYNCREATE(CECHttpListView, CFormView)

CECHttpListView::CECHttpListView()
        : CFormView(CECHttpListView::IDD)
{
    //{{AFX_DATA_INIT(CECHttpListView)
    // NOTE: the ClassWizard will add member initialization here
    //}}AFX_DATA_INIT
}


CECHttpListView::~CECHttpListView()
{}


void CECHttpListView::DoDataExchange(CDataExchange* pDX)
{
    CFormView::DoDataExchange(pDX);
    //{{AFX_DATA_MAP(CECHttpListView)
    DDX_Control(pDX, IDC_HTTP_STATS_CTRL, m_ctlHttpStatsList);
    //}}AFX_DATA_MAP
}



BEGIN_MESSAGE_MAP(CECHttpListView, CFormView)
//{{AFX_MSG_MAP(CECHttpListView)
ON_WM_SIZE()
//}}AFX_MSG_MAP
ON_WM_ERASEBKGND()
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CECHttpListView diagnostics

#ifdef _DEBUG
void CECHttpListView::AssertValid() const
{
    CFormView::AssertValid();
}

void CECHttpListView::Dump(CDumpContext& dc) const
{
    CFormView::Dump(dc);
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CECHttpListView message handlers


void CECHttpListView::OnSize(UINT nType, int cx, int cy)
{
    CFormView::OnSize(nType, cx, cy);

    if(m_ctlHttpStatsList.GetSafeHwnd())
        m_ctlHttpStatsList.MoveWindow(0,0,cx,cy);
}

void CECHttpListView::Initialize(int nRunIndex)
{
    CString strTestRun;
    g_pGlobalSettings->GetRunInfo(nRunIndex, strTestRun);

    CString strTestName;
    pEventSink->m_pVirtGrid->GetTestName(strTestName);
    //Set window title
    CString title;
    title.Format("Http Statistics - %s %s", strTestName,strTestRun);
    //  GetDocument()->SetTitle(title);

    //Remove scroll bars from view
    SIZE sz= {0};
    SetScrollSizes(MM_TEXT,sz);

    //size ocx to fit
    CRect rc;
    GetClientRect(&rc);
    m_ctlHttpStatsList.MoveWindow(rc);

    // Get the path for the data file
    CYRServer *pServer = (CYRServer*)AfxGetMainWnd()->SendMessage(WMU_GET_SERVERPTR);
    CString strPath = pServer->GetFolderPath(CYRServer::enTestsFolder);
    strPath += strTestName + "\\" + strTestRun + "\\";

    CString strStatTarget = strPath + "HTTP_ALL.stat";
    CString strURLTarget = strPath + "HTTP_ALL.urls";
    CFile fStatTarget(strStatTarget,CFile::modeCreate|CFile::modeWrite);
    CFile fUrlTarget(strURLTarget,CFile::modeCreate|CFile::modeWrite);

    CStringArray arstrFiles;
    pServer->GetFileList(CYRServer::enTestsFolder,strTestName+ "\\" + strTestRun,arstrFiles);

    //Check out and concatenate all the http files in
    //this test run directory
    DWORD dwRead=0;
    const DWORD dwBuffSize=4096;
    BYTE *pBuffer = new BYTE[dwBuffSize];
    for(int nFile=0;nFile<arstrFiles.GetSize();nFile++)
    {
        //Check if this is a http file
        CString &strFilename = arstrFiles[nFile];
        if(strFilename.Left(4).CompareNoCase("TEW_") != 0)
            continue;

        if(strFilename.Right(5).CompareNoCase(".stat") != 0)
            continue;

        CString strStatFile = strFilename;
        CString strUrlFile = strFilename.SpanExcluding(".") + ".urls";

        pServer->CheckOutFile(CYRServer::enTestsFolder,strTestName+ "\\" + strTestRun,strStatFile);
        pServer->CheckOutFile(CYRServer::enTestsFolder,strTestName+ "\\" + strTestRun,strUrlFile);

        // Get the path for the data file and open it
        CFile f(strPath + strStatFile,CFile::modeRead|CFile::typeBinary);

        DWORD dwLength = f.GetLength();

        //copy source file to target
        while(dwLength>0)
        {
            if(dwLength > dwBuffSize)
                dwRead = f.Read(pBuffer,dwBuffSize);
            else
                dwRead = f.Read(pBuffer,dwLength);

            fStatTarget.Write(pBuffer,dwRead);
            dwLength -= dwRead;
        }
        f.Close();


        // Get the path for the url file and open it
        f.Open(strPath + strUrlFile,CFile::modeRead|CFile::typeBinary);

        dwLength = f.GetLength();

        //copy source file to target
        while(dwLength>0)
        {
            if(dwLength > dwBuffSize)
                dwRead = f.Read(pBuffer,dwBuffSize);
            else
                dwRead = f.Read(pBuffer,dwLength);

            fUrlTarget.Write(pBuffer,dwRead);
            dwLength -= dwRead;
        }
        f.Close();
    }

    fStatTarget.Close();
    fUrlTarget.Close();
    delete []pBuffer;

    m_ctlHttpStatsList.LoadStatsFile(strStatTarget);

    DeleteFile(strStatTarget);
    DeleteFile(strURLTarget);
}

BOOL CECHttpListView::OnEraseBkgnd(CDC *pDC)
{
    return true;
}

void CECHttpListView::OnInitialUpdate()
{
    CFormView::OnInitialUpdate();

    CRect rc;
    GetClientRect(&rc);
    m_ctlHttpStatsList.MoveWindow(rc);
    SIZE sz= {0};
    SetScrollSizes(MM_TEXT,sz);
}
