/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

////////////////////////////////////////////////////////////////////////////////
// @doc     Document
// @module  CECVirtGridView | Hidded calculation grid to store and
//          manipulate the test run data.
//
////////////////////////////////////////////////////////////////////////////////

#if !defined(AFX_CECVIRTGRIDVIEW_H__0BC06A7D_4342_11D3_BE11_00A02456D506__INCLUDED_)
#define AFX_CECVIRTGRIDVIEW_H__0BC06A7D_4342_11D3_BE11_00A02456D506__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#pragma pack (1)
typedef struct
{
    DWORD dwTimeStamp;
    DWORD dwIP;     //IP Address one byte per field
    DWORD wPID;     //Process Id
    WORD wUser;     //User Number
    DWORD dwResponseTime;
    WORD wResponseCode;
    DWORD dwReplySize;
    int nURLNdx;
}
HttpData;
#pragma pack()

typedef CArray<HttpData*,HttpData*> HttpDataArray;

typedef struct
{
    DWORD dwStartTime;
    //  DWORD dwEndTime;
    int nValue;
}
OtaQueryData;

typedef CList <OtaQueryData*,OtaQueryData*> OtaDataList;

typedef struct
{
    CString strQuery;
    CString strNode;
    OtaDataList listQueryData;
}
OtaQuery;

typedef CArray<OtaQuery*,OtaQuery*> OtaQueryArray;

struct Timer
{
    DWORD dwTimeStamp;
    double dValue;
    DWORD dwActiveThreads;
    CString strTestName;
    CString strThreadName;
    CString strScriptName;
    CString strTimerName;
    bool bInFilter;

    Timer()
    {
        bInFilter = true;
    }
};

struct TimerInterval
{
    double dInterval;
    double dFrequency;
    double dCumFreq;
    double dPercentFreq;
};

struct RtAtData
{
    DWORD dwNumThreads;
    double dTime;
};

typedef struct
{
    double dMax;
    double dMin;
    double dMean;
    double dMedian;
    double dUpperQuartile;
    double dLowerQuartile;
    double dStdDev;

    CList <Timer*,Timer*> listTimers;
    CList <RtAtData*,RtAtData*> listRtAtData;
    CArray <TimerInterval*,TimerInterval*> arTimerIntervals;
}
TimerData;

struct TestRunData
{
    HttpDataArray* parHttpData;
    OtaQueryArray* parOtaData;
    TimerData* pTimerData;
    CMap <CString,LPCTSTR,int,int> mapUrls;

    DWORD dwStartTime;
    DWORD dwEndTime;

    TestRunData()
    {
        dwEndTime = 0;
        dwStartTime = 0xffffffff;
        parHttpData = NULL;
        parOtaData = NULL;
        pTimerData = NULL;
    }
};

class CYRServer;
class CECVirtGridView
{

    // @access Public Members
public:

    CECVirtGridView();
    virtual ~CECVirtGridView();

    BOOL AddTestRun(CString strTestName, CString strTestRunName, BOOL &bTimers, BOOL &bOTA,BOOL &bHttp);

    OtaQueryArray* GetOtaData(const CString& strTestRun);
    TimerData* GetTimerData(const CString& strTestRun);
    HttpDataArray* GetHttpData(const CString& strTestRun);
    void GetURLs(const CString& strTestRun, CStringArray& arstrURLs);

    int GetNumTimers(const CString& strTestRun);
    void GetTestName(CString& strTestName)
    {
        strTestName = m_strTestName;
    }
    // @cmember Modify the data selection
    void ModifySelection();

    BOOL GetHasHttp(int nRunIndex);
    BOOL GetHasOTA(int nRunIndex);
    BOOL GetHasTimers(int nRunIndex);

    DWORD GetStartTime(int nRunIndex);
    DWORD GetStopTime(int nRunIndex);
private:
    void LoadTimerData(const CString& strTestRun);
    void LoadOTAData(const CString& strTestRun);
    void LoadHttpData(const CString& strTestRun);

    int GetNextField(char sep, CString &inStr, CString &field);
    BOOL ApplyFilters(const CString& strTestRun);
    int GetHttpFiles(const CString& strTestRun,CStringArray& arstrHttpFiles);
    void CalculateTimerStats(const CString& strTestRun);

    static int SortHttpData( const void *arg1, const void *arg2 );

    CMap <CString,LPCTSTR,TestRunData*,TestRunData*> m_mapTestRunToData;

    static int SortTimers( const void *arg1, const void *arg2 );
    CYRServer *m_pServer;

    CString m_strTestName;
};
#endif // !defined(AFX_CECVIRTGRIDVIEW_H__0BC06A7D_4342_11D3_BE11_00A02456D506__INCLUDED_)
