/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// GraphTypes.h: interface for the CTimerGraph class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_GRAPHTYPES_H__24021AB3_F437_11D3_BD83_00C04F89E392__INCLUDED_)
#define AFX_GRAPHTYPES_H__24021AB3_F437_11D3_BD83_00C04F89E392__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "AnalyzerDefs.h"
#include "CECVirtGridView.h"

#define WMU_DATA_CHANGED (WM_APP+1)

#define GRAPH_MENU_ITEM_ID_START 0x8000
#define GRAPH_MENU_ITEM_ID_END 0x8005

#pragma warning (disable:4172)

#include "GraphView.h"

//CGraphTypeBase is the base class for all the graph types and
//controls the basic style of the graph,eg background , title
class CGraphTypeBase
{
protected:
    CGraphTypeBase(CGraphView *,const CString& strTitle);

public:
    virtual ~CGraphTypeBase();
    virtual void SetTitleComponent();
    virtual void SetDisplayComponent();
    virtual void SetGraphType(int nType);
    virtual int GetGraphType();

    // @cmember Apply the data line colours restored from the registry
    virtual void SetDataColours();

    // @cmember Called to add a legend to the graph display
    virtual void DisplayLegend();

    virtual BOOL AddTestRun(int nRun)=0;
    virtual BOOL RemoveTestRun(int nRun);

    BOOL ExportToExcel();

    struct MenuItems
    {
        CString strItem;
        UINT uiCommandID;
    };
    typedef CArray<MenuItems,MenuItems&> MenuItemArray;

    virtual void GetMenuItems(MenuItemArray& arMenuItems);
    virtual void OnCommand(UINT uiCommandID,CWnd *pParent);

    //Add all the data from another graph to this one
    void AddDataFromGraph(CGraphView *m_pView);

    // @access Private Members
protected:
    int m_nNextGroup;
    int m_nDataRun;

    void SetValue(int nIndex,int nGroup,double dValue,const CString& strAnnotation = "");

    CGraphView *m_pView;
    CString m_strTitle;
};

//CTimerGraph is the base class of all the graphs which use timer data
class CTimerGraph : public CGraphTypeBase
{
protected:
    CTimerGraph(CGraphView *,const CString& strTitle);
    virtual ~CTimerGraph();

    virtual BOOL AddTestRun(int nRun);
protected:
    virtual const double& GetDataPoint(TimerInterval*)=0;
};

class CCumFreqGraph : public CTimerGraph
{
public:
    CCumFreqGraph(CGraphView *);
    virtual ~CCumFreqGraph();
protected:
    const double& GetDataPoint(TimerInterval*);
};

class CStdFreqGraph : public CTimerGraph
{
public:
    CStdFreqGraph(CGraphView *);
    virtual ~CStdFreqGraph();

protected:
    const double& GetDataPoint(TimerInterval*);
};


class CPercentFreqGraph : public CTimerGraph
{
public:
    CPercentFreqGraph(CGraphView *);
    virtual ~CPercentFreqGraph();
protected:
    const double& GetDataPoint(TimerInterval*);
};


class CRTATGraph : public CTimerGraph
{
public:
    CRTATGraph(CGraphView *);
    virtual ~CRTATGraph();

    BOOL AddTestRun(int nRun);

private:
    const double& GetDataPoint(TimerInterval*)
    {
        return 0;
    }
};

class CRTTimeGraph : public CTimerGraph
{
public:
    CRTTimeGraph(CGraphView *);
    virtual ~CRTTimeGraph();

    BOOL AddTestRun(int nRunIndex);

private:
    const double& GetDataPoint(TimerInterval*)
    {
        return 0;
    }
};

class CUsersTimeGraph : public CTimerGraph
{
public:
    CUsersTimeGraph(CGraphView *);
    virtual ~CUsersTimeGraph();

    BOOL AddTestRun(int nRunIndex);
private:
    const double& GetDataPoint(TimerInterval*)
    {
        return 0;
    }
};

//COTAGraph is the base class of all the graphs which use OTA data
class COTAGraph : public CGraphTypeBase
{
public:
    COTAGraph(const CString& strQueryName,CGraphView *);
    virtual ~COTAGraph();

    virtual BOOL AddTestRun(int nRun)
    {
        return AddTestRun(nRun,m_strQueryName);
    }
    virtual BOOL AddTestRun(int nRun,const CString& strQueryName);

    virtual void GetMenuItems(MenuItemArray& arMenuItems);
    virtual void OnCommand(UINT uiCommandID,CWnd *pParent);

    BOOL AddQuery(int nRunIndex,const CString& strQueryName,const CString& strNode);
    BOOL AddTimer(int nRunIndex,const CString& strTimer);

    int GetGraphType();
protected:
    COTAGraph(CGraphView *,const CString& strTitle);
    CString m_strCurrentNode;
    CString m_strQueryName;
    int m_nNumTimers;

    int GetNumInjectors();
    ;
    typedef enum{enMenuItemNode=GRAPH_MENU_ITEM_ID_START,enMenuItemCustomise}MenuItemIDs;
};


class CPercentCPUGraph : public COTAGraph
{
public:
    CPercentCPUGraph(CGraphView *);
    virtual ~CPercentCPUGraph();
};


class CNetUseGraph : public COTAGraph
{
public:
    CNetUseGraph(CGraphView *);
    virtual ~CNetUseGraph();
};


class CPageSwapGraph : public COTAGraph
{
public:
    CPageSwapGraph(CGraphView *);
    virtual ~CPageSwapGraph();
};

class CCustomOTAGraph : public COTAGraph
{
public:
    CCustomOTAGraph(CGraphView*);
    virtual ~CCustomOTAGraph();

    virtual BOOL AddTestRun(int nRun);
    virtual void SetGraphType(int nType);

private:
    void GetMenuItems(MenuItemArray& arMenuItems);
    void OnCommand(UINT uiCommandID,CWnd *pParent);

    void ClearData();

    CStringArray m_arstrTimers;
    CStringArray m_arstrQueries;
};

//CHttpGraph is the base class of all the graphs which use HTTP data
class CHttpGraph : public CGraphTypeBase
{
    typedef enum{enMenuItemFilter=GRAPH_MENU_ITEM_ID_START}MenuItemIDs;
protected:
    CHttpGraph(CGraphView *,const CString& strTitle);
    virtual ~CHttpGraph();
    virtual BOOL AddTestRun(int nRun);

    virtual void GetMenuItems(MenuItemArray& arMenuItems);
    virtual void OnCommand(UINT uiCommandID,CWnd *pParent);

    CDWordArray m_ardwURLFilters[20];   //This means that analyzer will Explode if
    //someone adds more than 20 http graphs
    bool m_bHasURLFilter;

    BOOL IsURLInFilter(int nURL,int nGroup);
};

class CThroughputGraph : public CHttpGraph
{
public:
    CThroughputGraph(CGraphView *);
    virtual ~CThroughputGraph();

protected:
    BOOL AddTestRun(int nRun);
};


class CHitsPerSecondGraph : public CHttpGraph
{
public:
    CHitsPerSecondGraph(CGraphView *);
    virtual ~CHitsPerSecondGraph();

protected:
    BOOL AddTestRun(int nRun);
};


class CAvPageRTPerUserGraph : public CHttpGraph
{
public:
    CAvPageRTPerUserGraph(CGraphView *);
    virtual ~CAvPageRTPerUserGraph();

protected:
    BOOL AddTestRun(int nRun);
};

class CErrorsPerUserGraph : public CHttpGraph
{
public:
    CErrorsPerUserGraph(CGraphView *);
    virtual ~CErrorsPerUserGraph();

protected:
    BOOL AddTestRun(int nRun);
};


class CErrorsTimeGraph : public CHttpGraph
{
public:
    CErrorsTimeGraph(CGraphView *);
    virtual ~CErrorsTimeGraph();

protected:
    BOOL AddTestRun(int nRun);
};


class CPagesPerSecondGraph : public CHttpGraph
{
public:
    CPagesPerSecondGraph(CGraphView *);
    virtual ~CPagesPerSecondGraph();
protected:
    BOOL AddTestRun(int nRunIndex);
};

#endif // !defined(AFX_GRAPHTYPES_H__24021AB3_F437_11D3_BD83_00C04F89E392__INCLUDED_)
