/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// CECProjectViewerDlg.cpp : implementation file
//

#include "stdafx.h"
#include "CyrEventSink.h"
#include "CECProjectViewerDlg.h"
#include "..\..\Common\CYRExternalFile.h"
//
//    Product includes
//
#include "..\..\Interfaces\CyrPlugins.h"
#include "..\..\Common\CECRegistryTools.h"
#include "..\..\Common\CECClipboard.h"
#include "..\..\Common\CECTargetEnvironment.h"

#include "Solutions.h"
#include "server.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

extern CYREventSink* pEventSink;
/////////////////////////////////////////////////////////////////////////////
// CECProjectViewerDlg dialog


CECProjectViewerDlg::CECProjectViewerDlg(CWnd* pParent /*=NULL*/)
        : CDialog(CECProjectViewerDlg::IDD, pParent)
{
    //{{AFX_DATA_INIT(CECProjectViewerDlg)
    // NOTE: the ClassWizard will add member initialization here
    //}}AFX_DATA_INIT
    m_szActiveProject = "";
}


void CECProjectViewerDlg::DoDataExchange(CDataExchange* pDX)
{
    CDialog::DoDataExchange(pDX);
    //{{AFX_DATA_MAP(CECProjectViewerDlg)
    DDX_Control(pDX, IDC_XPROJECTVIEWERCTRL, m_ProjView);
    //}}AFX_DATA_MAP
}



BEGIN_MESSAGE_MAP(CECProjectViewerDlg, CDialog)
//{{AFX_MSG_MAP(CECProjectViewerDlg)
ON_WM_SIZE()
ON_WM_PAINT()
ON_WM_ERASEBKGND()
//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CECProjectViewerDlg message handlers

void CECProjectViewerDlg::OnSize(UINT nType, int cx, int cy)
{
    CDialog::OnSize(nType, cx, cy);

    if (m_ProjView.m_hWnd != NULL)
    {
        m_ProjView.SetWindowPos(NULL,0,0,cx,cy,SWP_NOZORDER);
    }

}

BEGIN_EVENTSINK_MAP(CECProjectViewerDlg, CDialog)
//{{AFX_EVENTSINK_MAP(CECProjectViewerDlg)
ON_EVENT(CECProjectViewerDlg, IDC_XPROJECTVIEWERCTRL, 1 /* XChangeActiveProject */, OnXChangeActiveProject, VTS_BSTR)
ON_EVENT(CECProjectViewerDlg, IDC_XPROJECTVIEWERCTRL, 2 /* XExecuteTest */, OnXExecuteTest, VTS_BSTR)
ON_EVENT(CECProjectViewerDlg, IDC_XPROJECTVIEWERCTRL, 3 /* XOpenTestRun */, OnXOpenTestRun, VTS_BSTR VTS_BSTR)
ON_EVENT(CECProjectViewerDlg, IDC_XPROJECTVIEWERCTRL, 8 /* XExportFile */, OnXExportFile, VTS_BSTR)
ON_EVENT(CECProjectViewerDlg, IDC_XPROJECTVIEWERCTRL, 5 /* XViewResults */, OnXViewResults, VTS_BSTR VTS_BSTR VTS_I2)
ON_EVENT(CECProjectViewerDlg, IDC_XPROJECTVIEWERCTRL, 10 /* XViewHttpStats */, OnXViewHttpStatsXprojectviewerctrl, VTS_BSTR VTS_BSTR)
ON_EVENT(CECProjectViewerDlg, IDC_XPROJECTVIEWERCTRL, 1 /* XChangeActiveProject */, OnXChangeActiveProject, VTS_NONE)
ON_EVENT(CECProjectViewerDlg, IDC_XPROJECTVIEWERCTRL, 2 /* XExecuteTest */, OnXExecuteTest, VTS_NONE)
ON_EVENT(CECProjectViewerDlg, IDC_XPROJECTVIEWERCTRL, 7 /* XRequestNewScript */, OnXRequestNewScript, VTS_NONE)
//}}AFX_EVENTSINK_MAP
END_EVENTSINK_MAP()

// @mfunc
//
// Event handler for when the user has changed to another project
//
// @parm LPCRSTR | ProjectName | The name of the new project to switch to
//
void CECProjectViewerDlg::OnXChangeActiveProject(LPCTSTR ProjectName)
{
    m_szActiveProject = ProjectName;
    //
    //    ... set the project in the framework
    //
    AfxGetMainWnd()->SendMessage (WMU_SET_PROJECT ,0 ,(LPARAM)ProjectName);
}

// @mfunc
//
// Event handler to execute a given test
//
// @parm LPCTSTR | TestName | The name of the test to execute
//
void CECProjectViewerDlg::OnXExecuteTest(LPCTSTR TestName)
{
    pEventSink->ExecuteTest(TestName);
}

// @mfunc
//
// Event handler to open a specified set of results
//
// @parm LPCTSTR | TestName | The name of the test to which the test run belongs
// @parm LPCTSTR | TestRunName | The Test run name to opened
//
void CECProjectViewerDlg::OnXOpenTestRun(LPCTSTR TestName, LPCTSTR TestRunName)
{
    CWnd* pMainWnd = AfxGetMainWnd();

    //GEnerate param for Test run in analyzer
    CString strAnalyzerParam;
    strAnalyzerParam.Format ("%s|%s|%s" ,SOLUTIONS_ANALYZER_DLL ,TestName,TestRunName);

    //tell analyzer to open test run
    pMainWnd->SendMessage (WMU_FIRE_SPECIFIC_EVENT ,(WPARAM )WMU_OPEN_TEST_RUN,(LPARAM)&strAnalyzerParam);
}


// @mfunc
//
// Sets the current Project In the Project Viewer
//
// @parm CString | szProjectName | The name of the project to activate
//
void CECProjectViewerDlg::SetActiveProject(CString szProjectName)
{
    m_szActiveProject = szProjectName;
    m_ProjView.SetActiveProject(szProjectName);
    if (szProjectName != "" && szProjectName != "<No Project Selected>")
    {
        m_ProjView.XRefresh();
    }
}

// @mfunc
//
// Initializes the dialog, sending all necessary information to the project viewer
// control.
//
// @rdesc BOOL | Initialization has been completed.
//
BOOL CECProjectViewerDlg::OnInitDialog()
{
    CDialog::OnInitDialog();

    // Now get the server login information from the framework
    CYRServer *pServer = (CYRServer *)AfxGetMainWnd()->SendMessage (WMU_GET_SERVERPTR);
    CString szUser = pServer->GetUserName();
    CString szPass = pServer->GetPassword();

    char *pszPrefix = (char*) AfxGetMainWnd()->SendMessage (WMU_GET_PRODUCT_PREFIX);
    CString szPrefix(pszPrefix);
    free(pszPrefix);

    m_ProjView.SetServer((long)pServer);
    m_ProjView.XInitialize(szPrefix, "TEST_TREE|NODE_LIST|SOURCE_TREE|OUTPUT_TREE");
    return TRUE;  // return TRUE unless you set the focus to a control
    // EXCEPTION: OCX Property Pages should return FALSE
}

// @mfunc
//
// Event handler for when the user has requested to create a new node
//

void CECProjectViewerDlg::OnXRequestNewNode()
{
    //
    // Send a message to invoke the node definition dialog
    //
    CWnd* pMainWnd = AfxGetMainWnd();
    ASSERT (pMainWnd != NULL);

    pMainWnd->SendMessage (WMU_FIRE_GLOBAL_EVENT ,(LPARAM)STUDIO_CUSTOM_LINK_3);
}

// @mfunc
//
// Event handler for when the user has selected to export a file from the repository
//
// @parm LPCTSTR | strFileName | The filename to be exported
void CECProjectViewerDlg::OnXExportFile(LPCTSTR strFileName)
{
    CYRExternalFile ex;
    ex.Export(strFileName);
}

// @mfunc
//
// Called to refresh the Project Viewer Control updating it's contents from
// the repository.
//
void CECProjectViewerDlg::RefreshView()
{
    m_ProjView.XRefresh();
}

void CECProjectViewerDlg::OnXViewResults(LPCTSTR TestName, LPCTSTR TestRunName, short nType)
{
    CString text;
    text.Format("%s|%s|%d", TestName, TestRunName, nType);
    CECClipboard cb;
    cb.SetClipboardText(text);

    CWnd* pMainWnd = AfxGetMainWnd();
    pMainWnd->SendMessage(WMU_FIRE_GLOBAL_EVENT, WMU_VIEW_RESULTS);
}

void CECProjectViewerDlg::OnOK()
{
}


void CECProjectViewerDlg::OnCancel()
{
}


void CECProjectViewerDlg::OnPaint()
{
    CPaintDC dc(this); // device context for painting

    // TODO: Add your message handler code here

    // Do not call CDialog::OnPaint() for painting messages
}


BOOL CECProjectViewerDlg::OnEraseBkgnd(CDC *pDC)
{
    return TRUE;
}

void CECProjectViewerDlg::OnXViewHttpStatsXprojectviewerctrl(LPCTSTR TestName, LPCTSTR TestRunName)
{
    CString text;
    text.Format("%s|%s", TestName, TestRunName);

    CECClipboard cb;
    cb.SetClipboardText(text);

    CWnd* pMainWnd = AfxGetMainWnd();
    pMainWnd->SendMessage(WMU_FIRE_GLOBAL_EVENT, WMU_VIEW_HTTP_STATS);
}

void CECProjectViewerDlg::OnXRequestNewScript()
{
    CECTargetEnvironment te;
    te.InvokeModeller();
}
