/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// CECSourceEditDoc.cpp : implementation file
//

#include "stdafx.h"
#include "resource.h"
#include "CECSourceEditDoc.h"
#include "..\..\Interfaces\CyrPlugIns.h"
#include "..\..\Common\CECXServerError.h"
#include "..\..\Common\CECSourceFileUtils.h"
#include <rw\ctoken.h>
#include "..\..\Common\CECClipboard.h"
#include "Solutions.h"

#include "CyrStAPI\CyrStLib.h"
#include "CyrStAPI\CECComSCLFile.h"
#include "..\..\framework\Commander.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CECSourceEditDoc

IMPLEMENT_DYNCREATE(CECSourceEditDoc, CDocument)

CECSourceEditDoc::CECSourceEditDoc()
{
    m_bClosingFrame = false;
}

BOOL CECSourceEditDoc::OnNewDocument()
{
    if (!CDocument::OnNewDocument())
        return FALSE;
    return TRUE;
}

CECSourceEditDoc::~CECSourceEditDoc()
{}



BEGIN_MESSAGE_MAP(CECSourceEditDoc, CDocument)
//{{AFX_MSG_MAP(CECSourceEditDoc)
ON_COMMAND(ID_FILE_SAVE, OnFileSave)
//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CECSourceEditDoc diagnostics

#ifdef _DEBUG
void CECSourceEditDoc::AssertValid() const
{
    CDocument::AssertValid();
}

void CECSourceEditDoc::Dump(CDumpContext& dc) const
{
    CDocument::Dump(dc);
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CECSourceEditDoc serialization

void CECSourceEditDoc::Serialize(CArchive& ar)
{
    if (ar.IsStoring())
    {
        // TODO: add storing code here
    }

    else
    {
        // TODO: add loading code here
    }

}

/////////////////////////////////////////////////////////////////////////////
// CECSourceEditDoc commands

BOOL CECSourceEditDoc::OnOpenDocument(LPCTSTR lpszPathName)
{
    BeginWaitCursor();
    m_szFilePath = lpszPathName;
    CWnd* pMainWnd = AfxGetMainWnd();

    //
    // Check the file out from the repository.
    //
    CString checkOutInfo;
    checkOutInfo.Format("%s|%s|%s" ,SOLUTIONS_SC_DLL ,GetFileName(), CECComSourceFolder::FOLDER_NAME);

    BOOL bIsInSCC = pMainWnd->SendMessage (WMU_FIRE_SPECIFIC_EVENT
                                           ,(WPARAM )WMU_SC_ISINSCC
                                           ,(LPARAM) &checkOutInfo);

    LRESULT retVal=1;
    if(bIsInSCC)
    {
        retVal = pMainWnd->SendMessage (WMU_FIRE_SPECIFIC_EVENT
                                        ,(WPARAM )WMU_SC_CHECKOUT
                                        ,(LPARAM) &checkOutInfo);
    }

    //
    // Open and read the file on the local machine
    //
    BOOL bReadOK=FALSE;
    if (m_Edit.ReadFile(lpszPathName) == OE_SUCCESS)
    {
        bReadOK = TRUE;
        SetSyntaxColouring();
    }

    //
    // If the file was checked out successfully allow editing.
    //
    if (retVal == 0)
    {
        m_Edit.SetReadOnly(FALSE);
    }
    EndWaitCursor();
    return bReadOK;
}


BOOL CECSourceEditDoc::OnSaveDocument(LPCTSTR lpszPathName)
{
    CWaitCursor wc;

    //
    // Write the local file to disk and compile the source file (if its not a template)
    //
    if (m_Edit.WriteFile(lpszPathName) == OE_SUCCESS && !IsTemplateFile())
    {
        CECSourceFileUtils utils(lpszPathName);
        if (!utils.Compile())
        {
            //
            //    The compile failed, check with the user whether to really close the window and if not
            //    then we will simply continue and not save the file back into the repository
            //
            if (m_bClosingFrame && AfxMessageBox(IDS_ERR_COMPILEFAIL ,MB_YESNO | MB_ICONQUESTION) != IDYES)
            {
                m_Edit.SetModified(TRUE);
                SetModifiedFlag(TRUE);
                return FALSE;
            }
        }
    }

    //
    //    Get a handle to our main window so we can keep it informed
    //
    CWnd* pMainWnd = AfxGetMainWnd();
    ASSERT (pMainWnd != NULL);

    //
    //  If the file is a template it shouldn't be checked in
    //
    if(!IsTemplateFile())
    {
        //Get working directory and compare it to the file path
        //if they match we can check the file into SCC
        CString szDll(SOLUTIONS_GENERIC_DLL);
        CString szWorkDir = *(CString*)pMainWnd->SendMessage(WMU_FIRE_SPECIFIC_EVENT,WMU_GET_PROJECT_WORKDIR,(LPARAM)&szDll);
        szWorkDir += "\\"+GetFileName();
        if(szWorkDir.CompareNoCase(m_szFilePath) == 0)
        {

            //
            // Check the file back in to the repository.
            //
            CString checkInInfo;
            checkInInfo.Format("%s|%s|%s" ,SOLUTIONS_SC_DLL ,GetFileName(), CECComSourceFolder::FOLDER_NAME);

            BOOL bIsInSCC = pMainWnd->SendMessage (WMU_FIRE_SPECIFIC_EVENT
                                                   ,(WPARAM )WMU_SC_ISINSCC
                                                   ,(LPARAM) &checkInInfo);
            if(bIsInSCC)
            {
                //Lock the file if we can
                CString strLock = checkInInfo + "|1";
                pMainWnd->SendMessage (WMU_FIRE_SPECIFIC_EVENT
                                       ,(WPARAM )WMU_SC_SETLOCK
                                       ,(LPARAM) &strLock);

                LRESULT retVal = pMainWnd->SendMessage (WMU_FIRE_SPECIFIC_EVENT
                                                        ,(WPARAM )WMU_SC_CHECKIN
                                                        ,(LPARAM) &checkInInfo);
            }
            else
            {
                LRESULT retVal = pMainWnd->SendMessage (WMU_FIRE_SPECIFIC_EVENT
                                                        ,(WPARAM )WMU_SC_ADDFILE
                                                        ,(LPARAM) &checkInInfo);
            }
        }
    }

    CString szDll(SOLUTIONS_GENERIC_DLL);
    pMainWnd->SendMessage(WMU_FIRE_SPECIFIC_EVENT,WMU_REFRESH_TREE_VIEWS,(LPARAM)&szDll);

    return TRUE;
}

void CECSourceEditDoc::SetSyntaxColouring ()
{
    CWinApp* pApp = AfxGetApp();

    //
    //    Determine type of this file
    //
    CString strExt = m_szFilePath.Right(3);
    strExt.MakeUpper();

    CWnd* pMainWnd = AfxGetMainWnd();
    CStringArray* extensions = (CStringArray*) pMainWnd->SendMessage (WMU_GET_KNOWN_EXTENSIONS);

    BOOL bExtFound = FALSE;

    int nCount = extensions->GetSize();
    for (int isub=0; isub<nCount; isub++)
    {

        if (strExt.CompareNoCase(extensions->GetAt(isub)) == 0)
        {
            bExtFound = TRUE;
            break;
        }
    }

    if (bExtFound)
    {
        CString regKey;
        regKey.Format("Editor\\%s Files", strExt);

        // Check if key exists
        CString strKeywords = "Keywords";
        CString retString = pApp->GetProfileString(regKey,strKeywords,"NULL");
        if (retString != "NULL")
        {
            m_Edit.SetFileMappingKey(regKey);
            m_Edit.InitFileTypeSettings();
            return;

        }
        else
        {
            m_Edit.SetFileMappingKey("No Colouring");
            m_Edit.InitFileTypeSettings();
            return;
        }

    }

    else
    {

        //
        //    All other file types do not have syntax colouring.
        //
        m_Edit.SetFileMappingKey("No Colouring");
    }

    m_Edit.InitFileTypeSettings();
    return;
}


BOOL CECSourceEditDoc::SaveModified()
{
    SetModifiedFlag(m_Edit.GetModified());

    return CDocument::SaveModified();
}

CString CECSourceEditDoc::GetFileName()
{
    int nLen = m_szFilePath.GetLength();
    int nPos = m_szFilePath.ReverseFind('\\');
    if (nPos == -1)
    {
        return "";
    }

    return m_szFilePath.Right(nLen-nPos-1);
}

void CECSourceEditDoc::OnCloseDocument()
{

    CDocument::OnCloseDocument();
}

//
//  IsTemplateFile
//  ==============
//
//  If this file is in the template directory return true
//
BOOL CECSourceEditDoc::IsTemplateFile()
{
    //Generate the path for this file if it was a template...
    CECCommanderApp* pApp = (CECCommanderApp*)AfxGetApp();
    CString strTemplateDir = pApp->GetInstallationDirectory();
    strTemplateDir += "\\Templates\\"+GetFileName();

    //... and compare it to the actual path, If they match its a template.
    return (m_szFilePath.CompareNoCase(strTemplateDir)==0) ? true : false;
}

void CECSourceEditDoc::OnFileSave()
{
    DoFileSave();
}

//
//  DoFileSave()
//  ============
//
//  Over written CDocument function prevents the showing of the file save dialog
//
BOOL CECSourceEditDoc::DoFileSave()
{
    DWORD dwAttrib = GetFileAttributes(m_szFilePath);
    if(dwAttrib & FILE_ATTRIBUTE_READONLY)
    {
        CString strMsg;
        strMsg.Format("The file \"%s\" is Read Only.\n\nDo you want to overwrite it?",m_szFilePath);
        if(AfxMessageBox(strMsg,MB_ICONWARNING|MB_YESNO)==IDNO)
            return false;

        dwAttrib &= ~FILE_ATTRIBUTE_READONLY;
        SetFileAttributes(m_szFilePath,dwAttrib);
    }

    //The file didn't exist so we need to save and refresh the sources
    return OnSaveDocument(m_szFilePath);
}

//
//  CanCloseFrame
//  =============
//
//  use this function to set a bool to tell OnSaveDocument
//  that we are closing the document
//
BOOL CECSourceEditDoc::CanCloseFrame(CFrameWnd* pFrame)
{
    //CanCloseFrame calls the savemodified function which calls dofilesave
    //which calls onSaveDocument
    m_bClosingFrame = true;
    if(CDocument::CanCloseFrame(pFrame))
    {
        m_bClosingFrame = false;
        return true;
    }
    m_bClosingFrame = false;
    return false;
}
