/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

/////////////////////////////////////////////////////////////////////////
//
// CYREventSink.cpp
// ----------------
//
//    This is the implementation file for the CYREventSink class
//
//=========================================================================

#include "stdafx.h"
#include "CYREventSink.h"
#include "CECProjectViewerDlg.h"
#include "resource.h"
#include "..\..\Interfaces\CyrCtrls.h"
#include "..\..\Interfaces\CyrPlugIns.h"
#include "..\..\Common\CECDLLFunctions.h"
#include "..\..\Common\CECRegistryTools.h"
#include "..\..\Common\CECTargetEnvironment.h"
#include "Solutions.h"

#include "..\..\Common\CYRExternalFile.h"
#include "CYRNodeDefinitionDlg.h"
#include "CECSetWorkFolderDlg.h"
#include "CECOTAEditorDlg.h"
#include "CYRGroupTestWizard.h"
#include "Server.h"


#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

extern  CECDLLFunctions*        pDllFunctions;
extern  CECProjectViewerDlg*    pProjectViewerDlg;
extern  CString                 m_szManagerName;
extern BOOL ExecuteWizard(const CString& strWizName,const CString& strTestName);

/////////////////////////////////////////////////////////////////////////////
// CYREventSink
int CALLBACK BrowserCallbackProc(HWND hwnd,UINT uMsg,LPARAM lp, LPARAM /*pData*/)
{
    TCHAR szDir[MAX_PATH];

    switch(uMsg)
    {
    case BFFM_INITIALIZED:
        {
            if (GetCurrentDirectory(sizeof(szDir)/sizeof(TCHAR), szDir))
            {
                // WParam is TRUE since you are passing a path.
                // It would be FALSE if you were passing a pidl.
                SendMessage(hwnd,BFFM_SETSELECTION,TRUE,(LPARAM)szDir);
            }
            break;

        }
    case BFFM_SELCHANGED:
        {
            // Set the status window to the currently selected path.
            if (SHGetPathFromIDList((LPITEMIDLIST) lp ,szDir))
            {
                SendMessage(hwnd,BFFM_SETSTATUSTEXT,0,(LPARAM)szDir);
            }
            break;
        }
    default:
        {
            break;
        }
    }
    return 0;
}

CYREventSink::CYREventSink()
{
    // In release we have discovered that our extension DLL's behave in a different
    // manner to the debug builds. It appears that any member variables are not
    // persistant for the entirety of this class and unless static they lose their data
    // after leaving the constructor.

    // As a workaround we have found that if we set up a 16-byte char value ALL other
    // member variables then retain their values during this objects lifetime.

    // Don't ask me why - it's an MFC Release Build Memory Management thing ...
    (void) strcpy (EventSink, "CYRGeneric");

    // Update Handlers
    m_bExecuteEnabled = FALSE;
    m_pServer = (CYRServer *)AfxGetMainWnd()->SendMessage (WMU_GET_SERVERPTR);

    // Get the parent window for this event sink
    CWnd* pParentWnd = AfxGetMainWnd();
    if (pParentWnd != NULL)
    {
        HWND hParent = pParentWnd->GetSafeHwnd();

        // Create a default window size
        if (hParent != NULL)
        {
            CRect rect(0,0,0,0);

            // And create it
            Create(NULL, NULL, WS_CHILD, rect, pParentWnd, ID_GENERIC_EVENT_SINK);
        }
    }
}

CYREventSink::~CYREventSink()
{}



BEGIN_MESSAGE_MAP(CYREventSink, CWnd)
//{{AFX_MSG_MAP(CYREventSink)
ON_COMMAND(IDB_EXECUTE_TEST, OnExecuteTest)
ON_MESSAGE(STUDIO_SWITCH_PROJECTS, OnSwitchProjects)
ON_COMMAND(IDB_IMPORT_FILE, OnImportFile)
ON_COMMAND(IDB_NODE_DEFINITIONS, OnNodeDefinitions)
ON_MESSAGE(STUDIO_CUSTOM_LINK_3, OnNodeDefinitions)
ON_COMMAND(IDB_SET_WORKFOLDER, OnSetWorkFolder)
ON_MESSAGE(WMU_EDIT_NODE, OnEditNode)
ON_MESSAGE(WMU_REFRESH_TREE_VIEWS, OnRefreshTrees)
//}}AFX_MSG_MAP
ON_COMMAND(ID_NEW_GROUP_TEST, OnNewGroupTest)
ON_MESSAGE(WMU_EDIT_OTA_TOF,OnEditOTA)
END_MESSAGE_MAP()


/////////////////////////////////////////////////////////////////////////////
// CYREventSink message handlers

BOOL CYREventSink::IsEnabled(UINT nID)
{
    BOOL bState = TRUE;

    switch (nID)
    {
    case ID_NEW_GROUP_TEST:
    case IDB_NODE_DEFINITIONS:
    case IDB_EXECUTE_TEST:
    case IDB_IMPORT_FILE:
    case IDB_LOGGING:
        {
            bState = m_pServer->IsThereAnOpenProject();
            break;
        }
    case IDB_MONITOR_TEST:
        {
            bState = TRUE;
            break;
        }
    default:
        {
            bState = TRUE;
        }
    }
    return bState;
}

//
// OnSwitchProjects
// ================
//
// The function is called whenever switching projects - All we have to do is obtain
// the new project name which will already of been set up for us in the framework
//
LRESULT CYREventSink::OnSwitchProjects(WPARAM,LPARAM)
{
    if (pProjectViewerDlg != NULL)
    {
        //
        //    recover the active project from the framework
        //
        if (m_pServer->IsThereAnOpenProject())
        {
            // Obtain the new project name
            CString szProjectName = m_pServer->GetOpenProjectName();
            ;

            // And set it to be active
            pProjectViewerDlg->SetActiveProject(szProjectName);
        }
        else
        {
            pProjectViewerDlg->SetActiveProject("<No Project Selected>");
        }
    }
    return 0;
}

//
// OnExecuteTest
// =============
//
// This function is used to execute a particular test
//
afx_msg void CYREventSink::OnExecuteTest()
{
    // See if we have any Tests available in this project
    CStringArray arstrTests;
    m_pServer->GetFolderList(CYRServer::enTestsFolder,"",arstrTests);
    if(arstrTests.GetSize())
    {
        ExecuteTest();
    }
    else
    {
        AfxMessageBox("You have not yet created any tests.\n\nPlease create a test using the wizard and try again.",MB_ICONSTOP|MB_OK);
    }
}

//
// ExecuteTest
// ===========
//
// The execute test functionality is kept inside the Generic Controls DLL. To
// use execute the test we must therefore obtain a pointer to the execute test function
// in the Generic Controls DLL and let that do all the work.
//
void CYREventSink::ExecuteTest(CString TestName)
{
    // Get the target environment
    CECTargetEnvironment te;
    CString szContext;
    te.GetCurrentContext(szContext);

    if (szContext == "WEB")
    {
        CString szProjectName = m_pServer->GetOpenProjectName();

        CWnd* pMainWnd = AfxGetMainWnd();
        CString execInfo;
        execInfo.Format("%s|%s|%s" ,SOLUTIONS_IMPACT_DLL ,szProjectName, TestName);


        BOOL bExec = pMainWnd->SendMessage (WMU_FIRE_SPECIFIC_EVENT
                                            ,(WPARAM )WMU_EXECUTE_WEB
                                            ,(LPARAM) &execInfo);

        if (bExec)
        {
            pMainWnd->SendMessage (WMU_FIRE_GLOBAL_EVENT ,(LPARAM)WMU_MONITOR_TEST);
        }
        return;
    }

    CString name = "Generic Ctrls";

    // Get a pointer to the execute test function from the Generic Controls DLL
    if (pDllFunctions != NULL)
    {
        LPEXECUTEFUNC execFunc = (LPEXECUTEFUNC)pDllFunctions->GetFunctionPtr("Generic_ExecuteTest", name, FALSE);

        // And execute the test by first obtaining the test manager and project name
        if (execFunc != NULL)
        {
            CString szProjectName = m_pServer->GetOpenProjectName();

            if( execFunc(szProjectName, TestName, m_szManagerName))
            {
                // Send a message to monitor the test
                CWnd* pMainWnd = AfxGetMainWnd();
                pMainWnd->SendMessage (WMU_FIRE_GLOBAL_EVENT ,(LPARAM)WMU_MONITOR_TEST);

            }
        }
    }

}

//
// OnImportFile
// ============
//
// This function is used to import a file into the project
//
afx_msg void CYREventSink::OnImportFile()
{
    CYRExternalFile file;
    file.Import();
}

//
// OnNodeDefinitions
// =================
//
// This function is used to invoke (and store) the current node definitions
//
void CYREventSink::OnNodeDefinitions()
{
    CYRNodeDefinitionDlg dlg;
    dlg.DoModal();

    CWnd *pMainWnd = AfxGetMainWnd();
    CString szDll(SOLUTIONS_GENERIC_DLL);
    pMainWnd->SendMessage(WMU_FIRE_SPECIFIC_EVENT,WMU_REFRESH_TREE_VIEWS,(LPARAM)&szDll);
}

LRESULT CYREventSink::OnNodeDefinitions(WPARAM,LPARAM)
{
    OnNodeDefinitions();
    return 0;
}

//
// OnEditNode
// ==========
//
// This function is used to edit the specified node definition (passed as argument 1)
//
LRESULT CYREventSink::OnEditNode    (WPARAM Param1, LPARAM Param2)
{
    CString* pNodeToEdit = (CString*) Param1;
    CYRNodeDefinitionDlg dlg (pNodeToEdit);
    dlg.DoModal();
    return 0;
}


//
// OnSetWorkFolder
// ===============
//
// This function is used to change the default working folder for projects
//
afx_msg void CYREventSink::OnSetWorkFolder()
{
    CECSetWorkFolderDlg Dlg;
    if(Dlg.DoModal()==IDOK)
	{
		//Set the project in the framework
		CWnd* pMainWnd = AfxGetMainWnd();
		pMainWnd->SendMessage (WMU_SET_PROJECT ,0 ,(LPARAM)"");
	}
}

//
//  OnEditOTA
//  =========
//
//  This function is used to invoke the OTA Editor
//
LRESULT CYREventSink::OnEditOTA(WPARAM wParam,LPARAM)
{
    CString* pTofToEdit = (CString*) &wParam;

    CECOTAEditorDlg dlg(*pTofToEdit);
    dlg.DoModal();
    return 0;
}

//
//  OnRefreshTrees
//  ================
//
//  This function is used to redraw the projectviewer window
//
LRESULT CYREventSink::OnRefreshTrees(WPARAM,LPARAM)
{
    pProjectViewerDlg->RefreshView();
    return 0;
}

void CYREventSink::OnNewGroupTest()
{
    CWnd *pMainWnd = AfxGetMainWnd();
    CYRGroupTestWizardSheet dlg("Group Test Wizard",pMainWnd,0,"");
    if(dlg.DoModal()==ID_WIZFINISH)
    {
        pProjectViewerDlg->RefreshView();
    }
}
