/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

/////////////////////////////////////////////////////////////////////////
//
// CYRNodeDefinitionDlg.cpp
// ------------------------
//
//    This is the implementation file for the CYRNodeDefinitionDlg class
//
//=========================================================================

#include "stdafx.h"

#include "resource.h"
#include "Solutions.h"

#include "CYRNodeDefinitionDlg.h"
#include "..\..\Common\CECRegistryTools.h"
#include "..\..\Interfaces\CyrPlugIns.h"
#include "..\..\Common\CECOTAThresholds.h"
#include "..\..\Common\CECDefaultCodeWriter.h"
#include "..\..\Common\CYRTestProperties.h"

#include "..\..\..\HTMLHelp\CYRGeneric\Node_Def.inl"
#include "htmlhelp.h"
#include "server.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CYRNodeDefinitionDlg dialog
CYRNodeDefinitionDlg::CYRNodeDefinitionDlg(CString* pNodeToEdit ,CWnd* pParent /*=NULL*/)
        : CDialog(CYRNodeDefinitionDlg::IDD, pParent)
{
    //{{AFX_DATA_INIT(CYRNodeDefinitionDlg)
    m_strPhysicalNode = _T("");
    m_strNodeName = _T("");
    m_nNetUseThresh = 0;
    m_strCPUThresh = _T("0%");
    m_nPageSwapThresh = 0;
    //}}AFX_DATA_INIT
    m_szHtmlHelpFile = getenv("CYRANO_HOME");
    m_szHtmlHelpFile += "\\Framework\\Help\\Commander.chm";

    if (pNodeToEdit)
        m_strNodeName = *pNodeToEdit;

    m_bAdvancedMode = false;
    m_bModified = false;
    m_pServer = (CYRServer *)AfxGetMainWnd()->SendMessage (WMU_GET_SERVERPTR);
}


void CYRNodeDefinitionDlg::DoDataExchange(CDataExchange* pDX)
{
    CDialog::DoDataExchange(pDX);
    //{{AFX_DATA_MAP(CYRNodeDefinitionDlg)
    DDX_Control(pDX, IDC_PAGE_SWAP_SPIN, m_ctlPageSwapSpin);
    DDX_Control(pDX, IDC_NETUSE_SPIN, m_ctlNetUseSpin);
    DDX_Control(pDX, IDC_CPU_SLIDER, m_ctlCPUSlider);
    DDX_Text(pDX, IDC_PHYSICAL_NODE, m_strPhysicalNode);
    DDV_MaxChars(pDX, m_strPhysicalNode, 15);
    DDX_CBString(pDX, IDC_NAME_COMBO, m_strNodeName);
    DDX_Text(pDX, IDC_NET_USE, m_nNetUseThresh);
    DDX_Text(pDX, IDC_CPU_THRESHOLD, m_strCPUThresh);
    DDX_Text(pDX, IDC_PAGE_SWAPPING, m_nPageSwapThresh);
    DDX_Control(pDX, IDC_NAME_COMBO, m_ctlNodeNameCombo);
    //}}AFX_DATA_MAP
}



BEGIN_MESSAGE_MAP(CYRNodeDefinitionDlg, CDialog)
//{{AFX_MSG_MAP(CYRNodeDefinitionDlg)
ON_WM_HELPINFO()
ON_WM_HSCROLL()
ON_BN_CLICKED(IDC_ADVANCED, OnAdvanced)
ON_BN_CLICKED(IDC_CONFIGURE_OTA, OnConfigureOta)
ON_EN_CHANGE(IDC_PHYSICAL_NODE, OnChangePhysicalNode)
ON_CBN_CLOSEUP(IDC_NAME_COMBO, OnCloseupNameCombo)
ON_CBN_DROPDOWN(IDC_NAME_COMBO, OnDropdownNameCombo)
ON_CBN_EDITUPDATE(IDC_NAME_COMBO, OnEditupdateNameCombo)
ON_EN_CHANGE(IDC_NET_USE, OnChangeNetUse)
ON_EN_CHANGE(IDC_PAGE_SWAPPING, OnChangePageSwapping)
//}}AFX_MSG_MAP
ON_UPDATE_COMMAND_UI(IDC_ADVANCED,OnUpdateAdvanced)
ON_UPDATE_COMMAND_UI(IDC_PHYSICAL_NODE,OnUpdatePhysicalNode)
ON_UPDATE_COMMAND_UI(IDC_NAME_COMBO,OnUpdateNameCombo)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CYRNodeDefinitionDlg message handlers

BOOL CYRNodeDefinitionDlg::OnInitDialog()
{
    CDialog::OnInitDialog();

    m_ctlNodeNameCombo.SetLength(16);

    ShowAdvanced();

    PrimeComboBox();

    if(!m_strNodeName.IsEmpty())
    {
        int nSel = m_ctlNodeNameCombo.FindStringExact(-1, m_strNodeName);
        if (nSel != CB_ERR)
        {
            m_ctlNodeNameCombo.SetCurSel(nSel);
            m_ctlNodeNameCombo.EnableWindow(FALSE);
            GetNodeDefinition(m_strNodeName);
        }
    }
    else
    {
        m_strPhysicalNode.Empty();
        m_strCPUThresh = "0%";
        m_nPageSwapThresh = 0;
        m_nNetUseThresh = 0;
    }


    //Remove list box items this way not with resetcontent
    //so that we retain the text in the edit box
    while(m_ctlNodeNameCombo.GetCount())
        m_ctlNodeNameCombo.DeleteString(0);

    UpdateData(FALSE);
    UpdateDialogControls(this,true);

    m_bModified =false;
    return TRUE;  // return TRUE unless you set the focus to a control
    // EXCEPTION: OCX Property Pages should return FALSE
}

//
// PrimeComboBox
// =============
//
// Populate the combo box
//

void CYRNodeDefinitionDlg::PrimeComboBox()
{
    CWaitCursor wc;

    m_ctlNodeNameCombo.ResetContent();

    //Get a list of nodes
    CStringArray arstrFiles;
    m_pServer->GetFileList(CYRServer::enNodesFolder,"",arstrFiles);

    //    ...and add the node definition names to the combo box.
    for(int nNdx =0;nNdx<arstrFiles.GetSize();nNdx++)
    {
        m_ctlNodeNameCombo.AddString(arstrFiles[nNdx]);
    }
}


void CYRNodeDefinitionDlg::OnOK()
{
    if(m_bModified)
    {
        // Apply any changes made ...
        CWaitCursor wc;
        if (!ValidateDefinition())
            return;

        //Save the thresholds for this physical node
        if(m_bAdvancedMode)
            SaveThresholdsForNode();


        //Generate a tof for ota
        GenerateDefaultOTA();
    }

    CDialog::OnOK();
}


//
// GetNodeDefinition
// =================
//
// Populate the node definition dialog
//
void CYRNodeDefinitionDlg::GetNodeDefinition(CString& strNodeName)
{
    //
    //    Read in the logical node if it exists already.ect
    //
    ReadNode (strNodeName,m_strPhysicalNode);

    UpdateData(FALSE);
}

//
// ValidateDefinition
// ==================
//
// This routine simply validates all the dialog data
//
BOOL CYRNodeDefinitionDlg::ValidateDefinition()
{
    UpdateData();

    // Check to see if we got anything
    if (m_strNodeName.IsEmpty())
    {
        // And tell them to enter a name
        AfxMessageBox(IDS_NO_NODE_NAME, MB_OK);
        return FALSE;
    }

    if (m_strPhysicalNode.IsEmpty())
    {
        AfxMessageBox(IDS_NO_PHYSICALNODE);
        return FALSE;
    }

    //Get a list of nodes
    CStringArray arstrFiles;
    m_pServer->GetFileList(CYRServer::enNodesFolder,"",arstrFiles);

    CString strPhysicalNode;
    for(int nNdx=0;nNdx<arstrFiles.GetSize();nNdx++)
    {
        CString &strNodeName = arstrFiles[nNdx];
        if((strNodeName.CompareNoCase(m_strNodeName)!=0) &&
                ReadNode(strNodeName,strPhysicalNode))
        {
            if(strPhysicalNode.CompareNoCase(m_strPhysicalNode)==0)
            {
                CString strMessage;
                strMessage.Format("The machine name \"%s\", is already mapped to injector \"%s\".\n\nAre you sure you want two injectors using the same machine?",m_strPhysicalNode,strNodeName);
                if(MessageBox(strMessage,"Injector Definition",MB_YESNO|MB_ICONWARNING)==IDNO)
                    return false;
                else
                    break;
            }
        }
    }


    // And create a new entry for it in the ini file
    return CreateNodeFile();
}

BOOL    CYRNodeDefinitionDlg::CreateNodeFile    ()
{
    //
    //    ...and then create a new logical node definition object

    //
    //    Format the output source file name
    //
    if (m_strNodeWorkDir.IsEmpty())
        m_strNodeWorkDir =m_pServer->GetFolderPath(CYRServer::enNodesFolder);

    CString strNodeFileName = m_strNodeWorkDir + m_strNodeName;

    //
    //    ...open this file
    //
    try
    {
        CStdioFile  File (strNodeFileName, CFile::modeCreate | CFile::modeWrite | CFile::shareExclusive);
        File.WriteString (m_strPhysicalNode);
        File.WriteString ("\n");
        File.Close();
    }
    catch (CFileException e)
    {
        AfxMessageBox("Failed to write the injector definition to the working file");
        return -1;
    }

    //
    //    ...and then check the file back in again.
    //
    return m_pServer->CheckInFile(CYRServer::enNodesFolder,"",m_strNodeName);
}


//
//    ReadNode
//    ========
//
//    Read in the logical node file settings from the file in the work directory
//
BOOL    CYRNodeDefinitionDlg::ReadNode  (const CString& strNodeName,CString& strPhysicalNode)
{
    if(strNodeName.IsEmpty())
        return false;

    //
    //    Ensure that we know where the work directory for the project nodes is.
    //
    if (m_strNodeWorkDir.IsEmpty())
        m_strNodeWorkDir = m_pServer->GetFolderPath(CYRServer::enNodesFolder);

    //
    //    ...then read it.
    //
    CString strNodeFileName = m_strNodeWorkDir + strNodeName;

    CFileStatus fs;
    if (!CFile::GetStatus(strNodeFileName,fs))
        return false;

    try
    {
        CStdioFile  File (strNodeFileName, CFile::modeRead | CFile::typeText);
        File.ReadString (strPhysicalNode);
        File.Close();
    }
    catch (CFileException e)
    {
        return FALSE;
    }

    return TRUE;
}

BOOL CYRNodeDefinitionDlg::OnHelpInfo(HELPINFO* pHelpInfo)
{
    CString contextHelp = m_szHtmlHelpFile+"::whatisthis.txt";

    if (pHelpInfo->iCtrlId == IDC_STATIC)
        return true;

    CWnd* pWnd = GetDlgItem(pHelpInfo->iCtrlId);

    HtmlHelp(pWnd->m_hWnd, contextHelp, HH_TP_HELP_CONTEXTMENU, (DWORD)(LPVOID)g_TopicIdArray);

    return true;
}

void CYRNodeDefinitionDlg::OnHScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar)
{
    if(pScrollBar)
    {
        m_bModified = true;
        UpdateData(TRUE);
        switch(pScrollBar->GetDlgCtrlID())
        {
        case IDC_CPU_SLIDER:
            m_strCPUThresh.Format("%d%%",m_ctlCPUSlider.GetPos());
            break;
        }
        UpdateData(FALSE);
    }

    CDialog::OnHScroll(nSBCode, nPos, pScrollBar);
}

void CYRNodeDefinitionDlg::ShowAdvanced()
{
    UpdateData();
    CRect rc;
    if(m_bAdvancedMode)
    {
        //show entire dlg
        GetDlgItem(IDC_FRAME_ADVANCED)->GetWindowRect(&rc);
        GetDlgItem(IDC_ADVANCED)->SetWindowText("<< &Advanced");

        UpdateData();
        LoadThresholdsForNode();
    }
    else
    {
        //Only show top half of dialog
        GetDlgItem(IDC_FRAME_NORMAL)->GetWindowRect(&rc);
        GetDlgItem(IDC_ADVANCED)->SetWindowText("&Advanced >>");
        SaveThresholdsForNode();
    }

    rc.bottom += GetSystemMetrics(SM_CYCAPTION) + GetSystemMetrics(SM_CYBORDER)*3;
    SetWindowPos(NULL,0,0,rc.Width(),rc.Height(),SWP_NOZORDER|SWP_NOMOVE);
}

void CYRNodeDefinitionDlg::LoadThresholdsForNode()
{
    CECOTAThresholds OT(m_strPhysicalNode);

    int nCPUThresh = OT.GetCPUThreshold();
    m_strCPUThresh.Format("%d%%",nCPUThresh);

    m_ctlCPUSlider.SetRange(0,100);
    m_ctlCPUSlider.SetTicFreq(10);
    m_ctlCPUSlider.SetPos(nCPUThresh);

    m_nPageSwapThresh = OT.GetSwapThreshold();
    m_ctlPageSwapSpin.SetRange32(0,0x7fffffff);
    m_ctlPageSwapSpin.SetPos(m_nPageSwapThresh);

    m_nNetUseThresh = OT.GetNetThreshold();
    m_ctlNetUseSpin.SetRange32(0,0x7fffffff);
    m_ctlNetUseSpin.SetPos(m_nNetUseThresh);

    const int nNumAccel =3;
    UDACCEL udAccel[nNumAccel];
    udAccel[0].nInc = 1;
    udAccel[0].nSec = 1;
    udAccel[1].nInc = 1000;
    udAccel[1].nSec = 2;
    udAccel[2].nInc = 100000;
    udAccel[2].nSec = 4;
    m_ctlNetUseSpin.SetAccel(nNumAccel,udAccel);
    m_ctlPageSwapSpin.SetAccel(nNumAccel,udAccel);

    UpdateData(FALSE);
}

void CYRNodeDefinitionDlg::OnAdvanced()
{
    m_bAdvancedMode = !m_bAdvancedMode;
    ShowAdvanced();
    UpdateDialogControls(this,TRUE);
}

void CYRNodeDefinitionDlg::SaveThresholdsForNode()
{
    UpdateData();
    CECOTAThresholds OT(m_strPhysicalNode);
    OT.SetCPUThreshold(m_ctlCPUSlider.GetPos() );
    OT.SetNetThreshold(m_nNetUseThresh);
    OT.SetSwapThreshold(m_nPageSwapThresh);
}


void CYRNodeDefinitionDlg::GenerateDefaultOTA()
{
    UpdateData();

    CString strOTAFile = m_strNodeName + "-MON.tof";
    CString strObjCodePath = m_pServer->GetFolderPath(CYRServer::enObjectCodeFolder);
    CString strOTAPath = strObjCodePath + strOTAFile;

    //  Check if this file already under Source control...
    if(m_pServer->DoesFileExist(CYRServer::enObjectCodeFolder,"",strOTAFile))
        return;

    CECDefaultCodeWriter DCW;
    DCW.GenerateDefaultOtaTof(strOTAPath,m_strPhysicalNode);

    m_pServer->CheckInFile(CYRServer::enObjectCodeFolder,"",strOTAFile, strOTAFile);
}

void CYRNodeDefinitionDlg::OnConfigureOta()
{
    UpdateData();
    //Ensure that the TOF exists
    GenerateDefaultOTA();

    CString strOTAFile = m_strNodeName + "-MON.tof";

    //Try to edit it
    CString strParams;
    strParams.Format("%s|%s" ,SOLUTIONS_GENERIC_DLL,strOTAFile);
    AfxGetMainWnd()->SendMessage (WMU_FIRE_SPECIFIC_EVENT
                                  ,(WPARAM )WMU_EDIT_OTA_TOF
                                  ,(LPARAM)&strParams);
}

void CYRNodeDefinitionDlg::OnUpdateAdvanced(CCmdUI *pCmdUI)
{
    UpdateData();
    pCmdUI->Enable(!m_strPhysicalNode.IsEmpty() && !m_strNodeName.IsEmpty());
}

void CYRNodeDefinitionDlg::OnUpdatePhysicalNode(CCmdUI *pCmdUI)
{
    pCmdUI->Enable(!m_bAdvancedMode);
}

void CYRNodeDefinitionDlg::OnUpdateNameCombo(CCmdUI *pCmdUI)
{
    pCmdUI->Enable(!m_bAdvancedMode);
}

void CYRNodeDefinitionDlg::OnChangePhysicalNode()
{
    m_bModified = true;
    UpdateDialogControls(this,TRUE);
}

void CYRNodeDefinitionDlg::OnCloseupNameCombo()
{
    int nCurSel = m_ctlNodeNameCombo.GetCurSel();
    if(nCurSel != CB_ERR)
    {
        m_bModified = false;
        m_ctlNodeNameCombo.GetLBText(nCurSel,m_strNodeName);
    }

    m_ctlNodeNameCombo.ResetContent();

    m_ctlNodeNameCombo.AddString(m_strNodeName);
    m_ctlNodeNameCombo.SelectString(-1,m_strNodeName);

    GetNodeDefinition(m_strNodeName);
    UpdateDialogControls(this,TRUE);
}

void CYRNodeDefinitionDlg::OnDropdownNameCombo()
{
    PrimeComboBox();
}

void CYRNodeDefinitionDlg::OnEditupdateNameCombo()
{
    DWORD dwSel = m_ctlNodeNameCombo.GetEditSel();

    //Remove list box items this way not with resetcontent
    //so that we retain the text in the edit box
    while(m_ctlNodeNameCombo.GetCount())
        m_ctlNodeNameCombo.DeleteString(0);

    m_ctlNodeNameCombo.GetWindowText(m_strNodeName);
    GetNodeDefinition(m_strNodeName);
    UpdateDialogControls(this,TRUE);

    m_ctlNodeNameCombo.SetEditSel(LOWORD(dwSel),HIWORD(dwSel));
    m_bModified = true;
}

void CYRNodeDefinitionDlg::OnChangeNetUse()
{
    m_bModified = true;
}

void CYRNodeDefinitionDlg::OnChangePageSwapping()
{
    m_bModified = true;
}
