/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

/////////////////////////////////////////////////////////////////////////
//
// CYRProjectTools.cpp
// -------------------
//
//    This is the implementation file for the CYRProjectTools class
//
//=========================================================================

#include "stdafx.h"

//
//    Project includes
//
#include "CYRProjectTools.h"
#include "CECSourceEditView.h"
#include "CECSourceEditDoc.h"


#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CYRProjectTools

CYRProjectTools::CYRProjectTools()
{}


CYRProjectTools::~CYRProjectTools()
{}

/////////////////////////////////////////////////////////////////////////////
// CYRProjectTools functions
//

// DoesDocumentExist
// =================
//
// Checks to see if a document is open, - if it is then the default is to
// Activate it and bring it to the top of the Z-Order
//

BOOL CYRProjectTools::DoesDocumentExist(CDocTemplate* pDocTemplate, CString szPath, BOOL bActivate /*=TRUE*/)
{
    ASSERT(FALSE);
    /// THIS FUNCTION IS NOT USED ANYWHERE AND IS UNTESTED

    // We must have a valid template to search on
    if (pDocTemplate == NULL)
    {
        int nDotPos = szPath.ReverseFind('.');
        if(nDotPos==-1)
            return false;

        CString strPathExt = szPath.Right(szPath.GetLength()-nDotPos-1);

        CWinApp *pApp = AfxGetApp();
        POSITION pos = pApp->GetFirstDocTemplatePosition();
        while(pos != NULL)
        {
            pDocTemplate = (CMultiDocTemplate*)pApp->GetNextDocTemplate(pos);
            CString strExt;
            if(pDocTemplate->GetDocString(strExt,CDocTemplate::filterExt))
            {
                if(strExt.CompareNoCase(strPathExt)==0)
                {
                    break;
                }
            }
        }
        if(pDocTemplate==NULL)
            return false;
    }

    // And assume that it is not open
    BOOL bMatched = FALSE;

    // And get hold of the first template position
    POSITION pos = pDocTemplate->GetFirstDocPosition();
    while (pos != NULL)
    {
        // And grab each document in turn (which increments the pos variable
        // in case youre wondering)
        CDocument* pDoc = pDocTemplate->GetNextDoc(pos);

        // First we access the file associated with the template
        CString szFile = pDoc->GetPathName();
        if (!szFile.IsEmpty())
        {
            // And truncate it so that we only compare it with the path that is passed
            // in which is of the form CYR.Project_1.x.scl
            szFile = szFile.Right(szPath.GetLength());

            if (szFile.CompareNoCase(szPath) == 0)
            {
                // Bingo - we found a match. Activate it if required else reurn TRUE
                // to the caller.
                if (bActivate)
                {
                    // Get the first view associated with this document
                    POSITION viewPos = pDoc->GetFirstViewPosition();
                    ASSERT (viewPos != NULL);

                    CECSourceEditView* pView = (CECSourceEditView*) pDoc->GetNextView(viewPos);
                    ASSERT (pView != NULL);

                    // And bring it to the top of the Z-Order
                    pView->BringToFront();
                }

                bMatched = TRUE;
                break;
            }
        }
    }

    return bMatched;
}

// GetDocumentCount
// ================
//
// Checks to see if a document is open, - if it is then the default is to
// Activate it and bring it to the top of the Z-Order
//
int CYRProjectTools::GetDocumentCount(CDocTemplate* pDocTemplate)
{
    int nDocumentCount = 0;

    CWinApp *pApp = AfxGetApp();
    CDocTemplate *pTemplate=pDocTemplate;

    //If pDocTemplate is not valid then get count of all open documents
    POSITION pos = pApp->GetFirstDocTemplatePosition();
    while((pDocTemplate!=NULL) || (pos != NULL))
    {
        if(pDocTemplate!=NULL)
        {
            //Only want to go round once if we've been given a template...
            pDocTemplate=NULL;
        }
        else
        {
            //otherwise go through all templates
            pTemplate = (CMultiDocTemplate*)pApp->GetNextDocTemplate(pos);
        }

        // And get hold of the first template position
        POSITION posDoc = pTemplate->GetFirstDocPosition();
        while (posDoc != NULL)
        {
            // And grab each document in turn (which increments the posDoc variable
            // in case youre wondering)
            CDocument* pDoc = pTemplate->GetNextDoc(posDoc);

            // First we access the file associated with the template
            CString szFile = pDoc->GetPathName();
            if (!szFile.IsEmpty())
            {
                nDocumentCount++;
            }
        }
    }

    return nDocumentCount;
}

// GetActiveView
// =============
//
// Checks to see if a document is open, - if it is then the default is to
// Activate it and bring it to the top of the Z-Order
//
CView* CYRProjectTools::GetActiveView()
{
    CFrameWnd* pFrameWnd = (CFrameWnd*)AfxGetMainWnd();
    ASSERT (pFrameWnd != NULL);

    CMDIChildWnd *pChild = (CMDIChildWnd *) pFrameWnd->GetActiveFrame();
    ASSERT (pChild != NULL);

    // And return whatever view is active
    return pChild->GetActiveView();
}

// GetActiveDocument
// =================
//
// Checks to see if a document is open, - if it is then the default is to
// Activate it and bring it to the top of the Z-Order
//
BOOL CYRProjectTools::GetActiveDocument(CDocTemplate* pDocTemplate, CString& /*szFile*/)
{
    CWinApp *pApp = AfxGetApp();
    CDocTemplate *pTemplate=pDocTemplate;

    // Get hold of the active view
    CView* pActiveView = GetActiveView();
    if (pActiveView == NULL)
        return FALSE;

    CDocument* pActiveEdit = pActiveView->GetDocument();
    ASSERT (pActiveEdit != NULL);

    // And assume that it is not open
    BOOL bFound = FALSE;

    //If pDocTemplate is not valid then get count of all open documents
    POSITION pos = pApp->GetFirstDocTemplatePosition();
    while(!bFound && ((pDocTemplate!=NULL) || (pos != NULL)))
    {
        if(pDocTemplate!=NULL)
        {
            //Only want to go round once if we've been given a template...
            pDocTemplate=NULL;
        }
        else
        {
            //otherwise go through all templates
            pTemplate = (CMultiDocTemplate*)pApp->GetNextDocTemplate(pos);
        }

        // And get hold of the first template position
        POSITION posDoc = pTemplate->GetFirstDocPosition();
        while (posDoc != NULL)
        {
            // And grab each document in turn (which increments the pos variable
            // in case youre wondering)
            CDocument* pDoc = pTemplate->GetNextDoc(posDoc);

            // First we access the file associated with the template
            CString szFile = pDoc->GetPathName();
            if (!szFile.IsEmpty())
            {
                // Get the first view associated with this document
                POSITION viewPos = pDoc->GetFirstViewPosition();
                ASSERT (viewPos != NULL);

                if (szFile.CompareNoCase(pActiveEdit->GetPathName()) == 0)
                {
                    // And return the file name
                    bFound = ExtractFileTitle(szFile);
                    break;

                }
            }
        }
    }

    return bFound;
}

// ExtractFileTitle
// =================
//
// Extract the file title from the full path name which is in the form of
// CYR.Impact/Project/Test.scl
//
BOOL CYRProjectTools::ExtractFileTitle(CString& szFile)
{
    // Strip off any trailing slashes
    if (!StripOffTrailingSlash(szFile))
        return FALSE;

    // And search for the previous directory
    int nSlash = szFile.ReverseFind('\\');
    if (nSlash == -1)
        return FALSE;

    // Now take everything to the right of it
    szFile = szFile.Mid(nSlash + 1);

    return TRUE;
}

// StripOffTrailingSlash
// =====================
//
// Strips off trailing slashes which appear in Windows 95
//
BOOL CYRProjectTools::StripOffTrailingSlash(CString& szFile)
{
    int nLength = szFile.GetLength();
    if (nLength == -1)
        return FALSE;

    if (szFile.Right(1) == "\\")
    {
        // We have a trailing slash so ...
        szFile = szFile.Left(nLength-1);
    }

    return TRUE;
}
