/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// CECInitiatorDlg.cpp : implementation file
//

#include "stdafx.h"
#include "CECInitiatorDlg.h"
#include "..\..\Interfaces\CyrPlugins.h"
#include "..\..\Common\CECXServerError.h"
#include "Server.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#define BUFSIZE 4096
void executeTest(CECInitiatorDlg* pDlg);

/////////////////////////////////////////////////////////////////////////////
// CECInitiatorDlg dialog


CECInitiatorDlg::CECInitiatorDlg(CString szTest, CWnd* pParent /*=NULL*/)
        : CDialog(CECInitiatorDlg::IDD, pParent)
{
    //{{AFX_DATA_INIT(CECInitiatorDlg)
    m_strExecStatus = _T("");
    //}}AFX_DATA_INIT

    m_strTest = szTest;
}

CECInitiatorDlg::~CECInitiatorDlg()
{}


void CECInitiatorDlg::DoDataExchange(CDataExchange* pDX)
{
    CDialog::DoDataExchange(pDX);
    //{{AFX_DATA_MAP(CECInitiatorDlg)
    DDX_Control(pDX, IDC_EXEC_STATUS, m_ctlExecStatus);
    DDX_Text(pDX, IDC_EXEC_STATUS, m_strExecStatus);
    //}}AFX_DATA_MAP
}



BEGIN_MESSAGE_MAP(CECInitiatorDlg, CDialog)
//{{AFX_MSG_MAP(CECInitiatorDlg)
//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CECInitiatorDlg message handlers

BOOL CECInitiatorDlg::OnInitDialog()
{
    CDialog::OnInitDialog();


    // Get a handle to the Mainframe
    CWnd* pMainWnd = AfxGetMainWnd();
    ASSERT (pMainWnd != NULL);

    // Set the initial scroll bar range to 10 lines
    m_ctlExecStatus.SetScrollRange(SB_VERT,0,10,TRUE);

    m_strExecStatus = "Starting...";
    UpdateData(FALSE);

    m_pExecTestThread = AfxBeginThread(&executeTest, this);

    return TRUE;  // return TRUE unless you set the focus to a control
    // EXCEPTION: OCX Property Pages should return FALSE
}


UINT CECInitiatorDlg::executeTest(LPVOID pParam)
{
    CECInitiatorDlg* pDlg = (CECInitiatorDlg*)pParam;

    try
    {
        CYRServer *pServer = (CYRServer*)AfxGetMainWnd()->SendMessage(WMU_GET_SERVERPTR);
        CString strProject = pServer->GetFullOpenProjectName();
        CString strHost = pServer->GetHostName();
        CString strUser = pServer->GetUserName();
        CString strPass = pServer->GetPassword();

        CString exeCmd = getenv("CYRANO_HOME");
        exeCmd += "\\Engines\\";

        exeCmd +="TestInit -start -n localhost ";
        exeCmd +="-P \""    +   strProject  +   "\" ";
        exeCmd +="-T "      +   pDlg->m_strTest     +   " ";

        CEdit* pEdit = (CEdit*)pDlg->GetDlgItem(IDC_EXEC_STATUS);

        SECURITY_ATTRIBUTES saAttr;

        //
        // Set the bInheritHandle flag so pipe handles are inherited.
        //
        saAttr.nLength = sizeof(SECURITY_ATTRIBUTES);
        saAttr.bInheritHandle = TRUE;
        saAttr.lpSecurityDescriptor = NULL;

        //
        // The steps for redirecting child's STDOUT:
        //     2.  Create anonymous pipe to be STDOUT for child.
        //     3.  Set STDOUT of parent to be write handle of pipe, so
        //         it is inherited by child.
        //

        //
        // Create a pipe for the child's STDOUT.
        //
        if (!CreatePipe (&pDlg->m_hChildStdoutRd, &pDlg->m_hChildStdoutWr, &saAttr, 0))
        {
            pDlg->m_strExecStatus = "Stdout pipe creation failed\n";
            pDlg->UpdateData(FALSE);
        }
        //
        // Now create the child process to execute the command
        //
        pDlg->InvokeImage (exeCmd.GetBuffer(0));

        //
        //  Read from pipe that is the standard output for child process.
        //
        pDlg->ReadFromChild();

        if ((pDlg->m_strExecStatus.Find("Exception",0) == -1) &&
                (pDlg->m_strExecStatus.Find("Failed",0) == -1))
        {
            pDlg->SetExecuted(TRUE);
        }
        else
        {
            CString szDisplayText;
            szDisplayText = "The CYRANO Daemon is not running.\x0D\x0A\x0D\x0A";
            szDisplayText += "The CYRANO Daemon is not running on the node on which ";
            szDisplayText += "you wish to execute a test - or part of it.\x0D\x0A\x0D\x0A";
            szDisplayText += "Please ensure that the CYRANO Daemon is running on ";
            szDisplayText += "all nodes used for this test. If you have just restarted the ";
            szDisplayText += "Daemon on this machine then you will need to restart TestCommander ";
            szDisplayText += "first. ";

            // Output all Corba details we have ...
            CECXServerError error;
            error.DisplayErrorMessage("OpenSTA", szDisplayText);

            pDlg->SetExecuted(FALSE);

        }

    }
    catch(...)
    {
        pDlg->SetExecuted(FALSE);
    }
    return 0;
}



void CECInitiatorDlg::OnCancel()
{
    TerminateThread(m_pExecTestThread->m_hThread,0);

    CDialog::OnCancel();
}

int CECInitiatorDlg::InvokeImage    (char*  pImageName)
{
    PROCESS_INFORMATION ProcInfo;
    CString     ErrorBuffer;
    STARTUPINFO StartInfo = {sizeof(StartInfo)};
    BOOL        InheritHandles = FALSE;
    int         CreateFlags = 0;


    //
    //    If we have requested that the window should be hidden then set up the STARTUPINFO
    //
    StartInfo.dwFlags = STARTF_USESTDHANDLES|STARTF_USESHOWWINDOW;
    StartInfo.hStdOutput = m_hChildStdoutWr;
    StartInfo.hStdError = m_hChildStdoutWr;
    StartInfo.wShowWindow = SW_HIDE;
    InheritHandles = TRUE;
    CreateFlags = CREATE_NEW_CONSOLE;

    //
    //    Create a process to execute the specified image.
    //
    if (!CreateProcess (NULL
                        ,pImageName                      // Command Line
                        ,NULL                            // Process Security attributes
                        ,NULL                            // Primary thread security attributes
                        ,InheritHandles                  // Handles are {NOT} inherited
                        ,CreateFlags                     // Creation flags
                        ,NULL                            // Use Parents environment
                        ,NULL                            // Use Parents current directory
                        ,&StartInfo                      // STARTUPINFO pointer
                        ,&ProcInfo))                     // Receives Process Information
    {
        ErrorBuffer.Format ("Error invoking the image %s" ,pImageName);
        LPTSTR lpMsgBuf;

        FormatMessage (FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM,
                       NULL,
                       GetLastError(),
                       MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // Default language
                       (LPTSTR) &lpMsgBuf,
                       0,
                       NULL);

        //
        //    Output the error message via a message box if we have requested the child process output
        //    be directed to a window, If we have requested that child process output be sent to a string
        //    then the error should be logged into that string
        //
        m_strExecStatus = lpMsgBuf;
        UpdateData(FALSE);

        //
        //    Free the buffer.
        //
        LocalFree( lpMsgBuf );
        return -1;

    }
    return 0;
}

void    CECInitiatorDlg::ReadFromChild  ()
{
    CString TextBuffer;
    CString s;

    DWORD dwRead;
    CHAR chBuf[BUFSIZE];


    //
    // Close the write end of the pipe before reading from the
    // read end of the pipe.
    //
    if (!CloseHandle(m_hChildStdoutWr))
    {
        m_strExecStatus = "Closing handle failed.";
        UpdateData(FALSE);
    }
    //
    // Read output from child, and write it to the output window edit box.
    //
    TextBuffer.Empty();
    for (;;)
    {
        if ((!ReadFile(m_hChildStdoutRd, chBuf, BUFSIZE, &dwRead, NULL)) || (dwRead == 0)) break;
        chBuf[dwRead] = '\0';
        TextBuffer += chBuf;
        m_strExecStatus = TextBuffer;
        m_ctlExecStatus.SetWindowText(m_strExecStatus);

        // And update the scroll bar - and text
        m_ctlExecStatus.SetScrollPos(SB_VERT, m_ctlExecStatus.GetLineCount(), TRUE);

        m_ctlExecStatus.SetSel(0,m_strExecStatus.GetLength(),FALSE);
    }


}

void CECInitiatorDlg::SetExecuted(BOOL bExec)
{
    if (bExec)
    {
        EndDialog(IDOK);
    }
    else
    {
        EndDialog(IDCANCEL);
    }
}
