/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

/////////////////////////////////////////////////////////////////////////
//
// CYRThreadPropertiesDlg.cpp
// --------------------------
//
//    This is the implementation file for the CYRThreadPropertiesDlg class.
//
//=========================================================================

#include "stdafx.h"
#include "resource.h"

#include "..\..\Common\CECClipboard.h"
#include "..\..\Common\CECTargetEnvironment.h"
#include "CYRThreadPropertiesDlg.h"
#include "htmlhelp.h"

//
//  PSL Macros.
//
extern "C"
{
#include "psl_macros.h"
#include "target_rtndef.h"  // Target/Executer routines
}
/*
#include "..\..\..\HTMLHelp\CYRImpact\Thread_Properties_dialog_box.inl"
*/
#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CYRThreadPropertiesDlg dialog


CYRThreadPropertiesDlg::CYRThreadPropertiesDlg(CWnd* pParent /*=NULL*/)
        : CDialog(CYRThreadPropertiesDlg::IDD, pParent)
{
    //{{AFX_DATA_INIT(CYRThreadPropertiesDlg)
    m_strThreadName = _T("");
    m_nInterval = 0;
    m_nUserThreads = 1;
    //}}AFX_DATA_INIT
    m_szHtmlHelpFile = getenv("CYRANO_HOME");
    m_szHtmlHelpFile += "\\Framework\\Help\\Commander.chm";

    m_pSpinInterval = NULL;
    m_pSpinUserThreads = NULL;
    m_pThreadProperties = NULL;

    m_szOriginalCBText = _T("");
}


void CYRThreadPropertiesDlg::DoDataExchange(CDataExchange* pDX)
{
    CDialog::DoDataExchange(pDX);
    //{{AFX_DATA_MAP(CYRThreadPropertiesDlg)
    DDX_Control(pDX, IDC_TARGET_COMBO, m_TargetList);
    DDX_Text(pDX, IDC_INTERVAL_EDIT, m_nInterval);
    DDV_MinMaxUInt(pDX, m_nInterval, 0, 1000);
    DDX_Text(pDX, IDC_NAME_EDIT, m_strThreadName);
    DDV_MaxChars(pDX, m_strThreadName, 16);
    DDX_Text(pDX, IDC_NUMBER_EDIT, m_nUserThreads);
    DDV_MinMaxUInt(pDX, m_nUserThreads, 1, 999);
    DDX_Control(pDX, IDC_NUMBER_SPIN, m_SpinUserThreads);
    DDX_Control(pDX, IDC_INTERVAL_SPIN, m_SpinInterval);
    //}}AFX_DATA_MAP
}



BEGIN_MESSAGE_MAP(CYRThreadPropertiesDlg, CDialog)
//{{AFX_MSG_MAP(CYRThreadPropertiesDlg)
ON_EN_KILLFOCUS(IDC_NUMBER_EDIT,    OnExitUserThreads)
ON_CBN_SELCHANGE(IDC_TARGET_COMBO, OnSelchangeTargetCombo)
ON_EN_CHANGE(IDC_NUMBER_EDIT,           OnExitUserThreads)
ON_WM_HELPINFO()
//}}AFX_MSG_MAP
ON_UPDATE_COMMAND_UI(IDC_INTERVAL_NAME_STATIC,  OnUpdateInterval)
ON_UPDATE_COMMAND_UI(IDC_INTERVAL_EDIT, OnUpdateInterval)
ON_UPDATE_COMMAND_UI(IDC_INTERVAL_SPIN, OnUpdateInterval)
ON_UPDATE_COMMAND_UI(IDC_INTERVAL_SECS_STATIC, OnUpdateInterval)
ON_UPDATE_COMMAND_UI(IDC_NUMBER_EDIT, OnUpdateNumber)
ON_UPDATE_COMMAND_UI(IDC_NUMBER_SPIN, OnUpdateNumber)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CYRThreadPropertiesDlg message handlers

BOOL CYRThreadPropertiesDlg::OnInitDialog()
{
    CDialog::OnInitDialog();

    //Setup html help handler
    // TODO: Add extra initialization here
    // Link in the Spin Controls with the window controls
    m_pSpinUserThreads = (CSpinButtonCtrl*) GetDlgItem(IDC_NUMBER_SPIN);
    ASSERT (m_pSpinUserThreads != NULL);

    m_pSpinInterval = (CSpinButtonCtrl*) GetDlgItem(IDC_INTERVAL_SPIN);
    ASSERT (m_pSpinInterval != NULL);

    // Set up the spin control ranges
    m_pSpinUserThreads->SetRange(1,999);
    m_pSpinInterval->SetRange(0,1000);

    // Now tell them to control certain edit boxes
    m_pSpinUserThreads->SetBuddy(GetDlgItem(IDC_NUMBER_EDIT));
    m_pSpinInterval->SetBuddy(GetDlgItem(IDC_INTERVAL_EDIT));

    // And finally set up their initial state
    m_pSpinUserThreads->SetPos(m_nUserThreads);
    m_pSpinInterval->SetPos(m_nInterval);

    // Collect the thread we are to edit from the clipboard
    CECClipboard cb;
    m_szOriginalCBText = cb.GetClipboardText();

    // And create a new CECThread Object to hold the data in
    if (m_pThreadProperties == NULL)
    {
        m_pThreadProperties = new CECThread();
    }
    // Import the clipboard text into the CECTest format
    m_pThreadProperties->ImportCFText(m_szOriginalCBText);

    // And extract the fields from the object
    m_strThreadName = m_pThreadProperties->GetName();
    m_nUserThreads = m_pThreadProperties->GetNumber();
    m_nInterval = m_pThreadProperties->GetInterval();

    PrimeTargetCombo();

    // Add the V-Test Dataname - yuch
    m_DataNameEdit.SubclassEdit (IDC_NAME_EDIT, this, 0);
    m_DataNameEdit.SetLength(16);
    m_DataNameEdit.SetData(m_strThreadName);

    // Send them to the dialog
    UpdateData(FALSE);

    // And update the dialog controls incase this is the second time in here
    UpdateDialogControls(GetParent(),FALSE);

    return TRUE;  // return TRUE unless you set the focus to a control
    // EXCEPTION: OCX Property Pages should return FALSE
}

//
// OnExitUserThreads
// =================
//
// Echo any change to the Dialog Controls
//

void CYRThreadPropertiesDlg::OnExitUserThreads()
{
    // As this field activates other controls - the interval
    UpdateDialogControls(GetParent(),FALSE);
}

//
// OnUpdateInterval
// ================
//
// Command Handler for the Interval Field
//
void CYRThreadPropertiesDlg::OnUpdateInterval(CCmdUI *pCmdUI)
{
    // To enable the interval the number must be greater than one, so
    int nNumber = GetDlgItemInt(IDC_NUMBER_EDIT);

    BOOL bState = (nNumber > 1) ? TRUE : FALSE;

    CString strTarget;
    int nSel = m_TargetList.GetCurSel();
    if(nSel != CB_ERR)
    {
        CString strTarget;
        m_TargetList.GetLBText(nSel, strTarget);
        bState = (bState && (strTarget != "OTA"));
    }

    pCmdUI->Enable(bState);
}

void CYRThreadPropertiesDlg::OnOK()
{
    UpdateData(TRUE);

    // Get the VTEST data name
    m_DataNameEdit.GetData(m_strThreadName, TRUE);

    if (m_strThreadName.IsEmpty())
    {
        AfxMessageBox(IDS_THREAD_PROPS_NAME);
        return;
    }

    // Set up the Thread Properties Object
    CString szText("");
    int nSel = m_TargetList.GetCurSel();
    if (nSel != CB_ERR)
    {
        m_TargetList.GetLBText(nSel, szText);
        TARGETID TargetID;
        if(szText == "OTA")
        {
            TargetID = TARGET_OTA;
            m_nUserThreads = 1;
            m_nInterval = 0;
        }
        else
        {
            TargetID = Tgt_GetTargetID(szText.GetBuffer(0));
        }

        m_pThreadProperties->SetTargetID(TargetID);
    }
    m_pThreadProperties->SetName(m_strThreadName);
    m_pThreadProperties->SetNumber(m_nUserThreads);
    m_pThreadProperties->SetInterval(m_nInterval);

    // Put the Thread Name on the Clipboard
    CECClipboard cb;
    cb.SetClipboardText(m_pThreadProperties->ExportCFText());

    if (m_pThreadProperties != NULL)
    {
        delete m_pThreadProperties;
        m_pThreadProperties = NULL;
    }

    CDialog::OnOK();
}

void CYRThreadPropertiesDlg::OnCancel()
{
    CECClipboard cb;
    cb.SetClipboardText(m_szOriginalCBText);

    if (m_pThreadProperties != NULL)
    {
        delete m_pThreadProperties;
        m_pThreadProperties = NULL;
    }

    CDialog::OnCancel();
}


void CYRThreadPropertiesDlg::PrimeTargetCombo()
{
    //
    //  Retrieve a list of available contexts...
    //
    CComboBox* pTargetCombo = (CComboBox*) GetDlgItem(IDC_TARGET_COMBO);
    ASSERT (pTargetCombo != NULL);

    CECTargetEnvironment TargetEnvironment;
    TargetEnvironment.GetTargets(*pTargetCombo);

    pTargetCombo->AddString("OTA");
    //
    //  ...and finally set the selection
    //
    const int nBuffLen = 256;
    char szCurrentTarget[nBuffLen];
    int nResult = Tgt_GetTarget( (TARGETID)m_pThreadProperties->GetTargetID(),szCurrentTarget,nBuffLen ,false);

    int nTargetNdx = pTargetCombo->FindString (-1,szCurrentTarget);
    if(nTargetNdx == -1)
    {
        nTargetNdx=0;
    }

    pTargetCombo->SetCurSel(nTargetNdx);
}

//
//  OnUpdateNumber
//  ==============
//
//  disable number fields if ota is target
//
void CYRThreadPropertiesDlg::OnUpdateNumber(CCmdUI *pCmdUI)
{
    //Disable the number fields if the target is OTA
    if(m_pThreadProperties != NULL)
    {
        CString strTarget;
        int nSel = m_TargetList.GetCurSel();
        if(nSel != CB_ERR)
        {
            m_TargetList.GetLBText(nSel, strTarget);
            pCmdUI->Enable(strTarget != "OTA");
        }
    }
}

void CYRThreadPropertiesDlg::OnSelchangeTargetCombo()
{
    UpdateDialogControls(GetParent(),FALSE);
}

BOOL CYRThreadPropertiesDlg::OnHelpInfo(HELPINFO* pHelpInfo)
{
    /*
        CString contextHelp = m_szHtmlHelpFile+"::whatisthis.txt";
     
        if (pHelpInfo->iCtrlId == IDC_STATIC)
            return true;
     
        CWnd* pWnd = GetDlgItem(pHelpInfo->iCtrlId);
     
        HtmlHelp(pWnd->m_hWnd, contextHelp, HH_TP_HELP_CONTEXTMENU, (DWORD)(LPVOID)g_TopicIdArray);
    */
    return true;
}
