/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// CYRSimpleTestWizard.cpp : implementation file
//

#include "stdafx.h"
#include "resource.h"
#include "CYRSimpleTestWizard.h"
#include "..\..\common\CYRTestProperties.h"
#include "..\..\common\cecTssWriter.h"
#include "..\..\common\cecsclWriter.h"
#include "..\..\interfaces\cyrplugins.h"
#include "htmlhelp.h"
#include "Server.h"
#include "CECSourceFileUtils.h"
#include "TestWizardDefs.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#include "..\..\..\HTMLHelp\CYRImpact\Simple_Wizard_Auxillary_Script.inl"
#include "..\..\..\HTMLHelp\CYRImpact\Simple_Wizard_Browser_Raito.inl"
#include "..\..\..\HTMLHelp\CYRImpact\Simple_Wizard_Name_Def.inl"
#include "..\..\..\HTMLHelp\CYRImpact\Simple_Wizard_Script_Depends.inl"
#include "..\..\..\HTMLHelp\CYRImpact\Simple_Wizard_Script_Properties.inl"
#include "..\..\..\HTMLHelp\CYRImpact\Simple_Wizard_Summary.inl"

IMPLEMENT_DYNCREATE(CYRWizardBasePage, CPropertyPage)

/////////////////////////////////////////////////////////////////////////////
// CYRSimpleTestWizardSheet

IMPLEMENT_DYNAMIC(CYRSimpleTestWizardSheet, CPropertySheet)

CYRSimpleTestWizardSheet::CYRSimpleTestWizardSheet(UINT nIDCaption, CWnd* pParentWnd, UINT iSelectPage,const CString& strTestName)
        :CPropertySheet(nIDCaption, pParentWnd, iSelectPage)
{
    SetupWizard(strTestName);
}

CYRSimpleTestWizardSheet::CYRSimpleTestWizardSheet(LPCTSTR pszCaption, CWnd* pParentWnd,UINT iSelectPage,const CString& strTestName)
        :CPropertySheet(pszCaption, pParentWnd, iSelectPage)
{
    SetupWizard(strTestName);
}

BOOL CYRSimpleTestWizardSheet::SetupWizard(const CString& strTestName)
{
    m_pServer = (CYRServer*)AfxGetMainWnd()->SendMessage(WMU_GET_SERVERPTR);
    m_bWizardMode = true;
    m_bAdvancedMode = false;

    CYRTests Tests(m_pServer);
    CString strTestPath;
    if(!strTestName.IsEmpty())
    {
        strTestPath = m_pServer->GetFolderPath(CYRServer::enTestsFolder) + strTestName + "\\" + strTestName + ".WIZ";
        m_bWizardMode = true;

        CString strTestFile(strTestName + ".WIZ");
        if(m_pServer->CheckOutFile(CYRServer::enTestsFolder,strTestName,strTestFile))
        {
            DWORD dwMaxBufferSize = 200;
            CString strType;
            GetPrivateProfileString(WIZARD_TEST,WIZARD_TEST_TYPE,"",strType.GetBuffer(dwMaxBufferSize),dwMaxBufferSize,strTestPath);
            strType.ReleaseBuffer();
            m_bWizardMode = (strType != SIMPLETEST);

            m_bAdvancedMode = GetPrivateProfileInt(WIZARD_TEST,WIZARD_TEST_MODE,0,strTestPath);
        }
    }

    if(m_bWizardMode)
    {
        AddPage(&m_ModeSelPage);
    }
    else
        SetupPages();

    m_ModeSelPage.LoadPageSettings(strTestPath);
    m_NameDefPage.LoadPageSettings(strTestPath);
    m_AuxScriptPage.LoadPageSettings(strTestPath);
    m_ScriptDepPage.LoadPageSettings(strTestPath);
    m_BrowserRatioPage.LoadPageSettings(strTestPath);
    m_SummaryPage.LoadPageSettings(strTestPath);

    if(m_bWizardMode)
        SetWizardMode();

    return true;
}

CYRSimpleTestWizardSheet::~CYRSimpleTestWizardSheet()
{}


CYRWizardBasePage* CYRSimpleTestWizardSheet::GetPage(int nPage)
{
    switch(nPage)
    {
    case enModeSelPage:
        return &m_ModeSelPage;
    case enNameDefPage:
        return &m_NameDefPage;
    case enAuxScriptPage:
        return &m_AuxScriptPage;
    case enScriptDepPage:
        return &m_ScriptDepPage;
    case enBrowserRatioPage:
        return &m_BrowserRatioPage;
    case enSummaryPage:
        return &m_SummaryPage;
    }
    return NULL;
}

void CYRSimpleTestWizardSheet::SetAdvancedMode(BOOL bAdvanced)
{
    m_bAdvancedMode = bAdvanced;
    SetupPages();
}

void CYRSimpleTestWizardSheet::SetupPages()
{
    int nNumPages = GetPageCount();
    int nPage = (m_bWizardMode ? 1 : 0);
    for(;nPage<nNumPages;nPage++)
        RemovePage(nPage);

    AddPage(&m_NameDefPage);

    if(m_bAdvancedMode)
    {
        AddPage(&m_AuxScriptPage);
    }

    AddPage(&m_ScriptDepPage);

    AddPage(&m_BrowserRatioPage);

    if(m_bWizardMode)
    {
        AddPage(&m_SummaryPage);
    }
}


BEGIN_MESSAGE_MAP(CYRSimpleTestWizardSheet, CPropertySheet)
//{{AFX_MSG_MAP(CYRSimpleTestWizardSheet)
// NOTE - the ClassWizard will add and remove mapping macros here.
//}}AFX_MSG_MAP
END_MESSAGE_MAP()

BOOL CYRSimpleTestWizardSheet::OnInitDialog()
{
    BOOL bResult = CPropertySheet::OnInitDialog();

    //Show the context sensitive help ? button
    LONG style = ::GetWindowLong(m_hWnd, GWL_EXSTYLE);
    style |= WS_EX_CONTEXTHELP;
    ::SetWindowLong(m_hWnd, GWL_EXSTYLE, style);

    return bResult;
}
/////////////////////////////////////////////////////////////////////////////
// CYRSimpleTestWizardSheet message handlers
void CYRSimpleTestWizardSheet::OnWizardFinish()
{
    CString strTest(m_NameDefPage.m_strTestName);
    CString strTestFile(strTest + ".WIZ");
    CString strTestPath(m_pServer->GetFolderPath(CYRServer::enTestsFolder));

    //Create Test folders
    BOOL b = CreateDirectory(strTestPath,NULL);
    strTestPath += strTest;
    b = CreateDirectory(strTestPath,NULL);
    strTestPath += "\\" + strTestFile;

    //Delete the Dependencies for the existing test
    WritePrivateProfileSection(WIZARD_DEPENDENCIES,"",strTestPath);

    WritePrivateProfileString(WIZARD_TEST,WIZARD_TEST_TYPE,SIMPLETEST,strTestPath);

    int nPage = 0;
    CYRWizardBasePage* pPage = (CYRWizardBasePage*)GetPage(nPage);
    while(pPage!=NULL)
    {
        pPage->SavePageSettings(strTestPath);
        pPage = (CYRWizardBasePage*)GetPage(++nPage);
    }

    /*
        //Write ota thread to dependency sectyion
        DWORD dwMaxBufferSize=200;
        CString strNumber;
        GetPrivateProfileString(WIZARD_DEPENDENCIES,WIZARD_DEPENDENCIES_NUMBER,"0",strNumber.GetBuffer(dwMaxBufferSize),dwMaxBufferSize,strTestPath);
        strNumber.ReleaseBuffer();
        int nNumDependencies = atoi(strNumber);
        strNumber.Format("%d",++nNumDependencies);
        WritePrivateProfileString(WIZARD_DEPENDENCIES,WIZARD_DEPENDENCIES_NUMBER,strNumber,strTestPath);
    */
    m_pServer->CheckInFile(CYRServer::enTestsFolder,strTest,strTestFile);

    GenerateScripts();
}

void CYRSimpleTestWizardSheet::GenerateScripts()
{
    WriteSCL();
}

void CYRSimpleTestWizardSheet::WriteSCL()
{
    // If so get the title
    CString strTestName = m_NameDefPage.m_strTestName;

    CString strSCLName = strTestName + "-TH.SCL";
    ;

    // szFileName = <Drive>:\\ ... \\Source\\szTestName
    CString strFileName = m_pServer->GetFolderPath(CYRServer::enSourceFolder) + strSCLName;
    strFileName.MakeUpper();

    // Now declare a file object
    CStdioFile file;

    // And a writer
    CECSclWriter writer;

    // And away we go ...
    CString String;
    try
    {
        CStdioFile file(strFileName, CFile::modeCreate | CFile::modeWrite | CFile::typeText);

        // Header Section
        // ==============

        // Formalate the Header Text
        String = writer.GetTemplateText(CECSclWriter::HEADER, strTestName);
        file.WriteString(String);


        // Environment section
        // ===================

        String = writer.GetSectionTitle(CECSclWriter::ENVIRONMENT);
        file.WriteString(String);

        // And add in any addition from all our pages at this point
        String = writer.GetNameDefinitionText(CECSclWriter::ENVIRONMENT, m_NameDefPage.m_strTestDescription);
        file.WriteString(String);

        String = writer.GetTemplateText(CECSclWriter::ENVIRONMENT);
        file.WriteString(String);

        // And do the rest for any other page in the future
        //
        // ...
        //


        // Definitions section
        // ===================

        String = writer.GetSectionTitle(CECSclWriter::DEFINITIONS);
        file.WriteString(String);

        // Variables
        // =========
        String = "\tCharacter\tCYR_HTTP-STAT-LV, Thread\n";
        file.WriteString(String);

        // And add in any addition from all our pages at this point
        for (int i=0; i<m_ScriptDepPage.GetNumScripts(); i++)
        {
            const ScriptProperties& Script = m_ScriptDepPage.GetScript(i);
            String = writer.GetScriptDependencyText(CECSclWriter::DEFINITIONS, Script.m_strName,0,i+1);
            file.WriteString(String);
        }

        if(m_bAdvancedMode)
        {
            String = "\tInteger \tRANDOM (1-100)\n";
            file.WriteString(String);
        }

		String = "\tInteger \tUSE_PAGE_TIMERS\n";
        file.WriteString(String);

        // Enter the Iteration variable as a normal variable or a prompt variable
        String = writer.GetDurationText(CECSclWriter::DEFINITIONS, FALSE);
        file.WriteString(String);

        String = writer.GetTemplateText(CECSclWriter::DEFINITIONS);
        file.WriteString(String);

        String = "\tInteger\t\tBROWSER(1-100)\n";
        file.WriteString(String);

        String = "\tCharacter*255\tBROWSER_TYPE\n";
        file.WriteString(String);

        String = "\tInclude \"GLOBAL_VARIABLES.INC\"\n";
        file.WriteString(String);

        int nNumBrowserTypes = m_BrowserRatioPage.GetNumBrowserTypes();
        if(!nNumBrowserTypes)
        {
            String = "\tCharacter*255\tDEFAULT_BROWSER\n";
            file.WriteString(String);
        }
        else
        {
            for(int nType=0;nType<nNumBrowserTypes;nType++)
            {
                BrowserInfo BI = m_BrowserRatioPage.GetBrowser(nType);
                String = "\tCharacter*255\t" + BI.strName + "\n";
                file.WriteString(String);
            }
        }

		if(m_BrowserRatioPage.m_bRandomIterationDelay)
		{
            String.Format("\tInteger\t\tITER_DELAY(%d-%d)\n",m_BrowserRatioPage.m_nMinIterationDelay,m_BrowserRatioPage.m_nMaxIterationDelay);
            file.WriteString(String);
		}

        // And do the rest for any other page in the future
        //
        // ...
        //

        // Code Section
        // ============

        String = writer.GetSectionTitle(CECSclWriter::CODE);
        file.WriteString(String);

        String = "\tSet CYR_HTTP-STAT-LV=\""+m_BrowserRatioPage.GetStatLevel()+"\"\n\n";
        file.WriteString(String);

        String = "\tSet USE_PAGE_TIMERS = ";
		String += (m_BrowserRatioPage.m_bGenPageTimers ? "1\n\n" : "0\n\n");
        file.WriteString(String);
		
        //Setup browser user-agent fields
        String = "\t!Setup browser user agent fields\n";
        if(!nNumBrowserTypes)
        {
            String += "\tSet DEFAULT_BROWSER = \"DEFAULT\"\n\n";
        }
        else
        {
            for(int nType=0;nType<nNumBrowserTypes;nType++)
            {
                BrowserInfo BI = m_BrowserRatioPage.GetBrowser(nType);
                String += "\tSet " + BI.strName + " = \"" + BI.strUserAgent + "\"\n";
            }
            String += "\n";
        }
        file.WriteString(String);

        //Decide which browser to use for this script
        if(!nNumBrowserTypes)
        {
            String = "\tSet BROWSER_TYPE = \"DEFAULT\"\n\n";
        }
        else
        {
            String = "!Select a browser to run this script with\n";
            String += "\tGenerate BROWSER\n\n";

            CString strPercent;
            int nTotalPC=0;
            for(int nType=0;nType<nNumBrowserTypes;nType++)
            {
                BrowserInfo BI = m_BrowserRatioPage.GetBrowser(nType);

                nTotalPC += BI.nPercent;
                strPercent.Format("%d",nTotalPC);

                if(nType==0)
                    String += "\tIf\t(BROWSER <=" + strPercent + ") then\n\n";
                else
                    if(nType <= nNumBrowserTypes-1)
                        String += "\tElseif\t(BROWSER <=" + strPercent + ") then\n\n";
                    else
                        String += "\tElse\n\n";

                String += "\t\tSet BROWSER_TYPE =  " + BI.strName + "\n\n";
            }
            String += "\tEndif\n\n";
        }
        file.WriteString(String);

        // And start off with the connection Script
        String = writer.GetAuxillaryScriptText(CECSclWriter::CODE, CECSclWriter::CONNECT, m_AuxScriptPage.m_strPrologScript);
        file.WriteString(String);

        // Enter the Iteration variable as a normal variable or a prompt variable
        String = writer.GetDurationText(CECSclWriter::CODE,
                                        FALSE,
                                        CECSclWriter::DO,
                                        m_BrowserRatioPage.m_nNumIterations);
        file.WriteString(String);

        String = GetSCLScriptsString();
        file.WriteString(String);

        // Enter the Iteration variable as a normal variable or a prompt variable
        String = writer.GetDurationText(CECSclWriter::CODE);
        file.WriteString(String);

        String = "\tNote \" COMPLETED AT \", TIME";
        if(m_bAdvancedMode)
        {
            String += ",\"WITH \", &\n";
            String += "\tTOTAL_REQUESTS, \" REQUESTS \"\n\n";
        }
        else
            String +="\n\n";
        file.WriteString(String);

        // And finish off with the disconnection Script
        String = writer.GetAuxillaryScriptText(CECSclWriter::CODE, CECSclWriter::DISCONNECT, m_AuxScriptPage.m_strEpilogScript);
        file.WriteString(String);

        // And do the rest for any other page in the future
        //
        // ...
        //
        String = writer.GetTemplateText(CECSclWriter::EXIT);
        file.WriteString(String);

        file.Close();

        //Compile the script and clear the output window
        CECSourceFileUtils utils(m_pServer,strFileName);
        utils.Compile(TRUE);

        // And import it into the project
        m_pServer->CheckInFile(CYRServer::enSourceFolder,"",strSCLName);
    }
    catch (CFileException* e)
    {
        e->ReportError();
        e->Delete();
    }
}

CString CYRSimpleTestWizardSheet::GetSCLScriptsString()
{
    CString String;
    if(m_bAdvancedMode)
        String = "\t\tGenerate RANDOM\n\n";

    // And now the core of the script - the If statement
    int nTotal = m_ScriptDepPage.GetNumScripts();

    CString szReport;
    CString szTotals;
    int nPercentageNeeded = 0;
    for(int i=0; i<nTotal; i++)
    {
        const ScriptProperties& Script = m_ScriptDepPage.GetScript(i);

        CString strScript;
        if(m_bAdvancedMode)
        {
            // Must have a script name to be able to code anything
            strScript.Format("TOTAL_SCRIPT_%d",i+1);

            nPercentageNeeded += Script.m_nPercentage;

            // Convert the percentage field
            CString szPercent,szPercentReqd;
            szPercent.Format("%d", Script.m_nPercentage);
            szPercentReqd.Format("%d", nPercentageNeeded);

            // Create a suitable comment for each script
            String += "! This Should Generate " + Script.m_strName + " scripts " + szPercent + "% of the time\n\n";

            if(i==0)
                String += "\t\tIf\t(RANDOM <=" + szPercentReqd + ") then\n\n";
            else
                if(i <= nTotal-1)
                    String += "\t\tElseif\t(RANDOM <=" + szPercentReqd + ") then\n\n";
                else
                    String += "\t\tElse\n\n";
        }

        if(m_bAdvancedMode)
            String += "\t";

		String += "\t\tCall Script \"" + Script.m_strName + "\" [BROWSER_TYPE,USE_PAGE_TIMERS]\n";

        if(m_bAdvancedMode)
        {
            // Increment it's counter
            String += "\t\t\tSet " + strScript + " = " + strScript + " + 1\n\n";

            // If the script count is greater than one tab all scripts into line
            if (i > 0)
            {
                szReport += "\t\t\t ";
                szTotals += "\t\t\t\t\t\t\t";
            }

            // And add in a report string
            szReport += "\t\"" + Script.m_strName + "s = \", " +strScript;
            szTotals += "\t" + strScript;

            // Add in a continuation if there's more to follow
            if (i < nTotal-1)
            {
                szReport += ", &";
                szTotals += "\t+ &";
            }

            // And throw in a new line
            szReport += "\n";
            szTotals += "\n";
        }
    }

    CString strTemp;
    if(m_bAdvancedMode)
    {
        // Finally we end the If statement
        String += "\t\tEndif\n\n";

        // And report the Script Totals
        String += "\t\tNote " + szReport + "\n";

        // And set the TOTAL_REQUESTS Variable
        String += "\t\tSet TOTAL_REQUESTS = " + szTotals + "\n";
    }
    else
        String += "\n";

    // And issue a delay
	if(m_BrowserRatioPage.m_bRandomIterationDelay)
	{
		strTemp = "\t\tGenerate ITER_DELAY\n";
		strTemp += "\t\tWait ITER_DELAY\n\n";
	}
	else
		strTemp.Format("\t\tWait %d\n\n",m_BrowserRatioPage.m_nIterationDelay);

    String += strTemp;

    return String;
}

/////////////////////////////////////////////////////////////////////////////
// CYRNameDefinitionPage property page

IMPLEMENT_DYNCREATE(CYRNameDefinitionPage,CYRWizardBasePage)

CYRNameDefinitionPage::CYRNameDefinitionPage() : CYRWizardBasePage(CYRNameDefinitionPage::IDD)
{
    //{{AFX_DATA_INIT(CYRNameDefinitionPage)
    m_strTestDescription = _T("");
    m_strTestName = _T("");
    //}}AFX_DATA_INIT
    m_szHtmlHelpFile = getenv("CYRANO_HOME");
    m_szHtmlHelpFile += "\\Framework\\Help\\Commander.chm";
}

CYRNameDefinitionPage::~CYRNameDefinitionPage()
{}


void CYRNameDefinitionPage::DoDataExchange(CDataExchange* pDX)
{
    CYRWizardBasePage::DoDataExchange(pDX);
    //{{AFX_DATA_MAP(CYRNameDefinitionPage)
    DDX_Text(pDX, IDC_TEST_DESCRIPTION, m_strTestDescription);
    DDV_MaxChars(pDX, m_strTestDescription, 50);
    DDX_Text(pDX, IDC_TEST_NAME, m_strTestName);
    DDX_Control(pDX, IDC_TEST_NAME, m_ctlTestName);
    //}}AFX_DATA_MAP
}



BEGIN_MESSAGE_MAP(CYRNameDefinitionPage, CYRWizardBasePage)
//{{AFX_MSG_MAP(CYRNameDefinitionPage)
ON_NOTIFY(NM_SETFOCUS, IDC_TEST_DESCRIPTION, OnSetfocusTestDescription)
ON_WM_HELPINFO()
//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CYRNameDefinitionPage message handlers
BOOL CYRNameDefinitionPage::OnInitDialog()
{
    CYRWizardBasePage::OnInitDialog();

    m_ctlTestName.SetLength(13);

    return TRUE;  // return TRUE unless you set the focus to a control
    // EXCEPTION: OCX Property Pages should return FALSE
}


void CYRNameDefinitionPage::OnOK()
{
    CYRSimpleTestWizardSheet* psheet = (CYRSimpleTestWizardSheet*) GetParent();
    psheet->OnWizardFinish();
}

BOOL CYRNameDefinitionPage::OnSetActive()
{
    CYRSimpleTestWizardSheet* psheet = (CYRSimpleTestWizardSheet*) GetParent();
    psheet->SetWizardButtons(PSWIZB_NEXT);
    m_ctlTestName.EnableWindow(psheet->IsWizardMode());

    CWnd* pWnd = GetDlgItem(IDC_TEXT);
    if (!psheet->IsWizardMode())
    {
        pWnd->ShowWindow(SW_HIDE);
    }
    return CYRWizardBasePage::OnSetActive();
}

void CYRNameDefinitionPage::OnSetfocusTestDescription(NMHDR* pNMHDR, LRESULT* pResult)
{
    UpdateData(TRUE);

    if(m_strTestDescription.IsEmpty())
    {
        m_strTestDescription = m_strTestName;
        UpdateData(FALSE);
    }
    *pResult = 0;
}

void CYRNameDefinitionPage::SavePageSettings(const CString& strWizardFile)
{
    WritePrivateProfileString(WIZARD_TEST,WIZARD_TEST_NAME,m_strTestName,strWizardFile);
    WritePrivateProfileString(WIZARD_TEST,WIZARD_TEST_DESCRIPTION,m_strTestDescription,strWizardFile);
}

void CYRNameDefinitionPage::LoadPageSettings(const CString& strWizardFile)
{
    DWORD dwMaxBufferSize = 200;
    GetPrivateProfileString(WIZARD_TEST,WIZARD_TEST_NAME,"TEST",m_strTestName.GetBuffer(dwMaxBufferSize),dwMaxBufferSize,strWizardFile);
    m_strTestName.ReleaseBuffer();
    GetPrivateProfileString(WIZARD_TEST,WIZARD_TEST_DESCRIPTION,"TEST",m_strTestDescription.GetBuffer(dwMaxBufferSize),dwMaxBufferSize,strWizardFile);
    m_strTestDescription.ReleaseBuffer();
}

BOOL CYRNameDefinitionPage::OnKillActive()
{
    UpdateData(TRUE);
    if(m_strTestName.IsEmpty())
    {
        AfxMessageBox("Please specify a name for this test",MB_ICONEXCLAMATION);
        m_ctlTestName.SetFocus();
        return 0;
    }

    CYRSimpleTestWizardSheet* pSheet = (CYRSimpleTestWizardSheet*) GetParent();
    if(!pSheet->IsValidFilename(m_strTestName))
    {
        return false;
    }


    if(pSheet->IsWizardMode())
    {
        if(pSheet->m_pServer->DoesFileExist(CYRServer::enTestsFolder,m_strTestName,m_strTestName + ".WIZ"))
        {
            //The test exists so ask to overwrite it
            CString strMessage;
            strMessage.Format("The test \"%s\" already exists.\n\nDo you want to overwrite it?",m_strTestName);
            if(AfxMessageBox(strMessage,MB_ICONWARNING|MB_YESNO) == IDNO)
                return false;
        }
    }

    if(m_strTestDescription.IsEmpty())
    {
        m_strTestDescription = m_strTestName;
        UpdateData(FALSE);
    }
    else
        if(m_strTestDescription.Find("\"")!=-1)
        {
            AfxMessageBox("The test description must not contain \" s");
            return false;
        }

    return CYRWizardBasePage::OnKillActive();
}

/////////////////////////////////////////////////////////////////////////////
// : property page

IMPLEMENT_DYNCREATE(CYRAuxillaryScriptPage, CYRWizardBasePage)

CYRAuxillaryScriptPage::CYRAuxillaryScriptPage() : CYRWizardBasePage(CYRAuxillaryScriptPage::IDD)
{
    //{{AFX_DATA_INIT(CYRAuxillaryScriptPage)
    m_strEpilogScript = _T("");
    m_strPrologScript = _T("");
    //}}AFX_DATA_INIT
    m_szHtmlHelpFile = getenv("CYRANO_HOME");
    m_szHtmlHelpFile += "\\Framework\\Help\\Commander.chm";
}

CYRAuxillaryScriptPage::~CYRAuxillaryScriptPage()
{}


void CYRAuxillaryScriptPage::DoDataExchange(CDataExchange* pDX)
{
    CYRWizardBasePage::DoDataExchange(pDX);
    //{{AFX_DATA_MAP(CYRAuxillaryScriptPage)
    DDX_Control(pDX, IDC_PROLOG_SCRIPT, m_ctlPrologScript);
    DDX_Control(pDX, IDC_EPILOG_SCRIPT, m_ctlEpilogScript);
    DDX_CBString(pDX, IDC_EPILOG_SCRIPT, m_strEpilogScript);
    DDX_CBString(pDX, IDC_PROLOG_SCRIPT, m_strPrologScript);
    //}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CYRAuxillaryScriptPage, CYRWizardBasePage)
//{{AFX_MSG_MAP(CYRAuxillaryScriptPage)
ON_WM_HELPINFO()
//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CYRAuxillaryScriptPage message handlers
BOOL CYRAuxillaryScriptPage::OnInitDialog()
{
    CYRWizardBasePage::OnInitDialog();

    m_ctlEpilogScript.SetLength(16);
    m_ctlPrologScript.SetLength(16);

    CStringArray arstrFiles;
    CYRSimpleTestWizardSheet* pSheet = (CYRSimpleTestWizardSheet*) GetParent();
    pSheet->m_pServer->GetFileList(CYRServer::enSourceFolder,"",arstrFiles);

    for(int nNdx=0;nNdx<arstrFiles.GetSize();nNdx++)
    {
        CString &strFile = arstrFiles[nNdx];

        //Don't show any scripts with -TH suffix
        if((strFile.Right(4) == ".SCL") && (strFile.Right(7) != "-TH.SCL"))
        {
            m_ctlPrologScript.AddString(strFile.SpanExcluding("."));
            m_ctlEpilogScript.AddString(strFile.SpanExcluding("."));
        }
    }

    return TRUE;  // return TRUE unless you set the focus to a control
    // EXCEPTION: OCX Property Pages should return FALSE
}


BOOL CYRAuxillaryScriptPage::OnSetActive()
{
    CYRSimpleTestWizardSheet* psheet = (CYRSimpleTestWizardSheet*) GetParent();
    psheet->SetWizardButtons(PSWIZB_NEXT|PSWIZB_BACK);

    return CYRWizardBasePage::OnSetActive();
}

BOOL CYRAuxillaryScriptPage::OnKillActive()
{
    UpdateData(TRUE);

    CYRSimpleTestWizardSheet* pSheet = (CYRSimpleTestWizardSheet*) GetParent();
    if(!pSheet->IsValidFilename(m_strPrologScript))
        return false;

    if(!pSheet->IsValidFilename(m_strEpilogScript))
        return false;

    if(m_strEpilogScript.Right(3) == "-TH" || m_strPrologScript.Right(3) == "-TH")
    {
        AfxMessageBox("The script name suffix \"-TH\" is reserved for use by OpenSTA.\n\nPlease enter another name");
        return false;
    }

    return CYRWizardBasePage::OnKillActive();
}

void CYRAuxillaryScriptPage::SavePageSettings(const CString& strWizardFile)
{
    DWORD dwMaxBufferSize=200;
    CString strNumber;
    GetPrivateProfileString(WIZARD_DEPENDENCIES,WIZARD_DEPENDENCIES_NUMBER,"0",strNumber.GetBuffer(dwMaxBufferSize),dwMaxBufferSize,strWizardFile);
    strNumber.ReleaseBuffer();
    int nNumDependencies = atoi(strNumber);

    WritePrivateProfileString(WIZARD_TEST,WIZARD_TEST_EPILOG_SCRIPT,m_strEpilogScript,strWizardFile);
    if(!m_strEpilogScript.IsEmpty())
    {
        strNumber.Format(WIZARD_DEPENDENCIES_FORMAT,++nNumDependencies);
        WritePrivateProfileString(WIZARD_DEPENDENCIES,strNumber,m_strEpilogScript+".SCL",strWizardFile);
    }

    WritePrivateProfileString(WIZARD_TEST,WIZARD_TEST_PROLOG_SCRIPT,m_strPrologScript,strWizardFile);
    if(!m_strPrologScript.IsEmpty())
    {
        strNumber.Format(WIZARD_DEPENDENCIES_FORMAT,++nNumDependencies);
        WritePrivateProfileString(WIZARD_DEPENDENCIES,strNumber,m_strPrologScript+".SCL",strWizardFile);
    }

    WritePrivateProfileString(WIZARD_DEPENDENCIES,WIZARD_DEPENDENCIES_NUMBER,strNumber,strWizardFile);
}

void CYRAuxillaryScriptPage::LoadPageSettings(const CString& strWizardFile)
{
    DWORD dwMaxBufferSize = 200;
    GetPrivateProfileString(WIZARD_TEST,WIZARD_TEST_EPILOG_SCRIPT,"",m_strEpilogScript.GetBuffer(dwMaxBufferSize),dwMaxBufferSize,strWizardFile);
    m_strEpilogScript.ReleaseBuffer();

    GetPrivateProfileString(WIZARD_TEST,WIZARD_TEST_PROLOG_SCRIPT,"",m_strPrologScript.GetBuffer(dwMaxBufferSize),dwMaxBufferSize,strWizardFile);
    m_strPrologScript.ReleaseBuffer();
}

/////////////////////////////////////////////////////////////////////////////
// CYRScriptDependencyPage property page

IMPLEMENT_DYNCREATE(CYRScriptDependencyPage, CYRWizardBasePage)

CYRScriptDependencyPage::CYRScriptDependencyPage() : CYRWizardBasePage(CYRScriptDependencyPage::IDD)
{
    //{{AFX_DATA_INIT(CYRScriptDependencyPage)
    // NOTE: the ClassWizard will add member initialization here
    //}}AFX_DATA_INIT
    m_nTotalPCUsed = 0;
    m_szHtmlHelpFile = getenv("CYRANO_HOME");
    m_szHtmlHelpFile += "\\Framework\\Help\\Commander.chm";
    m_bAdvancedMode = false;
    m_bOnBack = false;
}

CYRScriptDependencyPage::~CYRScriptDependencyPage()
{}


void CYRScriptDependencyPage::DoDataExchange(CDataExchange* pDX)
{
    CYRWizardBasePage::DoDataExchange(pDX);
    //{{AFX_DATA_MAP(CYRScriptDependencyPage)
    DDX_Control(pDX, IDC_REMOVE, m_ctlRemove);
    DDX_Control(pDX, IDC_MODIFY, m_ctlModify);
    DDX_Control(pDX, IDC_SCRIPTS, m_ctlScripts);
    //}}AFX_DATA_MAP
}



BEGIN_MESSAGE_MAP(CYRScriptDependencyPage, CYRWizardBasePage)
//{{AFX_MSG_MAP(CYRScriptDependencyPage)
ON_BN_CLICKED(IDC_ADD, OnAdd)
ON_BN_CLICKED(IDC_MODIFY, OnModify)
ON_BN_CLICKED(IDC_REMOVE, OnRemove)
ON_NOTIFY(LVN_ITEMCHANGED, IDC_SCRIPTS, OnItemchangedScripts)
ON_WM_HELPINFO()
//}}AFX_MSG_MAP
ON_UPDATE_COMMAND_UI(IDC_ADD,OnUpdateAdd)
ON_UPDATE_COMMAND_UI(IDC_MODIFY,OnUpdateModify)
ON_UPDATE_COMMAND_UI(IDC_REMOVE,OnUpdateRemove)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CYRScriptDependencyPage message handlers
BOOL CYRScriptDependencyPage::OnSetActive()
{
    CYRSimpleTestWizardSheet* psheet = (CYRSimpleTestWizardSheet*) GetParent();
    psheet->SetWizardButtons(PSWIZB_NEXT|PSWIZB_BACK);

    return CYRWizardBasePage::OnSetActive();
}

BOOL CYRScriptDependencyPage::OnKillActive()
{
    // Always allowed to go back!!
    if (m_bOnBack)
    {
        m_bOnBack = false;
        return true;
    }


    if(m_arScripts.GetSize()==0)
    {
        AfxMessageBox("Please specify at least one script to use in this test");
        return false;
    }

    if(m_bAdvancedMode)
    {
        int nPercentUsed=0;
        for(int nNdx=0;nNdx<m_arScripts.GetSize();nNdx++)
            nPercentUsed += m_arScripts[nNdx].m_nPercentage;
        if(nPercentUsed != 100)
        {
            AfxMessageBox("The percentages must add up to 100");
            return false;
        }
    }

    return CYRWizardBasePage::OnKillActive();
}

void CYRScriptDependencyPage::SavePageSettings(const CString& strWizardFile)
{
    CString strTemp;

    int nNumScripts = m_arScripts.GetSize();
    strTemp.Format("%d",nNumScripts);
    WritePrivateProfileString(WIZARD_TEST,WIZARD_TEST_NUM_DEP_SCRIPTS,strTemp,strWizardFile);

    CString strNumber;
    DWORD dwMaxBufferSize=200;
    GetPrivateProfileString(WIZARD_DEPENDENCIES,WIZARD_DEPENDENCIES_NUMBER,"0",strNumber.GetBuffer(dwMaxBufferSize),dwMaxBufferSize,strWizardFile);
    strNumber.ReleaseBuffer();
    int nNumDependencies = atoi(strNumber);

    CString strScriptNum,strPercent,strDep;
    for(int nScript=0;nScript<nNumScripts;nScript++)
    {
        strScriptNum.Format(WIZARD_TEST_DEP_SCRIPT_FORMAT,nScript+1);

        strTemp = strScriptNum + WIZARD_TEST_DEP_NAME_EXT;
        WritePrivateProfileString(WIZARD_TEST,strTemp,m_arScripts[nScript].m_strName ,strWizardFile);

        strTemp = strScriptNum + WIZARD_TEST_DEP_PERCENT_EXT;
        strPercent.Format("%d",m_arScripts[nScript].m_nPercentage);
        WritePrivateProfileString(WIZARD_TEST,strTemp,strPercent,strWizardFile);

        strTemp = strScriptNum + WIZARD_TEST_DEP_DESCRIPTION_EXT;
        WritePrivateProfileString(WIZARD_TEST,strTemp,m_arScripts[nScript].m_strDescription,strWizardFile);

        strDep.Format(WIZARD_DEPENDENCIES_FORMAT,++nNumDependencies);
        WritePrivateProfileString(WIZARD_DEPENDENCIES,strDep,m_arScripts[nScript].m_strName+".SCL",strWizardFile);
    }

    strNumber.Format("%d",nNumDependencies);
    WritePrivateProfileString(WIZARD_DEPENDENCIES,WIZARD_DEPENDENCIES_NUMBER,strNumber,strWizardFile);
}

void CYRScriptDependencyPage::LoadPageSettings(const CString& strWizardFile)
{
    CString strTemp;
    DWORD dwMaxBufferSize = 200;

    //get num scripts to add
    GetPrivateProfileString(WIZARD_TEST,WIZARD_TEST_NUM_DEP_SCRIPTS,"0",strTemp.GetBuffer(dwMaxBufferSize),dwMaxBufferSize,strWizardFile);
    strTemp.ReleaseBuffer();
    int nNumScripts =atoi(strTemp);
    m_arScripts.SetSize(nNumScripts);

    CString strName,strScriptNum,strPercent,strDescription;
    for(int nScript=0;nScript<nNumScripts;nScript++)
    {
        strScriptNum.Format(WIZARD_TEST_DEP_SCRIPT_FORMAT,nScript+1);

        strTemp = strScriptNum + WIZARD_TEST_DEP_NAME_EXT;
        GetPrivateProfileString(WIZARD_TEST,strTemp,"",strName.GetBuffer(dwMaxBufferSize),dwMaxBufferSize,strWizardFile);
        strName.ReleaseBuffer();
        m_arScripts[nScript].m_strName = strName;

        strTemp = strScriptNum + WIZARD_TEST_DEP_PERCENT_EXT;
        GetPrivateProfileString(WIZARD_TEST,strTemp,"",strPercent.GetBuffer(dwMaxBufferSize),dwMaxBufferSize,strWizardFile);
        strPercent.ReleaseBuffer();
        m_arScripts[nScript].m_nPercentage = atoi(strPercent);

        strTemp = strScriptNum + WIZARD_TEST_DEP_DESCRIPTION_EXT;
        GetPrivateProfileString(WIZARD_TEST,strTemp,"",strDescription.GetBuffer(dwMaxBufferSize),dwMaxBufferSize,strWizardFile);
        strDescription.ReleaseBuffer();
        m_arScripts[nScript].m_strDescription = strDescription;
    }
}

BOOL CYRScriptDependencyPage::OnInitDialog()
{
    CYRWizardBasePage::OnInitDialog();

    CYRSimpleTestWizardSheet* psheet = (CYRSimpleTestWizardSheet*) GetParent();
    m_bAdvancedMode = psheet->GetAdvancedMode();

    m_ctlScripts.InsertColumn(0,_T("Script Name"),LVCFMT_CENTER, (m_bAdvancedMode ? 90 : 165),0);
    if(m_bAdvancedMode)
        m_ctlScripts.InsertColumn(1,_T("Percentage"),LVCFMT_CENTER,75,1);
    m_ctlScripts.SetExtendedStyle(LVS_EX_GRIDLINES|LVS_EX_FULLROWSELECT);

    m_nTotalPCUsed = 0;
    CString strPercent;
    for(int nScript=0;nScript<m_arScripts.GetSize();nScript++)
    {
        m_ctlScripts.InsertItem(nScript,m_arScripts[nScript].m_strName);

        if(m_bAdvancedMode)
        {
            strPercent.Format("%d",m_arScripts[nScript].m_nPercentage);
            m_ctlScripts.SetItemText(nScript,1,strPercent);

            m_nTotalPCUsed += m_arScripts[nScript].m_nPercentage;
        }
    }
    UpdateDialogControls(this,true);

    return TRUE;  // return TRUE unless you set the focus to a control
    // EXCEPTION: OCX Property Pages should return FALSE
}


void CYRScriptDependencyPage::OnAdd()
{
    int nPercentUsed=0;
    for(int nNdx=0;nNdx<m_arScripts.GetSize();nNdx++)
        nPercentUsed += m_arScripts[nNdx].m_nPercentage;

    ScriptProperties Script;
    Script.m_nPercentage = 0;
    nNdx = m_arScripts.Add(Script);

    CYRScriptPropertiesDlg dlg(&m_arScripts[nNdx],nPercentUsed);
    dlg.SetAdvancedMode(m_bAdvancedMode);
    if(dlg.DoModal()==IDOK)
    {
        int nNumItems = m_ctlScripts.GetItemCount();
        int nItem = m_ctlScripts.InsertItem(nNumItems,m_arScripts[nNdx].m_strName);

        if(m_bAdvancedMode)
        {
            CString strPercent;
            strPercent.Format("%d",m_arScripts[nNdx].m_nPercentage);
            m_ctlScripts.SetItemText(nItem,1,strPercent);
        }
    }
    else
        m_arScripts.RemoveAt(nNdx);

    m_nTotalPCUsed = 0;

    for(nNdx=0;nNdx<m_arScripts.GetSize();nNdx++)
        m_nTotalPCUsed += m_arScripts[nNdx].m_nPercentage;

    UpdateDialogControls(this,true);
}

void CYRScriptDependencyPage::OnUpdateAdd(CCmdUI *pCmdUI)
{
    pCmdUI->Enable(!m_bAdvancedMode || (m_nTotalPCUsed != 100));
}

void CYRScriptDependencyPage::OnUpdateModify(CCmdUI *pCmdUI)
{
    POSITION pos = m_ctlScripts.GetFirstSelectedItemPosition();
    pCmdUI->Enable(pos != NULL);
}

void CYRScriptDependencyPage::OnUpdateRemove(CCmdUI *pCmdUI)
{
    POSITION pos = m_ctlScripts.GetFirstSelectedItemPosition();
    pCmdUI->Enable(pos != NULL);
}

void CYRScriptDependencyPage::OnModify()
{
    m_nTotalPCUsed = 0;
    for(int nNdx=0;nNdx<m_arScripts.GetSize();nNdx++)
        m_nTotalPCUsed += m_arScripts[nNdx].m_nPercentage;

    POSITION pos = m_ctlScripts.GetFirstSelectedItemPosition();
    ASSERT(pos != NULL);
    int nItem = m_ctlScripts.GetNextSelectedItem(pos);

    CYRScriptPropertiesDlg dlg(&m_arScripts[nItem],m_nTotalPCUsed);
    dlg.SetAdvancedMode(m_bAdvancedMode);
    if(dlg.DoModal()==IDOK)
    {
        m_ctlScripts.SetItemText(nItem,0,m_arScripts[nItem].m_strName);

        if(m_bAdvancedMode)
        {
            CString strPercent;
            strPercent.Format("%d",m_arScripts[nItem].m_nPercentage);
            m_ctlScripts.SetItemText(nItem,1,strPercent);
        }
    }

    m_nTotalPCUsed = 0;
    for(nNdx=0;nNdx<m_arScripts.GetSize();nNdx++)
        m_nTotalPCUsed += m_arScripts[nNdx].m_nPercentage;

    UpdateDialogControls(this,true);
}

void CYRScriptDependencyPage::OnRemove()
{
    POSITION pos = m_ctlScripts.GetFirstSelectedItemPosition();

    int nItem = m_ctlScripts.GetNextSelectedItem(pos);

    //Remove the script from the array
    m_arScripts.RemoveAt(nItem);

    //remove script from list
    m_ctlScripts.DeleteItem(nItem);

    m_nTotalPCUsed = 0;
    for(int nNdx=0;nNdx<m_arScripts.GetSize();nNdx++)
        m_nTotalPCUsed += m_arScripts[nNdx].m_nPercentage;

    UpdateDialogControls(this,true);
}

void CYRScriptDependencyPage::OnItemchangedScripts(NMHDR* pNMHDR, LRESULT* pResult)
{
    NM_LISTVIEW* pNMListView = (NM_LISTVIEW*)pNMHDR;

    if(pNMListView->uChanged & LVIF_STATE)
    {
        UpdateDialogControls(this,true);
    }

    *pResult = 0;
}

LRESULT CYRScriptDependencyPage::OnWizardBack()
{
    m_bOnBack = true;
    return 0;
}

/////////////////////////////////////////////////////////////////////////////
// CYRBrowserRatioPage property page

IMPLEMENT_DYNCREATE(CYRBrowserRatioPage, CYRWizardBasePage)

CYRBrowserRatioPage::CYRBrowserRatioPage() : CYRWizardBasePage(CYRBrowserRatioPage::IDD)
{
    //{{AFX_DATA_INIT(CYRBrowserRatioPage)
    m_nNumIterations = 0;
    m_nNumSimUsers = 0;
    m_nIterationDelay = 0;
	m_nMaxIterationDelay = 0;
	m_nMinIterationDelay = 0;
	m_bRandomIterationDelay = 1;
	m_bGenPageTimers = FALSE;
	//}}AFX_DATA_INIT
    m_szHtmlHelpFile = getenv("CYRANO_HOME");
    m_szHtmlHelpFile += "\\Framework\\Help\\Commander.chm";

    m_nStatLev = 1;
}

CYRBrowserRatioPage::~CYRBrowserRatioPage()
{}


void CYRBrowserRatioPage::DoDataExchange(CDataExchange* pDX)
{
    CYRWizardBasePage::DoDataExchange(pDX);
    //{{AFX_DATA_MAP(CYRBrowserRatioPage)
    DDX_Control(pDX, IDC_STAT_LEVEL, m_StatLevel);
    DDX_Text(pDX, IDC_ITERATIONS, m_nNumIterations);
    DDV_MinMaxInt(pDX, m_nNumIterations, 1, 32767);
    DDX_Text(pDX, IDC_SIM_USERS, m_nNumSimUsers);
    DDV_MinMaxInt(pDX, m_nNumSimUsers, 1, 32767);
    DDX_Text(pDX, IDC_ITERATION_DELAY, m_nIterationDelay);
    DDV_MinMaxInt(pDX, m_nIterationDelay, 0, 32767);
	DDX_Text(pDX, IDC_MAX_DELAY, m_nMaxIterationDelay);
    DDX_Text(pDX, IDC_MIN_DELAY, m_nMinIterationDelay);
	DDX_Radio(pDX, IDC_CONSTANT_DELAY, m_bRandomIterationDelay);
	DDX_Check(pDX, IDC_GEN_PAGE_TIMERS, m_bGenPageTimers);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CYRBrowserRatioPage, CYRWizardBasePage)
//{{AFX_MSG_MAP(CYRBrowserRatioPage)
ON_WM_HELPINFO()
	ON_BN_CLICKED(IDC_RANDOM_DELAY, OnRandomDelay)
	ON_BN_CLICKED(IDC_CONSTANT_DELAY, OnConstantDelay)
	ON_BN_CLICKED(IDC_BROWSER_TYPES, OnBrowserTypes)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CYRBrowserRatioPage message handlers
BOOL CYRBrowserRatioPage::OnInitDialog()
{
    CYRWizardBasePage::OnInitDialog();

    const int nNumAccel=4;
    UDACCEL Accel[nNumAccel];
    Accel[0].nSec=1;
    Accel[0].nInc=1;
    Accel[1].nSec=2;
    Accel[1].nInc=10;
    Accel[2].nSec=3;
    Accel[2].nInc=50;
    Accel[3].nSec=4;
    Accel[3].nInc=200;

    CSpinButtonCtrl* pVUSpin = (CSpinButtonCtrl*)GetDlgItem(IDC_SPIN_VUS);
    pVUSpin->SetRange(1,UD_MAXVAL);
    pVUSpin->SetAccel(nNumAccel,Accel);
    pVUSpin->SetBuddy(GetDlgItem(IDC_SIM_USERS));

    CSpinButtonCtrl* pIterSpin = (CSpinButtonCtrl*)GetDlgItem(IDC_SPIN_ITERATIONS);
    pIterSpin->SetRange(1,UD_MAXVAL);
    pIterSpin->SetAccel(nNumAccel,Accel);
    pIterSpin->SetBuddy(GetDlgItem(IDC_ITERATIONS));

    CSpinButtonCtrl* pDelaySpin = (CSpinButtonCtrl*)GetDlgItem(IDC_SPIN_ITERATION_DELAY);
    pDelaySpin->SetRange(0,UD_MAXVAL);
    pDelaySpin->SetAccel(nNumAccel,Accel);
    pDelaySpin->SetBuddy(GetDlgItem(IDC_ITERATION_DELAY));

    CSpinButtonCtrl* pMinDelaySpin = (CSpinButtonCtrl*)GetDlgItem(IDC_MIN_DELAY_SPIN);
    pMinDelaySpin->SetRange(0,UD_MAXVAL);
    pMinDelaySpin->SetAccel(nNumAccel,Accel);
    pMinDelaySpin->SetBuddy(GetDlgItem(IDC_MIN_DELAY));

    CSpinButtonCtrl* pMaxDelaySpin = (CSpinButtonCtrl*)GetDlgItem(IDC_MAX_DELAY_SPIN);
    pMaxDelaySpin->SetRange(0,UD_MAXVAL);
    pMaxDelaySpin->SetAccel(nNumAccel,Accel);
    pMaxDelaySpin->SetBuddy(GetDlgItem(IDC_MAX_DELAY));
    
	m_StatLevel.InsertString(0,"None");
    m_StatLevel.InsertString(1,"Low");
    m_StatLevel.InsertString(2,"High");
    m_StatLevel.SetCurSel(m_nStatLev);

	UpdateListOfBrowsers();

	if(m_bRandomIterationDelay)
		OnRandomDelay();
	else
		OnConstantDelay();

    return TRUE;  // return TRUE unless you set the focus to a control
    // EXCEPTION: OCX Property Pages should return FALSE
}

BOOL CYRBrowserRatioPage::OnSetActive()
{
    CYRSimpleTestWizardSheet* psheet = (CYRSimpleTestWizardSheet*) GetParent();
    psheet->SetWizardButtons(PSWIZB_NEXT|PSWIZB_BACK);

    return CYRWizardBasePage::OnSetActive();
}

BOOL CYRBrowserRatioPage::OnKillActive()
{
    UpdateData(TRUE);

	if(m_nMaxIterationDelay < m_nMinIterationDelay)
	{
		CString str;
		str.Format("The maximum delay value must be larger than the minimum");
		AfxMessageBox(str);
		return false;
	}
    
    return CYRWizardBasePage::OnKillActive();
}

void CYRBrowserRatioPage::SavePageSettings(const CString& strWizardFile)
{
    CString strTemp;
    strTemp.Format("%d",m_nNumSimUsers);
    WritePrivateProfileString(WIZARD_TEST,WIZARD_TEST_NUM_V_USERS,strTemp,strWizardFile);

    strTemp.Format("%d",m_nIterationDelay);
    WritePrivateProfileString(WIZARD_TEST,WIZARD_TEST_ITER_DELAY,strTemp,strWizardFile);

	strTemp.Format("%d",m_bRandomIterationDelay);
    WritePrivateProfileString(WIZARD_TEST,WIZARD_TEST_RANDOM_ITER_DELAY,strTemp,strWizardFile);
    
	strTemp.Format("%d",m_nMinIterationDelay);
    WritePrivateProfileString(WIZARD_TEST,WIZARD_TEST_MIN_ITER_DELAY,strTemp,strWizardFile);
	
	strTemp.Format("%d",m_nMaxIterationDelay);
    WritePrivateProfileString(WIZARD_TEST,WIZARD_TEST_MAX_ITER_DELAY,strTemp,strWizardFile);

    strTemp.Format("%d",m_nNumIterations);
    WritePrivateProfileString(WIZARD_TEST,WIZARD_TEST_NUM_ITERATIONS,strTemp,strWizardFile);

    strTemp.Format("%d",m_bGenPageTimers);
    WritePrivateProfileString(WIZARD_TEST,WIZARD_TEST_GEN_PAGE_TIMERS,strTemp,strWizardFile);

    CString strBrowser,strAgent;
    int nBrowserIndex = 0;
    for(int nBrowser=0;nBrowser<m_arBrowsers.GetSize();nBrowser++)
    {
        if(!m_arBrowsers[nBrowser].nPercent)
            continue;

        strBrowser.Format("Browser %d",nBrowserIndex);

        WritePrivateProfileString(WIZARD_TEST,strBrowser + WIZARD_TEST_BROWSER_NAME_EXT,m_arBrowsers[nBrowser].strName,strWizardFile);

        WritePrivateProfileString(WIZARD_TEST,strBrowser + WIZARD_TEST_BROWSER_AGENT_EXT,m_arBrowsers[nBrowser].strUserAgent,strWizardFile);

        strTemp.Format("%d",m_arBrowsers[nBrowser].nPercent);
        WritePrivateProfileString(WIZARD_TEST,strBrowser + WIZARD_TEST_BROWSER_PERCENT_EXT,strTemp,strWizardFile);
        nBrowserIndex++;
    }

    strTemp.Format("%d",nBrowserIndex);
    WritePrivateProfileString(WIZARD_TEST,WIZARD_TEST_NUM_BROWSER_TYPES,strTemp,strWizardFile);

    WritePrivateProfileString(WIZARD_TEST,WIZARD_TEST_STAT_LEVEL,GetStatLevel(),strWizardFile);

}

void CYRBrowserRatioPage::LoadPageSettings(const CString& strWizardFile)
{
    CString strTemp;
    DWORD dwMaxBufferSize = 200;

    m_nNumSimUsers = (int)GetPrivateProfileInt(WIZARD_TEST,WIZARD_TEST_NUM_V_USERS,1,strWizardFile);

	m_bRandomIterationDelay = (BOOL)GetPrivateProfileInt(WIZARD_TEST,WIZARD_TEST_RANDOM_ITER_DELAY,0,strWizardFile); 
    m_nIterationDelay = (int)GetPrivateProfileInt(WIZARD_TEST,WIZARD_TEST_ITER_DELAY,0,strWizardFile);
	m_nMinIterationDelay = (int)GetPrivateProfileInt(WIZARD_TEST,WIZARD_TEST_MIN_ITER_DELAY,0,strWizardFile);
	m_nMaxIterationDelay = (int)GetPrivateProfileInt(WIZARD_TEST,WIZARD_TEST_MAX_ITER_DELAY,0,strWizardFile);

    m_nNumIterations = (int)GetPrivateProfileInt(WIZARD_TEST,WIZARD_TEST_NUM_ITERATIONS,1,strWizardFile);

	m_bGenPageTimers = (int)GetPrivateProfileInt(WIZARD_TEST,WIZARD_TEST_GEN_PAGE_TIMERS,1,strWizardFile);

    int nNumBrowsers = GetPrivateProfileInt(WIZARD_TEST,WIZARD_TEST_NUM_BROWSER_TYPES,0,strWizardFile);

    m_arBrowsers.RemoveAll();
    CString strBrowser;
    for(int nBrowser=0;nBrowser<nNumBrowsers;nBrowser++)
    {
        BrowserInfo BI;
        strBrowser.Format(WIZARD_TEST_BROWSER_FORMAT,nBrowser);
        GetPrivateProfileString(WIZARD_TEST,strBrowser + WIZARD_TEST_BROWSER_NAME_EXT,"",BI.strName.GetBuffer(dwMaxBufferSize),dwMaxBufferSize,strWizardFile);
        BI.strName.ReleaseBuffer();
        if(BI.strName.IsEmpty())
            continue;

        GetPrivateProfileString(WIZARD_TEST,strBrowser + WIZARD_TEST_BROWSER_AGENT_EXT,"",BI.strUserAgent.GetBuffer(dwMaxBufferSize),dwMaxBufferSize,strWizardFile);
        BI.strUserAgent.ReleaseBuffer();

        BI.nPercent = GetPrivateProfileInt(WIZARD_TEST,strBrowser + WIZARD_TEST_BROWSER_PERCENT_EXT,0,strWizardFile);

        m_arBrowsers.Add(BI);
    }

    CString strLev = "";
    GetPrivateProfileString(WIZARD_TEST,WIZARD_TEST_STAT_LEVEL,"",strLev.GetBuffer(dwMaxBufferSize),dwMaxBufferSize,strWizardFile);

    if (strLev == "N")
    {
        m_nStatLev = 0;
    }
    else if (strLev == "H")
    {
        m_nStatLev = 2;
    }
	else
		m_nStatLev = 1;
}

void CYRBrowserRatioPage::OnRandomDelay() 
{
	((CButton*)GetDlgItem(IDC_RANDOM_DELAY))->SetCheck(1);
	((CButton*)GetDlgItem(IDC_CONSTANT_DELAY))->SetCheck(0);
	GetDlgItem(IDC_SPIN_ITERATION_DELAY)->EnableWindow(FALSE);	
	GetDlgItem(IDC_ITERATION_DELAY)->EnableWindow(FALSE);	
	GetDlgItem(IDC_MAX_DELAY)->EnableWindow(TRUE);	
	GetDlgItem(IDC_MIN_DELAY)->EnableWindow(TRUE);
	GetDlgItem(IDC_MAX_DELAY_SPIN)->EnableWindow(TRUE);
	GetDlgItem(IDC_MIN_DELAY_SPIN)->EnableWindow(TRUE);
}

void CYRBrowserRatioPage::OnConstantDelay() 
{
	((CButton*)GetDlgItem(IDC_RANDOM_DELAY))->SetCheck(0);
	((CButton*)GetDlgItem(IDC_CONSTANT_DELAY))->SetCheck(1);
	GetDlgItem(IDC_SPIN_ITERATION_DELAY)->EnableWindow(TRUE);	
	GetDlgItem(IDC_ITERATION_DELAY)->EnableWindow(TRUE);	
	GetDlgItem(IDC_MAX_DELAY)->EnableWindow(FALSE);	
	GetDlgItem(IDC_MIN_DELAY)->EnableWindow(FALSE);
	GetDlgItem(IDC_MAX_DELAY_SPIN)->EnableWindow(FALSE);
	GetDlgItem(IDC_MIN_DELAY_SPIN)->EnableWindow(FALSE);
}

void CYRBrowserRatioPage::OnBrowserTypes() 
{
	CYRBrowserTypeDlg	dlg(m_arBrowsers,GetParent());
	dlg.DoModal();
}

void CYRBrowserRatioPage::UpdateListOfBrowsers()
{
    //Read in available browsers from headers.ini
    CStringArray arstrBrowsers;
    CArray <BrowserInfo,BrowserInfo&> arBrowsers;
    BrowserInfo BI;
    BI.nPercent=0;
    try
    {
        //The headers.ini file is located in <CYRANO_HOME>\\Impact\\Web\\Modeller
        char CyranoHome[256];
        GetEnvironmentVariable ("CYRANO_HOME" ,CyranoHome ,sizeof (CyranoHome));

        CString strHeadersFile(CyranoHome);
        if(strHeadersFile.Right(1) != "\\")
            strHeadersFile += "\\";
        strHeadersFile += "Engines\\Web\\Modeller\\headers.ini";
        CStdioFile f(strHeadersFile,CFile::modeRead);

        CString str;
        bool bInBrowsers=false;

        //Search for browsers section
        while(f.ReadString(str))
        {
            if(str.IsEmpty())
                continue;

            //Is this line the start of a section
            if(str[0] == '[')
            {
                //If this is the start of a section after the browser
                //section has been found we can stop reading
                if(bInBrowsers)
                    break;

                //Is this section the browsers section
                bInBrowsers = (str.CompareNoCase("[BROWSERS]")==0) ? true : false;
            }
            else
                if(bInBrowsers)
                {
                    //Add the browser to the list
                    int nEqualsPos = str.Find('=');
                    if(nEqualsPos == -1)
                        continue;

                    BI.strName = str.Left(nEqualsPos);
                    BI.strName.TrimRight();
                    str.Delete(0,nEqualsPos+1);
                    str.TrimRight();
                    str.TrimLeft();
                    str.TrimRight("\"");    //remove leading and trailing quotes
                    str.TrimLeft("\"");
                    BI.strUserAgent = str;

                    arBrowsers.Add(BI);
                }
        }

        f.Close();
    }
    catch(...)
    {
        return;
    }

    //Remove any browser which don't exist anymore
    bool bFound = false;
    for(int nBrowser=0;nBrowser<m_arBrowsers.GetSize();nBrowser++)
    {
        for(int nAvailable=0;nAvailable<arBrowsers.GetSize()&& !bFound;nAvailable++)
            bFound = (m_arBrowsers[nBrowser].strName == arBrowsers[nAvailable].strName);

        if(!bFound)
            m_arBrowsers.RemoveAt(nBrowser--);
        else
            bFound = false;
    }

    //Add any new browsers
    for(int nAvailable=0;nAvailable<arBrowsers.GetSize();nAvailable++)
    {
        bFound = false;
        for(int nBrowser=0;nBrowser<m_arBrowsers.GetSize();nBrowser++)
        {
            if(m_arBrowsers[nBrowser].strName == arBrowsers[nAvailable].strName)
            {
                //Update user agent field
                m_arBrowsers[nBrowser].strUserAgent = arBrowsers[nAvailable].strUserAgent;
                bFound = true;
                break;
            }
        }

        if(!bFound)
        {
            BrowserInfo BI;
            BI.nPercent = 0;
            BI.strName = arBrowsers[nAvailable].strName;
            BI.strUserAgent = arBrowsers[nAvailable].strUserAgent;
            m_arBrowsers.Add(BI);
        }
    }
}

/////////////////////////////////////////////////////////////////////////////
// CYRSummaryPage property page

IMPLEMENT_DYNCREATE(CYRSummaryPage, CYRWizardBasePage)

CYRSummaryPage::CYRSummaryPage() : CYRWizardBasePage(CYRSummaryPage::IDD)
{
    //{{AFX_DATA_INIT(CYRSummaryPage)
    m_strSummary = _T("");
    //}}AFX_DATA_INIT
    m_szHtmlHelpFile = getenv("CYRANO_HOME");
    m_szHtmlHelpFile += "\\Framework\\Help\\Commander.chm";
}

void CYRSummaryPage::DoDataExchange(CDataExchange* pDX)
{
    CYRWizardBasePage::DoDataExchange(pDX);
    //{{AFX_DATA_MAP(CYRSummaryPage)
    DDX_Control(pDX, IDC_SUMMARY, m_ctlSummary);
    DDX_Text(pDX, IDC_SUMMARY, m_strSummary);
    //}}AFX_DATA_MAP
}



BEGIN_MESSAGE_MAP(CYRSummaryPage, CYRWizardBasePage)
//{{AFX_MSG_MAP(CYRSummaryPage)
ON_WM_HELPINFO()
//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CYRSummaryPage message handlers
BOOL CYRSummaryPage::OnSetActive()
{
    CYRSimpleTestWizardSheet* psheet = (CYRSimpleTestWizardSheet*) GetParent();
    psheet->SetWizardButtons(PSWIZB_FINISH|PSWIZB_BACK);
    m_bAdvanced = psheet->GetAdvancedMode();

    ComposeSummary();

    m_ctlSummary.SetSel(-1,-1);
    return CYRWizardBasePage::OnSetActive();
}

void CYRSummaryPage::ComposeSummary()
{
    CYRSimpleTestWizardSheet *pSheet = (CYRSimpleTestWizardSheet *) GetParent();
    ASSERT (pSheet!=NULL);

    m_strSummary.Empty();

    // First the Name Page
    CYRNameDefinitionPage* pNameDefinitionPage = (CYRNameDefinitionPage*) pSheet->GetPage(CYRSimpleTestWizardSheet::enNameDefPage);
    if(pNameDefinitionPage->GetSafeHwnd() != NULL)
    {
        m_strSummary += "Test Name:\t" + pNameDefinitionPage->m_strTestName + "\r\n";
        m_strSummary += "Description:\t" + pNameDefinitionPage->m_strTestDescription + "\r\n";
        m_strSummary += "\r\n";
    }

    // Now we add in some static text
    m_strSummary += "The Test will be structured in the following manner\r\n";
    m_strSummary += "\r\n";

    // First the Main Test Name
    m_strSummary += "Main Test:  " + pNameDefinitionPage->m_strTestName + "\r\n";
    m_strSummary += "\r\n";

    CYRAuxillaryScriptPage* pAuxScriptPage = (CYRAuxillaryScriptPage*) pSheet->GetPage(CYRSimpleTestWizardSheet::enAuxScriptPage);
    if(m_bAdvanced)
    {
        // First the connection script
        if (!pAuxScriptPage->m_strPrologScript.IsEmpty())
        {
            // If we have a connection script then enter at least something here
            m_strSummary += "Prolog Script\t: " + pAuxScriptPage->m_strPrologScript + "\r\n";
            m_strSummary += "\r\n";
        }
    }


    // Now for the Script details
    CYRScriptDependencyPage* pScriptPage = (CYRScriptDependencyPage*) pSheet->GetPage(CYRSimpleTestWizardSheet::enScriptDepPage);
    for (int nNdx=0; nNdx<pScriptPage->GetNumScripts(); nNdx++)
    {
        const ScriptProperties &Script = pScriptPage->GetScript(nNdx);

        // Indent the first script
        CString strNumber, strPercentage;
        strNumber.Format("%d", nNdx+1);
        strPercentage.Format("%d", Script.m_nPercentage);

        m_strSummary += "Script " + strNumber + ":\r\n";
        m_strSummary += "\tName\t: " + Script.m_strName + "\r\n";
        //      m_strSummary += "\tDescription\t: " + Script.m_strDescription + "\r\n";
        if(m_bAdvanced)
            m_strSummary += "\tPercentage\t: " + strPercentage + "\r\n";
        m_strSummary += "\r\n";
    }

    if(m_bAdvanced)
    {
        // the disconnection script
        if(!pAuxScriptPage->m_strEpilogScript.IsEmpty())
        {
            m_strSummary += "Epilog Script: " + pAuxScriptPage->m_strEpilogScript + "\r\n";
            m_strSummary += "\r\n";
        }
    }

    //the duration details
    CYRBrowserRatioPage *pBrowserRatioPage = (CYRBrowserRatioPage *)pSheet->GetPage(CYRSimpleTestWizardSheet::enBrowserRatioPage);

    CString strNumber;
    strNumber.Format("%d", pBrowserRatioPage->m_nNumIterations);
    CString strPlural = (pBrowserRatioPage->m_nNumIterations > 1) ? "s" : "";
    m_strSummary += "The Test will perform " + strNumber + " iteration" + strPlural + ".\r\n";
    m_strSummary += "\r\n";

    //the user thread details
    if(pBrowserRatioPage->m_nNumSimUsers > 1)
    {
        CString strThreads;
        strThreads.Format("%d", pBrowserRatioPage->m_nNumSimUsers);

        m_strSummary += "The Test will create " + strThreads + " Virtual Users \r\n";
    }
    else
    {
        m_strSummary += "The Test will create a single Virtual User.\r\n";
    }
    m_strSummary += "\r\n";


    // And finally the end of test
    m_strSummary += "End of Test\r\n";
    m_strSummary += "\r\n";
    UpdateData(FALSE);
}
/////////////////////////////////////////////////////////////////////////////
// CYRScriptPropertiesDlg dialog


CYRScriptPropertiesDlg::CYRScriptPropertiesDlg(ScriptProperties *pScript,int nPercentUsed,CWnd* pParent /*=NULL*/)
        : CDialog(CYRScriptPropertiesDlg::IDD, pParent)
{
    //{{AFX_DATA_INIT(CYRScriptPropertiesDlg)
    m_strDescription = _T("");
    m_nPercentage = 0;
    m_strScriptName = _T("");
    //}}AFX_DATA_INIT
    m_szHtmlHelpFile = getenv("CYRANO_HOME");
    m_szHtmlHelpFile += "\\Framework\\Help\\Commander.chm";

    m_pScript = pScript;
    ASSERT(m_pScript);

    m_strDescription = m_pScript->m_strDescription;
    m_nPercentage = m_pScript->m_nPercentage;
    m_strScriptName = m_pScript->m_strName;

    m_nPercentUsed= nPercentUsed - m_pScript->m_nPercentage;

    m_bAdvancedMode = false;
}

void CYRScriptPropertiesDlg::DoDataExchange(CDataExchange* pDX)
{
    CDialog::DoDataExchange(pDX);
    //{{AFX_DATA_MAP(CYRScriptPropertiesDlg)
    DDX_Text(pDX, IDC_WIZ_SCRIPTDESCRIPTION, m_strDescription);
    DDX_Text(pDX, IDC_WIZ_SCRIPTPERCENTAGE, m_nPercentage);
    DDX_Control(pDX, IDC_WIZ_SCRIPTNAME, m_ctlScriptName);
    DDX_CBString(pDX, IDC_WIZ_SCRIPTNAME, m_strScriptName);
    //}}AFX_DATA_MAP
}



BEGIN_MESSAGE_MAP(CYRScriptPropertiesDlg, CDialog)
//{{AFX_MSG_MAP(CYRScriptPropertiesDlg)
ON_WM_HELPINFO()
ON_EN_SETFOCUS(IDC_WIZ_SCRIPTDESCRIPTION, OnSetfocusWizScriptdescription)
//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CYRScriptPropertiesDlg message handlers

BOOL CYRScriptPropertiesDlg::OnInitDialog()
{
    CDialog::OnInitDialog();

    m_ctlScriptName.SetLength(16);

    if(!m_bAdvancedMode)
    {
        GetDlgItem(IDC_WIZ_SCRIPTPERCENTAGE)->ShowWindow(SW_HIDE);
        GetDlgItem(IDC_PERCENT_CAPTION)->ShowWindow(SW_HIDE);
    }

    CStringArray arstrFiles;
    CYRSimpleTestWizardSheet* pSheet = (CYRSimpleTestWizardSheet*) GetParent();
    pSheet->m_pServer->GetFileList(CYRServer::enSourceFolder,"",arstrFiles);

    for(int nNdx=0;nNdx<arstrFiles.GetSize();nNdx++)
    {
        CString &strFile = arstrFiles[nNdx];

        //Don't show any scripts with -TH suffix
        if((strFile.Right(4) == ".SCL") && (strFile.Right(7) != "-TH.SCL"))
        {
            m_ctlScriptName.AddString(strFile.SpanExcluding("."));
        }
    }

    m_nPercentage = 100 - m_nPercentUsed;


    UpdateData(FALSE);
    return TRUE;  // return TRUE unless you set the focus to a control
    // EXCEPTION: OCX Property Pages should return FALSE
}


void CYRScriptPropertiesDlg::OnOK()
{
    UpdateData(TRUE);

    if(m_strScriptName.IsEmpty())
    {
        AfxMessageBox("Please enter a script name");
        return;
    }

    CYRSimpleTestWizardSheet* psheet = (CYRSimpleTestWizardSheet*) GetParent();
    if(!psheet->IsValidFilename(m_strScriptName))
    {
        return;
    }

    if(m_strScriptName.Right(3) == "-TH")
    {
        AfxMessageBox("The script name suffix \"-TH\" is reserved for use by OpenSTA.\n\nPlease enter another name");
        return;
    }

    if(m_nPercentUsed + m_nPercentage > 100)
    {
        CString str;
        str.Format("Please enter a percentage value between 0 and %d",100-m_nPercentUsed);
        AfxMessageBox(str);
        return;
    }

    if(m_strDescription.IsEmpty())
        m_strDescription = m_strScriptName;

    m_pScript->m_strDescription = m_strDescription;
    m_pScript->m_nPercentage = m_nPercentage;
    m_pScript->m_strName = m_strScriptName;

    CDialog::OnOK();
}

BOOL CYRScriptPropertiesDlg::OnHelpInfo(HELPINFO* pHelpInfo)
{
    CString contextHelp = m_szHtmlHelpFile+"::whatisthis.txt";

    if (pHelpInfo->iCtrlId == IDC_STATIC)
        return true;

    CWnd* pWnd = GetDlgItem(pHelpInfo->iCtrlId);

    HtmlHelp(pWnd->m_hWnd, contextHelp, HH_TP_HELP_CONTEXTMENU, (DWORD)(LPVOID)g_TopicIdArrayProps);

    return true;
}

//
// OnSetfocusWizScriptdescription
// ==============================
//
// This function will default the description text to the
// script name
void CYRScriptPropertiesDlg::OnSetfocusWizScriptdescription()
{
    UpdateData(TRUE);
    if(m_strDescription.IsEmpty())
    {
        m_strDescription = m_strScriptName;
        UpdateData(FALSE);
    }

}

BOOL CYRWizardBasePage::OnWizardFinish()
{
    CYRSimpleTestWizardSheet* psheet = (CYRSimpleTestWizardSheet*) GetParent();
    psheet->OnWizardFinish();

    return true;
}




BOOL CYRAuxillaryScriptPage::OnHelpInfo(HELPINFO* pHelpInfo)
{
    CString contextHelp = m_szHtmlHelpFile+"::whatisthis.txt";

    if (pHelpInfo->iCtrlId == IDC_STATIC)
        return true;

    CWnd* pWnd = GetDlgItem(pHelpInfo->iCtrlId);

    HtmlHelp(pWnd->m_hWnd, contextHelp, HH_TP_HELP_CONTEXTMENU, (DWORD)(LPVOID)g_TopicIdArrayAux);

    return true;
}

BOOL CYRBrowserRatioPage::OnHelpInfo(HELPINFO* pHelpInfo)
{
    CString contextHelp = m_szHtmlHelpFile+"::whatisthis.txt";

    if (pHelpInfo->iCtrlId == IDC_STATIC)
        return true;

    CWnd* pWnd = GetDlgItem(pHelpInfo->iCtrlId);

    HtmlHelp(pWnd->m_hWnd, contextHelp, HH_TP_HELP_CONTEXTMENU, (DWORD)(LPVOID)g_TopicIdArrayRatios);

    return true;
}

BOOL CYRNameDefinitionPage::OnHelpInfo(HELPINFO* pHelpInfo)
{
    CString contextHelp = m_szHtmlHelpFile+"::whatisthis.txt";

    if (pHelpInfo->iCtrlId == IDC_STATIC)
        return true;

    CWnd* pWnd = GetDlgItem(pHelpInfo->iCtrlId);

    HtmlHelp(pWnd->m_hWnd, contextHelp, HH_TP_HELP_CONTEXTMENU, (DWORD)(LPVOID)g_TopicIdArrayNameDef);

    return true;
}

BOOL CYRScriptDependencyPage::OnHelpInfo(HELPINFO* pHelpInfo)
{
    CString contextHelp = m_szHtmlHelpFile+"::whatisthis.txt";

    if (pHelpInfo->iCtrlId == IDC_STATIC)
        return true;

    CWnd* pWnd = GetDlgItem(pHelpInfo->iCtrlId);

    HtmlHelp(pWnd->m_hWnd, contextHelp, HH_TP_HELP_CONTEXTMENU, (DWORD)(LPVOID)g_TopicIdArrayDepends);

    return true;
}

BOOL CYRSummaryPage::OnHelpInfo(HELPINFO* pHelpInfo)
{
    CString contextHelp = m_szHtmlHelpFile+"::whatisthis.txt";

    if (pHelpInfo->iCtrlId == IDC_STATIC)
        return true;

    CWnd* pWnd = GetDlgItem(pHelpInfo->iCtrlId);

    HtmlHelp(pWnd->m_hWnd, contextHelp, HH_TP_HELP_CONTEXTMENU, (DWORD)(LPVOID)g_TopicIdArraySummary);

    return true;
}

CString CYRBrowserRatioPage::GetStatLevel()
{
    CString strLevel = "";
    int nIndex = m_StatLevel.GetCurSel();
    m_StatLevel.GetLBText(nIndex, strLevel);

    // Just get the first character "N", "L" or "H" ready for adding to
    // the script.
    strLevel = strLevel.Left(1);

    return strLevel;
}

bool CYRSimpleTestWizardSheet::IsValidFilename(const CString &strPath,bool bTellUser)
{
    CString strFileName(strPath);

    int nSlashPos = strPath.ReverseFind('\\');
    if(nSlashPos != -1)
    {
        strFileName = strPath.Right(strPath.GetLength()-nSlashPos-1);
    }

    //remove file extension
    strFileName.TrimRight(".");
    strFileName.MakeUpper();

    //Check for reserved file names
    if(strFileName == "CON"
            || strFileName == "AUX"
            || strFileName == "CLOCK$"
            || strFileName == "PRN"

            //check for COM1,COM2... and LPT1,LPT2...
            || ((strFileName.GetLength()==4) && (isdigit(strFileName[3])) &&
                (strFileName == "COM" || strFileName == "LPT"))
      )
    {
        if(bTellUser)
        {
            CString strMsg;
            strMsg.Format("The filename \"%s\"\nis not allowed because it contains a name reserved for use by Windows",strPath);
            AfxMessageBox(strMsg,MB_ICONWARNING|MB_OK);
        }
        return false;
    }

    return true;
}
/////////////////////////////////////////////////////////////////////////////
// CYRModeSelectionPage property page

IMPLEMENT_DYNCREATE(CYRModeSelectionPage, CYRWizardBasePage)

CYRModeSelectionPage::CYRModeSelectionPage() : CYRWizardBasePage(CYRModeSelectionPage::IDD)
{
    //{{AFX_DATA_INIT(CYRModeSelectionPage)
    // NOTE: the ClassWizard will add member initialization here
    //}}AFX_DATA_INIT
}


CYRModeSelectionPage::~CYRModeSelectionPage()
{}


void CYRModeSelectionPage::DoDataExchange(CDataExchange* pDX)
{
    CYRWizardBasePage::DoDataExchange(pDX);
    //{{AFX_DATA_MAP(CYRModeSelectionPage)
    // NOTE: the ClassWizard will add DDX and DDV calls here
    //}}AFX_DATA_MAP
}



BEGIN_MESSAGE_MAP(CYRModeSelectionPage, CYRWizardBasePage)
//{{AFX_MSG_MAP(CYRModeSelectionPage)
ON_BN_CLICKED(IDC_ADVANCED, OnAdvanced)
ON_BN_CLICKED(IDC_STANDARD, OnStandard)
//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CYRModeSelectionPage message handlers

void CYRModeSelectionPage::OnAdvanced()
{
    m_bAdvancedMode = true;
}

void CYRModeSelectionPage::OnStandard()
{
    m_bAdvancedMode = false;
}

void CYRModeSelectionPage::LoadPageSettings(const CString& strWizardFile)
{
    m_bAdvancedMode = GetPrivateProfileInt(WIZARD_TEST,WIZARD_TEST_MODE,0,strWizardFile);
}

void CYRModeSelectionPage::SavePageSettings(const CString& strWizardFile)
{
    WritePrivateProfileString(WIZARD_TEST,WIZARD_TEST_MODE,(m_bAdvancedMode ? "1" : "0"),strWizardFile);
}

LRESULT CYRModeSelectionPage::OnWizardNext()
{
    CYRSimpleTestWizardSheet* psheet = (CYRSimpleTestWizardSheet*) GetParent();
    psheet->SetAdvancedMode(m_bAdvancedMode);

    return CYRWizardBasePage::OnWizardNext();
}

BOOL CYRModeSelectionPage::OnSetActive()
{
    ((CButton*)GetDlgItem(IDC_ADVANCED))->SetCheck( m_bAdvancedMode);
    ((CButton*)GetDlgItem(IDC_STANDARD))->SetCheck(!m_bAdvancedMode);

    return CYRWizardBasePage::OnSetActive();
}

/////////////////////////////////////////////////////////////////////////////
// CYRBrowserTypeDlg dialog


CYRBrowserTypeDlg::CYRBrowserTypeDlg(CArray <BrowserInfo,BrowserInfo&>& arBrowsers,CWnd* pParent /*=NULL*/)
	: CDialog(CYRBrowserTypeDlg::IDD, pParent),
	m_arBrowsers(arBrowsers)
{
	
	//{{AFX_DATA_INIT(CYRBrowserTypeDlg)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT
}


void CYRBrowserTypeDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CYRBrowserTypeDlg)
		// NOTE: the ClassWizard will add DDX and DDV calls here
	//}}AFX_DATA_MAP
    DDX_Control(pDX, IDC_BROWSERS, m_ctlBrowserList);
}


BEGIN_MESSAGE_MAP(CYRBrowserTypeDlg, CDialog)
	//{{AFX_MSG_MAP(CYRBrowserTypeDlg)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CYRBrowserTypeDlg message handlers
void CYRBrowserTypeDlg::SetupBrowserList()
{
    if(m_arBrowsers.GetSize()==0)
        m_ctlBrowserList.EnableWindow(FALSE);

    //Setup up the columns in the grid
    const int nNumColumns = 2;
    m_ctlBrowserList.SetColumnCount(nNumColumns);
    m_ctlBrowserList.SetFixedRowCount();
    m_ctlBrowserList.SetItemText(0,0, "Browser");
    m_ctlBrowserList.SetItemText(0,1, "%");
    m_ctlBrowserList.SetRowCount(m_arBrowsers.GetSize()+1);
    m_ctlBrowserList.ExpandColumnsToFit();

    /*
     
        //Set Browser column read only
        m_ctlBrowserList.SetStyleRange(CGXRange().SetCols(1)
                ,CGXStyle().SetEnabled(FALSE));
        
    */
    for (int i = 1; i<m_ctlBrowserList.GetRowCount(); i++)
    {
        m_ctlBrowserList.CreateSpinCell(i,1);
        m_ctlBrowserList.GetCell(i,0)->SetState(GVIS_READONLY);
		CECSpinEditCell* pCell = (CECSpinEditCell*)m_ctlBrowserList.GetCell(i,1);
		pCell->SetNumberOnly(true);
		pCell->SetRange(0,100);
    }

    int nNumBrowsers = m_arBrowsers.GetSize();
    int nPercent=100;
    int nTotal = 0;

    CString strPercent;
    for(int nRow=nNumBrowsers-1;nRow>=0;nRow--)
    {
        m_ctlBrowserList.SetItemText(nRow+1,0,m_arBrowsers[nRow].strName);

        nPercent = m_arBrowsers[nRow].nPercent;
        if(nTotal+nPercent>100)
        {
            nPercent = 100 - nTotal;
            nTotal = 100;
        }
        else
            nTotal += nPercent;

		if(nRow==0)
			nPercent += 100-nTotal;

        strPercent.Format("%d",nPercent);
        m_ctlBrowserList.SetItemText(nRow+1,1,strPercent);
    }
}

BOOL CYRBrowserTypeDlg::OnInitDialog() 
{
	CDialog::OnInitDialog();
	
    SetupBrowserList();
    
	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

void CYRBrowserTypeDlg::OnOK() 
{
	UpdateData(TRUE);

    int nTotal = 0;
    CString strPercent;
    for(int nRow=0;nRow<m_arBrowsers.GetSize();nRow++)
    {
        strPercent = m_ctlBrowserList.GetItemText(nRow+1,1);

        m_arBrowsers[nRow].nPercent = atoi(strPercent);
        if(m_arBrowsers[nRow].nPercent < 0)
        {
            m_arBrowsers[nRow].nPercent *= -1;
            strPercent.Format("%d",m_arBrowsers[nRow].nPercent);
            m_ctlBrowserList.SetItemText(nRow+1,1,strPercent);
        }

        nTotal += m_arBrowsers[nRow].nPercent;
    }

    if(m_arBrowsers.GetSize() && nTotal != 100)
    {
        CString str;
        str.Format("The total of all the browser must add up to 100%%\n\nYou have %s allocated by %d%%.",(nTotal > 100 ? "over" : "under"),abs(100-nTotal));
        AfxMessageBox(str,MB_OK);
        return;
    }
	
	CDialog::OnOK();
}

