/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// TParamRule.h
#ifndef CYR_TPARAMRULE_H
#define CYR_TPARAMRULE_H
/**
 * Object to represent rule for a parameter
 **/

#include <string>
#include <set>
#ifdef unix
#elif defined (__VMS)
#elif defined (WIN32)
#pragma warning(disable:4786)
using namespace std;
class TParamRule;
ostream& operator<<(ostream& os, const TParamRule& pR);
#else //unknown
#error Unknown Platform
#endif // unknown

#include "AtvfRecRule.h"
#include "TOptions.h"
#include "TSegStr.h"
#include "CConfigTable.h"

typedef set<AtvfKey, less<AtvfKey> > attrset_t;

// class definition
class TParamRule
{
    AtvfRecRule* _aRRuleP;
    TOptions _options;
    string _defVal;
    attrset_t _attrs;
    const CConfigTable* _configs;

protected:
    // construtors & destructor
    TParamRule();
    TParamRule(AtvfRecRule* aRRuleP);
    // accessors
    virtual void giveContents(TParamRule* pRP) const;
public:
    virtual ~TParamRule();

    // accessors
    AtvfKey key() const;
    const AtvfRecRule* innerRule() const;
    string token() const;
    const TOptions& options() const;
    bool hasAttr(const string& attr) const;
    virtual string defStrVal() const;
    virtual int defIntVal() const;
    virtual int intVal(const string&) const;
    virtual string intVal(int) const;
    virtual bool intVal() const;
    virtual TSegStr* segStr(const string& value) const;
    virtual TParamRule* duplicate(const string& name,
                                  const string& type) const = 0;
    virtual string type() const = 0;
    const CConfigTable* config() const;

    // mutators
    void addOption(const string& text);
    void setAttr(const string& attr);
    void setAttr(const attrset_t& attr);
    void addOption(const TOptions& opts);
    void defStrVal(const string& defVal);
    void addConfigs(const CConfigTable* configIn);
    void setAtvfRule(AtvfRecRule* const sharedRule );
    // friend operators
    friend ostream& operator<<(ostream& os, const TParamRule& pR);

}
; // TParamRule

// map to hold these parameter objects referenced by key
typedef map<AtvfKey, TParamRule*, less<AtvfKey> > prulep_cont_t;

#endif // CYR_TPARAMRULE_H
