/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

#include "CECMutexList.h"
/////////////////////////////////////////////////////////////////
// @mfunc   Constructor
CECMutexList::CECMutexList()
{
    m_mutexmap.clear();
}
/////////////////////////////////////////////////////////////////
// @mfunc   Destructor
CECMutexList::~CECMutexList()
{
    mutexmap_t::iterator i;
    for(i=m_mutexmap.begin();i!=m_mutexmap.end();i++)
    {
        //
        // LINUX Port Changes
        //
        // For Linux we cannot use the arrow operator on this object
        delete (*i).second;
    }
    m_mutexmap.clear();
}
/////////////////////////////////////////////////////////////////
// @mfunc   Get a mutex from the current list or create new one if required
// @parm    name of mutex
// @parm    scope (0=local,1=testwide)
// @rdesc   pointer to mutex
CECMutex* CECMutexList::GetMutex(string name,int scope)
{
    MutexId mtxid(name,scope);
    mutexmap_t::iterator i;
    i=m_mutexmap.find(mtxid);
    if(i!=m_mutexmap.end())
    {
        //
        // LINUX Port Changes
        //
        // For Linux we cannot use the arrow operator on this object
        return (*i).second;
    }
    else
    {
        CECMutex* mtx=new CECMutex(name,scope);
        m_mutexmap.insert(mutexmap_t::value_type(mtxid,mtx));
        return mtx;
    }

}
/////////////////////////////////////////////////////////////////
// @mfunc   Constructor
CECMutexList::MutexId::MutexId(string name,int scope)
        :_name(name),_scope(scope)
{}
/////////////////////////////////////////////////////////////////
// @mfunc   Destructor

CECMutexList::MutexId::~MutexId()
{}
/////////////////////////////////////////////////////////////////
// @mfunc   compare to mutex id's
// @parm    mutex to compare
// @rdesc   -1 mutex compared is less than current, 1 is greater, 0 is equal to

int CECMutexList::MutexId::compare(const CECMutexList::MutexId& rhs)const
{
    if(rhs._name<_name)return -1;//less than
    if(rhs._name>_name)return 1;//greater than
    if(rhs._name==_name)
    {
        if((rhs._scope==_scope))return 0;//are equal
        if(rhs._scope<_scope)
            return -1;
        else
            return 1;
    }
    return 0;
}
// less than operator, name most significant
#ifndef LINUX
bool operator<(const CECMutexList::MutexId& rhs,const CECMutexList::MutexId& lhs)
{
    if( rhs.compare(lhs)==-1)
        return true;
    else
        return false;
}
#endif
