/* Copyright (C) 2000 Cyrano SA

   This file is part of OpenSTA

   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.

   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.

   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.

   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html

   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// TCommonRcrd.inl

#include "TCommonRcrd.h"

// Define RTTI keys for each record type ...

// VARIABLE MANIPULATION
const AtvfKey TRcrdSetvar::			KEY("Setvar");
CTMutex TRcrdSetvar::m_mutex;
const AtvfKey TRcrdFormat::			KEY("Format");
CTMutex TRcrdFormat::m_mutex;
const AtvfKey TRcrdGenvar::			KEY("Genvar");
const AtvfKey TRcrdSeed::			KEY("Seed");

// DIAGNOSTIC / MESSAGING
const AtvfKey TRcrdLogmsg::			KEY("Logmsg");

// SYNCHRONISATION
const AtvfKey TRcrdWait::			KEY("Wait");
// FLOW CONTROL
const AtvfKey TRcrdLabel::			KEY("Label");
const AtvfKey TRcrdCond::			KEY("Conditional");
CTMutex TRcrdCond::m_mutex;
const AtvfKey TRcrdIF_Then::			KEY("IfThen");
CTMutex TRcrdIF_Then::m_mutex;
const AtvfKey TRcrdElse::			KEY("Else");
CTMutex TRcrdElse::m_mutex;
const AtvfKey TRcrdEndIF::			KEY("Eif");
const AtvfKey TRcrdLoop::			KEY("Loop");
CTMutex TRcrdLoop::m_mutex;
const AtvfKey TRcrdEndLoop::			KEY("Eloop");
const AtvfKey TRcrdBreakContinue::		KEY("BreakContinue");
const AtvfKey TRcrdGoto::			KEY("Goto");
CTMutex TRcrdGoto::m_mutex;

// CALL SUBROUTINE
const AtvfKey TRcrdCsub::			KEY("Csub");
CTMutex TRcrdCsub::m_mutex;
const AtvfKey TRcrdSubroutine::			KEY("Subroutine");
const AtvfKey TRcrdReturn::			KEY("Return");


//ERROR HANDLING/TERMINATION COMMANDS
const AtvfKey TRcrdGoError::			KEY("Goerror");
const AtvfKey TRcrdDisable::			KEY("Disable");
const AtvfKey TRcrdExit::			KEY("Exit");


// STATE
const AtvfKey TRcrdLoad::			KEY("Load");

// FILE HANDLING
const AtvfKey TRcrdOpen::			KEY("Open");
const AtvfKey TRcrdRead::			KEY("Read");
CTMutex TRcrdRead::m_mutex;
const AtvfKey TRcrdWrite::			KEY("Write");
CTMutex TRcrdWrite::m_mutex;
const AtvfKey TRcrdFcntl::			KEY("Fcntl");

// FORMAL TEST CONTROL
const AtvfKey TRcrdTcstart::			KEY("Tcstart");
const AtvfKey TRcrdTcterm::			KEY("Tcterm");
CTMutex TRcrdTcterm::m_mutex;
const AtvfKey TRcrdTcend::			KEY("Tcend");


// The following sections follow the headings detailed in the document
// 'Compiling SCL to TOF'

// *****************************************************************************
// * VARIABLE MANIPULATION
// *****************************************************************************

const bool TRcrdSetvar::_registeredCreator = TRcrdSetvar::_RegisterCreator();
const bool TRcrdSetvar::_RegisterCreator()
{
	if (!recCreatorMap) recCreatorMap = new RecCreator_map_t;
	(*recCreatorMap)[TRcrdSetvar::KEY] = new TRecMake<TRcrdSetvar>();
	return true;
}


const bool TRcrdFormat::_registeredCreator = TRcrdFormat::_RegisterCreator();
const bool TRcrdFormat::_RegisterCreator()
{
	if (!recCreatorMap) recCreatorMap = new RecCreator_map_t;
	(*recCreatorMap)[TRcrdFormat::KEY] = new TRecMake<TRcrdFormat>();
	return true;
}

const bool TRcrdGenvar::_registeredCreator = TRcrdGenvar::_RegisterCreator();
const bool TRcrdGenvar::_RegisterCreator()
{
	if (!recCreatorMap) recCreatorMap = new RecCreator_map_t;
	(*recCreatorMap)[TRcrdGenvar::KEY] = new TRecMake<TRcrdGenvar>();
	return true;
}
const bool TRcrdSeed::_registeredCreator = TRcrdSeed::_RegisterCreator();
const bool TRcrdSeed::_RegisterCreator()
{
	if (!recCreatorMap) recCreatorMap = new RecCreator_map_t;
	(*recCreatorMap)[TRcrdSeed::KEY] = new TRecMake<TRcrdSeed>();
	return true;
}


// *****************************************************************************
// * DIAGNOSTIC / MESSAGING
// *****************************************************************************

const bool TRcrdLogmsg::_registeredCreator = TRcrdLogmsg::_RegisterCreator();
const bool TRcrdLogmsg::_RegisterCreator()
{
	if (!recCreatorMap) recCreatorMap = new RecCreator_map_t;
	(*recCreatorMap)[TRcrdLogmsg::KEY] = new TRecMake<TRcrdLogmsg>();
	return true;
}

// *****************************************************************************
// * SYNCHRONISATION
// *****************************************************************************

const bool TRcrdWait::_registeredCreator = TRcrdWait::_RegisterCreator();
const bool TRcrdWait::_RegisterCreator()
{
	if (!recCreatorMap) recCreatorMap = new RecCreator_map_t;
	(*recCreatorMap)[TRcrdWait::KEY] = new TRecMake<TRcrdWait>();
	return true;
}

const bool TRcrdLabel::_registeredCreator = TRcrdLabel::_RegisterCreator();
const bool TRcrdLabel::_RegisterCreator()
{
	if (!recCreatorMap) recCreatorMap = new RecCreator_map_t;
	(*recCreatorMap)[TRcrdLabel::KEY] = new TRecMake<TRcrdLabel>();
	return true;
}



// *****************************************************************************
// * FLOW CONTROL
// *****************************************************************************
const bool TRcrdCond::_registeredCreator = TRcrdCond::_RegisterCreator();
const bool TRcrdCond::_RegisterCreator()
{
	if (!recCreatorMap) recCreatorMap = new RecCreator_map_t; 
	(*recCreatorMap)[TRcrdCond::KEY] = new TRecMake<TRcrdCond>();
	return true;
}

const bool TRcrdIF_Then::_registeredCreator = TRcrdIF_Then::_RegisterCreator();
const bool TRcrdIF_Then::_RegisterCreator()
{
	if (!recCreatorMap) recCreatorMap = new RecCreator_map_t;
	(*recCreatorMap)[TRcrdIF_Then::KEY] = new TRecMake<TRcrdIF_Then>();
	return true;
}

const bool TRcrdElse::_registeredCreator = TRcrdElse::_RegisterCreator();
const bool TRcrdElse::_RegisterCreator()
{
	if (!recCreatorMap) recCreatorMap = new RecCreator_map_t;
	(*recCreatorMap)[TRcrdElse::KEY] = new TRecMake<TRcrdElse>();
	return true;
}

const bool TRcrdEndIF::_registeredCreator = TRcrdEndIF::_RegisterCreator();
const bool TRcrdEndIF::_RegisterCreator()
{
	if (!recCreatorMap) recCreatorMap = new RecCreator_map_t;
	(*recCreatorMap)[TRcrdEndIF::KEY] = new TRecMake<TRcrdEndIF>();
	return true;
}

const bool TRcrdLoop::_registeredCreator = TRcrdLoop::_RegisterCreator();
const bool TRcrdLoop::_RegisterCreator()
{
	if (!recCreatorMap) recCreatorMap = new RecCreator_map_t;
	(*recCreatorMap)[TRcrdLoop::KEY] = new TRecMake<TRcrdLoop>();
	return true;
}

const bool TRcrdEndLoop::_registeredCreator = TRcrdEndLoop::_RegisterCreator();
const bool TRcrdEndLoop::_RegisterCreator()
{
	if (!recCreatorMap) recCreatorMap = new RecCreator_map_t;
	(*recCreatorMap)[TRcrdEndLoop::KEY] = new TRecMake<TRcrdEndLoop>();
	return true;
}

const bool TRcrdBreakContinue::_registeredCreator = TRcrdBreakContinue::_RegisterCreator();
const bool TRcrdBreakContinue::_RegisterCreator()
{
	if (!recCreatorMap) recCreatorMap = new RecCreator_map_t;
	(*recCreatorMap)[TRcrdBreakContinue::KEY] = new TRecMake<TRcrdBreakContinue>();
	return true;
}


const bool TRcrdGoto::_registeredCreator = TRcrdGoto::_RegisterCreator();
const bool TRcrdGoto::_RegisterCreator()
{
	if (!recCreatorMap) recCreatorMap = new RecCreator_map_t; 
	(*recCreatorMap)[TRcrdGoto::KEY] = new TRecMake<TRcrdGoto>();
	return true;
}

const bool TRcrdSubroutine::_registeredCreator = TRcrdSubroutine::_RegisterCreator();
const bool TRcrdSubroutine::_RegisterCreator()
{
	if (!recCreatorMap) recCreatorMap = new RecCreator_map_t; 
	(*recCreatorMap)[TRcrdSubroutine::KEY] = new TRecMake<TRcrdSubroutine>();
	return true;
}

const bool TRcrdReturn::_registeredCreator = TRcrdReturn::_RegisterCreator();
const bool TRcrdReturn::_RegisterCreator()
{
	if (!recCreatorMap) recCreatorMap = new RecCreator_map_t; 
	(*recCreatorMap)[TRcrdReturn::KEY] = new TRecMake<TRcrdReturn>();
	return true;
}

const bool TRcrdCsub::_registeredCreator = TRcrdCsub::_RegisterCreator();
const bool TRcrdCsub::_RegisterCreator()
{
	if (!recCreatorMap) recCreatorMap = new RecCreator_map_t; 
	(*recCreatorMap)[TRcrdCsub::KEY] = new TRecMake<TRcrdCsub>();
	return true;
}
///////////////////////////////////////////////////////////////////////////
//  ERROR HANDLING/TERMINATION COMMANDS
///////////////////////////////////////////////////////////////////////////
const bool TRcrdGoError::_registeredCreator = TRcrdGoError::_RegisterCreator();
const bool TRcrdGoError::_RegisterCreator()
{
	if (!recCreatorMap) recCreatorMap = new RecCreator_map_t; 
	(*recCreatorMap)[TRcrdGoError::KEY] = new TRecMake<TRcrdGoError>();
	return true;
}
const bool TRcrdDisable::_registeredCreator = TRcrdDisable::_RegisterCreator();
const bool TRcrdDisable::_RegisterCreator()
{
	if (!recCreatorMap) recCreatorMap = new RecCreator_map_t; 
	(*recCreatorMap)[TRcrdDisable::KEY] = new TRecMake<TRcrdDisable>();
	return true;
}
const bool TRcrdExit::_registeredCreator = TRcrdExit::_RegisterCreator();
const bool TRcrdExit::_RegisterCreator()
{
	if (!recCreatorMap) recCreatorMap = new RecCreator_map_t; 
	(*recCreatorMap)[TRcrdExit::KEY] = new TRecMake<TRcrdExit>();
	return true;
}

// *****************************************************************************
// * STATE
// *****************************************************************************

const bool TRcrdLoad::_registeredCreator = TRcrdLoad::_RegisterCreator();
const bool TRcrdLoad::_RegisterCreator()
{
	if (!recCreatorMap) recCreatorMap = new RecCreator_map_t;
	(*recCreatorMap)[TRcrdLoad::KEY] = new TRecMake<TRcrdLoad>();
	return true;
}

// *****************************************************************************
// * FILE HANDLING
// *****************************************************************************
const bool TRcrdOpen::_registeredCreator = TRcrdOpen::_RegisterCreator();
const bool TRcrdOpen::_RegisterCreator()
{
	if (!recCreatorMap) recCreatorMap = new RecCreator_map_t;
	(*recCreatorMap)[TRcrdOpen::KEY] = new TRecMake<TRcrdOpen>();
	return true;
}

const bool TRcrdRead::_registeredCreator = TRcrdRead::_RegisterCreator();
const bool TRcrdRead::_RegisterCreator()
{
	if (!recCreatorMap) recCreatorMap = new RecCreator_map_t;
	(*recCreatorMap)[TRcrdRead::KEY] = new TRecMake<TRcrdRead>();
	return true;
}

const bool TRcrdWrite::_registeredCreator = TRcrdWrite::_RegisterCreator();
const bool TRcrdWrite::_RegisterCreator()
{
	if (!recCreatorMap) recCreatorMap = new RecCreator_map_t;
	(*recCreatorMap)[TRcrdWrite::KEY] = new TRecMake<TRcrdWrite>();
	return true;
}

const bool TRcrdFcntl::_registeredCreator = TRcrdFcntl::_RegisterCreator();
const bool TRcrdFcntl::_RegisterCreator()
{
	if (!recCreatorMap) recCreatorMap = new RecCreator_map_t;
	(*recCreatorMap)[TRcrdFcntl::KEY] = new TRecMake<TRcrdFcntl>();
	return true;
}

// *****************************************************************************
// * FORMAL TEST CONTROL
// *****************************************************************************
const bool TRcrdTcstart::_registeredCreator = TRcrdTcstart::_RegisterCreator();
const bool TRcrdTcstart::_RegisterCreator()
{
	if (!recCreatorMap) recCreatorMap = new RecCreator_map_t;
	(*recCreatorMap)[TRcrdTcstart::KEY] = new TRecMake<TRcrdTcstart>();
	return true;
}

const bool TRcrdTcterm::_registeredCreator = TRcrdTcterm::_RegisterCreator();
const bool TRcrdTcterm::_RegisterCreator()
{
	if (!recCreatorMap) recCreatorMap = new RecCreator_map_t;
	(*recCreatorMap)[TRcrdTcterm::KEY] = new TRecMake<TRcrdTcterm>();
	return true;
}

const bool TRcrdTcend::_registeredCreator = TRcrdTcend::_RegisterCreator();
const bool TRcrdTcend::_RegisterCreator()
{
	if (!recCreatorMap) recCreatorMap = new RecCreator_map_t;
	(*recCreatorMap)[TRcrdTcend::KEY] = new TRecMake<TRcrdTcend>();
	return true;
}

// Lookup the TRecCreator template for a Transaction Record given its KEY.
TRecCreator* getTRecCreator(const AtvfKey& key)
{
	RecCreator_map_t::const_iterator i = recCreatorMap->find(key);
	return ((i == recCreatorMap->end()) ? new TRecMake<TRecord>() : (*i).second);
}
