/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// cxxutils.h

#ifndef CXXUTILS_H
#define CXXUTILS_H

#if defined(WIN32)
# if _MSC_VER >= 1100   // MSC version 5 and above
#pragma warning(disable:4786)
# endif
#endif

#include <string>
#include "mdbtypes2.h"

#ifndef CYR_NO_STD_NAMESPACE
using std::string;
#endif


extern "C"
{
#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include <errno.h>
}

// Forward declarations
class CRegExp;

// Prototypes

// Utility function, convert characters in string to upper case
void string_toupper(string & str);
// Utility function, convert characters in string to lower case
void string_tolower(string & str);
// Strip any leading directory from path return file name only
string filename_base(const string & path);
// Strip any file name from path return directory only
string filename_dir(const string & path);
// Strip any file extension from path (anything after a dot).
string strip_extension(const string & path);

// Utility function, convert integer to a string.
string int_to_string(int i);
// Utility function, convert unsigned integer to a string.
string int_to_string(unsigned i);
string int_to_string(long i);

// Check if file in path can be opened for write
bool file_is_writable(string& path, string& whynot);
// Check if file in path can be opened for read
bool file_is_readable(string& path, string& whynot);
// Check if file in path exists
bool file_exists(string& path);
// Replace text in string, return true if string was founs
bool replace_in_string(string& lookin, string& txtold, string& txtnew,
                       bool matchcase, unsigned startpos = 0, bool replaceall = true);
bool replace_in_string(string& lookin, CRegExp& regex, string& txtnew,
                       bool matchcase, unsigned startpos = 0, bool replaceall = true);

// See if a string begins with a certain upper case string
bool begins_with(string lookin, string lookfor);

// Find text in string, return index if found or -1 if not.
int find_in_string(string lookin, string lookfor, bool matchcase,
                   unsigned startpos = 0);
int find_in_string(string lookin, CRegExp& regex, bool matchcase,
                   unsigned startpos = 0);
// Compare two strings
bool string_compare(string string1, string string2, bool matchcase);
// Remove specified characters from string
void remove_from_string(string & str, const char * remchars);
// Remove specified characters from end of string
void remove_trailing(string & str, const char * remchars);
// Replace single newline with CR/LF pair
void string_lf2crlf(string & str);
// Replace single CR with CR/LF pair
void string_cr2crlf(string & str);

// Uppercase lookup table
//extern char uppercaseTable[255];

// Lowercase lookup table
//extern char lowercaseTable[255];

//extern bool bInitLookups;

// Initialise all lookup tables
void InitLookupTables();

string GetFUID();

#define CYRANO_NULL_VALUE   "CyRaNoNuLlVaLuE"

#endif
