/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// Cresultset : implementation file
//

#include "resset.h"

extern "C"
{
#include <stdlib.h>
#include <malloc.h>
}

static char * invcol = "<Invalid Column>";

/////////////////////////////////////////////////////////////////////////////
// Cresultset

// Constructor
Cresultset::Cresultset()
{
    // Initialize member variables
    ncols   = 0;
    colarrsz= 0;
    fetch_count = 0;
    // Allocate column array of default initial size
    cols    = (Cresultcol **)malloc(COLFAC * sizeof(Cresultcol *));
    if(cols != NULL)
    {
        colarrsz = COLFAC;
    }
}

// Destructor
Cresultset::~Cresultset()
{
    if(cols)
    {
        int i;
        // Free columns
        for(i=0 ; i < ncols ; i++)
        {
            delete cols[i];
        }
        free((void *)cols);
    }
}

// Add new column
int Cresultset::addcol(char *name, int width)
{
    Cresultcol  **ncp;

    // If current array is full
    if(ncols >= colarrsz)
    {
        // extend array by COLFAC entries
        ncp = (Cresultcol **)realloc((void *)cols,
                                     (colarrsz + COLFAC) * sizeof(Cresultcol *));
        // Extend ok
        if(ncp != NULL)
        {
            // Set pointer to possibly new array and set new size
            cols = ncp;
            colarrsz += COLFAC;
        }
        // Extend failed, return fail
        else
        {
            return(-1);
        }
    }
    cols[ncols] = new Cresultcol(name, width);
    ncols++;
    return 0;
}

// Return name of sepecified column
char * Cresultset::getcolname(int col)
{
    // Ensure valid column
    if( (col > ncols) || (col <= 0) )
    {
        return invcol;
    }
    return cols[col-1]->getname();
}

// Return defined maximum width for specified column
int Cresultset::getcolwidth(int col)
{
    // Ensure valid column
    if( (col > ncols) || (col <= 0) )
    {
        return 0;
    }
    return cols[col-1]->getwidth();
}

// Return defined maximum display width for specified column
// This takes the length of the column name into account.
int Cresultset::getcoldwidth(int col)
{
    // Ensure valid column
    if( (col > ncols) || (col <= 0) )
    {
        return 0;
    }
    return cols[col-1]->getdisplaywidth();
}

// Return total display width (characters) for all columns in
// this result set
int Cresultset::gettotalwidth(void)
{
    int twid;
    int i;

    // Add one character per column to allow for display spacing
    twid = ncols;

    // Loop through columns, add width of each
    for(i=0 ; i < ncols ; i++)
    {
        twid += cols[i]->getdisplaywidth();
    }
    return(twid);
}

// Allocate and return a new buffer for the next result
// in the specified column
char * Cresultset::nextcoladdr(int col)
{
    // Ensure valid column
    if( (col > ncols) || (col <= 0) )
    {
        return NULL;
    }
    return cols[col-1]->nextaddr();
}

// Return column count for this result set
int Cresultset::getncols(void)
{
    return ncols;
}

// Return fetched row count for this result set
int Cresultset::getnrows(void)
{
    return fetch_count;
}

// Return result data for specified row and column
char * Cresultset::getresdata(int row, int col)
{
    // Ensure valid column
    if( (col > ncols) || (col <= 0) )
    {
        return invcol;
    }
    return cols[col-1]->getvalue(row);
}

// Add rows to current fetch count, return updated value
int Cresultset::fetched_rows(int rows)
{
    fetch_count += rows;
    return fetch_count;
}
