/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/
#include "CORBARepository.h"
#include "CECSemaphore.h"
extern CORBAServer*     pCServer;//This has become a Global variable due to
//compilation problems if included in some headers
CECSemaphore::CECSemaphore(string name,int scope)
{
    switch(scope)
    {
    case TOF_SEMSCP_TESTWIDE:
        name=Repository->GetPrimaryTestRunString()+"."+name+".TEST-WIDE";
        break;
    case TOF_SEMSCP_LOCAL:
    default:
        name=Repository->GetTestRunString()+"."+name+".LOCAL";
        break;
    }
    CORBA::Object_var obj;
    try
    {
        obj=pCServer->GetObjectFromClass(TPSemaphore::CLASS_NAME,name.c_str());
    }
    catch(...)//no Semaphore of that name available so get one from TestManager Variable Factory
    {
        string factory_regname;
        if(scope == TOF_SEMSCP_TESTWIDE)//Test-Wide
        {
            factory_regname=Repository->GetPrimaryTestRunString();
        }
        else//Local
        {
            factory_regname=Repository->GetTestRunString();
        }
        CORBA::Object_var Ofac;
        Ofac=pCServer->GetObjectFromClass(TMVarFactory::CLASS_NAME,factory_regname.c_str());
        TMVarFactory_var TMVF=narrow_object((TMVarFactory_ptr)NULL,Ofac.inout());
        CORBA::String_var cb_name=CORBA::string_dup(name.c_str());
        obj=TMVF->CreateSimpleSemaphore(cb_name.in());//Create CYRANO Test type semaphore

    }

    m_Semaphore=narrow_object((TPSemaphore_ptr)NULL,obj.inout());
}
CECSemaphore::~CECSemaphore()
{}

bool CECSemaphore::acquire()
{
    try
    {
        m_Semaphore->acquire();
    }
    catch(...)
    {
        return false;
    }
    return true;
}
bool CECSemaphore::release()
{
    try
    {
        m_Semaphore->release();
    }
    catch(...)
    {
        return false;
    }
    return true;
}
bool CECSemaphore::tryacquire()
{
    bool bret=false;
    try
    {
        bret=m_Semaphore->tryacquire();
    }
    catch(...)
    {
        return false;
    }
    return bret;
}
