/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/
// 'Helper' functions (well encapsulated helper methods for Variable use only
#include "CECVar.h"
#include <ctime.hpp>
#include <vector>
unsigned long CECVar::ParseIndecies(const string &indecies,vector<int> *min_index,vector<int> *max_index,vector<int> *size_index)
{
    string::const_iterator iter;
    string str_num="";
    int minind;
    int maxind;
    max_index->clear();
    size_index->clear();
    size_index->push_back(1);//first index is multiplier of 1
    unsigned long value_list_size=1;
    for(iter=indecies.begin();iter!=indecies.end();iter++)
    {
        if(*iter==',')
        {
            // Have found all details of 1 index, parse to see if range within index
            string::size_type pos=str_num.find_first_of(":");
            if(pos!=string::npos)
            {
                minind=atoi(str_num.substr(0,pos).c_str());
                maxind=atoi(str_num.substr(pos+1,str_num.length()-pos).c_str());
            }
            else
            {
                minind=1;//default lower bound to 1
                maxind=atoi(str_num.c_str());
            }
            if(maxind==0)
                throw mdbParseException("Incorrect Characters in Array index definition");
            min_index->push_back(minind);
            max_index->push_back(maxind);
            if(maxind<minind)//not <= allow arrays of size 1 (or else it causes problems with some SCL scripts)
                throw mdbParseException("Invalid Range in Array index definition");
            //Set Maximum number of variables to hold
            //e.g I[2:13] has from 2 to 13 inclusive as indecies 13-2+1
            value_list_size*=(maxind-minind+1);
            size_index->push_back(value_list_size);
            str_num="";
        }
        else
        {
            str_num += *iter;
        }
    }
    //Once more for the end!
    string::size_type pos=str_num.find_first_of(":");
    if(pos!=string::npos)
    {
        minind=atoi(str_num.substr(0,pos).c_str());
        maxind=atoi(str_num.substr(pos+1,str_num.length()-pos).c_str());
    }
    else
    {
        minind=1;//default lower bound to 1
        maxind=atoi(str_num.c_str());
    }
    if(maxind==0)
        throw mdbParseException("Incorrect Characters in Array index definition");
    min_index->push_back(minind);
    max_index->push_back(maxind);
    if(maxind<minind)
        throw mdbParseException("Invalid Range in Array index definition");
    //Set Maximum number of variables to hold
    //e.g I[2:13] has from 2 to 13 inclusive as indecies 13-2+1
    value_list_size*=(maxind-minind+1);
    return value_list_size;
}
//@mfunc    Take comma separated index list and convert to indecies
//@parm     comma separated list of indecies
//      (NOT an expression, this must be worked out within a context where access
//      to variable values is possible)
//@rdesc    single dimensional array index
unsigned long CECVar::GetIndexValue(const string &index,const vector<int> &min_index,const vector<int> &max_index,const vector<int> &size_index)
{
    vector<int> index_list;
    string::const_iterator iter;
    string str_num="";
    int ind;
    index_list.clear();
    for(iter=index.begin();iter!=index.end();iter++)//parse the index into separate values
    {
        if(*iter==',')
        {
            ind=atoi(str_num.c_str());
            index_list.push_back(ind);
            str_num="";
        }
        else
        {
            str_num += *iter;
        }
    }
    // add last one
    ind=atoi(str_num.c_str());
    index_list.push_back(ind);
    if(max_index.size()!=index_list.size())//Check we have the right number of indecies
        throw mdbParseException("Incorrect index values used for array, Variable:"+_name+"["+index+"]");
    vector<int>::const_iterator i;
    vector<int>::const_iterator min;
    vector<int>::const_iterator j;
    vector<int>::const_iterator s;
    long single_index=0;
    //Sorry this looks horrible, it isn't really that bad
    //anyway It could have been a more awkward looking while loop
    for(i=max_index.begin(),min=min_index.begin(),j=index_list.begin(),s=size_index.begin();
            i!=max_index.end();
            i++,min++,j++,s++)
    {
        //Check for out of bounds
        if(*j > (*i) )
            throw mdbParseException("Array index out of bounds, Variable "+_name+"["+index+"]");
        if(*j < (*min) )
            throw mdbParseException("Array index below bounds, Variable "+_name+"["+index+"]");

        //was single_index+=(*j * *s); for 0 bound arrays
        // now must remove offset (i.e. lower bound of array)
        single_index+= ( ( *j - *min ) * *s );

    }
    return single_index;//This returns the index value for a single dimensioned array
}

int CECVar::GetRandomSeed()
{
    static int seed_fudge=0;//add to seed each time used to fudge duplicate time based seeds
    CTTime Tm;
    Tm.now();
    int _now = Tm.getSeconds();
    int iret=_now+seed_fudge++;//take int , add seed fudge and return result, seed_fudge incremented
    return iret;
}
