/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// Cif.h
// Base class interface to target server

#ifndef CIF_H
#define CIF_H


#include <string>
#include "TConnect.h"
#include "TCommand.h"
#include "resset.h"
#include "mdbstl.h"

#if defined(WIN32)
# if _MSC_VER >= 1100   // MSC version 5 and above
using namespace std;
# endif
#endif


class Cif
{

public:

    // Constructors & Destructor
    Cif();
    virtual ~Cif();


public:

    bool            want_server(void)
    {
        return req_server;
    }
    bool            want_user(void)
    {
        return req_username;
    }
    bool            want_password(void)
    {
        return req_password;
    }
    bool            want_database(void)
    {
        return req_database;
    }
    bool            want_maxval(void)
    {
        return req_maxval;
    }

    virtual bool    is_connected(void)
    {
        return false;
    }
    virtual bool    got_maxval(void)
    {
        return false;
    }

    const string&   get_server(void)
    {
        return server;
    }
    const string&   get_username(void)
    {
        return username;
    }
    const string&   get_password(void)
    {
        return password;
    }
    const string&   get_database(void)
    {
        return database;
    }
    int             get_maxval(void)
    {
        return maxval;
    }
    Cresultset *    get_results(void)
    {
        return results;
    }
    virtual void    free_results(void)
    {}

    virtual void    set_results(Cresultset* res)
    {}



    const string&   get_error_text(void)
    {
        return errmsg;
    }
    int             get_error_code(void)
    {
        return errcode;
    }

    virtual void    set_server(string nsv)
    {
        BSCLEAR(server);
        server   = nsv;
    }
    virtual void    set_username(string nus)
    {
        BSCLEAR(username);
        username = nus;
    }
    virtual void    set_password(string npw)
    {
        BSCLEAR(password);
        password = npw;
    }
    virtual void    set_database(string ndb)
    {
        BSCLEAR(database);
        database = ndb;
    }
    virtual void    set_maxval(int nmv)
    {
        maxval = nmv;
    }

    virtual int     connect(void)
    {
        return -1;
    }
    virtual int     use_database (void)
    {
        return 0;
    }
    virtual int     disconnect(void)
    {
        return -1;
    }
    virtual int     send_command(string command, bool getres = true)
    {
        return -1;
    }
    virtual int     fetch_results(void)
    {
        return -1;
    }

protected:

    // Requires server name to make connection
    bool        req_server;
    // Requires user name to make connection
    bool        req_username;
    // Requires password to make connection
    bool        req_password;
    // Requires/can set database when making connection
    bool        req_database;
    // Requires a maximum value for something (currently fetch count)
    bool        req_maxval;
    // Got maximum number of values
    bool        rec_maxval;

    string      server;
    string      username;
    string      password;
    string      database;
    int         maxval;
    Cresultset* results;

    string      errmsg;
    int         errcode;

private:


};

Cif * get_server_interface();


#endif // CIF_H

