/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/
///////////////////////////////////////////////////////////////////////////////
// @doc     TestInit
// @module  TestInit| Small utility to start/stop Tests
///////////////////////////////////////////////////////////////////////////////
#include <vector>
#include <exception>
#include <iostream>
#include "TestInitiator.h"
#include <TestExecuter.h>
#include <CyrHelper.inl>
using namespace std;
void usage();
void usage()
{
    std::cout<<"usage:  TestInit -stop"<<std::endl<<
    "   TestInit -start -n Node to Run on -P Project -T TestName "<<std::endl<<
    "   -Rn Repository Node -Ru repository user -Rp Repository Password"<<std::endl<<
    "   TestInit -monitor"<<std::endl;

    //[-start|-stop] TestInit -n Node -u user -p password -P project -T test "<<std::endl
    //  <<"-Rn Repository Node -Ru repository user -Rp Repository Password"<<std::endl;
}

int main(int argc, char * argv[])
{
    std::string node="";
    std::string Project="";
    std::string Test="";
    std::string Rnode="";
    std::string Ruser="";
    std::string Rpass="";
    bool start=false;
    bool stop=false;
    bool monitor=false;
    for(int i=1;i<argc;i++)
    {
        if(strcmp(argv[i],"-n")==0)//node to run on
        {
            if(i+1<argc)node=argv[++i];
        }
        else if(strcmp(argv[i],"-P")==0)//Project
        {
            if(i+1<argc)Project=argv[++i];
        }
        else if(strcmp(argv[i],"-T")==0)//Test
        {
            if(i+1<argc)Test=argv[++i];
        }
        else if(strcmp(argv[i],"-Rn")==0)//Repository node
        {
            if(i+1<argc)Rnode=argv[++i];
        }
        else if(strcmp(argv[i],"-Ru")==0)//Repository User
        {
            if(i+1<argc)Ruser=argv[++i];
        }
        else if(strcmp(argv[i],"-Rp")==0)//Repository Password
        {
            if(i+1<argc)Rpass=argv[++i];
        }
        else if(strcmp(argv[i],"-start")==0)//want to start a test
        {
            start=true;
        }
        else if(strcmp(argv[i],"-stop")==0)//want to start a test
        {
            stop=true;
        }
        else if(strcmp(argv[i],"-monitor")==0)//want to monitor
        {
            monitor=true;
        }
        else
        {
            usage();
            return 1;
        }
    }
    try
    {
        //if we just want to find an already started TestManager
        if(stop)
        {
            CORBA::Object_var obj;
            TestManager_var TM;
            CORBAServer* corba=new CORBAServer ( argc, argv );
            std::string strTMList=corba->GetRegisteredList(TestManager::CLASS_NAME);
            std::vector<std::string> TestManagerList;
            for(int i=0;i<strTMList.length();i++)
            {
                int n=i;
                i=strTMList.find_first_of("|",n);
                std::string TMname=strTMList.substr(n,i-n);
                if(TMname=="")break;
                TestManagerList.push_back(TMname);
            }
            std::cout<<"There are "<<TestManagerList.size()<<" TestManagers running"<<std::endl;
            for(std::vector<std::string>::size_type iter=0;iter<TestManagerList.size();iter++)
            {
                std::cout<<iter<<". "<<TestManagerList[iter]<<std::endl;
            }
            std::cout<<"Choose TestManager to Stop"<<std::endl;
            int number;
            char num=getc(stdin);
            number=atoi(&num);
            if(number>TestManagerList.size())
            {
                std::cout<<num<<" is invalid"<<std::endl;
            }
            else
            {
                obj=corba->GetObjectFromClass(TestManager::CLASS_NAME, TestManagerList[number].c_str());
                TM=narrow_object((TestManager_ptr)NULL,obj.inout());
                CORBA::String_var err;
                std::cout<<"Try and Stop "<< TestManagerList[number]<<std::endl<<std::endl;
                TM->Stop();
            }
            delete corba;
        }
        else if(start)//We want to kick off a new TestManager
        {
            if(node=="")
            {
                usage();
                return 1;
            }
            if(Project=="")
            {
                usage();
                return 1;
            }
            if(Test=="")
            {
                usage();
                return 1;
            }
            if(Rnode=="")
            {
                CECInetInfo info;
                Rnode=info.GetHostName();
            }
            if(Ruser=="")
            {
                Ruser=CyrGetCurrentUser();
            }
            //if(Rpass==""){usage();return 1;}
            CECNodeDefinition nodedef(node);
            TestInitiator* tinit;

            // Creating new Test Initiator
            std::cout<<"Initializing Test System..."<<std::endl;
            tinit=new TestInitiator(Project.c_str(),Test.c_str(),nodedef);

            // And proceed with initialization if successful
            if (tinit != NULL)
            {
                std::cout<<"Done."<<std::endl;

                // Preparing repository info
                std::cout<<"Configuring Test..."<<std::endl;
                tinit->SetRepositoryInformation(Rnode.c_str(),Ruser.c_str(),Rpass.c_str());
                std::cout<<"Done."<<std::endl;

                // Creating new Test Initiator
                std::cout<<"Initializing Test Management "<<std::endl;
                tinit->CreateTestManager();
                std::cout<<"Done."<<std::endl;

                // Starting test
                std::cout<<"Starting Test..."<<std::endl;

                // Add in a delay so that the user can see the final message
                ::Sleep(500);

                // Finally we can kick off the test
                tinit->ExecuteTest();

                delete tinit;
            }
            else
            {
                // Report an error
                std::cout<<"Failed to start Test."<<std::endl;
            }
        }
        else if(monitor)//We want to Monitor things
        {
            TestManager_var TM;
            TestLevel_var TL;
            CORBAServer* corba=new CORBAServer ( argc, argv );
            CORBA::Object_var obj;
            std::cout<<std::endl<<"GetList of TestManagers"<<std::endl;
            std::string strTMList=corba->GetRegisteredList(TestManager::CLASS_NAME);
            for(int i=0;i<strTMList.length();i++)
            {
                int n=i;
                i=strTMList.find_first_of("|",n);
                std::string TMname=strTMList.substr(n,i-n);
                std::cout<<TMname<<std::endl<<std::endl;
                if(TMname=="")break;
                obj=corba->GetObjectFromClass(TestManager::CLASS_NAME, TMname.c_str());
                TM=narrow_object((TestManager_ptr)NULL,obj.inout());
                std::cout<<"TestManager Name:" << TM->Name() <<std::endl;
                std::cout<<"TestManager PID:" << TM->GetPID() <<std::endl;
                std::cout<<"NodeName:" << TM->GetNodeName() <<std::endl;
                std::cout<<"Test Cases Passed:" << TM->GetTotalTestCasePasses() <<std::endl;
                std::cout<<"Test Cases Failed:" << TM->GetTotalTestCaseFails() <<std::endl<<std::endl;
                TestLevelList TestList;
                try
                {
                    TL=TM->GetTestLevel();
                    std::cout<<"TestName:" << TL->TestName() <<std::endl;
                    std::cout<<"StartTime:" << TL->StartTime() <<std::endl;
                    std::cout<<"NodeName:" << TL->GetNode() <<std::endl;
                    std::cout<<"Contains "<<TL->NoChildTestLevels(false)<<" TestLevels"<<std::endl;
                    std::cout<<"Contains "<<TL->NoChildThreads(false)<<" TestThreads"<<std::endl<<std::endl;
                    TestExecuterList TEList;
                    CORBA::ULong no_executers=TL->GetTestExecuterList(TEList);
                    std::cout<<no_executers<<" TestExecuters Counted"<<std::endl;
                    {
                        for(long TEiter=0;TEiter<TEList.length();TEiter++)
                        {
                            std::cout<<"TestExecuter:"<<TEList[TEiter]->Name()<<std::endl;
                            ThreadInfoList TIList;
                            CORBA::ULong no_threads=TEList[TEiter]->GetThreadInformation(TIList);
                            for(long TIiter=0;TIiter<TIList.length();TIiter++)
                            {
                                std::cout<<"Thread:"<<(const char*)TIList[TIiter].UIdent<<
                                ","<<TIList[TIiter].id<<","<<(const char*)TIList[TIiter].ScriptName<<
                                //","<<(const char*)RWTime((CORBA::ULong)TIList[TIiter].StartTime).asString()<<
                                ","<<TIList[TIiter].StartTime<<
                                ","<<(const char*)TIList[TIiter].NoteText<<","<<TIList[TIiter].Status<<std::endl;

                            }
                        }
                    }
                    unsigned long no_tests=TL->GetChildTestLevelList(TestList);
                }
                catch (const CyrClosingDownException& )
                {
                    std::cout<<"Test Level Closing Down"<<std::endl;
                    break;
                }
                catch(...)
                {
                    break;
                }

                for(long TLiter=0;TLiter<TestList.length();TLiter++)
                {
                    TestLevel_var  STL=TestList[TLiter];
                    std::cout<<"SubTestName:" << STL->TestName() <<std::endl;
                    std::cout<<"StartTime:" << STL->StartTime() <<std::endl;
                    std::cout<<"Node:" << STL->GetNode() <<std::endl;
                    std::cout<<"Contains "<<STL->NoChildTestLevels(false)<<" TestLevels"<<std::endl;
                    std::cout<<"Contains "<<STL->NoChildThreads(false)<<" TestThreads"<<std::endl<<std::endl;
                    TestExecuterList TEList;
                    CORBA::ULong no_executers=STL->GetTestExecuterList(TEList);
                    std::cout<<no_executers<<" TestExecuters Counted"<<std::endl;
                    for(long TEiter=0;TEiter<TEList.length();TEiter++)
                    {
                        std::cout<<"TestExecuter:"<<TEList[TEiter]->Name()<<std::endl;
                        ThreadInfoList TIList;
                        CORBA::ULong no_threads=TEList[TEiter]->GetThreadInformation(TIList);
                        for(long TIiter=0;TIiter<TIList.length();TIiter++)
                        {
                            std::cout<<"Thread:"<<(const char*)TIList[TIiter].UIdent<<
                            ","<<TIList[TIiter].id<<","<<(const char*)TIList[TIiter].ScriptName<<
                            ","<<TIList[TIiter].StartTime<<std::endl;

                        }
                    }

                }

            }
        }
        else
        {
            usage();
            return 1;
        }
    }
    catch (const CyrClosingDownException& )
    {
        std::cout<<"Test Closing Down Monitoring to Stop"<<std::endl;
    }
    catch (const CyrProcessException& e)
    {
        std::cout<<"A CyrProcessException "<<string(e.what)<<", "<<string(e.where)<<" status " <<e.status<<std::endl;
    }
    catch (CyrRepositoryException e)
    {
        std::cout<<"A CyrRepositoryException "<<e.what()<<", "<<e.where()<<" status "<<e.status()<<std::endl;
    }
    catch (CORBA::Exception &e)
    {
        std::cout<<"A CORBA::"<<e<<std::endl;
    }
    catch (CyrException & e)
    {
        std::cout<<"A CyrException "<<e.what()<<"," <<e.where()<<e.status()<<std::endl;
    }
    catch (exception& e)
    {
        std::cout<<"An exception "<<e.what()<<std::endl;
    }
    catch(...)
    {
        std::cout<<"Unknown"<<std::endl;
    }
    return 0;
}
/*    if(argc > 2&&strcmp(argv[1],"-find")==0)
    {
    CORBA::Object_var obj;
    TestManager_var TM;
    CORBAClient* corba=new CORBAClient ( argc, argv );
    std::string strTMList=corba->GetRegisteredList(TestManager::CLASS_NAME);
    std::cout<<"Active TestManagers "<<strTMList<<std::endl;
    for(int i=0;i<strTMList.length();i++)
    {
        int n=i;
        i=strTMList.find_first_of("|",n);
        std::string TMname=strTMList.substr(n,i-n);
        if(TMname=="")break;
        obj=corba->GetObjectFromClass(TestManager::CLASS_NAME, TMname.c_str());
        TM=narrow_object((TestManager_ptr)NULL,obj);
        CORBA::String_var err;
        if(strcmp(argv[2],"-ex")==0)
        {
        TM->Initialise("rimmer","marke","marke","");
        TM->ExecuteThread("Project","MTest",TestLevel_var(TestLevel::_nil()));
        }
        else
        {
        std::cout<<"Try and Stop "<<TMname<<std::endl<<std::endl;
        TM->Stop();
        }
    delete corba;
    }
 
*/
