/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/


////////////////////////////////////////////////////////////////////////////////
// @doc TManagerRcrd
// @module  TManagerRcrd| Include this within TCommonRcrd if compiling TestManager
// All Execute methods can, if required, dynamically cast the Ctontext to a TMThread
// Pointer to call TestManager Specific Methods
//
///////////////////////////////////////////////////////////////////////////////
#ifdef WIN32
#pragma warning(disable:4786)
#pragma warning(disable:4503)
#endif
#include <string>
#ifndef CYR_NO_STD_NAMESPACE
using std::string;
#endif

#include "TMThread.h"
// Non Database Specific Transaction Records...
#include "TManagerRcrd.h"
#include <ctime.hpp>
#include "TManagerRcrd.inl"
#include "ThreadTask.h"

// *****************************************************************************
// NEW TEST CONTROL INSTRUCTION RECORDS
// *****************************************************************************

///////////////////////////////////////////////////////////////////////////////
// EXECUTE method for Execute Test Record
// =====================================
///////////////////////////////////////////////////////////////////////////////
int TRcrdExecuteTest::execute(TContext& ctxt)
{
    //Create pointer to Test Manager type thread
    TMThread* TMctxt=static_cast<TMThread*>(&ctxt);
    //if not cast safely then this is NOT a test Manager
    //This should mean that there is a bug in the code as this code should only be
    //included in Test Manager and TMThreads have been created in preference to any other
    if(!TMctxt)
        throw mdbParseException("Invalid TestManager Execution Method Called");
    ctxt.set_phase_text(PHASE_EXECUTE);
    //common parameters
    string  str_name    = strPrmVal("name", ctxt);
    string  str_retvar  = strPrmVal("retvar", ctxt);
    string  str_node    = strPrmVal("onnode", ctxt);
    TMctxt->TestExecution(str_name,str_node);
    ctxt.set_phase_text(TP_PHASE_NONE);
    return 0;

}// end of TRcrdExecuteTest::execute

///////////////////////////////////////////////////////////////////////////////
// EXECUTE method for Execute Thread Record
// =====================================
///////////////////////////////////////////////////////////////////////////////
int TRcrdExecuteThread::execute(TContext& ctxt)
{
    //Create pointer to Test Manager type thread
    TMThread* TMctxt=static_cast<TMThread*>(&ctxt);
    //if not cast safely then this is NOT a test Manager
    //This should mean that there is a bug in the code as this code should only be
    //included in Test Manager and TMThreads have been created in preference to any other
    if(!TMctxt)
        throw mdbParseException("Invalid TestManager Execution Method Called");
    ctxt.set_phase_text(PHASE_EXECUTE);
    //common parameters
    string  str_name    = strPrmVal("name", ctxt);
    string  str_retvar  = strPrmVal("retvar", ctxt);
    string  str_nodename    = strPrmVal("onnode", ctxt);
    //Thread parameters
    string  str_scname  = strPrmVal("script", ctxt);
    unsigned int    number      = intPrmVal("number",ctxt);
    unsigned int    interval    = intPrmVal("interval",ctxt);
    unsigned    int delay       = intPrmVal("delay",ctxt);
    ctxt.set_phase_text(PHASE_EXECUTE_TH);
    TMctxt->ThreadExecution(str_name,str_scname,str_nodename,number,interval,delay);
    ctxt.set_phase_text(TP_PHASE_NONE);
    return 0;

}// end of TRcrdExecuteTest::execute
///////////////////////////////////////////////////////////////////////////////
// PREPARE method for Wait For Test Record
// =======================================
///////////////////////////////////////////////////////////////////////////////
int TRcrdWtest::prepare(TScript* script, int objIndex, bool verbose)
{
    //
    const TParam* prmP = param(TParam::stexpr);
    string  l_expr = prmP->strVal();
    string::iterator siter;
    string::iterator siter_minus;
    string::iterator siter_plus;
    //set up the vector NamesToWaitFor;
    string single_string="";
    for(siter=l_expr.begin();siter!=l_expr.end();siter++)
    {
        switch (*siter)
        {
        case '&':
            siter_minus=siter_plus=siter;
            siter_minus--;
            siter_plus++;
            if(*siter_minus==' ' && *siter_plus==' ')//have an AND
            {
                NamesToWaitFor.push_back(single_string);
                single_string="";
                siter++;
                break;//then set string and move on
            }//no break if not valid treat as part of name and continue

        default:
            single_string+= *siter;
            break;
        }
    }
    //put it all on at the end
    NamesToWaitFor.push_back(single_string);

    // Check that there is something to wait for
    if (NamesToWaitFor.empty())
    {
        // Set Error Here !!
        return -1;
    }
    _labelIndex=PrepareGoto(this,TParam::stTmolab,script,false);

    return 0;
}

///////////////////////////////////////////////////////////////////////////////
// EXECUTE method for Wait For Test Record
// =======================================
///////////////////////////////////////////////////////////////////////////////
int TRcrdWtest::execute(TContext& ctxt)
{
    TMThread* TMctxt=static_cast<TMThread*>(&ctxt);
    //if not cast safely then this is NOT a test Manager
    //This should mean that there is a bug in the code as this code should only be
    //included in Test Manager and TMThreads have been created in preference to any other
    if(!TMctxt)
        throw mdbParseException("Invalid TestManager Execution Method Called");
    ctxt.set_phase_text(PHASE_WTFOR);

    int l_type      = intPrmVal("type", ctxt);
    int i_timeout   = intPrmVal("timeout", ctxt);
    float f_timeout=i_timeout;//If milliseconds then /1000;//Granularity of seconds
    bool timed_out=false;
    bool can_time_out=(i_timeout==0)?false:true;
    time_t stime;//start time
    time_t ctime;//current time
    time(&stime);
    vector<string>::iterator i=NamesToWaitFor.begin();
    while(i!=NamesToWaitFor.end() && !timed_out )
    {
        bool running=false;
        if(l_type==0)//wait for tests
        {
            running=TMctxt->isTestRunning(*i);
        }
        else//wait for threads
        {
            TtChrExpr tce(&ctxt);
            tce.Parse(*i);
            running=TMctxt->isThreadRunning(tce.Evaluate(ctxt));
        }
        if(!running)i++;
        time(&ctime);
        if(((ctime-stime)>f_timeout)&& can_time_out)timed_out=true;//timeout
        CyrSleep(5000);//wait 5 secs
    }

    ctxt.set_phase_text(TP_PHASE_NONE);
    if(timed_out)
        return ExecuteGoto(this,ctxt,TParam::stTmolab,&_labelIndex,&m_mutex);
    return 0;

}// end of TRcrdWtest::execute




///////////////////////////////////////////////////////////////////////////////
// PREPARE method for Thread Definition record
// =====================================
// @mfunc   This method should only be called from within the decalrations section
// of TestManager threads (TMThread::declaration), where access to TMThread::ThreadMap
// is locked, this method has a Thread Definition record read in and can therefore
// create a new ThreadObject and insert into the map for use by TMThread when it comes to
// execute a TestExecuter thread
///////////////////////////////////////////////////////////////////////////////
int ThrDefRcrd::prepare(TScript* script, int objIndex, bool verbose)
{
    const TParam* prmPtarget = param("target");
    if(prmPtarget == NULL) return -1;
    int l_target=prmPtarget->intVal();
    const TParam* prmPtname = param("name");
    if(prmPtname == NULL) return -1;
    string l_tname=prmPtname->strVal();
    ThreadDef thrdef;
    thrdef.ThreadName=l_tname.c_str();
    thrdef.Target=l_target;
    TMThread::ThreadMap.insert(TMThread::ThreadMap_t::value_type(l_tname,thrdef));
    return 0;
}// end of ThrDefRcrd::prepare

///////////////////////////////////////////////////////////////////////////////
// HELPER FUNCTIONS
// ================
///////////////////////////////////////////////////////////////////////////////
int server_specific_init(void)
{
    return 0;
}
// Perform any server specific one time cleanup
int server_specific_cleanup(void)
{
    return 0;
}
