/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

///////////////////////////////////////////////////////////////////////////////
// @doc     TestLevel
///////////////////////////////////////////////////////////////////////////////

#ifndef _TestLevel_I_H_
#define _TestLevel_I_H_

#include <TestLevel.h>
#include <TestExecuter.h>
#include <string>
#include "psl_macros.h"
extern "C"
{
#include "target_rtndef.h"
}
#ifdef WIN32
#pragma warning(disable:4786)
#pragma warning(disable:4503)
#endif
#include <map>
#ifndef CYR_NO_STD_NAMESPACE
using std::string;
using std::map;
using std::list;
using std::less;
#endif
#ifdef WIN32
#pragma warning(disable:4786)
#pragma warning(disable:4503)
#endif
typedef map < string, TestLevel_var > TestLevelMap;

class TestLevelImpl : public _sk_TestLevel
{
public:
    TestLevelImpl(string TestName, string Node, TestLevel_ptr parent,string ThreadID);
    virtual ~TestLevelImpl();
    virtual void Stop();
    virtual char * TestName();
    virtual char * ScriptName();
    virtual CORBA::ULong StartTime();
    virtual char * GetNode();
    virtual char * GetNoteText();
    //NOT a CORBA method internal use only////////////
    virtual void SetNoteText(string ntext);
    virtual bool isTestRunning(const string & testname);
    virtual bool isThreadRunning(const string & scriptname);
    virtual void AddTestCase(TestCaseStatus status);

    void SetToCloseDown() //Set flag to say closing down is imminent
    {
        m_closing_down = true;
    }
    virtual CORBA::Long ExecuteThreadScript(const char * scriptname, const char * nodename, const ThreadDef & thrddef,
                                            TestLevel_ptr & TL);
    //////////////////////////////////////////////////
    virtual CORBA::Long GetTestCaseResults(TestCaseStatus Status, CORBA::Boolean recursive);

    virtual CORBA::Long NoChildTestLevels(CORBA::Boolean recursive);
    virtual char * GetChildTestLevelIdents();
    virtual CORBA::ULong GetChildTestLevelList(TestLevelList & TLList);
    virtual TestLevel_ptr GetChildTestLevel(const char * ident);
    virtual void AddChildTestLevel(TestLevel_ptr & newTL);
    virtual void RemoveChildTestLevel(TestLevel_ptr & TL);

    virtual CORBA::Long NoChildThreads(CORBA::Boolean recursive);
    virtual CORBA::ULong GetTestExecuterList(TestExecuterList & TElist);
    virtual void TestExecuterFinished();

    virtual char * ident();
    void WaitForEmpty();
    virtual TestLevel_ptr GetParentTestLevel();

    virtual void ScriptName(string scriptname);

    // @access Protected Members
protected:
    virtual TestExecuter_ptr CreateNewTestExecuter(TARGETID tgt, const string & nodename);

private:
    virtual void WaitForNoSubTests();
    virtual void WaitForNoSubExecuters();
    CORBA::Long m_StartTime;
    CORBA::Long m_TestCasePass;
    CORBA::Long m_TestCaseFail;
    string m_TestName;
    string m_ScriptName;
    string m_NodeName;
    string m_ident;
    string m_NoteText;
    TestLevel_ptr m_ParentTestLevel;

    TestLevelMap m_TestLevelMap; //Sub Tests Currently running
    map < string, int > m_TestsStartedMap; //No of SubTests with Name that have been started
    map < string, int > m_TestsRunningMap;
    map < string, int > m_TestsFinishedMap;

    typedef list < TestExecuter_ptr > TestExecuterList_t;
    typedef map < TARGETID, TestExecuterList_t, less < TARGETID > > TargetExecuterMap_t;
    typedef map < string, TargetExecuterMap_t, less < string > > NodeExecuterMap_t;
    NodeExecuterMap_t m_NodeExecuterMap;
    long m_no_executers_to_finish;
    bool m_closing_down; //set when this object believes it is closing down
}
;
#endif
