/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

/* scl_newenc_routines:
 * ====================
 *
 * This module contains all the TPA action routines used by SCL to encode lines
 * to be written a 'tof' file.
 */

/*
 *  PSL Macros.
 */
#include <psl_macros.h>
#include <vtest_macros.h>

/*
 *  Standard ANSI header files.
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

/*
 *  Function Prototypes.
 */
#include <psl_rtndef.h>         /* PSL routines */
#include <trc_rtndef.h>         /* Tracing routines */
#include "scl_rtndef.h"         /* SCL routines */
#include "scl_def_tbl_rtndef.h"     /* SCL definition routines */
#include "scl_env_tbl_rtndef.h"     /* SCL environment routines */
#include "scl_cod_tbl_rtndef.h"     /* SCL code routines */
#include "scl_sub_tbl_rtndef.h"     /* SCL subexpression routines */

static void     ADD_UNDEF_SUBRTN (struct tpaarg *argblk_ptr);

static void     APPEND_LTXT  (struct scl_com *com_ptr,
                              char *fldnam_ptr,
                              char *datbuf_ptr);

static void     APPEND_RECHDR   (struct scl_com *com_ptr,
                                 char *rechdr_ptr);

static unsigned long    APPEND_TOFLIN   (struct scl_com *com_ptr,
        char *usrlin_ptr);

static void     ENC_EXPR_BYID   (struct scl_com *com_ptr,
                                 char *lin_ptr,
                                 unsigned long req_id);

static void     ENC_EXPR_BYPOS  (struct scl_com *com_ptr,
                                 char *lin_ptr);

static void     ENC_OPD_BYID    (struct scl_com *com_ptr,
                                 char *usrlin_ptr,
                                 unsigned long req_id);

static void     ENC_OPD_BYPOS   (struct scl_com *com_ptr,
                                 char *usrlin_ptr,
                                 char *llb_pos_ptr,
                                 char **ret_llb_pos_pptr);

static void     ENC_OPD_CHRIMM  (struct scl_com *com_ptr,
                                 char *usrlin_ptr,
                                 struct oph *oph_ptr,
                                 struct opd *opd_ptr);

static void     ENC_OPD_FUNC    (struct scl_com *com_ptr,
                                 char *lin_ptr,
                                 struct oph *oph_ptr,
                                 struct opd *opd_ptr,
                                 char *llb_opdpos_ptr);

static void     ENC_OPD_HTTP_CODE (
    struct scl_com *com_ptr,
    char *lin_ptr,
    struct oph *oph_ptr,
    struct opd *opd_ptr);

static void     ENC_OPD_HTTP_STATUS (
    struct scl_com *com_ptr,
    char *lin_ptr,
    struct oph *oph_ptr,
    struct opd *opd_ptr);

static void     ENC_OPD_INTIMM  (struct scl_com *com_ptr,
                                 char *usrlin_ptr,
                                 struct oph *oph_ptr,
                                 struct opd *opd_ptr);

static void     ENC_OPD_OPTS    (struct scl_com *com_ptr,
                                 char *lin_ptr,
                                 struct oph *oph_ptr,
                                 struct opd *opd_ptr);

static void     ENC_OPD_THIS    (struct scl_com *com_ptr,
                                 char *lin_ptr,
                                 struct oph *oph_ptr,
                                 struct opd *opd_ptr,
                                 char *llb_opdpos_ptr);

static void     ENC_OPD_TIM (struct scl_com *com_ptr,
                             char *usrlin_ptr,
                             struct oph *oph_ptr,
                             struct opd *opd_ptr);

static void     ENC_OPD_TST (struct scl_com *com_ptr,
                             char *lin_ptr,
                             struct oph *oph_ptr,
                             struct opd *opd_ptr);

static void     ENC_OPD_TSTTHR  (struct scl_com *com_ptr,
                                 char *lin_ptr,
                                 struct oph *oph_ptr,
                                 struct opd *opd_ptr);

static void     ENC_OPD_VAR (struct scl_com *com_ptr,
                             char *usrlin_ptr,
                             struct oph *oph_ptr,
                             struct opd *opd_ptr,
                             char *llb_opdpos_ptr);

static unsigned long    GET_OPD_COUNT   (struct scl_tpa *tpa_ptr);

static void     RESOLVE_LABEL   (struct tpaarg *argblk_ptr,
                                 char *label_ptr,
                                 unsigned long label_len);
/*
 *  Structure definitions.
 */
#include <shrmsg.h>         /* PSL messages */
#include <psldiag_prmdef.h>     /* Diagnostics paramters */
#include <psl_tpaargdef.h>      /* TPARSE argument block */
#include "scl_limits.h"         /* SCL limits */
#include "scl_tbldef.h"         /* SCL table definitions */
#include <vtest_prmdef.h>       /* VTEST parameters */
#include "scl_cmd.h"            /* Command Line Data Structure */
#include "scl_comdef.h"         /* SCL common data */
#include "scl_tpacom.h"         /* SCL Tparse data */
#include "scl_tof.h"            /* SCL TOF data */
#include <psl_trc_prmdef.h>     /* PSL tracing parameters */
#include <psl_vm_prmdef.h>      /* PSL linked list parameters */
#include <pslerr_prmdef.h>      /* PSL error routine parameters */
#include <tofprmdef.h>          /* TOF parameters */

/******************************************************************************
 * @func    This routine adds an 'undefined subroutine' entry to the
 *      Subroutines Symbol Table reporting an error, if the Symbol Table is
 *      full. It should be called when the first reference to an undefined
 *      subroutine is encountered.
 *
 * @parm    Pointer to Tparse argument block.
 */
static void ADD_UNDEF_SUBRTN (struct tpaarg *argblk_ptr)
{
    /*
     *  Local data.
     */
    struct   scl_com *com_ptr;      /* SCL common data */
    struct   scl_tpa *tpa_ptr;      /* SCL Tparse data */
    struct   subtbl  *subtbl_ptr;       /* Pointer to subroutines symbol table */

    /*
     *  Copy pointers for ease of reference.
     */
    com_ptr = (struct scl_com *) argblk_ptr->user_ptrp;
    tpa_ptr = com_ptr->tpa_ptrps;

    /*
     *  If the Subroutines Symbol Table is full report an error and return.
     */
    if (tpa_ptr->subrtn_cntl >= MAX_SUBROUTINES)
    {
        REPORT_ERROR (argblk_ptr, SCL_MAXSUB);
        goto end;
    }

    /*
     *  Get pointer to next free entry in Subroutines Symbol Table.
     */
    subtbl_ptr = tpa_ptr->subroutine_begps + tpa_ptr->subrtn_cntl;

    /*
     *  Add 'undefined subroutine' entry to the Subroutines Symbol Table.
     */
    (void) strcpy (subtbl_ptr->sub_namec20, tpa_ptr->subroutine_namec20);
    /* Set subroutine name */
    subtbl_ptr->sub_defn = PFALSE;  /* Flag subroutine not yet defined */
    subtbl_ptr->sub_param_cntl = tpa_ptr->subprm_cntl;
    /* Set number of paramaters */
    tpa_ptr->subrtn_cntl += 1;      /* Increment subroutine count */

end:
    ;
    return;
}

/******************************************************************************
 * @func    This is a generic routine for appending the lines for an ltxt field
 *      to the list of 'TOF' lines. It is supplied with the ltxt field name
 *      and a buffer containing the ltxt data. An ltxt field has the
 *      following format:
 *
 *      <PRE>
 *      <field_name><ltxt_delimiter>
 *      <field_data><ltxt_delimiter>
 *      </PRE>
 *
 *      The field data buffer may be large so we don't want to copy it.
 *      Therefore, we simply append the ltxt delimiter to the supplied
 *      buffer (which must be big enough) to write the field data line and
 *      clear the buffer before returning.
 *
 * @parm    Pointer to SCL common data.
 * @parm    Pointer to field name string.
 * @parm    Pointer to ltxt data buffer. Clear on return.
 */
static void APPEND_LTXT (struct scl_com *com_ptr,
                         char *fldnam_ptr,
                         char *datbuf_ptr)
{
    /*
     *  Local data.
     */
    char   linbuf [TOFBUFLEN]; /* Local TOF line buffer */

    /*
     *  Add field name line to list:
     *
     *  <field_name> <ltxt_delimiter>
     */
    (void) sprintf (linbuf, "%s %s",fldnam_ptr ,LTXT_DELIMITER);
    APPEND_TOFLIN (com_ptr, linbuf);

    /*
     *  Add field data line to list:
     *
     *  <field_data><ltxt_delimiter>
     */
    (void) strcat (datbuf_ptr, LTXT_DELIMITER);
    APPEND_TOFLIN (com_ptr, datbuf_ptr);

    /*
     *  Clear the supplied data buffer.
     */
    *datbuf_ptr = '\0';

    return;
}

/******************************************************************************
 * @func    This routine appends a record header to the linked list of 'TOF'
 *      lines. Its purpose is to isolate in a single place the code that
 *      appends the record separator - a blank line - to the linked list of
 *      'TOF' lines.
 *
 * @rdesc   Unsigned long PSL routine return status.
 * @parm    Pointer to scl common data structure.
 * @parm    Pointer to buffer containing null-terminated object header record.
 */
static void APPEND_RECHDR (struct scl_com *com_ptr,
                           char *rechdr_ptr)
{
    char   linbuf = '\0';          /* Local TOF line buffer */

    APPEND_TOFLIN (com_ptr, &linbuf);       /* Blank line for record separator */
    APPEND_TOFLIN (com_ptr, rechdr_ptr);    /* Record header line */
    return;
}

/******************************************************************************
 * @func    This routine appends a line to the linked list of 'TOF' lines
 *      maintained while parsing the code section of a script. It also
 *      resets (i.e. clears) the default 'TOF' line buffer.
 *
 * @rdesc   Unsigned long PSL routine return status.
 * @parm    Pointer to scl common data structure.
 * @parm    Pointer to buffer containing null-terminated script object ('tof')
 *      line string. If NULL string is picked up from the default 'TOF'
 *      line buffer.
 */
static unsigned long APPEND_TOFLIN (struct scl_com *com_ptr,
                                    char *usrlin_ptr)
{
    /*
     *  Local data.
     */
    struct scl_tofl  *tofl_ptr;     /* Pointer to tof line list */
    struct scl_tpa   *tpa_ptr;      /* SCL tparse data */
    unsigned long    iret;          /* PSL routine return status */
    char    *inplin_ptr;       /* Pointer to input 'TOF' line buffer */
    char    *outlin_ptr;       /* Pointer to output 'TOF' line buffer */
    long    linlen;        /* Line length */

    /*
     *  Copy pointers for ease of reference.
     */
    tpa_ptr = com_ptr->tpa_ptrps;

    /*
     *  Use the default line buffer, unless a buffer is supplied in the call.
     */
    if (usrlin_ptr == NULL)
    {
        inplin_ptr = com_ptr->toflinc1024;
    }
    else
    {
        inplin_ptr = usrlin_ptr;
    }

    /*
     *  Allocate space for 'TOF' line buffer and copy line to it.
     */
    linlen = strlen (inplin_ptr);
    outlin_ptr = (char *) MALLOC (linlen + 1);
    (void) strcpy (outlin_ptr, inplin_ptr);

#ifdef PSL_TRACING
    if (PSL_TEST_TRACE (PTRC_USR_USER5))
    {
        PSL_OUTPUT_TRACE ("MALLOC: outlin_ptr - '%d/%p' - APPEND_TOFLIN\n",
                          linlen, outlin_ptr);
    }
#endif  /* PSL_TRACING */

    /*
     *  Set up 'TOF' line structure with pointer to 'TOF' line buffer and
     *  append to list.
     */
    tofl_ptr = (struct scl_tofl *) MALLOC (sizeof (struct scl_tofl));
    tofl_ptr->tofl_linptrpc = outlin_ptr;

    iret = PSL_VM_INSERT (tpa_ptr->tofl_llps, (char *) &tofl_ptr,
                          NULL, NULL, PSL_VM_POS_END);
    if (!psl_test_status(iret))
    {
        PSL_SIGDIAG (tpa_ptr->diag_ptrps, 2, PSL_SYSERVERR,
                     "PSL_VM_INSERT (in APPEND_TOFLIN)", iret);
    }

    /*
     *  Reset the default line buffer, if we have just processed it.
     */
    if (usrlin_ptr == NULL) com_ptr->toflinc1024 [0] = '\0';

    return iret;            /* Return with status */
}

/******************************************************************************
 * @func    This routine creates an immediate character value in encoded format
 *      from a raw character value string. This currently only requires the
 *      routine to delimit the supplied string with an appropriate quote
 *      character. The routine delimits the string with the double quote
 *      character, unless the string contains a double quote character,
 *      in which case it uses a single quote.
 *
 * @parm    Pointer to SCL common data.
 * @parm    Pointer to buffer to hold encoded value
 * @parm    Length of buffer to hold encoded value
 * @parm    Pointer to raw character value to be encoded
 * @parm    Raw character value length
 */
void CREATE_ENCCHRIMM (struct scl_com *com_ptr,
                       char *encbuf_ptr,
                       size_t encbuflen,
                       const char *rawval_ptr,
                       PSL_U_LONGWORD_T rawvallen)
{
    /*
     *  Local data.
     */
    struct scl_tpa   *tpa_ptr;      /* SCL tparse data */
    char   delim = '\"';       /* String delimiter character */
    char   *tmp_ptr;       /* Temporary pointer */

    /*
     *  Copy pointers for ease of reference.
     */
    tpa_ptr = com_ptr->tpa_ptrps;

    /*
     *  Check that there is enough space in the supplied buffer. If not
     *  report an error and clear the buffer.
     *
     *  "+3" in the following line is to account for 2 delimiter characters
     *  and the NULL terminator character.
     */
    if ((rawvallen + 3) > encbuflen)
    {
        PSL_PUTDIAG ((*com_ptr).diag_ptrps,
                     1,
                     SCL_MAXSTRLEN);
        PSL_OUTPUT_DIAG ((*com_ptr).diag_ptrps, 0, DIOUFL_LSTERR);
        /* Output diagnostic */
        (*tpa_ptr).error_countl += 1;   /* Increment error count */
        *encbuf_ptr = '\0';
    }

    /*
     *  Add the character to the encoded value buffer enclosed in double quotes,
     *  unless operand contains a double quote, in which case use single quotes.
     */
    if (strchr (rawval_ptr, delim) != NULL)
    {
        delim = '\'';
    }

    tmp_ptr = encbuf_ptr;               /* Point to start of output buffer */
    *tmp_ptr++ = delim;                 /* Copy first quote */
    (void) memcpy (tmp_ptr, rawval_ptr, rawvallen); /* Copy string */
    tmp_ptr += rawvallen;               /* Point to character after string */
    *tmp_ptr++ = delim;                 /* Copy terminating quote */
    *tmp_ptr = '\0';                    /* Null terminate */

    return;
}

/******************************************************************************
 * @func    This Tparse action routine is called after parsing an ABORT TEST
 *      command. Its purpose is to append an Abort record to the list of
 *      'TOF' lines. This record has the following format:
 *
 *      <PRE>
 *      Abort
 *      target TOF_ABTTGT_TEST
 *      </PRE>
 *
 * @rdesc   Tparse status (Always returns PSL_NORMAL).
 * @parm    Pointer to Tparse argument block.
 */
unsigned long ENC_ABORT_TEST (struct tpaarg *argblk_ptr)
{
    /*
     *    Local data
     */
    struct scl_com  *com_ptr;       /* SCL comon data */
    struct scl_tpa  *tpa_ptr;       /* SCL Tparse data */
    char   *lin_ptr;       /* Pointer to line buffer */

    /*
     *  Copy pointers for ease of reference.
     */
    com_ptr = (struct scl_com *) argblk_ptr->user_ptrp;
    tpa_ptr = com_ptr->tpa_ptrps;
    lin_ptr = com_ptr->toflinc1024; /* 'tof' line buffer pointer */

    /*
     *  Do not encode anything if we have had an error in the command.
     */
    if (tpa_ptr->error_in_commandn) goto end;

    /*
     *  Output tracing information if required.
     */
#ifdef PSL_TRACING
    if (PSL_TEST_TRACE (PTRC_USR_USER1))
    {
        PSL_OUTPUT_TRACE ("\nEncoding an ABORT TEST Command\n");
        PSL_OUTPUT_TRACE ("Operand Count = %ld\n" ,GET_OPD_COUNT (tpa_ptr));
    }
#endif

    /*
     *  Write the following records to the 'TOF' line list:
     *
     *  Abort
     *  target TOF_ABTTGT_TEST
     */
    APPEND_RECHDR (com_ptr, "Abort");

    (void) sprintf (lin_ptr, "target %d", TOF_ABTTGT_TEST);
    APPEND_TOFLIN (com_ptr, NULL);
end:
    ;
    return PSL_NORMAL;
}

/******************************************************************************
 * @func    This Tparse action routine is called after parsing an ACQUIRE MUTEX
 *      command. Its purpose is to append an Acmutex record to the list of
 *      'TOF' lines. The Acmutex record has the following format:
 *
 *      <PRE>
 *      Acmutex
 *      scope <scope>
 *      name <mutex_name>
 *      {timeout <timeout_period>}
 *      {tmolab <timeout_label>}
 *      {errlab <error_label>}
 *      </PRE>
 *
 * @rdesc   Tparse status (Always returns PSL_NORMAL).
 * @parm    Pointer to Tparse argument block.
 */
unsigned long ENC_ACQUIRE_MUTEX (struct tpaarg *argblk_ptr)
{
    /*
     *  Local data.
     */
    struct scl_com  *com_ptr;       /* SCL common data */
    struct scl_tpa  *tpa_ptr;       /* SCL Tparse data */
    unsigned long   mtxscp;         /* Mutext scope flag */
    char   *lin_ptr;       /* Pointer to line buffer */
    char   mtxnam [TOFBUFLEN]; /* Mutex name */

    /*
     *  Copy pointers for ease of reference.
     */
    com_ptr = (struct scl_com *) argblk_ptr->user_ptrp;
    tpa_ptr = com_ptr->tpa_ptrps;
    lin_ptr = com_ptr->toflinc1024; /* 'tof' line buffer pointer */

    /*
     *  Do not encode anything if we have had an error in the command.
     */
    if (tpa_ptr->error_in_commandn) goto end;

    /*
     *  Output tracing information if required.
     */
#ifdef PSL_TRACING
    if (PSL_TEST_TRACE (PTRC_USR_USER1))
    {
        PSL_OUTPUT_TRACE ("Encoding an ACQUIRE MUTEX Command\n");
        PSL_OUTPUT_TRACE ("Operand Count = %ld\n" ,GET_OPD_COUNT (tpa_ptr));
    }
#endif

    /*
     *  Get the acquire mutex scope flag.
     */
    if ((tpa_ptr->pipe_flagsl & PIPFLG_TESTWIDE) != 0)
    {
        mtxscp = TOF_MTXSCP_TESTWIDE;
    }
    else
    {
        mtxscp = TOF_MTXSCP_LOCAL;
    }

    /*
     *  Get mutex name.
     */
    mtxnam [0] = '\0';
    ENC_OPD_BYPOS (com_ptr, mtxnam, NULL, NULL);

    /*
     *  Write the following records to the 'TOF' line list:
     *
     *  Acmutex
     *  scope <scope>
     *  name <mutex_name>
     *  {timeout <timeout_period>}
     *  {tmolab <timeout_label>}
     *  {errlab <error_label>}
     */
    APPEND_RECHDR (com_ptr, "Acmutex");

    (void) sprintf (lin_ptr, "scope %d", mtxscp);
    APPEND_TOFLIN (com_ptr, NULL);

    (void) sprintf (lin_ptr, "name %s", mtxnam);
    APPEND_TOFLIN (com_ptr, NULL);

    if ((tpa_ptr->cmnd_optl & CMND_OPT_WAIT_PRD) != 0)
    {
        (void) strcpy (lin_ptr, "timeout ");
        ENC_OPD_BYPOS (com_ptr, lin_ptr, NULL, NULL);
        APPEND_TOFLIN (com_ptr, NULL);
    }

    if ((tpa_ptr->cmnd_optl & CMND_OPT_TIMEOUT) != 0)
    {
        (void) sprintf (lin_ptr, "tmolab %s", tpa_ptr->ontimeout_labelc20);
        APPEND_TOFLIN (com_ptr, NULL);
    }

    if (strlen (tpa_ptr->onerror_labelc16) != 0)
    {
        (void) strcpy (lin_ptr, "errlab ");
        RESOLVE_LABEL (argblk_ptr,
                       tpa_ptr->onerror_labelc16,
                       tpa_ptr->onerror_lenl);
        APPEND_TOFLIN (com_ptr, NULL);
    }

end:
    ;
    return PSL_NORMAL;          /* Return success */
}

/******************************************************************************
 * @func    This Tparse action routine is called after parsing a CALL SCRIPT
 *      command. Its purpose is to append a Cscr record to the list of
 *      'TOF' lines. The Cscr record has the following format:
 *
 *      <PRE>
 *      Cscr
 *      name <script_name>
 *      {parameters <parameters>}
 *      {errlab <error_label>}
 *      {retvar <variable>}
 *      </PRE>
 *
 * @rdesc   Tparse status (Always returns PSL_NORMAL).
 * @parm    Pointer to Tparse argument block.
 */
unsigned long ENC_CALL_SCRIPT (struct tpaarg *argblk_ptr)
{
    /*
     *  Local data.
     */
    struct   scl_com *com_ptr;      /* SCL common data */
    struct   scl_tpa *tpa_ptr;      /* SCL TPARSE data */
    unsigned long    isub;          /* Loop control */
    char    *lin_ptr;      /* Pointer to line buffer */
    char    namlin [TOFBUFLEN];    /* Name line */
    char    prmlin [TOFBUFLEN];    /* Parameters line */
    char    stslin [TOFBUFLEN];    /* Status line */

    /*
     *  Copy pointers for ease of reference.
     */
    com_ptr = (struct scl_com *) argblk_ptr->user_ptrp;
    tpa_ptr = com_ptr->tpa_ptrps;
    lin_ptr = com_ptr->toflinc1024; /* 'tof' line buffer pointer */

    /*
     *  Do not encode anything if we have had an error in the command.
     */
    if (tpa_ptr->error_in_commandn) goto end;

    /*
     *  Output tracing information if required.
     */
#ifdef PSL_TRACING
    if (PSL_TEST_TRACE (PTRC_USR_USER1))
    {
        PSL_OUTPUT_TRACE ("\nEncoding a CALL SCRIPT Command\n");
        PSL_OUTPUT_TRACE ("Operand Count = %ld\n" ,GET_OPD_COUNT (tpa_ptr));
    }
#endif

    /*
     *  Get script name operand and create name sub-record.
     */
    (void) strcpy (namlin, "name ");
    ENC_OPD_BYPOS (com_ptr, namlin, NULL, NULL);

    /*
     *  Construct the 'parameters' sub-record by encoding the subroutine
     *  parameters (stored in the parsed operands linked list). The sub-record has
     *  the following format:
     *
     *  parameters p1 {{,pn}...}
     */
    (void) strcpy (prmlin, "parameters ");

    for (isub=0; isub < tpa_ptr->call_prm_cntl; isub++)
    {
        ENC_OPD_BYPOS (com_ptr, prmlin, NULL, NULL);
        if (isub < (tpa_ptr->call_prm_cntl - 1)) (void) strcat (prmlin, ",");
    }

    /*
     *  If a RETURNING clause was specified, get the status variable name and create
     *  a retvar sub-record.
     */
    if ((tpa_ptr->cmnd_optl & CMND_OPT_RETURNING) != 0)
    {
        (void) strcpy (stslin, "retvar ");
        ENC_OPD_BYPOS (com_ptr, stslin, NULL, NULL);
    }

    /*
     *  Write the following records to the 'TOF' line list:
     *
     *  Cscr
     *  name <script_name>
     *  {parameters <parameters>}
     *  {errlab <error_label>}
     *  {retvar <variable>}
     */
    APPEND_RECHDR (com_ptr, "Cscr");

    APPEND_TOFLIN (com_ptr, namlin);

    if (tpa_ptr->call_prm_cntl > 0)
    {
        APPEND_TOFLIN (com_ptr, prmlin);
    }

    if (strlen (tpa_ptr->onerror_labelc16) != 0)
    {
        (void) strcpy (lin_ptr, "errlab ");
        RESOLVE_LABEL (argblk_ptr,
                       tpa_ptr->onerror_labelc16,
                       tpa_ptr->onerror_lenl);
        APPEND_TOFLIN (com_ptr, NULL);
    }

    if ((tpa_ptr->cmnd_optl & CMND_OPT_RETURNING) != 0)
    {
        APPEND_TOFLIN (com_ptr, stslin);
    }

    /*
     *  Clear script called indicator.
     */
    tpa_ptr->script_flagl = 0;

end:
    ;
    return PSL_NORMAL;          /* Return success */
}

/******************************************************************************
 * @func    This Tparse action routine is called after parsing a CALL
 *      SUBROUTINE command. Its purpose is to append a Csub record to the
 *      list of 'TOF' lines. The Csub record has the following format:
 *
 *      <PRE>
 *      Csub
 *      name <subroutine_name>
 *      {parameters <parameters>}
 *      </PRE>
 *
 *      If the subroutine has not been defined or referenced an entry is
 *      added to the Subroutines Symbol Table.
 *
 * @rdesc   Tparse status (Always returns PSL_NORMAL).
 * @parm    Pointer to Tparse argument block.
 */
unsigned long ENC_CALL_SUBROUTINE (struct tpaarg *argblk_ptr)
{
    /*
     *  Local data.
     */
    struct   scl_com *com_ptr;      /* SCL common data */
    struct   scl_tpa *tpa_ptr;      /* SCL TPARSE data */
    struct   subtbl  *subtbl_ptr;       /* Pointer to subroutine table */
    unsigned long    isub;          /* Loop control */
    long    comp_result;       /* Result of comparison */
    char    *lin_ptr;      /* Pointer to line buffer */
    char    prmlin [TOFBUFLEN];    /* Parameters line */

    /*
     *  Copy pointers for ease of reference.
     */
    com_ptr = (struct scl_com *) argblk_ptr->user_ptrp;
    tpa_ptr = com_ptr->tpa_ptrps;
    lin_ptr = com_ptr->toflinc1024; /* 'tof' line buffer pointer */

    /*
     *  Do not encode anything if we have had an error in the command.
     */
    if (tpa_ptr->error_in_commandn) goto end;

    /*
     *  Output tracing information if required.
     */
#ifdef PSL_TRACING
    if (PSL_TEST_TRACE (PTRC_USR_USER1))
    {
        PSL_OUTPUT_TRACE ("\nEncoding a CALL SUBROUTINE Command\n");
        PSL_OUTPUT_TRACE ("Operand Count = %ld\n" ,GET_OPD_COUNT (tpa_ptr));
    }
#endif

    /*
     *  Check the Subroutine Symbol Table to see if this subroutine has been
     *  encountered before.
     */
    subtbl_ptr = tpa_ptr->subroutine_begps;
    /* Pointer to start of table */
    for (isub=0; isub < tpa_ptr->subrtn_cntl; isub++, subtbl_ptr++)
    {
        comp_result = strcmp (tpa_ptr->subroutine_namec20,
                              subtbl_ptr->sub_namec20);
        /* Compare the 2 subroutines    */
        if (comp_result == 0) goto found_sub;
        /* ...and branch if names match */
    }

    /*
     *  Undefined Subroutine First Reference
     *  ------------------------------------
     *
     *  The subroutine has not been defined or referenced. Add an entry to the
     *  Subroutines Symbol Table and branch to encode the call.
     */
    ADD_UNDEF_SUBRTN (argblk_ptr);
    goto enc_call_subrtn;

    /*
     *  Subroutine Defined or Referenced
     *  --------------------------------
     *
     *  We come here having found an entry for the current subroutine in the
     *  Subroutines Symbol Table; check the parameter count is the same in this
     *  call as in the Symbol Table and report an error if it is not.
     */
found_sub:
    if (tpa_ptr->subprm_cntl != subtbl_ptr->sub_param_cntl)
    {
        REPORT_ERROR (argblk_ptr, SCL_INVSUBPRM);
    }

    /*
     *  Encode call subroutine.
     *
     *  First, construct the 'parameters' sub-record by encoding the subroutine
     *  parameters (stored in the parsed operands linked list). The sub-record has
     *  the following format:
     *
     *  parameters p1 {{,pn}...}
     */
enc_call_subrtn:
    ;
    (void) strcpy (prmlin, "parameters ");

    for (isub=0; isub != tpa_ptr->subprm_cntl; isub++)
    {
        ENC_OPD_BYPOS (com_ptr, prmlin, NULL, NULL);
        if (isub < (tpa_ptr->subprm_cntl - 1)) (void) strcat (prmlin, ",");

#ifdef PSL_TRACING
        if (PSL_TEST_TRACE (PTRC_USR_USER1))
        {
            PSL_OUTPUT_TRACE ("\nEncoded SUBROUTINE Parameter %ld\n", isub);
            PSL_OUTPUT_TRACE ("Operand Count = %ld\n" ,GET_OPD_COUNT (tpa_ptr));
        }
#endif

    }   /* End for ... */

    /*
     *  Write the following records to the 'TOF' line list:
     *
     *  Csub
     *  name <subroutine_name>
     *  {parameters <parameters>}
     */
    APPEND_RECHDR (com_ptr, "Csub");

    (void) sprintf (lin_ptr, "name %s", tpa_ptr->subroutine_namec20);
    APPEND_TOFLIN (com_ptr, NULL);

    if (tpa_ptr->subprm_cntl > 0)
    {
        APPEND_TOFLIN (com_ptr, prmlin);
    }

end:
    ;
    return PSL_NORMAL;          /* Return success */
}

/******************************************************************************
 * @func    This Tparse action routine is called after parsing a CANCEL ON
 *      ERROR command. Its purpose is to append a Disable record to the list
 *      of 'TOF' lines. This record has the following format:
 *
 *      <PRE>
 *      Disable
 *      target TOF_ENDTGT_ONERROR
 *      </PRE>
 *
 * @rdesc   Tparse status (Always returns PSL_NORMAL).
 * @parm    Pointer to Tparse argument block.
 */
unsigned long ENC_CANCEL_ON (struct tpaarg *argblk_ptr)
{
    /*
     *    Local data
     */
    struct scl_com  *com_ptr;       /* SCL comon data */
    struct scl_tpa  *tpa_ptr;       /* SCL Tparse data */
    char   *lin_ptr;       /* Pointer to line buffer */

    /*
     *  Copy pointers for ease of reference.
     */
    com_ptr = (struct scl_com *) argblk_ptr->user_ptrp;
    tpa_ptr = com_ptr->tpa_ptrps;
    lin_ptr = com_ptr->toflinc1024; /* 'tof' line buffer pointer */

    /*
     *  Do not encode anything if we have had an error in the command.
     */
    if (tpa_ptr->error_in_commandn) goto end;

    /*
     *  Output tracing information if required.
     */
#ifdef PSL_TRACING
    if (PSL_TEST_TRACE (PTRC_USR_USER1))
    {
        PSL_OUTPUT_TRACE ("\nEncoding a CANCEL ON ERROR Command\n");
        PSL_OUTPUT_TRACE ("Operand Count = %ld\n" ,GET_OPD_COUNT (tpa_ptr));
    }
#endif

    /*
     *  Write the following records to the 'TOF' line list:
     *
     *  Disable
     *  target TOF_ENDTGT_ONERROR
     */
    APPEND_RECHDR (com_ptr, "Disable");

    (void) sprintf (lin_ptr, "target %d", TOF_ENDTGT_ONERROR);
    APPEND_TOFLIN (com_ptr, NULL);
end:
    ;
    return PSL_NORMAL;
}

/******************************************************************************
 * @func    This Tparse action routine is called after parsing a CANCEL WHEN
 *      command. Its purpose is to append a Disable record to the list
 *      of 'TOF' lines. This record has the following format:
 *
 *      <PRE>
 *      Disable
 *      target TOF_ENDTGT_WHEN
 *      </PRE>
 *
 * @rdesc   Tparse status (Always returns PSL_NORMAL).
 * @parm    Pointer to Tparse argument block.
 */
unsigned long ENC_CANCEL_WHEN (struct tpaarg *argblk_ptr)
{
    /*
     *    Local data
     */
    struct scl_com  *com_ptr;       /* SCL comon data */
    struct scl_tpa  *tpa_ptr;       /* SCL Tparse data */
    char   *lin_ptr;       /* Pointer to line buffer */

    /*
     *  Copy pointers for ease of reference.
     */
    com_ptr = (struct scl_com *) argblk_ptr->user_ptrp;
    tpa_ptr = com_ptr->tpa_ptrps;
    lin_ptr = com_ptr->toflinc1024; /* 'tof' line buffer pointer */

    /*
     *  Do not encode anything if we have had an error in the command.
     */
    if (tpa_ptr->error_in_commandn) goto end;

    /*
     *  Output tracing information if required.
     */
#ifdef PSL_TRACING
    if (PSL_TEST_TRACE (PTRC_USR_USER1))
    {
        PSL_OUTPUT_TRACE ("\nEncoding a CANCEL WHEN Command\n");
        PSL_OUTPUT_TRACE ("Operand Count = %ld\n" ,GET_OPD_COUNT (tpa_ptr));
    }
#endif

    /*
     *  Write the following records to the 'TOF' line list:
     *
     *  Disable
     *  target TOF_ENDTGT_WHEN
     */
    APPEND_RECHDR (com_ptr, "Disable");

    (void) sprintf (lin_ptr, "target %d", TOF_ENDTGT_WHEN);
    APPEND_TOFLIN (com_ptr, NULL);
end:
    ;
    return PSL_NORMAL;
}

/******************************************************************************
 * @func    This Tparse action routine is called after parsing a CLEAR
 *      command. Its purpose is to append a Clsemaphore record to
 *      the list of 'TOF' lines. The Clsemaphore record has the following
 *      format:
 *
 *      <PRE>
 *      Clsemaphore
 *      scope <scope>
 *      name <semaphore_name>
 *      {errlab <error_label>}
 *      </PRE>
 *
 * @rdesc   Tparse status (Always returns PSL_NORMAL).
 * @parm    Pointer to Tparse argument block.
 */
unsigned long ENC_CLEAR_SEMAPH (struct tpaarg *argblk_ptr)
{
    /*
     *  Local data.
     */
    struct scl_com  *com_ptr;       /* SCL common data */
    struct scl_tpa  *tpa_ptr;       /* SCL Tparse data */
    unsigned long   semscp;         /* Clear semaphore scope flag */
    char   *lin_ptr;       /* Pointer to line buffer */
    char   semnam [TOFBUFLEN]; /* Semaphore name */

    /*
     *  Copy pointers for ease of reference.
     */
    com_ptr = (struct scl_com *) argblk_ptr->user_ptrp;
    tpa_ptr = com_ptr->tpa_ptrps;
    lin_ptr = com_ptr->toflinc1024; /* 'tof' line buffer pointer */

    /*
     *  Do not encode anything if we have had an error in the command.
     */
    if (tpa_ptr->error_in_commandn) goto end;

    /*
     *  Output tracing information if required.
     */
#ifdef PSL_TRACING
    if (PSL_TEST_TRACE (PTRC_USR_USER1))
    {
        PSL_OUTPUT_TRACE ("Encoding a CLEAR SEMAPHORE Command\n");
        PSL_OUTPUT_TRACE ("Operand Count = %ld\n" ,GET_OPD_COUNT (tpa_ptr));
    }
#endif

    /*
     *  Get the clear semaphore scope flag.
     */
    if ((tpa_ptr->pipe_flagsl & PIPFLG_TESTWIDE) != 0)
    {
        semscp = TOF_SEMSCP_TESTWIDE;
    }
    else
    {
        semscp = TOF_SEMSCP_LOCAL;
    }

    /*
     *  Get semaphore name.
     */
    semnam [0] = '\0';
    ENC_OPD_BYPOS (com_ptr, semnam, NULL, NULL);

    /*
     *  Write the following records to the 'TOF' line list:
     *
     *  Clsemaphore
     *  scope <scope>
     *  name <semaphore_name>
     *  {errlab <error_label>}
     */
    APPEND_RECHDR (com_ptr, "Clsemaphore");

    (void) sprintf (lin_ptr, "scope %d", semscp);
    APPEND_TOFLIN (com_ptr, NULL);

    (void) sprintf (lin_ptr, "name %s", semnam);
    APPEND_TOFLIN (com_ptr, NULL);

    if (strlen (tpa_ptr->onerror_labelc16) != 0)
    {
        (void) strcpy (lin_ptr, "errlab ");
        RESOLVE_LABEL (argblk_ptr,
                       tpa_ptr->onerror_labelc16,
                       tpa_ptr->onerror_lenl);
        APPEND_TOFLIN (com_ptr, NULL);
    }

end:
    ;
    return PSL_NORMAL;          /* Return success */
}

/******************************************************************************
 * @func    This Tparse action routine is called after parsing a CLOSE (file)
 *      command. Its purpose is to append an Fcntl record to the list of
 *      'TOF' lines. The Fcntl record has the following format:
 *
 *      <PRE>
 *      Fcntl
 *      operation TOF_FCNOP_CLOSE
 *      filid <file_id>
 *      {errlab <error_label>}
 *      </PRE>
 *
 *      Note that if this is a passed file, the index is incremented by
 *      the constant PASSED_FILE_BASE, to show that it is a passed file.
 *
 * @rdesc   Tparse status (Always returns PSL_NORMAL).
 * @parm    Pointer to Tparse argument block.
 */
unsigned long ENC_CLOSE_FILE (struct tpaarg *argblk_ptr)
{
    /*
     *  Local data.
     */
    struct scl_com  *com_ptr;       /* SCL common data */
    struct scl_tpa  *tpa_ptr;       /* SCL Tparse data */
    struct fattbl   *fat_ptr;       /* File Access Table pointer */
    char   *lin_ptr;       /* Pointer to line buffer */
    char   filidlin [TOFBUFLEN];   /* File Id line */

    /*
     *  Copy pointers for ease of reference.
     */
    com_ptr = (struct scl_com *) argblk_ptr->user_ptrp;
    tpa_ptr = com_ptr->tpa_ptrps;
    lin_ptr = com_ptr->toflinc1024; /* 'tof' line buffer pointer */
    fat_ptr = tpa_ptr->fat_begps + tpa_ptr->open_fididxl - 1;
    /* Point to current entry in table */
    /*
     *  Do not encode anything if we have had an error in the command.
     */
    if (tpa_ptr->error_in_commandn) goto end;

    /*
     *  Output tracing information if required.
     */
#ifdef PSL_TRACING
    if (PSL_TEST_TRACE (PTRC_USR_USER1))
    {
        PSL_OUTPUT_TRACE ("Encoding a CLOSE (file) Command\n");
        PSL_OUTPUT_TRACE ("Operand Count = %ld\n" ,GET_OPD_COUNT (tpa_ptr));
    }
#endif

    /*
     *    Check to see if this file has been DEFINED (by the FILE statement).
     *    If it has then we set the referenced flag to ACCESSED_FILE
     *
     *    ...otherwise...
     *
     *    If the file is a passed file, we need to show this by incrementing its
     *    index into the passed file index range.
     */
    if (fat_ptr->fat_refflgl == DEFINED_FILE)
    {
        fat_ptr->fat_refflgl = ACCESSED_FILE;
    }
    else if (fat_ptr->fat_refflgl == PASSED_FILE)
    {
        tpa_ptr->open_fididxl += PASSED_FILE_BASE;
    }

    /*
     *  Get file ID and create 'filid' sub-record.
     */
    (void) sprintf (filidlin, "filid %s", fat_ptr->fat_namec20);

    /*
     *  Write the following records to the 'TOF' line list:
     *
     *  Fcntl
     *  operation TOF_FCNOP_CLOSE
     *  filid <file_id>
     *  {errlab <error_label>}
     */
    APPEND_RECHDR (com_ptr, "Fcntl");

    (void) sprintf (lin_ptr, "operation %d", TOF_FCNOP_CLOSE);
    APPEND_TOFLIN (com_ptr, NULL);

    APPEND_TOFLIN (com_ptr, filidlin);

    if (strlen (tpa_ptr->onerror_labelc16) != 0)
    {
        (void) strcpy (lin_ptr, "errlab ");
        RESOLVE_LABEL (argblk_ptr,
                       tpa_ptr->onerror_labelc16,
                       tpa_ptr->onerror_lenl);
        APPEND_TOFLIN (com_ptr, NULL);
    }

end:
    ;
    return PSL_NORMAL;          /* Return success */
}

/******************************************************************************
 * @func    This Tparse action routine is called after parsing a CLOSE PIPE
 *      command. Its purpose is to append a Clpipe record to the list of
 *      'TOF' lines. The Clpipe record has the following format:
 *
 *      <PRE>
 *      Clpipe
 *      pipid <pipe_id>
 *      {errlab <error_label>}
 *      </PRE>
 *
 *      Note that if this is a passed pipe, the index is incremented by
 *      the constant PASSED_PIPE_BASE, to show that it is a passed pipe.
 *
 * @rdesc   Tparse status (Always returns PSL_NORMAL).
 * @parm    Pointer to Tparse argument block.
 */
unsigned long ENC_CLOSE_PIPE (struct tpaarg *argblk_ptr)
{
    /*
     *  Local data.
     */
    struct scl_com  *com_ptr;       /* SCL common data */
    struct scl_tpa  *tpa_ptr;       /* SCL Tparse data */
    struct fattbl   *fat_ptr;       /* Pipe Access Table pointer */
    char   *lin_ptr;       /* Pointer to line buffer */
    char   pipidlin [TOFBUFLEN];   /* Pipe Id line */

    /*
     *  Copy pointers for ease of reference.
     */
    com_ptr = (struct scl_com *) argblk_ptr->user_ptrp;
    tpa_ptr = com_ptr->tpa_ptrps;
    lin_ptr = com_ptr->toflinc1024; /* 'tof' line buffer pointer */
    fat_ptr = tpa_ptr->pipe_begps + tpa_ptr->open_fididxl - 1;
    /* Point to current entry in table */
    /*
     *  Do not encode anything if we have had an error in the command.
     */
    if (tpa_ptr->error_in_commandn) goto end;

    /*
     *  Output tracing information if required.
     */
#ifdef PSL_TRACING
    if (PSL_TEST_TRACE (PTRC_USR_USER1))
    {
        PSL_OUTPUT_TRACE ("Encoding a CLOSE PIPE Command\n");
        PSL_OUTPUT_TRACE ("Operand Count = %ld\n" ,GET_OPD_COUNT (tpa_ptr));
    }
#endif

    /*
     *  If the pipe has been passed as a parameter, add the 'passed pipe base'
     *  offset to the pipe entry index index to point to the entry in the 'passed
     *  pipe' section of the table.
     */
    if ((fat_ptr->fat_refflgl & PIPATT_PRM) != 0)
    {
        tpa_ptr->open_fididxl += PASSED_PIPE_BASE;
    }

    /*
     *  Get pipe ID and create 'pipid' sub-record.
     */
    (void) sprintf (pipidlin, "pipid %s", fat_ptr->fat_namec20);

    /*
     *  Write the following records to the 'TOF' line list:
     *
     *  Clpipe
     *  pipid <pipe_id>
     *  {errlab <error_label>}
     */
    APPEND_RECHDR (com_ptr, "Clpipe");

    APPEND_TOFLIN (com_ptr, pipidlin);

    if (strlen (tpa_ptr->onerror_labelc16) != 0)
    {
        (void) strcpy (lin_ptr, "errlab ");
        RESOLVE_LABEL (argblk_ptr,
                       tpa_ptr->onerror_labelc16,
                       tpa_ptr->onerror_lenl);
        APPEND_TOFLIN (com_ptr, NULL);
    }

    /*
     *  Mark the pipe as having been accessed.
     */
    fat_ptr->fat_refflgl |= PIPATT_ACC;

end:
    ;
    return PSL_NORMAL;          /* Return success */
}

/******************************************************************************
 * @func    This Tparse action routine is called after parsing an CONNECT
 *      command. Its purpose is to append a Connect record to the list of
 *      'TOF' lines. The Connect record has the following format:
 *
 *      <PRE>
 *      Connect
 *      server <server_port>
 *      conid <conid>
 *      </PRE>
 *
 * @rdesc   Tparse status (Always returns PSL_NORMAL).
 * @parm    Pointer to Tparse argument block.
 */
unsigned long ENC_CONNECT (struct tpaarg *argblk_ptr)
{
    /*
     *  Local data.
     */
    struct scl_com  *com_ptr;       /* SCL common data */
    struct scl_tpa  *tpa_ptr;       /* SCL Tparse data */
    char   *lin_ptr;       /* Pointer to line buffer */
    char   svrnam [TOFBUFLEN]; /* Server name */
    char   conid [TOFBUFLEN];  /* Conid */

    /*
     *  Copy pointers for ease of reference.
     */
    com_ptr = (struct scl_com *) argblk_ptr->user_ptrp;
    tpa_ptr = com_ptr->tpa_ptrps;
    lin_ptr = com_ptr->toflinc1024; /* 'tof' line buffer pointer */

    /*
     *  Do not encode anything if we have had an error in the command.
     */
    if (tpa_ptr->error_in_commandn) goto end;

    /*
     *  Output tracing information if required.
     */
#ifdef PSL_TRACING
    if (PSL_TEST_TRACE (PTRC_USR_USER1))
    {
        PSL_OUTPUT_TRACE ("Encoding an CONNECT Command\n");
        PSL_OUTPUT_TRACE ("Operand Count = %ld\n" ,GET_OPD_COUNT (tpa_ptr));
    }
#endif

    /* Get server/port name */
    svrnam [0] = '\0';
    ENC_EXPR_BYPOS (com_ptr, svrnam);

    /* Get connection id */
    conid [0] = '\0';
    ENC_EXPR_BYPOS (com_ptr, conid);

    /*
     *  Write the following records to the 'TOF' line list:
     *
     *  Connect
     *  server <server_port>
     *  conid <conid>
     */
    APPEND_RECHDR (com_ptr, "Connect");

    (void) sprintf (lin_ptr, "server %s", svrnam);
    APPEND_TOFLIN (com_ptr, NULL);

    (void) sprintf (lin_ptr, "conid %s", conid);
    APPEND_TOFLIN (com_ptr, NULL);

end:
    ;
    return PSL_NORMAL;          /* Return success */
}

/******************************************************************************
 * @func    This Tparse action routine is called after parsing a CONVERT
 *      command. Its purpose is to append a Convert record to the list of
 *      'TOF' lines. The Convert record has the following format:
 *
 *      <PRE>
 *      Convert
 *      from <variable>
 *      to <variable>
 *      radix <radix>
 *      options <options>
 *      {errlab <error_label>}
 *      </PRE>
 *
 * @rdesc   Tparse status (Always returns PSL_NORMAL).
 * @parm    Pointer to Tparse argument block.
 */
unsigned long ENC_CONVERT (struct tpaarg *argblk_ptr)
{
    /*
     *  Local data.
     */
    struct scl_com  *com_ptr;       /* SCL common data */
    struct scl_tpa  *tpa_ptr;       /* SCL Tparse data */
    char   *lin_ptr;       /* Pointer to line buffer */
    char   fromvarlin [TOFBUFLEN]; /* From variable line */
    char   tovarlin [TOFBUFLEN];   /* To variable line */
    char   radixlin [TOFBUFLEN];   /* Radix line */
    unsigned long   cnvopt = 0;     /* TOF Convert options */

    /*
     *  Copy pointers for ease of reference.
     */
    com_ptr = (struct scl_com *) argblk_ptr->user_ptrp;
    tpa_ptr = com_ptr->tpa_ptrps;
    lin_ptr = com_ptr->toflinc1024; /* 'tof' line buffer pointer */

    /*
     *  Do not encode anything if we have had an error in the command.
     */
    if (tpa_ptr->error_in_commandn) goto end;

    /*
     *  Output tracing information if required.
     */
#ifdef PSL_TRACING
    if (PSL_TEST_TRACE (PTRC_USR_USER1))
    {
        PSL_OUTPUT_TRACE ("Encoding a CONVERT Command\n");
        PSL_OUTPUT_TRACE ("Operand Count = %ld\n" ,GET_OPD_COUNT (tpa_ptr));
    }
#endif

    /*
     *  Get 'from' variable name and create from sub-record.
     */
    (void) strcpy (fromvarlin, "from ");
    ENC_OPD_BYPOS (com_ptr, fromvarlin, NULL, NULL);

    /*
     *  Get 'to' variable name and create to sub-record.
     */
    (void) strcpy (tovarlin, "to ");
    ENC_OPD_BYPOS (com_ptr, tovarlin, NULL, NULL);

    /*
     *  Get 'radix' operand and create radix sub-record.
     */
    (void) strcpy (radixlin, "radix ");
    if (((*tpa_ptr).cnv_optl & CNVOPT_RADIX) != 0)
    {
        ENC_OPD_BYPOS (com_ptr, radixlin, NULL, NULL);
    }
    else
    {
        (void) strcat (radixlin, "10");
    }

    /*
     *  Set the TOF Convert options mask.
     */
    cnvopt = 0;
    if ((tpa_ptr->cnv_optl & CNVOPT_SIGN) != 0)
    {
        cnvopt |= TOF_CNVOPT_SIGN;
    }

    if ((tpa_ptr->cnv_optl & CNVOPT_LEAD) != 0)
    {
        cnvopt |= TOF_CNVOPT_LEAD;
    }

    if ((tpa_ptr->cnv_optl & CNVOPT_JUST) != 0)
    {
        cnvopt |= TOF_CNVOPT_JUST;
    }

    /*
     *  Write the following records to the 'TOF' line list:
     *
     *  Convert
     *  from <variable>
     *  to <variable>
     *  radix <radix>
     *  options <options>
     *  {errlab <error_label>}
     */
    APPEND_RECHDR (com_ptr, "Convert");

    APPEND_TOFLIN (com_ptr, fromvarlin);
    APPEND_TOFLIN (com_ptr, tovarlin);
    APPEND_TOFLIN (com_ptr, radixlin);

    (void) sprintf (lin_ptr, "options %d", cnvopt);
    APPEND_TOFLIN (com_ptr, NULL);

    if (strlen (tpa_ptr->onerror_labelc16) != 0)
    {
        (void) strcpy (lin_ptr, "errlab ");
        RESOLVE_LABEL (argblk_ptr,
                       tpa_ptr->onerror_labelc16,
                       tpa_ptr->onerror_lenl);
        APPEND_TOFLIN (com_ptr, NULL);
    }

end:
    ;
    return PSL_NORMAL;          /* Return success */
}

/******************************************************************************
 * @func    This Tparse action routine is called after parsing a CREATE PIPE
 *      command. Its purpose is to append a Crpipe record to the list of
 *      'TOF' lines. The Crpipe record has the following format:
 *
 *      <PRE>
 *      Crpipe
 *      type <type>
 *      scope <scope>
 *      name <pipe_name> | variable <variable_name>
 *      {timeout <timeout_period>}
 *      {tmolab <timeout_label>}
 *      {errlab <error_label>}
 *      </PRE>
 *
 * @rdesc   Tparse status (Always returns PSL_NORMAL).
 * @parm    Pointer to Tparse argument block.
 */
unsigned long ENC_CREATE_PIPE (struct tpaarg *argblk_ptr)
{
    /*
     *  Local data.
     */
    struct scl_com  *com_ptr;       /* SCL common data */
    struct scl_tpa  *tpa_ptr;       /* SCL Tparse data */
    unsigned long   pipscp;         /* Pipe scope flag */
    unsigned long   piptyp;         /* Pipe type flag */
    char   *lin_ptr;       /* Pointer to line buffer */
    char   pipnamlin [TOFBUFLEN];  /* Pipe name line */
    char   tmoprdlin [TOFBUFLEN];  /* Timeout line */

    /*
     *  Copy pointers for ease of reference.
     */
    com_ptr = (struct scl_com *) argblk_ptr->user_ptrp;
    tpa_ptr = com_ptr->tpa_ptrps;
    lin_ptr = com_ptr->toflinc1024; /* 'tof' line buffer pointer */

    /*
     *  Do not encode anything if we have had an error in the command.
     */
    if (tpa_ptr->error_in_commandn) goto end;

    /*
     *  Output tracing information if required.
     */
#ifdef PSL_TRACING
    if (PSL_TEST_TRACE (PTRC_USR_USER1))
    {
        PSL_OUTPUT_TRACE ("Encoding a CREATE PIPE Command\n");
        PSL_OUTPUT_TRACE ("Operand Count = %ld\n" ,GET_OPD_COUNT (tpa_ptr));
    }
#endif

    /*
     *  Get the pipe type flag..
     */
    if ((tpa_ptr->pipe_flagsl & PIPFLG_SYNCHRONOUS) != 0)
    {
        piptyp = TOF_PIPTYP_SYNCHRONOUS;
    }
    else
    {
        piptyp = TOF_PIPTYP_ASYNCHRONOUS;
    }

    /*
     *  Get the scope flag.
     */
    if ((tpa_ptr->pipe_flagsl & PIPFLG_TESTWIDE) != 0)
    {
        pipscp = TOF_PIPSCP_TESTWIDE;
    }
    else
    {
        pipscp = TOF_PIPSCP_LOCAL;
    }

    /*
     *  Get pipe name or RETURNING TO variable and create corresponding sub-record,
     *  i.e. name <pipe_name> or variable <variable_name>.
     */
    if ((tpa_ptr->pipe_flagsl & PIPFLG_SYSNAME) != 0)
    {
        (void) strcpy (pipnamlin, "variable ");
    }
    else
    {
        (void) strcpy (pipnamlin, "name ");
    }

    ENC_OPD_BYPOS (com_ptr, pipnamlin, NULL, NULL);

    /*
     *  Get the wait period, if any, and create 'timeout' sub-record.
     */
    if ((tpa_ptr->cmnd_optl & CMND_OPT_WAIT_PRD) != 0)
    {
        (void) strcpy (tmoprdlin, "timeout ");
        ENC_OPD_BYPOS (com_ptr, tmoprdlin, NULL, NULL);
    }

    /*
     *  Write the following records to the 'TOF' line list:
     *
     *  Crpipe
     *  type <type>
     *  scope <scope>
     *  name <pipe_name> | variable <variable_name>
     *  {timeout <timeout_period>}
     *  {tmolab <timeout_label>}
     *  {errlab <error_label>}
     */
    APPEND_RECHDR (com_ptr, "Crpipe");

    (void) sprintf (lin_ptr, "type %d", piptyp);
    APPEND_TOFLIN (com_ptr, NULL);

    (void) sprintf (lin_ptr, "scope %d", pipscp);
    APPEND_TOFLIN (com_ptr, NULL);

    APPEND_TOFLIN (com_ptr, pipnamlin);

    if ((tpa_ptr->cmnd_optl & CMND_OPT_WAIT_PRD) != 0)
    {
        APPEND_TOFLIN (com_ptr, tmoprdlin);
    }

    if ((tpa_ptr->cmnd_optl & CMND_OPT_TIMEOUT) != 0)
    {
        (void) sprintf (lin_ptr, "tmolab %s", tpa_ptr->ontimeout_labelc20);
        APPEND_TOFLIN (com_ptr, lin_ptr);
    }

    if (strlen (tpa_ptr->onerror_labelc16) != 0)
    {
        (void) strcpy (lin_ptr, "errlab ");
        RESOLVE_LABEL (argblk_ptr,
                       tpa_ptr->onerror_labelc16,
                       tpa_ptr->onerror_lenl);
        APPEND_TOFLIN (com_ptr, NULL);
    }

end:
    ;
    return PSL_NORMAL;          /* Return success */
}

/******************************************************************************
 * @func    This Tparse action routine is called after parsing a DELETE PIPE
 *      command. Its purpose is to append a Dlpipe record to the list of
 *      'TOF' lines. The Dlpipe record has the following format:
 *
 *      <PRE>
 *      Dlpipe
 *      scope <scope>
 *      pipename <pipe_name>
 *      {timeout <timeout_period>}
 *      {tmolab <timeout_label>}
 *      {errlab <error_label>}
 *      </PRE>
 *
 * @rdesc   Tparse status (Always returns PSL_NORMAL).
 * @parm    Pointer to Tparse argument block.
 */
unsigned long ENC_DELETE_PIPE (struct tpaarg *argblk_ptr)
{
    /*
     *  Local data.
     */
    struct scl_com  *com_ptr;       /* SCL common data */
    struct scl_tpa  *tpa_ptr;       /* SCL Tparse data */
    unsigned long   pipscp;         /* Pipe scope */
    char   *lin_ptr;       /* Pointer to line buffer */
    char   pipnamlin [TOFBUFLEN];  /* Pipe name line */
    char   tmoprdlin [TOFBUFLEN];  /* Timeout line */

    /*
     *  Copy pointers for ease of reference.
     */
    com_ptr = (struct scl_com *) argblk_ptr->user_ptrp;
    tpa_ptr = com_ptr->tpa_ptrps;
    lin_ptr = com_ptr->toflinc1024; /* 'tof' line buffer pointer */

    /*
     *  Do not encode anything if we have had an error in the command.
     */
    if (tpa_ptr->error_in_commandn) goto end;

    /*
     *  Output tracing information if required.
     */
#ifdef PSL_TRACING
    if (PSL_TEST_TRACE (PTRC_USR_USER1))
    {
        PSL_OUTPUT_TRACE ("Encoding a DELETE PIPE Command\n");
        PSL_OUTPUT_TRACE ("Operand Count = %ld\n" ,GET_OPD_COUNT (tpa_ptr));
    }
#endif

    /*
     *  Get the scope flag.
     */
    if ((tpa_ptr->pipe_flagsl & PIPFLG_TESTWIDE) != 0)
    {
        pipscp = TOF_PIPSCP_TESTWIDE;
    }
    else
    {
        pipscp = TOF_PIPSCP_LOCAL;
    }

    /*
     *  Get pipe name and create 'name' sub-record.
     */
    (void) strcpy (pipnamlin, "pipename ");
    ENC_OPD_BYPOS (com_ptr, pipnamlin, NULL, NULL);

    /*
     *  Get the wait period, if any, and create 'timeout' sub-record.
     */
    if ((tpa_ptr->cmnd_optl & CMND_OPT_WAIT_PRD) != 0)
    {
        (void) strcpy (tmoprdlin, "timeout ");
        ENC_OPD_BYPOS (com_ptr, tmoprdlin, NULL, NULL);
    }

    /*
     *  Write the following records to the 'TOF' line list:
     *
     *  Dlpipe
     *  scope <scope>
     *  pipename <pipe_name>
     *  {timeout <timeout_period>}
     *  {tmolab <timeout_label>}
     *  {errlab <error_label>}
     */
    APPEND_RECHDR (com_ptr, "Dlpipe");

    (void) sprintf (lin_ptr, "scope %d", pipscp);
    APPEND_TOFLIN (com_ptr, NULL);

    APPEND_TOFLIN (com_ptr, pipnamlin);

    if ((tpa_ptr->cmnd_optl & CMND_OPT_WAIT_PRD) != 0)
    {
        APPEND_TOFLIN (com_ptr, tmoprdlin);
    }

    if ((tpa_ptr->cmnd_optl & CMND_OPT_TIMEOUT) != 0)
    {
        (void) sprintf (lin_ptr, "tmolab %s", tpa_ptr->ontimeout_labelc20);
        APPEND_TOFLIN (com_ptr, lin_ptr);
    }

    if (strlen (tpa_ptr->onerror_labelc16) != 0)
    {
        (void) strcpy (lin_ptr, "errlab ");
        RESOLVE_LABEL (argblk_ptr,
                       tpa_ptr->onerror_labelc16,
                       tpa_ptr->onerror_lenl);
        APPEND_TOFLIN (com_ptr, NULL);
    }

end:
    ;
    return PSL_NORMAL;          /* Return success */
}

/******************************************************************************
 * @func    This Tparse action routine is called after parsing a DETACH THREAD
 *      or TERMINAL command. Its purpose is to append an Abort record to
 *      the list of 'TOF' lines. This record has the following format:
 *
 *      <PRE>
 *      Abort
 *      target TOF_ABTTGT_THREAD
 *      </PRE>
 *
 * @rdesc   Tparse status (Always returns PSL_NORMAL).
 * @parm    Pointer to Tparse argument block.
 */
unsigned long ENC_DETACH_TRM (struct tpaarg *argblk_ptr)
{
    /*
     *    Local data
     */
    struct scl_com  *com_ptr;       /* SCL comon data */
    struct scl_tpa  *tpa_ptr;       /* SCL Tparse data */
    char   *lin_ptr;       /* Pointer to line buffer */

    /*
     *  Copy pointers for ease of reference.
     */
    com_ptr = (struct scl_com *) argblk_ptr->user_ptrp;
    tpa_ptr = com_ptr->tpa_ptrps;
    lin_ptr = com_ptr->toflinc1024; /* 'tof' line buffer pointer */

    /*
     *  Do not encode anything if we have had an error in the command.
     */
    if (tpa_ptr->error_in_commandn) goto end;

    /*
     *  Output tracing information if required.
     */
#ifdef PSL_TRACING
    if (PSL_TEST_TRACE (PTRC_USR_USER1))
    {
        PSL_OUTPUT_TRACE ("\nEncoding a DETACH THREAD Command\n");
        PSL_OUTPUT_TRACE ("Operand Count = %ld\n" ,GET_OPD_COUNT (tpa_ptr));
    }
#endif

    /*
     *  Write the following records to the 'TOF' line list:
     *
     *  Abort
     *  target TOF_ABTTGT_THREAD
     */
    APPEND_RECHDR (com_ptr, "Abort");

    (void) sprintf (lin_ptr, "target %d", TOF_ABTTGT_THREAD);
    APPEND_TOFLIN (com_ptr, NULL);
end:
    ;
    return PSL_NORMAL;
}

/******************************************************************************
 * @func    This Tparse action routine is called after parsing a DISCONNECT
 *      command. Its purpose is to append a Disconnect record
 *      to the linked list of 'TOF' lines. The Disconnect record has the
 *      following format:
 *
 *      <PRE>
 *      Disconnect
 *      {conid <conid>}
 *      options <options_code>
 *      </PRE>
 *
 * @rdesc   Tparse status (Always returns PSL_NORMAL).
 * @parm    Pointer to Tparse argument block.
 */
unsigned long ENC_DISCONNECT (struct tpaarg *argblk_ptr)
{
    /*
     *  Local data.
     */
    struct   scl_com *com_ptr;      /* SCL common data */
    struct   scl_tpa *tpa_ptr;      /* SCL Tparse data */
    char    *lin_ptr;      /* Pointer to line buffer */
    unsigned long    disopt;        /* TOF Disconnect options */

    /*
     *  Copy pointers for ease of reference.
     */
    com_ptr = (struct scl_com *) argblk_ptr->user_ptrp;
    tpa_ptr = com_ptr->tpa_ptrps;
    lin_ptr = com_ptr->toflinc1024; /* 'tof' line buffer pointer */

    /*
     *  Do not encode anything if we have had an error in the command.
     */
    if (tpa_ptr->error_in_commandn) goto end;

    /*
     *  Output tracing information if required.
     */
#ifdef PSL_TRACING
    if (PSL_TEST_TRACE (PTRC_USR_USER1))
    {
        PSL_OUTPUT_TRACE ("Encoding a DISCONNECT Command\n");
    }
#endif

    /*
     *  Set the TOF Disconnect options mask.
     */
    disopt = 0;
    if ((tpa_ptr->disconnect_maskl & DISCONNECT_CANCEL) != 0)
    {
        disopt |= TOF_DCNOPT_CANCEL;
    }

    /*
     *  Write the following records to the 'TOF' line list:
     *
     *  Disconnect
     *  {conid <conid>}
     *  options <options_code>
     */
    APPEND_RECHDR (com_ptr, "Disconnect");

    if ((tpa_ptr->disconnect_maskl & DISCONNECT_CONID) != 0)
    {
        (void) strcpy (lin_ptr, "conid ");
        ENC_EXPR_BYPOS (com_ptr, lin_ptr);
        APPEND_TOFLIN (com_ptr, NULL);
    }

    (void) sprintf (lin_ptr, "options %d", disopt);
    APPEND_TOFLIN (com_ptr, NULL);

end:
    ;
    return PSL_NORMAL;
}

/******************************************************************************
 * @func    This Tparse action routine is called after parsing a DO command.
 *      Its purpose is to append a Loop record to the list of 'TOF' lines.
 *      The Loop record has the following format:
 *
 *      <PRE>
 *      Loop
 *      name <variable>
 *      initial <expression>
 *      eValue <expression>
 *      step <expression>
 *      </PRE>
 *
 * @rdesc   Tparse status (Always returns PSL_NORMAL).
 * @parm    Pointer to Tparse argument block.
 */
unsigned long ENC_DO (struct tpaarg *argblk_ptr)
{
    /*
     *    Local data
     */
    struct scl_com  *com_ptr;       /* SCL comon data */
    struct scl_tpa  *tpa_ptr;       /* SCL Tparse data */
    char   *lin_ptr;       /* Pointer to line buffer */
    char   loop_var [TOFBUFLEN];   /* Loop variable operand */
    char   loop_start [TOFBUFLEN]; /* Loop start operand */
    char   loop_end [TOFBUFLEN];   /* Loop end operand */
    char   loop_step [TOFBUFLEN];  /* Loop step operand */

    /*
     *  Copy pointers for ease of reference.
     */
    com_ptr = (struct scl_com *) argblk_ptr->user_ptrp;
    tpa_ptr = com_ptr->tpa_ptrps;
    lin_ptr = com_ptr->toflinc1024; /* 'tof' line buffer pointer */

    /*
     *  Do not encode anything if we have had an error in the command.
     */
    if (tpa_ptr->error_in_commandn) goto end;

    /*
     *  Output tracing information if required.
     */
#ifdef PSL_TRACING
    if (PSL_TEST_TRACE (PTRC_USR_USER1))
    {
        PSL_OUTPUT_TRACE ("\nEncoding a DO Command\n");
        PSL_OUTPUT_TRACE ("Operand Count = %ld\n" ,GET_OPD_COUNT (tpa_ptr));
    }
#endif
    /*
     *  Get loop control variable, start, limit and step operands into local buffers.
     *  If STEP operand not specified then default to 1.
     */
    loop_var [0] = '\0';
    ENC_OPD_BYPOS (com_ptr, loop_var, NULL, NULL);

    loop_start [0] = '\0';
    ENC_OPD_BYPOS (com_ptr, loop_start, NULL, NULL);

    loop_end [0] = '\0';
    ENC_OPD_BYPOS (com_ptr, loop_end, NULL, NULL);

    loop_step [0] = '\0';
    ENC_OPD_BYPOS (com_ptr, loop_step, NULL, NULL);

    /*
     *  Write the following records to the 'TOF' line list:
     *
     *  Loop
     *  name <variable>
     *  initial <expression>
     *  eValue <expression>
     *  step <expression>
     */
    APPEND_RECHDR (com_ptr, "Loop");

    (void) sprintf (lin_ptr, "name %s", loop_var);
    APPEND_TOFLIN (com_ptr, NULL);

    (void) sprintf (lin_ptr, "initial %s", loop_start);
    APPEND_TOFLIN (com_ptr, NULL);

    (void) sprintf (lin_ptr, "eValue %s", loop_end);
    APPEND_TOFLIN (com_ptr, NULL);

    (void) sprintf (lin_ptr, "step %s", loop_step);
    APPEND_TOFLIN (com_ptr, NULL);
end:
    ;
    return PSL_NORMAL;
}

/******************************************************************************
 * @func    This Tparse action routine is called after parsing an ELSE
 *      command. Its purpose is to append an Else record to the list
 *      of 'TOF' lines. The Else record has the following format:
 *
 *      <PRE>
 *      Else
 *      </PRE>
 *
 *      It reports an error if an ELSE is not valid here.
 *
 * @rdesc   Tparse status (Always returns PSL_NORMAL).
 * @parm    Pointer to Tparse argument block.
 */
unsigned long ENC_ELSE (struct tpaarg *argblk_ptr)
{
    /*
     *  Local data.
     */
    struct scl_com  *com_ptr;       /* SCL common data */
    struct scl_tpa  *tpa_ptr;       /* SCL Tparse data */
    struct ifstack  *if_ptr;        /* Pointer to IF stack */
    char   *lin_ptr;       /* Pointer to line buffer */

    /*
     *  Copy pointers for ease of reference.
     */
    com_ptr = (struct scl_com *) argblk_ptr->user_ptrp;
    tpa_ptr = com_ptr->tpa_ptrps;
    lin_ptr = com_ptr->toflinc1024; /* 'tof' line buffer pointer */

    /*
     *  Do not encode anything if we have had an error in the command.
     */
    if (tpa_ptr->error_in_commandn) goto end;

    /*
     *  Output tracing information if required.
     */
#ifdef PSL_TRACING
    if (PSL_TEST_TRACE (PTRC_USR_USER1))
    {
        PSL_OUTPUT_TRACE ("Encoding an ELSE Command\n");
        PSL_OUTPUT_TRACE ("Operand Count = %ld\n" ,GET_OPD_COUNT (tpa_ptr));
    }
#endif

    /*
     *  If the current IF nesting level is 0 or less then there is no IF command
     *  to match up with this ELSE. We must therefore report an error.
     */
    if (tpa_ptr->ifstack_levell <= 0)
    {
        REPORT_ERROR (argblk_ptr, SCL_INVSTRUCT);
        goto end;
    }

    /*
     *  If we have already parsed an ELSE command report an error - cannot have
     *  an ELSE after an ELSE.
     */
    if_ptr = tpa_ptr->ifstack_begps + tpa_ptr->ifstack_levell - 1;

    if (if_ptr->else_parsedn)
    {
        REPORT_ERROR (argblk_ptr, SCL_INVSTRUCT);
        goto end;
    }

    /*
     *  Write the following records to the 'TOF' line list:
     *
     *  Else
     */
    APPEND_RECHDR (com_ptr, "Else");

end:
    ;
    return PSL_NORMAL;
}

/******************************************************************************
 * @func    This Tparse action routine is called after parsing an ELSEIF ...
 *      THEN command. Its purpose is to append an Else record to the list
 *      of 'TOF' lines. The Else record has the following format:
 *
 *      <PRE>
 *      Else
 *      expr <condition>
 *      </PRE>
 *
 * @rdesc   Tparse status (Always returns PSL_NORMAL).
 * @parm    Pointer to Tparse argument block.
 */
unsigned long ENC_ELSEIF_THEN (struct tpaarg *argblk_ptr)
{
    /*
     *  Local data.
     */
    struct scl_com  *com_ptr;       /* SCL common data */
    struct scl_tpa  *tpa_ptr;       /* SCL Tparse data */
    struct ifstack  *if_ptr;        /* Pointer to IF stack */
    char   *lin_ptr;       /* Pointer to line buffer */

    /*
     *  Copy pointers for ease of reference.
     */
    com_ptr = (struct scl_com *) argblk_ptr->user_ptrp;
    tpa_ptr = com_ptr->tpa_ptrps;
    lin_ptr = com_ptr->toflinc1024; /* 'tof' line buffer pointer */

    /*
     *  Do not encode anything if we have had an error in the command.
     */
    if (tpa_ptr->error_in_commandn) goto end;

    /*
     *  Output tracing information if required.
     */
#ifdef PSL_TRACING
    if (PSL_TEST_TRACE (PTRC_USR_USER1))
    {
        PSL_OUTPUT_TRACE ("Encoding an ELSEIF THEN Command\n");
        PSL_OUTPUT_TRACE ("Operand Count = %ld\n" ,GET_OPD_COUNT (tpa_ptr));
    }
#endif

    /*
     *  If we have already parsed an ELSE command report an error - cannot have
     *  an ELSEIF after an ELSE.
     */
    if_ptr = tpa_ptr->ifstack_begps + tpa_ptr->ifstack_levell - 1;

    if (if_ptr->else_parsedn)
    {
        REPORT_ERROR (argblk_ptr, SCL_INVSTRUCT);
        goto end;
    }

    /*
     *  Write the following records to the 'TOF' line list:
     *
     *  Else
     *  expr <condition>
     */
    APPEND_RECHDR (com_ptr, "Else");

    /*  Flush the 'expr <condition>' line from the default 'TOF' line buffer */
    APPEND_TOFLIN (com_ptr, NULL);

end:
    ;
    return PSL_NORMAL;
}

/******************************************************************************
 * @func    This Tparse action routine is called after parsing an ENDDO
 *      command. Its purpose is to append an Eloop record to the linked
 *      list of 'TOF' lines. The Eloop record has the following format:
 *
 *      <PRE>
 *      Eloop
 *      </PRE>
 *
 * @rdesc   Tparse status (Always returns PSL_NORMAL).
 * @parm    Pointer to Tparse argument block.
 */
unsigned long ENC_ENDDO (struct tpaarg *argblk_ptr)
{
    /*
     *    Local data
     */
    struct   scl_com *com_ptr;      /* SCL common data */
    struct   scl_tpa *tpa_ptr;      /* SCL Tparse data */

    /*
     *  Copy pointers for ease of reference.
     */
    com_ptr = (struct scl_com *) argblk_ptr->user_ptrp;
    tpa_ptr = com_ptr->tpa_ptrps;

    /*
     *  Do not encode anything if we have had an error in the command.
     */
    if (tpa_ptr->error_in_commandn) goto end;

    /*
     *  Output tracing information if required.
     */
#ifdef PSL_TRACING
    if (PSL_TEST_TRACE (PTRC_USR_USER1))
    {
        PSL_OUTPUT_TRACE ("Encoding an ENDDO Command\n");
    }
#endif

    /*
     *  Check to ensure that we have a DO loop started and if not report an error.
     */
    if (tpa_ptr->do_cntl <= 0)  /* If nesting depth <= 0 */
    {
        REPORT_ERROR (argblk_ptr, SCL_INVSTRUCT);
        goto end;
    }

    /*
     *  ...end of a DO loop - decrement the DO index.
     */
    tpa_ptr->do_cntl -= 1;      /* ...decrement it */

    /*
     *  Write the following records to the 'TOF' line list:
     *
     *  Eloop
     */
    APPEND_RECHDR (com_ptr, "Eloop");

end:
    ;
    return PSL_NORMAL;
}

/******************************************************************************
 * @func    This Tparse action routine is called after parsing an ENDIF
 *      command. Its purpose is to append an EndIf record to the linked
 *      list of 'TOF' lines. The EndIf record has the following format:
 *
 *      <PRE>
 *      Eif
 *      </PRE>
 *
 * @rdesc   Tparse status (Always returns PSL_NORMAL).
 * @parm    Pointer to Tparse argument block.
 */
unsigned long ENC_ENDIF (struct tpaarg *argblk_ptr)
{
    /*
     *  Local data.
     */
    struct scl_com  *com_ptr;       /* SCL common data */
    struct scl_tpa  *tpa_ptr;       /* SCL Tparse data */

    /*
     *  Copy pointers for ease of reference.
     */
    com_ptr = (struct scl_com *) argblk_ptr->user_ptrp;
    tpa_ptr = com_ptr->tpa_ptrps;

    /*
     *  Do not encode anything if we have had an error in the command.
     */
    if (tpa_ptr->error_in_commandn) goto end;

    /*
     *  Output tracing information if required.
     */
#ifdef PSL_TRACING
    if (PSL_TEST_TRACE (PTRC_USR_USER1))
    {
        PSL_OUTPUT_TRACE ("Encoding an ENDIF Command\n");
    }
#endif

    /*
     *    There is a possibility that we have not actually parsed an IF (or at
     *    least not sucessfully). We MUST check to ensure that the level is not
     *    already zero and if it is must skip to the end.
     */
    if (tpa_ptr->ifstack_levell <= 0) goto end;

    /*
     *  End of an IF ... ENDIF - decrement the IF level.
     */
    tpa_ptr->ifstack_levell -= 1;

    /*
     *  Write the following records to the 'TOF' line list:
     *
     *  Eif
     */
    APPEND_RECHDR (com_ptr, "Eif");

end:
    ;
    return PSL_NORMAL;
}

/******************************************************************************
 * @func    This Tparse action routine is called after parsing an ENTRY
 *      command. Its purpose is to append an Entry record to the list of
 *      'TOF' lines. The Entry record has the following format:
 *
 *      <PRE>
 *      Entry
 *      parameters <parameters>
 *      </PRE>
 *
 * @rdesc   Tparse status (Always returns PSL_NORMAL).
 * @parm    Pointer to Tparse argument block.
 */
unsigned long ENC_ENTRY (struct tpaarg *argblk_ptr)
{
    /*
     *  Local data.
     */
    struct   scl_com *com_ptr;      /* SCL common data */
    struct   scl_tpa *tpa_ptr;      /* SCL TPARSE data */
    struct   prmtbl  *prm_ptr;      /* Pointer to script parameter data */
    unsigned long    isub;          /* Loop control */
    char    *lin_ptr;      /* Pointer to line buffer */
    char    prmlin [TOFBUFLEN];    /* Parameters line */
    char    prmval [TOFBUFLEN];    /* Parameter value */

    /*
     *  Copy pointers for ease of reference.
     */
    com_ptr = (struct scl_com *) argblk_ptr->user_ptrp;
    tpa_ptr = com_ptr->tpa_ptrps;
    lin_ptr = com_ptr->toflinc1024; /* 'tof' line buffer pointer */

    /*
     *  Do not encode anything if we have had an error in the command.
     */
    if (tpa_ptr->error_in_commandn) goto end;

    /*
     *  Output tracing information if required.
     */
#ifdef PSL_TRACING
    if (PSL_TEST_TRACE (PTRC_USR_USER1))
    {
        PSL_OUTPUT_TRACE ("\nEncoding an ENTRY Command\n");
        PSL_OUTPUT_TRACE ("Operand Count = %ld\n" ,GET_OPD_COUNT (tpa_ptr));
    }
#endif

    /*
     *  Construct the 'parameters' sub-record by encoding the parameters
     *  (stored in the script parameter list). The sub-record has the following format:
     *
     *  parameters p1{{,pn}...}
     *
     *  A parameter may be a file name, pipe name or simple (i.e. not an array
     *  reference) variable name.
     */
    (void) strcpy (prmlin, "parameters ");

    for (isub=0, prm_ptr = (*tpa_ptr).parameter_begps;
            isub < tpa_ptr->scpprm_cntl;
            isub++, prm_ptr = (*tpa_ptr).parameter_begps + isub)
    {
        (void) memcpy (prmval, (*prm_ptr).prm_namec16, (*prm_ptr).prm_nlengthl);
        prmval [(*prm_ptr).prm_nlengthl] = '\0';
        (void) strcat (prmlin, prmval);
        if (isub < (tpa_ptr->scpprm_cntl - 1)) (void) strcat (prmlin, ",");
    }

    /*
     *  Write the following records to the 'TOF' line list:
     *
     *  Entry
     *  parameters <parameters>
     */
    APPEND_RECHDR (com_ptr, "Entry");

    APPEND_TOFLIN (com_ptr, prmlin);

end:
    ;
    return PSL_NORMAL;          /* Return success */
}

/******************************************************************************
 * @func    This Tparse action routine is called after parsing an EXECUTE
 *      THREAD command. Its purpose is to append an Exthread record to the
 *      list of 'TOF' lines. The Exthread record has the following format:
 *
 *      <PRE>
 *      Exthread
 *      name <thread>
 *      script <name>
 *      {onnode <node_name>}
 *      {delay <delay>}
 *      {number <number>}
 *      {interval <interval>}
 *      </PRE>
 *
 * @rdesc   Tparse status (Always returns PSL_NORMAL).
 * @parm    Pointer to Tparse argument block.
 */
unsigned long ENC_EXECUTE_THR (struct tpaarg *argblk_ptr)
{
    /*
     *  Local data.
     */
    struct scl_com  *com_ptr;       /* SCL common data */
    struct scl_tpa  *tpa_ptr;       /* SCL Tparse data */
    char   *lin_ptr;       /* Pointer to line buffer */
    unsigned long   linlen;         /* Line length */

    /*
     *  Copy pointers for ease of reference.
     */
    com_ptr = (struct scl_com *) argblk_ptr->user_ptrp;
    tpa_ptr = com_ptr->tpa_ptrps;
    lin_ptr = com_ptr->toflinc1024; /* 'tof' line buffer pointer */

    /*
     *  Do not encode anything if we have had an error in the command.
     */
    if (tpa_ptr->error_in_commandn) goto end;

    /*
     *  Output tracing information if required.
     */
#ifdef PSL_TRACING
    if (PSL_TEST_TRACE (PTRC_USR_USER1))
    {
        PSL_OUTPUT_TRACE ("Encoding an EXECUTE THREAD Command\n");
        PSL_OUTPUT_TRACE ("Operand Count = %ld\n" ,GET_OPD_COUNT (tpa_ptr));
    }
#endif
    /*
     *  Write the following records to the 'TOF' line list:
     *
     *  Exthread
     *  name <thread>
     *  script <name>
     *  {onnode <node_name>}
     *  {delay <delay>}
     *  {number <number>}
     *  {interval <interval>}
     */
    APPEND_RECHDR (com_ptr, "Exthread");

    (void) strcpy (lin_ptr, "name ");
    ENC_OPD_BYPOS (com_ptr, NULL, NULL, NULL);
    APPEND_TOFLIN (com_ptr, NULL);

    (void) strcpy (lin_ptr, "script ");
    linlen = strlen (lin_ptr);
    ENC_OPD_BYPOS (com_ptr, NULL, NULL, NULL);
    if (strlen (lin_ptr) != linlen) APPEND_TOFLIN (com_ptr, NULL);

    (void) strcpy (lin_ptr, "onnode ");
    linlen = strlen (lin_ptr);
    ENC_OPD_BYID (com_ptr, lin_ptr, CMND_OPT_ONNODE);
    if (strlen (lin_ptr) != linlen) APPEND_TOFLIN (com_ptr, NULL);

    (void) strcpy (lin_ptr, "delay ");
    linlen = strlen (lin_ptr);
    ENC_OPD_BYID (com_ptr, lin_ptr, CMND_OPT_DELAY);
    if (strlen (lin_ptr) != linlen) APPEND_TOFLIN (com_ptr, NULL);

    (void) strcpy (lin_ptr, "number ");
    linlen = strlen (lin_ptr);
    ENC_OPD_BYID (com_ptr, lin_ptr, CMND_OPT_NUMBER);
    if (strlen (lin_ptr) != linlen) APPEND_TOFLIN (com_ptr, NULL);

    (void) strcpy (lin_ptr, "interval ");
    linlen = strlen (lin_ptr);
    ENC_OPD_BYID (com_ptr, lin_ptr, CMND_OPT_INTERVAL);
    if (strlen (lin_ptr) != linlen) APPEND_TOFLIN (com_ptr, NULL);

end:
    ;
    return PSL_NORMAL;
}

/******************************************************************************
 * @func    This Tparse action routine is called after parsing an EXECUTE TEST
 *      command. Its purpose is to append an Extest record to the list
 *      of 'TOF' lines. The Extest record has the following format:
 *
 *      <PRE>
 *      Extest
 *      name <test>
 *      {onnode <node_name>}
 *      </PRE>
 *
 * @rdesc   Tparse status (Always returns PSL_NORMAL).
 * @parm    Pointer to Tparse argument block.
 */
unsigned long ENC_EXECUTE_TST (struct tpaarg *argblk_ptr)
{
    /*
     *  Local data.
     */
    struct scl_com  *com_ptr;       /* SCL common data */
    struct scl_tpa  *tpa_ptr;       /* SCL Tparse data */
    char   *lin_ptr;       /* Pointer to line buffer */
    unsigned long   linlen;         /* Line length */

    /*
     *  Copy pointers for ease of reference.
     */
    com_ptr = (struct scl_com *) argblk_ptr->user_ptrp;
    tpa_ptr = com_ptr->tpa_ptrps;
    lin_ptr = com_ptr->toflinc1024; /* 'tof' line buffer pointer */

    /*
     *  Do not encode anything if we have had an error in the command.
     */
    if (tpa_ptr->error_in_commandn) goto end;

    /*
     *  Output tracing information if required.
     */
#ifdef PSL_TRACING
    if (PSL_TEST_TRACE (PTRC_USR_USER1))
    {
        PSL_OUTPUT_TRACE ("Encoding an EXECUTE TEST Command\n");
        PSL_OUTPUT_TRACE ("Operand Count = %ld\n" ,GET_OPD_COUNT (tpa_ptr));
    }
#endif
    /*
     *  Write the following records to the 'TOF' line list:
     *
     *  Extest
     *  name <test>
     *  {onnode <node_name>}
     */
    APPEND_RECHDR (com_ptr, "Extest");

    (void) strcpy (lin_ptr, "name ");
    ENC_OPD_BYPOS (com_ptr, NULL, NULL, NULL);
    APPEND_TOFLIN (com_ptr, NULL);

    (void) strcpy (lin_ptr, "onnode ");
    linlen = strlen (lin_ptr);
    ENC_OPD_BYID (com_ptr, lin_ptr, CMND_OPT_ONNODE);
    if (strlen (lin_ptr) != linlen) APPEND_TOFLIN (com_ptr, NULL);

end:
    ;
    return PSL_NORMAL;
}

/******************************************************************************
 * @func    This Tparse action routine appends an Exit record to the linked
 *      list of 'TOF' lines. The Exit record has the following format:
 *
 *      <PRE>
 *      Exit
 *      {status <status>}
 *      {options <options_code>}
 *      </PRE>
 *
 * @rdesc   Tparse status (Always returns PSL_NORMAL).
 * @parm    Pointer to Tparse argument block.
 */
unsigned long ENC_EXIT (struct tpaarg *argblk_ptr)
{
    /*
     *  Local data.
     */
    struct   scl_com *com_ptr;      /* SCL common data */
    struct   scl_tpa *tpa_ptr;      /* SCL Tparse data */
    char    *lin_ptr;      /* Pointer to line buffer */
    unsigned long    extopt;        /* TOF Exit options */

    /*
     *  Copy pointers for ease of reference.
     */
    com_ptr = (struct scl_com *) argblk_ptr->user_ptrp;
    tpa_ptr = com_ptr->tpa_ptrps;
    lin_ptr = com_ptr->toflinc1024; /* 'tof' line buffer pointer */

    /*
     *  Do not encode anything if we have had an error in the command.
     */
    if (tpa_ptr->error_in_commandn) goto end;

    /*
     *  Output tracing information if required.
     */
#ifdef PSL_TRACING
    if (PSL_TEST_TRACE (PTRC_USR_USER1))
    {
        PSL_OUTPUT_TRACE ("Encoding an EXIT Command\n");
    }
#endif

    /*
     *  Set the TOF Exit options mask.
     */
    extopt = 0;
    if ((tpa_ptr->exit_maskl & EXIT_KEEPALIVE) != 0)
    {
        extopt |= TOF_EXTOPT_KEEPALIVE;
    }

    /*
     *  Write the following records to the 'TOF' line list:
     *
     *  Exit
     *  {status <status>}
     *  {options <options_code>}
     */
    APPEND_RECHDR (com_ptr, "Exit");

    if ((tpa_ptr->exit_maskl & EXIT_STATUS) != 0)
    {
        (void) strcpy (lin_ptr, "status ");
        ENC_OPD_BYPOS (com_ptr, lin_ptr, NULL, NULL);
        APPEND_TOFLIN (com_ptr, NULL);
    }

    if (extopt != 0)
    {
        (void) sprintf (lin_ptr, "options %d", extopt);
        APPEND_TOFLIN (com_ptr, NULL);
    }
end:
    ;
    return PSL_NORMAL;
}

/******************************************************************************
 * @func    This routine encodes an expression and appends it to the specified
 *      line buffer. The expression consists of all the operands from the
 *      head of the parsed operands linked list to the first NULL operand
 *      (which terminates the expression). This routine simply encodes all
 *      the operands up the the terminating NULL operand in the order in
 *      which they appear. The operands are removed from the list as they
 *      are encoded.
 *
 * @parm    Pointer to SCL common data
 * @parm    Pointer to buffer into which the operand is to be encoded.
 */
static void ENC_EXPR_BYPOS (struct scl_com *com_ptr,
                            char *lin_ptr)
{
    /*
     *  Local data.
     */
    struct scl_tpa  *tpa_ptr;       /* SCL Tparse data */
    struct oph  *oph_ptr;       /* Pointer to operand header */
    unsigned long   iret;           /* Status return variable */
    Boolean     expend;         /* End of expression flag */

    /*
     *  Copy pointers for ease of reference.
     */
    tpa_ptr = com_ptr->tpa_ptrps;

    /*
     *  Loop through the parsed operand linked list encoding each operand as it is
     *  encountered. The expression is terminated by a NULL operand.
     */
    for (expend = PFALSE; expend == PFALSE;)
    {
        iret = PSL_VM_COPY (tpa_ptr->parsed_operand_llps ,(char *) &oph_ptr, NULL,
                            NULL ,PSL_VM_MODE_SPEC);
        /* Copy the OPH pointer */
        if  (!(psl_test_status (iret))) /* Report any errors */
        {
            PSL_SIGDIAG (tpa_ptr->diag_ptrps, 2, PSL_SYSERVERR,
                         "PSL_VM_COPY (in ENC_EXPR_BYPOS Remove OPH)", iret);
        }

        if (oph_ptr->addrl == ADDR_NONE) expend = PTRUE;

        ENC_OPD_BYPOS (com_ptr, lin_ptr, NULL, NULL);
    }

    return;
}

/******************************************************************************
 * @func    This routine encodes an expression and appends it to the specified
 *      line buffer. The first operand in the expression (if any) is the
 *      first operand in the parsed operands linked list whose operand ID
 *      matches the operand ID supplied to this routine. This routine
 *      simply encodes all the operands in the list in the order in which
 *      they appear. The end of the expression is signified by a NULL
 *      operand. The operands are removed from the list as they are
 *      encoded. If no operand in the list has an operand ID matching
 *      the operand ID supplied to this routine nothing is encoded.
 *
 * @parm    Pointer to SCL common data
 * @parm    Pointer to buffer into which the operand is to be encoded.
 * @parm    ID of operand(s) to be encoded.
 */
static void ENC_EXPR_BYID (struct scl_com *com_ptr,
                           char *lin_ptr,
                           unsigned long req_id)
{
    /*
     *  Local data.
     */
    struct scl_tpa  *tpa_ptr;       /* SCL Tparse data */
    struct oph  *oph_ptr;       /* Pointer to operand header */
    char   *llb_pos_ptr;       /* Linked list position */
    char   *llb_ctx_ptr;       /* Linked list context */
    unsigned long   iret;           /* Status return variable */
    Boolean     expend=PFALSE;      /* End of expression flag */
    Boolean     expfnd=PFALSE;      /* Expression found flag */
    char   mode=PSL_VM_MODE_FIRST; /* PSL Linked list mode */

    /*
     *  Copy pointers for ease of reference.
     */
    tpa_ptr = com_ptr->tpa_ptrps;

    /*
     *  Do not encode anything if we have had an error in the command.
     */
    if (tpa_ptr->error_in_commandn) goto end;

    /*
     *  Loop through the parsed operand linked list. First locate first
     *  operand in the expression, then encode each operand as it is encountered.
     *  The expression is terminated by a NULL operand. There may be no
     *  operands matching the specified operand ID.
     */
    for (llb_pos_ptr = NULL; expend == PFALSE;)
    {
        iret = PSL_VM_COPY (tpa_ptr->parsed_operand_llps ,(char *) &oph_ptr,
                            llb_pos_ptr ,&llb_ctx_ptr ,mode);
        /* Copy the OPH pointer */
        if  (!(psl_test_status (iret))) goto end;   /* All done, if error from copy */

        /* Set flag if start of expression found */
        if (oph_ptr->idl == req_id) expfnd = PTRUE;

        /* If expression found encode operand; set flag if end of expression found */
        if (expfnd)
        {
            if (oph_ptr->addrl == ADDR_NONE) expend = PTRUE;
            ENC_OPD_BYPOS (com_ptr, lin_ptr, llb_ctx_ptr, &llb_pos_ptr);
            mode = PSL_VM_MODE_SPEC;    /* Reset mode to pick up specified element */
        }
        else
        {
            llb_pos_ptr = llb_ctx_ptr;  /* Update position to next item */
            mode = PSL_VM_MODE_NEXT;    /* Reset mode to pick up next element */
        }
    }

end:
    ;
    return;
}

/******************************************************************************
 * @func    This Tparse action routine is called after parsing a FORMAT
 *      command. Its purpose is to append a Format record to the list of
 *      'TOF' lines. The Format record has the following format:
 *
 *      <PRE>
 *      Format
 *      to <variable>
 *      string <format_string>
 *      sValues <arguments>     (arguments are comma separated)
 *      errlab <error_label>
 *      </PRE>
 *
 * @rdesc   Tparse status (Always returns PSL_NORMAL).
 * @parm    Pointer to Tparse argument block.
 */
unsigned long ENC_FORMAT (struct tpaarg *argblk_ptr)
{
    /*
     *    Local data
     */
    struct scl_com  *com_ptr;       /* SCL comon data */
    struct scl_tpa  *tpa_ptr;       /* SCL Tparse data */
    char   *lin_ptr;       /* Pointer to line buffer */
    char   targetlin [TOFBUFLEN];  /* Target sub-record line */
    char   formatlin [TOFBUFLEN];  /* Format sub-record line */
    char   valueslin [TOFBUFLEN];  /* Values sub-record line */
    unsigned long   isub;           /* Loop control */

    /*
     *  Copy pointers for ease of reference.
     */
    com_ptr = (struct scl_com *) argblk_ptr->user_ptrp;
    tpa_ptr = com_ptr->tpa_ptrps;
    lin_ptr = com_ptr->toflinc1024; /* 'tof' line buffer pointer */

    /*
     *  Do not encode anything if we have had an error in the command.
     */
    if (tpa_ptr->error_in_commandn) goto end;

    /*
     *  Output tracing information if required.
     */
#ifdef PSL_TRACING
    if (PSL_TEST_TRACE (PTRC_USR_USER1))
    {
        PSL_OUTPUT_TRACE ("\nEncoding a FORMAT Command\n");
        PSL_OUTPUT_TRACE ("Operand Count = %ld\n" ,GET_OPD_COUNT (tpa_ptr));
    }
#endif

    /*
     *  Report an error if the number of FORMAT string arguments does not the number
     *  of entries on the parsed operand linked list - 2 (we subtract 2 to discount
     *  the target and format operands).
     */
    if (tpa_ptr->format_arguml != (GET_OPD_COUNT (tpa_ptr) - 2))
    {
        REPORT_ERROR (argblk_ptr, VTEST_INVNUMARGS);
        goto end;
    }

    /*
     *  Get to, string and sValues lines into local buffers. The operands in the
     *  sValues line are comma separated.
     */
    (void) strcpy (targetlin, "to ");
    ENC_OPD_BYPOS (com_ptr, targetlin, NULL, NULL);

    (void) strcpy (formatlin, "string ");
    ENC_OPD_BYPOS (com_ptr, formatlin, NULL, NULL);

    (void) strcpy (valueslin, "sValues ");

    for (isub = tpa_ptr->format_arguml; isub > 0; isub--)
    {
        ENC_OPD_BYPOS (com_ptr, valueslin, NULL, NULL);
        if (isub > 1) (void) strcat (valueslin, ",");
    }

    /*
     *  Write the following records to the 'TOF' line list:
     *
     *  Format
     *  to <variable>
     *  string <format_string>
     *  sValues <arguments>
     *  {errlab <error_label>}
     */
    APPEND_RECHDR (com_ptr, "Format");

    APPEND_TOFLIN (com_ptr, targetlin);

    APPEND_TOFLIN (com_ptr, formatlin);

    APPEND_TOFLIN (com_ptr, valueslin);

    if (strlen (tpa_ptr->onerror_labelc16) != 0)
    {
        (void) strcpy (lin_ptr, "errlab ");
        RESOLVE_LABEL (argblk_ptr,
                       tpa_ptr->onerror_labelc16,
                       tpa_ptr->onerror_lenl);
        APPEND_TOFLIN (com_ptr, NULL);
    }

    /*
     *  Reset "In Format Command" flag.
     */
    tpa_ptr->format_flagl = 0;

end:
    ;
    return PSL_NORMAL;
}

/******************************************************************************
 * @func    This Tparse action routine is called after parsing a GENERATE
 *      command. Its purpose is to append a Genvar record to the list of
 *      'TOF' lines. The Genvar record has the following format:
 *
 *      <PRE>
 *      Genvar
 *      mode TOF_GVRMOD_RANDOM
 *      name <variable>
 *      </PRE>
 *
 * @rdesc   Tparse status (Always returns PSL_NORMAL).
 * @parm    Pointer to Tparse argument block.
 */
unsigned long ENC_GENERATE (struct tpaarg *argblk_ptr)
{
    /*
     *    Local data
     */
    struct scl_com  *com_ptr;       /* SCL comon data */
    struct scl_tpa  *tpa_ptr;       /* SCL Tparse data */
    char   *lin_ptr;       /* Pointer to line buffer */

    /*
     *  Copy pointers for ease of reference.
     */
    com_ptr = (struct scl_com *) argblk_ptr->user_ptrp;
    tpa_ptr = com_ptr->tpa_ptrps;
    lin_ptr = com_ptr->toflinc1024; /* 'tof' line buffer pointer */

    /*
     *  Do not encode anything if we have had an error in the command.
     */
    if (tpa_ptr->error_in_commandn) goto end;

    /*
     *  Output tracing information if required.
     */
#ifdef PSL_TRACING
    if (PSL_TEST_TRACE (PTRC_USR_USER1))
    {
        PSL_OUTPUT_TRACE ("\nEncoding a GENERATE Command\n");
        PSL_OUTPUT_TRACE ("Operand Count = %ld\n" ,GET_OPD_COUNT (tpa_ptr));
    }
#endif

    /*
     *  Write the following records to the 'TOF' line list:
     *
     *  Genvar
     *  mode TOF_GVRMOD_RANDOM
     *  name <variable>
     */
    APPEND_RECHDR (com_ptr, "Genvar");

    (void) sprintf (lin_ptr, "mode %d", TOF_GVRMOD_RANDOM);
    APPEND_TOFLIN (com_ptr, NULL);

    (void) strcpy (lin_ptr, "name ");
    ENC_OPD_BYPOS (com_ptr, lin_ptr, NULL, NULL);
    APPEND_TOFLIN (com_ptr, NULL);

end:
    ;
    return PSL_NORMAL;
}

/******************************************************************************
 * @func    This Tparse action routine is called after parsing a GOTO
 *      command. Its purpose is to append a Goto record to the list of
 *      'TOF' lines. This record has the following format:
 *
 *      <PRE>
 *      Goto
 *      golab <label>
 *      </PRE>
 *
 * @rdesc   Tparse status (Always returns PSL_NORMAL).
 * @parm    Pointer to Tparse argument block.
 */
unsigned long ENC_GOTO (struct tpaarg *argblk_ptr)
{
    /*
     *  Local data.
     */
    struct scl_com  *com_ptr;       /* SCL comon data */
    struct scl_tpa  *tpa_ptr;       /* SCL Tparse data */
    char   *lin_ptr;       /* Pointer to line buffer */

    /*
     *  Copy pointers for ease of reference.
     */
    com_ptr = (struct scl_com *) argblk_ptr->user_ptrp;
    tpa_ptr = com_ptr->tpa_ptrps;
    lin_ptr = com_ptr->toflinc1024; /* 'tof' line buffer pointer */

    /*
     *  Do not encode anything if we have had an error in the command.
     */
    if (tpa_ptr->error_in_commandn) goto end;

    /*
     *  Output tracing information if required.
     */
#ifdef PSL_TRACING
    if (PSL_TEST_TRACE (PTRC_USR_USER1))
    {
        PSL_OUTPUT_TRACE ("\nEncoding a GOTO Command\n");
        PSL_OUTPUT_TRACE ("Operand Count = %ld\n" ,GET_OPD_COUNT (tpa_ptr));
    }
#endif

    /*
     *  Write the following records to the 'TOF' line list:
     *
     *  Goto
     *  golab <label>
     */
    APPEND_RECHDR (com_ptr, "Goto");

    (void) strcpy (lin_ptr, "golab ");
    RESOLVE_LABEL (argblk_ptr, tpa_ptr->label_namec16, tpa_ptr->label_lenl);
    APPEND_TOFLIN (com_ptr, NULL);

end:
    ;
    return PSL_NORMAL;
}

/******************************************************************************
 * @func    This Tparse action routine is called after parsing an HTTP command.
 *      Its purpose is to append a Request record for the HTTP request
 *      to the list of 'TOF' lines. The Request records have the following
 *      format:
 *
 *      <PRE>
 *      Request
 *      type <request_type>
 *      options <options_mask>
 *      uri <uri_httpversion>
 *      conid <conid>
 *      header <ltxt_delimiter>
 *      <http_header><ltxt_delimiter>
 *      {body <ltxt_delimiter>
 *      <http_body><ltxt_delimiter>}    // POST request only
 *      {with <ltxt_delimiter>
 *      <header_values><ltxt_delimiter>}
 *      {wout <ltxt_delimiter>
 *      <header_fields><ltxt_delimiter>}    // Without
 *      {retcod <return_code_variable>}
 *      {retsts <return_status_variable>}
 *      {timer <timer_name>}
 *  *** {timeout <timeout_period>}      //*** Not currently supported
 *  *** {tmolab <timeout_label>}        //*** Not currently supported
 *      </PRE>
 *
 * @rdesc   Tparse status (Always returns PSL_NORMAL).
 * @parm    Pointer to Tparse argument block.
 */
unsigned long ENC_HTTP_REQUEST (struct tpaarg *argblk_ptr)
{
    /*
     *  Local data.
     */
    struct scl_com  *com_ptr;       /* SCL common data */
    struct scl_tpa  *tpa_ptr;       /* SCL Tparse data */
    unsigned long   reqopt = 0;     /* Request options mask */
    char   *lin_ptr;       /* Pointer to line buffer */
    char   uri [TOFBUFLEN];    /* Uri_httpversion */
    char   conid [TOFBUFLEN];  /* Conid */
    long   isub;           /* Loop control */
    /*
     *  Create a static buffer to hold long text field values in
     *  encoded format. We do not know in advance how big the value is
     *  going to be so we allocate a buffer of the maximum size. As this
     *  is BIG, we do not want to allocate and deallocate it every time
     *  this routine is entered so we make it static.
     */
    static   char   linbuf [CHRBUFSIZ]; /* Line buffer */
    long   linlen;         /* Line length */

    /*
     *  Copy pointers for ease of reference.
     */
    com_ptr = (struct scl_com *) argblk_ptr->user_ptrp;
    tpa_ptr = com_ptr->tpa_ptrps;
    lin_ptr = com_ptr->toflinc1024; /* 'tof' line buffer pointer */

    /*
     *  Do not encode anything if we have had an error in the command.
     */
    if (tpa_ptr->error_in_commandn) goto end;

    /*
     *  Output tracing information if required.
     */
#ifdef PSL_TRACING
    if (PSL_TEST_TRACE (PTRC_USR_USER1))
    {
        PSL_OUTPUT_TRACE ("Encoding an HTTP Request Command\n");
        PSL_OUTPUT_TRACE ("Operand Count = %ld\n" ,GET_OPD_COUNT (tpa_ptr));
    }
#endif

    /* Get request options */
    if (((tpa_ptr->http_reqtypl == TOF_REQTYP_POST)     /*If POST request */
            &&  (tpa_ptr->cmnd_opt_specifiedl & CMND_OPT_BODY) != 0))   /* and {BINARY} BODY specified */
    {
        if ((tpa_ptr->cmnd_optl & CMND_OPT_BODY) != 0)  /* If BINARY BODY specified */
        {
            reqopt |= TOF_REQOPT_BINBOD;        /* Set 'binary body' option bit */
        }
    }

    if ((tpa_ptr->http_flagsl & HTTPFLG_PRIMARY) != 0)  /* If PRIMARY specified */
    {
        reqopt |= TOF_REQOPT_PRIMARY;           /* Set 'primary' option bit */
    }

    /* Get uri_httpversion */
    uri [0] = '\0';
    ENC_EXPR_BYPOS (com_ptr, uri);

    /* Get connection id */
    conid [0] = '\0';
    ENC_EXPR_BYPOS (com_ptr, conid);

    /*
     *  Write the following records to the 'TOF' line list:
     *
     *  Request
     *  type <request_type>
     *  options <options_mask>
     *  uri <uri_httpversion>
     *  conid <conid>
     *  header <ltxt_delimiter>
     *  <http_header><ltxt_delimiter>
     *  {body <ltxt_delimiter>
     *  <http_body><ltxt_delimiter>}    // POST request only
     *  {with <ltxt_delimiter>
     *  <header_values><ltxt_delimiter>}
     *  {wout <ltxt_delimiter>
     *  <header_fields><ltxt_delimiter>}    // Without
     *  {retcod <return_code_variable>}
     *  {retsts <return_status_variable>}
     *  {timer <timer_name>}
     *  {timeout <timeout_period>}      //*** Not currently supported
     *  {tmolab <timeout_label>}        //*** Not currently supported
     */
    APPEND_RECHDR (com_ptr, "Request");

    (void) sprintf (lin_ptr, "type %d", tpa_ptr->http_reqtypl);
    APPEND_TOFLIN (com_ptr, NULL);

    (void) sprintf (lin_ptr, "options %d", reqopt);
    APPEND_TOFLIN (com_ptr, NULL);

    (void) sprintf (lin_ptr, "uri %s", uri);
    APPEND_TOFLIN (com_ptr, NULL);

    (void) sprintf (lin_ptr, "conid %s", conid);
    APPEND_TOFLIN (com_ptr, NULL);

    linbuf[0] = '\0';
    for (isub = tpa_ptr->http_hdrval_cntl; isub > 0; isub--)
    {
        ENC_EXPR_BYPOS (com_ptr, linbuf);
    }
    APPEND_LTXT (com_ptr, "header", linbuf);

    if ((tpa_ptr->http_reqtypl == TOF_REQTYP_POST)      /* If POST request */
            &&  ((tpa_ptr->cmnd_opt_specifiedl & CMND_OPT_BODY) != 0))  /* & {BINARY} BODY specified */
    {
        linbuf[0] = '\0';
        ENC_EXPR_BYID (com_ptr, linbuf, CMND_OPT_BODY);
        if (strlen (linbuf) != 0) APPEND_LTXT (com_ptr, "body", linbuf);
    }

    if ((tpa_ptr->cmnd_optl & CMND_OPT_WITH) != 0)
    {
        linbuf[0] = '\0';
        for (isub = tpa_ptr->http_withval_cntl; isub > 0; isub--)
        {
            ENC_EXPR_BYID (com_ptr, linbuf, CMND_OPT_WITH);
        }
        if (strlen (linbuf) != 0) APPEND_LTXT (com_ptr, "with", linbuf);
    }

    if ((tpa_ptr->cmnd_optl & CMND_OPT_WITHOUT) != 0)
    {
        linbuf[0] = '\0';
        for (isub = tpa_ptr->http_woutval_cntl; isub > 0; isub--)
        {
            ENC_EXPR_BYID (com_ptr, linbuf, CMND_OPT_WITHOUT);
        }
        if (strlen (linbuf) != 0) APPEND_LTXT (com_ptr, "wout", linbuf);
    }

    if ((tpa_ptr->cmnd_optl & CMND_OPT_RETCOD) != 0)
    {
        (void) strcpy (lin_ptr, "retcod ");
        ENC_OPD_BYID (com_ptr, lin_ptr, CMND_OPT_RETCOD);
        APPEND_TOFLIN (com_ptr, NULL);
    }

    if ((tpa_ptr->cmnd_optl & CMND_OPT_RETURNING) != 0)
    {
        (void) strcpy (lin_ptr, "retsts ");
        ENC_OPD_BYID (com_ptr, lin_ptr, CMND_OPT_RETURNING);
        APPEND_TOFLIN (com_ptr, NULL);
    }

    if ((tpa_ptr->cmnd_optl & CMND_OPT_RESPONSE) != 0)
    {
        (void) strcpy (lin_ptr, "timer ");
        linlen = strlen (lin_ptr);
        ENC_OPD_BYID (com_ptr, lin_ptr, CMND_OPT_RESPONSE);
        if (strlen (lin_ptr) != linlen) APPEND_TOFLIN (com_ptr, NULL);
    }

    /***    if ((tpa_ptr->cmnd_optl & CMND_OPT_WAIT_PRD) != 0)
     ***    {
     ***    (void) strcpy (lin_ptr, "timeout ");
     ***    linlen = strlen (lin_ptr);
     ***    ENC_OPD_BYID (com_ptr, lin_ptr, CMND_OPT_WAIT_PRD);
     ***    if (strlen (lin_ptr) != linlen) APPEND_TOFLIN (com_ptr, NULL);
     ***    }
     ***
     ***    if ((tpa_ptr->cmnd_optl & CMND_OPT_TIMEOUT) != 0)
     ***    {
     ***    (void) sprintf (lin_ptr, "tmolab %s", tpa_ptr->ontimeout_labelc20);
     ***    APPEND_TOFLIN (com_ptr, lin_ptr);
     ***    }
     ***/
end:
    ;
    return PSL_NORMAL;          /* Return success */
}

/******************************************************************************
 * @func    This Tparse action routine is called after parsing an IF ... GOTO
 *      command. Its purpose is to append a Conditional record to the list
 *      of 'TOF' lines. The Conditional record has the following format:
 *
 *      <PRE>
 *      Conditional
 *      expr <condition>
 *      name <label>
 *      </PRE>
 *
 * @rdesc   Tparse status (Always returns PSL_NORMAL).
 * @parm    Pointer to Tparse argument block.
 */
unsigned long ENC_IF_GOTO (struct tpaarg *argblk_ptr)
{
    /*
     *  Local data
     */
    struct scl_com  *com_ptr;       /* SCL common data */
    struct scl_tpa  *tpa_ptr;       /* SCL Tparse data */
    char   *lin_ptr;       /* Pointer to line buffer */

    /*
     *  Copy pointers for ease of reference.
     */
    com_ptr = (struct scl_com *) argblk_ptr->user_ptrp;
    tpa_ptr = com_ptr->tpa_ptrps;
    lin_ptr = com_ptr->toflinc1024; /* 'tof' line buffer pointer */

    /*
     *  Do not encode anything if we have had an error in the command.
     */
    if (tpa_ptr->error_in_commandn) goto end;

    /*
     *  Output tracing information if required.
     */
#ifdef PSL_TRACING
    if (PSL_TEST_TRACE (PTRC_USR_USER1))
    {
        PSL_OUTPUT_TRACE ("Encoding an IF GOTO Command\n");
        PSL_OUTPUT_TRACE ("Operand Count = %ld\n" ,GET_OPD_COUNT (tpa_ptr));
    }
#endif
    /*
     *  Write the following records to the 'TOF' line list:
     *
     *  Conditional
     *  expr <condition>
     *  name <label>
     */
    APPEND_RECHDR (com_ptr, "Conditional");

    /*  Flush the 'expr <condition>' line from the default 'TOF' line buffer */
    APPEND_TOFLIN (com_ptr, NULL);

    (void) strcpy (lin_ptr, "name ");
    RESOLVE_LABEL (argblk_ptr, tpa_ptr->label_namec16, tpa_ptr->label_lenl);
    APPEND_TOFLIN (com_ptr, NULL);

end:
    ;
    return PSL_NORMAL;
}

/******************************************************************************
 * @func    This Tparse action routine is called after parsing an IF ... THEN
 *      command. Its purpose is to append an IfThen record to the list
 *      of 'TOF' lines. The Conditional record has the following format:
 *
 *      <PRE>
 *      IfThen
 *      expr <condition>
 *      </PRE>
 *
 * @rdesc   Tparse status (Always returns PSL_NORMAL).
 * @parm    Pointer to Tparse argument block.
 */
unsigned long ENC_IF_THEN (struct tpaarg *argblk_ptr)
{
    /*
     *  Local data
     */
    struct scl_com  *com_ptr;       /* SCL common data */
    struct scl_tpa  *tpa_ptr;       /* SCL Tparse data */
    char   *lin_ptr;       /* Pointer to line buffer */

    /*
     *  Copy pointers for ease of reference.
     */
    com_ptr = (struct scl_com *) argblk_ptr->user_ptrp;
    tpa_ptr = com_ptr->tpa_ptrps;
    lin_ptr = com_ptr->toflinc1024; /* 'tof' line buffer pointer */

    /*
     *  Do not encode anything if we have had an error in the command.
     */
    if (tpa_ptr->error_in_commandn) goto end;

    /*
     *  Output tracing information if required.
     */
#ifdef PSL_TRACING
    if (PSL_TEST_TRACE (PTRC_USR_USER1))
    {
        PSL_OUTPUT_TRACE ("Encoding an IF THEN Command\n");
        PSL_OUTPUT_TRACE ("Operand Count = %ld\n" ,GET_OPD_COUNT (tpa_ptr));
    }
#endif

    /*
     *  Write the following records to the 'TOF' line list:
     *
     *  IfThen
     *  expr <condition>
     */
    APPEND_RECHDR (com_ptr, "IfThen");

    /*  Flush the 'expr <condition>' line from the default 'TOF' line buffer */
    APPEND_TOFLIN (com_ptr, NULL);

    /*
     *    Increment the IF Stack index to point to the next FREE entry. We have
     *    already ensured that there is sufficient space.
     */
    tpa_ptr->ifstack_levell += 1;

end:
    ;
    return PSL_NORMAL;
}

/******************************************************************************
 * @func    This routine is a Tparse action routine and is called to encode the
 *      'tof' lines required at the start of the Setvar record. The
 *      following records are appended to teh 'tof' line list:
 *
 *      <PRE>
 *      Setvar
 *      </PRE>
 *
 * @rdesc   Tparse status (Always returns PSL_NORMAL).
 * @parm    Pointer to Tparse argument block.
 */
unsigned long ENC_INI_SET (struct tpaarg *argblk_ptr)
{
    /*
     *  Local data.
     */
    struct scl_com  *com_ptr;       /* SCL common data */
    struct scl_tpa  *tpa_ptr;       /* SCL Tparse data */

    /*
     *  Copy pointers for ease of reference.
     */
    com_ptr = (struct scl_com *) argblk_ptr->user_ptrp;
    tpa_ptr = com_ptr->tpa_ptrps;

    /*
     *  Do not encode anything if we have had an error in the command.
     */
    if (tpa_ptr->error_in_commandn) goto end;

    /*
     *  Output tracing information if required.
     */
#ifdef PSL_TRACING
    if (PSL_TEST_TRACE (PTRC_USR_USER1))
    {
        PSL_OUTPUT_TRACE ("\nEncoding a Setvar record\n");
        PSL_OUTPUT_TRACE ("Operand Count = %ld\n" ,GET_OPD_COUNT (tpa_ptr));
    }
#endif

    /*
     *  Write the following records to the 'TOF' line list:
     *
     *  Setvar
     */
    APPEND_RECHDR (com_ptr, "Setvar");

end:
    ;
    return PSL_NORMAL;          /* Return success */
}

/******************************************************************************
 * @func    This Tparse action routine is called after parsing a label
 *      definition. Its purpose is to append a Label record to the list of
 *      'TOF' lines. The Label record has the following format:
 *
 *      <PRE>
 *      Label
 *      name <label>
 *      </PRE>
 *
 * @rdesc   Tparse status (Always returns PSL_NORMAL).
 * @parm    Pointer to Tparse argument block.
 */
void ENC_LABEL (struct tpaarg *argblk_ptr,
                char *label_ptr,
                unsigned long label_len)
{
    /*
     *  Local data.
     */
    struct scl_com  *com_ptr;       /* SCL common data */
    struct scl_tpa  *tpa_ptr;       /* SCL Tparse data */
    char   *lin_ptr;       /* Pointer to line buffer */
    char   labnam [TOFBUFLEN]; /* Mutex name */

    /*
     *  Copy pointers for ease of reference.
     */
    com_ptr = (struct scl_com *) argblk_ptr->user_ptrp;
    tpa_ptr = com_ptr->tpa_ptrps;
    lin_ptr = com_ptr->toflinc1024; /* 'tof' line buffer pointer */

    /*
     *  Do not encode anything if we have had an error in the command.
     */
    if (tpa_ptr->error_in_commandn) goto end;

    /*
     *  Output tracing information if required.
     */
#ifdef PSL_TRACING
    if (PSL_TEST_TRACE (PTRC_USR_USER1))
    {
        PSL_OUTPUT_TRACE ("Encoding a LABEL definition\n");
        PSL_OUTPUT_TRACE ("Operand Count = %ld\n" ,GET_OPD_COUNT (tpa_ptr));
    }
#endif
    /*
     *  Write the following records to the 'TOF' line list:
     *
     *  Label
     *  name <label>
     */
    APPEND_RECHDR (com_ptr, "Label");

    (void) memcpy (labnam, label_ptr, label_len);
    labnam [label_len] = '\0';
    (void) sprintf (lin_ptr, "name %s", labnam);
    APPEND_TOFLIN (com_ptr, NULL);

end:
    ;
    return;
}

/******************************************************************************
 * @func    This Tparse action routine is called after parsing a LOAD command.
 *      Its purpose is to append a Load record to the list of 'TOF' lines.
 *      The Load record has the following format:
 *
 *      <PRE>
 *      Load
 *      data TOF_LODDAT_*
 *      {from <timer>}      // Only used by load timer
 *      into <variable>
 *      </PRE>
 *
 *      The data code to be written is passed in the parameter field of the
 *      Tparse argument block (argblk_ptr->paraml).
 *
 * @rdesc   Tparse status (Always returns PSL_NORMAL).
 * @parm    Pointer to Tparse argument block.
 */
unsigned long ENC_LOAD (struct tpaarg *argblk_ptr)
{
    /*
     *    Local data
     */
    struct scl_com  *com_ptr;       /* SCL comon data */
    struct scl_tpa  *tpa_ptr;       /* SCL Tparse data */
    char   *lin_ptr;       /* Pointer to line buffer */

    /*
     *  Copy pointers for ease of reference.
     */
    com_ptr = (struct scl_com *) argblk_ptr->user_ptrp;
    tpa_ptr = com_ptr->tpa_ptrps;
    lin_ptr = com_ptr->toflinc1024; /* 'tof' line buffer pointer */

    /*
     *  Do not encode anything if we have had an error in the command.
     */
    if (tpa_ptr->error_in_commandn) goto end;

    /*
     *  Output tracing information if required.
     */
#ifdef PSL_TRACING
    if (PSL_TEST_TRACE (PTRC_USR_USER1))
    {
        PSL_OUTPUT_TRACE ("\nEncoding a LOAD Command\n");
        PSL_OUTPUT_TRACE ("Operand Count = %ld\n" ,GET_OPD_COUNT (tpa_ptr));
    }
#endif

    /*
     *  Write the following records to the 'TOF' line list:
     *
     *  Load
     *  data TOF_LODDAT_*
     *  {from <timer>}      // Only used by load timer
     *  into <variable>
     */
    APPEND_RECHDR (com_ptr, "Load");

    (void) sprintf (lin_ptr, "data %d", tpa_ptr->loddatl);
    APPEND_TOFLIN (com_ptr, NULL);

    if (tpa_ptr->loddatl == TOF_LODDAT_TIMER)
    {
        (void) strcpy (lin_ptr, "from ");
        ENC_OPD_BYPOS (com_ptr, lin_ptr, NULL, NULL);
        APPEND_TOFLIN (com_ptr, NULL);
    }

    (void) strcpy (lin_ptr, "into ");
    ENC_OPD_BYPOS (com_ptr, lin_ptr, NULL, NULL);
    APPEND_TOFLIN (com_ptr, NULL);

end:
    ;
    return PSL_NORMAL;
}

/******************************************************************************
 * @func    This Tparse action routine is called after parsing an HTTP-specific
 *      LOAD command. Its purpose is to append an Lload record to the list
 *      of 'TOF' lines. The Lload record has the following format:
 *
 *      <PRE>
 *      Lload
 *      data TOF_LLDDAT_*
 *      {conid <conid>}     // Only used by load response_info
 *      into <variable>
 *      {identifier <identifier>    // Only used by load response_info
 *      </PRE>
 *
 *      The data code to be written is passed in the parameter field of the
 *      Tparse argument block (argblk_ptr->paraml).
 *
 * @rdesc   Tparse status (Always returns PSL_NORMAL).
 * @parm    Pointer to Tparse argument block.
 */
unsigned long ENC_LOAD_HTTP (struct tpaarg *argblk_ptr)
{
    /*
     *    Local data
     */
    struct scl_com  *com_ptr;       /* SCL comon data */
    struct scl_tpa  *tpa_ptr;       /* SCL Tparse data */
    long linlen;         /* Line length */
    char   *lin_ptr;       /* Pointer to line buffer */

    /*
     *  Copy pointers for ease of reference.
     */
    com_ptr = (struct scl_com *) argblk_ptr->user_ptrp;
    tpa_ptr = com_ptr->tpa_ptrps;
    lin_ptr = com_ptr->toflinc1024; /* 'tof' line buffer pointer */

    /*
     *  Do not encode anything if we have had an error in the command.
     */
    if (tpa_ptr->error_in_commandn) goto end;

    /*
     *  Output tracing information if required.
     */
#ifdef PSL_TRACING
    if (PSL_TEST_TRACE (PTRC_USR_USER1))
    {
        PSL_OUTPUT_TRACE ("\nEncoding a LOAD Command\n");
        PSL_OUTPUT_TRACE ("Operand Count = %ld\n" ,GET_OPD_COUNT (tpa_ptr));
    }
#endif

    /*
     *  Write the following records to the 'TOF' line list:
     *
     *  Lload
     *  data TOF_LLDDAT_*
     *  {conid <conid>}     // Only used by load response_info
     *  into <variable>
     *  {identifier <identifier>    // Only used by load response_info
     */
    APPEND_RECHDR (com_ptr, "Lload");

    (void) sprintf (lin_ptr, "data %d", tpa_ptr->loddatl);
    APPEND_TOFLIN (com_ptr, NULL);

    if ((tpa_ptr->loddatl == TOF_LLDDAT_RSPHDR)
            ||  (tpa_ptr->loddatl == TOF_LLDDAT_RSPBDY))
    {
        (void) strcpy (lin_ptr, "conid ");
        ENC_EXPR_BYPOS (com_ptr, lin_ptr);
        APPEND_TOFLIN (com_ptr, NULL);
    }

    (void) strcpy (lin_ptr, "into ");
    ENC_OPD_BYPOS (com_ptr, lin_ptr, NULL, NULL);
    APPEND_TOFLIN (com_ptr, NULL);

    if ((tpa_ptr->loddatl == TOF_LLDDAT_RSPHDR)
            ||  (tpa_ptr->loddatl == TOF_LLDDAT_RSPBDY))
    {
        (void) strcpy (lin_ptr, "identifier ");
        linlen = strlen (lin_ptr);
        ENC_EXPR_BYID (com_ptr, lin_ptr, CMND_OPT_WITH);
        if (strlen (lin_ptr) != linlen) APPEND_TOFLIN (com_ptr, NULL);
    }

end:
    ;
    return PSL_NORMAL;
}

/******************************************************************************
 * @func    This Tparse action routine is called after parsing an SCL logging
 *      command, i.e. a LOG, HISTORY, NOTE, REPORT and TRACE command. Its
 *      purpose is to append a Logmsg record to the list of 'TOF' lines
 *      The Logmsg record has the following format:
 *
 *      <PRE>
 *      Logmsg
 *      destination <destination_code>
 *      lmsg <ltxt_delimiter>
 *      <lmsg_data><ltxt_delimiter>
 *      </PRE>
 *
 * @rdesc   Tparse status (Always returns PSL_NORMAL).
 * @parm    Pointer to Tparse argument block.
 */
unsigned long ENC_LOG (struct tpaarg *argblk_ptr)
{
    /*
     *  Local data.
     */
    struct scl_com   *com_ptr;      /* SCL common data */
    struct scl_tpa   *tpa_ptr;      /* SCL Tparse data */
    long    logdst;        /* TOF log destination code */
    char    dstlin [TOFBUFLEN];    /* Destination line buffer */
    char    *lin_ptr;      /* Pointer to line buffer */

    /*
     *  Copy pointers for ease of reference.
     */
    com_ptr = (struct scl_com *) argblk_ptr->user_ptrp;
    tpa_ptr = com_ptr->tpa_ptrps;
    lin_ptr = com_ptr->toflinc1024; /* 'tof' line buffer pointer */

    /*
     *  Do not encode anything if we have had an error in the command.
     */
    if (tpa_ptr->error_in_commandn) goto end;

    /*
     *  Output tracing information if required.
     */
#ifdef PSL_TRACING
    if (PSL_TEST_TRACE (PTRC_USR_USER1))
    {
        PSL_OUTPUT_TRACE ("Encoding a LOG Command\n");
        PSL_OUTPUT_TRACE ("Operand Count = %ld\n" ,GET_OPD_COUNT (tpa_ptr));
    }
#endif

    /*
     *  Determine the Logmsg destination code.
     */
    switch (tpa_ptr->logdstl)
    {
    case LOG_AUDIT:         /* Log to local audit log */
        {
            logdst = TOF_LOGDST_AUDIT;
            break;
        }
    case LOG_CENTRAL:           /* Log to central audit log */
        {
            logdst = TOF_LOGDST_CENTRAL;
            break;
        }
    case LOG_HISTORY:           /* Log to history log */
        {
            logdst = TOF_LOGDST_HISTORY;
            break;
        }
    case LOG_NOTE:          /* Log to note buffer */
        {
            logdst = TOF_LOGDST_NOTE;
            break;
        }
    case LOG_REPORT:            /* Log to report log */
        {
            logdst = TOF_LOGDST_REPORT;
            break;
        }
    case LOG_TRACE:             /* Log to trace log */
        {
            logdst = TOF_LOGDST_TRACE;
            break;
        }
    }

    /*
     *  Write the following records to the 'TOF' line list:
     *
     *  Logmsg
     *  destination <destination_code>
     *  lmsg <ltxt_delimiter>
     *  <lmsg_data><ltxt_delimiter>
     */
    APPEND_RECHDR (com_ptr, "Logmsg");

    (void) sprintf (dstlin, "destination %u", logdst);
    APPEND_TOFLIN (com_ptr, dstlin);

    APPEND_LTXT (com_ptr, "lmsg", lin_ptr);

end:
    ;
    return PSL_NORMAL;          /* Return success */
}

/******************************************************************************
 * @func    This Tparse action routine is called after parsing an operand in a
 *      'log' command. Its purpose is to add an entry for the operand to
 *      the TOF line buffer. The 'log' command does NOT encode operands in
 *      the standard way - for compatibility with earlier TOF releases the
 *      'log' command encodes the 'log' lmsg field as a 'segmented string'.
 *      In such fields a string is unquoted and variable references are
 *      delimited by the variable delimiter character. This routine calls
 *      the standard operand encoding routine and modifies the returned
 *      value locally as required before writing it to the default TOF
 *      line buffer. This is not ideal as we are making assumptions about
 *      the format of the values returned to us, which is undesirable.
 *
 * @rdesc   Tparse status (Always returns PSL_NORMAL).
 * @parm    Pointer to Tparse argument block.
 */
unsigned long ENC_LOG_OPD (struct tpaarg *argblk_ptr)
{
    /*
     *  Local data.
     */
    struct scl_com  *com_ptr;       /* SCL common data */
    struct scl_tpa  *tpa_ptr;       /* SCL Tparse data */
    struct oph  *oph_ptr;       /* Pointer to perand header */
    unsigned long   iret;           /* Status return variable */
    PSL_U_LONGWORD_T
    opd_addrl;      /* Operand addressing mode */
    PSL_U_LONGWORD_T
    opd_dtypl;      /* Operand data type */
    char   opdbuf [TOFBUFLEN] = "\0";
    /* Encoded operand buffer */
    char   *lin_ptr;       /* Pointer to line buffer */

    /*
     *  Copy pointers for ease of reference.
     */
    com_ptr = (struct scl_com *) argblk_ptr->user_ptrp;
    tpa_ptr = com_ptr->tpa_ptrps;
    lin_ptr = com_ptr->toflinc1024; /* 'tof' line buffer pointer */

    /*
     *  Do not encode anything if we have had an error in the command.
     */
    if (tpa_ptr->error_in_commandn) goto end;

    /*
     *  Output tracing information if required.
     */
#ifdef PSL_TRACING
    if (PSL_TEST_TRACE (PTRC_USR_USER1))
    {
        PSL_OUTPUT_TRACE ("\nEncoding a LOG Operand\n");
        PSL_OUTPUT_TRACE ("Operand Count = %ld\n" ,GET_OPD_COUNT (tpa_ptr));
    }
#endif

    /*
     *  Get the operand header and save the operand addressing mode and data type, as
     *  we may need to encode this operand in a non-standard way within this routine
     *  dedpending upon the settings of these attributes.
     */
    iret = PSL_VM_COPY (tpa_ptr->parsed_operand_llps ,(char *) &oph_ptr,
                        NULL ,NULL ,PSL_VM_MODE_FIRST);
    /* Copy the OPH pointer */
    if  (!(psl_test_status (iret))) /* Report any errors */
    {
        PSL_SIGDIAG (tpa_ptr->diag_ptrps, 2, PSL_SYSERVERR,
                     "PSL_VM_COPY (in ENC_LOG_OPD Copy OPH)", iret);
    }

    opd_addrl = oph_ptr->addrl;     /* Save operand addressing mode */
    opd_dtypl = oph_ptr->dtypl;     /* Save operand data type */

    /*
     *  Encode the operand into a local buffer.
     */
    ENC_OPD_BYPOS (com_ptr, opdbuf, NULL, NULL);

    /*
     *  If this is an immediate character operand remove the quotes before encoding,
     *  else if this is a variable operand enclose the operand with the variable
     *  delimiter character, otherwise encode the operand unchanged.
     */
    if ((opd_addrl == ADDR_IMMEDIATE) &&    /* Immediate character operand */
            (opd_dtypl == DTYP_CHARACTER))
    {
        opdbuf [strlen(opdbuf)-1] = '\0';   /* Remove quotes */
        (void) strcat (lin_ptr, &opdbuf [1]);   /* ... */
    }
    else if (((opd_addrl == ADDR_LOCAL) ||  /* Variable operand */
              (opd_addrl == ADDR_THREAD) ||
              (opd_addrl == ADDR_GLOBAL) ||
              (opd_addrl == ADDR_TESTWIDE) ||
              (opd_addrl == ADDR_SCRIPT)))
    {
        (void) strcat (lin_ptr, VNIDELIM);  /* Delimit variable reference */
        (void) strcat (lin_ptr, opdbuf);    /* ... */
        (void) strcat (lin_ptr, VNIDELIM);  /* ... */
    }
    else                    /* All other operands */
    {
        (void) strcat (lin_ptr, &opdbuf[0]);    /* Operand unchanged */
    }
end:
    ;
    return PSL_NORMAL;          /* Return success */
}

/******************************************************************************
 * @func    This Tparse action routine is called after parsing a MAP (file)
 *      command. Its purpose is to append a Fcntl record to the list of
 *      'TOF' lines. The Fcntl record has the following format:
 *
 *      <PRE>
 *      Fcntl
 *      operation TOF_FCNOP_MAP
 *      filid <file_id>
 *      drecord <data_record>
 *      {hrecord <header_record>}
 *      {errlab <error_label>}
 *      </PRE>
 *
 *      Note that if this is a passed file, the index is incremented by
 *      the constant PASSED_FILE_BASE, to show that it is a passed file.
 *
 * @rdesc   Tparse status (Always returns PSL_NORMAL).
 * @parm    Pointer to Tparse argument block.
 */
unsigned long ENC_MAP_FILE (struct tpaarg *argblk_ptr)
{
    /*
     *  Local data.
     */
    struct scl_com  *com_ptr;       /* SCL common data */
    struct scl_tpa  *tpa_ptr;       /* SCL Tparse data */
    struct fattbl   *fat_ptr;       /* File Access Table pointer */
    char   *lin_ptr;       /* Pointer to line buffer */
    char   filidlin [TOFBUFLEN];   /* File Id line */
    char   dreclin [TOFBUFLEN];    /* Drecord line */
    char   hreclin [TOFBUFLEN];    /* Hrecord line */

    /*
     *  Copy pointers for ease of reference.
     */
    com_ptr = (struct scl_com *) argblk_ptr->user_ptrp;
    tpa_ptr = com_ptr->tpa_ptrps;
    lin_ptr = com_ptr->toflinc1024; /* 'tof' line buffer pointer */
    fat_ptr = tpa_ptr->fat_begps + tpa_ptr->open_fididxl - 1;
    /* Point to current entry in table */
    /*
     *  Do not encode anything if we have had an error in the command.
     */
    if (tpa_ptr->error_in_commandn) goto end;

    /*
     *  Output tracing information if required.
     */
#ifdef PSL_TRACING
    if (PSL_TEST_TRACE (PTRC_USR_USER1))
    {
        PSL_OUTPUT_TRACE ("Encoding a MAP Command\n");
        PSL_OUTPUT_TRACE ("Operand Count = %ld\n" ,GET_OPD_COUNT (tpa_ptr));
    }
#endif

    /*
     *    Check to see if this file has been DEFINED (by the FILE statement).
     *    If it has then we set the referenced flag to ACCESSED_FILE
     *
     *    ...otherwise...
     *
     *    If the file is a passed file, we need to show this by incrementing its
     *    index into the passed file index range.
     */
    if (fat_ptr->fat_refflgl == DEFINED_FILE)
    {
        fat_ptr->fat_refflgl = ACCESSED_FILE;
    }
    else if (fat_ptr->fat_refflgl == PASSED_FILE)
    {
        tpa_ptr->open_fididxl += PASSED_FILE_BASE;
    }

    /*
     *  Get file ID and create 'filid' sub-record.
     */
    (void) sprintf (filidlin, "filid %s", fat_ptr->fat_namec20);

    /*
     *  Get data record name and create 'drecord' sub-record.
     */
    (void) strcpy (dreclin, "drecord ");
    ENC_OPD_BYPOS (com_ptr, dreclin, NULL, NULL);

    /*
     *  Get variable that defines the layout of the header record (this may or may
     *  not have been specified) and create 'hrecord' sub-record.
     */
    if ((tpa_ptr->cmnd_optl & CMND_OPT_MAP_HEADER) != 0)
    {
        (void) strcpy (hreclin, "hrecord ");
        ENC_OPD_BYPOS (com_ptr, hreclin, NULL, NULL);
    }
    /*
     *  Write the following records to the 'TOF' line list:
     *
     *  Fcntl
     *  operation TOF_FCNOP_MAP
     *  filid <file_id>
     *  drecord <data_record>
     *  {hrecord <header_record>}
     *  {errlab <error_label>}
     */
    APPEND_RECHDR (com_ptr, "Fcntl");

    (void) sprintf (lin_ptr, "operation %d", TOF_FCNOP_MAP);
    APPEND_TOFLIN (com_ptr, NULL);

    APPEND_TOFLIN (com_ptr, filidlin);

    APPEND_TOFLIN (com_ptr, dreclin);

    if ((tpa_ptr->cmnd_optl & CMND_OPT_MAP_HEADER) != 0)
    {
        APPEND_TOFLIN (com_ptr, hreclin);
    }

    if (strlen (tpa_ptr->onerror_labelc16) != 0)
    {
        (void) strcpy (lin_ptr, "errlab ");
        RESOLVE_LABEL (argblk_ptr,
                       tpa_ptr->onerror_labelc16,
                       tpa_ptr->onerror_lenl);
        APPEND_TOFLIN (com_ptr, NULL);
    }

end:
    ;
    return PSL_NORMAL;          /* Return success */
}

/******************************************************************************
 * @func    This Tparse action routine is called after parsing a NEXT command.
 *      Its purpose is to append a Genvar record to the list of 'TOF' lines.
 *      The Genvar record has the following format:
 *
 *      <PRE>
 *      Genvar
 *      mode TOF_GVRMOD_SEQUENTIAL
 *      name <variable>
 *      </PRE>
 *
 * @rdesc   Tparse status (Always returns PSL_NORMAL).
 * @parm    Pointer to Tparse argument block.
 */
unsigned long ENC_NEXT (struct tpaarg *argblk_ptr)
{
    /*
     *    Local data
     */
    struct scl_com  *com_ptr;       /* SCL comon data */
    struct scl_tpa  *tpa_ptr;       /* SCL Tparse data */
    char   *lin_ptr;       /* Pointer to line buffer */

    /*
     *  Copy pointers for ease of reference.
     */
    com_ptr = (struct scl_com *) argblk_ptr->user_ptrp;
    tpa_ptr = com_ptr->tpa_ptrps;
    lin_ptr = com_ptr->toflinc1024; /* 'tof' line buffer pointer */

    /*
     *  Do not encode anything if we have had an error in the command.
     */
    if (tpa_ptr->error_in_commandn) goto end;

    /*
     *  Output tracing information if required.
     */
#ifdef PSL_TRACING
    if (PSL_TEST_TRACE (PTRC_USR_USER1))
    {
        PSL_OUTPUT_TRACE ("\nEncoding a NEXT Command\n");
        PSL_OUTPUT_TRACE ("Operand Count = %ld\n" ,GET_OPD_COUNT (tpa_ptr));
    }
#endif

    /*
     *  Write the following records to the 'TOF' line list:
     *
     *  Genvar
     *  mode TOF_GVRMOD_SEQUENTIAL
     *  name <variable>
     */
    APPEND_RECHDR (com_ptr, "Genvar");

    (void) sprintf (lin_ptr, "mode %d", TOF_GVRMOD_SEQUENTIAL);
    APPEND_TOFLIN (com_ptr, NULL);

    (void) strcpy (lin_ptr, "name ");
    ENC_OPD_BYPOS (com_ptr, lin_ptr, NULL, NULL);
    APPEND_TOFLIN (com_ptr, NULL);

end:
    ;
    return PSL_NORMAL;
}

/******************************************************************************
 * @func    This Tparse action routine is called after parsing a ON ERROR GOTO
 *      command. Its purpose is to append a Goerror record to the list of
 *      'TOF' lines. This record has the following format:
 *
 *      <PRE>
 *      Goerror
 *      errlab <label>
 *      </PRE>
 *
 * @rdesc   Tparse status (Always returns PSL_NORMAL).
 * @parm    Pointer to Tparse argument block.
 */
unsigned long ENC_ON_ERROR (struct tpaarg *argblk_ptr)
{
    /*
     *  Local data.
     */
    struct scl_com  *com_ptr;       /* SCL comon data */
    struct scl_tpa  *tpa_ptr;       /* SCL Tparse data */
    char   *lin_ptr;       /* Pointer to line buffer */

    /*
     *  Copy pointers for ease of reference.
     */
    com_ptr = (struct scl_com *) argblk_ptr->user_ptrp;
    tpa_ptr = com_ptr->tpa_ptrps;
    lin_ptr = com_ptr->toflinc1024; /* 'tof' line buffer pointer */

    /*
     *  Do not encode anything if we have had an error in the command.
     */
    if (tpa_ptr->error_in_commandn) goto end;

    /*
     *  Output tracing information if required.
     */
#ifdef PSL_TRACING
    if (PSL_TEST_TRACE (PTRC_USR_USER1))
    {
        PSL_OUTPUT_TRACE ("\nEncoding an ON ERROR GOTO Command\n");
        PSL_OUTPUT_TRACE ("Operand Count = %ld\n" ,GET_OPD_COUNT (tpa_ptr));
    }
#endif

    /*
     *  Write the following records to the 'TOF' line list:
     *
     *  Goerror
     *  errlab <label>
     */
    APPEND_RECHDR (com_ptr, "Goerror");

    (void) strcpy (lin_ptr, "errlab ");
    RESOLVE_LABEL (argblk_ptr, tpa_ptr->onerror_labelc16, tpa_ptr->onerror_lenl);
    APPEND_TOFLIN (com_ptr, NULL);

end:
    ;
    return PSL_NORMAL;
}


/******************************************************************************
 * @func    This routine encodes an immediate character operand at the current
 *      position in the 'TOF' line buffer from. The operand header and
 *      operand descriptor describing the operand to be encoded are
 *      supplied as parameters. An immediate character operand is
 *      delimited by single or double quotes, but is supplied to us without
 *      quotes. We add a leading and trailing double quote character,
 *      unless the operand contains one or more double quotes, in which
 *      case we add a leading and trailing single quote.
 *
 * @parm    Pointer to SCL common data
 * @parm    Pointer to buffer into which the operand is to be encoded.
 * @parm    Boolean flag indicating whether (PTRUE) or not (PFALSE) to
 *      delete operand.
 * @parm    Pointer to operand descriptor for operand to be encoded.
 * @parm    New llb_pos returned in *ret_llb_pos_pptr if not NULL.
 */
static void ENC_OPD_CHRIMM (struct scl_com *com_ptr,
                            char *lin_ptr,
                            struct oph *oph_ptr,
                            struct opd *opd_ptr)
{
    /*
     *  Create a static buffer to hold the immediate character value in
     *  encoded format. We do not know in advance how big the value is
     *  going to be so we allocate a buffer of the maximum size. As this
     *  is BIG, we do not want to allocate and deallocate it every time
     *  this routine is entered so we make it static.
     */
    static  char   encval [CHRBUFSIZ];      /* Encoded value buffer */

    /*
     *  Call a support routine to create an immediate character value in encoded format
     *  from an immediate character value string as held in an operand descriptor.
     */
    CREATE_ENCCHRIMM (com_ptr, encval, sizeof(encval),
                      opd_ptr->uopd.imc.val_pc, oph_ptr->sizel);

    /*
     *  Add the value to the default 'TOF' line buffer.
     */
    (void) strcat (lin_ptr, encval);

    return;
}

/******************************************************************************
 * @func    This routine encodes a function operand and appends it to the
 *      default 'TOF' line buffer. The operand header and operand
 *      descriptor describing the operand to be encoded are supplied
 *      as parameters. It encodes both integer and character function
 *      references.
 *
 *      The function operand supplied to this routine will be followed - in
 *      the parsed operand linked list - by each of the function parameters
 *      delimited by a NULL operand, e.g. for LOCATE:
 *
 *      <PRE>
 *      OPD1: LOCATE Function Operand
 *      OPD2: Substring Operand, e.g. "Usernname: "
 *      OPD3: NULL
 *      OPD4: String Operand, e.g. CUSTOMER_DATA
 *      OPD5: NULL
 *      </PRE>
 *
 *      The above would be encoded as:
 *
 *      ~LOCATE ("Username: ",CUSTOMER_DATA)
 *
 *      This routine is responsible for establishing and writting the
 *      function name, writting the parameter list delimiters and parameter
 *      separators and encoding the parameter operands.
 *
 * @parm    Pointer to SCL common data
 * @parm    Pointer to buffer into which the operand is to be encoded.
 * @parm    Pointer to operand header for operand to be encoded.
 * @parm    Pointer to operand descriptor for operand to be encoded.
 * @parm    Pointer to linked list context/position for current operand
 *      descriptor.
 */
static void ENC_OPD_FUNC (struct scl_com *com_ptr,
                          char *lin_ptr,
                          struct oph *oph_ptr,
                          struct opd *opd_ptr,
                          char *llb_opdpos_ptr)
{
    /*
     *  Local data.
     */
    struct scl_tpa  *tpa_ptr;       /* SCL Tparse data */
    struct oph  *tmpoph_ptr;        /* Pointer to temporary operand header */
    char   *llb_ctx_ptr;       /* Returned linked list context */
    char   *llb_pos_ptr;       /* Input linked list position */
    unsigned long   iret;           /* Status return variable */
    Boolean     prmend;         /* End of parameter flag */
    long   isub;           /* Loop control */
    long   prmcnt;         /* Function parameter count*/
    char   mode=PSL_VM_MODE_NEXT;  /* PSL Linked list mode */

    /*
     *  Copy pointers for ease of reference.
     */
    tpa_ptr = com_ptr->tpa_ptrps;
    llb_pos_ptr = llb_opdpos_ptr;   /* Pointer to current operand descriptor */

    /*
     *  Add the function name to the line buffer.
     */
    (void) strcat (lin_ptr, TOF_FNCNAM[opd_ptr->uopd.fnc.idl]);

    /*
     *  If there are any parameters, encode them into the line buffer.
     */
    prmcnt = opd_ptr->uopd.fnc.prmcntl;

    if (prmcnt > 0)
    {
        (void) strcat (lin_ptr, TOF_SPRMDELIM);

        for (isub = 1; isub <= prmcnt; isub++)
        {
            for (prmend = PFALSE; prmend == PFALSE;)
            {
                iret = PSL_VM_COPY (tpa_ptr->parsed_operand_llps ,(char *) &tmpoph_ptr
                                    ,llb_pos_ptr ,&llb_ctx_ptr ,mode);
                /* Copy the OPH */
                if  (!(psl_test_status (iret))) /* Report any errors */
                {
                    PSL_SIGDIAG (tpa_ptr->diag_ptrps, 2, PSL_SYSERVERR
                                 ,"PSL_VM_COPY (in ENC_OPD_FUNC Remove OPH)", iret);
                }

                if (tmpoph_ptr->addrl == ADDR_NONE)    /* If end of parameter */
                {
                    if (isub < prmcnt)
                    {
                        (void) strcat (lin_ptr, TOF_PRMSEP);
                    }
                    prmend = PTRUE;
                }

                /* Encode operand. Pass in the position of current operand header and */
                /* get returned the position of the operand header following the */
                /* encoded operand (may be made up of multiple operands). */
                llb_pos_ptr = llb_ctx_ptr;  /* Pointer to current OPH */
                ENC_OPD_BYPOS (com_ptr, lin_ptr, llb_pos_ptr, &llb_ctx_ptr);
                mode = PSL_VM_MODE_SPEC;    /* Reset mode to pick up specified element */
                llb_pos_ptr = llb_ctx_ptr;  /* Pointer to next OPH */

            }
        }

        (void) strcat (lin_ptr, TOF_EPRMDELIM);
    }

    /*
     *  Encode function operand options.
     */
    ENC_OPD_OPTS (com_ptr, lin_ptr, oph_ptr, opd_ptr);

    return;
}


/******************************************************************************
 * @func    This routine encodes an _HTTP_CODE operand and appends it to the
 *      default 'TOF' line buffer. The operand header and operand
 *      descriptor describing the operand to be encoded are supplied
 *      as parameters.
 *
 * @parm    Pointer to SCL common data
 * @parm    Pointer to buffer into which the operand is to be encoded.
 * @parm    Pointer to operand header for operand to be encoded.
 * @parm    Pointer to operand descriptor for operand to be encoded.
 */
static void ENC_OPD_HTTP_CODE (struct scl_com *com_ptr,
                               char *lin_ptr,
                               struct oph *oph_ptr,
                               struct opd *opd_ptr)
{
    /*
     *  Local data.
     */
    struct scl_tpa  *tpa_ptr;       /* SCL Tparse data */

    /*
     *  Copy pointers for ease of reference.
     */
    tpa_ptr = com_ptr->tpa_ptrps;

    /*
     *  Add _HTTP_CODE[conid] to the line buffer.
     */
    (void) strcat (lin_ptr, "_HTTP_CODE");
    (void) strcat (lin_ptr, TOF_SMODDELIM);

    ENC_OPD_BYPOS (com_ptr, lin_ptr, NULL, NULL);

    (void) strcat (lin_ptr, TOF_EMODDELIM);

    return;
}


/******************************************************************************
 * @func    This routine encodes an _HTTP_STATUS operand and appends it to the
 *      default 'TOF' line buffer. The operand header and operand
 *      descriptor describing the operand to be encoded are supplied
 *      as parameters.
 *
 * @parm    Pointer to SCL common data
 * @parm    Pointer to buffer into which the operand is to be encoded.
 * @parm    Pointer to operand header for operand to be encoded.
 * @parm    Pointer to operand descriptor for operand to be encoded.
 */
static void ENC_OPD_HTTP_STATUS (struct scl_com *com_ptr,
                                 char *lin_ptr,
                                 struct oph *oph_ptr,
                                 struct opd *opd_ptr)
{
    /*
     *  Local data.
     */
    struct scl_tpa  *tpa_ptr;       /* SCL Tparse data */

    /*
     *  Copy pointers for ease of reference.
     */
    tpa_ptr = com_ptr->tpa_ptrps;

    /*
     *  Add _HTTP_STATUS[conid] to the line buffer.
     */
    (void) strcat (lin_ptr, "_HTTP_STATUS");
    (void) strcat (lin_ptr, TOF_SMODDELIM);

    ENC_OPD_BYPOS (com_ptr, lin_ptr, NULL, NULL);

    (void) strcat (lin_ptr, TOF_EMODDELIM);

    return;
}

/******************************************************************************
 * @func    This routine encodes an immediate integer operand at the current
 *      position in the 'TOF' line buffer from. The operand header and
 *      operand descriptor describing the operand to be encoded are
 *      supplied as parameters.
 *
 * @parm    Pointer to SCL common data
 * @parm    Pointer to buffer into which the operand is to be encoded.
 * @parm    Boolean flag indicating whether (PTRUE) or not (PFALSE) to
 *      delete operand.
 * @parm    Pointer to operand descriptor for operand to be encoded.
 * @parm    New llb_pos returned in *ret_llb_pos_pptr if not NULL.
 */
static void ENC_OPD_INTIMM (struct scl_com *com_ptr,
                            char *lin_ptr,
                            struct oph *oph_ptr,
                            struct opd *opd_ptr)
{
    /*
     *  Local data.
     */
    char   intval [TOFBUFLEN]; /* Integer value buffer */

    sprintf (intval, "%d", opd_ptr->uopd.imi.vall);
    (void) strcat (lin_ptr, intval);

    return;
}

/******************************************************************************
 * @func    This routine encodes an operator operand at the current
 *      position in the 'TOF' line buffer from. The operand header and
 *      operand descriptor describing the operand to be encoded are
 *      supplied as parameters.
 *
 *      Binary operators have a single leading and trailing space. This is
 *      not only to produce neat output, but also to correctly delimit
 *      operands, that may otherwise be ambiguous, e.g.
 *
 *      <PRE>
 *      isub-1 - jsub
 *      </PRE>
 *
 *      Here the value of variable jsub is subtracted from variable
 *      isub-1. If there were no whitespace, it would not be possible to
 *      determine whether the expression contains one (isub-1-jsub), two
 *      (isub-1 and jsub) or three (isub, 1 and jsub) operands.
 *
 * @parm    Pointer to SCL common data
 * @parm    Pointer to buffer into which the operand is to be encoded.
 * @parm    Boolean flag indicating whether (PTRUE) or not (PFALSE) to
 *      delete operand.
 * @parm    Pointer to operand descriptor for operand to be encoded.
 * @parm    New llb_pos returned in *ret_llb_pos_pptr if not NULL.
 */
static void ENC_OPD_OPR (struct scl_com *com_ptr,
                         char *lin_ptr,
                         struct oph *oph_ptr,
                         struct opd *opd_ptr)
{
    /*
     *  Local data.
     */
    char       oprbuf [TOFBUFLEN]; /* Operator buffer */

    switch (opd_ptr->uopd.opr.operatorl)
    {
    case OPR_EQ:
        (void) sprintf (oprbuf, " %s ", TOF_OPR_EQ);
        break;
    case OPR_NE:
        (void) sprintf (oprbuf, " %s ", TOF_OPR_NE);
        break;
    case OPR_LE:
        (void) sprintf (oprbuf, " %s ", TOF_OPR_LE);
        break;
    case OPR_GE:
        (void) sprintf (oprbuf, " %s ", TOF_OPR_GE);
        break;
    case OPR_LT:
        (void) sprintf (oprbuf, " %s ", TOF_OPR_LT);
        break;
    case OPR_GT:
        (void) sprintf (oprbuf, " %s ", TOF_OPR_GT);
        break;
    case OPR_CONTAINS:
        (void) sprintf (oprbuf, " %s ", TOF_OPR_CONTAINS);
        break;
    case OPR_NOT_CONTAINS:
        (void) sprintf (oprbuf, " %s ", TOF_OPR_NOT_CONTAINS);
        break;
    case OPR_ADD:
        (void) sprintf (oprbuf, " %s ", TOF_OPR_ADD);
        break;
    case OPR_SUBTRACT:
        (void) sprintf (oprbuf, " %s ", TOF_OPR_SUBTRACT);
        break;
    case OPR_AND:
        (void) sprintf (oprbuf, " %s ", TOF_OPR_AND);
        break;
    case OPR_OR:
        (void) sprintf (oprbuf, " %s ", TOF_OPR_OR);
        break;
    case OPR_MULTIPLY:
        (void) sprintf (oprbuf, " %s ", TOF_OPR_MULTIPLY);
        break;
    case OPR_DIVIDE:
        (void) sprintf (oprbuf, " %s ", TOF_OPR_DIVIDE);
        break;
    case OPR_MODULO:
        (void) sprintf (oprbuf, " %s ", TOF_OPR_MODULO);
        break;
    case OPR_MATCHES:
        (void) sprintf (oprbuf, " %s ", TOF_OPR_MATCHES);
        break;
    case OPR_NOT_MATCHES:
        (void) sprintf (oprbuf, " %s ", TOF_OPR_NOT_MATCHES);
        break;
    case OPR_NOT:
        (void) strcpy (oprbuf, TOF_OPR_NOT);
        break;
    case OPR_SEXPR:
        (void) strcpy (oprbuf, TOF_OPR_SEXPR);
        break;
    case OPR_EEXPR:
        (void) strcpy (oprbuf, TOF_OPR_EEXPR);
        break;
    case OPR_VALSEP:
        (void) strcpy (oprbuf, TOF_OPR_VALSEP);
        break;
    default:
        oprbuf[0] = '\0';   // Do nothing
    }


    (void) strcat (lin_ptr, oprbuf);

    return;
}

/******************************************************************************
 * @func    This routine encodes any operand options associated with the
 *      supplied operand parameters and appends them to the specified line
 *      buffer.
 *
 * @parm    Pointer to SCL common data
 * @parm    Pointer to buffer into which the operand is to be encoded.
 * @parm    Pointer to operand header.
 * @parm    Pointer to operand descriptor.
 */
static void ENC_OPD_OPTS (struct scl_com *com_ptr,
                          char *lin_ptr,
                          struct oph *oph_ptr,
                          struct opd *opd_ptr)
{
    /*
     *  Local data.
     */
    struct scl_tpa  *tpa_ptr;       /* SCL Tparse data */
    char optstr [TOFBUFLEN]; /* Option string buffer */

    /*
     *  Copy pointers for ease of reference.
     */
    tpa_ptr = com_ptr->tpa_ptrps;

    /*
     *  Encode trailing space option, if any.
     */
    if ((oph_ptr->optl & VOPT_ONE_TRAIL) != 0)
    {
        (void) strcat (lin_ptr, ", One Trailing");
    }
    else if ((oph_ptr->optl & VOPT_TRAILING) != 0)
    {
        (void) strcat (lin_ptr, ", Trailing");
    }

    /*
     *  Encode case blind option, if any.
     */
    if ((oph_ptr->optl & VOPT_CASEBLIND) != 0)
    {
        (void) sprintf (optstr, ", %s", TOF_OPTSTR_CASEBLIND);
        (void) strcat (lin_ptr, optstr);
    }

    return;
}

/******************************************************************************
 * @func    This routine encodes the operand - identified by the supplied
 *      operand header and operand descriptor records - into the 'TOF' line
 *      buffer identified in the call.
 *
 * @parm    Pointer to SCL common data
 * @parm    Pointer to buffer into which the operand is to be encoded.
 * @parm    Pointer to operand header for operand to be encoded.
 * @parm    Pointer to operand descriptor for operand to be encoded.
 * @parm    Pointer to linked list context/position for current operand
 *      descriptor.
 */
static void ENC_OPD_THIS (struct scl_com *com_ptr,
                          char *lin_ptr,
                          struct oph *oph_ptr,
                          struct opd *opd_ptr,
                          char *llb_opdpos_ptr)
{
    /*
     *  What is written to the 'TOF' line buffer depends upon what type of
     *  operand we have. So now determine what type of operand we have and
     *  encode the operand as appropriate.
     */
    if (oph_ptr->addrl == ADDR_TIMER)           /* Timer */
    {
        ENC_OPD_TIM (com_ptr, lin_ptr, oph_ptr, opd_ptr);
    }

    else if (oph_ptr->addrl == ADDR_OPERATOR)       /* Operator */
    {
        ENC_OPD_OPR (com_ptr, lin_ptr, oph_ptr, opd_ptr);
    }

    else if ((oph_ptr->addrl == ADDR_IMMEDIATE) &&  /* Immediate integer */
             (oph_ptr->dtypl == DTYP_INTEGER))
    {
        ENC_OPD_INTIMM (com_ptr, lin_ptr, oph_ptr, opd_ptr);
    }

    else if ((oph_ptr->addrl == ADDR_IMMEDIATE) &&  /* Immediate character */
             (oph_ptr->dtypl == DTYP_CHARACTER))
    {
        ENC_OPD_CHRIMM (com_ptr, lin_ptr, oph_ptr, opd_ptr);
    }

    else if ((oph_ptr->addrl == ADDR_LOCAL) ||      /* Variable */
             (oph_ptr->addrl == ADDR_THREAD) ||
             (oph_ptr->addrl == ADDR_GLOBAL) ||
             (oph_ptr->addrl == ADDR_TESTWIDE) ||
             (oph_ptr->addrl == ADDR_SCRIPT))
    {
        ENC_OPD_VAR (com_ptr, lin_ptr, oph_ptr, opd_ptr, llb_opdpos_ptr);
    }

    else if (oph_ptr->addrl == ADDR_TST)        /* Test */
    {
        ENC_OPD_TST (com_ptr, lin_ptr, oph_ptr, opd_ptr);
    }

    else if (oph_ptr->addrl == ADDR_TSTTHR)     /* Test thread */
    {
        ENC_OPD_TSTTHR (com_ptr, lin_ptr, oph_ptr, opd_ptr);
    }

    else if (oph_ptr->addrl == ADDR_FUNCTION)
    {
        ENC_OPD_FUNC (com_ptr, lin_ptr, oph_ptr, opd_ptr, llb_opdpos_ptr);
    }
    /***    else if (oph_ptr->addrl == ADDR_HTTP_CODE)
     *   {
     *  ENC_OPD_HTTP_CODE (com_ptr, lin_ptr, oph_ptr, opd_ptr);
     *  }
     *
     *  else if (oph_ptr->addrl == ADDR_HTTP_STATUS)
     *  {
     *  ENC_OPD_HTTP_STATUS (com_ptr, lin_ptr, oph_ptr, opd_ptr);
     *  }
     ***/

    return;
}

/******************************************************************************
 * @func    This routine is a Tparse action routine and is called to encode the
 *      first entry in the parsed operands linked list into the default
 *      'TOF' line buffer. The operand is removed from the parsed operands
 *      linked list by this routine.
 *
 * @rdesc   Tparse status (Always returns PSL_NORMAL).
 * @parm    Pointer to Tparse argument block.
 */
unsigned long ENC_OPD_TPA (struct tpaarg *argblk_ptr)
{
    /*
     *  Local data.
     */
    struct scl_com   *com_ptr;      /* SCL comon ata */
    struct scl_tpa   *tpa_ptr;      /* SCL Tparse data */

    /*
     *  Copy pointers for ease of reference.
     */
    com_ptr = (struct scl_com *) argblk_ptr->user_ptrp;
    tpa_ptr = com_ptr->tpa_ptrps;

    /*
     *  Do not encode anything if we have had an error in the command.
     */
    if (tpa_ptr->error_in_commandn) goto end;

    /*
     *  Output tracing information if required.
     */
#ifdef PSL_TRACING
    if (PSL_TEST_TRACE (PTRC_USR_USER1))
    {
        PSL_OUTPUT_TRACE ("\nEncoding an Operand using ENC_OPD_TPA\n");
        PSL_OUTPUT_TRACE ("Operand Count = %ld\n" ,GET_OPD_COUNT (tpa_ptr));
    }
#endif

    /*
     *  Encode the first operand in the parsed operands linked list at the
     *  end of the default 'TOF' line buffer.
     */
    ENC_OPD_BYPOS (com_ptr, NULL, NULL, NULL);

end:
    ;
    return PSL_NORMAL;          /* Return success */
}

/******************************************************************************
 * @func    This routine encodes a test thread operand and appends it to the
 *      supplied 'TOF' line buffer. The operand header and operand
 *      descriptor describing the operand to be encoded are supplied
 *      as parameters. A test thread operand consists simply of the thread
 *      name.
 *
 * @parm    Pointer to SCL common data
 * @parm    Pointer to buffer into which the operand is to be encoded.
 * @parm    Pointer to operand header for operand to be encoded.
 * @parm    Pointer to operand descriptor for operand to be encoded.
 */
static void ENC_OPD_TSTTHR (struct scl_com *com_ptr,
                            char *lin_ptr,
                            struct oph *oph_ptr,
                            struct opd *opd_ptr)
{
    /*
     *  Local data.
     */
    struct tstthrtbl    *tstthrtbl_ptr; /* Test thread symbol table pointer */
    char       tstthrnam [TOFBUFLEN];
    /* Test thread name */
    /*
     *  Get the test thread name from the test threads symbol table.
     */
    tstthrtbl_ptr = opd_ptr->uopd.tstthr.tstthrtbl_ptrps;

    (void) strcpy (tstthrnam, tstthrtbl_ptr->tstthr_namec20);

    /*
     *  Add the test thread operand to the line buffer.
     */
    (void) strcat (lin_ptr, tstthrnam);

    return;
}

/******************************************************************************
 * @func    This routine encodes a Timer operand at the current position in the
 *      'TOF' line buffer from. The operand header and operand descriptor
 *      describing the time operand to be encoded are supplied as
 *      parameters.
 *
 * @parm    Pointer to SCL common data.
 * @parm    Pointer to buffer into which the operand is to be encoded.
 * @parm    Pointer to operand header for operand to be encoded.
 * @parm    Pointer to operand descriptor for operand to be encoded.
 */
static void ENC_OPD_TIM (struct scl_com *com_ptr,
                         char *lin_ptr,
                         struct oph *oph_ptr,
                         struct opd *opd_ptr)
{
    /*
     *  Local data.
     */
    struct timtbl   *timtbl_ptr;        /* Timer symbol table pointer */

    /*
     *  Get the timer name from the timers symbol table.
     */
    timtbl_ptr = opd_ptr->uopd.tim.timtbl_ptrps;
    (void) strcat (lin_ptr, timtbl_ptr->tim_namec20);

    return;
}

/******************************************************************************
 * @func    This routine encodes a test operand and appends it to the
 *      default 'TOF' line buffer. The operand header and operand
 *      descriptor describing the operand to be encoded are supplied
 *      as parameters. A test operand consists simply of the test name.
 *
 * @parm    Pointer to SCL common data
 * @parm    Pointer to buffer into which the operand is to be encoded.
 * @parm    Pointer to operand header for operand to be encoded.
 * @parm    Pointer to operand descriptor for operand to be encoded.
 */
static void ENC_OPD_TST (struct scl_com *com_ptr,
                         char *lin_ptr,
                         struct oph *oph_ptr,
                         struct opd *opd_ptr)
{
    /*
     *  Local data.
     */
    struct tsttbl   *tsttbl_ptr;        /* Test symbol table pointer */
    char   tstnam [TOFBUFLEN]; /* Test name */

    /*
     *  Get the test name from the tests symbol table.
     */
    tsttbl_ptr = opd_ptr->uopd.tst.tsttbl_ptrps;
    (void) strcpy (tstnam, tsttbl_ptr->tst_namec20);

    /*
     *  Add the test operand to the line buffer.
     */
    (void) strcat (lin_ptr, tstnam);

    return;
}

/******************************************************************************
 * @func    This routine encodes a variable operand and appends it to the
 *      default 'TOF' line buffer. The operand header and operand
 *      descriptor describing the operand to be encoded are supplied
 *      as parameters. All the operands associated with the supplied
 *      operand are removed from the Parsed Operands Linked list by
 *      this routine (i.e. including operands for variable subscripts).
 *
 * @parm    Pointer to SCL common data
 * @parm    Pointer to buffer into which the operand is to be encoded.
 * @parm    Pointer to operand header for operand to be encoded.
 * @parm    Pointer to operand descriptor for operand to be encoded.
 * @parm    Pointer to linked list context/position for current operand
 *      descriptor.
 */
static void ENC_OPD_VAR (struct scl_com *com_ptr,
                         char *lin_ptr,
                         struct oph *oph_ptr,
                         struct opd *opd_ptr,
                         char *llb_opdpos_ptr)
{
    /*
     *  Local data.
     */
    struct symtbl   *symtbl_ptr;        /* Variable symbol table pointer */
    struct scl_tpa  *tpa_ptr;       /* SCL Tparse data */
    struct oph  *tmpoph_ptr;        /* Pointer to temporary operand header */
    char   *llb_ctx_ptr;       /* Returned linked list context */
    char   *llb_pos_ptr;       /* Input linked list position */
    unsigned long   iret;           /* Status return variable */
    Boolean     idxend;         /* End of array subscript flag */
    long   isub;           /* Loop control */
    char   varnam [TOFBUFLEN]; /* Variable name */
    char   mode=PSL_VM_MODE_NEXT;  /* PSL Linked list mode */

    /*
     *  Copy pointers for ease of reference.
     */
    tpa_ptr = com_ptr->tpa_ptrps;
    symtbl_ptr = opd_ptr->uopd.var.symtbl_ptrps;
    llb_pos_ptr = llb_opdpos_ptr;   /* Pointer to current operand descriptor */
    /*
     *  Get the variable name from the variables symbol table and add it to the line buffer.
     */
    (void) memcpy (varnam, symtbl_ptr->var_namec16, symtbl_ptr->var_nlengthl);
    varnam [symtbl_ptr->var_nlengthl] = '\0';
    (void) strcat (lin_ptr, varnam);

    /*
     *  If this is an array variable, encode the array subscripts and add them to
     *  the line buffer.
     */
    if (symtbl_ptr->var_array_dimcntl > 0)
    {
        (void) strcat (lin_ptr, TOF_SIDXDELIM);

        for (isub = 1; isub <= symtbl_ptr->var_array_dimcntl; isub++)
        {
            for (idxend = PFALSE; idxend == PFALSE;)
            {
                iret = PSL_VM_COPY (tpa_ptr->parsed_operand_llps ,(char *) &tmpoph_ptr
                                    ,llb_pos_ptr ,&llb_ctx_ptr ,mode);
                /* Copy the OPH */
                if  (!(psl_test_status (iret))) /* Report any errors */
                {
                    PSL_SIGDIAG (tpa_ptr->diag_ptrps, 2, PSL_SYSERVERR
                                 ,"PSL_VM_COPY (in ENC_OPD_VAR Copy OPH)", iret);
                }

                if (tmpoph_ptr->addrl == ADDR_NONE)    /* If end of array subscript */
                {
                    if (isub < symtbl_ptr->var_array_dimcntl)
                    {
                        (void) strcat (lin_ptr, TOF_IDXSEP);
                    }
                    idxend = PTRUE;
                }
                /* Encode operand. Pass in the position of current operand header and */
                /* get returned the position of the operand header following the */
                /* encoded operand (may be made up of multiple operands). */
                llb_pos_ptr = llb_ctx_ptr;  /* Pointer to current OPH */
                ENC_OPD_BYPOS (com_ptr, lin_ptr, llb_pos_ptr, &llb_ctx_ptr);
                mode = PSL_VM_MODE_SPEC;    /* Reset mode to pick up specified element */
                llb_pos_ptr = llb_ctx_ptr;  /* Pointer to next OPH */
            }
        }

        (void) strcat (lin_ptr, TOF_EIDXDELIM);
    }

    /*
     *  Encode variable operand options.
     */
    ENC_OPD_OPTS (com_ptr, lin_ptr, oph_ptr, opd_ptr);

    return;
}

/******************************************************************************
 * @func    This is a generic routine for encoding an operand at the end of a
 *      'TOF' line buffer.
 *
 *      The operands to be encoded are held in the parsed operands linked
 *      list and are unconditionaly removed it by this routine.
 *
 *      This routine may be called recursively, in order to encode operands
 *      that require further operands to define themselves completely. For
 *      example, variable array operands will have operands describing each
 *      of the subscripts.
 *
 * @parm    Pointer to SCL common data.
 * @parm    Pointer to buffer into which the operand is to be encoded. If NULL,
 *      encode into the default 'TOF' line buffer.
 * @parm    ID of operand to be encoded.
 */
static void ENC_OPD_BYID (struct scl_com *com_ptr,
                          char *usrlin_ptr,
                          unsigned long req_id)
{
    /*
     *  Local data.
     */
    unsigned long   iret;           /* Status return variable */
    struct scl_tpa  *tpa_ptr;       /* SCL tparse data */
    struct oph  *oph_ptr;       /* Operand header pointer */
    struct opd  *opd_ptr;       /* Operand descriptor pointer */
    char   *llb_ctx_ptr;       /* Linked list context */
    char   *llb_pos_ptr;       /* Linked list context */
    char   *llb_ophpos_ptr;    /* Linked list context */
    char   *llb_opdpos_ptr;    /* Linked list context */
    char   mode;           /* Linked list mode */
    char   *lin_ptr;       /* Pointer to line buffer */

    /*
     *  Copy pointers for ease of reference.
     */
    tpa_ptr = com_ptr->tpa_ptrps;

    /*
     *  Do not encode anything if we have had an error in the command.
     */
    if (tpa_ptr->error_in_commandn) goto end;

    /*
     *  Output tracing information if required.
     */
#ifdef PSL_TRACING
    if (PSL_TEST_TRACE (PTRC_USR_USER1))
    {
        PSL_OUTPUT_TRACE("Encoding an OPERAND by ID (ID = %ld)\n",req_id);
        PSL_OUTPUT_TRACE("Operand Count = %ld\n",GET_OPD_COUNT (tpa_ptr));
    }
#endif

    /*
     *  Use the default line buffer, unless a buffer is supplied in the call.
     */
    if (usrlin_ptr == NULL)
    {
        lin_ptr = com_ptr->toflinc1024;
    }
    else
    {
        lin_ptr = usrlin_ptr;
    }

    /*
     *  Loop through the Parsed Operands linked list looking for one with the
     *  specified ID - exit loop and encode operand if found, otherwise drop out
     *  of loop and return.
     */
    for (mode = PSL_VM_MODE_FIRST, iret = PSL_NORMAL, llb_pos_ptr = NULL;
            iret != PSL_RECNOTFOU;
            llb_pos_ptr = llb_ctx_ptr, mode = PSL_VM_MODE_NEXT)
    {
        iret = PSL_VM_COPY (tpa_ptr->parsed_operand_llps ,(char *) &oph_ptr,
                            llb_pos_ptr ,&llb_ctx_ptr ,mode);
        if  (!(psl_test_status (iret)) && (iret != PSL_RECNOTFOU))
        {
            PSL_SIGDIAG (tpa_ptr->diag_ptrps, 2, PSL_SYSERVERR
                         ,"PSL_VM_COPY (in ENC_OPD_BYID Copy OPH)", iret);
        }

        else if (psl_test_status (iret))
        {
            llb_pos_ptr = llb_ctx_ptr;      /* Update position to next item */
            llb_ophpos_ptr = llb_pos_ptr;   /* Save OPH position */

            iret = PSL_VM_COPY (tpa_ptr->parsed_operand_llps, (char *) &opd_ptr,
                                llb_pos_ptr ,&llb_ctx_ptr, PSL_VM_MODE_NEXT);
            if  (!(psl_test_status (iret)) && (iret != PSL_RECNOTFOU))
            {
                PSL_SIGDIAG (tpa_ptr->diag_ptrps, 2, PSL_SYSERVERR,
                             "PSL_VM_COPY (in GET_OPD_BYID Copy OPD)", iret);
            }

            llb_opdpos_ptr = llb_ctx_ptr;   /* Save OPD position */

            if (oph_ptr->idl == req_id) goto opd_found;
        }
    }   /* End of for loop */

    /*
     *  Drop through here if the operand was NOT found - just return, as there
     *  is no operand to encode.
     */
    return;

    /*
     *  Come here if the operand was found.
     */
opd_found:
    ;
    /*
     *  Encode the operand into the line buffer.
     */
    ENC_OPD_THIS (com_ptr, lin_ptr, oph_ptr, opd_ptr, llb_opdpos_ptr);

    /*
     *  Remove the operand header and operand descriptor records from the
     *  Parsed Operands linked list and free associated memory.
     */
    iret = PSL_VM_REMOVE (tpa_ptr->parsed_operand_llps, (char *) &oph_ptr,
                          llb_ophpos_ptr, NULL, PSL_VM_MODE_SPEC);
    if  (!(psl_test_status (iret)))
    {
        PSL_SIGDIAG (tpa_ptr->diag_ptrps, 2, PSL_SYSERVERR,
                     "PSL_VM_REMOVE (in ENC_OPD_BYID Remove OPH)", iret);
    }

    iret = PSL_VM_REMOVE (tpa_ptr->parsed_operand_llps, (char *) &opd_ptr,
                          llb_opdpos_ptr, NULL, PSL_VM_POS_SPEC);
    if  (!(psl_test_status (iret)))
    {
        PSL_SIGDIAG (tpa_ptr->diag_ptrps, 2, PSL_SYSERVERR,
                     "PSL_VM_REMOVE (in ENC_OPD_BYID Remove OPD)", iret);
    }

    /* Free memory associated with the operand */
    FREE_OPH_OPD (oph_ptr, opd_ptr);

end:
    ;
    return;
}

/******************************************************************************
 * @func    This is a generic routine for encoding an operand from the Parsed
 *      Operands linked list at the end of a 'TOF' line buffer. The operand
 *      encoded is unconditionaly removed from the list by this routine.
 *
 *      This routine may be called recursively, in order that we may
 *      encode operands which require further operands to define
 *      themselves completely. For example, variable array operands will
 *      have operands describing each of the subscripts.
 *
 * @parm    Pointer to SCL common data
 * @parm    Pointer to buffer into which the operand is to be encoded. If
 *      NULL, encode into the default 'TOF' line buffer.
 * @parm    Pointer to the entry (OPH) for the operand to be encoded in the parsed
 *      operand linked list. NULL to encode the first operand in the list.
 * @parm    Pointer to a pointer into which will be returned the position of the
 *      operand header which follows the last removed operand (in the Parsed
 *      Operand Linked List). Nothing is returned if this parameter is set
 *      to NULL.
 */
static void ENC_OPD_BYPOS (struct scl_com *com_ptr,
                           char *usrlin_ptr,
                           char *llb_pos_ptr,
                           char **ret_llb_pos_pptr)
{
    /*
     *  Local data.
     */
    unsigned long   iret;           /* Status return variable */
    struct scl_tpa  *tpa_ptr;       /* SCL tparse data */
    struct   oph    *oph_ptr;       /* Operand header pointer */
    struct   opd    *opd_ptr;       /* Operand descriptor pointer */
    char   *llb_ophpos_ptr;    /* Linked list context */
    char   *llb_opdpos_ptr;    /* Linked list context */
    char   *llb_ctx_ptr;       /* Linked list context */
    char   *lin_ptr;       /* Pointer to target line buffer */
    char   mode;           /* PSL_VM_COPY mode */

    /*
     *  Copy pointers for ease of reference.
     */
    tpa_ptr = com_ptr->tpa_ptrps;

    /*
     *  Do not encode anything if we have had an error in the command.
     */
    if (tpa_ptr->error_in_commandn) goto end;

    /*
     *  Output tracing information if required.
     */
#ifdef PSL_TRACING
    if (PSL_TEST_TRACE (PTRC_USR_USER1))
    {
        PSL_OUTPUT_TRACE("Encoding an OPERAND by POSITION (pos = %ld)\n",
                         llb_pos_ptr);
        PSL_OUTPUT_TRACE("Operand Count = %ld\n",GET_OPD_COUNT (tpa_ptr));
    }
#endif

    /*
     *  Use the default line buffer, unless a buffer is supplied in the call.
     */
    if (usrlin_ptr == NULL)
    {
        lin_ptr = com_ptr->toflinc1024;
    }
    else
    {
        lin_ptr = usrlin_ptr;
    }

    /*
     *  Set up the copy mode. If the supplied 'position' is NULL we
     *  are at the begining of the parsed operand linked list and set the mode
     *  to pick up the FIRST entry, otherwise we set the mode to pick up the
     *  next entry (using the supplied position).
     */
    if (llb_pos_ptr == NULL)
    {
        mode = PSL_VM_MODE_FIRST;
    }
    else
    {
        mode = PSL_VM_MODE_SPEC;
    }

    /*
     *  Pick up the next OPH and OPD from the parsed operand linked list.
     *
     *  Although we want to remove entries from the list we ALWAYS copy the
     *  items out of the list here and remove them later. This is so that there
     *  is a consistent interface to ENC_OPD_THIS.
     *
     *  PSL_VM_COPY returns a pointer to the copied item which we use if this
     *  item is an array or requires further operands to qualify it.
     */
    iret = PSL_VM_COPY (tpa_ptr->parsed_operand_llps ,(char *) &oph_ptr
                        ,llb_pos_ptr ,&llb_ctx_ptr ,mode);
    /* Copy the OPH pointer */
    if  (!(psl_test_status (iret))) /* Report any errors */
    {
        PSL_SIGDIAG (tpa_ptr->diag_ptrps, 2, PSL_SYSERVERR
                     ,"PSL_VM_COPY (in ENC_OPD_BYPOS Copy OPH)", iret);
    }

    llb_ophpos_ptr = llb_ctx_ptr;   /* Save position of OPH */

    iret = PSL_VM_COPY (tpa_ptr->parsed_operand_llps ,(char *) &opd_ptr
                        ,llb_ophpos_ptr ,&llb_ctx_ptr ,PSL_VM_MODE_NEXT);
    /* Copy OPD pointer */
    if  (!(psl_test_status (iret))) /* Report any errors */
    {
        PSL_SIGDIAG (tpa_ptr->diag_ptrps, 2, PSL_SYSERVERR
                     ,"PSL_VM_COPY (in ENC_OPD_BYPOS Copy OPD)", iret);
    }

    llb_opdpos_ptr = llb_ctx_ptr;   /* Save position of OPD */

    /*
     *  Encode the operand.
     */
    ENC_OPD_THIS (com_ptr, lin_ptr, oph_ptr, opd_ptr, llb_opdpos_ptr);

    /*
     *  Remove the operand header and operand descriptor records from the Parsed
     *  Operands linked list and free associated memory.
     */
    iret = PSL_VM_REMOVE (tpa_ptr->parsed_operand_llps, (char *) &oph_ptr,
                          llb_ophpos_ptr, NULL, PSL_VM_MODE_SPEC);
    if  (!(psl_test_status (iret)))
    {
        PSL_SIGDIAG (tpa_ptr->diag_ptrps, 2, PSL_SYSERVERR,
                     "PSL_VM_REMOVE (in ENC_OPD_BYPOS Remove OPH)", iret);
    }

    iret = PSL_VM_REMOVE (tpa_ptr->parsed_operand_llps, (char *) &opd_ptr,
                          llb_opdpos_ptr, &llb_ctx_ptr, PSL_VM_POS_SPEC);
    if  (!(psl_test_status (iret)))
    {
        PSL_SIGDIAG (tpa_ptr->diag_ptrps, 2, PSL_SYSERVERR,
                     "PSL_VM_REMOVE (in ENC_OPD_BYPOS Remove OPD)", iret);
    }

    /* Free memory associated with the operand */
    FREE_OPH_OPD (oph_ptr, opd_ptr);

    /*
     *  If the call specified a return position pointer return the position
     *  of the item which follows the last removed one - this will be an OPH.
     */
end:
    ;
    if (ret_llb_pos_pptr != NULL) *ret_llb_pos_pptr = llb_ctx_ptr;

    return;
}

/******************************************************************************
 * @func    This Tparse action routine is called after parsing an OPEN FILE
 *      command. Its purpose is to append a Open record to the list of
 *      'TOF' lines. The Open record has the following format:
 *
 *      <PRE>
 *      Open
 *      name <filename>
 *      filid <file_id>
 *      access <access_code>
 *      options <options_mask>
 *      {errlab <error_label>}
 *      </PRE>
 *
 * @rdesc   Tparse status (Always returns PSL_NORMAL).
 * @parm    Pointer to Tparse argument block.
 */
unsigned long ENC_OPEN_FILE (struct tpaarg *argblk_ptr)
{
    /*
     *  Local data.
     */
    struct scl_com  *com_ptr;       /* SCL common data */
    struct scl_tpa  *tpa_ptr;       /* SCL Tparse data */
    struct fattbl   *fat_ptr;       /* File Access Table pointer */
    unsigned long   filacc;         /* File access type */
    unsigned long   filopt = 0;     /* File open options */
    char   *lin_ptr;       /* Pointer to line buffer */
    char   filnamlin [TOFBUFLEN];  /* Filename line */
    char   filidlin [TOFBUFLEN];   /* File Id line */

    /*
     *  Copy pointers for ease of reference.
     */
    com_ptr = (struct scl_com *) argblk_ptr->user_ptrp;
    tpa_ptr = com_ptr->tpa_ptrps;
    lin_ptr = com_ptr->toflinc1024; /* 'tof' line buffer pointer */
    fat_ptr = tpa_ptr->fat_begps + tpa_ptr->open_fididxl - 1;
    /* Point to current entry in table */
    /*
     *  Do not encode anything if we have had an error in the command.
     */
    if (tpa_ptr->error_in_commandn) goto end;

    /*
     *  Output tracing information if required.
     */
#ifdef PSL_TRACING
    if (PSL_TEST_TRACE (PTRC_USR_USER1))
    {
        PSL_OUTPUT_TRACE ("Encoding an OPEN FILE Command\n");
        PSL_OUTPUT_TRACE ("Operand Count = %ld\n" ,GET_OPD_COUNT (tpa_ptr));
    }
#endif

    /*
     *  If the file has been passed as a parameter, report an error and return.
     */
    if ((fat_ptr->fat_refflgl & PASSED_FILE) == 0)
    {                   /* ...if file passed as parameter */
        REPORT_ERROR (argblk_ptr, SCL_FILPRMNOOPEN);
        goto end;
    }

    /*
     *  Get file name and create 'name' sub-record.
     */
    (void) strcpy (filnamlin, "name ");
    ENC_OPD_BYPOS (com_ptr, filnamlin, NULL, NULL);

    /*
     *  Get file ID and create 'filid' sub-record.
     */
    (void) sprintf (filidlin, "filid %s", fat_ptr->fat_namec20);

    /*
     *  Get file access type and create 'access' sub-record.
     */
    if ((tpa_ptr->open_maskl & OPEN_OUTPUT) != 0)
    {
        filacc = TOF_FILACC_OUTPUT;
    }
    else if ((tpa_ptr->open_maskl & OPEN_INPUT) != 0)
    {
        filacc = TOF_FILACC_INPUT;
    }
    else if ((tpa_ptr->open_maskl & OPEN_APPEND) != 0)
    {
        filacc = TOF_FILACC_APPEND;
    }
    else
    {
        filacc = TOF_FILACC_INPUT;
    }

    /*
     *  Get the options value.
     */
    if ((tpa_ptr->cmnd_optl & OPEN_BINFILE) != 0)
    {
        filopt |= TOF_FILOPT_BINFIL;
    }

    if ((tpa_ptr->cmnd_optl & OPEN_COMMASEP) != 0)
    {
        filopt |= TOF_FILOPT_COMSEP;
    }

    if ((tpa_ptr->cmnd_optl & OPEN_SHARED) != 0)
    {
        filopt |= TOF_FILOPT_SHARED;
    }

    /*
     *  Write the following records to the 'TOF' line list:
     *
     *  Open
     *  name <filename>
     *  filid <file_id>
     *  access <access_code>
     *  options <options_mask>
     *  {errlab <error_label>}
     */
    APPEND_RECHDR (com_ptr, "Open");

    APPEND_TOFLIN (com_ptr, filnamlin);

    APPEND_TOFLIN (com_ptr, filidlin);

    (void) sprintf (lin_ptr, "access %d", filacc);
    APPEND_TOFLIN (com_ptr, NULL);

    (void) sprintf (lin_ptr, "options %d", filopt);
    APPEND_TOFLIN (com_ptr, NULL);

    if (strlen (tpa_ptr->onerror_labelc16) != 0)
    {
        (void) strcpy (lin_ptr, "errlab ");
        RESOLVE_LABEL (argblk_ptr,
                       tpa_ptr->onerror_labelc16,
                       tpa_ptr->onerror_lenl);
        APPEND_TOFLIN (com_ptr, NULL);
    }

    /*
     *  Mark the file as having been opened.
     */
    fat_ptr->fat_refflgl = OPEN_FILE;

end:
    ;
    return PSL_NORMAL;          /* Return success */
}

/******************************************************************************
 * @func    This Tparse action routine is called after parsing an OPEN PIPE
 *      command. Its purpose is to append a Oppipe record to the list of
 *      'TOF' lines. The Oppipe record has the following format:
 *
 *      <PRE>
 *      Oppipe
 *      type <type>
 *      scope <scope>
 *      name <pipe_name>
 *      pipid <pipe_id>
 *      access <access_code>
 *      {timeout <timeout_period>}
 *      {tmolab <timeout_label>}
 *      {errlab <error_label>}
 *      </PRE>
 *
 * @rdesc   Tparse status (Always returns PSL_NORMAL).
 * @parm    Pointer to Tparse argument block.
 */
unsigned long ENC_OPEN_PIPE (struct tpaarg *argblk_ptr)
{
    /*
     *  Local data.
     */
    struct scl_com  *com_ptr;       /* SCL common data */
    struct scl_tpa  *tpa_ptr;       /* SCL Tparse data */
    struct fattbl   *fat_ptr;       /* Pipe Access Table pointer */
    unsigned long   pipacc;         /* Pipe access type */
    unsigned long   pipscp;         /* Pipe scope */
    unsigned long   piptyp;         /* Pipe type */
    char   *lin_ptr;       /* Pointer to line buffer */
    char   pipnamlin [TOFBUFLEN];  /* Pipe name line */
    char   pipidlin [TOFBUFLEN];   /* Pipe Id line */
    char   tmoprdlin [TOFBUFLEN];  /* Timeout line */

    /*
     *  Copy pointers for ease of reference.
     */
    com_ptr = (struct scl_com *) argblk_ptr->user_ptrp;
    tpa_ptr = com_ptr->tpa_ptrps;
    lin_ptr = com_ptr->toflinc1024; /* 'tof' line buffer pointer */
    fat_ptr = tpa_ptr->pipe_begps + tpa_ptr->open_fididxl - 1;
    /* Point to current entry in table */
    /*
     *  Do not encode anything if we have had an error in the command.
     */
    if (tpa_ptr->error_in_commandn) goto end;

    /*
     *  Output tracing information if required.
     */
#ifdef PSL_TRACING
    if (PSL_TEST_TRACE (PTRC_USR_USER1))
    {
        PSL_OUTPUT_TRACE ("Encoding a OPEN PIPE Command\n");
        PSL_OUTPUT_TRACE ("Operand Count = %ld\n" ,GET_OPD_COUNT (tpa_ptr));
    }
#endif

    /*
     *  If the pipe has been passed as a parameter, report an error and return.
     */
    if ((fat_ptr->fat_refflgl & PIPATT_PRM) != 0)
    {                   /* ...if pipe passed as parameter */
        REPORT_ERROR (argblk_ptr, SCL_PIPPRMNOOPEN);
        goto end;
    }

    /*
     *  Get the type flag from the symbol table.
     */
    if ((fat_ptr->fat_refflgl & PIPATT_SYNC) != 0)
    {
        piptyp = TOF_PIPTYP_SYNCHRONOUS;
    }
    else
    {
        piptyp = TOF_PIPTYP_ASYNCHRONOUS;
    }

    /*
     *  Get the scope flag.
     */
    if ((tpa_ptr->pipe_flagsl & PIPFLG_TESTWIDE) != 0)
    {
        pipscp = TOF_PIPSCP_TESTWIDE;
    }
    else
    {
        pipscp = TOF_PIPSCP_LOCAL;
    }

    /*
     *  Get pipe name and create 'name' sub-record.
     */
    (void) strcpy (pipnamlin, "name ");
    ENC_OPD_BYPOS (com_ptr, pipnamlin, NULL, NULL);

    /*
     *  Get pipe ID and create 'pipid' sub-record.
     */
    (void) sprintf (pipidlin, "pipid %s", fat_ptr->fat_namec20);

    /*
     *  Get pipe access type and create 'access' sub-record.
     */
    if ((tpa_ptr->open_maskl & OPEN_OUTPUT) != 0)
    {
        pipacc = TOF_PIPACC_OUTPUT;
    }
    else if ((tpa_ptr->open_maskl & OPEN_INPUT) != 0)
    {
        pipacc = TOF_PIPACC_INPUT;
    }
    else
    {
        pipacc = TOF_PIPACC_INPUT;
    }

    /*
     *  Get the wait period, if any, and create 'timeout' sub-record.
     */
    if ((tpa_ptr->cmnd_optl & CMND_OPT_WAIT_PRD) != 0)
    {
        (void) strcpy (tmoprdlin, "timeout ");
        ENC_OPD_BYPOS (com_ptr, tmoprdlin, NULL, NULL);
    }

    /*
     *  Write the following records to the 'TOF' line list:
     *
     *  Oppipe
     *  type <type>
     *  scope <scope>
     *  name <pipe_name>
     *  pipid <pipe_id>
     *  access <access_code>
     *  {timeout <timeout_period>}
     *  {tmolab <timeout_label>}
     *  {errlab <error_label>}
     */
    APPEND_RECHDR (com_ptr, "Oppipe");

    (void) sprintf (lin_ptr, "type %d", piptyp);
    APPEND_TOFLIN (com_ptr, NULL);

    (void) sprintf (lin_ptr, "scope %d", pipscp);
    APPEND_TOFLIN (com_ptr, NULL);

    APPEND_TOFLIN (com_ptr, pipnamlin);

    APPEND_TOFLIN (com_ptr, pipidlin);

    (void) sprintf (lin_ptr, "access %d", pipacc);
    APPEND_TOFLIN (com_ptr, NULL);

    if ((tpa_ptr->cmnd_optl & CMND_OPT_WAIT_PRD) != 0)
    {
        APPEND_TOFLIN (com_ptr, tmoprdlin);
    }

    if ((tpa_ptr->cmnd_optl & CMND_OPT_TIMEOUT) != 0)
    {
        (void) sprintf (lin_ptr, "tmolab %s", tpa_ptr->ontimeout_labelc20);
        APPEND_TOFLIN (com_ptr, lin_ptr);
    }

    if (strlen (tpa_ptr->onerror_labelc16) != 0)
    {
        (void) strcpy (lin_ptr, "errlab ");
        RESOLVE_LABEL (argblk_ptr,
                       tpa_ptr->onerror_labelc16,
                       tpa_ptr->onerror_lenl);
        APPEND_TOFLIN (com_ptr, NULL);
    }

    /*
     *  Mark the pipe as having been opened.
     */
    fat_ptr->fat_refflgl |= PIPATT_OPEN;

end:
    ;
    return PSL_NORMAL;          /* Return success */
}

/******************************************************************************
 * @func    This routine is a Tparse action routine and is called to encode an
 *      operator into the default 'TOF' line buffer. An operator code is
 *      passed in the parameter field of the Tparse argument block. This is
 *      encoded into the 'TOF' line buffer using the corresponding 'TOF'
 *      operator. A leading and trailing space are also encoded - except
 *      for unary operators for which no leading or trailing space is
 *      encoded.
 *
 * @rdesc   Tparse status (Always returns PSL_NORMAL).
 * @parm    Pointer to Tparse argument block.
 */
unsigned long ENC_OPR (struct tpaarg *argblk_ptr)
{
    /*
     *  Local data.
     */
    struct scl_com  *com_ptr;       /* SCL common data */
    struct scl_tpa  *tpa_ptr;       /* SCL Tparse data */
    char    *lin_ptr;      /* Pointer to line buffer */

    /*
     *  Copy pointers for ease of reference.
     */
    com_ptr = (struct scl_com *) argblk_ptr->user_ptrp;
    tpa_ptr = com_ptr->tpa_ptrps;
    lin_ptr = com_ptr->toflinc1024; /* 'tof' line buffer pointer */

    /*
     *  First handle any special cases.
     *
     *  The NOT unary operator is encoded without a leading and a trailing space.
     */
    if (argblk_ptr->paraml == OPR_NOT)
    {
        (void) strcat (lin_ptr, TOF_OPR_NOT);
        goto end;
    }

    /*
     *  Encode the operator with a leading and trailing space.
     */
    (void) strcat (lin_ptr, " ");   /* Append leading space */

    switch (argblk_ptr->paraml)
    {
    case OPR_NE:
        (void) strcat (lin_ptr, TOF_OPR_NE);
        break;
    case OPR_EQ:
        (void) strcat (lin_ptr, TOF_OPR_EQ);
        break;
    case OPR_LE:
        (void) strcat (lin_ptr, TOF_OPR_LE);
        break;
    case OPR_GE:
        (void) strcat (lin_ptr, TOF_OPR_GE);
        break;
    case OPR_LT:
        (void) strcat (lin_ptr, TOF_OPR_LT);
        break;
    case OPR_GT:
        (void) strcat (lin_ptr, TOF_OPR_GT);
        break;
    case OPR_CONTAINS:
        (void) strcat (lin_ptr, TOF_OPR_CONTAINS);
        break;
    case OPR_NOT_CONTAINS:
        (void) strcat (lin_ptr, TOF_OPR_NOT_CONTAINS);
        break;
    case OPR_ADD:
        (void) strcat (lin_ptr, TOF_OPR_ADD);
        break;
    case OPR_SUBTRACT:
        (void) strcat (lin_ptr, TOF_OPR_SUBTRACT);
        break;
    case OPR_AND:
        (void) strcat (lin_ptr, TOF_OPR_AND);
        break;
    case OPR_OR:
        (void) strcat (lin_ptr, TOF_OPR_OR);
        break;
    case OPR_MULTIPLY:
        (void) strcat (lin_ptr, TOF_OPR_MULTIPLY);
        break;
    case OPR_DIVIDE:
        (void) strcat (lin_ptr, TOF_OPR_DIVIDE);
        break;
    case OPR_MODULO:
        (void) strcat (lin_ptr, TOF_OPR_MODULO);
        break;
    case OPR_MATCHES:
        (void) strcat (lin_ptr, TOF_OPR_MATCHES);
        break;
    case OPR_NOT_MATCHES:
        (void) strcat (lin_ptr, TOF_OPR_NOT_MATCHES);
        break;
    default:
        break;
    }

    (void) strcat (lin_ptr, " ");   /* Append trailing space */
end:
    ;
    return PSL_NORMAL;
}

/******************************************************************************
 * @func    This Tparse action routine is called after parsing a READ (file)
 *      command. Its purpose is to append a Read record to the list of
 *      'TOF' lines. The Read record has the following format:
 *
 *      <PRE>
 *      Read
 *      variable <variable>
 *      filid <file_id>
 *      {index <index>}
 *      {endlab <end_label>}
 *      {errlab <error_label>}
 *      </PRE>
 *
 *      Note that if this is a passed file, the index is incremented by
 *      the constant PASSED_FILE_BASE, to show that it is a passed file.
 *
 * @rdesc   Tparse status (Always returns PSL_NORMAL).
 * @parm    Pointer to Tparse argument block.
 */
unsigned long ENC_READ_FILE (struct tpaarg *argblk_ptr)
{
    /*
     *  Local data.
     */
    struct scl_com  *com_ptr;       /* SCL common data */
    struct scl_tpa  *tpa_ptr;       /* SCL Tparse data */
    struct fattbl   *fat_ptr;       /* File Access Table pointer */
    char   *lin_ptr;       /* Pointer to line buffer */
    char   varlin [TOFBUFLEN]; /* Variable line */
    char   filidlin [TOFBUFLEN];   /* File Id line */
    char   idxlin [TOFBUFLEN]; /* Index line */

    /*
     *  Copy pointers for ease of reference.
     */
    com_ptr = (struct scl_com *) argblk_ptr->user_ptrp;
    tpa_ptr = com_ptr->tpa_ptrps;
    lin_ptr = com_ptr->toflinc1024; /* 'tof' line buffer pointer */
    fat_ptr = tpa_ptr->fat_begps + tpa_ptr->open_fididxl - 1;
    /* Point to current entry in table */
    /*
     *  Do not encode anything if we have had an error in the command.
     */
    if (tpa_ptr->error_in_commandn) goto end;

    /*
     *  Output tracing information if required.
     */
#ifdef PSL_TRACING
    if (PSL_TEST_TRACE (PTRC_USR_USER1))
    {
        PSL_OUTPUT_TRACE ("Encoding a READ (file) Command\n");
        PSL_OUTPUT_TRACE ("Operand Count = %ld\n" ,GET_OPD_COUNT (tpa_ptr));
    }
#endif

    /*
     *    Check to see if this file has been DEFINED (by the FILE statement).
     *    If it has then we set the referenced flag to ACCESSED_FILE
     *
     *    ...otherwise...
     *
     *    If the file is a passed file, we need to show this by incrementing its
     *    index into the passed file index range.
     */
    if (fat_ptr->fat_refflgl == DEFINED_FILE)
    {
        fat_ptr->fat_refflgl = ACCESSED_FILE;
    }

    else if (fat_ptr->fat_refflgl == PASSED_FILE)
    {
        tpa_ptr->open_fididxl += PASSED_FILE_BASE;
    }

    /*
     *  Get record buffer variable name and create 'variable' sub-record.
     */
    (void) strcpy (varlin, "variable ");
    ENC_OPD_BYPOS (com_ptr, varlin, NULL, NULL);

    /*
     *  Get file ID and create 'filid' sub-record.
     */
    (void) sprintf (filidlin, "filid %s", fat_ptr->fat_namec20);

    /*
     *  Get the index value (if any).
     */
    if ((tpa_ptr->cmnd_optl & READ_INDEXED) != 0)
    {
        (void) strcpy (idxlin, "index ");
        ENC_OPD_BYPOS (com_ptr, idxlin, NULL, NULL);
    }

    /*
     *  Write the following records to the 'TOF' line list:
     *
     *  Read
     *  variable <variable>
     *  filid <file_id>
     *  {index <index>}
     *  {endlab <end_label>}
     *  {errlab <error_label>}
     */
    APPEND_RECHDR (com_ptr, "Read");

    APPEND_TOFLIN (com_ptr, varlin);

    APPEND_TOFLIN (com_ptr, filidlin);

    if ((tpa_ptr->cmnd_optl & READ_INDEXED) != 0)
    {
        APPEND_TOFLIN (com_ptr, idxlin);
    }

    if (strlen (tpa_ptr->label_namec16) != 0)
    {
        (void) strcpy (lin_ptr, "endlab ");
        RESOLVE_LABEL (argblk_ptr,
                       tpa_ptr->label_namec16,
                       tpa_ptr->label_lenl);
        APPEND_TOFLIN (com_ptr, NULL);
    }

    if (strlen (tpa_ptr->onerror_labelc16) != 0)
    {
        (void) strcpy (lin_ptr, "errlab ");
        RESOLVE_LABEL (argblk_ptr,
                       tpa_ptr->onerror_labelc16,
                       tpa_ptr->onerror_lenl);
        APPEND_TOFLIN (com_ptr, NULL);
    }

end:
    ;
    return PSL_NORMAL;          /* Return success */
}

/******************************************************************************
 * @func    This Tparse action routine is called after parsing a READ PIPE
 *      command. Its purpose is to append a Rdpipe record to the list of
 *      'TOF' lines. The Rdpipe record has the following format:
 *
 *      <PRE>
 *      Rdpipe
 *      variable <variable>
 *      pipid <pipe_id>
 *      {timeout <timeout_period>}
 *      {tmolab <timeout_label>}
 *      {errlab <error_label>}
 *      </PRE>
 *
 *      Note that if this is a passed pipe, the index is incremented by
 *      the constant PASSED_PIPE_BASE, to show that it is a passed pipe.
 *
 * @rdesc   Tparse status (Always returns PSL_NORMAL).
 * @parm    Pointer to Tparse argument block.
 */
unsigned long ENC_READ_PIPE (struct tpaarg *argblk_ptr)
{
    /*
     *  Local data.
     */
    struct scl_com  *com_ptr;       /* SCL common data */
    struct scl_tpa  *tpa_ptr;       /* SCL Tparse data */
    struct fattbl   *fat_ptr;       /* Pipe Access Table pointer */
    char   *lin_ptr;       /* Pointer to line buffer */
    char   varlin [TOFBUFLEN]; /* Variable line */
    char   pipidlin [TOFBUFLEN];   /* Pipe Id line */
    char   tmoprdlin [TOFBUFLEN];  /* Timeout line */

    /*
     *  Copy pointers for ease of reference.
     */
    com_ptr = (struct scl_com *) argblk_ptr->user_ptrp;
    tpa_ptr = com_ptr->tpa_ptrps;
    lin_ptr = com_ptr->toflinc1024; /* 'tof' line buffer pointer */
    fat_ptr = tpa_ptr->pipe_begps + tpa_ptr->open_fididxl - 1;
    /* Point to current entry in table */
    /*
     *  Do not encode anything if we have had an error in the command.
     */
    if (tpa_ptr->error_in_commandn) goto end;

    /*
     *  Output tracing information if required.
     */
#ifdef PSL_TRACING
    if (PSL_TEST_TRACE (PTRC_USR_USER1))
    {
        PSL_OUTPUT_TRACE ("Encoding a READ PIPE Command\n");
        PSL_OUTPUT_TRACE ("Operand Count = %ld\n" ,GET_OPD_COUNT (tpa_ptr));
    }
#endif

    /*
     *  If the pipe has been passed as a parameter, add the 'passed pipe base'
     *  offset to the pipe entry index index to point to the entry in the 'passed
     *  pipe' section of the table.
     */
    if ((fat_ptr->fat_refflgl & PIPATT_PRM) != 0)
    {
        tpa_ptr->open_fididxl += PASSED_PIPE_BASE;
    }

    /*
     *  Get record buffer variable name and create 'variable' sub-record.
     */
    (void) strcpy (varlin, "variable ");
    ENC_OPD_BYPOS (com_ptr, varlin, NULL, NULL);

    /*
     *  Get pipe ID and create 'pipid' sub-record.
     */
    (void) sprintf (pipidlin, "pipid %s", fat_ptr->fat_namec20);

    /*
     *  Get the wait period, if any, and create 'timeout' sub-record.
     */
    if ((tpa_ptr->cmnd_optl & CMND_OPT_WAIT_PRD) != 0)
    {
        (void) strcpy (tmoprdlin, "timeout ");
        ENC_OPD_BYPOS (com_ptr, tmoprdlin, NULL, NULL);
    }

    /*
     *  Write the following records to the 'TOF' line list:
     *
     *  Rdpipe
     *  variable <variable>
     *  pipid <pipe_id>
     *  {timeout <timeout_period>}
     *  {tmolab <timeout_label>}
     *  {errlab <error_label>}
     */
    APPEND_RECHDR (com_ptr, "Rdpipe");

    APPEND_TOFLIN (com_ptr, varlin);

    APPEND_TOFLIN (com_ptr, pipidlin);

    if ((tpa_ptr->cmnd_optl & CMND_OPT_WAIT_PRD) != 0)
    {
        APPEND_TOFLIN (com_ptr, tmoprdlin);
    }

    if ((tpa_ptr->cmnd_optl & CMND_OPT_TIMEOUT) != 0)
    {
        (void) sprintf (lin_ptr, "tmolab %s", tpa_ptr->ontimeout_labelc20);
        APPEND_TOFLIN (com_ptr, lin_ptr);
    }

    if (strlen (tpa_ptr->onerror_labelc16) != 0)
    {
        (void) strcpy (lin_ptr, "errlab ");
        RESOLVE_LABEL (argblk_ptr,
                       tpa_ptr->onerror_labelc16,
                       tpa_ptr->onerror_lenl);
        APPEND_TOFLIN (com_ptr, NULL);
    }

    /*
     *  Mark the pipe as having been accessed.
     */
    fat_ptr->fat_refflgl |= PIPATT_ACC;

end:
    ;
    return PSL_NORMAL;          /* Return success */
}

/******************************************************************************
 * @func    This Tparse action routine is called after parsing a RELEASE MUTEX
 *      command. Its purpose is to append a Rlmutex record to the list of
 *      'TOF' lines. The Rlmutex record has the following format:
 *
 *      <PRE>
 *      Rlmutex
 *      scope <scope>
 *      name <mutex_name>
 *      {errlab <error_label>}
 *      </PRE>
 *
 * @rdesc   Tparse status (Always returns PSL_NORMAL).
 * @parm    Pointer to Tparse argument block.
 */
unsigned long ENC_RELEASE_MUTEX (struct tpaarg *argblk_ptr)
{
    /*
     *  Local data.
     */
    struct scl_com  *com_ptr;       /* SCL common data */
    struct scl_tpa  *tpa_ptr;       /* SCL Tparse data */
    unsigned long   mtxscp;         /* Acquire mutext scope flag */
    char   *lin_ptr;       /* Pointer to line buffer */
    char   mtxnam [TOFBUFLEN]; /* Mutex name */

    /*
     *  Copy pointers for ease of reference.
     */
    com_ptr = (struct scl_com *) argblk_ptr->user_ptrp;
    tpa_ptr = com_ptr->tpa_ptrps;
    lin_ptr = com_ptr->toflinc1024; /* 'tof' line buffer pointer */

    /*
     *  Do not encode anything if we have had an error in the command.
     */
    if (tpa_ptr->error_in_commandn) goto end;

    /*
     *  Output tracing information if required.
     */
#ifdef PSL_TRACING
    if (PSL_TEST_TRACE (PTRC_USR_USER1))
    {
        PSL_OUTPUT_TRACE ("Encoding an RELEASE MUTEX Command\n");
        PSL_OUTPUT_TRACE ("Operand Count = %ld\n" ,GET_OPD_COUNT (tpa_ptr));
    }
#endif

    /*
     *  Get the release mutex scope flag.
     */
    if ((tpa_ptr->pipe_flagsl & PIPFLG_TESTWIDE) != 0)
    {
        mtxscp = TOF_MTXSCP_TESTWIDE;
    }
    else
    {
        mtxscp = TOF_MTXSCP_LOCAL;
    }

    /*
     *  Get mutex name.
     */
    mtxnam [0] = '\0';
    ENC_OPD_BYPOS (com_ptr, mtxnam, NULL, NULL);

    /*
     *  Write the following records to the 'TOF' line list:
     *
     *  Acmutex
     *  scope <scope>
     *  name <mutex_name>
     *  {errlab <error_label>}
     */
    APPEND_RECHDR (com_ptr, "Rlmutex");

    (void) sprintf (lin_ptr, "scope %d", mtxscp);
    APPEND_TOFLIN (com_ptr, NULL);

    (void) sprintf (lin_ptr, "name %s", mtxnam);
    APPEND_TOFLIN (com_ptr, NULL);

    if (strlen (tpa_ptr->onerror_labelc16) != 0)
    {
        (void) strcpy (lin_ptr, "errlab ");
        RESOLVE_LABEL (argblk_ptr,
                       tpa_ptr->onerror_labelc16,
                       tpa_ptr->onerror_lenl);
        APPEND_TOFLIN (com_ptr, NULL);
    }

end:
    ;
    return PSL_NORMAL;          /* Return success */
}

/******************************************************************************
 * @func    This Tparse action routine is called after parsing a REPLY TO PIPE
 *      command. Its purpose is to append a Rppipe record to the list of
 *      'TOF' lines. The Rppipe record has the following format:
 *
 *      <PRE>
 *      Rppipe
 *      string <value>
 *      pipid <pipe_id>
 *      {errlab <error_label>}
 *      </PRE>
 *
 *      Note that if this is a passed pipe, the index is incremented by
 *      the constant PASSED_PIPE_BASE, to show that it is a passed pipe.
 *
 * @rdesc   Tparse status (Always returns PSL_NORMAL).
 * @parm    Pointer to Tparse argument block.
 */
unsigned long ENC_REPLY_PIPE (struct tpaarg *argblk_ptr)
{
    /*
     *  Local data.
     */
    struct scl_com  *com_ptr;       /* SCL common data */
    struct scl_tpa  *tpa_ptr;       /* SCL Tparse data */
    struct fattbl   *fat_ptr;       /* Pipe Access Table pointer */
    char   *lin_ptr;       /* Pointer to line buffer */
    char   repstrlin [TOFBUFLEN];  /* Reply string line */
    char   pipidlin [TOFBUFLEN];   /* Pipe Id line */

    /*
     *  Copy pointers for ease of reference.
     */
    com_ptr = (struct scl_com *) argblk_ptr->user_ptrp;
    tpa_ptr = com_ptr->tpa_ptrps;
    lin_ptr = com_ptr->toflinc1024; /* 'tof' line buffer pointer */
    fat_ptr = tpa_ptr->pipe_begps + tpa_ptr->open_fididxl - 1;
    /* Point to current entry in table */
    /*
     *  Do not encode anything if we have had an error in the command.
     */
    if (tpa_ptr->error_in_commandn) goto end;

    /*
     *  Output tracing information if required.
     */
#ifdef PSL_TRACING
    if (PSL_TEST_TRACE (PTRC_USR_USER1))
    {
        PSL_OUTPUT_TRACE ("Encoding a REPLY PIPE Command\n");
        PSL_OUTPUT_TRACE ("Operand Count = %ld\n" ,GET_OPD_COUNT (tpa_ptr));
    }
#endif

    /*
     *  If the pipe has been passed as a parameter, add the 'passed pipe base'
     *  offset to the pipe entry index index to point to the entry in the 'passed
     *  pipe' section of the table.
     */
    if ((fat_ptr->fat_refflgl & PIPATT_PRM) != 0)
    {
        tpa_ptr->open_fididxl += PASSED_PIPE_BASE;
    }

    /*
     *  Get reply string operand and create 'string' sub-record.
     */
    (void) strcpy (repstrlin, "string ");
    ENC_OPD_BYPOS (com_ptr, repstrlin, NULL, NULL);

    /*
     *  Get pipe ID and create 'pipid' sub-record.
     */
    (void) sprintf (pipidlin, "pipid %s", fat_ptr->fat_namec20);

    /*
     *  Write the following records to the 'TOF' line list:
     *
     *  Rppipe
     *  string <value>
     *  pipid <pipe_id>
     *  {errlab <error_label>}
     */
    APPEND_RECHDR (com_ptr, "Rppipe");

    APPEND_TOFLIN (com_ptr, repstrlin);

    APPEND_TOFLIN (com_ptr, pipidlin);

    if (strlen (tpa_ptr->onerror_labelc16) != 0)
    {
        (void) strcpy (lin_ptr, "errlab ");
        RESOLVE_LABEL (argblk_ptr,
                       tpa_ptr->onerror_labelc16,
                       tpa_ptr->onerror_lenl);
        APPEND_TOFLIN (com_ptr, NULL);
    }

    /*
     *  Mark the pipe as having been accessed.
     */
    fat_ptr->fat_refflgl |= PIPATT_ACC;

end:
    ;
    return PSL_NORMAL;          /* Return success */
}

/******************************************************************************
 * @func    This Tparse action routine is called after parsing a RESET command.
 *      Its purpose is to append a Genvar - and optionally a Seed - record
 *      to the list of 'TOF' lines. The Genvar and Seed records have the
 *      following format:
 *
 *      <PRE>
 *      Genvar
 *      mode TOF_GVRMOD_RESET
 *      name <variable>
 *
 *      Seed
 *      name <variable>
 *      ivalue <seed_value>
 *      </PRE>
 *
 *      A Seed record is only written if the SEED option is specified.
 *
 * @rdesc   Tparse status (Always returns PSL_NORMAL).
 * @parm    Pointer to Tparse argument block.
 */
unsigned long ENC_RESET (struct tpaarg *argblk_ptr)
{
    /*
     *    Local data
     */
    struct scl_com  *com_ptr;       /* SCL comon data */
    struct scl_tpa  *tpa_ptr;       /* SCL Tparse data */
    char   *lin_ptr;       /* Pointer to line buffer */
    char   namlin [TOFBUFLEN]; /* Name line */

    /*
     *  Copy pointers for ease of reference.
     */
    com_ptr = (struct scl_com *) argblk_ptr->user_ptrp;
    tpa_ptr = com_ptr->tpa_ptrps;
    lin_ptr = com_ptr->toflinc1024; /* 'tof' line buffer pointer */

    /*
     *  Do not encode anything if we have had an error in the command.
     */
    if (tpa_ptr->error_in_commandn) goto end;

    /*
     *  Output tracing information if required.
     */
#ifdef PSL_TRACING
    if (PSL_TEST_TRACE (PTRC_USR_USER1))
    {
        PSL_OUTPUT_TRACE ("\nEncoding a RESET Command\n");
        PSL_OUTPUT_TRACE ("Operand Count = %ld\n" ,GET_OPD_COUNT (tpa_ptr));
    }
#endif

    /*
     *  Write the following records to the 'TOF' line list:
     *
     *  Genvar
     *  mode TOF_GVRMOD_RESET
     *  name <variable>
     */
    APPEND_RECHDR (com_ptr, "Genvar");

    (void) sprintf (lin_ptr, "mode %d", TOF_GVRMOD_RESET);
    APPEND_TOFLIN (com_ptr, NULL);

    (void) strcpy (namlin, "name ");
    ENC_OPD_BYPOS (com_ptr, namlin, NULL, NULL);
    APPEND_TOFLIN (com_ptr, namlin);

    /*
     *  If SEED is NOT to be reset, all done.
     */
    if ((tpa_ptr->cmnd_optl & CMND_OPT_SEED) == 0) goto end;

    /*
     *  Write the following records to the 'TOF' line list:
     *
     *  Seed
     *  name <variable>
     *  ivalue <seed_value>
     */
    APPEND_RECHDR (com_ptr, "Seed");

    APPEND_TOFLIN (com_ptr, namlin);

    (void) strcpy (lin_ptr, "ivalue ");
    ENC_OPD_BYPOS (com_ptr, lin_ptr, NULL, NULL);
    APPEND_TOFLIN (com_ptr, NULL);

end:
    ;
    return PSL_NORMAL;
}

/******************************************************************************
 * @func    This Tparse action routine is called after parsing a RETURN
 *      command. Its purpose is to append a Return record to the linked
 *      list of 'TOF' lines. The Return record has the following format:
 *
 *      <PRE>
 *      Return
 *      </PRE>
 *
 * @rdesc   Tparse status (Always returns PSL_NORMAL).
 * @parm    Pointer to Tparse argument block.
 */
unsigned long ENC_RETURN (struct tpaarg *argblk_ptr)
{
    /*
     *  Local data.
     */
    struct scl_com  *com_ptr;       /* SCL common data */
    struct scl_tpa  *tpa_ptr;       /* SCL Tparse data */

    /*
     *  Copy pointers for ease of reference.
     */
    com_ptr = (struct scl_com *) argblk_ptr->user_ptrp;
    tpa_ptr = com_ptr->tpa_ptrps;

    /*
     *  Do not encode anything if we have had an error in the command.
     */
    if (tpa_ptr->error_in_commandn) goto end;

    /*
     *  Output tracing information if required.
     */
#ifdef PSL_TRACING
    if (PSL_TEST_TRACE (PTRC_USR_USER1))
    {
        PSL_OUTPUT_TRACE ("Encoding a RETURN Command\n");
    }
#endif

    /*
     *  Write the following records to the 'TOF' line list:
     *
     *  Return
     */
    APPEND_RECHDR (com_ptr, "Return");

end:
    ;
    return PSL_NORMAL;
}

/******************************************************************************
 * @func    This Tparse action routine is called after parsing a REWIND
 *      command. Its purpose is to append a Fcntl record to the list of
 *      'TOF' lines. The Fcntl record has the following format:
 *
 *      <PRE>
 *      Fcntl
 *      operation TOF_FCNOP_REWIND
 *      filid <file_id>
 *      {errlab <error_label>}
 *      </PRE>
 *
 *      Note that if this is a passed file, the index is incremented by
 *      the constant PASSED_FILE_BASE, to show that it is a passed file.
 *
 * @rdesc   Tparse status (Always returns PSL_NORMAL).
 * @parm    Pointer to Tparse argument block.
 */
unsigned long ENC_REWIND_FILE (struct tpaarg *argblk_ptr)
{
    /*
     *  Local data.
     */
    struct scl_com  *com_ptr;       /* SCL common data */
    struct scl_tpa  *tpa_ptr;       /* SCL Tparse data */
    struct fattbl   *fat_ptr;       /* File Access Table pointer */
    char   *lin_ptr;       /* Pointer to line buffer */
    char   filidlin [TOFBUFLEN];   /* File Id line */

    /*
     *  Copy pointers for ease of reference.
     */
    com_ptr = (struct scl_com *) argblk_ptr->user_ptrp;
    tpa_ptr = com_ptr->tpa_ptrps;
    lin_ptr = com_ptr->toflinc1024; /* 'tof' line buffer pointer */
    fat_ptr = tpa_ptr->fat_begps + tpa_ptr->open_fididxl - 1;
    /* Point to current entry in table */
    /*
     *  Do not encode anything if we have had an error in the command.
     */
    if (tpa_ptr->error_in_commandn) goto end;

    /*
     *  Output tracing information if required.
     */
#ifdef PSL_TRACING
    if (PSL_TEST_TRACE (PTRC_USR_USER1))
    {
        PSL_OUTPUT_TRACE ("Encoding a REWIND Command\n");
        PSL_OUTPUT_TRACE ("Operand Count = %ld\n" ,GET_OPD_COUNT (tpa_ptr));
    }
#endif

    /*
     *    Check to see if this file has been DEFINED (by the FILE statement).
     *    If it has then we set the referenced flag to ACCESSED_FILE
     *
     *    ...otherwise...
     *
     *    If the file is a passed file, we need to show this by incrementing its
     *    index into the passed file index range.
     */
    if (fat_ptr->fat_refflgl == DEFINED_FILE)
    {
        fat_ptr->fat_refflgl = ACCESSED_FILE;
    }
    else if (fat_ptr->fat_refflgl == PASSED_FILE)
    {
        tpa_ptr->open_fididxl += PASSED_FILE_BASE;
    }

    /*
     *  Get file ID and create 'filid' sub-record.
     */
    (void) sprintf (filidlin, "filid %s", fat_ptr->fat_namec20);

    /*
     *  Write the following records to the 'TOF' line list:
     *
     *  Fcntl
     *  operation TOF_FCNOP_REWIND
     *  filid <file_id>
     *  {errlab <error_label>}
     */
    APPEND_RECHDR (com_ptr, "Fcntl");

    (void) sprintf (lin_ptr, "operation %d", TOF_FCNOP_REWIND);
    APPEND_TOFLIN (com_ptr, NULL);

    APPEND_TOFLIN (com_ptr, filidlin);

    if (strlen (tpa_ptr->onerror_labelc16) != 0)
    {
        (void) strcpy (lin_ptr, "errlab ");
        RESOLVE_LABEL (argblk_ptr,
                       tpa_ptr->onerror_labelc16,
                       tpa_ptr->onerror_lenl);
        APPEND_TOFLIN (com_ptr, NULL);
    }

end:
    ;
    return PSL_NORMAL;          /* Return success */
}

/******************************************************************************
 * @func    This routine is a Tparse action routine and is called to encode a
 *      'name' sub-record of a Setvar record and append it to the
 *      'TOF' line list. This record has the following format:
 *
 *      <PRE>
 *      name <variable>
 *      </PRE>
 *
 * @rdesc   Tparse status (Always returns PSL_NORMAL).
 * @parm    Pointer to Tparse argument block.
 */
unsigned long ENC_SETVAR_NAM (struct tpaarg *argblk_ptr)
{
    /*
     *  Local data.
     */
    struct scl_com   *com_ptr;      /* SCL comon ata */
    struct scl_tpa   *tpa_ptr;      /* SCL Tparse data */
    char    *lin_ptr;      /* Pointer to line buffer */

    /*
     *  Copy pointers for ease of reference.
     */
    com_ptr = (struct scl_com *) argblk_ptr->user_ptrp;
    tpa_ptr = com_ptr->tpa_ptrps;
    lin_ptr = com_ptr->toflinc1024; /* 'tof' line buffer pointer */

    /*
     *  Do not encode anything if we have had an error in the command.
     */
    if (tpa_ptr->error_in_commandn) goto end;

    /*
     *  Output tracing information if required.
     */
#ifdef PSL_TRACING
    if (PSL_TEST_TRACE (PTRC_USR_USER1))
    {
        PSL_OUTPUT_TRACE ("\nEncoding a Setvar name record\n");
        PSL_OUTPUT_TRACE ("Operand Count = %ld\n" ,GET_OPD_COUNT (tpa_ptr));
    }
#endif

    /*
     *  Write the following records to the 'TOF' line list:
     *
     *  name <variable>
     */
    (void) strcpy (lin_ptr, "name ");
    ENC_OPD_BYPOS (com_ptr, NULL, NULL, NULL);
    APPEND_TOFLIN (com_ptr, NULL);

end:
    ;
    return PSL_NORMAL;          /* Return success */
}

/******************************************************************************
 * @func    This Tparse action routine is called after parsing all the
 *      operands in a 'set variable' command. Its
 *      purpose is to complete the Setvar record. This is done by encoding
 *      and writing the value line and, if necessary an errlab line to the
 *      'tof' line list. The Setvar record has the following format:
 *
 *      <PRE>
 *      Setvar
 *      name <variable>
 *      value <expression>
 *      {errlab <label>}
 *      </PRE>
 *
 *      This routine writes the last two lines of this record. The errlab
 *      sub-record is only written if 'ON ERROR GOTO' has been specified.
 *
 * @rdesc   Tparse status (Always returns PSL_NORMAL).
 * @parm    Pointer to Tparse argument block.
 */
unsigned long ENC_SETVAR_VAL (struct tpaarg *argblk_ptr)
{
    /*
     *  Local data.
     */
    struct   scl_com *com_ptr;      /* SCL common data */
    struct   scl_tpa *tpa_ptr;      /* SCL Tparse data */
    char    *lin_ptr;      /* Pointer to line buffer */

    /*
     *  Copy pointers for ease of reference.
     */
    com_ptr = (struct scl_com *) argblk_ptr->user_ptrp;
    tpa_ptr = com_ptr->tpa_ptrps;
    lin_ptr = com_ptr->toflinc1024; /* 'tof' line buffer pointer */

    /*
     *  Do not encode anything if we have had an error in the command.
     */
    if (tpa_ptr->error_in_commandn) goto end;

    /*
     *  Output tracing information if required.
     */
#ifdef PSL_TRACING
    if (PSL_TEST_TRACE (PTRC_USR_USER1))
    {
        PSL_OUTPUT_TRACE ("\nEncoding Setvar value and errlab records\n");
        PSL_OUTPUT_TRACE ("Operand Count = %ld\n" ,GET_OPD_COUNT (tpa_ptr));
    }
#endif

    /*
     *  Write the following records to the 'TOF' line list:
     *
     *  value <expression>
     */
    (void) strcpy (lin_ptr, "value ");
    ENC_EXPR_BYPOS (com_ptr, lin_ptr);
    APPEND_TOFLIN (com_ptr, NULL);

    /*
     *  If 'ON ERROR GOTO label' has been specified, append the following record
     *  to the 'TOF' line list:
     *
     *  errlab <label>
     */
    if (strlen (tpa_ptr->onerror_labelc16) != 0)
    {
        (void) strcpy (lin_ptr, "errlab ");
        RESOLVE_LABEL (argblk_ptr,
                       tpa_ptr->onerror_labelc16,
                       tpa_ptr->onerror_lenl);
        APPEND_TOFLIN (com_ptr, NULL);
    }

end:
    ;
    return PSL_NORMAL;          /* Return success */
}

/******************************************************************************
 * @func    This Tparse action routine is called after parsing a SET SEMAPHORE
 *      command. Its purpose is to append a Stsemaphore record to the list
 *      of 'TOF' lines. The Rlmutex record has the following format:
 *
 *      <PRE>
 *      Stsemaphore
 *      scope <scope>
 *      name <semaphore_name>
 *      {errlab <error_label>}
 *      </PRE>
 *
 * @rdesc   Tparse status (Always returns PSL_NORMAL).
 * @parm    Pointer to Tparse argument block.
 */
unsigned long ENC_SET_SEMAPH (struct tpaarg *argblk_ptr)
{
    /*
     *  Local data.
     */
    struct scl_com  *com_ptr;       /* SCL common data */
    struct scl_tpa  *tpa_ptr;       /* SCL Tparse data */
    unsigned long   semscp;         /* Semaphore scope flag */
    char   *lin_ptr;       /* Pointer to line buffer */
    char   semnam [TOFBUFLEN]; /* Semaphore name */

    /*
     *  Copy pointers for ease of reference.
     */
    com_ptr = (struct scl_com *) argblk_ptr->user_ptrp;
    tpa_ptr = com_ptr->tpa_ptrps;
    lin_ptr = com_ptr->toflinc1024; /* 'tof' line buffer pointer */

    /*
     *  Do not encode anything if we have had an error in the command.
     */
    if (tpa_ptr->error_in_commandn) goto end;

    /*
     *  Output tracing information if required.
     */
#ifdef PSL_TRACING
    if (PSL_TEST_TRACE (PTRC_USR_USER1))
    {
        PSL_OUTPUT_TRACE ("Encoding an SET SEMAPHORE Command\n");
        PSL_OUTPUT_TRACE ("Operand Count = %ld\n" ,GET_OPD_COUNT (tpa_ptr));
    }
#endif

    /*
     *  Get the semaphore scope flag.
     */
    if ((tpa_ptr->pipe_flagsl & PIPFLG_TESTWIDE) != 0)
    {
        semscp = TOF_SEMSCP_TESTWIDE;
    }
    else
    {
        semscp = TOF_SEMSCP_LOCAL;
    }

    /*
     *  Get semaphore name.
     */
    semnam [0] = '\0';
    ENC_OPD_BYPOS (com_ptr, semnam, NULL, NULL);

    /*
     *  Write the following records to the 'TOF' line list:
     *
     *  Stsemaphore
     *  scope <scope>
     *  name <semaphore_name>
     *  {errlab <error_label>}
     */
    APPEND_RECHDR (com_ptr, "Stsemaphore");

    (void) sprintf (lin_ptr, "scope %d", semscp);
    APPEND_TOFLIN (com_ptr, NULL);

    (void) sprintf (lin_ptr, "name %s", semnam);
    APPEND_TOFLIN (com_ptr, NULL);

    if (strlen (tpa_ptr->onerror_labelc16) != 0)
    {
        (void) strcpy (lin_ptr, "errlab ");
        RESOLVE_LABEL (argblk_ptr,
                       tpa_ptr->onerror_labelc16,
                       tpa_ptr->onerror_lenl);
        APPEND_TOFLIN (com_ptr, NULL);
    }

end:
    ;
    return PSL_NORMAL;          /* Return success */
}

/******************************************************************************
 * @func    This routine is a Tparse action routine and is called to encode the
 *      'tof' lines required at the end of a script. This consists of
 *      writing an Exit command if no subroutines or sub-tests have been
 *      encountered.
 *
 * @rdesc   Tparse status (Always returns PSL_NORMAL).
 * @parm    Pointer to Tparse argument block.
 */
unsigned long ENC_SCRIPT_END (struct tpaarg *argblk_ptr)
{
    /*
     *  Local data.
     */
    struct scl_com   *com_ptr;      /* SCL common ata */
    struct scl_tpa   *tpa_ptr;      /* SCL Tparse data */

    /*
     *  Copy pointers for ease of reference.
     */
    com_ptr = (struct scl_com *) argblk_ptr->user_ptrp;
    tpa_ptr = com_ptr->tpa_ptrps;

    /*
     *  Output tracing information if required.
     */
#ifdef PSL_TRACING
    if (PSL_TEST_TRACE (PTRC_USR_USER1))
    {
        PSL_OUTPUT_TRACE ("\nEncoding End of Script Commands\n");
        PSL_OUTPUT_TRACE ("Operand Count = %ld\n" ,GET_OPD_COUNT (tpa_ptr));
    }
#endif

    /*
     *  If no subroutines or sub-tests have been defined then write an Exit command
     *  to terminate the code section - just in case it has not been terminated in
     *  the script.
     */
    if ((tpa_ptr->subrtn_def_cntl == 0) &&
            ((!com_ptr->type_testn) ||
             (tpa_ptr->subtst_cntl == 0)))
    {
        tpa_ptr->exit_maskl = 0;    /* Clear exit mask */
        ENC_EXIT (argblk_ptr);      /* ...and encode EXIT command */
    }

    return PSL_NORMAL;          /* Return success */
}

/******************************************************************************
 * @func    This routine is a Tparse action routine and is called to encode the
 *      'tof' lines required for a SUBROUTINE command:
 *
 *      <PRE>
 *      {Exit command}  {generated for first subroutine/subtest only}
 *
 *      Subroutine
 *      name <name>
 *      {parameters <parameters>}
 *      </PRE>
 *
 *      An exit command is encoded before the first subroutine/subtest
 *      in order to prevent execution continuing from the end of the
 *      main code section into the first subroutine/sub-test if no
 *      explicit Exit or Return has been specified in the script.
 *
 *      We also flag that the subroutine has been defined, so that
 *      at the end of the script we can identify and report errors for
 *      calls to undefined subroutines.
 *
 *      If we are in DEBUG then we will add the subroutine to the list
 *      of script modules.
 *
 * @rdesc   Tparse status (Always returns PSL_NORMAL).
 * @parm    Pointer to Tparse argument block.
 */
unsigned long ENC_SUBROUTINE (struct tpaarg *argblk_ptr)
{
    /*
     *  Local data.
     */
    struct scl_com   *com_ptr;      /* SCL comon ata */
    struct scl_tpa   *tpa_ptr;      /* SCL Tparse data */
    char    *lin_ptr;      /* Pointer to line buffer */
    unsigned long    isub;          /* Loop counter */
    struct scl_cmd   *scl_cmd_ptr;      /* Pointer to command line data */
    char    prmlin [TOFBUFLEN];    /* Parameters sub-record buffer */

    /*
     *  Copy pointers for ease of reference.
     */
    com_ptr = (struct scl_com *) argblk_ptr->user_ptrp;
    tpa_ptr = com_ptr->tpa_ptrps;
    scl_cmd_ptr = com_ptr->cmd_ptrps;
    lin_ptr = com_ptr->toflinc1024; /* 'tof' line buffer pointer */

    /*
     *  Do not encode anything if we have had an error in the command.
     */
    if (tpa_ptr->error_in_commandn) goto end;

    /*
     *  Output tracing information if required.
     */
#ifdef PSL_TRACING
    if (PSL_TEST_TRACE (PTRC_USR_USER1))
    {
        PSL_OUTPUT_TRACE ("\nEncoding a SUBROUTINE Command\n");
        PSL_OUTPUT_TRACE ("Operand Count = %ld\n" ,GET_OPD_COUNT (tpa_ptr));
        PSL_OUTPUT_TRACE ("Subroutine Parameter Count = %ld\n",
                          tpa_ptr->subprm_cntl);
    }
#endif

    /*
     *  If this is the first subroutine/sub-test which has been defined then
     *  we should take care to ensure that the main line code is correctly
     *  terminated. To this  end we encode an EXIT command at this point.
     */
    if ((tpa_ptr->subrtn_def_cntl == 0) &&
            ((!com_ptr->type_testn) ||
             (tpa_ptr->subtst_cntl == 0)))
    {
        tpa_ptr->exit_maskl = 0;    /* Clear exit mask */
        ENC_EXIT (argblk_ptr);      /* ...and encode EXIT command */
    }

    /*
     *  Check for this subroutine already being defined etc.
     */
    CHK_SUBRTN_DEF (argblk_ptr);    /* Check defined and resolve refs */

    /*
     *  Construct the 'parameters' sub-record by encoding the subroutine parameters
     *  (stored in the parsed operands linked list). The sub-record has the
     *  following format:
     *
     *  parameters p1 {{,pn}...}
     */
    (void) strcpy (prmlin, "parameters ");

    for (isub=0; isub != tpa_ptr->subprm_cntl; isub++)
    {
        ENC_OPD_BYPOS (com_ptr, prmlin, NULL, NULL);
        if (isub < (tpa_ptr->subprm_cntl - 1)) (void) strcat (prmlin, ",");

#ifdef PSL_TRACING
        if (PSL_TEST_TRACE (PTRC_USR_USER1))
        {
            PSL_OUTPUT_TRACE ("\nEncoded SUBROUTINE Parameter %ld\n", isub);
            PSL_OUTPUT_TRACE ("Operand Count = %ld\n" ,GET_OPD_COUNT (tpa_ptr));
        }
#endif

    }   /* End for ... */

    /*
     *  Write the following records to the 'TOF' line list:
     *
     *  Subroutine
     *  name <name>
     *  {parameters <parameters>}
     */
    APPEND_RECHDR (com_ptr, "Subroutine");

    (void) sprintf (lin_ptr, "name %s", tpa_ptr->subroutine_namec20);
    APPEND_TOFLIN (com_ptr, NULL);

    if (tpa_ptr->subprm_cntl > 0)
    {
        APPEND_TOFLIN (com_ptr, prmlin);
    }

    /*
     *  Clear the labels count (as sub-tests have their own labels) and increment
     *  the sub-test count.
     */
    tpa_ptr->lab_cntl = 0;      /* Clear label count */
    tpa_ptr->subrtn_def_cntl += 1;  /* One more sub-test definition */

end:
    ;
    return PSL_NORMAL;          /* Return success */
}

/******************************************************************************
 * @func    This routine is a Tparse action routine and is called to encode the
 *      'tof' lines required for a SUB-TEST command:
 *
 *      {Exit command}  {generated for first subroutine/sub-test only}
 *
 *      <PRE>
 *      Subtest
 *      name <name>
 *      </PRE>
 *
 *      An exit command is encoded before the first subroutine/sub-test
 *      in order to prevent execution continuing from the end of the
 *      main code section into the first subroutine/sub-test if no
 *      explicit Exit or Return has been specified in the script.
 *
 *      If we are in DEBUG then we will add the sub-test to the list
 *      of script modules.
 *
 * @rdesc   Tparse status (Always returns PSL_NORMAL).
 * @parm    Pointer to Tparse argument block.
 */
unsigned long ENC_SUBTST (struct tpaarg *argblk_ptr)
{
    /*
     *  Local data.
     */
    struct scl_com   *com_ptr;      /* SCL comon ata */
    struct scl_tpa   *tpa_ptr;      /* SCL Tparse data */
    char    *lin_ptr;      /* Pointer to line buffer */
    struct scl_cmd   *scl_cmd_ptr;      /* Pointer to command line data */

    /*
     *  Copy pointers for ease of reference.
     */
    com_ptr = (struct scl_com *) argblk_ptr->user_ptrp;
    tpa_ptr = com_ptr->tpa_ptrps;
    scl_cmd_ptr = com_ptr->cmd_ptrps;
    lin_ptr = com_ptr->toflinc1024; /* 'tof' line buffer pointer */

    /*
     *  Do not encode anything if we have had an error in the command.
     */
    if (tpa_ptr->error_in_commandn) goto end;

    /*
     *  Output tracing information if required.
     */
#ifdef PSL_TRACING
    if (PSL_TEST_TRACE (PTRC_USR_USER1))
    {
        PSL_OUTPUT_TRACE ("\nEncoding a SUB-TEST Command\n");
        PSL_OUTPUT_TRACE ("Operand Count = %ld\n" ,GET_OPD_COUNT (tpa_ptr));
    }
#endif

    /*
     *  If this is the first subroutine/sub-test which has been defined then
     *  we should take care to ensure that the main line code is correctly
     *  terminated. To this  end we encode an EXIT command at this point.
     */
    if ((tpa_ptr->subrtn_def_cntl == 0) &&
            ((!com_ptr->type_testn) ||
             (tpa_ptr->subtst_cntl == 0)))
    {
        tpa_ptr->exit_maskl = 0;    /* Clear exit mask */
        ENC_EXIT (argblk_ptr);      /* ...and encode EXIT command */
    }

    /*
     *  Write the following records to the 'TOF' line list:
     *
     *  Subtest
     *  name <name>
     */
    APPEND_RECHDR (com_ptr, "Subtest");

    (void) sprintf (lin_ptr, "name %s", tpa_ptr->subtst_namec20);
    APPEND_TOFLIN (com_ptr, NULL);

    /*
     *  Clear the labels count as subtests have their own labels.
     */
    tpa_ptr->lab_cntl = 0;      /* Clear label count */
    tpa_ptr->subtst_cntl += 1;      /* One more sub-test */

end:
    ;
    return PSL_NORMAL;          /* Return success */
}

/******************************************************************************
 * @func    This Tparse action routine is called after parsing a SUSPEND
 *      {TERMINAL} command. Its purpose is to append a Disable record to
 *      the list of 'TOF' lines. The Disable record has the following
 *      format:
 *
 *      <PRE>
 *      Disable
 *      target TOF_ENDTGT_EXECUTION
 *      </PRE>
 *
 *
 * @rdesc   Tparse status (Always returns PSL_NORMAL).
 * @parm    Pointer to Tparse argument block.
 */
unsigned long ENC_SUSPEND_TRM (struct tpaarg *argblk_ptr)
{
    /*
     *  Local data.
     */
    struct scl_com  *com_ptr;       /* SCL common data */
    struct scl_tpa  *tpa_ptr;       /* SCL Tparse data */
    char   *lin_ptr;       /* Pointer to line buffer */

    /*
     *  Copy pointers for ease of reference.
     */
    com_ptr = (struct scl_com *) argblk_ptr->user_ptrp;
    tpa_ptr = com_ptr->tpa_ptrps;
    lin_ptr = com_ptr->toflinc1024; /* 'tof' line buffer pointer */

    /*
     *  Do not encode anything if we have had an error in the command.
     */
    if (tpa_ptr->error_in_commandn) goto end;

    /*
     *  Output tracing information if required.
     */
#ifdef PSL_TRACING
    if (PSL_TEST_TRACE (PTRC_USR_USER1))
    {
        PSL_OUTPUT_TRACE ("Encoding a SUSPEND TERMINAL Command\n");
        PSL_OUTPUT_TRACE ("Operand Count = %ld\n" ,GET_OPD_COUNT (tpa_ptr));
    }
#endif

    /*
     *  Write the following records to the 'TOF' line list:
     *
     *  Disable
     *  target TOF_ENDTGT_EXECUTION
     */
    APPEND_RECHDR (com_ptr, "Disable");

    (void) sprintf (lin_ptr, "target %d", TOF_ENDTGT_EXECUTION);
    APPEND_TOFLIN (com_ptr, NULL);
end:
    ;
    return PSL_NORMAL;
}

/******************************************************************************
 * @func    This Tparse action routine is called after parsing a SYNCHRONIZE
 *      command. Its purpose is to append a Synchronize record to the
 *      linked list of 'TOF' lines. The Synchronize record has the
 *      following format:
 *
 *      <PRE>
 *      Synchronize
 *      {timeout <timeout_period>}
 *      {tmolab <timeout_label>}
 *      </PRE>
 *
 * @rdesc   Tparse status (Always returns PSL_NORMAL).
 * @parm    Pointer to Tparse argument block.
 */
unsigned long ENC_SYNCHRONIZE (struct tpaarg *argblk_ptr)
{
    /*
     *  Local data.
     */
    struct scl_com  *com_ptr;       /* SCL common data */
    struct scl_tpa  *tpa_ptr;       /* SCL Tparse data */
    char   *lin_ptr;           /* Pointer to line buffer */
    long   linlen;         /* Line length */

    /*
     *  Copy pointers for ease of reference.
     */
    com_ptr = (struct scl_com *) argblk_ptr->user_ptrp;
    tpa_ptr = com_ptr->tpa_ptrps;
    lin_ptr = com_ptr->toflinc1024; /* 'tof' line buffer pointer */

    /*
     *  Do not encode anything if we have had an error in the command.
     */
    if (tpa_ptr->error_in_commandn) goto end;

    /*
     *  Output tracing information if required.
     */
#ifdef PSL_TRACING
    if (PSL_TEST_TRACE (PTRC_USR_USER1))
    {
        PSL_OUTPUT_TRACE ("Encoding a SYNCHRONIZE Command\n");
    }
#endif

    /*
     *  Write the following records to the 'TOF' line list:
     *
     *  Synchronize
     *  {timeout <timeout_period>}
     *  {tmolab <timeout_label>}
     */
    APPEND_RECHDR (com_ptr, "Synchronize");

    if ((tpa_ptr->cmnd_optl & CMND_OPT_WAIT_PRD) != 0)
    {
        (void) strcpy (lin_ptr, "timeout ");
        linlen = strlen (lin_ptr);
        ENC_OPD_BYID (com_ptr, lin_ptr, CMND_OPT_WAIT_PRD);
        if (strlen (lin_ptr) != linlen) APPEND_TOFLIN (com_ptr, NULL);
    }

    if ((tpa_ptr->cmnd_optl & CMND_OPT_TIMEOUT) != 0)
    {
        (void) sprintf (lin_ptr, "tmolab %s", tpa_ptr->ontimeout_labelc20);
        APPEND_TOFLIN (com_ptr, lin_ptr);
    }

end:
    ;
    return PSL_NORMAL;
}

/******************************************************************************
 * @func    This Tparse action routine is called after parsing a TEST-CASE
 *      command. Its purpose is to append the appropriate record to the
 *      linked list of 'TOF' lines. This routine is used to encode
 *      START, FAIL, PASS and END TEST-CASE commands. A single routine is
 *      used for all TEST-CASE commands so that the 'description' record
 *      specified in a START TEST-CASE command can be easily saved (in a
 *      static variable) and be used in the associated PASS, FAIL and END
 *      TEST-CASE records. A single record also keeps all TEST-CASE encoding
 *      in one place.
 *
 *      The particular command to be encoded is identified by the parameter
 *      field of the Tparse argument block (argblk_ptr->paraml).
 *
 * @rdesc   Tparse status (Always returns PSL_NORMAL).
 * @parm    Pointer to Tparse argument block.
 */
unsigned long ENC_TESTCASE (struct tpaarg *argblk_ptr)
{
    /*
     *    Local data
     */
    struct scl_com  *com_ptr;           /* SCL common data */
    struct scl_tpa  *tpa_ptr;       /* SCL Tparse data */
    char   *lin_ptr;           /* Pointer to line buffer */
    static   char   dsclin [TOFBUFLEN]; /* Description sub-record buffer */

    /*
     *  Copy pointers for ease of reference.
     */
    com_ptr = (struct scl_com *) argblk_ptr->user_ptrp;
    tpa_ptr = com_ptr->tpa_ptrps;
    lin_ptr = com_ptr->toflinc1024; /* 'tof' line buffer pointer */

    /*
     *  Do not encode anything if we have had an error in the command.
     */
    if (tpa_ptr->error_in_commandn) goto end;

    /*
     *    Output tracing information if required.
     */
#ifdef PSL_TRACING
    if (PSL_TEST_TRACE (PTRC_USR_USER1))
    {
        PSL_OUTPUT_TRACE ("Encoding a TEST-CASE Command\n");
        PSL_OUTPUT_TRACE ("Operand Count = %ld\n", GET_OPD_COUNT (tpa_ptr));
    }
#endif

    /*
     *  If START TEST-CASE command, create a local copy of the description
     *  sub-record as this is also required by the associated PASS, FAIL and
     *  END TEST-CASE commands. Also write the following records to the 'TOF'
     *  line list:
     *
     *  Tcstart
     *  description <description>
     */
    if (argblk_ptr->paraml == CMND_START_TESTCASE)
    {
        APPEND_RECHDR (com_ptr, "Tcstart");

        (void) strcpy (dsclin, "description ");
        ENC_OPD_BYPOS (com_ptr, dsclin, NULL, NULL);
        APPEND_TOFLIN (com_ptr, dsclin);
    }

    /*
     *  Elseif FAIL TEST-CASE command, write the following records to the 'TOF'
     *  line list:
     *
     *  Tcterm
     *  result <result_code>
     *  description <description>
     *  {golab <label>}
     */
    else if (argblk_ptr->paraml == CMND_FAIL_TESTCASE)
    {
        APPEND_RECHDR (com_ptr, "Tcterm");

        (void) sprintf (lin_ptr, "result %d", TOF_TCTRES_FAIL);
        APPEND_TOFLIN (com_ptr, NULL);

        APPEND_TOFLIN (com_ptr, dsclin);

        if ((tpa_ptr->testcase_flagsl & TCAS_GOTO_TC) != 0)
        {
            (void) strcpy (lin_ptr, "golab ");
            RESOLVE_LABEL (argblk_ptr, tpa_ptr->label_namec16,
                           tpa_ptr->label_lenl);
            APPEND_TOFLIN (com_ptr, NULL);
        }

        tpa_ptr->test_fgl = 1;      /* PASS/FAIL TEST-CASE just parsed */
    }

    /*
     *  Elseif PASS TEST-CASE command, write the following records to the 'TOF'
     *  line list:
     *
     *  Tcterm
     *  result <result_code>
     *  description <description>
     *  {golab <label>}
     */
    else if (argblk_ptr->paraml == CMND_PASS_TESTCASE)
    {
        APPEND_RECHDR (com_ptr, "Tcterm");

        (void) sprintf (lin_ptr, "result %d", TOF_TCTRES_PASS);
        APPEND_TOFLIN (com_ptr, NULL);

        APPEND_TOFLIN (com_ptr, dsclin);

        if ((tpa_ptr->testcase_flagsl & TCAS_GOTO_TC) != 0)
        {
            (void) strcpy (lin_ptr, "golab ");
            RESOLVE_LABEL (argblk_ptr, tpa_ptr->label_namec16,
                           tpa_ptr->label_lenl);
            APPEND_TOFLIN (com_ptr, NULL);
        }

        tpa_ptr->test_fgl = 1;      /* PASS/FAIL TEST-CASE just parsed */
    }

    /*
     *  Elseif END TEST-CASE command, write the following records to the 'TOF'
     *  line list:
     *
     *  Tcend
     *  description <description>
     */
    else if (argblk_ptr->paraml == CMND_END_TESTCASE)
    {
        APPEND_RECHDR (com_ptr, "Tcend");
        APPEND_TOFLIN (com_ptr, dsclin);

        tpa_ptr->testcase_flagsl = 0;   /* Show no TEST-CASE in progress */
    }

end:
    return PSL_NORMAL;          /* Return success */
}

/******************************************************************************
 * @func    This Tparse action routine is called after parsing a START TIMER or
 *      END TIMER command. Its purpose is to append the appropriate record
 *      to the linked list of 'TOF' lines. The commands have the following
 *      format:
 *
 *      <PRE>
 *      Stimer | Etimer
 *      name <timer_name>
 *      </PRE>
 *
 *      where Stimer is written for a start timer and Etimer for an end
 *      timer.
 *
 *      The timer command to be encoded is identified by the parameter field
 *      of the Tparse argument block (argblk_ptr->paraml).
 *
 * @rdesc   Tparse status (Always returns PSL_NORMAL).
 * @parm    Pointer to Tparse argument block.
 */
unsigned long ENC_TIMER (struct tpaarg *argblk_ptr)
{
    /*
     *  Local data.
     */
    struct scl_com  *com_ptr;           /* SCL common data */
    struct scl_tpa  *tpa_ptr;       /* SCL Tparse data */
    char   *lin_ptr;           /* Pointer to line buffer */

    /*
     *  Copy pointers for ease of reference.
     */
    com_ptr = (struct scl_com *) argblk_ptr->user_ptrp;
    tpa_ptr = com_ptr->tpa_ptrps;
    lin_ptr = com_ptr->toflinc1024; /* 'tof' line buffer pointer */

    /*
     *  Do not encode anything if we have had an error in the command.
     */
    if (tpa_ptr->error_in_commandn) goto end;

    /*
     *    Output tracing information if required.
     */
#ifdef PSL_TRACING
    if (PSL_TEST_TRACE (PTRC_USR_USER1))
    {
        PSL_OUTPUT_TRACE ("Encoding a TIMER Command\n");
        PSL_OUTPUT_TRACE ("Operand Count = %ld\n", GET_OPD_COUNT (tpa_ptr));
    }
#endif

    /*
     *  If START TEST-CASE command, create a local copy of the description
     *  sub-record as this is also required by the associated PASS, FAIL and
     *  END TEST-CASE commands. Also write the following records to the 'TOF'
     *  line list:
     *
     *  Stimer | Etimer
     *  name <timer_name>
     */
    if (argblk_ptr->paraml == CMND_START_TIMER)
    {
        APPEND_RECHDR (com_ptr, "Stimer");
    }
    else if (argblk_ptr->paraml == CMND_END_TIMER)
    {
        APPEND_RECHDR (com_ptr, "Etimer");
    }

    (void) strcpy (lin_ptr, "name ");
    ENC_OPD_BYPOS (com_ptr, NULL, NULL, NULL);
    APPEND_TOFLIN (com_ptr, NULL);
end:
    return PSL_NORMAL;          /* Return success */
}

/******************************************************************************
 * @func    This Tparse action routine is called after parsing a TRACE OFF
 *      command. Its purpose is to append a Disable record to the list of
 *      'TOF' lines. The Disable record has the following format:
 *
 *      <PRE>
 *      Disable
 *      target TOF_ENDTGT_TRACING
 *      </PRE>
 *
 *
 * @rdesc   Tparse status (Always returns PSL_NORMAL).
 * @parm    Pointer to Tparse argument block.
 */
unsigned long ENC_TRACE_OFF (struct tpaarg *argblk_ptr)
{
    /*
     *  Local data.
     */
    struct scl_com  *com_ptr;       /* SCL common data */
    struct scl_tpa  *tpa_ptr;       /* SCL Tparse data */
    char   *lin_ptr;           /* Pointer to line buffer */

    /*
     *  Copy pointers for ease of reference.
     */
    com_ptr = (struct scl_com *) argblk_ptr->user_ptrp;
    tpa_ptr = com_ptr->tpa_ptrps;
    lin_ptr = com_ptr->toflinc1024; /* 'tof' line buffer pointer */

    /*
     *  Do not encode anything if we have had an error in the command.
     */
    if (tpa_ptr->error_in_commandn) goto end;

    /*
     *  Output tracing information if required.
     */
#ifdef PSL_TRACING
    if (PSL_TEST_TRACE (PTRC_USR_USER1))
    {
        PSL_OUTPUT_TRACE ("\nEncoding a TRACE OFF command\n");
        PSL_OUTPUT_TRACE ("Operand Count = %ld\n" ,GET_OPD_COUNT (tpa_ptr));
    }
#endif

    /*
     *  Write the following records to the 'TOF' line list:
     *
     *  Disable
     *  target TOF_ENDTGT_TRACING
     */
    APPEND_RECHDR (com_ptr, "Disable");

    (void) sprintf (lin_ptr, "target %d", TOF_ENDTGT_TRACING);
    APPEND_TOFLIN (com_ptr, NULL);
end:
    ;
    return PSL_NORMAL;          /* Return success */
}

/******************************************************************************
 * @func    This Tparse action routine is called after parsing a TRACE ON
 *      command. Its purpose is to append an Enable record to the list of
 *      'TOF' lines. The Enable record has the following format:
 *
 *      <PRE>
 *      Enable
 *      target TOF_ENDTGT_TRACING
 *      </PRE>
 *
 *
 * @rdesc   Tparse status (Always returns PSL_NORMAL).
 * @parm    Pointer to Tparse argument block.
 */
unsigned long ENC_TRACE_ON (struct tpaarg *argblk_ptr)
{
    /*
     *  Local data.
     */
    struct scl_com  *com_ptr;       /* SCL common data */
    struct scl_tpa  *tpa_ptr;       /* SCL Tparse data */
    char   *lin_ptr;           /* Pointer to line buffer */

    /*
     *  Copy pointers for ease of reference.
     */
    com_ptr = (struct scl_com *) argblk_ptr->user_ptrp;
    tpa_ptr = com_ptr->tpa_ptrps;
    lin_ptr = com_ptr->toflinc1024; /* 'tof' line buffer pointer */

    /*
     *  Do not encode anything if we have had an error in the command.
     */
    if (tpa_ptr->error_in_commandn) goto end;

    /*
     *  Output tracing information if required.
     */
#ifdef PSL_TRACING
    if (PSL_TEST_TRACE (PTRC_USR_USER1))
    {
        PSL_OUTPUT_TRACE ("\nEncoding a TRACE ON command\n");
        PSL_OUTPUT_TRACE ("Operand Count = %ld\n" ,GET_OPD_COUNT (tpa_ptr));
    }
#endif

    /*
     *  Write the following records to the 'TOF' line list:
     *
     *  Enable
     *  target TOF_ENDTGT_TRACING
     */
    APPEND_RECHDR (com_ptr, "Enable");

    (void) sprintf (lin_ptr, "target %d", TOF_ENDTGT_TRACING);
    APPEND_TOFLIN (com_ptr, NULL);
end:
    ;
    return PSL_NORMAL;          /* Return success */
}

/******************************************************************************
 * @func    This routine is a Tparse action routine and is called to encode the
 *      'tof' lines required for a WAIT n command:
 *
 *      <PRE>
 *      Wait
 *      tdelay 1000 * <time_delay>      // If WAIT UNIT Seconds
 *      tdelay <time_delay>         // If WAIT UNIT Milliseconds
 *      </PRE>
 *
 *      The tdelay value is multiplied by 1000 if the wait unit specified
 *      in the Environment section is Seconds (or the wait unit has been
 *      defaulted), because wait periods in TOF are in milliseconds.
 *
 * @rdesc   Tparse status (Always returns PSL_NORMAL).
 * @parm    Pointer to Tparse argument block.
 */
unsigned long ENC_WAITPRD (struct tpaarg *argblk_ptr)
{
    /*
     *  Local data.
     */
    struct scl_com   *com_ptr;      /* SCL comon ata */
    struct scl_tpa   *tpa_ptr;      /* SCL Tparse data */
    char    *lin_ptr;      /* Pointer to line buffer */

    /*
     *  Copy pointers for ease of reference.
     */
    com_ptr = (struct scl_com *) argblk_ptr->user_ptrp;
    tpa_ptr = com_ptr->tpa_ptrps;
    lin_ptr = com_ptr->toflinc1024; /* 'tof' line buffer pointer */

    /*
     *  Do not encode anything if we have had an error in the command.
     */
    if (tpa_ptr->error_in_commandn) goto end;

    /*
     *  Output tracing information if required.
     */
#ifdef PSL_TRACING
    if (PSL_TEST_TRACE (PTRC_USR_USER1))
    {
        PSL_OUTPUT_TRACE ("\nEncoding a WAIT n Command\n");
        PSL_OUTPUT_TRACE ("Operand Count = %ld\n" ,GET_OPD_COUNT (tpa_ptr));
    }
#endif

    /*
     *  Write the following records to the 'TOF' line list:
     *
     *  Wait
     *  tdelay 1000 * <time_delay>  // If Wait Unit Seconds
     *  tdelay <time_delay>     // If Wait Unit Milliseconds
     */
    APPEND_RECHDR (com_ptr, "Wait");

    (void) strcpy (lin_ptr, "tdelay ");
    if (tpa_ptr->wait_unitl == WAIT_SECONDS) strcat (lin_ptr, "1000 * ");
    ENC_OPD_BYPOS (com_ptr, NULL, NULL, NULL);
    APPEND_TOFLIN (com_ptr, NULL);

end:
    ;
    return PSL_NORMAL;          /* Return success */
}

/******************************************************************************
 * @func    This Tparse action routine is called after parsing all the
 *      operands in a WAIT FOR SCRIPT command. Its purpose is to complete
 *      the Wtest record. This is done by flushing the 'expr' sub-record
 *      to the TOF line linked list and also appending the 'tomlab'
 *      sub-record. The Wtest record has the following format:
 *
 *      <PRE>
 *      Wtest
 *      type <type_code>
 *      {timeout <period>}
 *      --------------------
 *      expr <expression>
 *      {tmolab <label>}
 *      </PRE>
 *
 *      This routine writes the tmolab sub-record (if any) and the expr
 *      sub-record only.
 *
 * @rdesc   Tparse status (Always returns PSL_NORMAL).
 * @parm    Pointer to Tparse argument block.
 */
unsigned long ENC_WATSCR_END (struct tpaarg *argblk_ptr)
{
    /*
     *  Local data.
     */
    struct scl_com  *com_ptr;       /* SCL common data */
    struct scl_tpa  *tpa_ptr;       /* SCL Tparse data */
    char   *lin_ptr;       /* Pointer to line buffer */

    /*
     *  Copy pointers for ease of reference.
     */
    com_ptr = (struct scl_com *) argblk_ptr->user_ptrp;
    tpa_ptr = com_ptr->tpa_ptrps;
    lin_ptr = com_ptr->toflinc1024; /* 'tof' line buffer pointer */

    /*
     *  Do not encode anything if we have had an error in the command.
     */
    if (tpa_ptr->error_in_commandn) goto end;

    /*
     *  Output tracing information if required.
     */
#ifdef PSL_TRACING
    if (PSL_TEST_TRACE (PTRC_USR_USER1))
    {
        PSL_OUTPUT_TRACE ("\nEncoding the end of a WAIT FOR SCRIPT Command\n");
        PSL_OUTPUT_TRACE ("Operand Count = %ld\n" ,GET_OPD_COUNT (tpa_ptr));
    }
#endif

    /*
     *  Write the following records to the 'TOF' line list:
     *
     *  expr <expression>
     *  {tmolab <label>}
     *
     *  First flush the 'expr' line from the default 'TOF' line buffer.
     */
    APPEND_TOFLIN (com_ptr, NULL);

    if ((tpa_ptr->cmnd_optl & CMND_OPT_TIMEOUT) != 0)
    {
        (void) sprintf (lin_ptr, "tmolab %s", tpa_ptr->ontimeout_labelc20);
        APPEND_TOFLIN (com_ptr, NULL);
    }
end:
    ;
    return PSL_NORMAL;          /* Return success */
}

/******************************************************************************
 * @func    This routine is a Tparse action routine and is invoked immediately
 *      after parsing the timeout period operand (ifa ny) in a WAIT FOR
 *      SCRIPT command to encode the 'tof' lines required at the start of
 *      the Wtest record written for a WAIT FOR SCRIPT command. The Wtest
 *      record has the following format:
 *
 *      <PRE>
 *      Wtest
 *      type <type_code>
 *      {timeout <period>}
 *      --------------------
 *      expr <expression>
 *      {tmolab <label>}
 *      </PRE>
 *
 *      This routine writes all the lines down to the timeout sub-record
 *      (if any) inclusive.
 *
 *      This routine also initialises the default TOF line buffer with
 *      the "expr " prefix.
 *
 * @rdesc   Tparse status (Always returns PSL_NORMAL).
 * @parm    Pointer to Tparse argument block.
 */
unsigned long ENC_WATSCR_START (struct tpaarg *argblk_ptr)
{
    /*
     *  Local data.
     */
    struct scl_com  *com_ptr;       /* SCL common data */
    struct scl_tpa  *tpa_ptr;       /* SCL Tparse data */
    char   *lin_ptr;       /* Pointer to line buffer */

    /*
     *  Copy pointers for ease of reference.
     */
    com_ptr = (struct scl_com *) argblk_ptr->user_ptrp;
    tpa_ptr = com_ptr->tpa_ptrps;
    lin_ptr = com_ptr->toflinc1024; /* 'tof' line buffer pointer */

    /*
     *  Do not encode anything if we have had an error in the command.
     */
    if (tpa_ptr->error_in_commandn) goto end;

    /*
     *  Output tracing information if required.
     */
#ifdef PSL_TRACING
    if (PSL_TEST_TRACE (PTRC_USR_USER1))
    {
        PSL_OUTPUT_TRACE ("\nEncoding the start of a WAIT FOR SCRIPT Command\n");
        PSL_OUTPUT_TRACE ("Operand Count = %ld\n" ,GET_OPD_COUNT (tpa_ptr));
    }
#endif

    /*
     *  Write the following records to the 'TOF' line list:
     *
     *  Wtest
     *  type <type_code>
     *  {timeout <period>}
     */
    APPEND_RECHDR (com_ptr, "Wtest");

    (void) sprintf (lin_ptr, "type %d", TOF_WATTYP_SCRIPT);
    APPEND_TOFLIN (com_ptr, lin_ptr);

    /*
     *  If a timeout period was specified encode a 'timeout' sub-record and remove
     *  the timeout operand from the Parsed Operand linked list.
     */
    if ((tpa_ptr->cmnd_optl & CMND_OPT_WAIT_PRD) != 0)
    {
        (void) strcpy (lin_ptr, "timeout ");
        ENC_OPD_BYPOS (com_ptr, lin_ptr, NULL, NULL);
        APPEND_TOFLIN (com_ptr, NULL);
    }

    /*
     *  Initialise the default 'TOF' line buffer for the 'expr' sub-record.
     */
    (void) strcpy (lin_ptr, "expr ");

end:
    ;
    return PSL_NORMAL;          /* Return success */
}

/******************************************************************************
 * @func    This Tparse action routine is called after parsing a WAIT FOR
 *      SEMAPHORE command. Its purpose is to append a Wtsemaphore record to
 *      the list of 'TOF' lines. The Wtsemaphore record has the following
 *      format:
 *
 *      <PRE>
 *      Wtsemaphore
 *      scope <scope>
 *      name <semaphore_name>
 *      {timeout <timeout_period>}
 *      {tmolab <timeout_label>}
 *      {errlab <error_label>}
 *      </PRE>
 *
 * @rdesc   Tparse status (Always returns PSL_NORMAL).
 * @parm    Pointer to Tparse argument block.
 */
unsigned long ENC_WAIT_SEMAPH (struct tpaarg *argblk_ptr)
{
    /*
     *  Local data.
     */
    struct scl_com  *com_ptr;       /* SCL common data */
    struct scl_tpa  *tpa_ptr;       /* SCL Tparse data */
    unsigned long   semscp;         /* Wait for semaphore scope flag */
    char   *lin_ptr;       /* Pointer to line buffer */
    char   semnam [TOFBUFLEN]; /* Semaphore name */
    char   tmoprdlin [TOFBUFLEN];  /* Timeout line */

    /*
     *  Copy pointers for ease of reference.
     */
    com_ptr = (struct scl_com *) argblk_ptr->user_ptrp;
    tpa_ptr = com_ptr->tpa_ptrps;
    lin_ptr = com_ptr->toflinc1024; /* 'tof' line buffer pointer */

    /*
     *  Do not encode anything if we have had an error in the command.
     */
    if (tpa_ptr->error_in_commandn) goto end;

    /*
     *  Output tracing information if required.
     */
#ifdef PSL_TRACING
    if (PSL_TEST_TRACE (PTRC_USR_USER1))
    {
        PSL_OUTPUT_TRACE ("Encoding a WAIT FOR SEMAPHORE Command\n");
        PSL_OUTPUT_TRACE ("Operand Count = %ld\n" ,GET_OPD_COUNT (tpa_ptr));
    }
#endif

    /*
     *  Get the wait for semaphore scope flag.
     */
    if ((tpa_ptr->pipe_flagsl & PIPFLG_TESTWIDE) != 0)
    {
        semscp = TOF_SEMSCP_TESTWIDE;
    }
    else
    {
        semscp = TOF_SEMSCP_LOCAL;
    }

    /*
     *  Get the wait period, if any
     */
    if ((tpa_ptr->cmnd_optl & CMND_OPT_WAIT_PRD) != 0)
    {
        (void) strcpy (tmoprdlin, "timeout ");
        ENC_OPD_BYPOS (com_ptr, tmoprdlin, NULL, NULL);
    }

    /*
     *  Get semaphore name.
     */
    semnam [0] = '\0';
    ENC_OPD_BYPOS (com_ptr, semnam, NULL, NULL);

    /*
     *  Write the following records to the 'TOF' line list:
     *
     *  Wtsemaphore
     *  scope <scope>
     *  name <semaphore_name>
     *  {timeout <timeout_period>}
     *  {tmolab <timeout_label>}
     *  {errlab <error_label>}
     */
    APPEND_RECHDR (com_ptr, "Wtsemaphore");

    (void) sprintf (lin_ptr, "scope %d", semscp);
    APPEND_TOFLIN (com_ptr, NULL);

    (void) sprintf (lin_ptr, "name %s", semnam);
    APPEND_TOFLIN (com_ptr, NULL);

    if ((tpa_ptr->cmnd_optl & CMND_OPT_WAIT_PRD) != 0)
    {
        APPEND_TOFLIN (com_ptr, tmoprdlin);
    }

    if ((tpa_ptr->cmnd_optl & CMND_OPT_TIMEOUT) != 0)
    {
        (void) sprintf (lin_ptr, "tmolab %s", tpa_ptr->ontimeout_labelc20);
        APPEND_TOFLIN (com_ptr, lin_ptr);
    }

    if (strlen (tpa_ptr->onerror_labelc16) != 0)
    {
        (void) strcpy (lin_ptr, "errlab ");
        RESOLVE_LABEL (argblk_ptr,
                       tpa_ptr->onerror_labelc16,
                       tpa_ptr->onerror_lenl);
        APPEND_TOFLIN (com_ptr, NULL);
    }

end:
    ;
    return PSL_NORMAL;          /* Return success */
}

/******************************************************************************
 * @func    This Tparse action routine is called after parsing all the
 *      operands in a WAIT FOR TEST command. Its purpose is to complete
 *      the Wtest record. This is done by flushing the 'expr' sub-record
 *      to the TOF line linked list and also appending the 'tomlab'
 *      sub-record. The Wtest record has the following format:
 *
 *      <PRE>
 *      Wtest
 *      type <type_code>
 *      {timeout <period>}
 *      --------------------
 *      expr <expression>
 *      {tmolab <label>}
 *      </PRE>
 *
 *      This routine writes the tmolab sub-record (if any) and the expr
 *      sub-record only.
 *
 * @rdesc   Tparse status (Always returns PSL_NORMAL).
 * @parm    Pointer to Tparse argument block.
 */
unsigned long ENC_WATTST_END (struct tpaarg *argblk_ptr)
{
    /*
     *  Local data.
     */
    struct scl_com  *com_ptr;       /* SCL common data */
    struct scl_tpa  *tpa_ptr;       /* SCL Tparse data */
    char   *lin_ptr;       /* Pointer to line buffer */

    /*
     *  Copy pointers for ease of reference.
     */
    com_ptr = (struct scl_com *) argblk_ptr->user_ptrp;
    tpa_ptr = com_ptr->tpa_ptrps;
    lin_ptr = com_ptr->toflinc1024; /* 'tof' line buffer pointer */

    /*
     *  Do not encode anything if we have had an error in the command.
     */
    if (tpa_ptr->error_in_commandn) goto end;

    /*
     *  Output tracing information if required.
     */
#ifdef PSL_TRACING
    if (PSL_TEST_TRACE (PTRC_USR_USER1))
    {
        PSL_OUTPUT_TRACE ("\nEncoding the end of a WAIT FOR TEST Command\n");
        PSL_OUTPUT_TRACE ("Operand Count = %ld\n" ,GET_OPD_COUNT (tpa_ptr));
    }
#endif

    /*
     *  Write the following records to the 'TOF' line list:
     *
     *  expr <expression>
     *  {tmolab <label>}
     *
     *  First flush the 'expr' line from the default 'TOF' line buffer.
     */
    APPEND_TOFLIN (com_ptr, NULL);

    if ((tpa_ptr->cmnd_optl & CMND_OPT_TIMEOUT) != 0)
    {
        (void) sprintf (lin_ptr, "tmolab %s", tpa_ptr->ontimeout_labelc20);
        APPEND_TOFLIN (com_ptr, NULL);
    }
end:
    ;
    return PSL_NORMAL;          /* Return success */
}

/******************************************************************************
 * @func    This routine is a Tparse action routine and is invoked immediately
 *      after parsing the timeout period operand (ifa ny) in a WAIT FOR
 *      TEST command to encode the 'tof' lines required at the start of
 *      the Wtest record written for a WAIT FOR TEST command. The Wtest
 *      record has the following format:
 *
 *      <PRE>
 *      Wtest
 *      type <type_code>
 *      {timeout <period>}
 *      --------------------
 *      expr <expression>
 *      {tmolab <label>}
 *      </PRE>
 *
 *      This routine writes all the lines down to the timeout sub-record
 *      (if any) inclusive.
 *
 *      This routine also initialises the default TOF line buffer with
 *      the "expr " prefix.
 *
 * @rdesc   Tparse status (Always returns PSL_NORMAL).
 * @parm    Pointer to Tparse argument block.
 */
unsigned long ENC_WATTST_START (struct tpaarg *argblk_ptr)
{
    /*
     *  Local data.
     */
    struct scl_com  *com_ptr;       /* SCL common data */
    struct scl_tpa  *tpa_ptr;       /* SCL Tparse data */
    char   *lin_ptr;       /* Pointer to line buffer */

    /*
     *  Copy pointers for ease of reference.
     */
    com_ptr = (struct scl_com *) argblk_ptr->user_ptrp;
    tpa_ptr = com_ptr->tpa_ptrps;
    lin_ptr = com_ptr->toflinc1024; /* 'tof' line buffer pointer */

    /*
     *  Do not encode anything if we have had an error in the command.
     */
    if (tpa_ptr->error_in_commandn) goto end;

    /*
     *  Output tracing information if required.
     */
#ifdef PSL_TRACING
    if (PSL_TEST_TRACE (PTRC_USR_USER1))
    {
        PSL_OUTPUT_TRACE ("\nEncoding the start of a WAIT FOR TEST Command\n");
        PSL_OUTPUT_TRACE ("Operand Count = %ld\n" ,GET_OPD_COUNT (tpa_ptr));
    }
#endif

    /*
     *  Write the following records to the 'TOF' line list:
     *
     *  Wtest
     *  type <type_code>
     *  {timeout <period>}
     */
    APPEND_RECHDR (com_ptr, "Wtest");

    (void) sprintf (lin_ptr, "type %d", TOF_WATTYP_TEST);
    APPEND_TOFLIN (com_ptr, lin_ptr);

    /*
     *  If a timeout period was specified encode a 'timeout' sub-record and remove
     *  the timeout operand from the Parsed Operand linked list.
     */
    if ((tpa_ptr->cmnd_optl & CMND_OPT_WAIT_PRD) != 0)
    {
        (void) strcpy (lin_ptr, "timeout ");
        ENC_OPD_BYPOS (com_ptr, lin_ptr, NULL, NULL);
        APPEND_TOFLIN (com_ptr, NULL);
    }

    /*
     *  Initialise the default 'TOF' line buffer for the 'expr' sub-record.
     */
    (void) strcpy (lin_ptr, "expr ");

end:
    ;
    return PSL_NORMAL;          /* Return success */
}

/******************************************************************************
 * @func    This Tparse action routine is called after parsing a WHEN ... CALL
 *      command. Its purpose is to append a CALL subroutine command record
 *      and an Ewhen record to the list of 'TOF' lines. These records
 *      have the following format:
 *
 *      <PRE>
 *      CALL subroutine command record
 *
 *      Ewhen
 *      </PRE>
 *
 * @rdesc   Tparse status (Always returns PSL_NORMAL).
 * @parm    Pointer to Tparse argument block.
 */
unsigned long ENC_WHEN_CALL (struct tpaarg *argblk_ptr)
{
    /*
     *  Local data.
     */
    struct scl_com  *com_ptr;       /* SCL common data */
    struct scl_tpa  *tpa_ptr;       /* SCL Tparse data */

    /*
     *  Copy pointers for ease of reference.
     */
    com_ptr = (struct scl_com *) argblk_ptr->user_ptrp;
    tpa_ptr = com_ptr->tpa_ptrps;

    /*
     *  Do not encode anything if we have had an error in the command.
     */
    if (tpa_ptr->error_in_commandn) goto end;

    /*
     *  Output tracing information if required.
     */
#ifdef PSL_TRACING
    if (PSL_TEST_TRACE (PTRC_USR_USER1))
    {
        PSL_OUTPUT_TRACE ("Encoding a WHEN ... CALL Command\n");
        PSL_OUTPUT_TRACE ("Operand Count = %ld\n" ,GET_OPD_COUNT (tpa_ptr));
    }
#endif

    /*
    /*
     *  Write the following records to the 'TOF' line list:
     *
     *  CALL subroutine command record
     *
     *  Ewhen
     */
    (void) ENC_CALL_SUBROUTINE (argblk_ptr);

    APPEND_RECHDR (com_ptr, "Ewhen");

end:
    ;
    return PSL_NORMAL;
}

/******************************************************************************
 * @func    This Tparse action routine is called after parsing the conditional
 *      expression within a WHEN command. Its purpose is to append a When
 *      record to the list of 'TOF' lines. The When record has the
 *      following format:
 *
 *      <PRE>
 *      When
 *      expr <condition>
 *      {options <options>}
 *      </PRE>
 *
 *      At the end of command parsing the When action object records (for
 *      a CALL or a GOTO) are encoded followed by an Ewhen record.
 *
 * @rdesc   Tparse status (Always returns PSL_NORMAL).
 * @parm    Pointer to Tparse argument block.
 */
unsigned long ENC_WHEN_COND (struct tpaarg *argblk_ptr)
{
    /*
     *  Local data.
     */
    struct scl_com  *com_ptr;       /* SCL common data */
    struct scl_tpa  *tpa_ptr;       /* SCL Tparse data */
    char   *lin_ptr;       /* Pointer to line buffer */
    unsigned long   whenopt = 0;        /* When options mask */

    /*
     *  Copy pointers for ease of reference.
     */
    com_ptr = (struct scl_com *) argblk_ptr->user_ptrp;
    tpa_ptr = com_ptr->tpa_ptrps;
    lin_ptr = com_ptr->toflinc1024; /* 'tof' line buffer pointer */

    /*
     *  Do not encode anything if we have had an error in the command.
     */
    if (tpa_ptr->error_in_commandn) goto end;

    /*
     *  Output tracing information if required.
     */
#ifdef PSL_TRACING
    if (PSL_TEST_TRACE (PTRC_USR_USER1))
    {
        PSL_OUTPUT_TRACE ("Encoding a WHEN (condition) Command\n");
        PSL_OUTPUT_TRACE ("Operand Count = %ld\n" ,GET_OPD_COUNT (tpa_ptr));
    }
#endif

    /*
     *  Set the TOF When options mask.
     */
    if ((tpa_ptr->when_maskl & WHEN_NOCANCEL) != 0)
    {
        whenopt |= TOF_WHNOPT_NOCANCEL;
    }

    /*
    /*
     *  Write the following records to the 'TOF' line list:
     *
     *  When
     *  expr <condition>
     *  options <option>
     */
    APPEND_RECHDR (com_ptr, "When");

    /*  Flush the 'expr <condition>' line from the default 'TOF' line buffer */
    APPEND_TOFLIN (com_ptr, NULL);

    (void) sprintf (lin_ptr, "options %d", whenopt);
    APPEND_TOFLIN (com_ptr, NULL);

end:
    ;
    return PSL_NORMAL;
}

/******************************************************************************
 * @func    This Tparse action routine is called after parsing a WHEN ... GOTO
 *      command. Its purpose is to append a GOTO command record
 *      and an Ewhen record to the list of 'TOF' lines. These records
 *      have the following format:
 *
 *      <PRE>
 *      GOTO command record
 *
 *      Ewhen
 *      </PRE>
 *
 * @rdesc   Tparse status (Always returns PSL_NORMAL).
 * @parm    Pointer to Tparse argument block.
 */
unsigned long ENC_WHEN_GOTO (struct tpaarg *argblk_ptr)
{
    /*
     *  Local data.
     */
    struct scl_com  *com_ptr;       /* SCL common data */
    struct scl_tpa  *tpa_ptr;       /* SCL Tparse data */

    /*
     *  Copy pointers for ease of reference.
     */
    com_ptr = (struct scl_com *) argblk_ptr->user_ptrp;
    tpa_ptr = com_ptr->tpa_ptrps;

    /*
     *  Do not encode anything if we have had an error in the command.
     */
    if (tpa_ptr->error_in_commandn) goto end;

    /*
     *  Output tracing information if required.
     */
#ifdef PSL_TRACING
    if (PSL_TEST_TRACE (PTRC_USR_USER1))
    {
        PSL_OUTPUT_TRACE ("Encoding a WHEN ... GOTO Command\n");
        PSL_OUTPUT_TRACE ("Operand Count = %ld\n" ,GET_OPD_COUNT (tpa_ptr));
    }
#endif

    /*
    /*
     *  Write the following records to the 'TOF' line list:
     *
     *  GOTO command record
     *
     *  Ewhen
     */
    (void) ENC_GOTO (argblk_ptr);

    APPEND_RECHDR (com_ptr, "Ewhen");

end:
    ;
    return PSL_NORMAL;
}

/******************************************************************************
 * @func    This Tparse action routine is called after parsing a WRITE (file)
 *      command. Its purpose is to append a Write record to the list of
 *      'TOF' lines. The Write record has the following format:
 *
 *      <PRE>
 *      Write
 *      string <string_value>
 *      filid <file_id>
 *      {errlab <error_label>}
 *      </PRE>
 *
 *      Note that if this is a passed file, the index is incremented by
 *      the constant PASSED_FILE_BASE, to show that it is a passed file.
 *
 * @rdesc   Tparse status (Always returns PSL_NORMAL).
 * @parm    Pointer to Tparse argument block.
 */
unsigned long ENC_WRITE_FILE (struct tpaarg *argblk_ptr)
{
    /*
     *  Local data.
     */
    struct scl_com  *com_ptr;       /* SCL common data */
    struct scl_tpa  *tpa_ptr;       /* SCL Tparse data */
    struct fattbl   *fat_ptr;       /* File Access Table pointer */
    char   *lin_ptr;       /* Pointer to line buffer */
    char   wrtstrlin [TOFBUFLEN];  /* Write string line */
    char   filidlin [TOFBUFLEN];   /* File Id line */

    /*
     *  Copy pointers for ease of reference.
     */
    com_ptr = (struct scl_com *) argblk_ptr->user_ptrp;
    tpa_ptr = com_ptr->tpa_ptrps;
    lin_ptr = com_ptr->toflinc1024; /* 'tof' line buffer pointer */
    fat_ptr = tpa_ptr->fat_begps + tpa_ptr->open_fididxl - 1;
    /* Point to current entry in table */
    /*
     *  Do not encode anything if we have had an error in the command.
     */
    if (tpa_ptr->error_in_commandn) goto end;

    /*
     *  Output tracing information if required.
     */
#ifdef PSL_TRACING
    if (PSL_TEST_TRACE (PTRC_USR_USER1))
    {
        PSL_OUTPUT_TRACE ("Encoding a WRITE (file) Command\n");
        PSL_OUTPUT_TRACE ("Operand Count = %ld\n" ,GET_OPD_COUNT (tpa_ptr));
    }
#endif

    /*
     *    Check to see if this file has been DEFINED (by the FILE statement).
     *    If it has then we set the referenced flag to ACCESSED_FILE
     *
     *    ...otherwise...
     *
     *    If the file is a passed file, we need to show this by incrementing its
     *    index into the passed file index range.
     */
    if (fat_ptr->fat_refflgl == DEFINED_FILE)
    {
        fat_ptr->fat_refflgl = ACCESSED_FILE;
    }
    else if (fat_ptr->fat_refflgl == PASSED_FILE)
    {
        tpa_ptr->open_fididxl += PASSED_FILE_BASE;
    }

    /*
     *  Get write string opetand and create 'string' sub-record.
     */
    (void) strcpy (wrtstrlin, "string ");
    ENC_OPD_BYPOS (com_ptr, wrtstrlin, NULL, NULL);

    /*
     *  Get file ID and create 'filid' sub-record.
     */
    (void) sprintf (filidlin, "filid %s", fat_ptr->fat_namec20);

    /*
     *  Write the following records to the 'TOF' line list:
     *
     *  Write
     *  string <string_value>
     *  filid <file_id>
     *  {errlab <error_label>}
     */
    APPEND_RECHDR (com_ptr, "Write");

    APPEND_TOFLIN (com_ptr, wrtstrlin);

    APPEND_TOFLIN (com_ptr, filidlin);

    if (strlen (tpa_ptr->onerror_labelc16) != 0)
    {
        (void) strcpy (lin_ptr, "errlab ");
        RESOLVE_LABEL (argblk_ptr,
                       tpa_ptr->onerror_labelc16,
                       tpa_ptr->onerror_lenl);
        APPEND_TOFLIN (com_ptr, NULL);
    }

end:
    ;
    return PSL_NORMAL;          /* Return success */
}

/******************************************************************************
 * @func    This Tparse action routine is called after parsing a WRITE PIPE
 *      command. Its purpose is to append a Wrpipe record to the list of
 *      'TOF' lines. The Wrpipe record has the following format:
 *
 *      <PRE>
 *      Wrpipe
 *      string <string_value>
 *      pipid <pipe_id>
 *      {retvar <return_variable>}  Required for synchronous pipes only
 *      {timeout <timeout_period>}
 *      {tmolab <timeout_label>}
 *      {errlab <error_label>}
 *      </PRE>
 *
 *      Note that if this is a passed pipe, the index is incremented by
 *      the constant PASSED_PIPE_BASE, to show that it is a passed pipe.
 *
 * @rdesc   Tparse status (Always returns PSL_NORMAL).
 * @parm    Pointer to Tparse argument block.
 */
unsigned long ENC_WRITE_PIPE (struct tpaarg *argblk_ptr)
{
    /*
     *  Local data.
     */
    struct scl_com  *com_ptr;       /* SCL common data */
    struct scl_tpa  *tpa_ptr;       /* SCL Tparse data */
    struct fattbl   *fat_ptr;       /* Pipe Access Table pointer */
    char   *lin_ptr;       /* Pointer to line buffer */
    char   wrtstrlin [TOFBUFLEN];  /* Write string line */
    char   pipidlin [TOFBUFLEN];   /* Pipe Id line */
    char   retvarlin [TOFBUFLEN];  /* Return variable line */
    char   tmoprdlin [TOFBUFLEN];  /* Timeout line */

    /*
     *  Copy pointers for ease of reference.
     */
    com_ptr = (struct scl_com *) argblk_ptr->user_ptrp;
    tpa_ptr = com_ptr->tpa_ptrps;
    lin_ptr = com_ptr->toflinc1024; /* 'tof' line buffer pointer */
    fat_ptr = tpa_ptr->pipe_begps + tpa_ptr->open_fididxl - 1;
    /* Point to current entry in table */
    /*
     *  Do not encode anything if we have had an error in the command.
     */
    if (tpa_ptr->error_in_commandn) goto end;

    /*
     *  Output tracing information if required.
     */
#ifdef PSL_TRACING
    if (PSL_TEST_TRACE (PTRC_USR_USER1))
    {
        PSL_OUTPUT_TRACE ("Encoding a WRITE PIPE Command\n");
        PSL_OUTPUT_TRACE ("Operand Count = %ld\n" ,GET_OPD_COUNT (tpa_ptr));
    }
#endif

    /*
     *  If the pipe has been passed as a parameter, add the 'passed pipe base'
     *  offset to the pipe entry index index to point to the entry in the 'passed
     *  pipe' section of the table.
     */
    if ((fat_ptr->fat_refflgl & PIPATT_PRM) != 0)
    {
        tpa_ptr->open_fididxl += PASSED_PIPE_BASE;
    }

    /*
     *  Get write string opetand and create 'string' sub-record.
     */
    (void) strcpy (wrtstrlin, "string ");
    ENC_OPD_BYPOS (com_ptr, wrtstrlin, NULL, NULL);

    /*
     *  Get pipe ID and create 'pipid' sub-record.
     */
    (void) sprintf (pipidlin, "pipid %s", fat_ptr->fat_namec20);

    /*
     *  Get return variable operand, if any, and create 'retvar' sub-record.
     */
    if ((tpa_ptr->cmnd_optl & CMND_OPT_RETURNING) != 0)
    {
        (void) strcpy (retvarlin, "retvar ");
        ENC_OPD_BYPOS (com_ptr, retvarlin, NULL, NULL);
    }

    /*
     *  Get the wait period, if any, and create 'timeout' sub-record.
     */
    if ((tpa_ptr->cmnd_optl & CMND_OPT_WAIT_PRD) != 0)
    {
        (void) strcpy (tmoprdlin, "timeout ");
        ENC_OPD_BYPOS (com_ptr, tmoprdlin, NULL, NULL);
    }

    /*
     *  Write the following records to the 'TOF' line list:
     *
     *  Wrpipe
     *  string <string_value>
     *  pipid <pipe_id>
     *  {retvar <return_variable>}
     *  {timeout <timeout_period>}
     *  {tmolab <timeout_label>}
     *  {errlab <error_label>}
     */
    APPEND_RECHDR (com_ptr, "Wrpipe");

    APPEND_TOFLIN (com_ptr, wrtstrlin);

    APPEND_TOFLIN (com_ptr, pipidlin);

    if ((tpa_ptr->cmnd_optl & CMND_OPT_RETURNING) != 0)
    {
        APPEND_TOFLIN (com_ptr, retvarlin);
    }

    if ((tpa_ptr->cmnd_optl & CMND_OPT_WAIT_PRD) != 0)
    {
        APPEND_TOFLIN (com_ptr, tmoprdlin);
    }

    if ((tpa_ptr->cmnd_optl & CMND_OPT_TIMEOUT) != 0)
    {
        (void) sprintf (lin_ptr, "tmolab %s", tpa_ptr->ontimeout_labelc20);
        APPEND_TOFLIN (com_ptr, lin_ptr);
    }

    if (strlen (tpa_ptr->onerror_labelc16) != 0)
    {
        (void) strcpy (lin_ptr, "errlab ");
        RESOLVE_LABEL (argblk_ptr,
                       tpa_ptr->onerror_labelc16,
                       tpa_ptr->onerror_lenl);
        APPEND_TOFLIN (com_ptr, NULL);
    }

    /*
     *  Mark the pipe as having been accessed.
     */
    fat_ptr->fat_refflgl |= PIPATT_ACC;

end:
    ;
    return PSL_NORMAL;          /* Return success */
}

/******************************************************************************
 * @func    This routine appends a label reference operand to the default 'tof'
 *      line buffer.
 *
 * @parm    Pointer to Tparse argument block.
 * @parm    Pointer to label name.
 * @parm    Label name length.
 */
static void ENC_LAB_REF (struct tpaarg *argblk_ptr,
                         char *label_ptr,
                         unsigned long label_len)
{
    /*
     *  Local data.
     */
    struct   scl_com *com_ptr;      /* SCL common data */
    struct   scl_tpa *tpa_ptr;      /* SCL Tparse data */
    char    *lin_ptr;      /* Pointer to line buffer */

    /*
     *  Copy pointers for ease of reference.
     */
    com_ptr = (struct scl_com *) argblk_ptr->user_ptrp;
    tpa_ptr = com_ptr->tpa_ptrps;
    lin_ptr = com_ptr->toflinc1024; /* 'tof' line buffer pointer */

    /*
     *  Append null-terminated label reference operand to default 'tof' line buffer.
     */
    lin_ptr += strlen (lin_ptr);
    (void) memcpy (lin_ptr, label_ptr, label_len);
    lin_ptr += label_len;
    *lin_ptr = '\0';

    return;
}

/******************************************************************************
 * @func    This routine scans through the linked list of parsed operands
 *      and returns a count of those found.
 *
 * @rdesc   Unsigned long countof operands in operands linked list
 * @parm    Pointer to Tparse data
 */
static unsigned long GET_OPD_COUNT (struct scl_tpa *tpa_ptr)
{
    /*
     *  Local data.
     */
    struct   oph      *oph_ptr;     /* Pointer to OPH*/
    unsigned long     count = 0;        /* Count of operands found */
    unsigned long     iret;         /* Status return */
    char     *llb_pos;     /* Linked list position */
    char     *llb_ctx;     /* Linked list context */

    llb_pos = NULL;         /* Position at start of list */

    for ( ; ; count++)
    {
        iret = PSL_VM_COPY (tpa_ptr->parsed_operand_llps
                            ,(char *) &oph_ptr ,llb_pos ,&llb_ctx
                            ,PSL_VM_MODE_NEXT);
        if (!(psl_test_status (iret))) return count;
        llb_pos = llb_ctx;      /* Update the context */

        /*
         *  There should always be an OPD following an OPH so pick it up to
         *  complete the operand.
         */
        iret = PSL_VM_COPY (tpa_ptr->parsed_operand_llps
                            ,(char *) &oph_ptr ,llb_pos ,&llb_ctx
                            ,PSL_VM_MODE_NEXT);
        if (!(psl_test_status (iret))) return count;
        llb_pos = llb_ctx;      /* Update the context */
    }
}

/******************************************************************************
 * @func    This routine encodes a reference to a label in the default 'tof'
 *      line buffer. If the label has not been defined yet an entry for it
 *      is added to the labels symbol table.
 *
 * @parm    Pointer to Tparse argument block.
 * @parm    Pointer to label name.
 * @parm    Label name length.
 */
static void RESOLVE_LABEL (struct tpaarg *argblk_ptr,
                           char *label_ptr,
                           unsigned long label_len)
{
    /*
     *  Local data.
     */
    struct   scl_com *com_ptr;      /* Common data          */
    struct   scl_tpa *tpa_ptr;      /* TPARSE data pointer      */
    struct   labtbl  *labtbl_ptr;       /* Pointer to subroutine table. */
    unsigned long    isub;          /* Loop control variable.   */
    long    comp_result;       /* Result of comparsion     */

    /*
     *  Copy pointers for ease of reference.
     */
    com_ptr = (struct scl_com *) argblk_ptr->user_ptrp;
    tpa_ptr = com_ptr->tpa_ptrps;

    /*
     *  Encode label reference operand into the default 'tof' line buffer.
     */
    ENC_LAB_REF (argblk_ptr, label_ptr, label_len);

    /*
     *  Look in Labels Symbol Table to see if this label has been encountered before.
     *  If it has not we need to add an 'undefined label' entry to the Labels
     *  Symbol Table.
     */
    labtbl_ptr = tpa_ptr->label_begps; /* Pointer to start of table */

    for (isub=0; isub < tpa_ptr->lab_cntl; isub++, labtbl_ptr++)
    {
        if (label_len == labtbl_ptr->lab_nlengthl)
        {
            comp_result = memcmp (label_ptr,
                                  labtbl_ptr->lab_namec16,
                                  labtbl_ptr->lab_nlengthl);
            if (comp_result == 0) goto end;
        }
    }

    /*
     *  Drop through to here if we have the first reference to an undefined label.
     *  Add an 'undefined label' entry to the Labels Symbol Table.
     */
    ADD_UNDEF_LABEL (argblk_ptr, label_ptr, label_len);

end:
    ;
    return;
}
